package org.act.ws.manage;

import java.io.BufferedInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FilterOutputStream;
import java.io.IOException;
import java.util.List;

import org.act.xservices.rt.metaset.Attribute;
import org.act.xservices.rt.metaset.Metaset;
import org.act.xservices.rt.metaset.MetasetBuilder;
import org.act.xservices.rt.metaset.MetasetBuilderFactory;
import org.act.xservices.rt.metaset.MetasetParsingException;
import org.act.xservices.rt.xcontainer.management.IContainerManagement;
import org.act.xservices.rt.xcontainer.management.ManagementUtil;

/**
 * @version 2008-4-15
 * @author chenyy [chenyy@act.buaa.edu.cn]
 * @since JDK1.5
 *
 */
public class ConfigInfo {
    
	private static IContainerManagement containerManagement = ManagementUtil.getInstance().getContainerManagement();
		
	/**
	 * @return
	 * XServiceļ
	 */
	public static String getConfigFile() {

		try {
            String dir = containerManagement.getConfigFileAbsolutePath();
			File userConfigFile = new File(dir);
			BufferedInputStream input = new BufferedInputStream(new FileInputStream(userConfigFile));
			
			ByteArrayOutputStream byteOut = new ByteArrayOutputStream();
			int b = -1;
			while ( (b = input.read()) != -1 ) {
				byteOut.write(b);
			}
			input.close();
			
			String text = new String(byteOut.toByteArray(), "utf-8");
			text = text.trim();
			
			byteOut.close();
			return text;
			
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			return "file is not found";
		} catch (IOException e) {
			// TODO Auto-generated catch block
			return "io error";
		}
	}

	/**
	 * XServiceļ
	 * @param text
	 * @return
	 */
	public static String setConfigFile(String text) {
		File configFile = null;
		String dir = containerManagement.getConfigFileAbsolutePath();
		try {
			configFile = new File(dir);

			FilterOutputStream output = new FilterOutputStream(
					new FileOutputStream(configFile));
			output.write(text.getBytes());
			output.close();
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			//e.printStackTrace();
			return "fail";
		} catch (IOException e) {
			// TODO Auto-generated catch block
			//e.printStackTrace();
			return "fail";
		}

		return "success";
	} 
	
	
	private static String deployerUpdateIntervalPath = "xcontainer\\deployment\\wsDeployMonitor\\parameter@name='deployUpdateInterval'";
    
	private static String isDebugPath = "xcontainer\\containerEnvironment\\isDebug";
	
	private static String objectpoolSizePath = "xcontainer\\containerObjectPool\\objectpools\\objectpool";
	
	private static String httpEnabledPath = "xcontainer\\containerConnectors\\connector\\protocols\\protocol";
	
	private static String listionPortPath = "xcontainer\\containerConnectors\\connector\\protocols\\protocol\\httpReceiver";
	
	/**
	 *XServiceļϢ
	 *listionPort isDebug deployeInterval dispatcherPriority transportProtocol messageSecure objectPool
	 *@return 
	 */
	public static String getConfigInfo() {
        
		String dir = containerManagement.getConfigFileAbsolutePath();
	    MetasetBuilder metBuilder = MetasetBuilderFactory.newInstance().newMetasetBuilder();
	    try {
			Metaset configuration = metBuilder.parse(dir);
		    
			String deployerUpdateInterval = configuration.filter(deployerUpdateIntervalPath).car().getValue();
			
			String isDebug = configuration.filter(isDebugPath).car().getValue();
			
			//by lizq, need to fix
			String dispatcherPriority = "10";
				//configuration.filter("configuration\\container\\containerConfig\\DispatcherPriority").car().getValue();
			
			String objectpoolSize = configuration.selectorAttributeValue(objectpoolSizePath, "maxsize");
			
			String httpEnabled = configuration.selectorAttributeValue(httpEnabledPath, "enabled");
			
			String transportProtocol = null;
			if(httpEnabled.equals("true"))
				transportProtocol = "http";
			else
				transportProtocol = "smtp";
			
			String listionPort = configuration.selectorAttributeValue(listionPortPath, "port");
			
			//by lizq
//			List connectors = configuration.filter("xcontainer\\containerConnectors").car().getChildrens();			
//			
//			Metaset messageConnector = (Metaset)connectors.get(1);
//			
//			Metaset secureHandler = messageConnector.filter("connector\\HandlerChains").car().getChildrens().get(1);
//			
			String secure = "true";
				//secureHandler.selectorAttributeValue(".\\", "enable");
			
			String configInfo = listionPort + "#" + isDebug + "#" + deployerUpdateInterval + "#" +
			                    dispatcherPriority + "#" + transportProtocol + "#" + secure + "#" + objectpoolSize;
			 
            return configInfo;			
	    } catch (MetasetParsingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return "null";
		}
		
	} 
	
	/**
	 * XServiceļϢ
	 * @param configInfo
	 * @return
	 */
	@SuppressWarnings("unused")
	public static String setConfigInfo(String configInfo) {

		String dir = containerManagement.getConfigFileAbsolutePath();
        String[] configInfoList = configInfo.split("#");   
		String listionPort = configInfoList[0];
        String isDebug = configInfoList[1];
        String deployeInterval = configInfoList[2];
        String dispatcherPriority = configInfoList[3];
        String transportProtocol = configInfoList[4];
        String messageSecure = configInfoList[5];
        String objectPool = configInfoList[6];
        MetasetBuilder metBuilder = MetasetBuilderFactory.newInstance().newMetasetBuilder();
        try {
			Metaset configuration = metBuilder.parse(dir);
		    
			Metaset DeployerUpdateInterval = configuration.filter(deployerUpdateIntervalPath).car();
			DeployerUpdateInterval.setValue(deployeInterval);
			
			Metaset IsDebug = configuration.filter(isDebugPath).car();
			IsDebug.setValue(isDebug);
			
//			Metaset DispatcherPriority = configuration.filter("configuration\\container\\containerConfig\\DispatcherPriority").car();
//			DispatcherPriority.setValue(dispatcherPriority);
			
			List objectpoolList =  configuration.filter(objectpoolSizePath).car().getChildrens();
			for(int i = 0; i < objectpoolList.size(); i ++)
			{
				Metaset objectpool = (Metaset)objectpoolList.get(i);
				Attribute maxsize = objectpool.getAttributes().get(1);
				maxsize.setValue(objectPool);				
			}
			
			//by lizq 2009-07-20
//			List Protocol = configuration.filter("xcontainer\\containerConnectors\\connector\\protocols").car().getChildrens();
//			Metaset httpProtocol = (Metaset)Protocol.get(0);
//			Metaset smtpProtocol = (Metaset)Protocol.get(1);
//			Attribute httpEnabled = httpProtocol.getAttributes().get(4);
//			Attribute smtpEnabled = smtpProtocol.getAttributes().get(4);
//			if(transportProtocol.equals("http"))
//			{
//				httpEnabled.setValue("true");
//				smtpEnabled.setValue("false");
//			}
//			else if(transportProtocol.equals("smtp"))
//			{
//				httpEnabled.setValue("false");
//				smtpEnabled.setValue("true");
//			}
//			
//			Metaset serverSocket = configuration.filter("configuration\\container\\connectors\\connector\\protocols\\protocol\\httpDispatchSource\\serverSocket").car();
//            Attribute port = serverSocket.getAttributes().get(0);
//            port.setValue(listionPort);
//			
//			List connectors = configuration.filter("configuration\\container\\connectors").car().getChildrens();			
//			Metaset messageConnector = (Metaset)connectors.get(1);
//			Metaset secureHandler = messageConnector.filter("connector\\HandlerChains").car().getChildrens().get(1);
//			Attribute enable = secureHandler.getAttributes().get(1);
//			enable.setValue(messageSecure);
			
            String configurationInfo = configuration.toXML();            
            File configFile = new File(dir);
			FilterOutputStream output;
	        output = new FilterOutputStream(new FileOutputStream(configFile));
            output.write(configurationInfo.getBytes());
            output.close();
	    } catch (MetasetParsingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return "fail";
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return "fail";
		}catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return "fail";
		}
		
		return "success";
	}

//	public static void main(String[] args) {
//
//       ConfigInfo configInfo = new ConfigInfo();
//       String configInformation = "8090#ture#600000#10#http#false#100";
//       System.out.println(configInfo.setConfigInfo(configInformation));
//
//	}
}
