package org.act.ws.manage;

import java.io.BufferedInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import javax.xml.namespace.QName;

import org.act.xservices.rt.metaset.Metaset;
import org.act.xservices.rt.metaset.MetasetBuilder;
import org.act.xservices.rt.metaset.MetasetBuilderFactory;
import org.act.xservices.rt.metaset.MetasetParsingException;
import org.act.xservices.rt.xcontainer.management.IServiceManagement;
import org.act.xservices.rt.xcontainer.management.ManagementUtil;
import org.act.xservices.rt.xcontainer.management.MethodInfoBean;

import sun.misc.BASE64Decoder;

/**
 * @version 2008-4-15
 * @author chenyy [chenyy@act.buaa.edu.cn]
 * @since JDK1.5
 */
public class ServiceInfo {
	
	public static final char NL = '\n';

    public static final char CR = '\r';
    
    public static final String DEPLOY_DIR = "org.act.ws.container.deployDir";
    
    /**
     * The prefered line separator
     */
    public static final String LS = System.getProperty("line.separator",(new Character(NL)).toString());
    
    private static IServiceManagement serviceManagement = ManagementUtil.getInstance().getServiceManagement();

    
	/**
	 * ȡѲķ֣#ŷָ
	 * @return
	 */
	public static String getDeployedServices()
	{
		
		Set<QName> serviceNames = serviceManagement.getDeployedServicesName();

		if(serviceNames.isEmpty())
			return "null";
		else
		{
			String serviceList="#";
			Iterator<QName> iterator = serviceNames.iterator();
			while (iterator.hasNext())
			{
			    QName serviceQName = iterator.next();
			    if(serviceQName.getLocalPart().equals("SystemInfo"))
			    	continue;
			    else
				    serviceList = serviceList + serviceQName.toString() + "#";
			}
			return serviceList;
		}
		
	}
	
	
	/**
	 * ȡ÷ķͲ#**÷ʽ***ֿռ#
	 * @param serviceName
	 * @return
	 */
	public static String getOperation(String serviceName,String namespaceURI)
	{
		  
		  Set<MethodInfoBean> methodInfoBeanSet = serviceManagement.getMethodsInfo(serviceName, namespaceURI);
		  Iterator<MethodInfoBean> iterator = methodInfoBeanSet.iterator();
		  
		  String methodInfoList = "#";
          
		  while(iterator.hasNext())
		  {
			  MethodInfoBean methodInfoBean = iterator.next();
			  
			  String methodName = methodInfoBean.getMethodName();
			  
			  List<String> parameters = methodInfoBean.getParameterTypes();
			  String parametersInfo = "";
			  for(int i = 0 ; i < parameters.size(); i ++)
			  {
				  parametersInfo += parameters.get(i) + " ";
			  }
			  
			  String exchangeModel = methodInfoBean.getExchangeModel();
			  
			  String operationStyle = methodInfoBean.getOperationStyle();
			   
			  List<String> returnTypes = methodInfoBean.getReturnTypes();
			  String returntypesInfo = "";
			  for(int i = 0; i < returnTypes.size(); i ++)
			  {
				  returntypesInfo += returnTypes.get(i) + " "; 
			  }
			  
			  Set<List<String>> faultTypesSet = methodInfoBean.getFaultTypesSet();
			  Iterator<List<String>> faultTypeIterator = faultTypesSet.iterator();
			  String faultTypeInfo = "";
			  while(faultTypeIterator.hasNext())
			  {
				  List<String> faultTypeList = faultTypeIterator.next();
				  for(int i = 0; i < faultTypeList.size(); i ++)
				  {
					  faultTypeInfo += faultTypeList.get(i);
				  }
			  }
			  
			  methodInfoList += methodName + "~" + parametersInfo + "~" + exchangeModel + "~" + operationStyle + "~" + returntypesInfo + "~" + faultTypeInfo + "#";
		  }
		  
		  return methodInfoList;
	}
	
//	/**
//	 * WSDL
//	 * @param serviceName 
//	 * @return
//	 */
//	public static String getWsdl(String serviceName,String namespaceURI)
//	{
//
//		String dir = serviceManagement.getWSDLFileAbsolutePath(serviceName, namespaceURI);
//		String fileContent = "";
//		try
//		{
//		    File wsdlFile = new File(dir);
//		    //System.err.println(wsdlFile.getAbsolutePath());
//		    DataInputStream input = new DataInputStream(
//					new BufferedInputStream(new FileInputStream(wsdlFile)));
//			ByteArrayOutputStream outputstrem = new ByteArrayOutputStream();
//
//			int nNumber = 0;
//			byte[] buffer = new byte[4096];
//			while ((nNumber = input.read(buffer)) != -1) {
//				outputstrem.write(buffer, 0, nNumber);
//			}
//
//		    input.close();
//		    fileContent = new String(outputstrem.toByteArray());
//		}
//		catch (FileNotFoundException e) {
//		    System.err.print(serviceName+"wsdlļδҵ");
//		}
//		catch (IOException e) {
//		    System.err.println("ļ");
//		}
//		
//		return fileContent.trim();
//	}
	
	/**
	 * WSDLַ
	 * @param serviceName 
	 * @return
	 */
	public static String getWsdl(String serviceName,String namespaceURI)
	{
		String dir = serviceManagement.getWSDLFileAbsolutePath(serviceName, namespaceURI);
		String wsdlLocation = null;
		MetasetBuilder metBuilder = MetasetBuilderFactory.newInstance().newMetasetBuilder();
		try {
			Metaset wsdl = metBuilder.parse(dir);
		    
			Metaset address = wsdl.filter("definitions\\service\\port\\address").car();
			
            wsdlLocation = address.selectorAttributeValue(".\\", "location");
			 
            return wsdlLocation;			
	    } catch (MetasetParsingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return "null";
		}

	}
		
	/**
	 * WSDD
	 * @param serviceName 
	 * @return
	 */
	public static String getWsdd(String serviceName,String namespaceURI)
	{
        String wsdddir = serviceManagement.getWSDDFileAbsolutePath(serviceName, namespaceURI);
		String fileContent = "";
		try
		{
		File wsddFile = new File(wsdddir);
		DataInputStream input = new DataInputStream(
				new BufferedInputStream(new FileInputStream(wsddFile)));
		ByteArrayOutputStream outputstrem = new ByteArrayOutputStream();

		int nNumber = 0;
		byte[] buffer = new byte[4096];
		while ((nNumber = input.read(buffer)) != -1) {
			outputstrem.write(buffer, 0, nNumber);
		}

		input.close();
		fileContent = new String(outputstrem.toByteArray());
		}
		catch (FileNotFoundException e) {
		System.err.print(serviceName+"wsddļδҵ");
		}
		catch (IOException e) {
		System.err.println("ļ");
		}
		return fileContent.trim();
	}
	
	
	//	----------add 2008-06-16
	/**
     * ǰѲķ
     * @return еǰѲķ
     */
	public static int getDeployedServiceNum(){
		
		return serviceManagement.getDeployedServiceNum();
	}
	
	/**
	 * ǰеķ
	 * @return еķ
	 */
	public static int getRunningServiceNum(){
		
		return serviceManagement.getRunningServiceNum();
	}
	
	/**
	 * ʼǰʱ
	 * @return Ѵ
	 */
	public static int getProcessedServiceNum(){
		
		return serviceManagement.getProcessedServiceNum();
	}
	
	/**
	 * ʼǰʱõĴ
	 * @return 
	 */
	public static int getErrorInvokeNum(){
		
		return serviceManagement.getErrorInvokeNum();
	}
	
	/**
	 * һWeb
	 * @param wsar
	 * @return ɹʧ
	 */
	public static boolean deployService(String serviceWSAR, String serviceName){
		
		String dir = System.getProperty(DEPLOY_DIR)+ java.io.File.separator + serviceName + ".wsar";
		BASE64Decoder base64De = new BASE64Decoder();		
		try {
			 byte[] fileByte = base64De.decodeBuffer(serviceWSAR);
			 OutputStream fos = new FileOutputStream(dir);
			 fos.write(fileByte);
             fos.close();
             fos = null;
             return true;
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return false;
		}  catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return false;
		}
	}
	
	/**
	 * ĳ
	 * @param serviceName 
	 * @return ɹʧ
	 */
	public static boolean undeployService(String serviceName, String namespaceURI){
		
		return serviceManagement.undeployService(serviceName, namespaceURI);
	}
	
	/**
	 * ֹͣĳֹͣ󣬸÷ͣSOAP󣬿ͨStart½SOAP
	 * @param serviceName
	 * @return ֹͣɹʧ
	 */
	public static boolean stopService(String serviceName){
		
		return serviceManagement.stopService(serviceName);
	}
	
	/**
	 * ĳֹͣ״̬ķ񣬸÷½SOAP
	 * @param serviceName
	 * @return ɹʧ
	 */
    public static boolean startService(String serviceName){
    	
    	return serviceManagement.startService(serviceName);
    }
    
    /**
     * ĳĵǰ״̬ started/stopped
     * @param serviceName
     * @return ĳĵǰ״̬
     */
    public static String getServiceStatus(String serviceName){
    	
    	return serviceManagement.getServiceStatus(serviceName);
    }

}
