/**
 * Logback: the reliable, generic, fast and flexible logging framework.
 * Copyright (C) 1999-2009, QOS.ch. All rights reserved.
 *
 * This program and the accompanying materials are dual-licensed under
 * either the terms of the Eclipse Public License v1.0 as published by
 * the Eclipse Foundation
 *
 *   or (per the licensee's choosing)
 *
 * under the terms of the GNU Lesser General Public License version 2.1
 * as published by the Free Software Foundation.
 */
package org.act.xservices.log.core.joran.action;

import org.act.xservices.log.core.joran.spi.ActionException;
import org.act.xservices.log.core.joran.spi.InterpretationContext;
import org.act.xservices.log.core.spi.LifeCycle;
import org.act.xservices.log.core.status.StatusListener;
import org.act.xservices.log.core.util.OptionHelper;
import org.xml.sax.Attributes;



public class StatusListenerAction extends Action {

 
  boolean inError = false;
  StatusListener statusListener = null;
  
  public void begin(InterpretationContext ec, String name, Attributes attributes) throws ActionException {
    inError = false;
    String className = attributes.getValue(CLASS_ATTRIBUTE);
    if(OptionHelper.isEmpty(className)) {
      addError(
        "Missing class name for statusListener. Near ["
          + name + "] line " + getLineNumber(ec));
      inError = true;
      return;
    }
    
    try {
      statusListener = (StatusListener) OptionHelper.instantiateByClassName(
          className, StatusListener.class, context);
      ec.getContext().getStatusManager().add(statusListener);
      ec.pushObject(statusListener);
    } catch (Exception e) {
      inError = true;
      addError(
        "Could not create an StatusListener of type ["+className+"].", e);
      throw new ActionException(e);
    }
    
  }
 
  public void finish(InterpretationContext ec) {
  }

  public void end(InterpretationContext ec, String e) {
    if (inError) {
      return;
    }
    if (statusListener instanceof LifeCycle) {
      ((LifeCycle) statusListener).start();
    }
    Object o = ec.peekObject();
    if (o != statusListener) {
      addWarn(
        "The object at the of the stack is not the statusListener pushed earlier.");
    } else {
      ec.popObject();
    }
  }
}
