package org.act.xservices.engine.rt;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Stack;

import org.act.xservices.engine.rt.aspect.ThisJPActivity;
import org.act.xservices.engine.rt.debug.Debug;
import org.act.xservices.engine.rt.deploy.DeploymentUnit;
import org.act.xservices.engine.rt.persistence.StoreUtil;
import org.act.xservices.engine.rt.utils.TimeUtil;
import org.act.xservices.rt.xcontainer.framework.objectpool.IDispatchTarget;
import org.act.xservices.rt.xcontainer.inf.context.IServiceContext;


/**
 * ʵԪ
 * @author sun
 *
 */
public abstract class InstanceUnit implements Runnable{
	
	public static final int STATE_INITIATED = 0;
	
	public static final int STATE_RUNNING = 1;
	
	public static final int STATE_ACTIVE = 2;
	
	public static final int STATE_SUSPENDED = 3;
	
	public static final int STATE_COMPLETED = 4;
	
    public static final int STATE_TERMINATED = 5;
    
    //ƥĻ
    public static Boolean hasMatch = false;
    
    //¼Ļʵ
	private ActivityInstance activityInstanceFinished = null;
	
	//¼ǰƥϢ
	private ThisJPActivity thisJPActivity = null;
    
    /**
     * ʵID
     */
    private String id;
    
    /**
     * ʵ״̬
     */
    private int state = STATE_INITIATED;
    
    /**
     * ʵʼʱ
     */
    protected String startTime;
	
	/**
	 * ʵʱ
	 */
    protected String endTime;
    
    /**
     * Ϣ
     */
    private IDispatchTarget<IServiceContext> messageOut;
    
    /**
     * Ϣ
     */
	private ArrayList<MessageContainer> messageContainers = new ArrayList<MessageContainer>();
    
    /**
     * ʵ
     */
	private ArrayList<ActivityInstance> activityInstances = new ArrayList<ActivityInstance>();
	
	/**
	 * ջ
	 */
	private Stack<Object> activitiesStack = new Stack<Object>();

	/**
	 * ջ
	 */
	private Stack<Object> scopesStack = new Stack<Object>();

	/**
	 * 
	 */
	private VariableContainer variableContainer = new VariableContainer();
	
	/**
	 * 
	 */
	private HashMap<String, Object> links = new HashMap<String, Object>();
	
	/**
	 * ̲Ԫ
	 */
	private DeploymentUnit deploymentUnit;
    
	public void setThisJPActivity(ThisJPActivity thisJPActivity){
		this.thisJPActivity = thisJPActivity;
	}
	
	public ThisJPActivity getThisJPActivity(){
		return thisJPActivity;
	}
	
	public void setActivityInstanceFinished(ActivityInstance activityInstanceFinished){
		this.activityInstanceFinished = activityInstanceFinished;
	}
	
	public ActivityInstance getActivityInstanceFinished(){
		return activityInstanceFinished;
	}
	
	public String getId() {
    	return id;
    }
    
    public void setId(String id) {
    	this.id = id;
    }
    
    public void addMessageContainer(MessageContainer messageContainer) {
    	messageContainers.add(messageContainer);
    }
    
    public void addMessageContainer(String partnerLink, String operation) {
    	MessageContainer messageContainer = new MessageContainer();
    	messageContainer.setPartnerLink(partnerLink);
    	messageContainer.setOperation(operation);
    	messageContainers.add(messageContainer);
    }
	
	public void insertMessage(String partnerLink, String operation, IServiceContext message) {
		for(int i=0; i<messageContainers.size(); i++) {
			MessageContainer messageContainer = messageContainers.get(i);
			if(messageContainer.getPartnerLink().equals(partnerLink) &&
					messageContainer.getOperation().equals(operation)) {
				messageContainer.setMessage(message);
			}
		}
	}
	
	public IServiceContext getMessage(String partnerLink, String operation) {
		for(int i=0; i<messageContainers.size(); i++) {
			MessageContainer messageContainer = messageContainers.get(i);
			if(messageContainer.getPartnerLink().equals(partnerLink) &&
					messageContainer.getOperation().equals(operation)) {
				IServiceContext message = messageContainer.getMessage();
				messageContainer.removeMessage();
				return message;
			}
		}
		return null;
	}
	
	public boolean hasMessage(String partnerLink, String operation) {
		for(int i=0; i<messageContainers.size(); i++) {
			MessageContainer messageContainer = messageContainers.get(i);
			if(messageContainer.getPartnerLink().equals(partnerLink) &&
					messageContainer.getOperation().equals(operation)) {
				return messageContainer.hasMessage();
			}
		}
		return false;
	}
	
	public void storeMessageContext(String partnerLink, String operation, IServiceContext context) {
		for(int i=0; i<messageContainers.size(); i++) {
			MessageContainer messageContainer = messageContainers.get(i);
			if(messageContainer.getPartnerLink().equals(partnerLink) &&
					messageContainer.getOperation().equals(operation)) {
				messageContainer.setContext(context);
			}
		}
	}
	
	public IServiceContext getContext(String partnerLink, String operation) {
		for(int i=0; i<messageContainers.size(); i++) {
			MessageContainer messageContainer = messageContainers.get(i);
			if(messageContainer.getPartnerLink().equals(partnerLink) &&
					messageContainer.getOperation().equals(operation)) {
				IServiceContext context = messageContainer.getContext();
				messageContainer.removeContext();
				return context;
			}
		}
		return null;
	}
	
	public void addActivityInstance(ActivityInstance activityInstance) {
		activityInstances.add(activityInstance);
	}
	
	public ArrayList<ActivityInstance> getActivityInstances() {
		return activityInstances;
	}

	public int getState() {
    	return state;
    }
    
    public void start() {
    	state = STATE_RUNNING;
    	startTime = TimeUtil.getNowTime();
    	if(Debug.isDebug()) {
    		Debug.stop(this);
    	}
    }
    
    public void restart() {
    	state = STATE_INITIATED;
    }
    
    public void suspend() {
    	state = STATE_SUSPENDED;
    }
    
    public void resume() {
    	state = STATE_RUNNING;
    }
    
    public void terminate() {
    	state = STATE_TERMINATED;
    	endTime = TimeUtil.getNowTime();
    }
    
    public abstract void run();
    
    public void finish() {
		state = STATE_COMPLETED;
		endTime = TimeUtil.getNowTime();
		StoreUtil.storeProcessInstance(this);
	}

	public String getStartTime() {
		return startTime;
	}

	public String getEndTime() {
		return endTime;
	}

	public IDispatchTarget<IServiceContext> getMessageOut() {
		return messageOut;
	}

	public void setMessageOut(IDispatchTarget<IServiceContext> messageOut) {
		this.messageOut = messageOut;
	}
	
	public Stack<Object> getActivitiesStack() {
		return activitiesStack;
	}

	public void setDeploymentUnit(DeploymentUnit deploymentUnit) {
		this.deploymentUnit = deploymentUnit;
	}

	public DeploymentUnit getDeploymentUnit() {
		return deploymentUnit;
	}

	public VariableContainer getVariableContainer() {
		return variableContainer;
	}
	
	public void addLink(String linkName) {
		Object waitPoint = new Object();
		links.put(linkName, waitPoint);
	}
	
	public void doLinkSource(String linkName) {
		Object waitPoint = links.get(linkName);
		links.remove(linkName);
		synchronized(waitPoint) {
			waitPoint.notifyAll();
		}
	}
	
	public void doLinkTarget(String linkName) {
		Object waitPoint = links.get(linkName);
		if(waitPoint != null) {
			synchronized(waitPoint) {
				try {
					waitPoint.wait();
				} catch (InterruptedException e) {
					e.printStackTrace();
				}
			}
		}
	}

	public Stack<Object> getScopesStack() {
		return scopesStack;
	}
}
