package org.act.xservices.engine.rt.adapter;

import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.List;

import org.act.xservices.engine.rt.InstanceUnit;
import org.act.xservices.engine.rt.aspect.deploy.AspectDeploymentUnitRegistry;
import org.act.xservices.engine.rt.deploy.DeploymentUnit;
import org.act.xservices.engine.rt.deploy.DeploymentUnitRegistry;
import org.act.xservices.engine.rt.locale.StringManager;
import org.act.xservices.engine.rt.locale.StringManagerFactory;
import org.act.xservices.engine.rt.management.BPELManageUtil;
import org.act.xservices.engine.rt.persistence.EngineRecord;
import org.act.xservices.engine.rt.persistence.IDatabase;
import org.act.xservices.engine.rt.persistence.StoreUtil;
import org.act.xservices.engine.rt.persistence.berkeley.BerkeleyDatabase;
import org.act.xservices.engine.rt.utils.TimeUtil;
import org.act.xservices.log.Logger;
import org.act.xservices.log.LoggerFactory;
import org.act.xservices.rt.soap.ISOAPBody;
import org.act.xservices.rt.soap.ISOAPEnvelope;
import org.act.xservices.rt.soap.ISOAPHeader;
import org.act.xservices.rt.soap.util.SOAPMessageUtil;
import org.act.xservices.rt.xcontainer.adaptation.adapter.AbstractAdapter;
import org.act.xservices.rt.xcontainer.framework.config.IConfigElement;
import org.act.xservices.rt.xcontainer.framework.lifecycle.LifecycleException;
import org.act.xservices.rt.xcontainer.inf.context.IServiceContext;
import org.act.xservices.rt.xcontainer.inf.service.SpecialSOAPRegistry;
import org.act.xservices.rt.xom.Attribute;
import org.act.xservices.rt.xom.Metaset;
import org.act.xservices.rt.xsecurity.handler.DecryptionHandler;

public class BPELAdapter extends AbstractAdapter {

	final Logger logger = LoggerFactory.getLogger(BPELAdapter.class);

	private static StringManager sm = StringManagerFactory.getStringManager();

	/** 汾 */
	private String version = "5.2";

	/** ʱ */
	private String startupTime;

	/** ݿ */
	private IDatabase database;

	/** ̲ע */
	private DeploymentUnitRegistry deploymentUnitRegistry;
	/**
	 * Aspectע
	 */
	private static AspectDeploymentUnitRegistry aspectDeploymentUnitRegistry = null;

	/** һʵ */
	private HashMap<String, String> singletons;

	public void config(IConfigElement configurableElement) throws Exception {

		// ʼ
		super.config(configurableElement);

		// ---ʼ
		deploymentUnitRegistry = new DeploymentUnitRegistry();
		/******* Aspectע *********/
		aspectDeploymentUnitRegistry = new AspectDeploymentUnitRegistry();

		singletons = new HashMap<String, String>();

		// ģʽ
		IConfigElement singletonsConfigElement = configurableElement
				.getFirstChildWithName("Singletons");
		for (IConfigElement sigProCE : singletonsConfigElement.getChildren()) {
			String id = sigProCE.getAttribute("id");
			createSingleton(id);
		}

		// ---ʼݿ
		// ĬΪBerkeley DBԺɸļ
		database = new BerkeleyDatabase();

		// ݿ--by SunDou
		// IConfigurableElement datebaseConfigElement = connectorConfigElement
		// .getChildConfigurableElement()[3];
		// String usedb = datebaseConfigElement.getAttribute("valid");
		// System.setProperty("UseDB", usedb);
		//
		// if (usedb.equals("true")) {
		//
		// log.info(sm.getString("org.act.xservices.engine.rt.config.valid.true"));
		//
		// //ݿ
		// IConfigurableElement m_driver =
		// datebaseConfigElement.getChildConfigurableElement()[0];
		// String driver = m_driver.getValue();
		// System.setProperty("Driver", driver);
		// log.info(sm.getString("org.act.xservices.engine.rt.config.database.driver",driver));
		//
		// //ݿַ
		// IConfigurableElement m_dbaddr =
		// datebaseConfigElement.getChildConfigurableElement()[1];
		// String dbadd = m_dbaddr.getValue();
		// System.setProperty("DBaddress", dbadd);
		// log.info(sm.getString("org.act.xservices.engine.rt.config.database.address",dbadd));
		//
		// //ݿû
		// IConfigurableElement m_dbuser =
		// datebaseConfigElement.getChildConfigurableElement()[2];
		// String dbuser = m_dbuser.getValue();
		// System.setProperty("DBUser", dbuser);
		// log.info(sm.getString("org.act.xservices.engine.rt.config.database.user",dbuser));
		//
		// //ݿ
		// IConfigurableElement m_dbpw =
		// datebaseConfigElement.getChildConfigurableElement()[3];
		// String dbpw = m_dbpw.getValue();
		// if (dbpw != null) {
		// System.setProperty("DBPWD", dbpw);
		// } else {
		// System.setProperty("DBPWD", "");
		// }
		// }
		// else{
		// log.info(sm.getString("org.act.xservices.engine.rt.config.valid.false"));
		// }

		logger.info("Succeed add Component: " + getName() + " Version "
				+ version);
	}

	public void start() throws LifecycleException {
		logger.info("XServices Workflow Engine Start");
		// ʱ
		this.startupTime = TimeUtil.getNowTime();

		// ---SpecialSOAP---
		SpecialSOAPRegistry specialSOAPRegistry = this.containerEnv
				.getSpecialSOAPRegistry();
		// specialSOAPRegistry.put("/SystemInfo", "SystemInfo");
		specialSOAPRegistry.put("/Debug", "Debug");

		// ---ù---
		BPELManageUtil.setBpelProcessor(this);
		BPELManageUtil.setContainerEnvironment(this.containerEnv);
	}

	public void stop() throws LifecycleException {

		logger.info(getName() + " Stop");

		// ϵͳرǰڴеݴݿ
		DeploymentUnitRegistry registry = this.getDeploymentUnitRegistry();
		DeploymentUnit[] deploymentUnits = registry.getAll();
		for (int i = 0; i < deploymentUnits.length; i++) {
			DeploymentUnit deploymentUnit = deploymentUnits[i];
			InstanceUnit[] instanceUnits = deploymentUnit.getInstanceUnitPool()
					.getAll();
			for (int j = 0; j < instanceUnits.length; j++) {
				InstanceUnit instanceUint = instanceUnits[j];
				StoreUtil.storeProcessInstance(instanceUint);
			}
		}

		database.insertEngineRecord(new EngineRecord(startupTime, TimeUtil
				.getNowTime()));
		database.close();

	}

	public void destroy() {
		// TODO Auto-generated method stub

	}

	/**
	 * 
	 * 
	 * @param serviceContext
	 */
	public void process(IServiceContext serviceContext) {

		try {
			// ҪıķΪ
			// Metaset requestService = serviceContext.getInMessage();
			ISOAPEnvelope requestService = serviceContext.getInMessage();
			if(requestService.getHeader().element("xenc:EncryptedData")!=null)
			  {
			// * 
			Metaset decedSoapMt = null;
			//**********************************************************************//*
			//*********************** ܴ EncHandler **************************//*
			DecryptionHandler decryptionHandler = new DecryptionHandler();
			List<Attribute> attributesList = requestService.attributes();
				decryptionHandler
						.init(
								"http://www.w3.org/2001/04/xmlenc#rsa-1.5",
								"c:\\server.keystore",
								attributesList, "soap");

				try {
					decedSoapMt = decryptionHandler.process(requestService,
							"http://www.w3.org/2001/04/xmlenc#tripledes-cbc");		
//					"http://www.w3.org/2001/04/xmlenc#aes256-cbc");
					
				} catch (Exception e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				requestService = (ISOAPEnvelope)decedSoapMt;
			  }
		
		
			
			
			
			// õSOAPϢHeader
			ISOAPHeader headerContext = requestService.getHeader();
			// ҪısoapHeaderаΪTransactionIDServicesSet
			// Metaset transactionContext =
			// requestService.filter(".\\Header\\TransactionID");
			Metaset transactionContext = null;
			Metaset collaborationContext = null;
			Metaset manageContext = null;
			if (headerContext != null) {
				transactionContext = headerContext.element("TransactionID");

				// ЭͬҪısoapHeaderаΪCollaborationIDServicesSet
				// Metaset collaborationContext =
				// requestService.filter(".\\Header\\CollaborationID");
				collaborationContext = headerContext.element("CollaborationID");

				// ҪısoapHeaderаΪManageIDServicesSet
				// Metaset manageContext =
				// requestService.filter(".\\Header\\ManageID");
				manageContext = headerContext.element("ManageID");
			}
			if (transactionContext != null) {

				// Ϣ

			} else if (collaborationContext != null) {
				/**
				 * ЭͬϢ
				 * 
				 * @author Liumin 2008-1-18
				 */
				String methodName = getMethod(requestService);
				try {
					Class c = Class
							.forName("org.act.xservices.wscp.dynamic.RequestProcessor");
					// MessageWrapper wrapper = new MessageWrapper();

					Method method;
					method = c.getMethod(methodName,
							new Class[] { Metaset.class });
					Metaset msg = (Metaset) method.invoke(c.newInstance(),
							new Object[] { requestService });
					System.out.println(msg.toXML());
					// add by viking 2010 07 07
					ISOAPEnvelope envelope = SOAPMessageUtil
							.convertMetasetToSOAPEnvelope(msg);

					serviceContext.setOutMessage(envelope);

				} catch (Exception e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				try {
					this.targetObjectPool.put(serviceContext);
				} catch (InterruptedException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}

			} else if (manageContext != null) {
				// Ϣ
				// add by chenyy
				// String fullmethodName = getMethod(requestService);
				// int index = fullmethodName.indexOf('}');
				// String methodName = fullmethodName.substring(index+1);
				// /**
				// * ڰorg.act.xservices.engine.rt.admin.SystemInfoWrapper
				// * @author Liumin
				// * 2007-6-5
				// */
				// Class<?> c =
				// Class.forName("org.act.xservices.engine.rt.admin.SystemInfoWrapper");
				// SystemInfoWrapper a = new SystemInfoWrapper();
				// Method method = c.getMethod(methodName, new Class[] {
				// Metaset.class });
				// Metaset msg = (Metaset) method.invoke(a, new Object[] {
				// requestService });
				// System.out.println(msg.toXML());
				// serviceContext.setOutMessage(msg);
				// this.targetObjectPool.put(serviceContext);

			} else {
				// ʵϢ
				String URL = serviceContext.getRequestURI().substring(1);

				// String op =
				// requestService.filter(".\\Body\\*").car().getName();
				ISOAPBody requestBody = requestService.getBody();
				String op = requestBody.elements().get(0).getName();

				logger.info(sm.getString(
						"org.act.xservices.engine.rt.recMessage", URL, op));

				// Metaset messageID =
				// requestService.filter(".\\Header\\MessageID");
				ISOAPHeader requestHeader = requestService.getHeader();
				
				Metaset messageID = null;
				if(requestHeader != null){
					messageID = requestHeader.element("MessageID");
				}

				if (URL.equals("ABC")) {
					// ZYJ Ϣģʽ
					// ͬϢĿǰRuntimeײ㲻֧Out-InģʽʹõAxisͻˣⲿֲִ
					// String msgID = messageID.car().getText();
					if(messageID == null){
						throw new Exception("meessageID is null");
					}
					String msgID = messageID.elements().get(0).getText();
					String[] tags = msgID.split("-");
					String instanceId = tags[0];
					String[] instanceIdTags = instanceId.split(":");
					String id = instanceIdTags[0];
					String partnerLink = tags[1];
					String operation = tags[2];

					DeploymentUnit deploymentUnit = getDeploymentUnitRegistry()
							.get(id);
					InstanceUnit instance = deploymentUnit
							.getInstanceUnitPool().get(instanceId);
					instance.insertMessage(partnerLink, operation,
							serviceContext);
				} else {
					// ʵrequestϢͨURLе?pl=plNameжĸ
					int index = URL.indexOf("?");
					String processPart = URL.substring(0, index);
					String[] tags = processPart.split("/");
					String id = tags[0];
					for (int i = 1; i < tags.length; i++) {
						id = id + "#" + tags[i];
					}
					String plPart = URL.substring(index + 1);
					String partnerLink = plPart.substring(3);
					// String operation =
					// requestService.filter(".\\Body\\*").car().getName();
					ISOAPBody requestBody1 = requestService.getBody();
					String operation = requestBody1.elements().get(0).getName();

					DeploymentUnit deploymentUnit = getDeploymentUnitRegistry()
							.get(id);
					if (deploymentUnit != null) {
						// Ѳ
						if (deploymentUnit.getState() == DeploymentUnit.STATE_SUSPENDED) {
							// ̲Ԫ״̬suspendedʵ
							throw new Exception(
									"Can't create the instance. The state of "
											+ id + " is 'suspended'");
						}

						if (deploymentUnit.isInstanceEntry(partnerLink,
								operation)) {
							// Ǵʵڣ򴴽µʵ
							final InstanceUnit instance = deploymentUnit
									.createInstance();
							instance.setMessageOut(this.targetObjectPool);
							instance.insertMessage(partnerLink, operation,
									serviceContext);
							Runnable runnable = new Runnable() {
								public void run() {
									logger.info(sm
											.getString(
													"org.act.xservices.engine.rt.beginInstance",
													instance.getId()));
									instance.run();
									logger.info(sm
											.getString(
													"org.act.xservices.engine.rt.endInstance",
													instance.getId()));
								}
							};
							Thread thread = new Thread(runnable);
							thread.setName(instance.getId());
							thread.start();
						} else {
							// Ǵʵ
							if (isSingleton(id)) {
								// Ϊһʵ̵
								String instanceId = getSingleton(id);
								InstanceUnit instance = deploymentUnit
										.getInstanceUnitPool().get(instanceId);
								instance.insertMessage(partnerLink, operation,
										serviceContext);
							} else if (deploymentUnit.isCorrelationEntry(
									partnerLink, operation)) {
								// ؼƥʵ
								String instanceId = deploymentUnit
										.getCorrelatedInstanceUnit(partnerLink,
												operation, serviceContext);
								if (instanceId == null) {
									throw new Exception(
											"Can't find the process instance requested. ("
													+ URL + ":" + op + ")");
								}
								InstanceUnit instance = deploymentUnit
										.getInstanceUnitPool().get(instanceId);
								instance.insertMessage(partnerLink, operation,
										serviceContext);
							} else {
								// ͵ʵ
								// String msgID = messageID.car().getText();
								String msgID = messageID.elements().get(0)
										.getText();
								String[] msgIDTags = msgID.split("-");
								String instanceId = msgIDTags[0];
								InstanceUnit instance = deploymentUnit
										.getInstanceUnitPool().get(instanceId);
								instance.insertMessage(partnerLink, operation,
										serviceContext);
							}
						}
					} else {
						// ûв
						throw new Exception("Can't find the process named "
								+ id);
					}
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/*
	 * private String getMethod(Metaset set) { // TODO
	 * ,2005-05-20,ʱֻǷƺͲĸǷ //
	 * ͬһͬʱӦrpcdocument͵bindingô Metaset childElements =
	 * set.filter(".\\Body\\*"); if (!childElements.isEmpty()) { Metaset
	 * firstChildElement = childElements.car(); String name =
	 * firstChildElement.getFullStringName(); return name; } return null; }
	 */

	private String getMethod(Metaset set) {
		// ʱֻǷƺͲĸǷ
		ISOAPEnvelope envelope = SOAPMessageUtil
				.convertMetasetToSOAPEnvelope(set);
		ISOAPBody body = envelope.getBody();
		List<Metaset> childElements = body.elements();
		if (!childElements.isEmpty()) {
			Metaset firstChildElement = childElements.get(0);
			String localPart = firstChildElement.getName();
			String uri = firstChildElement.getNamespaceURI();
			String fullName = "{" + uri + "}" + localPart;
			return fullName;
		}
		return null;
	}

	// ---------------------------------------
	public DeploymentUnitRegistry getDeploymentUnitRegistry() {
		return deploymentUnitRegistry;
	}

	public static AspectDeploymentUnitRegistry getAspectDeploymentUnitRegistry() {
		return aspectDeploymentUnitRegistry;
	}

	public void createSingleton(String id) {
		singletons.put(id, "");
	}

	public boolean isSingleton(String id) {
		String instanceID = singletons.get(id);
		if (instanceID != null) {
			return true;
		}
		return false;
	}

	public void setSingleton(String id, String instanceID) {
		singletons.put(id, instanceID);
	}

	public String getSingleton(String id) {
		return singletons.get(id);
	}

	public IDatabase getDatabase() {
		return database;
	}

	public String getVersion() {
		return version;
	}

	public String getStartupTime() {
		return startupTime;
	}

}
