package org.act.xservices.engine.rt.bpel11;

import java.io.File;
import java.io.FileInputStream;
import java.util.Collections;
import java.util.List;

import javax.wsdl.Message;
import javax.wsdl.Part;
import javax.wsdl.Port;
import javax.wsdl.PortType;
import javax.wsdl.Service;
import javax.wsdl.extensions.UnknownExtensibilityElement;

import org.act.xservices.bpel.wsdl.DefinitionsExtended;
import org.act.xservices.bpel.wsdl.PartnerLinkType;
import org.act.xservices.bpel11.TCorrelationSet;
import org.act.xservices.bpel11.TCorrelationSets;
import org.act.xservices.bpel11.TPartnerLink;
import org.act.xservices.bpel11.TPartnerLinks;
import org.act.xservices.bpel11.TProcess;
import org.act.xservices.bpel11.TVariable;
import org.act.xservices.bpel11.TVariables;
import org.act.xservices.bpel11.util.ProcessXMLProcessor;
import org.act.xservices.engine.rt.PartnerLink;
import org.act.xservices.engine.rt.VariableDef;
import org.act.xservices.engine.rt.addressing.EndpointReference;
import org.act.xservices.engine.rt.utils.ConstantUtil;
import org.act.xservices.engine.rt.utils.QNameUtil;
import org.act.xservices.engine.rt.utils.XSDUtil;
import org.act.xservices.rt.metaset.utils.MetasetUtil;
import org.act.xservices.rt.xcontainer.inf.service.wsdl.XcService;
import org.act.xservices.rt.xcontainer.inf.service.wsdl.schema.ComplexType;
import org.act.xservices.rt.xcontainer.inf.service.wsdl.schema.ElementType;
import org.act.xservices.rt.xcontainer.inf.service.wsdl.schema.MessageSchema;
import org.act.xservices.rt.xcontainer.inf.service.wsdl.schema.ServiceSchema;
import org.act.xservices.rt.xcontainer.wsdl.WSDLParseException;
import org.act.xservices.rt.xcontainer.wsdl.schemaImpl.MessagePartParser;
import org.act.xservices.rt.xom.QName;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;

import commonj.sdo.DataObject;
import commonj.sdo.Property;
import commonj.sdo.Type;
import commonj.sdo.helper.DataFactory;
import commonj.sdo.helper.TypeHelper;

public class Bpel11BarDeploy {

	/**
	 * BPEL1.1̵ĲԪ(.bar)
	 * 
	 * @param deploymentUnit
	 * @param bpelFile
	 * @throws Exception
	 */
	public static void deploy(Bpel11DeploymentUnit deploymentUnit, File bpelFile)
			throws Exception {
		// BPEL
		ProcessXMLProcessor processor = new ProcessXMLProcessor();
		FileInputStream inputStream = new FileInputStream(bpelFile.toString());
		Resource resource = processor.load(inputStream, Collections.EMPTY_MAP);
		EObject documentRoot = (EObject) resource.getContents().get(0);
		TProcess process = (TProcess) documentRoot.eContents().get(0);
		deploymentUnit.setBpel(process);

		// б
		createPartnerLinkList(deploymentUnit);

		// ؼ
		TCorrelationSets correlationSets = process.getCorrelationSets();
		if (correlationSets != null) {
			for (int i = 0; i < correlationSets.getCorrelationSet().size(); i++) {
				TCorrelationSet correlationSet = correlationSets
						.getCorrelationSet().get(i);
				deploymentUnit.addCorrelationSet(correlationSet);
			}
		}

		// SDOϢͺͱб
		defineVariableType(deploymentUnit);
	}

	/**
	 * б
	 * 
	 * @param deploymentUnit
	 */
	private static void createPartnerLinkList(
			Bpel11DeploymentUnit deploymentUnit) {
		TProcess process = deploymentUnit.getBpel();
		TPartnerLinks tpartnerLinks = process.getPartnerLinks();
		for (int i = 0; i < tpartnerLinks.getPartnerLink().size(); i++) {
			TPartnerLink tpartnerLink = tpartnerLinks.getPartnerLink().get(i);
			PartnerLink partnerLink = new PartnerLink();
			partnerLink.setName(tpartnerLink.getName());
			QName tpartnerLinkType = QNameUtil.translate(tpartnerLink
					.getPartnerLinkType());
			String pltURI = tpartnerLinkType.getNamespaceURI();
			String pltLocalPart = tpartnerLinkType.getName();
			DefinitionsExtended wsdl = deploymentUnit.getWsdl(pltURI);
			PartnerLinkType partnerLinkType = wsdl
					.getPartnerLinkType(pltLocalPart);
			partnerLink.setPartnerLinkType(partnerLinkType);
			String myRole = tpartnerLink.getMyRole();
			if (myRole != null) {
				partnerLink.setMyRole(myRole);
				EndpointReference endpoint = new EndpointReference();
				QName portTypeNS = MetasetUtil.jQNameToxQName(partnerLinkType
						.getRole(myRole).getPortType().getName());
				wsdl = deploymentUnit.getWsdl(portTypeNS.getNamespaceURI());
				PortType portType = wsdl.getDefinition().getPortType(
						MetasetUtil.xQNameTojQName(portTypeNS));
				endpoint.setPortType(portType);
				Service[] services = (Service[]) wsdl.getDefinition()
						.getServices().values().toArray(new Service[0]);
				for (int j = 0; j < services.length; j++) {
					Service service = services[j];
					Port[] ports = (Port[]) service.getPorts().values()
							.toArray(new Port[0]);
					for (int k = 0; k < ports.length; k++) {
						Port port = ports[k];
						if (port.getBinding().getPortType() == portType) {
							endpoint.setService(service);
							endpoint.setPort(port);
						}
					}
				}
				partnerLink.setMyEndpoint(endpoint);
			}
			String partnerRole = tpartnerLink.getPartnerRole();
			if (partnerRole != null) {
				partnerLink.setPartnerRole(partnerRole);
				EndpointReference endpoint = new EndpointReference();
				QName portTypeNS = MetasetUtil.jQNameToxQName(partnerLinkType
						.getRole(partnerRole).getPortType().getName());
				wsdl = deploymentUnit.getWsdl(portTypeNS.getNamespaceURI());
				PortType portType = wsdl.getDefinition().getPortType(
						MetasetUtil.xQNameTojQName(portTypeNS));
				endpoint.setPortType(portType);
				Service[] services = (Service[]) wsdl.getDefinition()
						.getServices().values().toArray(new Service[0]);
				for (int j = 0; j < services.length; j++) {
					Service service = services[j];
					Port[] ports = (Port[]) service.getPorts().values()
							.toArray(new Port[0]);
					for (int k = 0; k < ports.length; k++) {
						Port port = ports[k];
						if (port.getBinding().getPortType() == portType) {
							endpoint.setService(service);
							endpoint.setPort(port);
							List<UnknownExtensibilityElement> elements = port
									.getExtensibilityElements();
							for (int m = 0; m < elements.size(); m++) {
								UnknownExtensibilityElement element = elements
										.get(m);
								if (element.getElement().getLocalName()
										.equals("address")) {
									String address = element.getElement()
											.getAttribute("location");
									endpoint.setAddress(address);
								}
							}
						}
					}
				}
				partnerLink.setPartnerEndpoint(endpoint);
			}
			deploymentUnit.addPartnerLink(partnerLink);
		}
	}

	/**
	 * 
	 * 
	 * @param deploymentUnit
	 */
	private static void defineVariableType(Bpel11DeploymentUnit deploymentUnit) {
		// Ŀǰֻ֧processǩ¶ı
		TProcess process = deploymentUnit.getBpel();
		TVariables variables = process.getVariables();
		for (int j = 0; j < variables.getVariable().size(); j++) {
			VariableDef variableDef = new VariableDef();
			TVariable variable = variables.getVariable().get(j);
			String varName = variable.getName();
			variableDef.setName(varName);
			QName messageType = QNameUtil.translate(variable.getMessageType());

			if (messageType != null) {
				// ΪmessageType
				variableDef.setMessageType(messageType);
				String msgTypeURI = messageType.getNamespaceURI();
				String msgTypeLocalPart = messageType.getName();
				DefinitionsExtended wsdl = deploymentUnit.getWsdl(msgTypeURI);
				Message message = wsdl.getDefinition().getMessage(
						MetasetUtil.xQNameTojQName(messageType));

				DataObject messageObject = DataFactory.INSTANCE.create(
						"commonj.sdo", "Type");
				messageObject.set("uri", msgTypeURI);
				messageObject.set("name", msgTypeLocalPart);
				List parts = message.getOrderedParts(null);

				// wsdlmessageSchema
				File wsdlFile = deploymentUnit.getWsdlFile(msgTypeURI);
				MessagePartParser parser = new MessagePartParser(wsdlFile);
				XcService xservice = null;
				try {
					xservice = parser.parseWSDL();
				} catch (WSDLParseException e) {
					e.printStackTrace();
				}
				ServiceSchema schema = xservice.getServiceSchema();
				MessageSchema ms = schema.getMessageSchema(msgTypeLocalPart);
				//
				for (int k = 0; k < parts.size(); k++) {
					Part part = (Part) parts.get(k);
					DataObject ptProperty = messageObject
							.createDataObject("property");
					ptProperty.set("name", part.getName());
					Type type = null;

					ElementType partSchema = ms.getAllElements().get(k);
					String uri = partSchema.getType().getNamespaceURI();
					String local = partSchema.getType().getLocalPart();

					if (partSchema.isSimpleType()) {
						type = XSDUtil.getType(local);
					} else {
						ComplexType complexType = partSchema.getComplex();

						// SDOעḴ
						DataObject complexObject = DataFactory.INSTANCE.create(
								"commonj.sdo", "Type");
						complexObject.set("name", local);
						complexObject.set("uri", uri);
						registerComplexType(complexObject, complexType);
						// ڵܴͣ
						type = TypeHelper.INSTANCE.getType(uri, local);
					}
					ptProperty.set("type", type);
				}
				TypeHelper.INSTANCE.define(messageObject);
			}

			QName type = QNameUtil.translate(variable.getType());
			if (type != null) {
				//Ϊtype
				variableDef.setType(type);
			}

			QName element = QNameUtil.translate(variable.getElement());
			if (element != null) {
				//Ϊelement
				variableDef.setElement(element);
			}

			deploymentUnit.addVariableDef(variableDef);
		}
	}

	/**
	 * SDOעḴ
	 * 
	 * @param complexObject
	 * @param complexType
	 */
	private static void registerComplexType(DataObject complexObject,
			ComplexType complexType) {

		List<ElementType> typeElements = complexType.getElementTypes();
		for (int i = 0; i < typeElements.size(); i++) {
			ElementType typeElement = typeElements.get(i);
			DataObject complexProperty = complexObject
					.createDataObject("property");
			String name = typeElement.getName();
			complexProperty.set("name", name);
			Type type = null;
			String local = typeElement.getType().getLocalPart();
			String uri = typeElement.getType().getNamespaceURI();
			if (typeElement.isSimpleType()) {
				type = XSDUtil.getType(local);

			} else {
				ComplexType complexType0 = typeElement.getComplex();
				DataObject complexObject0 = DataFactory.INSTANCE.create(
						"commonj.sdo", "Type");
				complexObject0.set("name", local);
				complexObject0.set("uri", uri);
				registerComplexType(complexObject0, complexType0);

				type = TypeHelper.INSTANCE.getType(uri, local);
			}
			complexProperty.set("type", type);
		}

		TypeHelper.INSTANCE.define(complexObject);// עType
	}
}
