package org.act.xservices.engine.rt.bpel11;

import java.util.ArrayList;
import java.util.List;

import javax.wsdl.Message;
import javax.wsdl.Part;
import javax.wsdl.PortType;

import org.act.xservices.bpel.wsdl.DefinitionsExtended;
import org.act.xservices.bpel.wsdl.PropertyAlias;
import org.act.xservices.bpel11.PatternType;
import org.act.xservices.bpel11.TBoolean;
import org.act.xservices.bpel11.TCorrelationSet;
import org.act.xservices.bpel11.TCorrelationWithPattern;
import org.act.xservices.bpel11.TCorrelationsWithPattern;
import org.act.xservices.bpel11.TInvoke;
import org.act.xservices.bpel11.TVariable;
import org.act.xservices.bpel11.TVariables;
import org.act.xservices.engine.rt.CorrelatedInstanceUnit;
import org.act.xservices.engine.rt.CorrelationSetInstance;
import org.act.xservices.engine.rt.PropertyInstance;
import org.act.xservices.engine.rt.VariableContainer;
import org.act.xservices.engine.rt.VariableDef;
import org.act.xservices.engine.rt.addressing.EndpointReference;
import org.act.xservices.engine.rt.utils.DataUtil;
import org.act.xservices.engine.rt.utils.QNameUtil;
import org.act.xservices.engine.rt.utils.WSDLUtil;
import org.act.xservices.rt.client.ServiceClient;
import org.act.xservices.rt.metaset.MetasetBuilder;
import org.act.xservices.rt.metaset.utils.MetasetUtil;
import org.act.xservices.rt.xom.Metaset;
import org.act.xservices.rt.xom.QName;
import org.eclipse.emf.common.util.EList;

import commonj.sdo.DataObject;
import commonj.sdo.helper.DataFactory;

public class InvokeInstance extends Bpel11ActivityInstance {

	public InvokeInstance(TInvoke invoke, Bpel11InstanceUnit instanceUnit) {
		this.type = "Invoke";
		this.name = invoke.getName();
		this.activity = invoke;
		this.instanceUnit = instanceUnit;
	}

	// :
	// <invoke partnerLink="ncname" portType="qname" operation="ncname"
	// inputVariable="ncname"? outputVariable="ncname"? standard-attributes>
	// standard-elements
	// <correlations>?
	// <correlation set="ncname" initiate="yes|no"? pattern="in|out|out-in"/>+
	// </correlations>
	// <catch faultName="qname" faultVariable="ncname"?>* activity </catch>
	// <catchAll>?activity </catchAll>
	// <compensationHandler>? activity</compensationHandler>
	// </invoke>
	public void run() throws Exception {
		super.start();
		TInvoke invoke = (TInvoke) this.activity;
		Bpel11InstanceUnit instanceUnit = (Bpel11InstanceUnit) this.instanceUnit;

		String partnerLink = invoke.getPartnerLink();
		String operation = invoke.getOperation();
		String inputVariable = invoke.getInputVariable();
		String outputVariable = invoke.getOutputVariable();
		TCorrelationsWithPattern correlations = invoke.getCorrelations();

		System.out.println("partnerLink: " + partnerLink + " operation: "
				+ operation + "  inputVariable: " + inputVariable
				+ "  outputVariable: " + outputVariable);

		EndpointReference endpoint = instanceUnit.getDeploymentUnit()
				.getPartnerLink(partnerLink).getPartnerEndpoint();

		String address = endpoint.getAddress();
		PortType portType = endpoint.getPortType();
		String opUri = portType.getQName().getNamespaceURI();
		// жSOAPϢʽ
		boolean isDocument;
		Bpel11DeploymentUnit deploymentUnit = (Bpel11DeploymentUnit) instanceUnit
				.getDeploymentUnit();
		DefinitionsExtended wsdl = deploymentUnit.getWsdl(opUri);
		isDocument = WSDLUtil.isDocument(wsdl);

		if (outputVariable == null) {
			// 첽Invoke
			this.invokeAsyn(partnerLink, address, operation, opUri,
					inputVariable, isDocument);
		} else {
			// ͬInvoke
			this.invokeSync(partnerLink, address, operation, opUri,
					inputVariable, outputVariable, correlations, isDocument);
		}
		super.finish();
	}

	private void invokeAsyn(String partnerLink, String address, String op,
			String opUri, String inputVariable, boolean isDocument)
			throws Exception {
		// sendRequest(partnerLink, address, op, inputVariable);
		Metaset request = createRequest(op, opUri, inputVariable,
				isDocument);
		ServiceClient client = new ServiceClient(address);
		client.sendReceiveNonBlocking(request);

		/*
		 * Service service=new Service(); Call call =
		 * (Call)service.createCall(); Message reqmsg=new
		 * Message(request.toXML(),true); MessageContext
		 * context=call.getMessageContext();
		 * call.setTargetEndpointAddress(address);
		 * call.setOperationName(MetasetUtil.xQNameTojQName(new QName(op)));
		 * context.setRequestMessage(reqmsg); call.invokeOneWay(new Object[]{});
		 */
		// RPCServiceClient service = new RPCServiceClient(address);

		// service.invokeRobust(op,request);

	}

	private void invokeSync(String partnerLink, String address, String op,
			String opUri, String inputVariable, String outputVariable,
			TCorrelationsWithPattern correlations, boolean isDocument)
			throws Exception {

		Metaset request = createRequest(op, opUri, inputVariable,
				isDocument);
		ServiceClient client = new ServiceClient(address);
		// requestmessageBodyµĵһElement
		Metaset message = client.sendReceive(request);

		Bpel11InstanceUnit instanceUnit = (Bpel11InstanceUnit) this.instanceUnit;
		// Metaset paras = message.filter(".\\Body\\*").car();
		
		// дϢSOAP

		if (message.getName().equals("Fault")) {
			throw new Exception("SOAP message has faults!");
		}
		Metaset paras = amendReceive(message, isDocument);
		
		Bpel11DeploymentUnit deploymentUnit = (Bpel11DeploymentUnit) instanceUnit
				.getDeploymentUnit();
		VariableDef variableDef = deploymentUnit.getVariableDef(outputVariable);
		QName messageType = variableDef.getMessageType();
		String messageTypeURI = messageType.getNamespaceURI();
		String messageTypeLocalPart = messageType.getName();
		DataObject var = DataFactory.INSTANCE.create(messageTypeURI,
				messageTypeLocalPart);
		DataUtil.metaset2data(paras, var);
		VariableContainer vc = instanceUnit.getVariableContainer();
		vc.setVariable(outputVariable, var);

		if (correlations != null) {
			for (int i = 0; i < correlations.getCorrelation().size(); i++) {
				TCorrelationWithPattern correlation = correlations
						.getCorrelation().get(i);
				if (correlation.getInitiate().equals(TBoolean.YES)
						&& correlation.getPattern().equals(PatternType.IN)) {
					String set = correlation.getSet();
					CorrelatedInstanceUnit correlatedInstanceUnit = new CorrelatedInstanceUnit();
					correlatedInstanceUnit.setInstanceId(this.instanceUnit
							.getId());
					CorrelationSetInstance correlationSetInstance = new CorrelationSetInstance();
					correlationSetInstance.setName(set);
					TCorrelationSet correlationSet = deploymentUnit
							.getCorrelationSet(set);
					for (int j = 0; j < correlationSet.getProperties().size(); j++) {
						QName property = QNameUtil
								.translate((org.eclipse.emf.ecore.xml.type.internal.QName) correlationSet
										.getProperties().get(j));
						PropertyInstance propertyInstance = new PropertyInstance();
						propertyInstance.setQname(property);
						ArrayList<PropertyAlias> propertyAliases = deploymentUnit
								.getPropertyAlias(messageType);
						for (int k = 0; k < propertyAliases.size(); k++) {
							PropertyAlias propertyAlias = propertyAliases
									.get(k);
							String part = propertyAlias.getPart();
							String query = propertyAlias.getQuery();
							String path = query;
							if (part != null) {
								path = part + path;
							}
							Object value = var.get(path);
							propertyInstance.setValue(value);
						}
						correlationSetInstance
								.addPropertyInstance(propertyInstance);
					}
					correlatedInstanceUnit
							.setCorrelationSetInstance(correlationSetInstance);
					deploymentUnit
							.addCorrelatedInstanceUnit(correlatedInstanceUnit);
				}
			}
		}
	}

	private Metaset createRequest(String op, String opUri,
			String inputVariable, boolean isDocument) {
		Bpel11InstanceUnit instanceUnit = (Bpel11InstanceUnit) this.instanceUnit;

		Metaset paras = MetasetBuilder.getInstance()
				.newMetaset(opUri, op, "ns");
		DataUtil.data2metaset(instanceUnit, inputVariable, paras);
		if (isDocument) {
			Metaset paras0 = MetasetBuilder.getInstance().newMetaset(opUri, op,
					"ns");
			List<Metaset> parts = paras.elements();
			for (int i = 0; i < parts.size(); i++) {
				Metaset part = parts.get(i);
				paras.remove(part);
				List<Metaset> elements = part.elements();
				for (int j = 0; j < elements.size(); j++) {
					Metaset element = elements.get(j);
					part.remove(element);
					paras0.add(element);
				}
			}
			return paras0;
		}
		return paras;
	}
	
	private Metaset amendReceive(Metaset message , boolean isDocument){
		Metaset paras = null;
		if(isDocument){
		   paras = MetasetBuilder.getInstance().newMetaset();
		   Metaset part = MetasetBuilder.getInstance().newMetaset(null, "part0","ns");//Ŀǰֻһpart
		   List<Metaset> elements = message.elements();
		   for (int j = 0; j < elements.size(); j++) {
				Metaset element = elements.get(j);
				message.remove(element);
				part.add(element);
			}
		   paras.add(part);
		}else{
			paras = message;//RPC
		}
		return paras;
	}
	// private void sendRequest(String partnerLink, String address, String op,
	// String inputVariable) {
	// Bpel11InstanceUnit instanceUnit = (Bpel11InstanceUnit)this.instanceUnit;
	//
	// Metaset soapMsg = SoapUtil.createEmptySoap();
	// Metaset header = soapMsg.filter(".\\Header").car();
	// Metaset body = soapMsg.filter(".\\Body").car();
	// Metaset messageId = new Metaset("wsa:MessageID");
	//
	// messageId.setValue(instanceUnit.getId() + "-" + partnerLink + "-"+ op);
	// header.add(messageId);
	//
	// Metaset paras = new Metaset(op);
	// DataUtil.data2metaset(instanceUnit, inputVariable, paras);
	// body.add(paras);
	//
	// // ⷢϢ
	// try {
	// Service service=new Service();
	// Call call = (Call)service.createCall();
	// Message reqmsg=new Message(soapMsg.toXML(),true);
	// MessageContext context=call.getMessageContext();
	// call.setTargetEndpointAddress(address);
	// call.setOperationName(new QName(op));
	// context.setRequestMessage(reqmsg);
	// call.invoke();
	// } catch (Exception e) {
	// e.printStackTrace();
	// }
	// }
	//
	// private void getResponse(String partnerLink, String address, String op,
	// String outputVariable) {
	// Bpel11InstanceUnit instanceUnit = (Bpel11InstanceUnit)this.instanceUnit;
	//
	// IServiceContext message = instanceUnit.getMessage(partnerLink, op);
	// Metaset paras = (Metaset) message.getInMessage().filter(
	// ".\\Body\\*").car();
	// DeploymentUnit deploymentUnit = instanceUnit.getDeploymentUnit();
	// VariableDef variableDef = deploymentUnit.getVariableDef(outputVariable);
	// QName messageType = variableDef.getMessageType();
	// String messageTypeURI = messageType.getNamespaceURI();
	// String messageTypeLocalPart = messageType.getLocalPart();
	// DataObject var = DataFactory.INSTANCE.create(messageTypeURI,
	// messageTypeLocalPart);
	// DataUtil.metaset2data(paras, var);
	//
	// VariableContainer vc = instanceUnit.getVariableContainer();
	// vc.setVariable(outputVariable, var);
	// }
}
