package org.act.xservices.engine.rt.bpel20;

import java.io.File;
import java.io.FileInputStream;
import java.util.Collections;
import java.util.List;

import javax.wsdl.Message;
import javax.wsdl.Part;
import javax.wsdl.Port;
import javax.wsdl.PortType;
import javax.wsdl.Service;
import javax.wsdl.extensions.UnknownExtensibilityElement;

import org.act.xservices.bpel.wsdl.DefinitionsExtended;
import org.act.xservices.bpel.wsdl.PartnerLinkType;
import org.act.xservices.bpel20.TCorrelationSet;
import org.act.xservices.bpel20.TCorrelationSets;
import org.act.xservices.bpel20.TPartnerLink;
import org.act.xservices.bpel20.TPartnerLinks;
import org.act.xservices.bpel20.TProcess;
import org.act.xservices.bpel20.TVariable;
import org.act.xservices.bpel20.TVariables;
import org.act.xservices.bpel20.util.ExecutableXMLProcessor;
import org.act.xservices.engine.rt.PartnerLink;
import org.act.xservices.engine.rt.VariableDef;
import org.act.xservices.engine.rt.addressing.EndpointReference;
import org.act.xservices.engine.rt.utils.ConstantUtil;
import org.act.xservices.engine.rt.utils.QNameUtil;
import org.act.xservices.engine.rt.utils.XSDUtil;
import org.act.xservices.rt.metaset.utils.MetasetUtil;
import org.act.xservices.rt.xom.QName;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;

import commonj.sdo.DataObject;
import commonj.sdo.Type;
import commonj.sdo.helper.DataFactory;
import commonj.sdo.helper.TypeHelper;

public class Bpel20BarDeploy {
	
	/**
	 * BPEL2.0̵ĲԪ(.bar)
	 * 
	 * @param deploymentUnit
	 * @param bpelFile
	 * @throws Exception 
	 */
	public static void deploy(Bpel20DeploymentUnit deploymentUnit, File bpelFile) throws Exception {
		//BPEL
		ExecutableXMLProcessor processor = new ExecutableXMLProcessor();
		FileInputStream inputStream = new FileInputStream(bpelFile.toString());
		Resource resource = processor.load(inputStream,Collections.EMPTY_MAP);
		EObject documentRoot = (EObject) resource.getContents().get(0);
		TProcess process = (TProcess) documentRoot.eContents().get(0);
		deploymentUnit.setBpel(process);

		// б
		createPartnerLinkList(deploymentUnit);
			
		// ؼ
		TCorrelationSets correlationSets = process.getCorrelationSets();
		if(correlationSets != null) {
			for(int i = 0; i < correlationSets.getCorrelationSet().size(); i++) {
				TCorrelationSet correlationSet = correlationSets.getCorrelationSet().get(i);
				deploymentUnit.addCorrelationSet(correlationSet);
			}
		}
		
		// SDOϢͺͱб
		defineVariableType(deploymentUnit);
	}
	
	/**
	 * б
	 * 
	 * @param deploymentUnit
	 */
	private static void createPartnerLinkList(Bpel20DeploymentUnit deploymentUnit) {
		TProcess process = deploymentUnit.getBpel();
		TPartnerLinks tpartnerLinks = process.getPartnerLinks();
		for (int i = 0; i < tpartnerLinks.getPartnerLink().size(); i++) {
			TPartnerLink tpartnerLink = tpartnerLinks.getPartnerLink().get(i);
			PartnerLink partnerLink = new PartnerLink();
			partnerLink.setName(tpartnerLink.getName());
			QName tpartnerLinkType =  QNameUtil.translate(tpartnerLink.getPartnerLinkType());
			String pltURI = tpartnerLinkType.getNamespaceURI();
			String pltLocalPart = tpartnerLinkType.getName();
			DefinitionsExtended wsdl = deploymentUnit.getWsdl(pltURI);
			PartnerLinkType partnerLinkType = wsdl.getPartnerLinkType(pltLocalPart);
			partnerLink.setPartnerLinkType(partnerLinkType);
			String myRole = tpartnerLink.getMyRole();
			if (myRole != null) {
				partnerLink.setMyRole(myRole);
				EndpointReference endpoint = new EndpointReference();
				QName portTypeNS = MetasetUtil.jQNameToxQName(partnerLinkType.getRole(myRole).getPortType().getName());
				wsdl = deploymentUnit.getWsdl(portTypeNS.getNamespaceURI());
				PortType portType = wsdl.getDefinition().getPortType(MetasetUtil.xQNameTojQName( portTypeNS ));
				endpoint.setPortType(portType);
				Service[] services = (Service[]) wsdl.getDefinition().getServices().values().toArray(new Service[0]);
				for (int j = 0; j < services.length; j++) {
					Service service = services[j];
					Port[] ports = (Port[]) service.getPorts().values().toArray(new Port[0]);
					for (int k = 0; k < ports.length; k++) {
						Port port = ports[k];
						if (port.getBinding().getPortType() == portType) {
							endpoint.setService(service);
							endpoint.setPort(port);
						}
					}
				}
				partnerLink.setMyEndpoint(endpoint);
			}
			String partnerRole = tpartnerLink.getPartnerRole();
			if (partnerRole != null) {
				partnerLink.setPartnerRole(partnerRole);
				EndpointReference endpoint = new EndpointReference();
				QName portTypeNS = MetasetUtil.jQNameToxQName(partnerLinkType.getRole(partnerRole).getPortType().getName());
				wsdl = deploymentUnit.getWsdl(portTypeNS.getNamespaceURI());
				PortType portType = wsdl.getDefinition().getPortType(MetasetUtil.xQNameTojQName(portTypeNS));
				endpoint.setPortType(portType);
				Service[] services = (Service[]) wsdl.getDefinition().getServices().values().toArray(new Service[0]);
				for (int j = 0; j < services.length; j++) {
					Service service = services[j];
					Port[] ports = (Port[]) service.getPorts().values().toArray(new Port[0]);
					for (int k = 0; k < ports.length; k++) {
						Port port = ports[k];
						if (port.getBinding().getPortType() == portType) {
							endpoint.setService(service);
							endpoint.setPort(port);
							List<UnknownExtensibilityElement> elements = port.getExtensibilityElements();
							for (int m = 0; m < elements.size(); m++) {
								UnknownExtensibilityElement element = elements.get(m);
								if (element.getElement().getLocalName().equals("address")) {
									String address = element.getElement().getAttribute("location");
									endpoint.setAddress(address);
								}
							}
						}
					}
				}
				partnerLink.setPartnerEndpoint(endpoint);
			}
			deploymentUnit.addPartnerLink(partnerLink);
		}
	}
	
	/**
	 * 
	 * 
	 * @param deploymentUnit
	 */
	private static void defineVariableType(Bpel20DeploymentUnit deploymentUnit) {
		TProcess process = deploymentUnit.getBpel();
		TVariables variables = process.getVariables();
		for (int j = 0; j < variables.getVariable().size(); j++) {
			VariableDef variableDef = new VariableDef();
			TVariable variable = variables.getVariable().get(j);
			String varName = variable.getName();
			variableDef.setName(varName);
			QName messageType = QNameUtil.translate(variable.getMessageType());	
			
			if(messageType != null) {
				//ΪmessageType
				variableDef.setMessageType(messageType);
				String msgTypeURI = messageType.getNamespaceURI();
			    String msgTypeLocalPart = messageType.getName();
			    DefinitionsExtended wsdl = deploymentUnit.getWsdl(msgTypeURI);
			    Message message = wsdl.getDefinition().getMessage(MetasetUtil.xQNameTojQName(messageType));
			    DataObject messageObject = DataFactory.INSTANCE.create("commonj.sdo", "Type");
			    messageObject.set("uri", msgTypeURI);
			    messageObject.set("name", msgTypeLocalPart);
			    List parts = message.getOrderedParts(null);
			    for (int k = 0; k < parts.size(); k++) {
			    	Part part = (Part)parts.get(k);
				    DataObject ptProperty = messageObject.createDataObject("property");
				    ptProperty.set("name", part.getName());
				    QName partType = null;
				    if(part.getTypeName() != null){
				    	partType = MetasetUtil.jQNameToxQName(part.getTypeName());
				    }
				    if (partType != null) {
				    	Type type = null;
					    String uri = partType.getNamespaceURI();
					    String local = partType.getName();
					    if (uri.equals(ConstantUtil.NS_XSD)) {
					    	type = XSDUtil.getType(partType.getName());
					    } else {
						    type = TypeHelper.INSTANCE.getType(uri,local);
					    }
					    ptProperty.set("type", type);
				    }
				    else{
				    	QName element = null;
				    	if(part.getElementName() != null){
				    		element = MetasetUtil.jQNameToxQName(part.getElementName());
				    	}
				    	if(element != null) {
				    		String uri = element.getNamespaceURI();
				    		String local = element.getName();
				    		Type type = TypeHelper.INSTANCE.getType(uri,local);
				    		ptProperty.set("type", type);
				    	}
				    }
			    }
			    TypeHelper.INSTANCE.define(messageObject);
			}
			
			QName type = QNameUtil.translate(variable.getType());		
			if(type != null) {
				////Ϊtype
				variableDef.setType(type);
			}
			
			QName element = QNameUtil.translate(variable.getElement());
			if(element != null) {
				////Ϊelement
				variableDef.setElement(element);
			}
			
			deploymentUnit.addVariableDef(variableDef);
		}
	}
}
