package org.act.xservices.engine.rt.bpel20;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import javax.wsdl.Message;
import javax.wsdl.Operation;
import javax.wsdl.PortType;

import org.act.xservices.bpel.wsdl.DefinitionsExtended;
import org.act.xservices.bpel.wsdl.PropertyAlias;
import org.act.xservices.bpel20.TCorrelationSet;
import org.act.xservices.bpel20.TProcess;
import org.act.xservices.engine.rt.CorrelatedInstanceUnit;
import org.act.xservices.engine.rt.CorrelatedOperation;
import org.act.xservices.engine.rt.CorrelationSetInstance;
import org.act.xservices.engine.rt.InstanceUnit;
import org.act.xservices.engine.rt.PartnerLink;
import org.act.xservices.engine.rt.PropertyInstance;
import org.act.xservices.engine.rt.addressing.EndpointReference;
import org.act.xservices.engine.rt.deploy.DeploymentUnit;
import org.act.xservices.engine.rt.utils.DataUtil;
import org.act.xservices.engine.rt.utils.QNameUtil;
import org.act.xservices.rt.metaset.utils.MetasetUtil;
import org.act.xservices.rt.soap.ISOAPEnvelope;
import org.act.xservices.rt.xcontainer.inf.context.IServiceContext;
import org.act.xservices.rt.xom.Metaset;
import org.act.xservices.rt.xom.QName;

import commonj.sdo.DataObject;
import commonj.sdo.Type;
import commonj.sdo.helper.DataFactory;
import commonj.sdo.helper.TypeHelper;

/**
 * BPEL1.1 ̲Ԫ
 * @author sun
 *
 */
public class Bpel20DeploymentUnit extends DeploymentUnit {

	/**
	 * BPEL
	 */
	private TProcess bpel;

	/**
	 * WSDLĿ¼
	 */
	private HashMap<String, DefinitionsExtended> wsdlCatalog = new HashMap<String, DefinitionsExtended>();

	/**
	 * ؼ
	 */
	private HashMap<String, TCorrelationSet> correlationSets = new HashMap<String, TCorrelationSet>();

	public TProcess getBpel() {
		return bpel;
	}

	public void setBpel(TProcess bpel) {
		this.bpel = bpel;
	}

	public DefinitionsExtended getWsdl(String URI) {
		return wsdlCatalog.get(URI);
	}

	public void addWsdl(DefinitionsExtended wsdl) {
		wsdlCatalog.put(wsdl.getDefinition().getTargetNamespace(), wsdl);
	}

	public void addCorrelationSet(TCorrelationSet correlationSet) {
		String name = correlationSet.getName();
		correlationSets.put(name, correlationSet);
	}
	
	public TCorrelationSet getCorrelationSet(String set) {
		return correlationSets.get(set);
	}

	public InstanceUnit createInstance() {
		Bpel20InstanceUnit instanceUnit = new Bpel20InstanceUnit();
		instanceUnit.setId(this.createInstanceId());
		instanceUnit.setDeploymentUnit(this);
		this.getInstanceUnitPool().add(instanceUnit);
		for (int i = 0; i < getPartnerLinks().length; i++) {
			PartnerLink pl = getPartnerLinks()[i];
			EndpointReference endpoint = pl.getMyEndpoint();
			if (endpoint != null) {
				String partnerLink = pl.getName();
				PortType portType = endpoint.getPortType();
				for (int j = 0; j < portType.getOperations().size(); j++) {
					Operation op = (Operation) portType.getOperations().get(j);
					String operation = op.getName();
					instanceUnit.addMessageContainer(partnerLink, operation);
				}
			}
			// ͬõķϢ洢
			endpoint = pl.getPartnerEndpoint();
			if (endpoint != null) {
				String partnerLink = pl.getName();
				PortType portType = endpoint.getPortType();
				for (int j = 0; j < portType.getOperations().size(); j++) {
					Operation op = (Operation) portType.getOperations().get(j);
					String operation = op.getName();
					instanceUnit.addMessageContainer(partnerLink, operation);
				}
			}
		}
		instanceUnit.start();
		return instanceUnit;
	}

	public String getCorrelatedInstanceUnit(String partnerLink,
			String operation, IServiceContext message) {
		//Metaset paras = message.getInMessage().filter(".\\Body\\" + operation).car();
		ISOAPEnvelope envelope = message.getInMessage();
		Metaset paras = envelope.getBody().element(operation);
		Message messageType = null;
		PartnerLink pl = this.getPartnerLink(partnerLink);
		List<Operation> operations = pl.getMyEndpoint().getPortType().getOperations();
		for (int i = 0; i < operations.size(); i++) {
			Operation op = operations.get(i);
			if (op.getName().equals(operation)) {
				messageType = op.getInput().getMessage();
			}
		}
		QName messageQName = MetasetUtil.jQNameToxQName(messageType.getQName());
		String messageURI = messageQName.getNamespaceURI();
		String messageLocalPart = messageQName.getName();
		Type varType = TypeHelper.INSTANCE.getType(messageURI, messageLocalPart);
		DataObject var = DataFactory.INSTANCE.create(varType);
		DataUtil.metaset2data(paras, var);
		
		ArrayList<PropertyAlias> propertyAliases = getPropertyAlias(messageQName);
		HashMap<QName, Object> propertyValue = new HashMap<QName, Object>();
		for(int i=0; i<propertyAliases.size(); i++) {
			PropertyAlias propertyAlias = propertyAliases.get(i);
			QName property = MetasetUtil.jQNameToxQName(propertyAlias.getPropertyName());
			String part = propertyAlias.getPart();
			String query = propertyAlias.getQuery();
			Object value = var.get(part+query);
			propertyValue.put(property, value);
		}

		CorrelatedOperation correlatedOperation = getCorrelationEntry(partnerLink, operation);
		String set = correlatedOperation.getSet();
		
		CorrelationSetInstance csInstance = new CorrelationSetInstance();
		csInstance.setName(set);
		TCorrelationSet correlationSet = correlationSets.get(set);
		for(int i=0; i<correlationSet.getProperties().size(); i++) {
			QName property = QNameUtil.translate(correlationSet.getProperties().get(i));
			PropertyInstance propInstance = new PropertyInstance();
			propInstance.setQname(property);
			propInstance.setValue(propertyValue.get(property));
			csInstance.addPropertyInstance(propInstance);
		}
		
		ArrayList<CorrelatedInstanceUnit> cInstanceUnits = getCorrelatedInstanceUnits(set);
		for(int i=0; i<cInstanceUnits.size(); i++) {
			CorrelatedInstanceUnit correlatedInstanceUnit = cInstanceUnits.get(i);
			CorrelationSetInstance correlationSetInstance = correlatedInstanceUnit.getCorrelationSetInstance();
			if(correlationSetInstance.equals(csInstance)) {
				return correlatedInstanceUnit.getInstanceId();
			}
		}
		return null;
	}
}
