package org.act.xservices.engine.rt.bpel20;

import javax.wsdl.PortType;

import org.act.xservices.bpel20.TInvoke;
import org.act.xservices.engine.rt.VariableContainer;
import org.act.xservices.engine.rt.VariableDef;
import org.act.xservices.engine.rt.addressing.EndpointReference;
import org.act.xservices.engine.rt.deploy.DeploymentUnit;
import org.act.xservices.engine.rt.utils.DataUtil;
import org.act.xservices.engine.rt.utils.SoapUtil;
import org.act.xservices.rt.client.ServiceClient;
import org.act.xservices.rt.metaset.MetasetBuilder;
import org.act.xservices.rt.metaset.MetasetParserFactory;
import org.act.xservices.rt.xom.Metaset;
import org.act.xservices.rt.xom.QName;
import org.apache.axis.Message;
import org.apache.axis.MessageContext;
import org.apache.axis.client.Call;
import org.apache.axis.client.Service;
import org.apache.axis.message.SOAPEnvelope;

import commonj.sdo.DataObject;
import commonj.sdo.helper.DataFactory;

public class InvokeInstance extends Bpel20ActivityInstance {

	public InvokeInstance(TInvoke invoke, Bpel20InstanceUnit instanceUnit) {
		this.type = "Invoke";
		this.name = invoke.getName();
		this.activity = invoke;
		this.instanceUnit = instanceUnit;
	}

	// :
	// <invoke partnerLink="ncname" portType="qname" operation="ncname"
	// inputVariable="ncname"? outputVariable="ncname"? standard-attributes>
	// standard-elements
	// <correlations>?
	// <correlation set="ncname" initiate="yes|no"? pattern="in|out|out-in"/>+
	// </correlations>
	// <catch faultName="qname" faultVariable="ncname"?>* activity </catch>
	// <catchAll>?activity </catchAll>
	// <compensationHandler>? activity</compensationHandler>
	// </invoke>
	public void run() throws Exception {
		super.start();
		TInvoke invoke = (TInvoke) this.activity;
		Bpel20InstanceUnit instanceUnit = (Bpel20InstanceUnit) this.instanceUnit;

		String partnerLink = invoke.getPartnerLink();
		String operation = invoke.getOperation();
		String inputVariable = invoke.getInputVariable();
		String outputVariable = invoke.getOutputVariable();

		System.out.println("partnerLink: " + partnerLink + " operation: "
				+ operation + "  inputVariable: " + inputVariable
				+ "  outputVariable: " + outputVariable);

		EndpointReference endpoint = instanceUnit.getDeploymentUnit()
				.getPartnerLink(partnerLink).getPartnerEndpoint();

		String address = endpoint.getAddress();
		PortType portType = endpoint.getPortType();
		String opUri = portType.getQName().getNamespaceURI();

		if (outputVariable == null) {
			// 첽Invoke
			this.invokeAsyn(partnerLink, address, operation, opUri,
					inputVariable);
		} else {
			// ͬInvoke
			this.invokeSync(partnerLink, address, operation, opUri,
					inputVariable, outputVariable);
		}
		super.finish();
	}

	private void invokeAsyn(String partnerLink, String address, String op,
			String opUri, String inputVariable) {

		Metaset request = createRequest(partnerLink, op, opUri, inputVariable);
		ServiceClient client = new ServiceClient(address);
		client.sendReceiveNonBlocking(request);
	}

	private void invokeSync(String partnerLink, String address, String op,
			String opUri, String inputVariable, String outputVariable) {

		Metaset request = createRequest(partnerLink, op, opUri, inputVariable);
		// ⷢϢ
		try {
			ServiceClient client = new ServiceClient(address);
			// requestmessageBodyµĵһElement
			Metaset message = client.sendReceive(request);
			Bpel20InstanceUnit instanceUnit = (Bpel20InstanceUnit) this.instanceUnit;
			Metaset paras = message;

			DeploymentUnit deploymentUnit = instanceUnit.getDeploymentUnit();
			VariableDef variableDef = deploymentUnit
					.getVariableDef(outputVariable);
			QName messageType = variableDef.getMessageType();
			String messageTypeURI = messageType.getNamespaceURI();
			String messageTypeLocalPart = messageType.getName();
			DataObject var = DataFactory.INSTANCE.create(messageTypeURI,
					messageTypeLocalPart);
			DataUtil.metaset2data(paras, var);
			VariableContainer vc = instanceUnit.getVariableContainer();
			vc.setVariable(outputVariable, var);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	private Metaset createRequest(String partnerLink, String op, String opUri,
			String inputVariable) {
		Bpel20InstanceUnit instanceUnit = (Bpel20InstanceUnit) this.instanceUnit;
		Metaset paras = MetasetBuilder.getInstance()
				.newMetaset(opUri, op, "ns");
		DataUtil.data2metaset(instanceUnit, inputVariable, paras);

		return paras;
	}

	// private void sendRequest(String partnerLink, String address, String op,
	// String inputVariable) {
	// Bpel11InstanceUnit instanceUnit = (Bpel11InstanceUnit)this.instanceUnit;
	//
	// Metaset soapMsg = SoapUtil.createEmptySoap();
	// Metaset header = soapMsg.filter(".\\Header").car();
	// Metaset body = soapMsg.filter(".\\Body").car();
	// Metaset messageId = new Metaset("wsa:MessageID");
	//
	// messageId.setValue(instanceUnit.getId() + "-" + partnerLink + "-"+ op);
	// header.add(messageId);
	//
	// Metaset paras = new Metaset(op);
	// DataUtil.data2metaset(instanceUnit, inputVariable, paras);
	// body.add(paras);
	//
	// // ⷢϢ
	// try {
	// Service service=new Service();
	// Call call = (Call)service.createCall();
	// Message reqmsg=new Message(soapMsg.toXML(),true);
	// MessageContext context=call.getMessageContext();
	// call.setTargetEndpointAddress(address);
	// call.setOperationName(new QName(op));
	// context.setRequestMessage(reqmsg);
	// call.invoke();
	// } catch (Exception e) {
	// e.printStackTrace();
	// }
	// }
	//
	// private void getResponse(String partnerLink, String address, String op,
	// String outputVariable) {
	// Bpel11InstanceUnit instanceUnit = (Bpel11InstanceUnit)this.instanceUnit;
	//
	// IServiceContext message = instanceUnit.getMessage(partnerLink, op);
	// Metaset paras = (Metaset) message.getInMessage().filter(
	// ".\\Body\\*").car();
	// DeploymentUnit deploymentUnit = instanceUnit.getDeploymentUnit();
	// VariableDef variableDef = deploymentUnit.getVariableDef(outputVariable);
	// QName messageType = variableDef.getMessageType();
	// String messageTypeURI = messageType.getNamespaceURI();
	// String messageTypeLocalPart = messageType.getLocalPart();
	// DataObject var = DataFactory.INSTANCE.create(messageTypeURI,
	// messageTypeLocalPart);
	// DataUtil.metaset2data(paras, var);
	//
	// VariableContainer vc = instanceUnit.getVariableContainer();
	// vc.setVariable(outputVariable, var);
	// }
}
