package org.act.xservices.engine.rt.bpel20;

import java.util.ArrayList;

import org.act.xservices.bpel.wsdl.PropertyAlias;
import org.act.xservices.bpel20.TBoolean;
import org.act.xservices.bpel20.TCorrelation;
import org.act.xservices.bpel20.TCorrelationSet;
import org.act.xservices.bpel20.TCorrelations;
import org.act.xservices.bpel20.TReceive;
import org.act.xservices.engine.rt.CorrelatedInstanceUnit;
import org.act.xservices.engine.rt.CorrelationSetInstance;
import org.act.xservices.engine.rt.PartnerLink;
import org.act.xservices.engine.rt.PropertyInstance;
import org.act.xservices.engine.rt.VariableDef;
import org.act.xservices.engine.rt.utils.DataUtil;
import org.act.xservices.rt.metaset.utils.MetasetUtil;
import org.act.xservices.rt.soap.ISOAPEnvelope;
import org.act.xservices.rt.xcontainer.inf.context.IServiceContext;
import org.act.xservices.rt.xom.Metaset;
import org.act.xservices.rt.xom.QName;

import commonj.sdo.DataObject;
import commonj.sdo.Type;
import commonj.sdo.helper.DataFactory;
import commonj.sdo.helper.TypeHelper;

public class ReceiveInstance extends Bpel20ActivityInstance {

	public static String XMLNS_XSD = "http://www.w3.org/2001/XMLSchema";

	public ReceiveInstance(TReceive receive, Bpel20InstanceUnit instanceUnit) {
		this.type = "Receive";
		this.name = receive.getName();
		this.activity = receive;
		this.instanceUnit = instanceUnit;
	}

	// :
	// <receive partnerLink="ncname" portType="qname" operation="ncname"
	// variable="ncname"? createInstance="yes|no"? standard-attributes>
	// standard-elements
	// <correlations>?
	// <correlation set="ncname" initiate="yes|no"?>+
	// </correlations>
	// </receive>
	public void run() throws Exception {
		super.start();
		TReceive receive = (TReceive) activity;
		Bpel20InstanceUnit instanceUnit = (Bpel20InstanceUnit)this.instanceUnit;
		
		String partnerLink = receive.getPartnerLink();
		String operation = receive.getOperation();
		String varName = receive.getVariable();
		TCorrelations correlations = receive.getCorrelations();
		
		Bpel20DeploymentUnit deploymentUnit = (Bpel20DeploymentUnit)instanceUnit.getDeploymentUnit();

		System.out.println("partnerLink: " + partnerLink + " operation: " + operation + "  variable: " + varName);

		IServiceContext message = this.instanceUnit.getMessage(partnerLink, operation);
		PartnerLink pl = deploymentUnit.getPartnerLink(partnerLink);
		//if (pl.getPartnerEndpoint() == null) {
			this.instanceUnit.storeMessageContext(partnerLink, operation, message);
		//}

		//Metaset paras = message.getInMessage().filter(".\\Body\\" + operation).car();
		ISOAPEnvelope envelope = message.getInMessage();
		Metaset paras = envelope.getBody().element(operation);
		VariableDef variableDef = deploymentUnit.getVariableDef(varName);
		QName messageType = variableDef.getMessageType();
		String messageTypeURI = messageType.getNamespaceURI();
		String messageTypeLocalPart = messageType.getName();
		Type varType = TypeHelper.INSTANCE.getType(messageTypeURI, messageTypeLocalPart);
		DataObject var = DataFactory.INSTANCE.create(varType);
		DataUtil.metaset2data(paras, var);
		instanceUnit.getVariableContainer().setVariable(varName, var);

		if (correlations != null) {
			for (int i = 0; i < correlations.getCorrelation().size(); i++) {
				TCorrelation correlation = correlations.getCorrelation().get(i);
				if (correlation.getInitiate().equals(TBoolean.YES)) {
					String set = correlation.getSet();
					CorrelatedInstanceUnit correlatedInstanceUnit = new CorrelatedInstanceUnit();
					correlatedInstanceUnit.setInstanceId(this.instanceUnit.getId());
					CorrelationSetInstance correlationSetInstance = new CorrelationSetInstance();
					correlationSetInstance.setName(set);
					TCorrelationSet correlationSet = deploymentUnit.getCorrelationSet(set);
					for (int j = 0; j < correlationSet.getProperties().size(); j++) {
						QName property = MetasetUtil.jQNameToxQName(correlationSet.getProperties().get(j));
						PropertyInstance propertyInstance = new PropertyInstance();
						propertyInstance.setQname(property);
						ArrayList<PropertyAlias> propertyAliases = deploymentUnit.getPropertyAlias(messageType);
						for(int k=0; k<propertyAliases.size(); k++) {
							PropertyAlias propertyAlias = propertyAliases.get(k);
							String part = propertyAlias.getPart();
							String query = propertyAlias.getQuery();
							Object value = var.get(part+query);
							propertyInstance.setValue(value);
						}
						correlationSetInstance.addPropertyInstance(propertyInstance);	
					}
					correlatedInstanceUnit.setCorrelationSetInstance(correlationSetInstance);
					deploymentUnit.addCorrelatedInstanceUnit(correlatedInstanceUnit);
				}
			}
		}
		super.finish();
	}
}
