package org.act.xservices.engine.rt.deploy;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.act.xservices.log.Logger;
import org.act.xservices.log.LoggerFactory;
import org.act.xservices.rt.xcontainer.deployment.ADeployMonitor;
import org.act.xservices.rt.xcontainer.deployment.DeployManager;
import org.act.xservices.rt.xcontainer.deployment.DeploymentException;
import org.act.xservices.rt.xcontainer.deployment.WSInfo;
import org.act.xservices.rt.xcontainer.deployment.deployer.IDeployer;
import org.act.xservices.rt.xcontainer.deployment.util.DeployUtil;
import org.act.xservices.rt.xcontainer.deployment.util.DeploymentConstants;
import org.act.xservices.rt.xcontainer.deployment.util.FileUtil;
import org.act.xservices.rt.xcontainer.framework.IContainerEnvironment;
import org.act.xservices.rt.xcontainer.framework.config.IConfigElement;

public class BPELDeployMonitor extends ADeployMonitor{
	
	final Logger logger = LoggerFactory.getLogger(BPELDeployMonitor.class);
	
    /** This is to store all the jar files in a specified folder */
    private List<String> barList = new ArrayList<String>();
    
    /** All the currently updated jars */
    public Map<String, WSInfo> currentDeployedBars = new HashMap<String, WSInfo>();

    /** Stores all the bpel Services to deploy. */
    protected List<WSInfo> bpelWSToDeploy = new ArrayList<WSInfo>();

    /** Stores all the bpel Services to undeploy. */
    protected List<WSInfo> bpelWSToUnDeploy = new ArrayList<WSInfo>();
	
    
	//---
	protected File bpelservices_Deploy_Dir;
	protected File bpelservices_Working_Dir;
	
	
	
	
	public void init(DeployManager deployManager, IConfigElement deployMonitorConfig){
		
		super.init(deployManager, deployMonitorConfig);
		
		this.bpelservices_Deploy_Dir = (File) 
			containerEnv.getAttribute(DeploymentConstants.DIR_DEPLOY_BPELSERVICES);
		this.bpelservices_Working_Dir = (File) 
			containerEnv.getAttribute(DeploymentConstants.DIR_WORKING_BPELSERVICES);
	}
	
	
	/**
	 * BPELprocessorǷ񱻳ʼûгʼBPELProcessor޷BPELĲ
	 * true--ʼfalse--δʼ
	 * @return
	 */
	private boolean checkBPELProcessor(){
		return !(containerEnv.getAttribute(IContainerEnvironment.BPEL_PROCESSOR) == null);
	}

	//
	public void run() {
		
		//BPELProcessorδʼֹBPEL
		if( !checkBPELProcessor() ){
			logger.info("BPELProcessor may not be initialized, stop monitor BPEL Services!");
			return ;
		}
		
		//ɾWorkingµоļ
		deleteOldWorkingFile(this.bpelservices_Working_Dir);
		
		//ʱ
		while (!isStop) {
			
			logger.info("Start to deploy BPEL Service");
			
			String id = ""; //ڶĿ¼
			
			checkServices(this.bpelservices_Deploy_Dir, id);
			
			//ûпͨȲܣڼµķ
			if( !isHotDeployment )
				break;
			
			//һʱ
			try {
				Thread.sleep(deployerUpdateInterval);
			} catch (InterruptedException e) {
				logger.info("BPELDeployMonitor interrupt." );
			}
			
		}//end of while
	}//end of run
	
	
	
	
    
    /** Finds a list of services in the folder */
	public void checkServices(File file, String id) {
		synchronized (this) {
			// 鿴Ŀ¼µķʶҪºͲķ
			findServices(file, id);

			// ʶѱɾҪķ
			checkForUndeployedServices();
			
			this.unDeploy();
			this.doDeploy();
		}
	}
    
    /**
	 * Searches a given folder for wsar or zip files and 
	 * adds them to the list bpelWSToDeploy or bpelWSToUnDeploy.
	 */
    protected void findServices(File file, String id) {
        
        File[] subFiles = file.listFiles();
        
        for(File subFile : subFiles){
        	String name = subFile.getName();
			
			if (subFile.isDirectory()) {
				//ﴦļǰĿ¼ʽ֯ļǶļУ
				if (id.equals("")) {
					findServices(subFile, name);
				} else {
					findServices(subFile, id + "#" + name);
				}
				
			} else {
				//ǷҪ
				String ext = DeployUtil.getFileExtension(name);

            	if("bar".equalsIgnoreCase(ext)){ //Bar͵ļ
            		
            		// If we found a deployer for this type of file, use it. Otherwise ignore the file.
            		IDeployer deployer = deployManager.getDeployerForExtension(ext);
            		if (deployer != null) {
            			
            			String processId = null;
	            		String str = DeployUtil.getFileNameWithoutExtension(name);
					    if (id.equals("")) {
					    	processId = str;
					    } else {
					    	processId = id + "#" + str;
					    }
	            		addFileToDeploy(subFile, processId, deployer, WSInfo.TYPE_BPELSERVICES);
            		
            		} else{
            			logger.info("Can not find Deployer for file type: "+ ext);
            			logger.info("Give up deploy file: "+ subFile.getName());
            		}
            		
            	}else{ // ΪBARʽ½ӿ
			    }
			}
        }//---end of for---
    }
	
    
    /**
     * First checks whether the file is already available by the
     * system call fileExists. If it is not deployed yet then adds to the jarList
     * and to the deployment engine as a new service or module.
     * While adding new item to jarList, first creates the WSInfo object and
     * then adds to the jarlist and actual jar file is added to DeploymentEngine.
     * <p/>
     * If the files already exists, then checks whether it has been updated
     * then changes the last update date of the wsInfo and adds two entries to
     * DeploymentEngine - one for new deployment and other for undeployment.
     *
     * @param file actual jar files for either Module or service
     */
    public synchronized void addFileToDeploy(File file, String fileID, IDeployer deployer , int type) {
        
    	WSInfo info = currentDeployedBars.get(fileID);
    	
        if (info != null) {
        	//֮ǰ
            if (isHotUpdate && isModified(file, info)) {
            	//ȸ¹,add entry to undeploy list
            	this.bpelWSToUnDeploy.add(info);
                
                WSInfo wsInfo = new WSInfo(file, fileID, file.lastModified(), deployer, type);
                
                // add entry to deploylist 
                this.bpelWSToDeploy.add(wsInfo);
                
                currentDeployedBars.remove(info.getFileID());
                currentDeployedBars.put(wsInfo.getFileID(), wsInfo);
            }
        } else {
        	//˷֮ǰ
            info = new WSInfo(file, fileID, file.lastModified(), deployer ,type);
            currentDeployedBars.put(fileID, info);
            this.bpelWSToDeploy.add(info);
        }
        
        //Ŀ¼еķƶӵб
        barList.add(info.getFileID());
    }
    
    
    /**
     * Checks undeployed Services. Checks old jars files and current jars.
     * If name of the old jar file does not exist in the current jar
     * list then it is assumed that the jar file has been removed
     * and that is hot undeployment.
     */
    private synchronized void checkForUndeployedServices() {

        Iterator infoItems = currentDeployedBars.keySet().iterator();
        
        List<String> tobeRemoved = new ArrayList<String>();
        
        while (infoItems.hasNext()) {
            String  fileID = (String) infoItems.next();
            WSInfo infoItem = currentDeployedBars.get(fileID);

            //seems like someone has deleted the file , so need to undeploy
            boolean found = false;
            //ҴļǷɾ
            for (int i = 0; i < barList.size(); i++) {
                String s = barList.get(i);
                if(fileID.equals(s)){
                    found = true;
                    break;
                }
            }
            
            if(!found){ //˷ļɾҪз
                tobeRemoved.add(fileID);
                this.bpelWSToUnDeploy.add(infoItem);
            }
        }

        for (int i = 0; i < tobeRemoved.size(); i++) {
            String fileID = tobeRemoved.get(i);
            currentDeployedBars.remove(fileID);
        }
        
        tobeRemoved.clear();
        
        barList.clear(); //importent
    }
    
    
    
    /**
     * беļ丽Deployerв
     */
    public void doDeploy() {
		try {
			if (bpelWSToDeploy.size() > 0) {
				
				for (WSInfo fileToDeploy : bpelWSToDeploy) {
					
					long deployStart = System.currentTimeMillis();
					
					try {
						String workingDir = this.bpelservices_Working_Dir.getAbsolutePath();
						String fileName = fileToDeploy.getFile().getName();
						String fileID = fileToDeploy.getFileID();
						
						if ( !fileID.equals(DeployUtil.getFileNameWithoutExtension(fileName)) ){
							//ļĳһĿ¼
							String[] subIds = fileID.split("#");
							for (int j = 0; j < subIds.length-1; j++) { //ע˴fileIDһγȥҪ
								String subId = subIds[j];
								workingDir = workingDir + File.separator + subId;
							}
						}
						
						//BARѹworkingĿ¼
						workingDir = workingDir + File.separator + fileName;
						FileUtil.inflate(fileToDeploy.getFile(), workingDir);
						
						//ô˷WorkingĿ¼µ·
						fileToDeploy.setServiceWorkingLocation(workingDir);
						
						IDeployer deployer = fileToDeploy.getDeployer();
						if (deployer != null) {
							deployer.deploy(fileToDeploy);
							//ɹ
							long deployEnd = System.currentTimeMillis();
							logger.info("Succeed Deployed BPEL Service " 
									+ fileToDeploy.getFileID()
									+" in " + (deployEnd - deployStart) + " milliseconds");
						}
						
					} catch (IOException e) {
						//ѹгִ
						logger.info("Error to Unzip bpel File "
										+ fileToDeploy.getFileID() + " for "
										+ e.toString());
						continue; //ļ

					} catch (DeploymentException e) {
						//гִ
						logger.info("Error to Deploy bpel File "
										+ fileToDeploy.getFileID() + " for "
										+ e.getMessage());
						continue; //ļ
					}

				}//end of for
			}
		} finally {
			bpelWSToDeploy.clear();
		}
	}

	/**
	 * беļ丽Deployerз
	 */
	public void unDeploy() {
		try {
			for (WSInfo wsInfo : bpelWSToUnDeploy) {
				try {
					String workingDir = this.bpelservices_Working_Dir.getAbsolutePath();
					String fileName = wsInfo.getFile().getName();
					String fileID = wsInfo.getFileID();
					
					if ( !fileID.equals(DeployUtil.getFileNameWithoutExtension(fileName)) ){
						//ļĳһĿ¼
						String[] subIds = fileID.split("#");
						for (int j = 0; j < subIds.length-1; j++) { //ע˴fileIDһγȥҪ
							String subId = subIds[j];
							workingDir = workingDir + File.separator + subId;
						}
					}
					
					workingDir = workingDir + File.separator + fileName;
					
					File oldWsarWorkFile = new File(workingDir);
					
					if (oldWsarWorkFile.exists()) {
						// ɾWorkingĿ¼µǰĲļ
						FileUtil.deleteRecursive(oldWsarWorkFile);
					}

					IDeployer deployer = wsInfo.getDeployer();
					if (deployer != null) {
						deployer.unDeploy(wsInfo.getFileID());
						logger.info("UnDeployed BPEL Service " + wsInfo.getFileID());
					}
				} catch (Exception e) {
					logger.info("Error to UnDeployed BPEL Service "
							+ wsInfo.getFileID() + " for " + e.toString());
					continue;
				}
			}//end of for
		} finally {
			bpelWSToUnDeploy.clear();
		}
	}
    
}
