package org.act.xservices.engine.rt.deploy;

import java.io.File;
import java.io.FileFilter;
import java.io.FileInputStream;
import java.io.StringWriter;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import javax.wsdl.Types;
import javax.wsdl.extensions.UnknownExtensibilityElement;
import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.act.xservices.bpel.wsdl.DefinitionsExtended;
import org.act.xservices.bpel.wsdl.Property;
import org.act.xservices.bpel.wsdl.PropertyAlias;
import org.act.xservices.bpel.wsdl.impl.DefinitionsExtendedImpl;
import org.act.xservices.engine.rt.CorrelatedOperation;
import org.act.xservices.engine.rt.InstanceUnit;
import org.act.xservices.engine.rt.OperationEntry;
import org.act.xservices.engine.rt.PartnerLink;
import org.act.xservices.engine.rt.adapter.BPELAdapter;
import org.act.xservices.engine.rt.bpel11.Bpel11BarDeploy;
import org.act.xservices.engine.rt.bpel11.Bpel11DeploymentUnit;
import org.act.xservices.engine.rt.bpel20.Bpel20BarDeploy;
import org.act.xservices.engine.rt.bpel20.Bpel20DeploymentUnit;
import org.act.xservices.engine.rt.locale.StringManager;
import org.act.xservices.engine.rt.locale.StringManagerFactory;
import org.act.xservices.engine.rt.utils.ConstantUtil;
import org.act.xservices.engine.rt.utils.TimeUtil;
import org.act.xservices.log.Logger;
import org.act.xservices.log.LoggerFactory;
import org.act.xservices.rt.metaset.MetasetParserFactory;
import org.act.xservices.rt.metaset.exception.MetasetParsingException;
import org.act.xservices.rt.xcontainer.deployment.DeploymentException;
import org.act.xservices.rt.xcontainer.deployment.WSInfo;
import org.act.xservices.rt.xcontainer.deployment.deployer.IDeployer;
import org.act.xservices.rt.xcontainer.framework.IContainerEnvironment;
import org.act.xservices.rt.xcontainer.framework.policy.IPolicyConfiger;
import org.act.xservices.rt.xcontainer.framework.policy.PolicyConfigException;
import org.act.xservices.rt.xcontainer.inf.service.wsdl.WSDL11Constants;
import org.act.xservices.rt.xcontainer.inf.service.wsdl.WSDL20Constants;
import org.act.xservices.rt.xcontainer.inf.service.wsdl.XcService;
import org.act.xservices.rt.xcontainer.inf.service.wsdl.XcServiceRegistry;
import org.act.xservices.rt.xcontainer.parser.wsdl.IWSDLParser;
import org.act.xservices.rt.xcontainer.parser.wsdl.WSDLParserFactory;
import org.act.xservices.rt.xcontainer.wsdl.WSDLParseException;
import org.act.xservices.rt.xom.Metaset;
import org.apache.tuscany.sdo.util.DataObjectUtil;

import commonj.sdo.helper.XSDHelper;

public class BPELDeployer implements IDeployer {

	final Logger logger = LoggerFactory.getLogger(BPELDeployer.class);

	static StringManager sm = StringManagerFactory.getStringManager();

	// private MetasetBuilder mb = MetasetBuilder.getInstance();

	private XcServiceRegistry xcServiceRegistry;

	private String extension;

	// private IContainerEnvironment containerEnv;

	private BPELAdapter bpelProcessor;

	private IPolicyConfiger[] policyConfigers;

	/**
	 * ʼ
	 * 
	 * @param extension
	 * @param containerEnv
	 */
	public void init(String extension, IContainerEnvironment containerEnv,
			IPolicyConfiger[] policyConfigers) {
		this.extension = extension;
		// this.containerEnv = containerEnv;
		this.xcServiceRegistry = containerEnv.getXcServiceRegistry();
		this.bpelProcessor = (BPELAdapter) containerEnv
				.getAttribute(IContainerEnvironment.BPEL_PROCESSOR);

		DataObjectUtil.initRuntime(); // deployǰ--ڸأһ--sundou

		this.policyConfigers = policyConfigers;
	}

	public String getExtension() {
		return extension;
	}

	/**
	 * 
	 * 
	 * @param fileId
	 * @throws DeploymentException
	 */
	public synchronized void unDeploy(String fileId) throws DeploymentException {

		// BPELע
		bpelProcessor.getDeploymentUnitRegistry().delete(fileId);

		// WSServiceעxcServiceRegistry
		// ˴ٶƼΪbpel
		String serviceName = getServiceName(fileId);
		Set<String> serviceURIs = xcServiceRegistry
				.getServiceURLByServiceName(serviceName);
		if (serviceURIs != null) {
			xcServiceRegistry.removeXcServiceByURL(serviceURIs.iterator()
					.next());
		}
	}

	/**
	 * Retrieves service name from the archive file name. If the archive file
	 * name is service1.bar , then bpelservice name would be service1
	 * 
	 * @param fileId
	 * @return
	 */
	private String getServiceName(String fileId) {
		String[] subIds = fileId.split("#");
		return subIds[subIds.length - 1];
	}

	/**
	 * WSARļ
	 * 
	 * @param wsFile
	 * @throws DeploymentException
	 */
	public void deploy(WSInfo wsFile) throws DeploymentException {
		try {
			File file = wsFile.getFile(); // 

			final String bpelFileWorkingDir = wsFile
					.getServiceWorkingLocation()
					+ File.separator + "META-INF" + File.separator;

			File dir = new File(bpelFileWorkingDir);
			File[] bpelFiles = dir.listFiles(new FileFilter() {
				public boolean accept(File file) {
					String fn = file.getName();
					int len = fn.length();
					if (len > 5) {
						String ext = fn.substring(len - 5, len);
						return ext.equalsIgnoreCase(".bpel");
					}
					return false;
				}
			});
			if (bpelFiles.length == 0) {
				throw new Exception(file.getName()
						+ " does't contain the BPEL file (.bpel)");
			}
			File bpelFile = bpelFiles[0];

			// ʹMetasetֻΪȡxmlnsж̵İ汾1.12.0(name)ƿռ(targetNamespace)
			// ӦиõĽ취
			// /System.out.println(bpelFile.toString());
			Metaset bpel = MetasetParserFactory.getBpelMetasetParser().parse(
					new FileInputStream(bpelFile));
			final String xmlns = bpel.getQName().getNamespaceURI();
			final String bpelServiceName = bpel.attributeValue("name");
			final String tns = bpel.attributeValue("targetNamespace");

			DeploymentUnit deploymentUnit = null;

			if (xmlns.equals(ConstantUtil.XMLNS_BPEL11)) {
				// BPEL4WS 1.1
				deploymentUnit = new Bpel11DeploymentUnit();

				// ̰汾
				deploymentUnit.setVersion("1.1");

			} else if (xmlns.equals(ConstantUtil.XMLNS_BPEL20)) {
				// ws-BPEL 2.0
				deploymentUnit = new Bpel20DeploymentUnit();

				// ̰汾
				deploymentUnit.setVersion("2.0");

			} else {
				// ݲֳ֧1.12.0֮BPEL汾ɹԺ
				throw new Exception("Can't support this version of BPEL :"
						+ xmlns + "(" + bpelServiceName + ")");
			}

			// ID
			deploymentUnit.setId(wsFile.getFileID());

			// 
			deploymentUnit.setName(bpelServiceName);

			// ƿռ
			deploymentUnit.setTargetNamespace(tns);

			// ̲ʱ
			String deployTime = TimeUtil.getNowTime();
			deploymentUnit.setDeployTime(deployTime);

			// ̲ļ
			deploymentUnit.setFile(file);

			// ̹Ŀ¼
			deploymentUnit.setDirectory(bpelFileWorkingDir);

			// ̲еWSDLǣǺ
			File[] wsdlFiles = dir.listFiles(new FileFilter() {
				public boolean accept(File file) {
					String fn = file.getName();
					int len = fn.length();
					if (len > 5) {
						String ext = fn.substring(len - 5, len);
						return ext.equalsIgnoreCase(".wsdl");
					}
					return false;
				}
			});
			if (wsdlFiles.length == 0) {
				throw new Exception(file.getName()
						+ " does't contain the WSDL files (.wsdl)");
			}
			for (int i = 0; i < wsdlFiles.length; i++) {
				String wsdlFile = wsdlFiles[i].toString();
				DefinitionsExtended wsdl = new DefinitionsExtendedImpl(wsdlFile);


				// ӵԪWSDLб
				deploymentUnit.addWsdl(wsdl);
				
				//ӵԪWSDLFileб 
				String wsdlUri = wsdl.getDefinition().getTargetNamespace();
				deploymentUnit.addWsdlFile(wsdlUri, wsdlFiles[i]);

				// 帴
				defineComplexTypeInWSDL(wsdl);

				// ӵԪ(Property)б
				ArrayList<Property> properties = wsdl.getPropertys();
				for (int j = 0; j < properties.size(); j++) {
					Property property = properties.get(j);
					deploymentUnit.addProperty(property);
				}

				// ӵԪԱ(PropertyAlias)б
				ArrayList<PropertyAlias> propertyAliases = wsdl
						.getPropertyAliases();
				for (int j = 0; j < propertyAliases.size(); j++) {
					PropertyAlias propertyAlias = propertyAliases.get(j);
					deploymentUnit.addPropertyAlias(propertyAlias);
				}
			}

			// ̲иͶXSD
			File[] xsdFiles = dir.listFiles(new FileFilter() {
				public boolean accept(File file) {
					String fn = file.getName();
					int len = fn.length();
					if (len > 4) {
						String ext = fn.substring(len - 4, len);
						return ext.equalsIgnoreCase(".xsd");
					}
					return false;
				}
			});
			for (int i = 0; i < xsdFiles.length; i++) {
				File xsdFile = xsdFiles[i];
				defineComplexTypeInXSD(xsdFile);
			}

			// ʵб
			setEntryList(bpel, deploymentUnit);

			// ΪһЩģͽϵĲʱķ汾һֲԺʱ޸
			if (deploymentUnit instanceof Bpel11DeploymentUnit) {
				Bpel11BarDeploy.deploy((Bpel11DeploymentUnit) deploymentUnit,
						bpelFile);
			} else if (deploymentUnit instanceof Bpel20DeploymentUnit) {
				Bpel20BarDeploy.deploy((Bpel20DeploymentUnit) deploymentUnit,
						bpelFile);
			}

			// ӻʵʰ󶨵ַ̲ļPDDȡã
			File[] pddFiles = dir.listFiles(new FileFilter() {
				public boolean accept(File file) {
					String fn = file.getName();
					int len = fn.length();
					if (len > 5) {
						String ext = fn.substring(len - 4, len);
						return ext.equalsIgnoreCase(".pdd");
					}
					return false;
				}
			});
			if (pddFiles.length != 0) {
				Metaset pdd = MetasetParserFactory.getBpelMetasetParser()
						.parse(pddFiles[0].toString());
				/*
				 * //Metaset partnerLinks = pdd
				 * .filter(".\\partnerLinks\\partnerLink");
				 */
				List<Metaset> partnerLinks = pdd.element("partnerLinks")
						.elements("partnerLink");
				for (int i = 0; i < partnerLinks.size(); i++) {
					Metaset partnerLink = partnerLinks.get(i);
					String plName = partnerLink.attributeValue("name");
					/*
					 * Metaset address = partnerLink.filter(
					 * ".\\EndpointReference\\Address").car();
					 */
					Metaset address = partnerLink.element("EndpointReference")
							.element("Address");
					PartnerLink pl = deploymentUnit.getPartnerLink(plName);
					pl.getPartnerEndpoint().setAddress(address.getText());
				}
			}

			deploymentUnit.activize();
			DeploymentUnitRegistry deploymentUnitRegistry = bpelProcessor
					.getDeploymentUnitRegistry();
			deploymentUnitRegistry.add(deploymentUnit);

			// ---XcServiceRegistryעBPELWSDLĵ---
			// ̵WSDL(.wsdl)עᵽײWSDLעУⲿֿѯʣ
			String wsdlLocation = bpelFileWorkingDir + bpelServiceName
					+ ".wsdl";
			deployWSDL(wsdlLocation);

			// log.info(sm.getString("org.act.xservices.engine.rt.deploy.process",deploymentUnit.getId()));

			// ---liumin---ģʽ---עҪڲ---
			deploySingleton(wsFile.getFileID());

		} catch (DeploymentException e) {
			// XcServiceRegistryעBPELWSDLĵʧ,DeploymentUnitRegistry
			bpelProcessor.getDeploymentUnitRegistry()
					.delete(wsFile.getFileID());
			logger.info("Error to deploy BPEL WSDL to XcServiceRegistry: "
					+ wsFile.getFileID());
			throw new DeploymentException(e);

		} catch (Exception e) {
			throw new DeploymentException(e);
		}
	}

	/**
	 * Ե̽⴦
	 * 
	 * @param processId
	 */
	private void deploySingleton(String processId) {

		if (bpelProcessor.isSingleton(processId)) {
			DeploymentUnitRegistry deploymentUnitRegistry = bpelProcessor
					.getDeploymentUnitRegistry();
			// Եģʽ̲⴦ڲʱһʵԺжԸ̵󶼷͵ʵ
			DeploymentUnit deploymentUnit = deploymentUnitRegistry
					.get(processId);
			final InstanceUnit instance = deploymentUnit.createInstance();
			bpelProcessor.setSingleton(processId, instance.getId());
			Runnable runnable = new Runnable() {
				public void run() {
					logger.info("org.act.xservices.engine.rt.beginInstance "
							+ instance.getId());
					instance.run();
					logger.info("org.act.xservices.engine.rt.endInstance "
							+ instance.getId());
				}
			};
			Thread thread = new Thread(runnable);
			thread.setName(instance.getId());
			thread.start();
		}
	}

	/**
	 * ɴʵںؼ
	 * 
	 * @param bpel
	 * @param deploymentUnit
	 */
	private void setEntryList(Metaset bpel, DeploymentUnit deploymentUnit) {
		// Metaset receives = bpel.filter(".\\receive");
		// ZYJ ֱֻܾԪ
		List<Metaset> receives = null;
		receives = bpel.elements("receive");
		if (receives != null) {
			for (int j = 0; j < receives.size(); j++) {
				Metaset receive = (Metaset) receives.get(j);
				String partnerLink = receive.attributeValue("partnerLink");
				String operation = receive.attributeValue("operation");
				if (receive.attributeValue("createInstance") != null
						&& receive.attributeValue("createInstance").equals(
								"yes")) {
					deploymentUnit.addInstanceEntry(partnerLink, operation);
				} else if (receive.element("correlations") != null) {
					List<Metaset> correlations = receive
							.element("correlations").elements("correlation");
					CorrelatedOperation correlationEntry = new CorrelatedOperation();
					OperationEntry operationEntry = new OperationEntry();
					operationEntry.setPartnerLink(partnerLink);
					operationEntry.setOperation(operation);
					correlationEntry.setOperationEntry(operationEntry);
					for (int k = 0; k < correlations.size(); k++) {
						Metaset correlation = (Metaset) correlations.get(k);
						String set = correlation.attributeValue("set");
						String initiate = correlation
								.attributeValue("initiate");
						if (initiate == null || initiate.equals("no")) {
							correlationEntry.setSet(set);
						}
					}
					deploymentUnit.addCorrelationEntry(correlationEntry);
				}
			}
		}

		// Metaset picks = bpel.filter(".\\pick");
		List<Metaset> picks = bpel.elements("pick");
		if (picks != null) {
			for (int j = 0; j < picks.size(); j++) {
				Metaset pick = (Metaset) picks.get(j);
				// Metaset onMessages = pick.filter(".\\onMessage");
				List<Metaset> onMessages = pick.elements("onMessage");
				for (int k = 0; k < onMessages.size(); k++) {
					Metaset onMessage = (Metaset) onMessages.get(k);
					String partnerLink = onMessage
							.attributeValue("partnerLink");
					String operation = onMessage.attributeValue("operation");
					if (pick.attributeValue("createInstance").equals("yes")) {
						deploymentUnit.addInstanceEntry(partnerLink, operation);
					} else if (onMessage.element("correlations") != null) {
						/*
						 * Metaset correlations = onMessage
						 * .filter(".\\correlations\\correlation");
						 */
						List<Metaset> correlations = onMessage.element(
								"correlations").elements("correlation");
						CorrelatedOperation correlationEntry = new CorrelatedOperation();
						OperationEntry operationEntry = new OperationEntry();
						operationEntry.setPartnerLink(partnerLink);
						operationEntry.setOperation(operation);
						correlationEntry.setOperationEntry(operationEntry);
						for (int m = 0; m < correlations.size(); m++) {
							Metaset correlation = (Metaset) correlations.get(m);
							String set = correlation.attributeValue("set");
							String initiate = correlation
									.attributeValue("initiate");
							if (initiate == null || initiate.equals("no")) {
								correlationEntry.setSet(set);
							}
						}
						deploymentUnit.addCorrelationEntry(correlationEntry);
					}
				}
			}
		}

		// Metaset onMessages = bpel.filter(".\\eventHandlers\\onMessage");
		Metaset eventHandlers = bpel.element("eventHandlers");
		if (eventHandlers != null) {
			List<Metaset> onMessages = eventHandlers.elements("onMessage");
			if (onMessages != null) {
				for (int i = 0; i < onMessages.size(); i++) {
					Metaset onMessage = (Metaset) onMessages.get(i);
					String partnerLink = onMessage
							.attributeValue("partnerLink");
					String operation = onMessage.attributeValue("operation");
					if (onMessage.element("correlations") != null) {
						List<Metaset> correlations = onMessage.element(
								"correlations").elements("correlation");
						CorrelatedOperation correlationEntry = new CorrelatedOperation();
						OperationEntry operationEntry = new OperationEntry();
						operationEntry.setPartnerLink(partnerLink);
						operationEntry.setOperation(operation);
						correlationEntry.setOperationEntry(operationEntry);
						for (int m = 0; m < correlations.size(); m++) {
							Metaset correlation = (Metaset) correlations.get(m);
							String set = correlation.attributeValue("set");
							String initiate = correlation
									.attributeValue("initiate");
							if (initiate == null || initiate.equals("no")) {
								correlationEntry.setSet(set);
							}
						}
						deploymentUnit.addCorrelationEntry(correlationEntry);
					}
				}
			}
		}

		for (int i = 0; i < bpel.elements().size(); i++) {
			setEntryList(bpel.elements().get(i), deploymentUnit);
		}
	}

	/**
	 * WSDLSchema帴
	 * 
	 * @param wsdl
	 * @throws Exception
	 */
	@SuppressWarnings("unchecked")
	private void defineComplexTypeInWSDL(DefinitionsExtended wsdl)
			throws Exception {
		Transformer serializer = TransformerFactory.newInstance()
				.newTransformer();
		Types types = wsdl.getDefinition().getTypes();
		if (types != null) {
			List<UnknownExtensibilityElement> schemas = types
					.getExtensibilityElements();
			for (int k = 0; k < schemas.size(); k++) {
				UnknownExtensibilityElement schema = schemas.get(k);
				StringWriter schemaString = new StringWriter();
				Source source = new DOMSource(schema.getElement());
				Result result = new StreamResult(schemaString);
				serializer.transform(source, result);
				XSDHelper.INSTANCE.define(schemaString.toString());
			}
		}
	}

	/**
	 * XSD帴
	 * 
	 * @param xsdFile
	 * @throws Exception
	 */
	private void defineComplexTypeInXSD(File xsdFile) throws Exception {
		FileInputStream fis = new FileInputStream(xsdFile);
		XSDHelper.INSTANCE.define(fis, null);
		fis.close();
	}

	private boolean deployWSDL(String wsdlLocation) throws DeploymentException {

		File wsdlFile = new File(wsdlLocation);
		if (!wsdlFile.exists()) {
			throw new DeploymentException("Error: Can not find wsdl file at "
					+ wsdlFile.getAbsolutePath());
		}

		// WSDLFile,XcServiceʵ
		XcService xcService = processWSDLFile(wsdlFile);
		xcService.setWsdlFile(wsdlFile);
		xcService.setServiceType(XcService.SERVICE_TYPE_BPEL); // ÷ΪBPELService

		// ʹPolicyProcessorWSDLеĲϢ
		XcService xcServiceAfterPolicy = processPolicyInfo(xcService);

		// XcServiceӵע
		boolean isRegistrySucceed = xcServiceRegistry
				.addXcService(xcServiceAfterPolicy);

		if (!isRegistrySucceed) {
			throw new DeploymentException(
					"Error: False to Registry Service "
							+ xcServiceAfterPolicy.getQName()
							+ " , for the Service QName or ServiceURL already been Registryed!");
		}

		return isRegistrySucceed;
	}

	/**
	 * ʹPolicyProcessorWSDLеĲϢ εеPolicyProcesorд
	 * 
	 * @param xcService
	 * @return
	 * @throws DeploymentException
	 */
	private XcService processPolicyInfo(XcService xcService)
			throws DeploymentException {

		if (policyConfigers != null) { // вԴ
			try {
				for (IPolicyConfiger pc : policyConfigers) {
					pc.configPolicyProcessor(xcService);
				}
			} catch (PolicyConfigException e) {
				throw new DeploymentException(
						"Error in process Policy Information", e);
			}
		}

		return xcService;
	}

	/**
	 * WSDLĵɵXcServiceserviceMap
	 * 
	 * @param wsdlFile
	 * @param servicesMap
	 */
	private XcService processWSDLFile(File wsdlFile) throws DeploymentException {

		Metaset wsdlMetaset = null;
		try {
			wsdlMetaset = MetasetParserFactory.getBpelMetasetParser().parse(
					wsdlFile.getAbsolutePath());
		} catch (MetasetParsingException e) {
			throw new DeploymentException("Error: failed to parse WSDL file : "
					+ wsdlFile.getAbsolutePath());
		}

		// ȷWSDLƿռ
		String wsdlNamespace = wsdlMetaset.getNamespaceURI();

		if (wsdlNamespace != null && !"".equals(wsdlNamespace)) {

			// wsdlNamespaceöӦWSDLParser
			IWSDLParser wsdlparser;

			if (WSDL20Constants.WSDL20_NAMESPACE.equals(wsdlNamespace)) {

				// we have a WSDL 2.0 document here.
				wsdlparser = WSDLParserFactory.CreatWSDLParser(
						WSDLParserFactory.WSDL20PARSER_OF_WODEN, wsdlFile);

			} else if (WSDL11Constants.NS_URI_WSDL11.equals(wsdlNamespace)) {

				// we have a WSDL 1.1 document here
				wsdlparser = WSDLParserFactory.CreatWSDLParser(
						WSDLParserFactory.WSDL11PARSER_OF_WSDL4J, wsdlFile);

			} else {
				throw new DeploymentException("Error: invalidWSDLFound : "
						+ wsdlFile.getAbsolutePath());
			}

			XcService xcService;
			try {
				xcService = wsdlparser.parseWSDL();
			} catch (WSDLParseException e) {
				throw new DeploymentException(e);
			}

			return xcService;

		} else {
			throw new DeploymentException(
					"Error: can not find wsdlNamespace of file "
							+ wsdlFile.getAbsolutePath());
		}

	}

}
