package org.act.xservices.engine.rt.deploy;

import java.io.File;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Random;

import javax.wsdl.WSDLException;

import org.act.xservices.bpel.wsdl.DefinitionsExtended;
import org.act.xservices.bpel.wsdl.Property;
import org.act.xservices.bpel.wsdl.PropertyAlias;
import org.act.xservices.bpel.wsdl.impl.DefinitionsExtendedImpl;
import org.act.xservices.engine.rt.CorrelatedInstanceUnit;
import org.act.xservices.engine.rt.CorrelatedOperation;
import org.act.xservices.engine.rt.InstanceUnit;
import org.act.xservices.engine.rt.InstanceUnitPool;
import org.act.xservices.engine.rt.OperationEntry;
import org.act.xservices.engine.rt.PartnerLink;
import org.act.xservices.engine.rt.VariableDef;
import org.act.xservices.rt.metaset.utils.MetasetUtil;
import org.act.xservices.rt.xcontainer.inf.context.IServiceContext;
import org.act.xservices.rt.xom.QName;

/**
 * ̲Ԫ
 * @author sun
 *
 */
public class DeploymentUnit {
	
	public static final int STATE_ACTIVE = 0;
	
	public static final int STATE_SUSPENDED = 1;
	
	/**
	 * ID
	 */
	private String id;
	
	/**
	 * 
	 */
	private String name;
	
	/**
	 * ̰汾1.12.0
	 */
	private String version;
	
	/**
	 * ̲ʱ
	 */
	private String deployTime;
	
	/**
	 * ƿռ
	 */
	private String targetNamespace;

	/**
	 * ״̬
	 */
	private int state = STATE_SUSPENDED;
	
	/**
	 * ̲ļ
	 */
	private File file;
	
	/**
	 * ̲Ŀ¼
	 */
	private String directory;
	
	/**
	 * Ӷ
	 */
	private HashMap<String, PartnerLink> partnerLinks = new HashMap<String, PartnerLink>();
	
	/**
	 * 
	 */
	private HashMap<String, VariableDef> variableDefs = new HashMap<String, VariableDef>();
	
	/**
	 * ʵ
	 */
	private ArrayList<OperationEntry> createInstanceEntries = new ArrayList<OperationEntry>();
	
	/**
	 * ؼ
	 */
	private ArrayList<CorrelatedOperation> correlationEntries = new ArrayList<CorrelatedOperation>();
	
	/**
	 * WSDLĿ¼
	 */
	private HashMap<String, DefinitionsExtended> wsdlCatalog = new HashMap<String, DefinitionsExtended>();
	
	/**
	 * WSDLļ
	 */
	private HashMap<String, File> wsdlFiles = new HashMap<String, File>();

	/**
	 * Ϣ
	 */
	private HashMap<QName, Property> properties = new HashMap<QName, Property>();

	/**
	 * ϢԱ
	 */
	private ArrayList<PropertyAlias> propertyAliases = new ArrayList<PropertyAlias>();
	
	/**
	 * ؼʵ
	 */
	private ArrayList<CorrelatedInstanceUnit> correlatedInstanceUnits = new ArrayList<CorrelatedInstanceUnit>();
	
	/**
	 * ʵ
	 */
	private InstanceUnitPool instanceUnitPool = new InstanceUnitPool();

	public String getId() {
		return id;
	}

	public void setId(String id) {
		this.id = id;
	}
	
	public String getName() {
		return name;
	}
	
	public void setName(String name) {
		this.name = name;
	}
	
	public String getTargetNamespace() {
		return targetNamespace;
	}

	public void setTargetNamespace(String targetNamespace) {
		this.targetNamespace = targetNamespace;
	}
	
	public int getState() {
		return state;
	}
	
	public void activize() {
		this.state = STATE_ACTIVE;
	}
	
	public void suspended() {
		this.state = STATE_SUSPENDED;
	}
	
	public File getFile() {
		return file;
	}

	public void setFile(File file) {
		this.file = file;
	}
	
	public void addPartnerLink(PartnerLink partnerLink) {
		partnerLinks.put(partnerLink.getName(), partnerLink);
	}
	
	public void removePartnerLink(String name) {
		partnerLinks.remove(name);
	}
	
	public PartnerLink[] getPartnerLinks() {
		return partnerLinks.values().toArray(new PartnerLink[0]);
	}
	
	public PartnerLink getPartnerLink(String name) {
		return partnerLinks.get(name);
	}
	
	public void addVariableDef(VariableDef variableDef) {
		variableDefs.put(variableDef.getName(), variableDef);
	}
	
	public VariableDef getVariableDef(String name) {
		return variableDefs.get(name);
	}
	
	public boolean isInstanceEntry(String partnerLink, String operation) {
		for(int i=0; i<createInstanceEntries.size(); i++) {
			OperationEntry createInstanceEntry = createInstanceEntries.get(i);
			if(createInstanceEntry.getPartnerLink().equals(partnerLink) && 
					createInstanceEntry.getOperation().equals(operation)) {
				return true;
			}
		}
		return false;
	}
	
	public void addInstanceEntry(String partnerLink, String operation) {
		OperationEntry createInstanceEntry = new OperationEntry();
		createInstanceEntry.setPartnerLink(partnerLink);
		createInstanceEntry.setOperation(operation);
		createInstanceEntries.add(createInstanceEntry);
	}
	
	public boolean isCorrelationEntry(String partnerLink, String operation) {
		for(int i=0; i<correlationEntries.size(); i++) {
			CorrelatedOperation correlationEntry = correlationEntries.get(i);
			if(correlationEntry.getOperationEntry().getPartnerLink().equals(partnerLink) && 
					correlationEntry.getOperationEntry().getOperation().equals(operation)) {
				return true;
			}
		}
		return false;
	}
	
	public CorrelatedOperation getCorrelationEntry(String partnerLink, String operation) {
		for(int i=0; i<correlationEntries.size(); i++) {
			CorrelatedOperation correlationEntry = correlationEntries.get(i);
			if(correlationEntry.getOperationEntry().getPartnerLink().equals(partnerLink) && 
					correlationEntry.getOperationEntry().getOperation().equals(operation)) {
				return correlationEntry;
			}
		}
		return null;
	}
	
	public void addCorrelationEntry(CorrelatedOperation correlationEntry) {
		correlationEntries.add(correlationEntry);
	}

	public InstanceUnitPool getInstanceUnitPool() {
		return instanceUnitPool;
	}
	
	public InstanceUnit createInstance() {
		return null;
	}
	
	public String getCorrelatedInstanceUnit(String partnerLink, String operation, IServiceContext message) {
		return null;
	}
	
	protected String createInstanceId() {
		Date date = new Date();
		SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmssSSS");
		String time = sdf.format(date).toString();
		String instanceId = id + ":" + time + ":" + Math.abs((new Random()).nextLong());
		return instanceId;
	}

	public String getDeployTime() {
		return deployTime;
	}

	public void setDeployTime(String deployTime) {
		this.deployTime = deployTime;
	}

	public String getDirectory() {
		return directory;
	}

	public void setDirectory(String directory) {
		this.directory = directory;
	}
	
	public String getWSDLPath() {
		return directory + name + ".wsdl";
	}
	
	public String getBPELPath() {
		return directory + name + ".bpel";
	}
	
	public String getPDDPath() {
		return directory + name + ".pdd";
	}

	public String getVersion() {
		return version;
	}

	public void setVersion(String version) {
		this.version = version;
	}

	public void setState(int state) {
		this.state = state;
	}
	
	public DefinitionsExtended getWsdl(String URI) {
		return wsdlCatalog.get(URI);
	}
	
	public DefinitionsExtended[] getWsdls() {
		return wsdlCatalog.values().toArray(new DefinitionsExtended[0]);
	}

	public void addWsdl(DefinitionsExtended wsdl) {
		wsdlCatalog.put(wsdl.getDefinition().getTargetNamespace(), wsdl);
	}
	
	public File getWsdlFile(String URI){
		return wsdlFiles.get(URI);
	}
	
	public void addWsdlFile(String uri,File wsdlFile){
		wsdlFiles.put(uri, wsdlFile);
	}
	
	public void addProperty(Property property) {
		properties.put(MetasetUtil.jQNameToxQName(property.getQName()), property);
	}

	public void addPropertyAlias(PropertyAlias propertyAlias) {
		propertyAliases.add(propertyAlias);
	}

	public ArrayList<PropertyAlias> getPropertyAlias(QName messageType) {
		ArrayList<PropertyAlias> propAliases = new ArrayList<PropertyAlias>();
		for (int i = 0; i < propertyAliases.size(); i++) {
			PropertyAlias propertyAlias = propertyAliases.get(i);
			if (propertyAlias.getMessageType().equals(messageType)) {
				propAliases.add(propertyAlias);
			}
		}
		return propAliases;
	}
	
	public void addCorrelatedInstanceUnit(CorrelatedInstanceUnit correlatedInstanceUnit) {
		correlatedInstanceUnits.add(correlatedInstanceUnit);
	}
	
	public ArrayList<CorrelatedInstanceUnit> getCorrelatedInstanceUnits(String set) {
		ArrayList<CorrelatedInstanceUnit> cInstanceUnits = new ArrayList<CorrelatedInstanceUnit>();
		for(int i=0; i<correlatedInstanceUnits.size(); i++) {
			CorrelatedInstanceUnit cInstanceUnit = correlatedInstanceUnits.get(i);
			String name = cInstanceUnit.getCorrelationSetInstance().getName();
			if(name.equals(set)) {
				cInstanceUnits.add(cInstanceUnit);
			}
		}
		return cInstanceUnits;
	}
}
