package org.act.xservices.engine.rt.persistence.berkeley;

import java.lang.reflect.Field;
import java.lang.reflect.Modifier;
import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import com.sleepycat.persist.model.ClassMetadata;
import com.sleepycat.persist.model.DeleteAction;
import com.sleepycat.persist.model.Entity;
import com.sleepycat.persist.model.EntityMetadata;
import com.sleepycat.persist.model.EntityModel;
import com.sleepycat.persist.model.FieldMetadata;
import com.sleepycat.persist.model.KeyField;
import com.sleepycat.persist.model.Persistent;
import com.sleepycat.persist.model.PrimaryKey;
import com.sleepycat.persist.model.PrimaryKeyMetadata;
import com.sleepycat.persist.model.Relationship;
import com.sleepycat.persist.model.SecondaryKey;
import com.sleepycat.persist.model.SecondaryKeyMetadata;

public class EngineEntityModel extends EntityModel {

	private Map<String, ClassMetadata> classMap;
	private Map<String, EntityMetadata> entityMap;

	public EngineEntityModel() {
		super();
		classMap = new HashMap<String, ClassMetadata>();
		entityMap = new HashMap<String, EntityMetadata>();

		PrimaryKeyMetadata enginePK = new PrimaryKeyMetadata(
				"id",
				"long",
				"org.act.xservices.engine.rt.persistence.EngineRecord", 
				"");
		ClassMetadata engineClass = new ClassMetadata(
				"org.act.xservices.engine.rt.persistence.EngineRecord", 0,
				null, 
				true, 
				enginePK,
				new HashMap<String, SecondaryKeyMetadata>(), 
				null);
		classMap.put("org.act.xservices.engine.rt.persistence.EngineRecord",
				engineClass);
		EntityMetadata engineEntity = new EntityMetadata(
				"org.act.xservices.engine.rt.persistence.EngineRecord",
				enginePK, 
				new HashMap<String, SecondaryKeyMetadata>());
		entityMap.put("org.act.xservices.engine.rt.persistence.EngineRecord",
				engineEntity);
		
        //--------------------------------------------------------------------
		
		PrimaryKeyMetadata processInstancePK = new PrimaryKeyMetadata(
				"id",
				"java.lang.String",
				"org.act.xservices.engine.rt.persistence.ProcessInstanceRecord",
				null);
		SecondaryKeyMetadata processInstanceSK = new SecondaryKeyMetadata(
				"type",
				"java.lang.String",
				"org.act.xservices.engine.rt.persistence.ProcessInstanceRecord",
				null, 
				"type", 
				Relationship.MANY_TO_ONE, 
				null, 
				null);
		Map<String, SecondaryKeyMetadata> processInstanceSKs = new HashMap<String, SecondaryKeyMetadata>();
		processInstanceSKs.put("type", processInstanceSK);
		ClassMetadata processInstanceClass = new ClassMetadata(
				"org.act.xservices.engine.rt.persistence.ProcessInstanceRecord",
				0, 
				null, 
				true, 
				processInstancePK, 
				processInstanceSKs, 
				null);
		classMap.put("org.act.xservices.engine.rt.persistence.ProcessInstanceRecord",
				processInstanceClass);
		EntityMetadata processInstanceEntity = new EntityMetadata(
				"org.act.xservices.engine.rt.persistence.ProcessInstanceRecord",
				processInstancePK, 
				processInstanceSKs);
		entityMap.put("org.act.xservices.engine.rt.persistence.ProcessInstanceRecord",
				processInstanceEntity);

		 //--------------------------------------------------------------------
		
		PrimaryKeyMetadata activityInstancePK = new PrimaryKeyMetadata(
				"id",
				"long",
				"org.act.xservices.engine.rt.persistence.ActivityInstanceRecord",
				"");
		SecondaryKeyMetadata activityInstanceSK = new SecondaryKeyMetadata(
				"processInstanceId",
				"java.lang.String",
				"org.act.xservices.engine.rt.persistence.ActivityInstanceRecord",
				null, 
				"processInstanceId", 
				Relationship.MANY_TO_ONE, 
				null, 
				null);
		Map<String, SecondaryKeyMetadata> activityInstanceSKs = new HashMap<String, SecondaryKeyMetadata>();
		activityInstanceSKs.put("processInstanceId", activityInstanceSK);
		ClassMetadata activityInstanceClass = new ClassMetadata(
				"org.act.xservices.engine.rt.persistence.ActivityInstanceRecord",
				0, 
				null, 
				true, 
				activityInstancePK, 
				activityInstanceSKs, 
				null);
		classMap.put("org.act.xservices.engine.rt.persistence.ActivityInstanceRecord",
				activityInstanceClass);
		EntityMetadata activityInstanceEntity = new EntityMetadata(
				"org.act.xservices.engine.rt.persistence.ActivityInstanceRecord",
				activityInstancePK, 
				activityInstanceSKs);
		entityMap.put("org.act.xservices.engine.rt.persistence.ActivityInstanceRecord",
				activityInstanceEntity);
	}

	@Override
	public ClassMetadata getClassMetadata(String className) {
		ClassMetadata metadata = classMap.get(className);
		if (metadata == null) {
			Class<?> type;
			try {
				type = EntityModel.classForName(className);
			} catch (ClassNotFoundException e) {
				return null;
			}
			/* Get class annotation. */
			Entity entity = type.getAnnotation(Entity.class);
			Persistent persistent = type.getAnnotation(Persistent.class);
			if (entity == null && persistent == null) {
				return null;
			}
			if (entity != null && persistent != null) {
				throw new IllegalArgumentException(
						"Both @Entity and @Persistent are not allowed: "
								+ type.getName());
			}
			boolean isEntity;
			int version;
			String proxiedClassName;
			if (entity != null) {
				isEntity = true;
				version = entity.version();
				proxiedClassName = null;
			} else {
				isEntity = false;
				version = persistent.version();
				Class proxiedClass = persistent.proxyFor();
				proxiedClassName = (proxiedClass != void.class) ? proxiedClass
						.getName() : null;
			}
			/* Get instance fields. */
			List<Field> fields = new ArrayList<Field>();
			for (Field field : type.getDeclaredFields()) {
				int mods = field.getModifiers();
				if (!Modifier.isTransient(mods) && !Modifier.isStatic(mods)) {
					fields.add(field);
				}
			}
			/* Get the rest of the metadata and save it. */
			metadata = new ClassMetadata(className, version, proxiedClassName,
					isEntity, getPrimaryKey(type, fields), getSecondaryKeys(
							type, fields), getCompositeKeyFields(type, fields));
			classMap.put(className, metadata);
		}
		return metadata;
	}

	private PrimaryKeyMetadata getPrimaryKey(Class<?> type, List<Field> fields) {
		Field foundField = null;
		String sequence = null;
		for (Field field : fields) {
			PrimaryKey priKey = field.getAnnotation(PrimaryKey.class);
			if (priKey != null) {
				if (foundField != null) {
					throw new IllegalArgumentException(
							"Only one @PrimaryKey allowed: " + type.getName());
				} else {
					foundField = field;
					sequence = priKey.sequence();
					if (sequence.length() == 0) {
						sequence = null;
					}
				}
			}
		}
		if (foundField != null) {
			return new PrimaryKeyMetadata(foundField.getName(), foundField
					.getType().getName(), type.getName(), sequence);
		} else {
			return null;
		}
	}

	private Map<String, SecondaryKeyMetadata> getSecondaryKeys(Class<?> type,
			List<Field> fields) {
		Map<String, SecondaryKeyMetadata> map = null;
		for (Field field : fields) {
			SecondaryKey secKey = field.getAnnotation(SecondaryKey.class);
			if (secKey != null) {
				Relationship rel = secKey.relate();
				String elemClassName = null;
				if (rel == Relationship.ONE_TO_MANY
						|| rel == Relationship.MANY_TO_MANY) {
					elemClassName = getElementClass(field);
				}
				String keyName = secKey.name();
				if (keyName.length() == 0) {
					keyName = field.getName();
				}
				Class<?> relatedClass = secKey.relatedEntity();
				String relatedEntity = (relatedClass != void.class) ? relatedClass
						.getName()
						: null;
				DeleteAction deleteAction = (relatedEntity != null) ? secKey
						.onRelatedEntityDelete() : null;
				SecondaryKeyMetadata metadata = new SecondaryKeyMetadata(field
						.getName(), field.getType().getName(), type.getName(),
						elemClassName, keyName, rel, relatedEntity,
						deleteAction);
				if (map == null) {
					map = new HashMap<String, SecondaryKeyMetadata>();
				}
				if (map.put(keyName, metadata) != null) {
					throw new IllegalArgumentException(
							"Only one @SecondaryKey with the same name allowed: "
									+ type.getName() + '.' + keyName);
				}
			}
		}
		if (map != null) {
			map = Collections.unmodifiableMap(map);
		}
		return map;
	}
	
	private String getElementClass(Field field) {
        Class cls = field.getType();
        if (cls.isArray()) {
            return cls.getComponentType().getName();
        }
        if (java.util.Collection.class.isAssignableFrom(cls)) {
            Type[] typeArgs = ((ParameterizedType) field.getGenericType()).
                getActualTypeArguments();
            if (typeArgs == null ||
                typeArgs.length != 1 ||
                !(typeArgs[0] instanceof Class)) {
                throw new IllegalArgumentException
                    ("Collection typed secondary key field must have a" +
                     " single generic type argument: " +
                     field.getDeclaringClass().getName() + '.' +
                     field.getName());
            }
            return ((Class) typeArgs[0]).getName();
        }
        throw new IllegalArgumentException
            ("ONE_TO_MANY or MANY_TO_MANY secondary key field must have" +
             " an array or Collection type: " +
             field.getDeclaringClass().getName() + '.' + field.getName());
    }

    private List<FieldMetadata> getCompositeKeyFields(Class<?> type,
                                                      List<Field> fields) {
        List<FieldMetadata> list = null;
        for (Field field : fields) {
            KeyField keyField = field.getAnnotation(KeyField.class);
            if (keyField != null) {
                int value = keyField.value();
                if (value < 1 || value > fields.size()) {
                    throw new IllegalArgumentException
                        ("Unreasonable @KeyField index value " + value +
                         ": " + type.getName());
                }
                if (list == null) {
                    list = new ArrayList<FieldMetadata>(fields.size());
                }
                if (value <= list.size() && list.get(value - 1) != null) {
                    throw new IllegalArgumentException
                        ("@KeyField index value " + value +
                         " is used more than once: " + type.getName());
                }
                while (value > list.size()) {
                    list.add(null);
                }
                FieldMetadata metadata = new FieldMetadata
                    (field.getName(), field.getType().getName(),
                     type.getName());
                list.set(value - 1, metadata);
            }
        }
        if (list != null) {
            if (list.size() < fields.size()) {
                throw new IllegalArgumentException
                    ("@KeyField is missing on one or more instance fields: " +
                     type.getName());
            }
            for (int i = 0; i < list.size(); i += 1) {
                if (list.get(i) == null) {
                    throw new IllegalArgumentException
                        ("@KeyField is missing for index value " + (i + 1) +
                         ": " + type.getName());
                }
            }
        }
        if (list != null) {
            list = Collections.unmodifiableList(list);
        }
        return list;
    }

	@Override
	public EntityMetadata getEntityMetadata(String className) {
		EntityMetadata metadata = entityMap.get(className);
		return metadata;
	}

	@Override
	public Set<String> getKnownClasses() {
		return Collections.unmodifiableSet(new HashSet<String>(classMap
				.keySet()));
	}

}
