package org.act.xservices.rt.xcontainer.adaptation;

import org.act.xservices.rt.xcontainer.adaptation.adapter.IAdapter;
import org.act.xservices.rt.xcontainer.framework.AConnector;
import org.act.xservices.rt.xcontainer.framework.IContainerEnvironment;
import org.act.xservices.rt.xcontainer.framework.config.IConfigElement;
import org.act.xservices.rt.xcontainer.framework.dispatch.ITaskFactory;
import org.act.xservices.rt.xcontainer.framework.lifecycle.LifecycleException;
import org.act.xservices.rt.xcontainer.framework.util.JavaUtils;
import org.act.xservices.rt.xcontainer.inf.context.IServiceContext;


/**
 * 
 * @LastModifyTime 2008-9-8
 * @CreationTime 2007-06-20
 * @author Lizq [lizq@act.buaa.edu.cn]
 * @since JDK1.5
 */
public final class AdaptationConnector extends AConnector<IServiceContext,IServiceContext> {
	
	/** Web */
	private IAdapter wsProcessor;

	/** BPEL */
	private IAdapter bpelProcessor;

	
	
	public AdaptationConnector(String connectorName) {
		super(connectorName);
	}

	public AdaptationConnector() {

	}

	public IAdapter getWsProcessor() {
		return wsProcessor;
	}

	public IAdapter getBpelProcessor() {
		return bpelProcessor;
	}
	
	/** 
	 * дĬʵֻ֣Ҫɷ
	 */
	protected void doInitDispatcher() {
		// ɷɷб
		this.doInitInputDispatcher();
	}

	/** 
	 * ʼԴ
	 */
	protected void doInitLocal() {
		
		//Processor
		IConfigElement processorsCE = 
			this.connectorConfiguration.getFirstChildWithName("Adapters");
		
		//---WSProcessor
		IConfigElement wsProcessorCE = processorsCE.getFirstChildWithName("WSAdapter");
		if(wsProcessorCE != null){
			boolean enable_wsp = JavaUtils.isTrue(wsProcessorCE.getAttribute("enable"), true);
			if (enable_wsp) { //WSProcessor
				try {
					wsProcessor = (IAdapter) Class.forName(
							wsProcessorCE.getAttribute("class")).newInstance();
					wsProcessor.config(wsProcessorCE);
					wsProcessor.setTargetObjectPool(getInputTarget());
					wsProcessor.setContainerEnvironment(
							this.connectorEnvironment.getContainerEnvironment());

					// WSProcessorContainerEnvironment
					this.connectorEnvironment.getContainerEnvironment().setAttribute(
							IContainerEnvironment.WS_PROCESSOR, wsProcessor);

				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		}
		
		//---BPELProcessor
		IConfigElement bpelProcessorCE = processorsCE.getFirstChildWithName("BPELAdapter");
		if(bpelProcessorCE != null){
			boolean enable_bpelp = JavaUtils.isTrue(bpelProcessorCE.getAttribute("enable"), true);
			if (enable_bpelp) { // BPELProcessor
				try {
					bpelProcessor = (IAdapter) Class.forName(
							bpelProcessorCE.getAttribute("class")).newInstance();
					bpelProcessor.config(bpelProcessorCE);
					bpelProcessor.setTargetObjectPool(getInputTarget());
					bpelProcessor.setContainerEnvironment(
							this.connectorEnvironment.getContainerEnvironment());

					// BPELProcessorContainerEnvironment
					this.connectorEnvironment.getContainerEnvironment().setAttribute(
							IContainerEnvironment.BPEL_PROCESSOR, bpelProcessor);
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		}
		
	}//---end of doInitLocal()---

	
	
	public ITaskFactory<IServiceContext> getInputTaskFactory() {
		AdaptationInTaskFactory adaInTaskFactory = new AdaptationInTaskFactory(this);
		return adaInTaskFactory;
	}
	
	public ITaskFactory<IServiceContext> getOutputTaskFactory() {
		System.err.println("This method AdaptationConnector.getOutputTaskFactory() is invalid. ");
		return null;
	}
	
	/**
	 * ,ע˴д˷
	 */
	public void start() throws LifecycleException{
		if(wsProcessor != null){
			wsProcessor.start();
		}
		if(bpelProcessor != null){
			bpelProcessor.start();
		}
		
		startInputDispatcher(); //ɷ
	}
	
	/** 
	 * ֹͣ,ע˴д˷
	 * @throws LifecycleException
	 */
	public void stop() throws LifecycleException{
		stopInputDispatcher();
		
		if(this.getInputThreadpool() != null){
			this.getInputThreadpool().shutdown();
		}
		
		if(wsProcessor != null){
			wsProcessor.stop();
		}
		if(bpelProcessor != null){
			bpelProcessor.stop();
		}
	}

	/** 
	 * TODO дǣby lizq 2009-10-02
	 */
	public void destroy() {
		if(wsProcessor != null){
			wsProcessor.destroy();
		}
		
		if(bpelProcessor != null){
			bpelProcessor.destroy();
		}
	}
	
}
