package org.act.xservices.rt.xcontainer.addressing;

import java.net.URI;
import java.util.Collection;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.act.xservices.rt.metaset.MetasetBuilder;
import org.act.xservices.rt.metaset.MetasetParserFactory;
import org.act.xservices.rt.xcontainer.addressing.faults.SoapFault;
import org.act.xservices.rt.xcontainer.addressing.uitl.MultiMap;
import org.act.xservices.rt.xcontainer.addressing.uitl.SoapConstants;
import org.act.xservices.rt.xcontainer.addressing.uitl.WsaConstants;
import org.act.xservices.rt.xcontainer.addressing.uuid.RandomUuidFactory;
import org.act.xservices.rt.xom.Attribute;
import org.act.xservices.rt.xom.Metaset;
import org.act.xservices.rt.xom.QName;
import org.act.xservices.rt.xom.util.MetasetHelper;

public class MessageHeaders {
	//
	// ڲе쳣Ϣ
	//

	protected static final String _REQUEST = "Request";

	protected static final String _RESPONSE = "Response";

	//
	// below are the standard WS-A header values for SOAP messages
	// SOAPϢı׼WS-A header values
	//

	private String _action = null;

	//
	// еķWS-A headers 
	//
	private Map _customHeadersByQName = new MultiMap();

	private EndpointReference _faultTo = null;

	private EndpointReference _from = null;

	private String _messageID = null;

	private String _relationship = null;

	private EndpointReference _replyTo = null;

	private EndpointReference _to = null;

	/**
	 * 
	 * ӸSOAP HeaderдһЧWS-AϢͷ. wsa:To URIreference parameter ΪһEPR;
	 * EPRԱжĸ WS-resourceĿWS-resource. <br>
	 * <br>
	 * һǡreply headersͨʹcreateReplyHeaders(URI) and createFaultHeaders()
	 * , SOAP Headerǡwsa:From, wsa:ReplyTo, wsa:FaultTo EPR.
	 * 
	 * @param soapHeaders
	 *            ʾSOAP envelope's HeaderԪ. HeaderӦðбWS-AԪ
	 * 
	 * @throws SoapFault
	 *             <ul>
	 *             <li>
	 *             WS-A EPR, wsa:To, wsa:From, wsa:ReplyToκһЧ,
	 *             wsa:FaultTo XMLЧ</li>
	 *             <li>
	 *             wsa:To, wsa:MessageID, wsa:RelatesToκһЧ, wsa:Action
	 *             ֵһЧURI.</li>
	 *             <li>
	 *             If either the wsa:To, wsa:MessageID, or wsa:Action value is
	 *             undefined.</li>
	 *             <li>
	 *             wsa:To, wsa:MessageIDκһЧ, wsa:Actionֵδ</li>
	 *             </ul>
	 * 
	 * @see #createFaultHeaders()
	 * @see #createReplyHeaders()
	 * 
	 */
	public MessageHeaders(Metaset soapHeaders) throws SoapFault {
		if (soapHeaders == null)
			throw new NullPointerException("The SOAP Header element is null.");

		String toURI = soapHeaders.elementText(WsaConstants.TO_QNAME.getName());
		_action = soapHeaders.elementText(WsaConstants.ACTION_QNAME.getName());
		_messageID = soapHeaders.elementText(WsaConstants.MESSAGE_ID_QNAME.getName());
		_relationship = soapHeaders.elementText(WsaConstants.RELATES_TO_QNAME.getName());

		//
		// еURIsǱֵ
		//

		if (toURI == null) {
			Object[] filler = { WsaConstants.TO_QNAME };
			throwInvalidAddressingHeaderFault("The SOAP headers are missing a required WS-Addressing header: " + filler.toString());
		}

		if (_action == null) {
			Object[] filler = { WsaConstants.ACTION_QNAME };
			throwInvalidAddressingHeaderFault("The SOAP headers are missing a required WS-Addressing header: " + filler.toString());
		}

		//
		// wsa:To һURI, ϣΪһEPR
		//
		_to = new EndpointReference(URI.create(toURI));
		//
		// ȡһѡEPRֵ, ֵӦðreplies
		//
		_faultTo = getEPR(soapHeaders, WsaConstants.FAULT_TO_QNAME);
		_from = getEPR(soapHeaders, WsaConstants.FROM_QNAME);
		_replyTo = getEPR(soapHeaders, WsaConstants.REPLY_TO_QNAME);

		if (_messageID == null && (_from != null || _replyTo != null))
			throwInvalidAddressingHeaderFault("The SOAP headers did not contain a wsa:MessageID. A wsa:MessageID value is required if the headers have a wsa:From or wsa:ReplyTo EPR.");

		List<Metaset> children = soapHeaders.elements();

		//
		// еSOAP headersжǷWS-A reference parameters
		//

		for (int n = 0; n < children.size(); ++n) {
			QName qname = children.get(n).getQName();

			//
			// 鿴ǷһWS-A reference parameter
			//
			Metaset tempMet = children.get(n);
			String wsaAttr = children.get(n).attributeValue(WsaConstants.IS_REFERENCE_PARAMETER);
			if (wsaAttr != null && Boolean.valueOf(wsaAttr) == Boolean.TRUE) {
				//
				// parameter XMLɾSOAPϢ - EPR XMLƶϢ/Ƭ
				// ȥʱһǳҪ. ΪЩϢ/ƬοûSOAPǰ׺.
				//
				tempMet.remove( tempMet.attribute(SoapConstants.ACTOR));
				tempMet.remove(tempMet.attribute(SoapConstants.MUST_UNDERSTAND));
				_to.addParameter(qname, children.get(n));
			}

			//
			// û,ȷһWS-AԪزҰӵcustom headers
			//
			else if (!qname.getNamespaceURI()
					.equals(WsaConstants.NAMESPACE_URI))
				_customHeadersByQName.put(qname, children.get(n).createCopy());
		}
	}

	/**
	 * 
	 * ָEPRActionһµWS-AϢͷ. 캯ԼΨһwsa:MessageID.
	 * 
	 * @param to
	 *            ܹΪwsa:ToṩURIEPR
	 * 
	 * @param action
	 *            The wsa:Action URI.
	 * 
	 */
	protected MessageHeaders(EndpointReference to, String action) {
		if (to == null)
			throw new NullPointerException("The EndpointReference representing wsa:To is null.");

		_to = to;
		_action = action;

		_messageID = createMessageID();
	}
	
	/**
     * 
     * һڸöWS-AϢ; ظһЯϢʱ, صheadersԱ
     * ʹáThe headersλwsa:FaultTo EPR WS-A fault action URI. 
     * ûwsa:FaultTo, wsa:FromWS-A anonymous EPRᱻʹá
     * <br><br>
     * <b>ע:</b> ÷Ӧñڴ"normal" responsesheaders - ˹Ӧʹ
     * createReplyHeaders(URI).
     *
     * @return A new MessageHeaders where:
     *         <ul>
     *         <li> wsa:ToǸö wsa:FaultTo,wsa:From, or the anonymous EPRURI</li>
     *         <li>the wsa:From is this object's wsa:To EPR</li>
     *         <li>the wsa:From Ǹö wsa:To EPR</li>
     *         <li>the wsa:RelatesTo Ǹöĵ wsa:MessageID URI</li>
     *         <li>the wsa:Action Ǳ׼ WS-A fault URI.</li>
     *         </ul>
     * 
     * @see #createReplyHeaders()
     * @see #getFaultToAddress()
     * @see #setFaultToAddress(EndpointReference)
     * @see WsaConstants#FAULT_URI
     *
     */
    public MessageHeaders createFaultHeaders()
    {
        EndpointReference faultTo = getFaultToAddress();
        String messageID = getMessageID();
        
        //
        // wsa:FaultTo  wsa:Fromڣֻܷfault headers
        //
        if (faultTo == null)
        {
            EndpointReference from = getFromAddress();
            
            if (from == null)
                from = WsaConstants.ANONYMOUS_EPR;
            
            faultTo = from;
        }
        
        //
        // 1. Ҫ wsa:FaultTo  headers
        // 2. ʹrequest MessageIDheaders
        // 3. wsa:Toеsource
        //
        MessageHeaders faultHeaders = 
            new MessageHeaders(faultTo, WsaConstants.FAULT_URI);
        
        if (messageID != null)
            faultHeaders.setRelationship(messageID);
        
        faultHeaders.setFromAddress(getToAddress());
        
        // Add
        
        return faultHeaders;
    }
    
    /**
     * 
     * ڸö,һµWS-AϢͷ; ظһϢ(Ǵ)ʱ÷صheaders 
     * ʹ. The headerswsa:ReplyTo EPR. ûwsa:ReplyTo,
     * wsa:From or the WS-A anonymous EPRᱻʹ.
     * <br><br>
     * <b>ע:</b> This method should not be used to create headers for 
     * responses where a fault has occurred - that should be done with 
     * createFaultHeaders().
     * дʱ, ÷Ӧñڴresponses headers - Ӧ
     * createFaultHeaders()
     *
     * @return A new MessageHeaders where:
     *         <ul>
     *         <li> wsa:To Ǹö wsa:ReplyTo,wsa:From, or the anonymous EPRURI</li>
     *         <li>the wsa:From Ǹö wsa:To EPR</li>
     *         <li>the wsa:RelatesTo is the URI of this object's 
     *         wsa:MessageID</li>
     *         <li>wsa:RelatesToǸöwsa:MessageIDURI</li>
     *         <li>the wsa:Action ʹ"Response"ǰ׺ĵǰURI</li>
     *         </ul>
     * 
     * @see #getReplyToAddress()
     * @see #setReplyToAddress(EndpointReference)
     *
     */
    public MessageHeaders createReplyHeaders()
    {
        EndpointReference replyTo = getReplyToAddress();
        String messageID = getMessageID();

        //
        // wsa:ReplyTo or wsa:From,ֻܷϢͷ
        //
        if (replyTo == null)
        {
            EndpointReference from = getFromAddress();
            
            if (from == null)
                from = WsaConstants.ANONYMOUS_EPR;
            
            replyTo = from;
        }
        
        String requestAction = getAction().toString();
        String replyAction = null;
        
        int requestSuffix = requestAction.lastIndexOf(_REQUEST);
        
        if (requestSuffix >= 0)
            replyAction = requestAction.substring(0, requestSuffix) + _RESPONSE;
        
        else
            replyAction = requestAction + _RESPONSE;
        
        //
        // 1. һ wsa:ReplyTo ͷ
        // 2. headersMessageID
        // 3. wsa:Toеsource
        //
        MessageHeaders replyHeaders = new MessageHeaders(replyTo, replyAction);
        
        
        if (messageID != null)
            replyHeaders.setRelationship(messageID);
        
        replyHeaders.setFromAddress(getToAddress());
        
        return replyHeaders;
    }
    
    /**
     * 
     * @return The wsa:Action URI. This is required value.
     *
     */
    public String getAction()
    {
        return _action;
    }
    
    /**
     * 
     * ֻҪָSOAP header elementһʵ,ͬʱҪһϻߵ,
     * ֻǻȡһ,ô˷õ.
     * 
     * @param elementName
     *        ҪصķWS-A SOAP header
     *        
     * @return ʹָҵĵһDOM Element,ûԪرҵ,򷵻null
     * 
     */
    public Metaset getCustomHeader(QName elementName)
    {
        Collection headers = getCustomHeaders(elementName);
        return headers.isEmpty() ? null : (Metaset)headers.iterator().next();
    }
    
    /**
     * 
     * @param elementName
     *        ҪصķWS-A SOAP header
     *        
     * @return ӵָƵDOM Elementsļ. üϿΪempty
     * 
     */
    public Collection getCustomHeaders(QName elementName)
    {
        Collection headers = (Collection)_customHeadersByQName.get(elementName);
        return headers != null ? headers : Collections.EMPTY_LIST;
    }
    
    /**
     * 
     * @return QNamesļ, ü϶SOAP header:
     *         a. ûWS-Aռ
     *         b. һWS-A reference parameter
     *         üϿΪ
     * 
     */
    public Collection getCustomHeaderNames()
    {
        return Collections.unmodifiableSet(_customHeadersByQName.keySet());
    }
    
    /**
     * 
     * @return The wsa:FaultTo EPR. ֵδ,÷null
     *
     */
    public EndpointReference getFaultToAddress()
    {
        return _faultTo;
    }
    
    /**
     *
     * @return The wsa:From EPR. ֵδ,÷null
     *
     */
    public EndpointReference getFromAddress()
    {
        return _from;
    }
    
    /**
     *
     * @return The unique wsa:MessageID UUID. һֵ
     *
     */
    public String getMessageID()
    {
        return _messageID;
    }
    
    /**
     * 
     * @return wsa:Action URI,һslashΪʶ. URI
     * 		   'http://example.com/spec/DoStuff',ôصĸֵΪ'DoStuff'.
     * 
     *
     */
    public String getMethodName()
    {
        String actionURI = getAction().toString();
        int lastSlash = actionURI.lastIndexOf('/');
        String name = actionURI.substring(lastSlash + 1);
        
        int request = name.indexOf("Request");
        
        if (request >= 0)
            name = name.substring(0, request);
        
        return Character.toLowerCase(name.charAt(0)) + name.substring(1);
    }
    
    /**
     *
     * @return wsa:RelatesToֵ. ֵδ, ˷null. wsa:RelatesTo
     * 		        һUUID,UUIDӳ䵽һMessageHeader's wsa:MessageID.
     *
     */
    public String getRelationship()
    {
        return _relationship;
    }
    
    /**
     * 
     * @return This method Ƿ WsaUtils.REPLY_RELATIONSHIP_QNAME.
     * 
     * @see WsaConstants#REPLY_RELATIONSHIP_QNAME 
     *
     */
    public QName getRelationshipType()
    {
        return WsaConstants.REPLY_RELATIONSHIP_QNAME;
    }
    
    /**
     *
     * @return The wsa:ReplyTo EPR. ֵδ,÷null
     *
     */
    public EndpointReference getReplyToAddress()
    {
        return _replyTo;
    }
    
    /**
     *
     * @return wsa:To URIͶSOAP Headerеreference properties()EPR.
     * 		          һֵ
     *
     */
    public EndpointReference getToAddress()
    {
        return _to;
    }
    
    /**
     * 
     * @param action
     *        The wsa:Action URI. Ϊnull
     *
     */
    protected void setAction(String action)
    {
        if (action == null)
            throw new NullPointerException("The wsa:Action URI is null.");
        
        _action = action;
    }
    
    /**
     * 
     * @param faultTo
     *        The wsa:FaultTo EPR - ǿ,ᴴEPRһ,ԭʼEPR
     *        ޸ľͲӰ쵽ͷ.
     *
     */
    public void setFaultToAddress(EndpointReference faultTo)
    {
        if (faultTo == null)
            _faultTo = null;
        
        //
        // ҪԼĿÿĸԪwsa:FaultTo
        //
        else
            _faultTo = new EndpointReference(faultTo, WsaConstants.FAULT_TO_QNAME);
    }
    
    /**
     * 
     * @param from
     *        The wsa:From EPR - ǿ,ᴴEPRһ,ԭʼEPR
     *        ޸ľͲӰ쵽ͷ.
     *
     */
    public void setFromAddress(EndpointReference from)
    {
        if (from == null)
            _from = null;

        //
        // ҪԼĿÿĸԪwsa:From
        //
        else
            _from = new EndpointReference(from, WsaConstants.FROM_QNAME);
    }
    
    /**
     * 
     * @param messageID
     *        The wsa:MessageID. Ϊnull
     *
     */
    protected void setMessageID(String messageID)
    {
        if (messageID == null)
            throw new NullPointerException("The wsa:MessageID is null.");
        
        _messageID = messageID;
    }
    
    /**
     * 
     * @param relationship
     *        The wsa:RelatesTo ID. Ϊnull(messageID)
     *
     */
    public void setRelationship(String relationship)
    {
        if (relationship == null)
            throw new NullPointerException("The wsa:RelatesTo ID is null.");
        
        _relationship = relationship;
    }
    
    /**
     * 
     * @param replyTo
     *        The wsa:ReplyTo EPR - ǿ,ᴴEPRһ,ԭʼEPR
     *        ޸ľͲӰ쵽ͷ.
     *
     */
    public void setReplyToAddress(EndpointReference replyTo)
    {
        if (replyTo == null)
            _replyTo = null;

        //
        // ҪԼĿÿĸԪwsa:ReplyTo
        //
        else
            _replyTo = new EndpointReference(replyTo, WsaConstants.REPLY_TO_QNAME);
    }
    
    /**
     * 
     * @param to
     *        An EPR with the address value for the wsa:To URI. This cannot 
     *        be null. No copy of the given EPR is made, so subsequent 
     *        modifications to it will affect this object.
     *        ӵпwsa:To URIEPR. Ϊnull. EPRıݲᱻ,Զ
     *        ޸ĻӰ쵽ö.
     *
     */
    protected void setToAddress(EndpointReference to)
    {
        if (to == null)
            throw new NullPointerException("The EndpointReference representing wsa:To is null.");
        
        _to = to;
    }


	protected void throwInvalidAddressingHeaderFault(String message)
			throws SoapFault {
		SoapFault fault = new SoapFault(message);
		fault.setCode(SoapConstants.SENDER_QNAME);
		fault.setSubCode(WsaConstants.INVALID_HEADER_FAULT_QNAME);
		throw fault;
	}
	
	/**
     * 
     * ָXML,һЧEPR. EPR XMLԪرָQName
     *
     * @param root
     *        EPRԪΪ亢֮һĵԪ
     * 
     * @param qname
     *        һEPRԪصQName
     * 
     * @return ָXMLһЧEPR
     *
     */
    private EndpointReference getEPR(Metaset root, QName qname)
        throws SoapFault
    {
    	Metaset eprMet = root.element(qname);
        
        if (eprMet == null)
            return null;
        
        return new EndpointReference(eprMet);
    }
    
    /**
     * 
     * @return һwsa:MessageIDΨһUUID
     *
     */
    private String createMessageID()
    {
        return RandomUuidFactory.getInstance().createUUID();
    }
    
    /**
     * 
     * 
     *
     * @param doc
     *        ΪSOAP Header XMLԪʱʹõXML Document. Ԫ
     *        ᱻӵDocument
     * 
     * @return 
     *         ʹЧWS-A headersΪԪص SOAP Header element. ܱ֤
     *         WS-A headers˳. wsa:To EPRеreference parameters and properties
     *         Ϊ root SOAP Header elementĺӱ׷ӵ root SOAP Header element.
     * 
     * @see SoapConstants#HEADER_QNAME
     *
     */
    public Metaset toMetaset()
    {
    	MetasetBuilder metBuilder = MetasetBuilder.getInstance();
    	Metaset soapHeaders = metBuilder.createElement(SoapConstants.HEADER_QNAME);
    	this.populateIntoMetaset(soapHeaders);
    	return soapHeaders;
    }
    
    /**
     * This method used for resource's ws-addressing
     * @param soapHeaders
     * @return
     */
    protected void populateIntoMetaset(Metaset soapHeaders){
    	MetasetBuilder metBuilder = MetasetBuilder.getInstance();
        //
        // wsa:To
        //
        URI toURI = getToAddress().getAddress();
        Metaset to = metBuilder.createElement(WsaConstants.TO_QNAME);
        to.setText(toURI.toString());
        soapHeaders.add(to);
        
        //
        // wsa:Action
        //
        Metaset actionMet = metBuilder.createElement(WsaConstants.ACTION_QNAME);
        actionMet.setText(getAction());
        soapHeaders.add(actionMet);
        
        //
        // wsa:MessageID
        //
        String messageID = getMessageID();
        
        if (messageID != null){
            Metaset msgIdMet = metBuilder.createElement(WsaConstants.MESSAGE_ID_QNAME);
            msgIdMet.setText(messageID);
            soapHeaders.add(msgIdMet);
        }
        
        //
        // wsa:RelatesTo (optional)
        //
        String relatesTo = getRelationship();
        
        if (relatesTo != null)
        {
            Metaset relationshipMet = metBuilder.createElement(WsaConstants.RELATES_TO_QNAME);
            relationshipMet.setText(relatesTo);
            
            String type = MetasetHelper.getInstance().convertQNameToString(getRelationshipType());
            // TODO: setAttributeNS
            relationshipMet.addAttribute(WsaConstants.RELATIONSHIP_TYPE, type);
            
            soapHeaders.add(relationshipMet);
        }
        
        //
        // wsa:From, wsa:ReplyTo, and wsa:FaultTo (optional)
        //
        
        EndpointReference[] eprs = new EndpointReference[] {
            getFromAddress(), getReplyToAddress(), getFaultToAddress() 
        };
        
        for (int n = 0; n < eprs.length; ++n)
        {
            if (eprs[n] != null)
            {
            	Metaset xml = eprs[n].toMetaset();
                xml = xml.createCopy();
                soapHeaders.add(xml);
            }
        }
        
        //
        // wsa:ReferenceParameters (optional)
        //
        
        Metaset[] parameters = getToAddress().getParameters();
        
        for (int n = 0; n < parameters.length; ++n)
        {
        	Metaset copy = parameters[n].createCopy();
        	Attribute irpAttr = copy.attribute(WsaConstants.IS_REFERENCE_PARAMETER_QNAME);
        	if(irpAttr == null){
        		irpAttr = metBuilder.createAttribute(copy,WsaConstants.IS_REFERENCE_PARAMETER_QNAME, "true");
        	} else {
				irpAttr.setValue("true");
			}
            
            soapHeaders.add(copy);
        	
        }
        
        //
        // non-WS-A headers
        //
        Iterator i = getCustomHeaderNames().iterator();
        
        while (i.hasNext())
        {
            QName qname = (QName)i.next();
            Iterator j = getCustomHeaders(qname).iterator();
            
            while (j.hasNext())
            {
                Metaset next = ((Metaset)j.next()).createCopy();
                soapHeaders.add(next);
            }
        }
    }
    
    public static void main(String[] args) throws Exception{
    	String xmlHeader = "<soapenv:Header xmlns:soapenv=\"http://schemas.xmlsoap.org/soap/envelope/\" xmlns:wsa=\"http://www.w3.org/2005/08/addressing\">";
		xmlHeader += "<wsa:To>http://192.168.2.100:8091/JvmResource/GetRequest</wsa:To>";
		xmlHeader += "<wsa:From>";
		xmlHeader += "<wsa:Address>http://192.168.2.99/Client</wsa:Address>";
		xmlHeader += "</wsa:From>";
		xmlHeader += "<wsa:ReplyTo>";
		xmlHeader += "<wsa:Address>http://192.168.2.101/Receiver</wsa:Address>";
		xmlHeader += "</wsa:ReplyTo>";
		xmlHeader += "<wsa:Action>http://docs.oasis-open.org/wsrf/rpw-2/GetResourceProperty/GetResourcePropertyRequest</wsa:Action>";
		xmlHeader += "<wsa:MessageID>uuid:f4246094-4e55-4487-d7d6-73ab9ebc7311</wsa:MessageID>";
		xmlHeader += "<wsa:ReferenceParameters>";
		xmlHeader += "<act:ResourceID xmlns:act=\"http://act.buaa.edu.cn/ws/xservices/resource\">uuid:12356</act:ResourceID>";
		xmlHeader += "</wsa:ReferenceParameters>";
		xmlHeader += "</soapenv:Header>";
		
		Metaset soapHeaders = MetasetParserFactory.getMetasetParser().parseXMLString(xmlHeader);
		String toURI = soapHeaders.elementText(WsaConstants.TO_QNAME.getName());
		
		// Create MessageHeaders
		MessageHeaders msgHeaders = new MessageHeaders(soapHeaders);
		System.out.println("msgHeaders: " + msgHeaders.toMetaset().asXML());
		
		MessageHeaders replyHeaders = msgHeaders.createReplyHeaders();
		System.out.println("replyHeaders: " + replyHeaders.toMetaset().asXML());
		
		MessageHeaders faultHeaders = msgHeaders.createFaultHeaders();
		System.out.println("faultHeaders: " + faultHeaders.toMetaset().asXML());
    }
}
