package org.act.xservices.rt.xcontainer.addressing.faults;

import org.act.xservices.rt.metaset.MetasetBuilder;
import org.act.xservices.rt.xcontainer.addressing.uitl.SoapConstants;
import org.act.xservices.rt.xom.Metaset;
import org.act.xservices.rt.xom.QName;
import org.act.xservices.rt.xom.util.MetasetHelper;

public class SoapFault extends Exception {
private static final long serialVersionUID = -8612773172898054578L;
    
    private QName _code = SoapConstants.RECEIVER_QNAME;
    
    private QName _subCode = null;
    private String _reason = null;
    private String _nodeURI = null;
    private String _roleURI = null;
    
    private Metaset _detail = null;
    
    private MetasetHelper metHelper = MetasetHelper.getInstance();
    
    public SoapFault(Metaset xml)
    {
    	Metaset codeXML = xml.element(SoapConstants.CODE_QNAME);
        _code = metHelper.getQNameFromChildElemText(codeXML, SoapConstants.VALUE_QNAME);
        
        Metaset subCodeXML = codeXML.element(SoapConstants.SUBCODE_QNAME);
         
        if (subCodeXML != null){
        	_subCode = metHelper.getQNameFromChildElemText(subCodeXML, SoapConstants.VALUE_QNAME);
        }
        
        Metaset reasonXML = xml.element(SoapConstants.REASON_QNAME);
        _reason = reasonXML.elementText(SoapConstants.TEXT_QNAME);
        _nodeURI = xml.elementText(SoapConstants.NODE_QNAME);
        _roleURI = xml.elementText(SoapConstants.ROLE_QNAME);
        Metaset detailXML = xml.element(SoapConstants.DETAIL_QNAME);
        
        if (detailXML != null)
            _detail = detailXML.elements().get(0);
    }
    
    public SoapFault(String message)
    {
        super(message);
        _reason = message;
    }
    
    public SoapFault(String message, Throwable cause)
    {
        super(message, cause);
        _reason = message;     
    }
    
    public SoapFault(Throwable cause)
    {
        super(cause);
        _reason = cause.getMessage();
    }
    
    public QName getCode()
    {
        return _code;
    }

    public void setCode(QName code)
    {
        _code = code;
    }

    public Metaset getDetail()
    {
        return _detail;
    }

    public void setDetail(Metaset detail)
    {
        _detail = detail;
    }
    
    public String getMessage()
    {
        return getReason();
    }

    public String getNodeURI()
    {
        return _nodeURI;
    }

    public void setNodeURI(String nodeURI)
    {
        _nodeURI = nodeURI;
    }

    public String getReason()
    {
        return _reason;
    }

    public void setReason(String reason)
    {
        _reason = reason;
    }

    public String getRoleURI()
    {
        return _roleURI;
    }
    
    public void setRoleURI(String roleURI)
    {
        _roleURI = roleURI;
    }
    
    public QName getSubCode()
    {
        return _subCode;
    }
    
    public void setSubCode(QName subCode)
    {
        _subCode = subCode;
    }

    public Metaset toMetaset()
    {
    	//Metaset faultXML = metHelper.createElement(doc, SoapConstants.FAULT_QNAME);
    	MetasetBuilder metBuilder = MetasetBuilder.getInstance();
    	Metaset faultXML = metBuilder.createElement(SoapConstants.FAULT_QNAME);
    	Metaset codeXML = metBuilder.createElement(SoapConstants.CODE_QNAME);
        faultXML.add(codeXML);
        
        // metHelper.setElement(codeXML, SoapConstants.VALUE_QNAME, getCode());
        Metaset codeMetaset = codeXML.element(SoapConstants.VALUE_QNAME);
        codeMetaset = codeMetaset == null ? (metBuilder.createElement(SoapConstants.VALUE_QNAME)) : codeMetaset;
        codeMetaset.setText(metHelper.convertQNameToString(getCode()));
        codeXML.add(codeMetaset);

        QName subCode = getSubCode();
        if (subCode != null)
        {
        	Metaset subCodeXML = metBuilder.createElement(SoapConstants.SUBCODE_QNAME);
        	codeXML.add(subCodeXML);
        	
        	Metaset subCodeMet = metBuilder.createElement(SoapConstants.VALUE_QNAME);
        	subCodeMet.setText(metHelper.convertQNameToString(subCode));
        	subCodeXML.add(subCodeMet);
        }
        
        Metaset reasonXML = metBuilder.createElement(SoapConstants.REASON_QNAME);
        faultXML.add(reasonXML);
        
        Metaset reasonMet = metBuilder.createElement(SoapConstants.TEXT_QNAME);
        reasonMet.setText(getReason());
        faultXML.add(reasonMet);
        
        
        String nodeURI = getNodeURI();
        if (nodeURI != null){
            Metaset nodeMet = metBuilder.createElement(SoapConstants.NODE_QNAME);
            nodeMet.setText(nodeURI);
            faultXML.add(nodeMet);
        }
        
        String roleURI = getRoleURI();
        if (roleURI != null){
            Metaset roleMet = metBuilder.createElement(SoapConstants.ROLE_QNAME);
            roleMet.setText(roleURI);
            faultXML.add(roleMet);
        }
        
        Metaset detail = getDetail();
        if (detail != null){
            Metaset detailMet = metBuilder.createElement(SoapConstants.DETAIL_QNAME);
            detailMet.setText(detail.asXML());
            faultXML.add(detailMet);
        }
        
        return faultXML;
    }
}
