package org.act.xservices.rt.client;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;

import org.act.xservices.rt.client.util.ClientConstants;
import org.act.xservices.rt.soap.ISOAPMessage;
import org.act.xservices.rt.soap.builder.ISOAPBuilder;
import org.act.xservices.rt.soap.builder.SOAPBuilderFactory;
import org.act.xservices.rt.xom.Metaset;
import org.apache.http.Header;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.HttpClient;
import org.apache.http.client.ResponseHandler;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.AbstractHttpEntity;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.BasicResponseHandler;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.params.CoreProtocolPNames;
/**
 * 
 * @author viking
 * @date 2010-8-8
 * @since JDK 1.5.0
 * @version 1.0.0
 *  
 *  This class is used to send and receive soap message with http.
 *
 */
public class SOAPHttpClient {

	private AbstractHttpEntity requestEntity;
	private HttpClient httpclient = new DefaultHttpClient();
	private HttpPost httppost;

	public SOAPHttpClient(String uri) {
		httppost = new HttpPost(uri);
	}

	public void setContentType(String contentType) {		
		requestEntity.setContentType(contentType);
	}

	public void setContentEncoding(String encoding) {
		requestEntity.setContentEncoding(encoding);
	}

	public void addHttpHeader(Header header) {
		httppost.addHeader(header);
	}

	public void addHttpHeader(String name, String value) {
		httppost.addHeader(name, value);
	}

	public Metaset sendAndRecieveSOAPString(String soap)
			throws XServicesException {
		try {
			requestEntity = new StringEntity(soap,ClientConstants.UTF8);
			requestEntity.setChunked(false);
			this.setContentType(ClientConstants.CONTENT_TYPE_TEXTXML);
			this.setContentEncoding(ClientConstants.UTF8);
			return sendAndReceive(requestEntity);
		} catch (UnsupportedEncodingException e) {
			throw new XServicesException(e);
		}

	}

	public void sendSOAPString(String soap) throws XServicesException {
		try {
			requestEntity = new StringEntity(soap);
			this.setContentType(ClientConstants.CONTENT_TYPE_TEXTXML);
			this.setContentEncoding(ClientConstants.UTF8);
			sendOnly(requestEntity);
		} catch (UnsupportedEncodingException e) {
			throw new XServicesException(e);
		}
	}

	private Metaset sendAndReceive(AbstractHttpEntity entity)
			throws XServicesException {
		httppost.setEntity(entity);		 
		httppost.getParams().setBooleanParameter(
				CoreProtocolPNames.USE_EXPECT_CONTINUE, false);
		ResponseHandler<String> responseHandler = new BasicResponseHandler();
		String response = null;
		try {			 
			response = httpclient.execute(httppost, responseHandler);			
			httpclient.getConnectionManager().shutdown();
			if (response != null) { 
				InputStream result = new ByteArrayInputStream(response.getBytes(ClientConstants.UTF8));
				if (result != null) {
					ISOAPBuilder soapbuilder = SOAPBuilderFactory
							.getSOAPBuilder();
					ISOAPMessage soapmessage = soapbuilder
							.readSOAPMessage(result);
					return soapmessage.getSOAPEnvelope().getBody()
							.getFirstElement();
				}
			} else {
				throw new XServicesException("no response from "
						+ httppost.getURI().getPath() + " .Pleanse check uri.");
			}
			return null;

		} catch (ClientProtocolException e) {
			throw new XServicesException(e);
		} catch (IOException e) {
			throw new XServicesException(e);
		} catch (IllegalStateException e) {
			throw new XServicesException(e);
		}
	}

	private void sendOnly(AbstractHttpEntity entity) throws XServicesException {
		httppost.setEntity(entity);
		httppost.getParams().setBooleanParameter(
				CoreProtocolPNames.USE_EXPECT_CONTINUE, false);
		try {
			httpclient.execute(httppost);
		} catch (ClientProtocolException e) {
			throw new XServicesException(e);
		} catch (IOException e) {
			throw new XServicesException(e);
		}
		httpclient.getConnectionManager().shutdown();
	}
}
