package org.act.xservices.rt.client;

import java.util.List;

import org.act.xservices.log.Logger;
import org.act.xservices.log.LoggerFactory;
import org.act.xservices.rt.soap.ISOAPEnvelope;
import org.act.xservices.rt.soap.ISOAPHeader;
import org.act.xservices.rt.soap.ISOAPHeaderBlock;
import org.act.xservices.rt.soap.factory.SOAPFactory;
import org.act.xservices.rt.xom.Metaset;
import org.act.xservices.rt.xom.QName;

/**
 * 
 * @author viking
 * @date 2010-8-6
 * @since JDK 1.5.0
 * @version 1.0.0
 * 
 *          This class is used to send and receive soap message.
 */
public class ServiceClient {
	protected static final Logger log = LoggerFactory
			.getLogger(ServiceClient.class);
	private SOAPHttpClient soapHttpClient;
	// header block
	private List<ISOAPHeaderBlock> headerBlocks;
	// each parameter name
	private String[] argNames = null;

	private String uri;
	private SOAPFactory factory = SOAPFactory.getInstance();

	public ServiceClient() {

	}

	public ServiceClient(String uri) {
		this.setURI(uri);
		this.uri = uri;
	}

	public void setURI(String uri) {
		soapHttpClient = new SOAPHttpClient(uri);
		this.uri = uri;
	}

	public void restart() {
		setURI(uri);
	}

	public void addHttpHeader(String name, String value) {
		soapHttpClient.addHttpHeader(name, value);
	}

	public void sendReceiveNonBlocking(Metaset xmlPayload) {
		ISOAPEnvelope envelope = createSOAPMessage(xmlPayload);
		String soap = envelope.toDocument();
		try {
			soapHttpClient.sendSOAPString(soap);
		} catch (XServicesException e) {
			e.printStackTrace();
		}
	}

	public void sendRobust(QName opName, Metaset xmlPayload)
			throws XServicesException {
		ISOAPEnvelope envelope = createSOAPMessage(xmlPayload);
		String soap = envelope.toDocument();
		soapHttpClient.sendSOAPString(soap);
	}

	public void sendRobust(Metaset xmlPayload) throws XServicesException {
		sendRobust(xmlPayload);
	}

	public Metaset sendReceive(Metaset xmlPayload) throws XServicesException {
		ISOAPEnvelope envelope = createSOAPMessage(xmlPayload);
		String soap = envelope.toDocument();
		return soapHttpClient.sendAndRecieveSOAPString(soap);
	}

	public void addHeaderBlock(ISOAPHeaderBlock block) {
		headerBlocks.add(block);
	}

	public void setArgNames(String[] argNames) {
		this.argNames = argNames;
	}

	public String[] getArgNames() {
		return this.argNames;
	}

	/**
	 * Set soap factory.
	 * 
	 * @param factory
	 */
	public void setSOAPFactory(SOAPFactory factory) {
		if (factory != null) {
			this.factory = factory;
		}
	}

	/**
	 * Build soap message.The default soap message is soap1.1.
	 * 
	 * @param xmlPayload
	 * @return
	 */
	private ISOAPEnvelope createSOAPMessage(Metaset xmlPayload) {
		ISOAPEnvelope envelope = factory.getDefaultEnvelope();
		ISOAPHeader header = envelope.getHeader();
		if (headerBlocks != null) {
			for (ISOAPHeaderBlock headerblock : headerBlocks) {
				header.add(headerblock);
			}
		}
		envelope.getBody().add(xmlPayload);
		return envelope;
	}
}
