package org.act.xservices.rt.xcontainer.container;

import java.io.File;
import java.util.HashMap;
import java.util.concurrent.ExecutorService;

import org.act.xservices.log.Logger;
import org.act.xservices.log.LoggerFactory;
import org.act.xservices.rt.xcontainer.deployment.DeployManager;
import org.act.xservices.rt.xcontainer.deployment.DeploymentException;
import org.act.xservices.rt.xcontainer.framework.AConnector;
import org.act.xservices.rt.xcontainer.framework.ARuntimeContainer;
import org.act.xservices.rt.xcontainer.framework.IConnector;
import org.act.xservices.rt.xcontainer.framework.config.IConfigElement;
import org.act.xservices.rt.xcontainer.framework.lifecycle.LifecycleException;
import org.act.xservices.rt.xcontainer.framework.objectpool.IObjectPool;
import org.act.xservices.rt.xcontainer.framework.objectpool.impl.ObjectBufferedPool;
import org.act.xservices.rt.xcontainer.inf.config.ContainerConfiguration;
import org.act.xservices.rt.xcontainer.log.XCLogConfig;
import org.act.xservices.rt.xcontainer.util.TimeUtil;

/**
 * Web
 * 
 * @version 1.0 2007-06-20
 * @author Lizq [lizq@act.buaa.edu.cn]
 * @since JDK1.5
 */

public class XContainer extends ARuntimeContainer {

	/**XServicesRuntimeļ· */
	private static final String CONST_CONFIGFILEPATH = "conf"+File.separator+"xservices.xml";
	
	private static final String LOGCONFIG_ELEMENT_NAME = "logConfig";
	
	//final Logger logger = LoggerFactory.getLogger(XContainer.class.getName());
	
	/**ʱ */
	private String startupTime;
	
	public DeployManager deployManager;
	


	// Container ʼ
	public void initialize() throws Exception {

		//ʱ
		startupTime = TimeUtil.getNowTime();
		
		//ȡԪ,װxcontainerԪ
		containerConfiguration = new ContainerConfiguration(CONST_CONFIGFILEPATH);
		
		//־ļ
		this.configLogger(containerConfiguration.getXcEnvironmentConfig(
				).getFirstChildWithName(LOGCONFIG_ELEMENT_NAME));
		
		final Logger logger = LoggerFactory.getLogger(XContainer.class.getName());
		logger.info("{} Version {}, {} ", new Object[] {getName(), getVersion(), getDescription()});
		logger.info( "XServices Initialize use config File: " + CONST_CONFIGFILEPATH );
		
		
		//
		this.containerEnvironment = new ContainerEnvironment();
		containerEnvironment.config(containerConfiguration.getXcEnvironmentConfig());; 
		
		//ʼ
		this.configContainerObjectPool();
		
		//ʼ
		this.configContainerConnector();
	}
	
	
	private void configContainerObjectPool() {
		IConfigElement xcObjectPoolConfig = containerConfiguration.getXcObjectPoolConfig();
		try {
			// Ԫɶ
			//TypeName -->ClassName
			HashMap<String, String> objectPoolTypeMap = new HashMap<String, String>();
			
			objectPoolMap = new HashMap<String, IObjectPool>();

			IConfigElement[] objectPoolTypeConfigList = 
				xcObjectPoolConfig.getFirstChildWithName("objectpoolTypes").getChildren();
			
			IConfigElement[] objectPoolConfigList = 
				xcObjectPoolConfig.getFirstChildWithName("objectpools").getChildren();

			//ֵ֧Ķ
			for(IConfigElement objectPoolTypeConfig : objectPoolTypeConfigList){
				objectPoolTypeMap.put(objectPoolTypeConfig.getAttribute("name"), 
						objectPoolTypeConfig.getAttribute("class"));
			}
			
			
			for(int i=0; i<objectPoolConfigList.length; i++){
				
				String objectPoolCN = objectPoolTypeMap.get(
						objectPoolConfigList[i].getAttribute("type"));
				
				IObjectPool objectPool = (IObjectPool)Class.forName(objectPoolCN).newInstance();
				
				//öأҪ
				objectPool.config(objectPoolConfigList[i]);
				objectPoolMap.put(objectPool.getName(), objectPool);
			}
			// end of Ԫɶ
		} catch (InstantiationException e) {
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			e.printStackTrace();
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	private void configContainerConnector() {
		
		IConfigElement xcConnectorsConfig = containerConfiguration.getXcConnectorsConfig();
		try {
			// Ԫ
			IConfigElement[] connectorConfigList = xcConnectorsConfig.getChildren();

			// γʼӵContainer
			for (int i = 0; i < connectorConfigList.length; i++) {
				IConnector connector = (IConnector) Class.forName(
						connectorConfigList[i].getAttribute("class")).newInstance();
				connector.initialize(connectorConfigList[i], this); //
				this.connectorList.add(connector);
			}
		} catch (InstantiationException e) {
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			e.printStackTrace();
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
		
	
	
	/**
	 * 
	 */
	public void start() throws LifecycleException{

		//---ʼ---
		try {
			deployManager = new DeployManager(this.containerEnvironment);
			deployManager.config(containerConfiguration.getDeploymentConfig());
			deployManager.start();
		} catch (DeploymentException e) {
			throw new LifecycleException(e);
		}
		final Logger logger = LoggerFactory.getLogger(XContainer.class.getName());
		//---log
    	logger.info( getName()+ " Start");
    	
		// start()
		for (int i = 0; i < connectorList.size(); i++) {
				connectorList.get(i).start();
		}
	}
	

	/** 
	 * ֹͣ
	 * @throws LifecycleException
	 */
	public void stop() throws LifecycleException{
		
		//ֹͣ
		deployManager.stop();
		
		
		//ֹͣµ
		connectorList.get(0).stop(); //stopProtocols
		
		//ر벿
		for (int i = 1; i < connectorList.size() - 1; i++) {
			// Դ
			((IObjectPool) (connectorList.get(i).getInputSource())).destroy();
			// رɷ
			((AConnector) connectorList.get(i)).stopInputDispatcher();
			// ر̳߳
			ExecutorService inputThreadpool = connectorList.get(i).getInputThreadpool();
			inputThreadpool.shutdown(); //ر̳߳أٽµ񣬵ȴ߳ɺر
		}

		// Ե
		AConnector pilotConnector = (AConnector) connectorList.get(connectorList.size() - 1);
		// Դ
		((IObjectPool) (pilotConnector.getInputSource())).destroy();
		// رɷ
		pilotConnector.stopInputDispatcher();
		// ر̳߳
		ExecutorService inputThreadpool = pilotConnector.getInputThreadpool();
		inputThreadpool.shutdown(); //ر̳߳أٽµ񣬵ȴ߳ɺر

		//ر
		for (int i = connectorList.size() - 2; i >= 0; i--) {
			// Դ
			((ObjectBufferedPool) (connectorList.get(i).getOutputSource())).destroy();
			// رɷ
			((AConnector) connectorList.get(i)).stopOutputDispatcher();
			// ر̳߳
			ExecutorService outputThreadpool = connectorList.get(i).getOutputThreadpool();
			outputThreadpool.shutdown(); //ر̳߳أٽµ񣬵ȴ߳ɺر
		}
	}
	

	/** 
	 * ٣ͷռԴ
	 */
	public void destroy() {	
		
		//øٷ
		for (int i = 0; i < connectorList.size(); i++) {
			connectorList.get(i).destroy();
		}
		
		this.containerEnvironment = null;
		this.objectPoolMap = null;
		this.connectorList = null;
	}
	
	
	public String getName() {
		return containerConfiguration.getContainerName();
	}

	public String getDescription() {
		return containerConfiguration.getDescription();
	}

	public String getVersion() {
		return containerConfiguration.getVersion();
	}

	public String getStartupTime() {
		return startupTime;
	}

	/**
	 * ļľ·
	 * @return
	 */
	public String getConfigFileAbsolutePath(){
		return System.getProperty("user.dir") + File.separator + CONST_CONFIGFILEPATH;
	}

	public DeployManager getDeployManager() {
		return deployManager;
	}
	
	
	/**
	 * ʼ־ļ
	 */
	private void configLogger(IConfigElement logConfig) {
		// ʵ־,־ʼù־
		new XCLogConfig(logConfig);
	}
	public static void main(String[] args){
		
		XContainer xContainer = new XContainer();
		try {
			xContainer.initialize();
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		Logger log = LoggerFactory.getLogger(XContainer.class.getName());
		log.error("just test if the program will run!");
	}
}
