package org.act.xservices.rt.xcontainer.log;

import java.util.HashMap;
import org.act.xservices.log.Logger;
import org.act.xservices.log.LoggerFactory;
import org.act.xservices.log.classic.Level;
import org.act.xservices.log.classic.LoggerContext;
import org.act.xservices.log.classic.encoder.PatternLayoutEncoder;
import org.act.xservices.log.classic.spi.ILoggingEvent;
import org.act.xservices.log.core.FileAppender;
import org.act.xservices.log.core.status.InfoStatus;
import org.act.xservices.log.core.status.StatusManager;
import org.act.xservices.rt.xcontainer.framework.config.IConfigElement;

/**
 * ־
 * 
 * @LastModifyTime 2010-7-02
 * @CreationTime
 * @since JDK1.5
 */
public class XCLogConfig {

	private final String LOG_LEVEL_ALL = "all";
	private final String LOG_LEVEL_OFF = "off";
	private final String LOG_LEVEL_TRACE = "trace";
	private final String LOG_LEVEL_DEBUG = "debug";
	private final String LOG_LEVEL_INFO = "info";
	private final String LOG_LEVEL_WARN = "warn";
	private final String LOG_LEVEL_ERROR = "error";

	private final String LOG_CONFIG_KEY_ROOTLOGLEVEL = "rootLoggerLevel";
	private final String LOG_CONFIG_KEY_PATTERN = "pattern";
	private final String LOG_CONFIG_KEY_FILE = "file";

	private HashMap<String, String> configProperties = new HashMap<String, String>();

	public XCLogConfig(IConfigElement logConfig) {
		// ȡ־ò
		if (logConfig != null) {
			IConfigElement[] parameters = logConfig
					.getChildrenWithName("parameter");
			for (IConfigElement p : parameters) {
				configProperties.put(p.getAttribute("name"), p
						.getAttribute("value"));
			}
		}

		this.logConfig();
	}

	/**
	 * ־
	 */
	private void logConfig() {
		LoggerContext lc = (LoggerContext) LoggerFactory.getILoggerFactory();
		configure(lc);
		Logger log = LoggerFactory.getLogger(XCLogConfig.class.getName());
	}

	public void configure(LoggerContext lc) {
		StatusManager sm = lc.getStatusManager();
		if (sm != null) {
			sm.add(new InfoStatus("Setting up a new configuration.", lc));
		}
		org.act.xservices.log.classic.Logger rootLogger = lc
				.getLogger(org.act.xservices.log.classic.Logger.ROOT_LOGGER_NAME);

		String rll = configProperties.get(LOG_CONFIG_KEY_ROOTLOGLEVEL);
		String pattern = configProperties.get(LOG_CONFIG_KEY_PATTERN);
		String file = configProperties.get(LOG_CONFIG_KEY_FILE);
		if (LOG_LEVEL_ALL.equals(rll)) {
			rootLogger.setLevel(Level.ALL);
		} else if (LOG_LEVEL_TRACE.equalsIgnoreCase(rll)) {
			rootLogger.setLevel(Level.TRACE);

		} else if (LOG_LEVEL_DEBUG.equalsIgnoreCase(rll)) {
			rootLogger.setLevel(Level.DEBUG);

		} else if (LOG_LEVEL_INFO.equalsIgnoreCase(rll)) {
			rootLogger.setLevel(Level.INFO);

		} else if (LOG_LEVEL_WARN.equalsIgnoreCase(rll)) {
			rootLogger.setLevel(Level.WARN);

		} else if (LOG_LEVEL_ERROR.equalsIgnoreCase(rll)) {
			rootLogger.setLevel(Level.ERROR);

		} else {
			rootLogger.setLevel(Level.OFF);
		}
		FileAppender<ILoggingEvent> fa = new FileAppender<ILoggingEvent>();
		fa.setFile(file);
		fa.setContext(lc);
		fa.setName("file");
		PatternLayoutEncoder pl = new PatternLayoutEncoder();
		pl.setContext(lc);
		pl.setPattern(pattern);
		pl.start();
		fa.setEncoder(pl);
		fa.start();
		rootLogger.addAppender(fa);
	}
}
