package org.act.xservices.rt.xcontainer.management;

import java.util.List;

import org.act.xservices.rt.xcontainer.framework.IRuntimeContainer;
import org.act.xservices.rt.xcontainer.framework.transport.ITransportReceiver;

/**
 * HTTP
 * @version 1.0 2008-6-16
 * @author Lizq [lizq@act.buaa.edu.cn]
 * @since JDK1.5
 */
public class HTTPTransportManagement extends TransportManagement{
	
	
	private ITransportReceiver httpTransportReceiver;
	
	private boolean available = false;
	
	public HTTPTransportManagement(IRuntimeContainer wsRuntimeContainer){
		super(wsRuntimeContainer);
		httpTransportReceiver = getHTTPTransportDispatchSource();
		if(httpTransportReceiver != null)
			available = true;
	}
	

	
	private ITransportReceiver getHTTPTransportDispatchSource(){
		
		List<ITransportReceiver> httpSourceList = getTransportReceiverByName("http");
		if(httpSourceList != null){
			return httpSourceList.get(0);
		}
		return null;
	}
	
	
	/**
	 * жϴHTTPǷ
	 * @return
	 */
	public boolean isAvailable(){
		return available;
	}
	
	/**
	 * ôɷԴĶ˿ں
	 * @return
	 */
	public int getListenPort(){
    	if(!isAvailable())
    		return -1;
        return httpTransportReceiver.getListenPort();
	}
	
	//----------------------ͳϢ-----------------------
    /**
     * һ˶ٸ
     * @return
     */
    public long getAcceptedSocketCount() {
    	if(!isAvailable())
    		return -1;
        return httpTransportReceiver.getAcceptedSocketCount();
    }
    
	
	/**
	 * ʾͳƹǷ
	 * @return
	 */
	public boolean isThroughputStatisticAvailable(){
    	if(!isAvailable())
    		return false;
		return httpTransportReceiver.isThroughputStatisticAvailable();
	}
	

    /**
     * Get average receive speed, in byte/s.
     * ƽٶ
     * 
     * @return average receive speed,or -1 means statistic Function Unavailable.
     */
    public double getAvgReceiveSpeed() {
    	if(!isAvailable())
    		return -1;
        return httpTransportReceiver.getAvgReceiveSpeed();
    }

    /**
     * Get average send speed, in byte/s.
     * ƽٶ
     * 
     * @return average send speed,or -1 means statistic Function Unavailable.
     */
    public double getAvgSendSpeed() {
    	if(!isAvailable())
    		return -1;
        return httpTransportReceiver.getAvgSendSpeed();
    }
    
    /**
     * Get instantaneous receive speed, in byte/s.
     * ʵʱĽٶ
     * 
     * @return received speed,or -1 means statistic Function Unavailable.
     */
    public double getReceiveSpeed() {
    	if(!isAvailable())
    		return -1;
        return httpTransportReceiver.getReceiveSpeed();
    }

    /**
     * Get instantaneous send speed, in byte/s.
     * ʵʱķٶ
     * 
     * @return send speed,or -1 means statistic Function Unavailable.
     */
    public double getSendSpeed() {
    	if(!isAvailable())
    		return -1;
        return httpTransportReceiver.getSendSpeed();
    }
    
    

    /**
     * Get elapsed time after session started and before session closed, in ms.
     * ʱ
     * 
     * @return elapsed time,or -1 means statistic Function Unavailable.
     */
    public long getRunningTimeInMillis() {
    	if(!isAvailable())
    		return -1;
        return httpTransportReceiver.getRunningTimeInMillis();
    }

    /**
     * Get received bytes count. If session restart, the value will reset to 0.
     * ֽܹܵ
     * 
     * @return received bytes count,or -1 means statistic Function Unavailable.
     */
    public long getReceivedBytes() {
    	if(!isAvailable())
    		return -1;
        return httpTransportReceiver.getReceivedBytes();
    }

    /**
     * Get sent bytes count. If session restart, the value will reset to 0.
     * ֽܹ
     * 
     * @return sent bytes count,or -1 means statistic Function Unavailable.
     */
    public long getSentBytes() {
    	if(!isAvailable())
    		return -1;
        return httpTransportReceiver.getSentBytes();
    }



	public double getAvgMessageReceiveSpeed() {
    	if(!isAvailable())
    		return -1;
        return httpTransportReceiver.getAvgMessageReceiveSpeed();
	}



	public double getAvgMessageSendSpeed() {
    	if(!isAvailable())
    		return -1;
        return httpTransportReceiver.getAvgMessageSendSpeed();
	}



	public long getLargestManagedSocketCount() {
    	if(!isAvailable())
    		return -1;
        return httpTransportReceiver.getLargestManagedSocketCount();
	}



	public double getLargestMessageReceiveSpeed() {
    	if(!isAvailable())
    		return -1;
        return httpTransportReceiver.getLargestMessageReceiveSpeed();
	}



	public double getLargestMessageSendSpeed() {
    	if(!isAvailable())
    		return -1;
        return httpTransportReceiver.getLargestMessageSendSpeed();
	}



	public double getLargestReceiveSpeed() {
    	if(!isAvailable())
    		return -1;
        return httpTransportReceiver.getLargestReceiveSpeed();
	}



	public double getLargestSendSpeed() {
    	if(!isAvailable())
    		return -1;
        return httpTransportReceiver.getLargestSendSpeed();
	}


	//TODO long to int ?
	public int getManagedSocketCount() {
    	if(!isAvailable())
    		return -1;
        return (int)httpTransportReceiver.getManagedSocketCount();
	}



	public double getMessageReceiveSpeed() {
    	if(!isAvailable())
    		return -1;
        return httpTransportReceiver.getMessageReceiveSpeed();
	}



	public long getReceivedMessages() {
    	if(!isAvailable())
    		return -1;
        return httpTransportReceiver.getReceivedMessages();
	}



	public double getSendMessageSpeed() {
    	if(!isAvailable())
    		return -1;
        return httpTransportReceiver.getSendMessageSpeed();
	}



	public long getSentMessages() {
    	if(!isAvailable())
    		return -1;
        return httpTransportReceiver.getSentMessages();
	}



	public long getStartTimeInMillis() {
    	if(!isAvailable())
    		return -1;
        return httpTransportReceiver.getStartTimeInMillis();
	}



	public int getThroughputCalculationInterval() {
    	if(!isAvailable())
    		return -1;
        return httpTransportReceiver.getThroughputCalculationInterval();
	}



	public void setThroughputCalculationInterval(int throughputCalculationInterval) {
    	if(!isAvailable())
    		return ;
       httpTransportReceiver.setThroughputCalculationInterval(throughputCalculationInterval);
		
	}

	
}
