package org.act.xservices.rt.xcontainer.deployment;

import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import org.act.xservices.log.Logger;
import org.act.xservices.log.LoggerFactory;
import org.act.xservices.rt.xcontainer.deployment.util.FileUtil;
import org.act.xservices.rt.xcontainer.framework.IContainerEnvironment;
import org.act.xservices.rt.xcontainer.framework.config.IConfigElement;
import org.act.xservices.rt.xcontainer.framework.util.JavaUtils;



public abstract class ADeployMonitor implements Runnable{

	final Logger logger = LoggerFactory.getLogger(ADeployMonitor.class);
	
	/** Ƿֹͣ */
	protected boolean isStop = false;
	
	protected DeployManager deployManager;

	/** Ĭϸ¼ */
	protected int deployerUpdateInterval = 30000;
    
    /** Support for hot update is controlled by this flag */
	protected boolean isHotUpdate = true;

    /** Support for hot deployment is controlled by this flag */
	protected boolean isHotDeployment = true;
	
	/**  */
	protected IContainerEnvironment containerEnv; 
	
	public void init(DeployManager deployManager, IConfigElement deployMonitorConfig){
		
		this.deployManager = deployManager;
		this.containerEnv = deployManager.getContainerEnv();
		
		//DeployMonitorϢ뵽ӳ
		IConfigElement[] deployConfigParameters = 
			deployMonitorConfig.getChildrenWithName("parameter");
		Map<String,String> parameterMap = new HashMap<String,String>();
		for (int i = 0; i < deployConfigParameters.length; i++) {
			parameterMap.put(deployConfigParameters[i].getAttribute("name"),
					deployConfigParameters[i].getValue());
		}
		
		//ò
		String deployUpIn = (String) parameterMap.get("deployUpdateInterval");
		if(deployUpIn != null){
			deployerUpdateInterval = Integer.parseInt(deployUpIn);
		}
		isHotDeployment = JavaUtils.isTrue(parameterMap.get("hotDeployment"), true);
		isHotUpdate = JavaUtils.isTrue(parameterMap.get("hotUpdate"), true);
	}
	
	//ֹͣ
	public void stop() {
		this.isStop = true;
	}
	
	
	
	//-------------
    /**
     * Checks if a file has been modified by comparing the last update date of
     * both files and WSInfo. If they are different, the file is assumed to have
     * been modified.
     *
     * @param file
     * @param wsInfo
     */
    protected boolean isModified(File file, WSInfo wsInfo) {
        long currentTimeStamp = wsInfo.getLastModifiedDate();
        setLastModifiedDate(file, wsInfo);
        return (currentTimeStamp != wsInfo.getLastModifiedDate());
    }

    /**
     * Obtains the newest (as compared with timestamp stored in wsInfo)
     * timestamp and stores it in WSInfo. 
     */
    protected void setLastModifiedDate(File file, WSInfo wsInfo) {
        if (file.isDirectory()) {
            File files [] = file.listFiles();
            for (int i = 0; i < files.length; i++) {
                File fileItem = files[i];
                if (fileItem.isDirectory()) {
                    setLastModifiedDate(fileItem, wsInfo);
                }
                else if(wsInfo.getLastModifiedDate() < fileItem.lastModified()) {
                    wsInfo.setLastModifiedDate(fileItem.lastModified());
                }
            }
        }
        else if(wsInfo.getLastModifiedDate() < file.lastModified()) {
            wsInfo.setLastModifiedDate(file.lastModified());
        }
    }
    
    
    protected void deleteOldWorkingFile(File servicesWorkingDir) {
		// ɾWorkingĿ¼µļ
		for (File oldWorkingFile : servicesWorkingDir.listFiles()) {
			try {
				FileUtil.deleteRecursive(oldWorkingFile);
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	}
	
	
	
	
	
	
}
