package org.act.xservices.rt.xcontainer.deployment;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import org.act.xservices.log.Logger;
import org.act.xservices.log.LoggerFactory;
import org.act.xservices.rt.xcontainer.deployment.deployer.IDeployer;
import org.act.xservices.rt.xcontainer.deployment.util.DeployUtil;
import org.act.xservices.rt.xcontainer.deployment.util.DeploymentConstants;
import org.act.xservices.rt.xcontainer.framework.IContainerEnvironment;
import org.act.xservices.rt.xcontainer.framework.config.IConfigElement;
import org.act.xservices.rt.xcontainer.framework.config.IConfigurable;
import org.act.xservices.rt.xcontainer.framework.policy.IPolicyConfiger;
import org.act.xservices.rt.xcontainer.framework.util.JavaUtils;

/**
 * @LastModifyTime 2009-10-05
 * @CreationTime 2005-6-23
 * @author Lizq [lizq@act.buaa.edu.cn]
 * @since JDK1.5
 */

public class DeployManager implements IConfigurable{
	
	final Logger logger = LoggerFactory.getLogger(DeployManager.class);
	
	//ws
	public Thread wsMonitorThread; //¶ⲿṩжϹ
	private ADeployMonitor wsDeployMonitor;
	
	//bpel
	public Thread bpelMonitorThread;
	private ADeployMonitor bpelDeployMonitor;
	
	//aspect
	public Thread aspectMonitorThread;
	private ADeployMonitor aspectDeployMonitor;

	
	/**  */
	private IContainerEnvironment containerEnv;
	

	//--Щ·ϢDeployManagerгʼҲӦⲿ޸---
	private File deploy_Dir = null;
	private File working_Dir = null;
    
	private File wsservices_Deploy_Dir = null;
	private File bpelservices_Deploy_Dir = null;
	private File modules_Deploy_Dir = null;
	private File aspect_Deploy_Dir = null;
    
	private File wsservices_Working_Dir = null;
	private File bpelservices_Working_Dir = null;
	private File modules_Working_Dir = null;
	private File aspect_Working_Dir = null;
	

    //---end of ·Ϣ---
    
	//---Classloader---
    protected ClassLoader systemDeployClassLoader;
    protected ClassLoader wsserviceCommonClassLoader;
    protected ClassLoader moduleCommonClassLoader;
	
    /** 
     * to keep map of which deployer can process which file extension
     * for example WSServiceDeployer will process .wsar or .zip file
     */
    private HashMap<String, IDeployer> extensionToDeployerMap = new HashMap<String, IDeployer>();
    
    
    
	public DeployManager(IContainerEnvironment containerEnv){
		this.containerEnv = containerEnv;
	}
	
    public IContainerEnvironment getContainerEnv() {
		return containerEnv;
	}
	
    
	/** 
	 * ò
	 * @param deploymentConfig
	 * @throws DeploymentException
	 */
	public void config(IConfigElement deploymentConfig) throws DeploymentException{
		
		//ò͹·
		prepareDeploy();
        
        //
        setClassLoaders(deploy_Dir);
		
		// WSDeployMonitor
		IConfigElement wsDeployMonitorConf = deploymentConfig.getFirstChildWithName("wsDeployMonitor");
		boolean enable_ws = JavaUtils.isTrue(wsDeployMonitorConf.getAttribute("enable"), false);
		if(enable_ws){
			try {
				String wsMonitorClassName = wsDeployMonitorConf.getAttribute("class");
				if (wsMonitorClassName != null) {
					wsDeployMonitor = (ADeployMonitor) Class.forName(wsMonitorClassName).newInstance();
					wsDeployMonitor.init(this, wsDeployMonitorConf);
					logger.info("Succeed add WS DeployMonitor ");
				}
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		
		// BPELDeployMonitor
		IConfigElement bpelDeployMonitorConf = deploymentConfig.getFirstChildWithName("bpelDeployMonitor");
		boolean enable_bpel = JavaUtils.isTrue(bpelDeployMonitorConf.getAttribute("enable"), false);
		if(enable_bpel){
			try {
				String bpelMonitorClassName = bpelDeployMonitorConf.getAttribute("class");
				if (bpelMonitorClassName != null) {
					bpelDeployMonitor = (ADeployMonitor) Class.forName(bpelMonitorClassName).newInstance();
					bpelDeployMonitor.init(this, bpelDeployMonitorConf);
					logger.info("Succeed add BPEL DeployMonitor ");
				}
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		
		//AspectDeployMonitor
		IConfigElement aspectDeployMonitorConf = deploymentConfig.getFirstChildWithName("aspectDeployMonitor");
		boolean enable_aspect = JavaUtils.isTrue(aspectDeployMonitorConf.getAttribute("enable"),false);
		if(enable_aspect){
			try {
				String aspectMonitorClassName = aspectDeployMonitorConf.getAttribute("class");
				if(aspectMonitorClassName != null){
					aspectDeployMonitor = (ADeployMonitor) Class.forName(aspectMonitorClassName).newInstance();
					aspectDeployMonitor.init(this, aspectDeployMonitorConf);
					logger.info("Succeed add Aspect DeployMonitor ");
					
				}
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		
		// Deployer
		configDeployer(deploymentConfig);
		
	}
	
	private void configDeployer(IConfigElement deploymentConfig){
		
		//IPolicyConfiger
		List<IPolicyConfiger> pcList = new ArrayList<IPolicyConfiger>();
		IConfigElement policyConfigersConfig = deploymentConfig.getFirstChildWithName("policyConfigers");
		for(IConfigElement pcConfig : policyConfigersConfig.getChildren()){
			boolean enable_PC = JavaUtils.isTrue(pcConfig.getAttribute("enable"), false);
			if(enable_PC){
				try{
					String name = pcConfig.getAttribute("name");
				    String pcClassName = pcConfig.getAttribute("class");
				    
				    if(name != null && pcClassName!=null){
				    	
				    	IPolicyConfiger pc = (IPolicyConfiger) Class.forName(pcClassName).newInstance();
				    	pc.init(name, pcConfig); //ʼ
				    	pcList.add(pc);
				    	
				    	logger.info("Succeed add PolicyConfiger " + name);
				    }
				} catch(Exception e){
					e.printStackTrace();
				}
			}
		}
		
		// Deployer
		IConfigElement[] deployerConfigs = deploymentConfig.getChildrenWithName("deployer");
		for (IConfigElement deployerConfig : deployerConfigs) {
			boolean enable_DP = JavaUtils.isTrue(deployerConfig.getAttribute("enable"), false);
			if(enable_DP){
				try{
					String extension = deployerConfig.getAttribute("extension");
				    String deployerClassName = deployerConfig.getAttribute("class");
				    
				    if(extension != null && deployerClassName!=null){
				    	 IDeployer deployer = (IDeployer)Class.forName(deployerClassName).newInstance();
				    	 deployer.init(extension, this.containerEnv, pcList.toArray(new IPolicyConfiger[0])); //ʼ
				    	 
				    	 extensionToDeployerMap.put(deployer.getExtension(), deployer); //ӵӳ
				    	 
				    	 logger.info("Succeed add Deployer for Extension " + extension);
				    }
				} catch(Exception e){
					e.printStackTrace();
				}
			}
		}
	}

	
	/**
	 * 
	 * @throws DeploymentException
	 */
	public void start() throws DeploymentException{

		//һµ߳̽в
        if(wsDeployMonitor != null){
        	wsMonitorThread = new Thread(wsDeployMonitor);
        	wsMonitorThread.setName("WSDeployMonitor_Thread");
    		wsMonitorThread.start();
        }
        
        if(bpelDeployMonitor != null){
        	bpelMonitorThread = new Thread(bpelDeployMonitor);
        	bpelMonitorThread.setName("BPELDeployMonitor_Thread");
        	bpelMonitorThread.start();
        }
        
        if(aspectDeployMonitor != null){
        	aspectMonitorThread = new Thread(aspectDeployMonitor);
        	aspectMonitorThread.setName("AspectDeployMonitor_Thread");
        	aspectMonitorThread.start();
        }
	}
	
	/**
	 * ֹͣ
	 */
	public void stop() {
		if(wsDeployMonitor != null){
			wsDeployMonitor.stop();
		}
		
		if(bpelDeployMonitor != null){
			bpelDeployMonitor.stop();
		}
		
		if(aspectDeployMonitor != null){
			aspectDeployMonitor.stop();
		}
	}

	
	/**
	 * ø·
	 * @throws DeploymentException
	 */
	private void prepareDeploy() throws DeploymentException {
		
		String deploy_Path = containerEnv.getRuntimePath() + File.separator
				+ DeploymentConstants.Config_Deploy_Directory;

		String working_Path = containerEnv.getRuntimePath() + File.separator
				+ DeploymentConstants.Config_Work_Directory;
		//鲿·Ƿ
		File xcDeploy = new File(deploy_Path);
		if ( !xcDeploy.exists() ) {
			throw new DeploymentException("Can not find Deploy Files!");
		}
		deploy_Dir = new File(deploy_Path);
		
		//鹤·Ƿ
		working_Dir = new File(working_Path);
		if( !working_Dir.exists() ){
			working_Dir.mkdir();
		}

		// File(String parent, String child) 
		wsservices_Deploy_Dir = new File(deploy_Dir, DeploymentConstants.Config_Deploy_WSService_Path);
		bpelservices_Deploy_Dir = new File(deploy_Dir, DeploymentConstants.Config_Deploy_BPELService_Path);
		modules_Deploy_Dir = new File(deploy_Dir,DeploymentConstants.Config_Deploy_Modules_Path);
		aspect_Deploy_Dir = new File(deploy_Dir,DeploymentConstants.Config_Deploy_Aspect_Path);
		
		//Ŀ¼
	    wsservices_Working_Dir = new File(working_Dir, DeploymentConstants.Config_Deploy_WSService_Path);
		if( !wsservices_Working_Dir.exists() ){
			wsservices_Working_Dir.mkdir();
		}
		
	    bpelservices_Working_Dir = new File(working_Dir, DeploymentConstants.Config_Deploy_BPELService_Path);
		if( !bpelservices_Working_Dir.exists() ){
			bpelservices_Working_Dir.mkdir();
		}

	    modules_Working_Dir = new File(working_Dir,DeploymentConstants.Config_Deploy_Modules_Path);
		if( !modules_Working_Dir.exists() ){
			modules_Working_Dir.mkdir();
		}
		
		aspect_Working_Dir = new File(working_Dir,DeploymentConstants.Config_Deploy_Aspect_Path);
		if( !aspect_Working_Dir.exists()){
			aspect_Working_Dir.mkdir();
		}
		
		containerEnv.setAttribute(DeploymentConstants.DIR_DEPLOY, deploy_Dir);
		containerEnv.setAttribute(DeploymentConstants.DIR_WORKING, working_Dir);
		
		containerEnv.setAttribute(DeploymentConstants.DIR_DEPLOY_WSSERVICES, wsservices_Deploy_Dir);
		containerEnv.setAttribute(DeploymentConstants.DIR_DEPLOY_BPELSERVICES, bpelservices_Deploy_Dir);
		containerEnv.setAttribute(DeploymentConstants.DIR_DEPLOY_MODULES, modules_Deploy_Dir);
		containerEnv.setAttribute(DeploymentConstants.DIR_DEPLOY_ASPECTS,aspect_Deploy_Dir);
		
		containerEnv.setAttribute(DeploymentConstants.DIR_WORKING_WSSERVICES, wsservices_Working_Dir);
		containerEnv.setAttribute(DeploymentConstants.DIR_WORKING_BPELSERVICES, bpelservices_Working_Dir);
		containerEnv.setAttribute(DeploymentConstants.DIR_WORKING_MODULES, modules_Working_Dir);
		containerEnv.setAttribute(DeploymentConstants.DIR_WORKING_ASPECTS, aspect_Working_Dir);
	}
	
	
    /**
     * ClassLoader
     * To set the all the classLoader hierarchy this method can be used , 
     * the top most parent is SCL(system Class Loader)
     * 
     * SCL
     * :  :
     * MCCL  SCCL
     * :      : 
     * MCL    SCL
     * <p/>
     * <p/>
     * MCCL :  module common class loader
     * SCCL : Service common class loader
     * MCL : module class loader
     * SCL : Service class loader
     *
     * @param deployDir : The repository folder of XServices
     * @throws DeploymentException if there's a problem
     */
	private void setClassLoaders(File deployDir) throws DeploymentException {
        
		systemDeployClassLoader = DeployUtil.getClassLoader(this.getClass().getClassLoader(), deployDir);	
        
        if (wsservices_Deploy_Dir.exists()) {
        	wsserviceCommonClassLoader = 
        			DeployUtil.getClassLoader(systemDeployClassLoader, wsservices_Deploy_Dir);
        } else {
        	wsserviceCommonClassLoader = systemDeployClassLoader;
        }
        
        if (modules_Deploy_Dir.exists()) {
        	moduleCommonClassLoader = 
            		DeployUtil.getClassLoader(systemDeployClassLoader, modules_Deploy_Dir);
        } else {
        	moduleCommonClassLoader = systemDeployClassLoader;
        }
        
        containerEnv.setAttribute(DeploymentConstants.CLASSLOADER_SYSTEM_DEPLOY, systemDeployClassLoader);
		containerEnv.setAttribute(DeploymentConstants.CLASSLOADER_WSSERVICE_COMMON, wsserviceCommonClassLoader);
		containerEnv.setAttribute(DeploymentConstants.CLASSLOADER_MODULE_COMMON, moduleCommonClassLoader);
    }
	
    /**
     * ô˺׺ļӦDeployer
     * @param extension
     * @return
     */
    public IDeployer getDeployerForExtension(String extension) {
        return extensionToDeployerMap.get(extension);
    }
    
}