package org.act.xservices.rt.xcontainer.deployment;

import java.io.File;
import java.io.FileFilter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.act.xservices.log.Logger;
import org.act.xservices.log.LoggerFactory;
import org.act.xservices.rt.xcontainer.deployment.deployer.IDeployer;
import org.act.xservices.rt.xcontainer.deployment.util.DeployUtil;
import org.act.xservices.rt.xcontainer.deployment.util.DeploymentConstants;
import org.act.xservices.rt.xcontainer.deployment.util.FileUtil;
import org.act.xservices.rt.xcontainer.framework.IContainerEnvironment;
import org.act.xservices.rt.xcontainer.framework.config.IConfigElement;

/**
 * Web
 * ԼWebĿ¼µķı仯,DeployManagerҪͷķ
 * @LastModifyTime 2009-09-30
 * @CreationTime 2005-6-23
 * @author liujian [liujian@act.buaa.edu.cn]
 * @author Lizq [lizq@act.buaa.edu.cn]
 * @since JDK1.5
 */
public class WSDeployMonitor extends ADeployMonitor {
	
	final Logger logger = LoggerFactory.getLogger(WSDeployMonitor.class);
	
	
    /** This is to store all the jar files in a specified folder */
    private List<String> jarList = new ArrayList<String>();
    
    /** All the currently updated jars */
    public Map<String, WSInfo> currentDeployedJars = new HashMap<String, WSInfo>();

    /** Stores all the web Services to deploy. */
    protected List<WSInfo> wsToDeploy = new ArrayList<WSInfo>();

    /** Stores all the web Services to undeploy. */
    protected List<WSInfo> wsToUnDeploy = new ArrayList<WSInfo>();
    
	/** ļѡȡ׺Ϊwsarzipļ */
	private FileFilter serviceFileFilter = new FileFilter() {
		
		public boolean accept(File file) {	
			if(file.isDirectory())
				return false;
			
			String fn = file.getName();
			return ((fn.endsWith(".wsar")) || (fn.endsWith(".zip")));
		}
	};
	
	//---
	protected File wsservices_Deploy_Dir;
	protected File wsservices_Working_Dir;
	

	public void init(DeployManager deployManager, IConfigElement deployMonitorConfig){
		
		super.init(deployManager, deployMonitorConfig);
		
		this.wsservices_Deploy_Dir = (File) 
			containerEnv.getAttribute(DeploymentConstants.DIR_DEPLOY_WSSERVICES);
		this.wsservices_Working_Dir = (File) 
			containerEnv.getAttribute(DeploymentConstants.DIR_WORKING_WSSERVICES);
	}
	
	
	
	/**
	 * WSprocessorǷ񱻳ʼûгʼWSProcessor޷WSĲ
	 * true--ʼfalse--δʼ
	 * @return
	 */
	private boolean checkWSProcessor(){
		return !(containerEnv.getAttribute(IContainerEnvironment.WS_PROCESSOR) == null);
	}
    
	//
	public void run() {
		
		//WSProcessorδʼֹWS
		if( !checkWSProcessor() ){
			logger.info("WSProcessor may not be initialized, stop monitor WS Services!");
			return ;
		}
		
		//ɾWorkingµоļ
		deleteOldWorkingFile(this.wsservices_Working_Dir);
		
		//ʱ
		while (!isStop) {
			logger.info("Start to deploy web service File ");

			checkServices();

			//ûпͨȲܣڼµķ
			if( !isHotDeployment )
				break;
			
			//һʱ
			try {
				Thread.sleep(deployerUpdateInterval);
			} catch (InterruptedException e) {
				logger.info("WSDeployMonitor interrupt." );
			}
			
		}//end of while
	}//end of run
	
	
    /** Finds a list of services in the folder */
	public void checkServices() {
		synchronized (this) {
			// 鿴Ŀ¼µķʶҪºͲķ
			findServicesInDirectory();

			// ʶѱɾҪķ
			checkForUndeployedServices();
			
			this.unDeploy();
			this.doDeploy();
		}
	}
	
    
    /**
	 * Searches a given folder for wsar or zip files and 
	 * adds them to the list wsToDeploy or wsToUnDeploy.
	 */
    protected void findServicesInDirectory() {
    	
        File root = this.wsservices_Deploy_Dir;
        File[] files = root.listFiles(serviceFileFilter);

        if (files != null && files.length > 0) {
        	
        	for (int i = 0; i < files.length; i++) {
            	File file = files[i];
            	
            	String ext = DeployUtil.getFileExtension(file.getName());
            	IDeployer deployer = deployManager.getDeployerForExtension(ext);
                
            	// If we found a deployer for this type of file, use it.  
                // Otherwise ignore the file.
                if (deployer != null) {
                	if("wsar".equalsIgnoreCase(ext) || "zip".equalsIgnoreCase(ext))
                		addFileToDeploy(file, deployer, WSInfo.TYPE_WSSERVICE);
                }else{
                	logger.info("Can not find Deployer for file type: "+ ext);
                	logger.info("Give up deploy file: "+ file.getName());
                }

            }
        }
    }
	
    
    /**
     * First checks whether the file is already available by the
     * system call fileExists. If it is not deployed yet then adds to the jarList
     * and to the deployment engine as a new service or module.
     * While adding new item to jarList, first creates the WSInfo object and
     * then adds to the jarlist and actual jar file is added to DeploymentEngine.
     * <p/>
     * If the files already exists, then checks whether it has been updated
     * then changes the last update date of the wsInfo and adds two entries to
     * DeploymentEngine - one for new deployment and other for undeployment.
     *
     * @param file actual jar files for either Module or service
     */
    public synchronized void addFileToDeploy(File file, IDeployer deployer , int type) {
        
    	WSInfo info = currentDeployedJars.get(file.getName());
    	
        if (info != null) {
        	//˷֮ǰ
            if (isHotUpdate && isModified(file, info)) {
            	//ȸ¹,add entry to undeploy list
            	this.wsToUnDeploy.add(info);
                
                WSInfo wsInfo = new WSInfo(file, file.getName(), file.lastModified(), deployer, type);
                
                // add entry to deploylist 
                this.wsToDeploy.add(wsInfo);
                
                currentDeployedJars.remove(info.getFileID());
                currentDeployedJars.put(wsInfo.getFileID(), wsInfo);

            }
        } else {
        	//˷֮ǰ
            info = new WSInfo(file, file.getName(), file.lastModified(), deployer ,type);
            currentDeployedJars.put(file.getName(), info);
            this.wsToDeploy.add(info);
        }
        
        //Ŀ¼еķƶӵб
        jarList.add(info.getFileID());
    }
    
    
    /**
     * Checks undeployed Services. Checks old jars files and current jars.
     * If name of the old jar file does not exist in the current jar
     * list then it is assumed that the jar file has been removed
     * and that is hot undeployment.
     */
    private synchronized void checkForUndeployedServices() {

        Iterator<String> infoItems = currentDeployedJars.keySet().iterator();
        
        List<String> tobeRemoved = new ArrayList<String>();
        
        while (infoItems.hasNext()) {
            String  fileName = infoItems.next();
            WSInfo infoItem = currentDeployedJars.get(fileName);
            if (infoItem.getType() == WSInfo.TYPE_MODULE) {
                continue;
            }
            //seems like someone has deleted the file , so need to undeploy
            boolean found = false;
            //ҴļǷɾ
            for (int i = 0; i < jarList.size(); i++) {
                String s = jarList.get(i);
                if(fileName.equals(s)){
                    found = true;
                    break;
                }
            }
            
            if(!found){ //˷ļɾҪз
                tobeRemoved.add(fileName);
                this.wsToUnDeploy.add(infoItem);
            }
        }

        for (int i = 0; i < tobeRemoved.size(); i++) {
            String fileName = tobeRemoved.get(i);
            currentDeployedJars.remove(fileName);
        }
        
        tobeRemoved.clear();
        
        jarList.clear(); //importent
    }
    
    
    /**
     * беļ丽Deployerв
     */
    public void doDeploy() {
		try {
			if (wsToDeploy.size() > 0) {
				for (WSInfo fileToDeploy : wsToDeploy) {
					
					long deployStart = System.currentTimeMillis();

					try {
						//WorkingĿ¼µĽѹλ
						String wsarLoc = this.wsservices_Working_Dir.getAbsolutePath()
								+ File.separator + fileToDeploy.getFileID();
						//wsarļѹĿ¼
						FileUtil.inflate(fileToDeploy.getFile(), wsarLoc);

						//ô˷WorkingĿ¼µ·
						fileToDeploy.setServiceWorkingLocation(wsarLoc);
						
						//ݹĿ¼ô˷
						fileToDeploy.setClassLoader( deployManager.wsserviceCommonClassLoader, 
								new File(wsarLoc));

						IDeployer deployer = fileToDeploy.getDeployer();

						if (deployer != null) {
							deployer.deploy(fileToDeploy);
							
							//ɹ
							long deployEnd = System.currentTimeMillis();
							logger.info("Succeed Deployed Web Service " 
									+ fileToDeploy.getFileID()
									+" in " + (deployEnd - deployStart) + " milliseconds");
						}
					} catch (IOException e) {
						//ѹгִ
						logger.info("Error to Unzip wsar File "
										+ fileToDeploy.getFileID() + " for "
										+ e.toString());
						continue; //WSARļ

					} catch (DeploymentException e) {
						//гִ
						logger.info("Error to Deploy wsar File "
										+ fileToDeploy.getFileID() + " for "
										+ e.getMessage());
						continue; //WSARļ
					}

				}//end of for
			}
		} finally {
			wsToDeploy.clear();
		}
	}

	/**
	 * беļ丽Deployerз
	 */
	public void unDeploy() {
		try {
			for (int i = 0; i < wsToUnDeploy.size(); i++) {

				WSInfo wsInfo = wsToUnDeploy.get(i);
				try {
					File oldWsarWorkFile = 
						new File(this.wsservices_Working_Dir, wsInfo.getFileID());
					if (oldWsarWorkFile.exists()) {
						// ɾWorkingĿ¼µǰĲļ
						FileUtil.deleteRecursive(oldWsarWorkFile);
					}

					IDeployer deployer = wsInfo.getDeployer();
					if (deployer != null) {
						deployer.unDeploy(wsInfo.getFileID());

						logger.info("UnDeployed Web Service " + wsInfo.getFileID());
					}
				} catch (Exception e) {
					logger.info("Error to UnDeployed  Web Service "
							+ wsInfo.getFileID() + " for " + e.toString());
					continue;
				}
			}//end of for
		} finally {
			wsToUnDeploy.clear();
		}
	}
	
}
