package org.act.xservices.rt.xcontainer.deployment.deployer;

import java.io.File;
import java.util.Set;

import org.act.xservices.rt.metaset.MetasetParser;
import org.act.xservices.rt.metaset.MetasetParserFactory;
import org.act.xservices.rt.metaset.exception.MetasetParsingException;
import org.act.xservices.rt.xcontainer.deployment.DeploymentException;
import org.act.xservices.rt.xcontainer.deployment.WSInfo;
import org.act.xservices.rt.xcontainer.deployment.util.DeploymentConstants;
import org.act.xservices.rt.xcontainer.framework.IContainerEnvironment;
import org.act.xservices.rt.xcontainer.framework.config.ConfigElementParser;
import org.act.xservices.rt.xcontainer.framework.config.IConfigElement;
import org.act.xservices.rt.xcontainer.framework.policy.IPolicyConfiger;
import org.act.xservices.rt.xcontainer.framework.policy.PolicyConfigException;
import org.act.xservices.rt.xcontainer.inf.service.wsdl.WSDL11Constants;
import org.act.xservices.rt.xcontainer.inf.service.wsdl.WSDL20Constants;
import org.act.xservices.rt.xcontainer.inf.service.wsdl.XcService;
import org.act.xservices.rt.xcontainer.inf.service.wsdl.XcServiceRegistry;
import org.act.xservices.rt.xcontainer.parser.wsdd.IWSDDParser;
import org.act.xservices.rt.xcontainer.parser.wsdd.WSDDConstants;
import org.act.xservices.rt.xcontainer.parser.wsdd.WSDDParserFactory;
import org.act.xservices.rt.xcontainer.parser.wsdl.IWSDLParser;
import org.act.xservices.rt.xcontainer.parser.wsdl.WSDLParserFactory;
import org.act.xservices.rt.xcontainer.wsdl.WSDLParseException;
import org.act.xservices.rt.xom.Metaset;

/**
 * WSAR һʵ
 * 
 * @LastModifyTime 2009-04-14
 * @CreationTime 2007-10-15
 * @author Lizq [lizq@act.buaa.edu.cn]
 * @since JDK1.5
 */
public class WSServiceDeployer implements IDeployer {

	private MetasetParser mb = MetasetParserFactory.getMetasetParser();

	private XcServiceRegistry xcServiceRegistry;

	private String extension;

	private IContainerEnvironment containerEnv;

	private IPolicyConfiger[] policyConfigers;

	/**
	 * ʼ
	 * 
	 * @param extension
	 * @param containerEnv
	 */
	public void init(String extension, IContainerEnvironment containerEnv,
			IPolicyConfiger[] policyConfigers) {
		this.extension = extension;
		this.containerEnv = containerEnv;
		this.xcServiceRegistry = containerEnv.getXcServiceRegistry();
		this.policyConfigers = policyConfigers;
	}

	public String getExtension() {
		return extension;
	}

	/**
	 *  WSARƼΪQNameLocalpartӶͬĴ
	 * 
	 * @param wsFileName
	 *            WSAR
	 */
	public synchronized void unDeploy(String wsFileName)
			throws DeploymentException {

		// ˴ٶƼΪQNameLocalpart
		String serviceName = getServiceName(wsFileName);

		Set<String> serviceURIs = xcServiceRegistry
				.getServiceURLByServiceName(serviceName);

		if (serviceURIs != null) {
			xcServiceRegistry.removeXcServiceByURL(serviceURIs.iterator()
					.next());
		}
	}

	/**
	 * Retrieves service name from the archive file name. If the archive file
	 * name is service1.wsar , then xcontainer service name would be service1
	 * 
	 * @param fileName
	 * @return
	 */
	private String getServiceName(String fileName) {
		char seperator = '.';
		String value;
		int index = fileName.lastIndexOf(seperator);
		if (index > 0) {
			value = fileName.substring(0, index);
			return value;
		}
		return fileName;
	}

	/**
	 * WSARļ
	 * 
	 * @param wsFile
	 * @throws DeploymentException
	 */
	public void deploy(WSInfo wsFile) throws DeploymentException {

		File serviceWorkingFile = new File(wsFile.getServiceWorkingLocation());

		File metaInfFolder = new File(serviceWorkingFile,
				DeploymentConstants.META_INF);

		if (!metaInfFolder.exists()) {
			metaInfFolder = new File(serviceWorkingFile,
					DeploymentConstants.META_INF.toLowerCase());

			if (!metaInfFolder.exists()) {
				throw new DeploymentException(
						"Error: Can not find META-INF folder in"
								+ serviceWorkingFile.getAbsolutePath());
			}
		}

		// WSDDļ
		File serviceXMLFile = new File(metaInfFolder,
				DeploymentConstants.SERVICE_XML);
		if (!serviceXMLFile.exists()) {
			throw new DeploymentException(
					"Error: Can not find service.xml file in"
							+ metaInfFolder.getAbsolutePath());
		}

		// wsddļ
		Metaset wsddMetaset = null;
		try {
			wsddMetaset = mb.parse(serviceXMLFile.getAbsolutePath());
		} catch (MetasetParsingException e) {
			throw new DeploymentException(e);
		}

		// WSDLļ
		String wsdlFileLocation = wsddMetaset
				.attributeValue(WSDDConstants.ATTR_WSDLLOCATION);

		if (wsdlFileLocation == null) {
			throw new DeploymentException("Error: Can not find "
					+ WSDDConstants.ATTR_WSDLLOCATION
					+ " in file service.xml file ");
		}

		File wsdlFile = new File(metaInfFolder, wsdlFileLocation);
		if (!wsdlFile.exists()) {
			throw new DeploymentException("Error: Can not find wsdl file at "
					+ wsdlFile.getAbsolutePath());
		}

		// WSDLFile,XcServiceʵ
		XcService xcService = processWSDLFile(wsdlFile);
		xcService.setWsdlFile(wsdlFile);
		xcService.setServiceType(XcService.SERVICE_TYPE_WS); // ÷ΪWSService

		// WSDDļXcServiceӱҪϢ
		XcService xcServiceAfterWSDD = processWSDDFile(wsddMetaset, xcService,
				wsFile);

		if (xcServiceAfterWSDD == null) {
			throw new DeploymentException(
					"Error: False to deploy WSDD File of Service "
							+ xcService.getQName());
		}

		// ʹPolicyProcessorWSDLеĲϢ
		XcService xcServiceAfterPolicy = processPolicyInfo(xcServiceAfterWSDD);

		// XcServiceӵע
		boolean isRegistrySucceed = xcServiceRegistry
				.addXcService(xcServiceAfterPolicy);

		if (!isRegistrySucceed) {
			throw new DeploymentException(
					"Error: False to Registry Service "
							+ xcService.getQName()
							+ " , for the Service QName or ServiceURL already been Registryed!");
		}

		//
		// PolicyConfiger.config(xcServiceAfterWSDD);
	}

	/**
	 * ʹPolicyProcessorWSDLеĲϢ εеPolicyProcesorд
	 * 
	 * @param xcService
	 * @return
	 * @throws DeploymentException
	 */
	private XcService processPolicyInfo(XcService xcService)
			throws DeploymentException {

		if (policyConfigers != null) { // вԴ
			try {
				for (IPolicyConfiger pc : policyConfigers) {
					pc.configPolicyProcessor(xcService);
				}
			} catch (PolicyConfigException e) {
				throw new DeploymentException(
						"Error in process Policy Information", e);
			}
		}

		return xcService;
	}

	/**
	 * WSDDļXcServiceӱҪϢ
	 * 
	 * @param wsddMetaset
	 * @param xcService
	 * @return
	 * @throws DeploymentException
	 */
	private XcService processWSDDFile(Metaset wsddMetaset, XcService xcService,
			WSInfo wsFile) throws DeploymentException {

		IConfigElement serviceConfig = ConfigElementParser
				.creatConfigElement(wsddMetaset);

		String scope = serviceConfig.getAttribute("scope");

		IWSDDParser wsddParser;

		if (scope == null) { // ɰ汾WSDDļ
			wsddParser = WSDDParserFactory
					.CreatWSDDParser(WSDDConstants.WSDDPARSER_OLD);
		} else {
			wsddParser = WSDDParserFactory
					.CreatWSDDParser(WSDDConstants.WSDDPARSER);
		}

		return wsddParser.parseWSDD(serviceConfig, xcService, wsFile,
				containerEnv.getProviderFactory());
	}

	/**
	 * WSDLĵɵXcServiceserviceMap
	 * 
	 * @param wsdlFile
	 * @param servicesMap
	 */
	private XcService processWSDLFile(File wsdlFile) throws DeploymentException {

		Metaset wsdlMetaset = null;
		try {
			wsdlMetaset = mb.parse(wsdlFile.getAbsolutePath());
		} catch (MetasetParsingException e) {
			throw new DeploymentException("Error: failed to parse WSDL file : "
					+ wsdlFile.getAbsolutePath());
		}

		// ȷWSDLƿռ
		String wsdlNamespace = wsdlMetaset.getNamespaceURI();

		if (wsdlNamespace != null && !"".equals(wsdlNamespace)) {

			// wsdlNamespaceöӦWSDLParser
			IWSDLParser wsdlparser;

			if (WSDL20Constants.WSDL20_NAMESPACE.equals(wsdlNamespace)) {

				// we have a WSDL 2.0 document here.
				wsdlparser = WSDLParserFactory.CreatWSDLParser(
						WSDLParserFactory.WSDL20PARSER_OF_WODEN, wsdlFile);

			} else if (WSDL11Constants.NS_URI_WSDL11.equals(wsdlNamespace)) {

				// we have a WSDL 1.1 document here
				wsdlparser = WSDLParserFactory.CreatWSDLParser(
						WSDLParserFactory.WSDL11PARSER_OF_WSDL4J, wsdlFile);

			} else {
				throw new DeploymentException("Error: invalidWSDLFound : "
						+ wsdlFile.getAbsolutePath());
			}

			XcService xcService;
			try {
				xcService = wsdlparser.parseWSDL();
			} catch (WSDLParseException e) {
				throw new DeploymentException(e);
			}

			return xcService;

		} else {
			throw new DeploymentException(
					"Error: can not find wsdlNamespace of file "
							+ wsdlFile.getAbsolutePath());
		}

	}

}
