package org.act.xservices.rt.xcontainer.parser.wsdd;

import org.act.xservices.log.Logger;
import org.act.xservices.log.LoggerFactory;
import org.act.xservices.rt.xcontainer.deployment.DeploymentException;
import org.act.xservices.rt.xcontainer.deployment.WSInfo;
import org.act.xservices.rt.xcontainer.framework.config.IConfigElement;
import org.act.xservices.rt.xcontainer.framework.provider.IProvider;
import org.act.xservices.rt.xcontainer.framework.provider.ProviderFactory;
import org.act.xservices.rt.xcontainer.framework.provider.ProviderFactory.ProviderInfo;
import org.act.xservices.rt.xcontainer.inf.service.wsdl.XcOperation;
import org.act.xservices.rt.xcontainer.inf.service.wsdl.XcService;
import org.act.xservices.rt.xom.QName;

/**
 * @LastModifyTime 2009-4-14
 * @CreationTime
 * @author Lizq [lizq@act.buaa.edu.cn]
 * @since JDK1.5
 */
public class WSDDParser implements IWSDDParser {

	final Logger logger = LoggerFactory.getLogger(WSDDParser.class);

	public XcService parseWSDD(IConfigElement serviceConfig,
			XcService xcService, WSInfo wsFile, ProviderFactory providerFactory)
			throws DeploymentException {

		// ServiceԪص
		String serviceName = serviceConfig
				.getAttribute(WSDDConstants.ATTR_SERVICENAME);

		if (!xcService.getServiceName().equals(serviceName)) {
			throw new DeploymentException(
					"Error: WSDD File serviceName attribute: " + serviceName
							+ ", is not same with WSDL ServiceName: "
							+ xcService.getServiceName());
		}

		if (XcService.SCOPE_REQUEST.equalsIgnoreCase(serviceConfig
				.getAttribute(WSDDConstants.ATTR_SCOPE))) {
			xcService.setScope(XcService.SCOPE_REQUEST);

		} else if (XcService.SCOPE_APPLICATION.equalsIgnoreCase(serviceConfig
				.getAttribute(WSDDConstants.ATTR_SCOPE))) {
			xcService.setScope(XcService.SCOPE_APPLICATION);
		}

		// descriptionԪ
		IConfigElement desConfig = serviceConfig
				.getFirstChildWithName(WSDDConstants.ELEM_NAME_DESCRIPTION);
		if (desConfig != null) {
			xcService.setDescription(desConfig.getValue());
		}

		// parameterԪ
		String serviceClass = null;

		String paraName, paraValue;

		for (IConfigElement paraConfig : serviceConfig
				.getChildrenWithName(WSDDConstants.ELEM_NAME_PARAMETER)) {

			paraName = paraConfig.getAttribute(WSDDConstants.ATTR_NAME);
			paraValue = paraConfig.getValue().trim();

			if (paraName == null || "".equals(paraName) || paraValue == null
					|| "".equals(paraValue)) {
				throw new DeploymentException(
						"Error: Parameter Element error. Service: "
								+ xcService.getServiceName());
			}

			if (paraName.equals(WSDDConstants.ATTR_VALUE_SERVICECLASS)) {
				serviceClass = paraValue;
			}

			// ӵXcService
			xcService.addParameter(paraName, paraValue);
		}

		// serviceClass,ԿԲ
		xcService.setServiceClassLoader(wsFile.getClassLoader());

		if (serviceClass == null || "".equals(serviceClass)) {
			logger.info("No ServiceClass Parameter of service: "
					+ xcService.getServiceName());

		} else {
			try {
				xcService.setServiceClass(wsFile.getClassLoader().loadClass(
						serviceClass));

			} catch (ClassNotFoundException e) {
				throw new DeploymentException(
						"Error: Can not load ServiceClass: " + serviceClass, e);
			} catch (Exception e) {
				throw new DeploymentException(
						"Error: Can not load ServiceClass: " + serviceClass, e);
			}
		}

		// workingļµķĿ¼
		xcService.setWsarLocation(wsFile.getServiceWorkingLocation());

		// ܴڵProviders
		IConfigElement providersConfig = serviceConfig
				.getFirstChildWithName(WSDDConstants.ELEM_NAME_PROVIDERS);
		if (providersConfig != null) {
			processProvider(providersConfig, xcService);
		}

		// ܴڵoperations
		// ĿǰoperationactionMappingprovider߶Ժ
		IConfigElement operationsConfig = serviceConfig
				.getFirstChildWithName(WSDDConstants.ELEM_NAME_OPERATIONS);
		if (operationsConfig != null) {
			processOperations(operationsConfig, xcService, providerFactory);
		}

		// XcServiceеXcOperation
		processXcOperation(xcService, providerFactory);

		// add by viking , use to validate wsdl and service
		ServiceValidator.validate(xcService);
		return xcService;
	}

	/**
	 * XcOperationProvider
	 * 
	 * @param xcService
	 * @param providerFactory
	 */
	private void processXcOperation(XcService xcService,
			ProviderFactory providerFactory) {

		for (XcOperation xcope : xcService.getAllXcOperation()) {

			if (xcope.getProvider() != null) { // ProviderѾ
				continue;
			}

			// ôMEPӦϵͳĬϵProvider
			String proType = providerFactory.getDefaultProviderType(xcope
					.getMepURI());

			IProvider pro = xcService.getProvider(proType);

			if (pro == null) {
				// ϵͳprovider,µProviderʵXcService
				ProviderInfo proInfo = providerFactory.getProviderInfo(xcope
						.getMepURI(), proType);

				String proClassName = null;

				if (proInfo != null) {
					proClassName = proInfo.getProviderClassName();
				}

				if (proClassName != null) {
					try {
						pro = (IProvider) xcService.getServiceClassLoader()
								.loadClass(proClassName).newInstance();
						pro.init(proInfo.getProviderConfig(), xcService);
						xcService.addProvider(pro);
					} catch (Exception e) {
						e.printStackTrace();
					}
				}
			}

			if (pro == null) { // δҵӦProvider,
				logger.info("Can not find Provider for operation: "
						+ xcope.getQName());
			} else {
				xcope.setProvider(pro);
			}

		}// end of for
	}

	/**
	 * operationsԪ
	 * 
	 * @param operationsConfig
	 * @param xcService
	 * @throws DeploymentException
	 */
	private void processOperations(IConfigElement operationsConfig,
			XcService xcService, ProviderFactory providerFactory)
			throws DeploymentException {

		if (operationsConfig == null || xcService == null) {
			return;
		}

		for (IConfigElement opeConfig : operationsConfig
				.getChildrenWithName(WSDDConstants.ELEM_NAME_OPERATION)) {

			String opeName = opeConfig.getAttribute(WSDDConstants.ATTR_NAME);

			if (opeName == null || "".equals(opeName)) {
				logger.info("No operation name attribute in wsdd file :"
						+ xcService.getServiceName());
				continue;
			}

			String opeAction = null;
			if (opeConfig
					.getFirstChildWithName(WSDDConstants.ELEM_NAME_ACTIONMAPPING) != null) {
				opeAction = opeConfig.getFirstChildWithName(
						WSDDConstants.ELEM_NAME_ACTIONMAPPING).getValue();
			}

			String opeProviderType = null;
			if (opeConfig
					.getFirstChildWithName(WSDDConstants.ELEM_NAME_PROVIDER) != null) {
				opeProviderType = opeConfig.getFirstChildWithName(
						WSDDConstants.ELEM_NAME_PROVIDER).getValue();
			}

			// XcServiceвҶӦXcOperation,δҵµXcOperation
			XcOperation xcOperation = xcService.getXcOperation(new QName(
					opeName));

			if (xcOperation == null) { // XcServiceµXcOperation
				xcOperation = new XcOperation(new QName(xcService
						.getTargetNamespace(), opeName));
				xcOperation.setXcService(xcService);
				xcService.addXcOperation(xcOperation);
			}

			// XcOperationProvider
			if (opeProviderType != null) {

				// XcServiceжProvider
				IProvider pro = xcService.getProvider(opeProviderType);

				if (pro == null) {
					// ϵͳprovider,µProviderʵXcService
					ProviderInfo proInfo = providerFactory.getProviderInfo(
							xcOperation.getMepURI(), opeProviderType);

					String proClassName = null;

					if (proInfo != null) {
						proClassName = proInfo.getProviderClassName();
					}

					if (proClassName != null) {
						try {
							pro = (IProvider) xcService.getServiceClassLoader()
									.loadClass(proClassName).newInstance();

							pro.init(proInfo.getProviderConfig(), xcService);

							xcService.addProvider(pro);

						} catch (Exception e) {
							e.printStackTrace();
						}
					}
				}

				if (pro == null) { // δҵӦProvider
					logger.info("Can not find Provider for operation: "
							+ xcOperation.getQName());
				} else {
					xcOperation.setProvider(pro);
				}
			}

			// XcOperationaction
			if (opeAction != null) {
				if (xcOperation.getSoapAction() == null) {
					xcOperation.setSoapAction(opeAction);
					xcService.mapActionToXcOperation(opeAction, xcOperation);// ӴxcOperationactionֵ
				}
			}

		}// end of for

	}

	/**
	 * providersԪ
	 * 
	 * @param providersConfig
	 * @param xcService
	 * @throws DeploymentException
	 */
	private void processProvider(IConfigElement providersConfig,
			XcService xcService) throws DeploymentException {

		if (providersConfig == null || xcService == null) {
			return;
		}

		// ServiceԶProvider, typeproviderClass
		for (IConfigElement proConfig : providersConfig
				.getChildrenWithName(WSDDConstants.ELEM_NAME_PROVIDER)) {

			String type = proConfig.getAttribute(WSDDConstants.ATTR_TYPE);
			String mepURI = proConfig.getAttribute(WSDDConstants.ATTR_MEP);
			String providerClass = proConfig
					.getAttribute(WSDDConstants.ATTR_CLASS);

			try {
				if (type != null && providerClass != null) {
					IProvider pro = (IProvider) xcService
							.getServiceClassLoader().loadClass(providerClass)
							.newInstance();

					pro.init(proConfig, xcService); // Provider÷

					xcService.addProvider(pro); //

					logger.info("Succeed add Provider: " + type
							+ " to Service: " + xcService.getServiceName());
				} else {
					logger.info("Error Provider Parameter: " + type + " "
							+ mepURI + " " + providerClass);
				}
			} catch (Exception e) {
				logger.info("Failed add Provider: " + type);
				throw new DeploymentException(e);
			}

		}// end of for

	}

}
