package org.act.xservices.rt.xcontainer.parser.wsdd;

import org.act.xservices.log.Logger;
import org.act.xservices.log.LoggerFactory;
import org.act.xservices.rt.xcontainer.deployment.DeploymentException;
import org.act.xservices.rt.xcontainer.deployment.WSInfo;
import org.act.xservices.rt.xcontainer.framework.config.IConfigElement;
import org.act.xservices.rt.xcontainer.framework.provider.IProvider;
import org.act.xservices.rt.xcontainer.framework.provider.ProviderFactory;
import org.act.xservices.rt.xcontainer.framework.provider.ProviderFactory.ProviderInfo;
import org.act.xservices.rt.xcontainer.inf.service.wsdl.XcOperation;
import org.act.xservices.rt.xcontainer.inf.service.wsdl.XcService;

/**
 * @LastModifyTime 2009-4-14
 * @CreationTime
 * @author Lizq [lizq@act.buaa.edu.cn]
 * @since JDK1.5
 */
public class WSDDParserOld implements IWSDDParser {

	final Logger logger = LoggerFactory.getLogger(WSDDParserOld.class);

	public XcService parseWSDD(IConfigElement serviceConfig,
			XcService xcService, WSInfo wsFile, ProviderFactory providerFactory)
			throws DeploymentException {
		String serviceName = serviceConfig
				.getAttribute(WSDDConstants.WSDD_ATTR_SERVICENAME);
		if (!xcService.getServiceName().equals(serviceName)) {
			throw new DeploymentException(
					"Error: WSDD File serviceName attribute: " + serviceName
							+ ", is not same with WSDL ServiceName: "
							+ xcService.getServiceName());
		}

		// ProviderԪ
		IConfigElement providerConfig = serviceConfig
				.getFirstChildWithName(WSDDConstants.WSDD_ELEM_NAME_PROVIDER);
		if (providerConfig == null) {
			throw new DeploymentException(
					"Error: no provider element in wsdd file: "
							+ xcService.getServiceName());
		}

		// ProviderԪеParameter
		String serviceClass = null;
		String paraName, paraValue;

		for (IConfigElement paraConfig : providerConfig
				.getChildrenWithName(WSDDConstants.WSDD_ELEM_NAME_PARAMETER)) {

			paraName = paraConfig.getAttribute(WSDDConstants.WSDD_ATTR_NAME);
			paraValue = paraConfig.getAttribute(WSDDConstants.WSDD_ATTR_VALUE);

			if (paraName == null || "".equals(paraName) || paraValue == null
					|| "".equals(paraValue)) {
				throw new DeploymentException(
						"Error: Parameter Element error. Service: "
								+ xcService.getServiceName());
			}

			if (paraName.equals(WSDDConstants.WSDD_ATTR_VALUE_CLASSNAME)
					|| paraName
							.equals(WSDDConstants.WSDD_ATTR_VALUE_SERVICECLASSNAME)) {
				serviceClass = paraValue;
			}

			// ӵXcService
			xcService.addParameter(paraName, paraValue);
		}

		// serviceClass,ԿԲ
		xcService.setServiceClassLoader(wsFile.getClassLoader());

		if (serviceClass == null || "".equals(serviceClass)) {
			logger.info("No ClassName Parameter of service: "
					+ xcService.getServiceName());

		} else {
			try {
				xcService.setServiceClass(wsFile.getClassLoader().loadClass(
						serviceClass));

			} catch (ClassNotFoundException e) {
				throw new DeploymentException(
						"Error: Can not load ServiceClass: " + serviceClass, e);
			}
		}

		// workingļµķĿ¼
		xcService.setWsarLocation(wsFile.getServiceWorkingLocation());

		// Provider
		String providerType = providerConfig
				.getAttribute(WSDDConstants.WSDD_ATTR_TYPE);
		if (providerType == null) {
			throw new DeploymentException("Error: no Type provider attribute: "
					+ xcService.getServiceName());
		}

		// XcServiceеXcOperation
		if (providerType.equalsIgnoreCase(WSDDConstants.WSDD_ATTR_VALUE_JAVA)) { // RawXMLProvider
			processXcOperation(xcService, providerConfig, providerFactory,
					WSDDConstants.WSDD_ATTR_VALUE_JAVA);

		} else if (providerType
				.equalsIgnoreCase(WSDDConstants.WSDD_ATTR_VALUE_RESOURCE)) {

			processXcOperation(xcService, providerConfig, providerFactory,
					WSDDConstants.WSDD_ATTR_VALUE_RESOURCE);

		} else if (providerType
				.equalsIgnoreCase(WSDDConstants.WSDD_ATTR_VALUE_RESOURCEWITHNOTIFICATION)) {

			processXcOperation(xcService, providerConfig, providerFactory,
					WSDDConstants.WSDD_ATTR_VALUE_RESOURCEWITHNOTIFICATION);
		}

		return xcService;
	}

	/**
	 * XcOperationProvider
	 * 
	 * @param xcService
	 * @param providerFactory
	 */
	private void processXcOperation(XcService xcService,
			IConfigElement providerConfig, ProviderFactory providerFactory,
			String providerType_Old) {

		for (XcOperation xcope : xcService.getAllXcOperation()) {

			if (xcope.getProvider() != null) { // ProviderѾ
				continue;
			}

			// ôMEPӦϵͳĬϵProvider
			String providerType = null;

			IProvider provider = null; //

			// Java Provider
			if (WSDDConstants.WSDD_ATTR_VALUE_JAVA.equals(providerType_Old)) {

				providerType = providerFactory.getDefaultProviderType(xcope
						.getMepURI());

				provider = xcService.getProvider(providerType);

				if (provider == null) {
					// ϵͳprovider,µProviderʵXcService
					ProviderInfo proInfo = providerFactory.getProviderInfo(
							xcope.getMepURI(), providerType);

					String proClassName = null;

					if (proInfo != null) {
						proClassName = proInfo.getProviderClassName();
					}

					if (proClassName != null) {
						try {
							provider = (IProvider) xcService
									.getServiceClassLoader().loadClass(
											proClassName).newInstance();

							provider.init(proInfo.getProviderConfig(),
									xcService);

							xcService.addProvider(provider);

						} catch (Exception e) {
							e.printStackTrace();
						}
					}
				}

			} else if (WSDDConstants.WSDD_ATTR_VALUE_RESOURCE
					.equals(providerType_Old)
					|| WSDDConstants.WSDD_ATTR_VALUE_RESOURCEWITHNOTIFICATION
							.equals(providerType_Old)) {

				// Դ֪ͨProvider

				providerType = providerType_Old;

				provider = xcService.getProvider(providerType);

				if (provider == null) {

					String proClassName = null;

					if (WSDDConstants.WSDD_ATTR_VALUE_RESOURCE
							.equals(providerType)) {
						proClassName = WSDDConstants.RESOURCE_PROVIDER_CLASS;
					} else if (WSDDConstants.WSDD_ATTR_VALUE_RESOURCEWITHNOTIFICATION
							.equals(providerType)) {
						proClassName = WSDDConstants.NOTIFICATION_PROVIDER_CLASS;
					}

					if (proClassName != null) {
						try {
							provider = (IProvider) xcService
									.getServiceClassLoader().loadClass(
											proClassName).newInstance();

							provider.init(providerConfig, xcService);

							xcService.addProvider(provider);

						} catch (Exception e) {
							e.printStackTrace();
						}
					}

				}

			}

			// Xcoperationprovider
			if (provider == null) { // δҵӦProvider,
				logger.info("Can not find Provider for operation: "
						+ xcope.getQName());
			} else {
				xcope.setProvider(provider);
			}

		}// end of for

	}

}
