package org.act.xservices.rt.xcontainer.framework;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

import org.act.xservices.rt.xcontainer.framework.config.IConfigElement;
import org.act.xservices.rt.xcontainer.framework.dispatch.Dispatcher;
import org.act.xservices.rt.xcontainer.framework.dispatch.ITaskFactory;
import org.act.xservices.rt.xcontainer.framework.lifecycle.ALifecycle;
import org.act.xservices.rt.xcontainer.framework.lifecycle.LifecycleException;
import org.act.xservices.rt.xcontainer.framework.objectpool.IDispatchSource;
import org.act.xservices.rt.xcontainer.framework.objectpool.IDispatchTarget;
import org.act.xservices.rt.xcontainer.framework.objectpool.IObjectPool;


/**
 * Connector connect two types of <code>BufferedPool</code> pool with
 * templated type <code>T</code> and <code>V</code>.
 * 
 * @LastModifyTime 2009-10-02
 * @CreationTime 2007-03-20
 * @author liujian[liujian@act.buaa.edu.cn]
 * @author Lizq [lizq@act.buaa.edu.cn]
 * @since JDK1.5
 */

public abstract class AConnector<T, V> extends ALifecycle implements IConnector<T, V> {

	/** Ԫ  */
	private static final String CONST_NAME = "name";
	private static final String CONST_THREADPOOL_MINTHREAD = "minThread";
	private static final String CONST_THREADPOOL_MAXTHREAD = "maxThread";
	private static final String CONST_THREADPOOL_KEEPALIVETIMES = "keepAliveTimes";
	private static final String CONST_THREADPOOL_USECACHEDTHREADPOOL = "useCachedThreadPool";
	
	/**  */
	private String connectorName;

	/**  */
	private String description;
	
	/** Ԫ */
	protected IConfigElement connectorConfiguration;

	/** ɷб */
	private List<Dispatcher<T>> inputDispatcherList;

	/** ɷб */
	private List<Dispatcher<V>> outputDispatcherList;

	/**  */
	protected IConnectorEnvironment<T, V> connectorEnvironment;
	
	/** ɷԴ */
	private IDispatchSource<T> inputSource;

	/** ɷĿ */
	private IDispatchTarget<V> inputTarget;

	/** ɷԴ */
	private IDispatchSource<V> outputSource;

	/** ɷĿ */
	private IDispatchTarget<T> outputTarget;

	/** е̳߳ */
	private ExecutorService inputThreadpool;

	/** е̳߳ */
	private ExecutorService outputThreadpool;

	/**
	 * 캯
	 * @param runtimeEnvir
	 */
	public AConnector(String connectorName) {
		this.connectorName = connectorName;
	}
	
	public AConnector() {
		
	}
	
	public IConfigElement getConnectorConfiguration() {
		return connectorConfiguration;
	}
	
	/**
	 * ʼ(ģ巽ģʽ)
	 * @param connectorConfigElement
	 * @param runtimeContainer
	*/
	public final void initialize(IConfigElement connectorConfigElement, 
			IRuntimeContainer runtimeContainer){
		
		this.connectorConfiguration = connectorConfigElement;
		
		//ʼ
		this.connectorName = connectorConfiguration.getAttribute("name");
		this.description = connectorConfiguration.getAttribute("description");
		
		//---ע⣺˴˳ǳҪܵ
		
		// ʼ
		this.doInitEnvironment(connectorConfiguration.getFirstChildWithName("environment"),
				runtimeContainer.getContainerEnvironment());
		
		// ̳߳
		this.doInitThreadPool();
		
		// 򻷾װض
		this.doInitObjectPool(runtimeContainer);
		
		// ʼԴ
		this.doInitLocal();
		
		// ʼDispatcher
		this.doInitDispatcher();
	}
	
	/**
	 * ݻԪسʼ Ļ
	 * @param envConfigElement
	 * @return
	 */
	@SuppressWarnings("unchecked")
	protected void doInitEnvironment(IConfigElement envConfigElement, 
			IContainerEnvironment containerEnvironment){
		try {
			connectorEnvironment = (IConnectorEnvironment<T, V>) Class.forName(
					envConfigElement.getAttribute("class")).newInstance();
			
			//ע⣺ñconfig()֮ǰ
			connectorEnvironment.setContainerEnvironment(containerEnvironment);
			
			connectorEnvironment.config(envConfigElement);// ʼ̳߳
			
		} catch (InstantiationException e) {
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			e.printStackTrace();
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	
	protected void doInitThreadPool(){
		//̳߳
		IConfigElement[] threadPoolList = 
			connectorConfiguration.getChildrenWithName("threadpool");

		if (threadPoolList.length == 2) { // ʼ̳߳
			if (threadPoolList[0].getAttribute(CONST_NAME).indexOf("in") != -1) {
				// Ϊ̳߳
				inputThreadpool = creatThreadPool(threadPoolList[0]);
//				
//				IContainerContext.containerLogger
//				.info(connectorEnvironmentName+" -- InThreadpool: "+"min-"+ inputThreadpool.getCorePoolSize()+"  max-"+inputThreadpool.getMaximumPoolSize());

				// Ϊ̳߳
				outputThreadpool = creatThreadPool(threadPoolList[1]);
//				
//				IContainerContext.containerLogger
//				.info(connectorEnvironmentName+" -- OutThreadpool: "+"min-"+ outputThreadpool.getCorePoolSize()+"  max-"+outputThreadpool.getMaximumPoolSize());

			} else { //
				//̳߳
				inputThreadpool = creatThreadPool(threadPoolList[1]);
//				
//				IContainerContext.containerLogger
//				.info(connectorEnvironmentName+" -- InThreadpool: "+"min-"+ inputThreadpool.getCorePoolSize()+"  max-"+inputThreadpool.getMaximumPoolSize());

				//̳߳
				outputThreadpool = creatThreadPool(threadPoolList[0]);
				
//				IContainerContext.containerLogger
//				.info(connectorEnvironmentName+" -- OutThreadpool: "+"min-"+ outputThreadpool.getCorePoolSize()+"  max-"+outputThreadpool.getMaximumPoolSize());

			}
			
		} else if (threadPoolList.length == 1) { //ʼһ̳߳
			if (threadPoolList[0].getAttribute(CONST_NAME).indexOf("in") != -1) {
				// Ϊ̳߳
				inputThreadpool = creatThreadPool(threadPoolList[0]);
				
//				IContainerContext.containerLogger
//				.info(connectorEnvironmentName+" -- inputThreadpool: "+"min-"+ inputThreadpool.getCorePoolSize()+"  max-"+inputThreadpool.getMaximumPoolSize());

			} else {
				//̳߳
				outputThreadpool = creatThreadPool(threadPoolList[0]);
				
//				IContainerContext.containerLogger
//				.info(connectorEnvironmentName+" -- OutThreadpool: "+"min-"+ outputThreadpool.getCorePoolSize()+"  max-"+outputThreadpool.getMaximumPoolSize());

			}
		} //̳߳سʼ
	}
	
	
	/**
	 * òһ̳߳
	 * @param threadPoolConfig
	 * @return
	 */
	private ExecutorService creatThreadPool(IConfigElement threadPoolConfig) {
		
		//ʹJDKṩĻ̳߳
		if("true".equalsIgnoreCase(threadPoolConfig.getAttribute(
				CONST_THREADPOOL_USECACHEDTHREADPOOL))){
			return Executors.newCachedThreadPool();
		}
		
		//ʹò̳߳
		int minTreadIn = Integer.parseInt(threadPoolConfig
				.getAttribute(CONST_THREADPOOL_MINTHREAD));
		int maxThreadIn = Integer.parseInt(threadPoolConfig
				.getAttribute(CONST_THREADPOOL_MAXTHREAD));
		int keepAliveTimesIn = Integer.parseInt(threadPoolConfig
				.getAttribute(CONST_THREADPOOL_KEEPALIVETIMES));
		
		//TODO עɵеĲ,ע˴ʹõ޽ѭУζmaxThreadInʧЧдǣ by lizq 2008-07-05
		LinkedBlockingQueue<Runnable> queueIn = new LinkedBlockingQueue<Runnable>(); 
		//ArrayBlockingQueue <Runnable> queueIn = new ArrayBlockingQueue<Runnable>(20); 
		
		ThreadPoolExecutor tp = new ThreadPoolExecutor(minTreadIn, maxThreadIn,
				keepAliveTimesIn, TimeUnit.SECONDS, queueIn);
		//tp.setRejectedExecutionHandler(new ThreadPoolExecutor.CallerRunsPolicy());
		
		return tp;
	}

	
	/**
	 * ʼ
	 * @param runtimeContainer
	 */
	@SuppressWarnings("unchecked")
	protected void doInitObjectPool(IRuntimeContainer runtimeContainer){
		//򻷾װض
		HashMap<String, IObjectPool> objectPoolMap = runtimeContainer.getObjectPoolMap();

		String sourceName;
		if( !(sourceName = connectorConfiguration.getAttribute("inputsourcepool")).equals("")){
			inputSource =(IObjectPool<T>)objectPoolMap.get(sourceName);
		}
		if( !(sourceName = connectorConfiguration.getAttribute("inputtargetpool")).equals("")){
			inputTarget = (IObjectPool<V>)objectPoolMap.get(sourceName);
		}
		if( !(sourceName = connectorConfiguration.getAttribute("outputsourcepool")).equals("")){
		    outputSource = (IObjectPool<V>)objectPoolMap.get(sourceName);
		}
		if( !(sourceName = connectorConfiguration.getAttribute("outputtargetpool")).equals("")){
			outputTarget = (IObjectPool<T>)objectPoolMap.get(sourceName);
		}
	}
	
	
	/**
	 * ڸԴĳʼ
	 *  Ԫ  Ԫ ѰϢ
	 * @param connectorConfigElement
	 */
	protected abstract void doInitLocal();
	
	/**
	 * ʼװɷɷ
	 * ԪصԪѰϢ
	 * @param connectorConfigElement
	 */
	protected void doInitDispatcher(){
		//ʼɷ
		this.doInitInputDispatcher();
		
		//ʼɷ
		this.doInitOutputDispatcher();
	}
	
	protected void doInitInputDispatcher(){
		// ɷɷб
		int inDispatcherNum = 
			Integer.parseInt(this.connectorConfiguration.getAttribute("inDispatcherNumber"));
		int inDispatcherPriority = 
			Integer.parseInt(this.connectorConfiguration.getAttribute("inDispatcherPriority"));
		
		for (int i = 0; i < inDispatcherNum; i++) {
			Dispatcher<T> inputDispatcher = new Dispatcher<T>();
			inputDispatcher.initialize(getInputSource(), getInputTaskFactory(), 
					getInputThreadpool(), inDispatcherPriority);
			getInputDispatcherList().add(inputDispatcher);
		}
	}
	
	protected void doInitOutputDispatcher(){
		// ɷɷб
		int outDispatcherNum = 
			Integer.parseInt(this.connectorConfiguration.getAttribute("outDispatcherNumber"));
		int outDispatcherPriority = 
			Integer.parseInt(this.connectorConfiguration.getAttribute("outDispatcherPriority"));
		
		for (int i = 0; i < outDispatcherNum; i++) {
			Dispatcher<V> outputDispatcher = new Dispatcher<V>();
			outputDispatcher.initialize( getOutputSource(), getOutputTaskFactory(), 
					getOutputThreadpool(), outDispatcherPriority );
			getOutputDispatcherList().add(outputDispatcher);
		}
	}
	
	
	/**
	 * 
	 * @return
	 */
	public IConnectorEnvironment<T, V> getConnectorEnvironment() {
		return connectorEnvironment;
	}

	/**
	 * @return   the description
	 */
	public String getDescription() {
		return description;
	}

	/**
	 * @return   the connectorName
	 */
	public String getConnectorName() {
		return connectorName;
	}

	/**
	 * ɷбӳٳʼ
	 * @return
	 */
	private final List<Dispatcher<T>> getInputDispatcherList() {
		if (inputDispatcherList == null) {
			inputDispatcherList = new ArrayList<Dispatcher<T>>();
		}
		return inputDispatcherList;
	}

	/**
	 * ɷбӳٳʼ
	 * @return
	 */
	private final List<Dispatcher<V>> getOutputDispatcherList() {
		if (outputDispatcherList == null) {
			outputDispatcherList = new ArrayList<Dispatcher<V>>();
		}
		return outputDispatcherList;
	}

//	/**
//	 * ɷ
//	 * @param inputDispatcher
//	 * @return
//	 */
//	protected final boolean addInputDispatcher(Dispatcher<T> inputDispatcher) {
//		return getInputDispatcherList().add(inputDispatcher);
//	}
//
//	/**
//	 * ɷ
//	 * @param outputDispatcher
//	 * @return
//	 */
//	protected final boolean addOutputDispatcher(Dispatcher<V> outputDispatcher) {
//		return getOutputDispatcherList().add(outputDispatcher);
//	}

	/**
	 * ȡ봦񹤳
	 * ģʽ
	 * @return
	 */
	protected abstract ITaskFactory<T> getInputTaskFactory();

	/**
	 * ȡ﹤
	 * ģʽ
	 * @return
	 */
	protected abstract ITaskFactory<V> getOutputTaskFactory();

	/**
	 * ɷ
	 * @throws LifecycleException
	 */
	protected void startInputDispatcher() throws LifecycleException{
		// ɷбȻɷбеÿɷ
		for ( Dispatcher<T> dispacher : getInputDispatcherList() ) {
			dispacher.start();
		}
	}

	/**
	 * ɷ
	 * @throws LifecycleException
	 */
	protected void startOutputDispatcher() throws LifecycleException{
		// ɷбȻɷбеÿɷ
		for ( Dispatcher<V> dispacher : getOutputDispatcherList() ) {
			dispacher.start();
		}
	}

	/** 
	 * 
	 * Ĭʵ֣ɷɷ
	 * @throws LifecycleException
	 */
	public void start() throws LifecycleException{
		startInputDispatcher();
		startOutputDispatcher();
	}

	/**
	 * ֹͣɷ
	 * @throws LifecycleException
	 */
	public void stopInputDispatcher() throws LifecycleException{
		for ( Dispatcher<T> dispacher : getInputDispatcherList() ) {
			dispacher.terminate();
		}
	}

	/**
	 * ֹͣɷ
	 * @throws LifecycleException
	 */
	public void stopOutputDispatcher() throws LifecycleException{
		for ( Dispatcher<V> dispacher : getOutputDispatcherList() ) {
			dispacher.terminate();
		}
	}

	/** 
	 * ֹͣ
	 * Ĭʵ֣رɷԼ̳߳
	 * @throws LifecycleException
	 */
	public void stop() throws LifecycleException{
		
		stopInputDispatcher();
		
		stopOutputDispatcher();
		
		if(inputThreadpool != null){
			inputThreadpool.shutdown();
		}
		
		if(outputThreadpool != null){
			outputThreadpool.shutdown();
		}
	}
	
	public final IDispatchSource<T> getInputSource() {
		return inputSource;
	}

	public final IDispatchTarget<V> getInputTarget() {
		return inputTarget;
	}

	public final IDispatchSource<V> getOutputSource() {
		return outputSource;
	}

	public final IDispatchTarget<T> getOutputTarget() {
		return outputTarget;
	}

	public final ExecutorService getInputThreadpool() {
		return inputThreadpool;
	}

	public final ExecutorService getOutputThreadpool() {
		return outputThreadpool;
	}
	
}
