package org.act.xservices.rt.xcontainer.framework.objectpool.impl;

import java.util.concurrent.locks.Condition;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;

import org.act.xservices.rt.xcontainer.framework.config.IConfigElement;
import org.act.xservices.rt.xcontainer.framework.objectpool.IObjectPool;

/**
 * BufferedPool of template type <code>T</code>.  󻺳
 * 
 * @version 1.0 2007-3-20
 * @author zhangdecai <zhangdc@act.buaa.edu.cn>
 * @author liujian [liujian@act.buaa.edu.cn]
 * @author Lizq [lizq@act.buaa.edu.cn]
 * @since JDK1.5
 */

public final class ObjectBufferedPool<T> implements IObjectPool<T>{

	//ѭУСӦñҪܴŵԪĿһԭֿնке
	//TODO CountĿ
	private static final int DEFAULT_SIZE = 100;


	private int size = DEFAULT_SIZE;


	private T[] list = null;


	private volatile int front = 0;


	private volatile int rear = 0;


	private volatile int count = 0;


	private final Lock lock = new ReentrantLock();


	private final Condition notFull = lock.newCondition();

	private final Condition notEmpty = lock.newCondition();
	

	private String name;
	
	

	public ObjectBufferedPool() {
		
	}
	

	public String getName() {
		return name;
	}
	

	public void setName(String name) {
		this.name = name;
	}


	public void put(T x) throws InterruptedException {
		lock.lockInterruptibly();
		try {
			while (count == size)
				notFull.await();
			list[front] = x;
			if (++front == size)
				front = 0;
			++count;
			notEmpty.signalAll();
		} finally {
			lock.unlock();
		}
	}

	/**
	 * Thread safe.
	 */
	public T get() throws InterruptedException {
		lock.lockInterruptibly();
		try {
			while (count == 0)
				notEmpty.await();
			T t = list[rear];
			if (++rear == size)
				rear = 0;
			--count;
			notFull.signalAll();
			return t;
		} finally {
			lock.unlock();
		}
	}

	//	/*
	//	 * @see org.act.webservice.IObjectPool#get()
	//	 */
	//	public synchronized T get() {
	//		lock.readLock().lock();
	//		while (empty()) {
	//			lock.readLock().unlock();
	//			try {
	//				readCondition.await();
	//			} catch (InterruptedException e) {
	//				return null;
	//			}
	//			lock.readLock().lock();
	//		}			
	//		front=(front+1)%size;
	//		T result = list[front];
	//		lock.readLock().unlock();
	//		count--;
	//		writeConditon.notify();
	//		return result;
	//	}

	//	/*
	//	 * @see org.act.webservice.IObjectPool#put(java.lang.Object)
	//	 */
	//	public synchronized void put(T obj) {
	//		lock.writeLock().lock();
	//		while(full()) {
	//			lock.writeLock().unlock();
	//			try {
	//				reentrantLock.lock();
	//				writeConditon.await();
	//			} catch (InterruptedException e) {
	//				//e.printStackTrace();
	//				return;
	//			}
	//			finally {
	//				reentrantLock.unlock();
	//			}
	//			lock.writeLock().lock();
	//		}
	//		rear=(rear+1)%size;
	//		list[rear]=obj;
	//		lock.writeLock().unlock();
	//		count++;
	//        readCondition.signal();
	//	}	


	public boolean isEmpty() {
		return count == 0;//front==rear;
	}


	public boolean isFull() {
		return count == size;//(rear+1)%size==front;
	}


	public synchronized int size() {
		return count;
	}


	@SuppressWarnings("unchecked")
	public synchronized void setMaxSize(int size) {
		//no need to create a new array
		if (this.size == size)
			return;
        this.size = size;
		T[] oldList = this.list;
		list = (T[]) new Object[size];
		//no element stored in the array
		if (count == 0)
			return;
		//copy data into new size.
		if (front < rear) {
			System.arraycopy(oldList, front, list, front, rear - front);
		} else {
			int head = size - (this.size - this.front);
			if (head < rear) {
				throw new IllegalArgumentException("data size is bigger then " + size + ".");
			}
			System.arraycopy(oldList, 0, list, 0, rear);
			System.arraycopy(oldList, front, list, head, this.size - this.front);
			front = head;
		}
	}


	public int getMaxSize() {
		return this.size;
	}
	

	 public void destroy() {
		 while (count!=0) {
			 try {
				Thread.sleep(1000);
			} catch (InterruptedException e) {
			}
		 } 
		 list = null;
	 }

	 
	@SuppressWarnings("unchecked")
	public void config(IConfigElement objectPoolconfElem) throws Exception {
		this.setName(objectPoolconfElem.getAttribute("name"));
		
		this.setMaxSize(Integer.parseInt(objectPoolconfElem.getAttribute("maxsize")));
		
	    this.list = (T[]) new Object[size];
		
	}


}
