package org.act.xservices.rt.xcontainer.framework.objectpool.impl;

import java.util.PriorityQueue;
import java.util.concurrent.locks.Condition;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;

import org.act.xservices.rt.xcontainer.framework.config.IConfigElement;
import org.act.xservices.rt.xcontainer.framework.objectpool.IObjectPool;

/**
 * ȼеĶ
 * 
 * @LastModifyTime 2008-7-8
 * @CreationTime 2008-7-8
 * @author Lizq [lizq@act.buaa.edu.cn]
 * @since JDK1.5
 */
public class ObjectPriorityQueuePool<T> implements IObjectPool<T> {
	
	private static final int DEFAULT_SIZE = 100;


	private int size = DEFAULT_SIZE;


	private PriorityQueue<T> priorityQueue;

	private volatile int count = 0;


	private final Lock lock = new ReentrantLock();


	private final Condition notFull = lock.newCondition();

	private final Condition notEmpty = lock.newCondition();
	

	private String name;

	public ObjectPriorityQueuePool() {

	}
	
	

	/**
	 * @return   the name
	 * 
	 */
	public String getName() {
		return name;
	}
	

	public void setName(String name) {
		this.name = name;
	}

	/**
	 * 
	 */
	public void put(T x) throws InterruptedException {
		lock.lockInterruptibly();
		try {
			while (count == size)
				notFull.await();
			priorityQueue.offer(x);
			++count;
			notEmpty.signalAll();
		} finally {
			lock.unlock();
		}
	}

	/**
	 * Thread safe.
	 */
	public T get() throws InterruptedException {
		lock.lockInterruptibly();
		try {
			while (count == 0)
				notEmpty.await();
			T t = priorityQueue.poll();
			--count;
			notFull.signalAll();
			return t;
		} finally {
			lock.unlock();
		}
	}

	//	/*
	//	 * @see org.act.webservice.IObjectPool#get()
	//	 */
	//	public synchronized T get() {
	//		lock.readLock().lock();
	//		while (empty()) {
	//			lock.readLock().unlock();
	//			try {
	//				readCondition.await();
	//			} catch (InterruptedException e) {
	//				return null;
	//			}
	//			lock.readLock().lock();
	//		}			
	//		front=(front+1)%size;
	//		T result = list[front];
	//		lock.readLock().unlock();
	//		count--;
	//		writeConditon.notify();
	//		return result;
	//	}

	//	/*
	//	 * @see org.act.webservice.IObjectPool#put(java.lang.Object)
	//	 */
	//	public synchronized void put(T obj) {
	//		lock.writeLock().lock();
	//		while(full()) {
	//			lock.writeLock().unlock();
	//			try {
	//				reentrantLock.lock();
	//				writeConditon.await();
	//			} catch (InterruptedException e) {
	//				//e.printStackTrace();
	//				return;
	//			}
	//			finally {
	//				reentrantLock.unlock();
	//			}
	//			lock.writeLock().lock();
	//		}
	//		rear=(rear+1)%size;
	//		list[rear]=obj;
	//		lock.writeLock().unlock();
	//		count++;
	//        readCondition.signal();
	//	}	


	public boolean isEmpty() {
		return count == 0;//front==rear;
	}


	public boolean isFull() {
		return count == size;
	}


	public synchronized int size() {
		return priorityQueue.size();
	}


	public synchronized void setMaxSize(int size) {
		//no need to create a new array
		if (this.size == size)
			return;
		
        this.size = size;
		return;

	}


	public int getMaxSize() {
		return this.size;
	}
	
	 /**
	  * 
	  */
	 public void destroy() {
		 while (count!=0) {
			 try {
				Thread.sleep(1000);
			} catch (InterruptedException e) {
			}
		 } 
		 priorityQueue = null;
	 }

	 
	public void config(IConfigElement objectPoolconfElem) throws Exception {
		
		this.setName(objectPoolconfElem.getAttribute("name"));
		
		this.setMaxSize(Integer.parseInt(objectPoolconfElem.getAttribute("maxsize")));
		
		this.priorityQueue = new PriorityQueue<T>(size);
		
	}



}
