package org.act.xservices.rt.xcontainer.framework.provider;

import org.act.xservices.rt.soap.ISOAPEnvelope;
import org.act.xservices.rt.soap.constants.SOAPConstants;
import org.act.xservices.rt.soap.util.SOAPMessageUtil;
import org.act.xservices.rt.xcontainer.framework.config.IConfigElement;
import org.act.xservices.rt.xcontainer.inf.context.IServiceContext;
import org.act.xservices.rt.xcontainer.inf.service.wsdl.XcService;


public abstract class AbstractProvider implements IProvider {

	protected String type;

	protected String mepURI;

	public void init(IConfigElement providerConfig, XcService xcService)
			throws Exception {
		this.type = providerConfig.getAttribute("type");
		this.mepURI = providerConfig.getAttribute("mep");
	}

	public IServiceContext process(IServiceContext serviceContext) {

		ClassLoader oldCL = Thread.currentThread().getContextClassLoader();

		try {

			Thread.currentThread().setContextClassLoader(
					serviceContext.getXcService().getServiceClassLoader());

			IServiceContext resultSC = invokeBusinessLogic(serviceContext);

			// з߼
			return resultSC;

		} catch (ProviderProcessException e) {
			// ޷ʵ߲Ҫõķ
			// Ϣ,ֹͣĴ

			ISOAPEnvelope faultMessage = SOAPMessageUtil.creatStdSimpleFaultMessage(
					serviceContext.getInMessage().getVersion(),
					SOAPConstants.FAULT_CODE_RECEIVER,
					"Error when invoke Service Inplementation in provider process. " + e.toString());
			serviceContext.setOutMessage(faultMessage);
			serviceContext.setFaultOutMessage(true);
			return serviceContext;

		} catch (ServiceProcessException e) {
			// ڷĵùг
			// Ϣ,,ֹͣĴ
			ISOAPEnvelope faultMessage = SOAPMessageUtil.creatStdSimpleFaultMessage(
					serviceContext.getInMessage().getVersion(),
					SOAPConstants.FAULT_CODE_RECEIVER,
					"Error when invoke Service Inplementation in service process. " + e.toString());
			serviceContext.setOutMessage(faultMessage);
			serviceContext.setFaultOutMessage(true);
			return serviceContext;

		} finally {
			Thread.currentThread().setContextClassLoader(oldCL);
		}

	}

	/**
	 * 
	 * @param serviceContext
	 * @return
	 * @throws ProviderProcessException
	 * @throws ServiceProcessException
	 */
	protected abstract IServiceContext invokeBusinessLogic(
			IServiceContext serviceContext) throws ProviderProcessException,
			ServiceProcessException;

	/**
	 * Retrieve the implementation object. This will either return a cached
	 * object if present in the XcService, or create a new one via
	 * makeNewServiceObject() (and then cache that).
	 * 
	 * @param msgContext
	 *            the active MessageContext
	 * @return the appropriate back-end service object.
	 * @throws AxisFault
	 *             if there's a problem
	 */
	protected Object getImplementationObject(XcService xcService)
			throws ProviderProcessException {

		if (XcService.SCOPE_REQUEST.equalsIgnoreCase(xcService.getScope())) {
			// ÿһµʵ
			return makeNewServiceObject(xcService);

		} else if (XcService.SCOPE_APPLICATION.equalsIgnoreCase(xcService.getScope())) {
			// һʵ
			Object serviceimpl = xcService.getParameter(XcService.SERVICE_OBJECT);

			if (serviceimpl == null) {
				serviceimpl = makeNewServiceObject(xcService);
				xcService.addParameter(XcService.SERVICE_OBJECT, serviceimpl);
			}

			return serviceimpl;

		} else {

			// ·µʵ
			return makeNewServiceObject(xcService);
		}
	}

	private Object makeNewServiceObject(XcService xcService)
			throws ProviderProcessException {

		try {
			Object serviceimpl = xcService.getServiceClass().newInstance();
			return serviceimpl;
		} catch (InstantiationException e) {
			e.printStackTrace();
			throw new ProviderProcessException(e);
		} catch (IllegalAccessException e) {
			e.printStackTrace();
			throw new ProviderProcessException(e);
		}
	}

	public String getMepURI() {
		return mepURI;
	}

	public void setMepURI(String mepURI) {
		this.mepURI = mepURI;
	}

	public String getType() {
		return type;
	}

	public void setType(String type) {
		this.type = type;
	}

}
