package org.act.xservices.rt.xcontainer.framework.provider;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.act.xservices.log.Logger;
import org.act.xservices.log.LoggerFactory;
import org.act.xservices.rt.xcontainer.framework.config.IConfigElement;
import org.act.xservices.rt.xcontainer.framework.config.IConfigurable;

public class ProviderFactory implements IConfigurable {

	final Logger logger = LoggerFactory.getLogger(ProviderFactory.class);

	/** Ϣӳ mepURI-->List<ProviderInfo> */
	private Map<String, List<ProviderInfo>> providerInfoMap = new HashMap<String, List<ProviderInfo>>();

	/**
	 * ôMEPURIӦĬϵProviderType
	 * 
	 * @param mepURI
	 * @return
	 */
	public String getDefaultProviderType(String mepURI) {

		ProviderInfo pro = getProviderInfo(mepURI, null);

		if (pro == null) {
			return null;
		} else {
			return pro.getType();
		}
	}

	/**
	 * ͨmepURITypeöӦProvider typeΪnull򷵻ĬϵĶӦMEPRawXMLProvider
	 * mepURIΪnulltypeвҡ typemepURIΪnull򷵻null
	 * 
	 * @param mepURI
	 * @param type
	 * @return
	 */
	public ProviderInfo getProviderInfo(String mepURI, String type) {

		if (mepURI == null) {
			if (type == null || "".equals(type)) {
				return null; // ޷Provider
			}

			// typeProvider
			for (List<ProviderInfo> proList : providerInfoMap.values()) {
				for (ProviderInfo pro : proList) {
					if (type.equals(pro.getType()))
						return pro;
				}
			}

		} else {
			// ȸMEP
			List<ProviderInfo> providerList = getProviderInfoByMepURI(mepURI);

			if (providerList == null || providerList.isEmpty()) {
				return null;
			}

			if (type == null || "".equals(type)) {
				// ĬϵRawXMLProvider
				for (ProviderInfo prod : providerList) {
					if (prod.getType().startsWith("RawXML")) {
						return prod;
					}
				}

			} else {
				// ҶӦtypeƵProvider
				for (ProviderInfo prod : providerList) {
					if (prod.getType().equals(type)) {
						return prod;
					}
				}
			}
		}

		// ûҵӦProvider
		return null;
	}

	public void config(IConfigElement providersElement) throws Exception {

		for (IConfigElement providerConfig : providersElement.getChildren()) {

			String type = providerConfig.getAttribute("type");
			String mepURI = providerConfig.getAttribute("mep");
			String providerClass = providerConfig.getAttribute("class");

			try {
				if (mepURI != null && providerClass != null && type != null) {

					ProviderInfo proInfo = new ProviderInfo();
					proInfo.setMepURI(mepURI);
					proInfo.setType(type);
					proInfo.setProviderClassName(providerClass);
					proInfo.setProviderConfig(providerConfig);

					addProviderInfo(proInfo);

					logger.info("Succeed add Provider: " + type);
				} else {
					logger.info("Error Provider Parameter: " + type + " "
							+ mepURI + " " + providerClass);
				}
			} catch (Exception e) {
				logger.info("Failed add Provider: " + type);
				e.printStackTrace();
			}
		}
	}

	private void addProviderInfo(ProviderInfo providerInfo) {

		// modified by viking 2010 07 22, I think this not necessary.
		// WSDLUtil.getStandardMepURI(providerInfo.getMepURI());
		String standardMepURI = providerInfo.getMepURI();

		List<ProviderInfo> providerList = getProviderInfoByMepURI(standardMepURI);

		if (providerList == null) {
			providerList = new ArrayList<ProviderInfo>();
			providerList.add(providerInfo);
			providerInfoMap.put(standardMepURI, providerList);
		} else {
			providerList.add(providerInfo);
		}
	}

	private List<ProviderInfo> getProviderInfoByMepURI(String mepURI) {

		//
		return providerInfoMap.get(mepURI);
	}

	/**
	 * ProviderϢ
	 * 
	 * @LastModifyTime 2009-4-23
	 * @CreationTime
	 * @author Lizq [lizq@act.buaa.edu.cn]
	 * @since JDK1.5
	 */
	public class ProviderInfo {

		private String providerClassName;

		private String type;

		private String mepURI;

		private IConfigElement providerConfig;

		public String getMepURI() {
			return mepURI;
		}

		public void setMepURI(String mepURI) {
			this.mepURI = mepURI;
		}

		public IConfigElement getProviderConfig() {
			return providerConfig;
		}

		public void setProviderConfig(IConfigElement providerConfig) {
			this.providerConfig = providerConfig;
		}

		public String getType() {
			return type;
		}

		public void setType(String type) {
			this.type = type;
		}

		public String getProviderClassName() {
			return providerClassName;
		}

		public void setProviderClassName(String className) {
			this.providerClassName = className;
		}
	}

}
