package org.act.xservices.rt.xcontainer.framework.provider;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import org.act.xservices.rt.xcontainer.inf.context.IServiceContext;
import org.act.xservices.rt.xcontainer.inf.service.wsdl.XcOperation;
import org.act.xservices.rt.xom.Metaset;

/**
 * @LastModifyTime 2009-4-15
 * @CreationTime
 * @author Lizq [lizq@act.buaa.edu.cn]
 * @since JDK1.5
 */
public class RawXMLInOnlyProvider extends AbstractInProvider{

	/** 
	 * 
	 * 
	 * @param serviceContext
	 * @return
	 * @throws ProviderProcessException
	 * @throws ServiceProcessException
	 */
	public IServiceContext invokeBusinessLogic(IServiceContext serviceContext
			) throws ProviderProcessException, ServiceProcessException{
		
		Class<?> serviceClass = serviceContext.getXcService().getServiceClass();
		
        // get the implementation class for the Web Service
        Object obj = getImplementationObject(serviceContext.getXcService());
		
		XcOperation xcOperation = serviceContext.getXcOperation();
        
		Method method = findMethod(xcOperation, serviceClass);
        
        if (method == null) {
            throw new ProviderProcessException("Method Does Not Exist for "
            		+ xcOperation.getQName());
        }
		        
        Metaset parameterMetaset = serviceContext.getInMessageBody().getFirstElement();
		
        if(parameterMetaset == null){
			throw new ProviderProcessException("Error Parameter in InSOAPMessage for Operation "
					+ xcOperation.getQName() );
		}
        
		try {
			
			method.invoke( obj, new Object[] {parameterMetaset} );
				
			serviceContext.setOutMessage(null);
			
			return serviceContext;
			
		} catch (IllegalArgumentException e) {
			throw new ServiceProcessException(e);
		} catch (IllegalAccessException e) {
			throw new ServiceProcessException(e);
		} catch (InvocationTargetException e) {
			throw new ServiceProcessException(e);
		}
	}
	
	  /**
     * Looking for a method of the form "Metaset method(Metaset)"
     * 
     * @param xcop
     * @param implClass
     * @return
     */
    protected Method findMethod(XcOperation xcop, Class<?> implClass) 
    		throws ProviderProcessException{
    	
        Method method = xcop.getMethod();
        
        if (method != null) 
        	return method;

        String methodName = xcop.getQName().getName();

        try {
            // Looking for a method of the form "void method(Metaset)"
            method = implClass.getMethod(methodName, new Class [] { Metaset.class });
           
            if (method.getReturnType().equals(void.class)) {
                
            	xcop.setMethod(method);
               
                return method;
            }
            
        } catch (NoSuchMethodException e) {
        	throw new ProviderProcessException(
        			"Error: Can not find method: "
        			+methodName+ " in class: " +implClass.getName() , e);
        }

        return null;
    }
}
