package org.act.xservices.rt.xcontainer.inf.service.wsdl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.act.xservices.rt.xom.QName;
import org.apache.ws.commons.schema.extensions.ExtensionRegistry;

/**
 * 
 * @version 1.0 2007-09-25
 * @author Lizq [lizq@act.buaa.edu.cn]
 * @since JDK1.5
 */
public abstract class AWSDLElement implements Serializable,
		IAttributeExtensible, IElementExtensible {

	/** */
	private static final long serialVersionUID = 1L;

	/** WSDLеչԪ */
	protected List<ExtenseElement> extElements = new ArrayList<ExtenseElement>();

	/** WSDLеչ */
	protected Map<QName, String> extensionAttributes = new HashMap<QName, String>();

	/**
	 * Add an extensibility element.
	 * 
	 * @param extElement
	 *            the extensibility element to be added
	 */
	public boolean addExtensibilityElement(ExtenseElement extElement) {
		return extElements.add(extElement);
	}

	/**
	 * Remove an extensibility element.
	 * 
	 * @param extElement
	 *            the extensibility element to be removed.
	 * @return the extensibility element which was removed.
	 */
	public ExtenseElement removeExtensibilityElement(ExtenseElement extElement) {
		if (extElements.remove(extElement)) {
			return extElement;
		} else {
			return null;
		}
	}

	/**
	 * Get all the extensibility elements defined here.
	 */
	public List<ExtenseElement> getExtensibilityElements() {
		return Collections.unmodifiableList(extElements);
	}

	/**
	 * ÷δʵ
	 */
	public List<ExtenseElement> getExtensibilityElementsByName(QName elementName) {
		return null;
	}

	/**
	 * Set an extension attribute on this element. Pass in a null value to
	 * remove an extension attribute.
	 * 
	 * @param name
	 *            the extension attribute name
	 * @param value
	 *            the extension attribute value. Can be a String, a QName, a
	 *            List of Strings, or a List of QNames.
	 * 
	 * @see #getExtensionAttribute
	 * @see #getExtensionAttributes
	 * @see ExtensionRegistry#registerExtensionAttributeType
	 * @see ExtensionRegistry#queryExtensionAttributeType
	 */
	public void setExtensionAttribute(QName name, String value) {
		if (value != null) {
			extensionAttributes.put(name, value);
		} else {
			extensionAttributes.remove(name);
		}
	}

	/**
	 * Retrieve an extension attribute from this element. If the extension
	 * attribute is not defined, null is returned.
	 * 
	 * @param name
	 *            the extension attribute name
	 * 
	 * @return the value of the extension attribute, or null if it is not
	 *         defined. Can be a String, a QName, a List of Strings, or a List
	 *         of QNames.
	 * 
	 * @see #setExtensionAttribute
	 * @see #getExtensionAttributes
	 * @see ExtensionRegistry#registerExtensionAttributeType
	 * @see ExtensionRegistry#queryExtensionAttributeType
	 */
	public String getExtensionAttribute(QName name) {
		return extensionAttributes.get(name);
	}

	/**
	 * Get the map containing all the extension attributes defined on this
	 * element. The keys are the qnames of the attributes.
	 * 
	 * @return a map containing all the extension attributes defined on this
	 *         element
	 * 
	 * @see #setExtensionAttribute
	 * @see #getExtensionAttribute
	 */
	public Map<QName, String> getExtensionAttributes() {
		return extensionAttributes;
	}

	public String toString() {
		StringBuffer strBuf = new StringBuffer();

		if (extElements.size() > 0) {
			Iterator<ExtenseElement> extIterator = extElements.iterator();

			if (extIterator.hasNext()) {
				strBuf.append(extIterator.next());

				while (extIterator.hasNext()) {
					strBuf.append("\n");
					strBuf.append(extIterator.next());
				}
			}
		}

		if (extensionAttributes.size() > 0) {
			Iterator<QName> keys = extensionAttributes.keySet().iterator();

			if (keys.hasNext()) {
				QName name = (QName) keys.next();

				strBuf.append("extension attribute: ");
				strBuf.append(name);
				strBuf.append("=");
				strBuf.append(extensionAttributes.get(name));

				while (keys.hasNext()) {
					name = (QName) keys.next();

					strBuf.append("\n");
					strBuf.append("extension attribute: ");
					strBuf.append(name);
					strBuf.append("=");
					strBuf.append(extensionAttributes.get(name));
				}
			}
		}

		return strBuf.toString();
	}

}
