package org.act.xservices.rt.xcontainer.inf.service.wsdl;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.act.xservices.rt.xcontainer.framework.policy.IPolicyProcessor;
import org.act.xservices.rt.xcontainer.framework.provider.IProvider;
import org.act.xservices.rt.xcontainer.inf.service.wsdl.schema.ServiceSchema;
import org.act.xservices.rt.xom.QName;
import org.apache.ws.commons.schema.XmlSchema;
import org.apache.ws.commons.schema.utils.NamespaceMap;

/**
 * еһ
 * 
 * @LastModifyTime 2009-04-14
 * @CreationTime 2007-10-20
 * @author Lizq [lizq@act.buaa.edu.cn]
 * @since JDK1.5
 */

/**
 * @LastModifyTime 2009-9-30
 * @CreationTime
 * @author Lizq [lizq@act.buaa.edu.cn]
 * @since JDK1.5
 */
public class XcService extends XcElement {

	/** serialize */
	private static final long serialVersionUID = 1L;

	// Service Type
	public static final String SERVICE_TYPE_WS = "wsService";
	public static final String SERVICE_TYPE_BPEL = "bpelService";

	/** ServiceType, default is SERVICE_TYPE_WS */
	private String serviceType = SERVICE_TYPE_WS;

	/** QName */
	private QName qName;

	/** WSDLз */
	private String serviceName;

	// to keep the service target name space
	private String targetNamespace;

	/** Map of prefix -> namespaceURI */
	private NamespaceMap namespaceMap;

	private List<String> importedNamespaces;

	// to keep the XMLScheam getting either from WSDL
	private List<XmlSchema> schemaList = new ArrayList<XmlSchema>();

	// ע⣺һServicePortʾһӿڣportTypeʵ֣BindingӦһServiceURIĶ˵ַ
	// ˾͵¿ܶBindingӦһServiceURIζһʵֶӿڣJavaеһʵֶӿ
	/** endpointName(portName) --> XcEndpoint */
	private Map<String, XcEndpoint> xcEndpointMap = new HashMap<String, XcEndpoint>();

	private Map<QName, XcOperation> xcOperationMap = new HashMap<QName, XcOperation>();

	// A map of (String alias, XcOperation operation). The aliases might
	// include: SOAPAction,
	// WS-Addressing action, the operation name, the XcInputMessage name. See:
	// - invalidOperationsAliases
	// - mapActionToOperatoin()
	// - getOperationByAction()
	// REVIEW: This really should be seperate maps for the different types of
	// aliases so they don't
	// conflict with each other. For example, so that an identical operation
	// name and soap action
	// on different operatoins don't cause a collision; the following can't be
	// routed because
	// "foo" is not unique across different operations:
	// operation 1: action = foo, name = bar
	// operation 2: action = bar, name = foo
	private HashMap<String, XcOperation> operationsAliasesMap = new HashMap<String, XcOperation>();

	// Collection of aliases that are invalid for this service because they are
	// duplicated across multiple operations under this service.
	private List<String> invalidOperationsAliases = new ArrayList<String>();

	/** XcServiceӦWSDLĵ */
	private File wsdlFile;

	// ----------WSDD------------
	public static final String SERVICE_OBJECT = "serviceObject";
	public static final String SCOPE_REQUEST = "request";
	public static final String SCOPE_APPLICATION = "application";

	private String description;

	private String scope = SCOPE_APPLICATION;

	private ClassLoader serviceClassLoader;

	private Class<?> serviceClass;

	// workingļµķĿ¼
	private String wsarLocation = null;

	/** type-->IProvider */
	private Map<String, IProvider> providerMap = new HashMap<String, IProvider>();

	// ---------end-WSDD---------

	// ---------Policy--------by lizq 2010-07-02---
	private List<IPolicyProcessor> policyProcessorChain;

	public boolean addPolicyProcessor(IPolicyProcessor policyProcessor) {
		if (policyProcessorChain == null) {
			policyProcessorChain = new ArrayList<IPolicyProcessor>();
		}
		return policyProcessorChain.add(policyProcessor);
	}

	public boolean removePolicyProcessor(IPolicyProcessor policyProcessor) {
		if (policyProcessorChain == null) {
			return false;
		}
		return policyProcessorChain.remove(policyProcessor);
	}

	public List<IPolicyProcessor> getPolicyProcessor() {
		return policyProcessorChain;
	}

	// --------end-Policy--------------------------

	public XcService() {
	}

	public XcService(QName qname) {
		this.qName = qname;
	}

	public XcService(String serviceName) {
		this.serviceName = serviceName;
	}

	public XcService(String serviceName, ClassLoader classLoader,
			String wsarLocation) {
		this.serviceName = serviceName;
		this.serviceClassLoader = classLoader;
		this.wsarLocation = wsarLocation;
	}

	// ---------------------------

	public QName getQName() {
		return qName;
	}

	public void setQName(QName serviceQName) {
		this.qName = serviceQName;
	}

	public String getServiceType() {
		return serviceType;
	}

	public void setServiceType(String serviceType) {
		this.serviceType = serviceType;
	}

	public void setNamespaceMap(NamespaceMap namespaceMap) {
		this.namespaceMap = namespaceMap;
	}

	/**
	 * Get the namespace map for this service.
	 * 
	 * @return a Map of prefix (String) to namespace URI (String)
	 */
	public Map getNamespaceMap() {
		return namespaceMap;
	}

	/**
	 * Get the namespaces associated with imported WSDLs
	 * 
	 * @return a <code>List</code> of namespace URIs (String)
	 */
	public List<String> getImportedNamespaces() {
		return importedNamespaces;
	}

	/**
	 * Set the namespaces associated with imported WSDLs
	 * 
	 * @param importedNamespaces
	 */
	public void setImportedNamespaces(List<String> importedNamespaces) {
		this.importedNamespaces = importedNamespaces;
	}

	public List<XmlSchema> getSchema() {
		return schemaList;
	}

	public void addSchema(XmlSchema schema) {
		if (schema != null) {
			schemaList.add(schema);
			if (schema.getTargetNamespace() != null) {
				addSchemaNameSpace(schema);
			}
		}
	}

	private int nsCount = 0;

	/**
	 * 
	 * @param schema
	 */
	@SuppressWarnings("unchecked")
	private void addSchemaNameSpace(XmlSchema schema) {
		String targetNameSpace = schema.getTargetNamespace();

		String prefix = schema.getNamespaceContext().getPrefix(targetNameSpace);

		if (namespaceMap == null) {
			namespaceMap = new NamespaceMap();
		}

		if (!namespaceMap.values().contains(targetNameSpace)) {
			// i.e this target namespace not exists in the namesapce map
			// find a non exists prefix to add this target namesapce
			while ((prefix == null) || namespaceMap.keySet().contains(prefix)) {
				prefix = "ns" + nsCount++;
			}
			namespaceMap.put(prefix, targetNameSpace);
		}
	}

	public XcEndpoint getXcEndpoint(String portName) {
		return xcEndpointMap.get(portName);
	}

	public List<XcEndpoint> getAllXcEndpoints() {
		return new ArrayList<XcEndpoint>(xcEndpointMap.values());
	}

	public XcEndpoint addXcEndpoint(XcEndpoint xcEndpoint) {
		return xcEndpointMap.put(xcEndpoint.getQName().getName(), xcEndpoint);

	}

	public XcEndpoint removeXcEndpoint(XcEndpoint xcEndpoint) {
		return xcEndpointMap.remove(xcEndpoint.getQName().getName());
	}

	public Set<String> getAllSupportURLPath() {
		Set<String> url = new HashSet<String>();
		for (XcEndpoint xcep : xcEndpointMap.values()) {
			url.add(xcep.getEndpointURLPath());
		}
		return url;
	}

	/**
	 * Method getOperation.
	 * 
	 * @param operationName
	 * @return Returns AxisOperation.
	 */
	public XcOperation getXcOperation(QName operationQName) {

		XcOperation xcOperation = xcOperationMap.get(operationQName);

		if (xcOperation == null) {
			xcOperation = xcOperationMap.get(new QName(getTargetNamespace(),
					operationQName.getName(), ""));
		}

		if (xcOperation == null) {
			xcOperation = operationsAliasesMap.get(operationQName.getName());
		}

		// The operation may be associated with a namespace other than the
		// target namespace, e.g. if the operation is from an imported wsdl.
		if (xcOperation == null) {
			List<String> namespaces = getImportedNamespaces();
			if (namespaces != null) {
				for (String namespace : namespaces) {
					xcOperation = xcOperationMap.get(new QName(namespace,
							operationQName.getName(), ""));
					if (xcOperation != null)
						break;
				}
			}
		}

		return xcOperation;
	}

	public List<XcOperation> getAllXcOperation() {
		return new ArrayList<XcOperation>(xcOperationMap.values());
	}

	/**
	 * Returns the XcOperation which has been mapped to the given alias.
	 * 
	 * @see #mapActionToOperation(String, XcOperation)
	 * 
	 * @param action
	 *            the alias key
	 * @return Returns the corresponding XcOperation or null if it isn't found.
	 */
	public XcOperation getOperationByAction(String action) {

		if ((action == null) || (action.length() == 0)) {
			return null;
		}

		if (invalidOperationsAliases.contains(action)) {
			return null;
		}

		return operationsAliasesMap.get(action);
	}

	/**
	 * XcOperationӵxcOperationMapԼoperationsAliasesMap
	 * 
	 * @param xcOperation
	 */
	public void addXcOperation(XcOperation xcOperation) {
		xcOperationMap.put(xcOperation.getQName(), xcOperation);

		// add operation localname
		mapActionToXcOperation(xcOperation.getQName().getName(), xcOperation);

		// add operation SOAPAction
		String soapAction = xcOperation.getSoapAction();
		if (soapAction != null && !"".equals(soapAction)) {
			mapActionToXcOperation(soapAction, xcOperation);
		}

		// add operation inMessage wsaAction
		String wsaAction = xcOperation.getInputXcMessage().getWSA_Action();
		if (wsaAction != null && !"".equals(wsaAction)) {
			mapActionToXcOperation(wsaAction, xcOperation);
		}

	}

	/**
	 * Maps an alias (such as a SOAPAction, WSA action, or an operation name) to
	 * the given AxisOperation. This is used by dispatching (both SOAPAction-
	 * and WSAddressing- based dispatching) to figure out which operation a
	 * given message is for. Some notes on restrictions of "action" - A null or
	 * empty action will be ignored - An action that is a duplicate and
	 * references an idential operation is allowed - An acton that is a
	 * duplicate and references a different operation is NOT allowed. In this
	 * case, the action for the original operation is removed from the alias
	 * table, thus removing the ability to route based on this action. This is
	 * necessary to prevent mis-directing incoming message to the wrong
	 * operation based on SOAPAction.
	 * 
	 * Note that an alias could be a SOAPAction, WS-Addressing Action, the
	 * operation name, or some other alias.
	 * 
	 * @see #getOperationByAction(String)
	 * 
	 * @param action
	 *            the alias key
	 * @param xcOperation
	 *            the operation to map to
	 */
	public void mapActionToXcOperation(String action, XcOperation xcOperation) {

		if (action == null || "".equals(action)) {
			return;
		}

		// First check if this action has already been flagged as invalid
		// because it is a duplicate.
		if (invalidOperationsAliases.contains(action)) {
			// This SOAPAction has already been determined to be invalid; log a
			// message and do not add it to the operation alias map.
			return;
		}

		// Check if the action is currently mapping to an operation.
		XcOperation currentlyMappedOperation = getOperationByAction(action);

		if (currentlyMappedOperation != null) {
			if (currentlyMappedOperation == xcOperation) {
				// This maps to the same operation, then it is already in the
				// alias table, so just silently ignore this mapping request.
				return;
			} else {
				// This action is already mapped, but it is to a different
				// operation.
				// Remove the action mapping from the alias table and add it to
				// the
				// list of invalid mappings
				operationsAliasesMap.remove(action);
				invalidOperationsAliases.add(action);
				return;
			}
		} else {
			operationsAliasesMap.put(action, xcOperation);
		}
	}

	public void removeXcOperation(QName operQName) {

		XcOperation xcOperation = getXcOperation(operQName);

		if (xcOperation != null) {
			xcOperationMap.remove(xcOperation.getQName());

			operationsAliasesMap.remove(xcOperation.getQName().getName());

			if (xcOperation.getSoapAction() != null
					&& !"".equals(xcOperation.getSoapAction())) {
				operationsAliasesMap.remove(xcOperation.getSoapAction());
			}

			String wsaAction = xcOperation.getInputXcMessage().getWSA_Action();
			if (wsaAction != null && !"".equals(wsaAction)) {
				operationsAliasesMap.remove(wsaAction);
			}

		}

	}

	// --------------------

	public ClassLoader getServiceClassLoader() {
		return serviceClassLoader;
	}

	public void setServiceClassLoader(ClassLoader classLoader) {
		this.serviceClassLoader = classLoader;
	}

	public String getServiceName() {
		return serviceName;
	}

	public void setServiceName(String serviceName) {
		this.serviceName = serviceName;

	}

	public String getWsarLocation() {
		return wsarLocation;
	}

	public void setWsarLocation(String wsarLocation) {
		this.wsarLocation = wsarLocation;
	}

	public Class<?> getServiceClass() {
		return serviceClass;
	}

	public void setServiceClass(Class<?> serviceClass) {
		this.serviceClass = serviceClass;
	}

	// ---------------------------------------

	public String getTargetNamespace() {
		return targetNamespace;
	}

	public void setTargetNamespace(String targetNamespace) {
		this.targetNamespace = targetNamespace;
	}

	public File getWsdlFile() {
		return wsdlFile;
	}

	public void setWsdlFile(File wsdlFile) {
		this.wsdlFile = wsdlFile;
	}

	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public String getScope() {
		return scope;
	}

	public void setScope(String scope) {
		this.scope = scope;
	}

	public void addProvider(IProvider provider) {
		providerMap.put(provider.getType(), provider);
	}

	public IProvider getProvider(String type) {
		return providerMap.get(type);
	}

	// ================= add by viking ======================//
	private boolean elementFormDefault = true;

	public void setElementFormDefault(boolean elementFormDefault) {
		this.elementFormDefault = elementFormDefault;
	}

	public boolean isElementFormDefault() {
		return elementFormDefault;
	}

	private String targetPrefix;

	public void setTargetPrefix(String targetPrefix) {
		this.targetPrefix = targetPrefix;
	}

	public String getTargetPrefix() {
		return targetPrefix;
	}

	private ServiceSchema serviceSchema = null;

	public ServiceSchema getServiceSchema() {
		return serviceSchema;
	}

	public void setServiceSchema(ServiceSchema serviceSchema) {
		this.serviceSchema = serviceSchema;
	}
	// ================= add by viking ======================//
}
