
package org.act.xservices.rt.xcontainer.message;

import java.util.ArrayList;
import java.util.List;

import org.act.xservices.rt.xcontainer.framework.AConnector;
import org.act.xservices.rt.xcontainer.framework.config.IConfigElement;
import org.act.xservices.rt.xcontainer.framework.dispatch.ITaskFactory;
import org.act.xservices.rt.xcontainer.inf.context.IATFTransportContext;
import org.act.xservices.rt.xcontainer.inf.context.IMessageContext;
import org.act.xservices.rt.xcontainer.message.handler.AbstractMessageHandler;
import org.act.xservices.rt.xcontainer.message.handler.MessageValidateHandler;
import org.act.xservices.rt.xcontainer.message.handler.MessageSerializeHandler;


/**
 * Ϣ
 * 
 * @LastModifyTime 2009-10-02
 * @CreationTime 2007-05-28
 * @author Lizq [lizq@act.buaa.edu.cn]
 * @since JDK1.5
 */
public class MessageConnector extends AConnector<IATFTransportContext, IMessageContext> {

	/** Ϣл뷴л */
	private MessageSerializeHandler messageSerializeHandler;

	/**  */
	private MessageValidateHandler messagePoolHandler;

	/** ϢеûԶHandler */
	private List<AbstractMessageHandler> messageHandlerList;

	
	
	public MessageConnector(String connectorName) {
		super(connectorName);
	}

	public MessageConnector() {

	}

	/**
	 * ڴ˴ʼеIHandler
	 * 
	 * @see org.act.xservices.rt.xcontainer.framework.AConnector#initLocal()
	 */
	protected void doInitLocal() {
	
		//ʼ transportPoolHandler, IHandlers<IMessageContext>[] messageHandlerChains, messagePoolHandler
		
		IConfigElement handlerChainsConfigElement = 
			this.connectorConfiguration.getFirstChildWithName("handlerChain");
		try {
			//messageSerializeHandler
			messageSerializeHandler = new MessageSerializeHandler();
			messageSerializeHandler.config(null); //ٸⲿļ
			messageSerializeHandler.setTransportOut(getOutputTarget());
			messageSerializeHandler.setMessageFormatterFactory(
					((MessageConnectorEnvironment)this.connectorEnvironment).getMessageFormatterFactory());
			messageSerializeHandler.setMessageBuilderFactory(
					((MessageConnectorEnvironment)this.connectorEnvironment).getMessageBuilderFactory());
		
			//messagePoolHandler
			messagePoolHandler = new MessageValidateHandler();
			messagePoolHandler.config(null); //ٸⲿļ
			messagePoolHandler.setMessageIn(getInputTarget());
			
			//AbstractMessageHandler
			IConfigElement[] chainHandlersConfigElement = handlerChainsConfigElement.getChildren();
			messageHandlerList = new ArrayList<AbstractMessageHandler>();
			
			AbstractMessageHandler temp = null;
			//ʼеĴ HandlerChain аװ
			for(int i=0; i<chainHandlersConfigElement.length; i++){
				if(chainHandlersConfigElement[i].getAttribute("enable").equalsIgnoreCase("false"))
					continue;
				temp = (AbstractMessageHandler) Class.forName(chainHandlersConfigElement[i].getAttribute("class")).newInstance();
				temp.config(chainHandlersConfigElement[i]); //дĳʼ
				messageHandlerList.add(temp); //ӿõHandler
			}//
		} catch (InstantiationException e) {
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			e.printStackTrace();
		} catch (Exception e) {
			e.printStackTrace();
		}
		
		//֮Ĺϵ
		
		// ûд
		if(messageHandlerList == null || messageHandlerList.size() == 0){
			messageSerializeHandler.setNextInHandler(messagePoolHandler);
			messagePoolHandler.setNextOutHandler(messageSerializeHandler);
		}else{  //д
			messageSerializeHandler.setNextInHandler(messageHandlerList.get(0));
			messagePoolHandler.setNextOutHandler(messageHandlerList.get(messageHandlerList.size()-1));
			
			messageHandlerList.get(0).setNextOutHandler(messageSerializeHandler);
			messageHandlerList.get(messageHandlerList.size()-1).setNextInHandler(messagePoolHandler);
			//
			if (messageHandlerList.size() > 1) {
				messageHandlerList.get(0).setNextInHandler(messageHandlerList.get(1));
				messageHandlerList.get(messageHandlerList.size()-1).setNextOutHandler(
					messageHandlerList.get(messageHandlerList.size()-2));
				for (int i = 1; i < messageHandlerList.size()-1; i++) {
					messageHandlerList.get(i).setNextInHandler(messageHandlerList.get(i+1));
					messageHandlerList.get(i).setNextOutHandler(messageHandlerList.get(i-1));
				}
			}//end of if
		}
	}


	public ITaskFactory<IATFTransportContext> getInputTaskFactory() {
		MessageInTaskFactory mInDHFactory = new MessageInTaskFactory(this);
		mInDHFactory.setTransportPoolHandler(messageSerializeHandler);
		return mInDHFactory;
	}

	public ITaskFactory<IMessageContext> getOutputTaskFactory() {
		MessageOutTaskFactory mOutDHFactory = new MessageOutTaskFactory(this);
		mOutDHFactory.setMessagePoolHandler(messagePoolHandler);
		return mOutDHFactory;
	}
	
	/** 
	 *
	 */
	public void destroy() {	
		messageSerializeHandler.destroy();
		for(AbstractMessageHandler mh : messageHandlerList){
			mh.destroy();
		}
		messagePoolHandler.destroy();
	}
	
}
