package org.act.xservices.rt.xcontainer.message.formatter;

import java.io.UnsupportedEncodingException;
import java.nio.charset.Charset;
import java.nio.charset.IllegalCharsetNameException;
import java.nio.charset.UnsupportedCharsetException;

import org.act.xservices.rt.metaset.MetasetBuilder;
import org.act.xservices.rt.soap.ISOAPEnvelope;
import org.act.xservices.rt.xcontainer.inf.context.IATFTransportContext;
import org.act.xservices.rt.xcontainer.inf.context.IMessageContext;
import org.act.xservices.rt.xom.Document;

/**
 * @LastModifyTime 2009-4-23
 * @CreationTime
 * @author Lizq [lizq@act.buaa.edu.cn]
 * @since JDK1.5
 */
public abstract class AMessageFormatter implements IMessageFormatter {

	protected String contentType;

	public String getContentType() {
		return contentType;
	}

	public void setContentType(String contentType) {
		this.contentType = contentType;
	}

	/**
	 * MessageContextTransportContext
	 * 
	 * @param messageContext
	 * @return
	 * @throws SOAPFormatException
	 */
	public IATFTransportContext formatTransportContext(
			IMessageContext messageContext) throws SOAPFormatException {

		String requestContentType = messageContext.getTransportcontext()
				.getRequestContentType();

		String charset = null;

		int index = requestContentType.indexOf("=");
		if (index != -1) {
			charset = requestContentType.substring(index + 1).trim();
		} else {
			charset = messageContext.getTransportcontext().getParameter(
					"Content-Encoding");
			if(charset==null){
				charset = "UTF-8";
			}
		}

		try {
			Charset.forName(charset); // жǷڴcharset

		} catch (IllegalCharsetNameException e) {
			e.printStackTrace();
			charset = "UTF-8";

		} catch (UnsupportedCharsetException e) {
			e.printStackTrace();
			charset = "UTF-8";
		}

		IATFTransportContext tranContext = messageContext.getTransportcontext();

		// TODO lizq 2008-10-10 ˴ӲhttpҪ޸
		if (tranContext.getATFTransportSession().getTransportIdentifier()
				.equals("http")) {
			tranContext.setResponseContentType(contentType + ";charset="
					+ charset); // дչ
		}

		// עûHTTPӦ

		// Ӧ
		byte[] contentByte = creatBytesContent(messageContext, charset);

		if (contentByte == null) { // Ӧĳ
			throw new SOAPFormatException("Error in SOAP Message Format!");
		}

		// InOnly

		tranContext.setResponseContent(contentByte);

		return tranContext;

	}

	/**
	 *  IMessageContext ֽʽ,null ΪInOnlyģʽؿ
	 * 
	 * @param messageContext
	 * @return
	 */
	private byte[] creatBytesContent(IMessageContext messageContext,
			String encodeStyle) {

		ISOAPEnvelope responseMetaset = messageContext.getSoapMessage();		
		if (responseMetaset == null)
			return new byte[0];// OutOnly

		else{
			responseMetaset.detach();
		}
		// Ҫı뷽ʽб
		Document doc = MetasetBuilder.getInstance().createDocument();
		doc.setXMLEncoding(encodeStyle);
		doc.add(responseMetaset);

		try {

			byte[] responseBytes = doc.asXML().getBytes(encodeStyle);

			return responseBytes;

		} catch (UnsupportedEncodingException e) {
			System.err.println("UnsupportedEncoding " + encodeStyle
					+ " for message:\n" + doc.asXML());
			System.err.println("Use default Encoded UTF-8");
			try {
				return doc.asXML().getBytes("UTF-8");
			} catch (UnsupportedEncodingException e1) {
				e1.printStackTrace();
				return null;
			}
		}
	}

}
