package org.act.xservices.rt.xcontainer.message.formatter;

import java.util.HashMap;
import java.util.Map;

import org.act.xservices.log.Logger;
import org.act.xservices.log.LoggerFactory;
import org.act.xservices.rt.xcontainer.framework.config.IConfigElement;
import org.act.xservices.rt.xcontainer.framework.config.IConfigurable;

public class MessageFormatterFactory implements IConfigurable{

	final Logger logger = LoggerFactory.getLogger(MessageFormatterFactory.class);
	
	
	private Map<String, IMessageFormatter> formattersMap = new HashMap<String, IMessageFormatter>();
	
	
	public void config(IConfigElement formattersElement) throws Exception {

		for (IConfigElement formatterConfig : formattersElement.getChildren()) {

			String contentType = formatterConfig.getAttribute("contentType");
			String formatterClass = formatterConfig.getAttribute("class");

			try {
				if (formatterClass != null && contentType != null) {
					IMessageFormatter formatter = 
						(IMessageFormatter) Class.forName(formatterClass).newInstance();

					formatter.setContentType(contentType);

					addMessageFormatter(formatter);

					logger.info("Succeed add MessageFormatter for: " + contentType);
				} else {
					logger.info("Error MessageFormatter Parameter: " + contentType 
							+ " " + formatterClass);
				}
			} catch (Exception e) {
				logger.info("Failed add MessageFormatter for: " + contentType);
				e.printStackTrace();
			}

		}

	}
	
	
	private void addMessageFormatter(IMessageFormatter builder) {
		formattersMap.put(builder.getContentType(), builder);
	}
	
	public IMessageFormatter getMessageFormatter(String contentType){
		return formattersMap.get(contentType);
	}
	
}
