package org.act.xservices.rt.xcontainer.message.handler;

import org.act.xservices.log.Logger;
import org.act.xservices.log.LoggerFactory;
import org.act.xservices.rt.metaset.exception.MetasetParsingException;
import org.act.xservices.rt.soap.ISOAPEnvelope;
import org.act.xservices.rt.soap.constants.SOAPConstants;
import org.act.xservices.rt.soap.exception.SOAPException;
import org.act.xservices.rt.soap.util.SOAPMessageUtil;
import org.act.xservices.rt.xcontainer.framework.config.IConfigElement;
import org.act.xservices.rt.xcontainer.framework.handler.AbstractHandler;
import org.act.xservices.rt.xcontainer.framework.objectpool.IDispatchTarget;
import org.act.xservices.rt.xcontainer.inf.context.IATFTransportContext;
import org.act.xservices.rt.xcontainer.inf.context.IMessageContext;
import org.act.xservices.rt.xcontainer.inf.context.impl.MessageContextInOutImpl;
import org.act.xservices.rt.xcontainer.message.builder.IMessageBuilder;
import org.act.xservices.rt.xcontainer.message.builder.MessageBuilderFactory;
import org.act.xservices.rt.xcontainer.message.formatter.IMessageFormatter;
import org.act.xservices.rt.xcontainer.message.formatter.MessageFormatterFactory;
import org.act.xservices.rt.xcontainer.message.formatter.SOAPFormatException;

/**
 * @LastModifyTime 2009-4-18
 * @CreationTime 2007-5-24
 * @author Lizq [lizq@act.buaa.edu.cn]
 * @since JDK1.5
 */
public final class MessageSerializeHandler extends
		AbstractHandler<IMessageContext> {

	final Logger logger = LoggerFactory
			.getLogger(MessageSerializeHandler.class);

	private IDispatchTarget<IATFTransportContext> transportOut; // Ŀ

	/** Ϣл */
	private MessageFormatterFactory messageFormatterFactory;

	/** Ϣл */
	private MessageBuilderFactory messageBuilderFactory;

	public void config(IConfigElement handlerconfigElement) {
		this.setHandlerName("MessageSerializeHandler");
		this.setHandlerDescription("MessageSerializeHandler");
	}

	/**
	 * MessagebuilderлTransportContextеĶ
	 * 
	 * @param transportContext
	 */
	public void processIn(IATFTransportContext transportContext) {

		String requestContentType = transportContext.getRequestContentType();
		String contentType = null;
		if (requestContentType != null) {
			/*
			 * modified by viking 2010 07 09
			 * 
			 * Content-Type has tow forms Form 1:
			 * Content-Type:"text/html;charset=utf-8" Form 2:
			 * Content-Type:"text/html" Content-Encoding:"charset=utf-8" Form 1
			 * equals Form 2. We need the content type "text/html"
			 */
			int index = requestContentType.indexOf(";");
			if (index != -1) {
				contentType = requestContentType.substring(0, index);
			} else {
				contentType = requestContentType;
			}
		}
		IMessageBuilder mesBuilder = messageBuilderFactory.getMessageBuilder(contentType.trim());
		if (mesBuilder == null) {
			mesBuilder = messageBuilderFactory.getMessageBuilder(contentType.trim().toLowerCase());
		}

		// ޷ҵӦContentTypeMessageBuilder
		if (mesBuilder == null) {

			ISOAPEnvelope faultSOAPMessage = SOAPMessageUtil.creatStdSimpleFaultMessage(
					null, SOAPConstants.FAULT_CODE_RECEIVER,
					"Do not support the request message of contentType: " + requestContentType);

			MessageContextInOutImpl messageContextFault = new MessageContextInOutImpl(transportContext);
			messageContextFault.setSOAPFaultMessage(true); // Ϣ
			// messageContextFault.setVersion(new Integer(11));

			messageContextFault.setSoapMessage(faultSOAPMessage);

			transportContext.setRequestContent(null);// ע˴ԭʼֽ
			this.handlerOutFault(messageContextFault);
			return;
		}

		// Ϣ
		try {
			IMessageContext messageContext = mesBuilder.buildMessageContext(transportContext);
			// IMessageContextTransportPoolHandler
			this.handlerIn(messageContext);
			return;

		} catch (MetasetParsingException e) {
			// XML
			e.printStackTrace();

			// XML﷨SOAPϢ
			ISOAPEnvelope faultSOAPMessage = SOAPMessageUtil
					.creatStdSimpleFaultMessage(null,
							SOAPConstants.FAULT_CODE_RECEIVER,
							"Error XML expression of the SOAP Message. "
									+ e.getMessage());

			MessageContextInOutImpl messageContextFault = new MessageContextInOutImpl(
					transportContext);
			messageContextFault.setSOAPFaultMessage(true); // Ϣ
			// messageContextFault.setVersion(new Integer(11));

			messageContextFault.setSoapMessage(faultSOAPMessage);
			transportContext.setRequestContent(null);// ע˴ԭʼֽ
			this.handlerOutFault(messageContextFault);
			return;

		} catch (SOAPException e) {
			// SOAPʽ
			e.printStackTrace();

			// SOAPʽSOAPϢ
			ISOAPEnvelope faultSOAPMessage = SOAPMessageUtil
					.creatStdSimpleFaultMessage(null,
							SOAPConstants.FAULT_CODE_RECEIVER, e.getMessage());

			MessageContextInOutImpl messageContextFault = new MessageContextInOutImpl(
					transportContext);
			messageContextFault.setSOAPFaultMessage(true); // Ϣ
			// messageContextFault.setVersion(new Integer(11));

			messageContextFault.setSoapMessage(faultSOAPMessage);
			transportContext.setRequestContent(null);// ע˴ԭʼֽ
			this.handlerOutFault(messageContextFault);
			return;
		}
	}

	public IMessageContext[] handlerIn(IMessageContext context) {

		// **************************
		logger.debug("*********** Received RequestMessage at MessageInWorker**************");
		logger.debug("\n{}", context.getSoapMessage().toFormatXML());
		// ********************************

		this.getNextInHandler().handlerIn(context);
		return new IMessageContext[] { context };
	}

	public IMessageContext[] handlerInFault(IMessageContext context) {
		this.getNextInHandler().handlerInFault(context);
		return new IMessageContext[] { context };
	}

	/**
	 * 
	 */
	public IMessageContext[] handlerOut(IMessageContext messageContext) {

		try {

			if (messageContext instanceof MessageContextInOutImpl) {

				String requestContentType = messageContext
						.getTransportcontext().getRequestContentType();

				/**
				 * modified by viking 2010 07 09
				 * 
				 */
				String contentType = requestContentType;
				int index = requestContentType.indexOf(";");
				if (index != -1) {
					contentType = requestContentType.substring(0, index);
				}

				IMessageFormatter mesFormatter = messageFormatterFactory.getMessageFormatter(contentType);
				if (mesFormatter == null) {
					mesFormatter = messageFormatterFactory.getMessageFormatter("text/xml");
				}

				IATFTransportContext transportContext = mesFormatter
						.formatTransportContext(messageContext);

				if (transportContext != null) {
					transportOut.put(transportContext);
				}

			} else {
				// ģʽʱûп
			}

		} catch (InterruptedException e) {
			e.printStackTrace();

		} catch (SOAPFormatException e) {
			e.printStackTrace();

			IATFTransportContext transportContext = messageContext
					.getTransportcontext();
			transportContext.setResponseContent(null);
			try {
				transportOut.put(transportContext);
			} catch (InterruptedException e1) {
				e1.printStackTrace();
			}

		}

		return new IMessageContext[] { messageContext };
	}

	/**
	 * Ϣ
	 */
	public IMessageContext[] handlerOutFault(IMessageContext messageContext) {

		if (messageContext.isSOAPFaultMessage()) { // ʾһϢ

			try {
				if (messageContext instanceof MessageContextInOutImpl) {

					String requestContentType = messageContext
							.getTransportcontext().getRequestContentType();

					// modified by viking 2010 07 12
					int index = requestContentType.indexOf(";");
					String contentType = "text/xml";
					if (index > 0) {
						contentType = requestContentType.substring(0, index);
					} else {
						contentType = messageContext.getTransportcontext()
								.getParameter("Content-Encoding");
						if (contentType == null) {
							contentType = "text/xml";
						}
					}

					IMessageFormatter mesFormatter = messageFormatterFactory
							.getMessageFormatter(contentType);
					if (mesFormatter == null) {
						mesFormatter = messageFormatterFactory
								.getMessageFormatter("text/xml");
					}

					IATFTransportContext transportContext = mesFormatter
							.formatTransportContext(messageContext);

					if (transportContext != null) {
						transportOut.put(transportContext);
					}

				} else {
					// ģʽʱûп
				}

			} catch (InterruptedException e) {
				e.printStackTrace();

			} catch (SOAPFormatException e) {
				e.printStackTrace();

				IATFTransportContext transportContext = messageContext
						.getTransportcontext();
				transportContext.setResponseContent(null);
				try {
					transportOut.put(transportContext);
				} catch (InterruptedException e1) {
					e1.printStackTrace();
				}

			}
		}

		return new IMessageContext[] { messageContext };
	}

	public void destroy() {
	}

	// /**
	// * messageContext == null,Ӧĳ򷵻null
	// *
	// * @param messageContext
	// * @return
	// */
	// private IATFTransportContext createTransportContext(
	// IMessageContext messageContext, String encodeStyle) {
	//		
	// if (messageContext != null) {
	// //inoutģʽmessagecontextжtransportcontextķװ
	// if (messageContext instanceof MessageContextInOutImpl) {
	//				
	// MessageContextInOutImpl mesContextInOut =
	// (MessageContextInOutImpl)messageContext;
	// IATFTransportContext tranContext = mesContextInOut.getTransportcontext();
	//				
	// //TODO lizq 2008-10-10 ˴ӲhttpҪ޸
	// if(tranContext.getATFTransportSession().getTransportIdentifier().equals("http")){
	// tranContext.setResponseContentType("text/xml;charset=" + encodeStyle); //
	// дչ
	// }
	// //עûHTTPӦ
	//
	// // Ӧ
	// byte[] contentByte = creatBytesContent(messageContext, encodeStyle);
	//				
	// if (contentByte == null)
	// return null; // Ӧĳ
	//				
	// // InOnly
	//				
	// tranContext.setResponseContent(contentByte);
	//				
	// return tranContext;
	// }else{
	// //ģʽʱûп
	// }
	// }
	// return null;
	// }
	//
	// /**
	// *  IMessageContext ֽʽ,null
	// * ΪInOnlyģʽؿ
	// *
	// * @param messageContext
	// * @return
	// */
	// private byte[] creatBytesContent(IMessageContext messageContext,
	// String encodeStyle) {
	//
	// Metaset responseMetaset = messageContext.getSoapMessage()
	// .getSoapEnvelope().getSoapEnvelopeMetaset();
	//
	// if(responseMetaset == null)
	// return new byte[0];//OutOnly
	//		
	// responseMetaset.setEncoding(encodeStyle); // XMLеıʽ
	// try {
	//			
	// //Ҫı뷽ʽб
	// byte[] responseBytes =
	// responseMetaset.toDocument().getBytes(encodeStyle);
	//			
	// return responseBytes;
	//			
	// } catch (UnsupportedEncodingException e) {
	// System.err.println("UnsupportedEncoding "+
	// encodeStyle+" for message:\n"+responseMetaset.toDocument());
	// System.err.println("Use default Encoded UTF-8");
	// try {
	// return responseMetaset.toDocument().getBytes("UTF-8");
	// } catch (UnsupportedEncodingException e1) {
	// e1.printStackTrace();
	// return null;
	// }
	// }
	// }

	public IDispatchTarget<IATFTransportContext> getTransportOut() {
		return transportOut;
	}

	public void setTransportOut(
			IDispatchTarget<IATFTransportContext> transportOut) {
		this.transportOut = transportOut;
	}

	public void setMessageFormatterFactory(
			MessageFormatterFactory messageFormatterFactory) {
		this.messageFormatterFactory = messageFormatterFactory;
	}

	public void setMessageBuilderFactory(
			MessageBuilderFactory messageBuilderFactory) {
		this.messageBuilderFactory = messageBuilderFactory;
	}

}
