package org.act.xservices.rt.xcontainer.service;

import org.act.xservices.rt.xcontainer.framework.AConnector;
import org.act.xservices.rt.xcontainer.framework.config.IConfigElement;
import org.act.xservices.rt.xcontainer.framework.dispatch.ITaskFactory;
import org.act.xservices.rt.xcontainer.inf.context.IMessageContext;
import org.act.xservices.rt.xcontainer.inf.context.IServiceContext;
import org.act.xservices.rt.xcontainer.service.handler.MessageDispatchHandler;
import org.act.xservices.rt.xcontainer.service.handler.PolicyHandler;
import org.act.xservices.rt.xcontainer.service.handler.ServiceValidationHandler;

/**
 * 
 * 
 * @LastModifyTime 2008-9-8
 * @CreationTime 2007-05-28
 * @author Lizq [lizq@act.buaa.edu.cn]
 * @since JDK1.5
 */
public class ServiceConnector extends AConnector<IMessageContext,IServiceContext>{

	/** ڷƥ */
	private MessageDispatchHandler messageDispatchHandler;

	/** ڷ֤ */
	private ServiceValidationHandler serviceValidationHandler;

	/** WS-PolicyԴ */
	private PolicyHandler policyHandler;

	
	
	public ServiceConnector(String connectorName) {
		super(connectorName);
	}

	public ServiceConnector() {

	}
	
	/**
	 * ڴ˴ʼеIHandler
	 */
	protected void doInitLocal() {

		// ʼ messagePoolHandlerS, IHandlers<IServiceObject>[]
		// messageHandlerChains, ServiceObjectPoolHandler
		IConfigElement handlersConfigElement = 
			this.connectorConfiguration.getFirstChildWithName("handlerChain");

		IConfigElement mphsConfigElement = handlersConfigElement.getChildren()[0];
		IConfigElement policyHandlerConfigElement = handlersConfigElement.getChildren()[1];
		IConfigElement sphConfigElement = handlersConfigElement.getChildren()[2];

		try {
			// messagePoolHandler
			messageDispatchHandler = (MessageDispatchHandler) Class.forName(
					mphsConfigElement.getAttribute("class")).newInstance();
			messageDispatchHandler.config(mphsConfigElement);
			messageDispatchHandler.setServiceOut(getOutputTarget());
			messageDispatchHandler.setXcServiceRegistry(
					this.connectorEnvironment.getContainerEnvironment().getXcServiceRegistry());
			messageDispatchHandler.setSpecialSOAPRegistry(
					this.connectorEnvironment.getContainerEnvironment().getSpecialSOAPRegistry());

			// policyHandler
			policyHandler = (PolicyHandler) Class.forName(
					policyHandlerConfigElement.getAttribute("class")).newInstance();
			policyHandler.config(policyHandlerConfigElement);

			// servicePoolHandler
			serviceValidationHandler = (ServiceValidationHandler) Class.forName(
					sphConfigElement.getAttribute("class")).newInstance();
			serviceValidationHandler.config(sphConfigElement);
			serviceValidationHandler.setServiceIn(getInputTarget());

			messageDispatchHandler.setNextInHandler(policyHandler);
			policyHandler.setNextOutHandler(messageDispatchHandler);
			policyHandler.setNextInHandler(serviceValidationHandler);
			serviceValidationHandler.setNextOutHandler(policyHandler);
			
			
		} catch (InstantiationException e) {
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			e.printStackTrace();
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		}

	}
	
	
	public ITaskFactory<IMessageContext> getInputTaskFactory() {
		ServiceInTaskFactory sInDHFactory = new ServiceInTaskFactory(this);
		sInDHFactory.setMessagePoolHandler(messageDispatchHandler);
		return sInDHFactory;
	}

	public ITaskFactory<IServiceContext> getOutputTaskFactory() {
		ServiceOutTaskFactory sOutDHFactory = new ServiceOutTaskFactory(this);
		sOutDHFactory.setServicePoolHandler(serviceValidationHandler);
		return sOutDHFactory;
	}
	
	/**
	 * 
	 * ˴ԴͷŹ
	 */
	public void destroy() {	
		messageDispatchHandler.destroy();
		policyHandler.destroy();
		serviceValidationHandler.destroy();
	}

}
