package org.act.xservices.rt.xcontainer.service.handler;

import java.util.ArrayList;
import java.util.List;

import org.act.xservices.log.Logger;
import org.act.xservices.log.LoggerFactory;
import org.act.xservices.rt.soap.ISOAPEnvelope;
import org.act.xservices.rt.soap.constants.SOAPConstants;
import org.act.xservices.rt.soap.util.SOAPMessageUtil;
import org.act.xservices.rt.xcontainer.framework.config.IConfigElement;
import org.act.xservices.rt.xcontainer.framework.handler.AbstractHandler;
import org.act.xservices.rt.xcontainer.framework.objectpool.IDispatchTarget;
import org.act.xservices.rt.xcontainer.inf.context.IMessageContext;
import org.act.xservices.rt.xcontainer.inf.context.IServiceContext;
import org.act.xservices.rt.xcontainer.inf.context.impl.ServiceContextInOutImpl;
import org.act.xservices.rt.xcontainer.inf.service.SpecialSOAPRegistry;
import org.act.xservices.rt.xcontainer.inf.service.wsdl.XcService;
import org.act.xservices.rt.xcontainer.inf.service.wsdl.XcServiceRegistry;
import org.act.xservices.rt.xcontainer.service.resolver.AbstractOperationResolver;
import org.act.xservices.rt.xcontainer.service.resolver.AbstractServiceResolver;
import org.act.xservices.rt.xcontainer.service.resolver.ResolverProcessException;

/**
 * Ϣɷ ϢӦķͲ
 * 
 * @LastModifyTime 2009-4-15
 * @CreationTime 2007-06-20
 * @author Lizq [lizq@act.buaa.edu.cn]
 * @since JDK1.5
 */
public final class MessageDispatchHandler extends
		AbstractHandler<IServiceContext> {

	final Logger logger = LoggerFactory.getLogger(MessageDispatchHandler.class);

	private IDispatchTarget<IMessageContext> serviceOut; // Ŀ

	private XcServiceRegistry xcServiceRegistry;

	private SpecialSOAPRegistry specialSOAPRegistry;

	/** б */
	private List<AbstractServiceResolver> serviceResolvers = new ArrayList<AbstractServiceResolver>();

	/** б */
	private List<AbstractOperationResolver> operationResolvers = new ArrayList<AbstractOperationResolver>();

	public void config(IConfigElement handlerConfigElement) {
		this.setHandlerName(handlerConfigElement.getAttribute("name"));
		this.setHandlerDescription(handlerConfigElement.getAttribute("description"));

		IConfigElement serviceResConfigs = handlerConfigElement.getFirstChildWithName("serviceResolvers");

		if (serviceResConfigs != null) {
			String className;
			AbstractServiceResolver serResolver;
			for (IConfigElement srConfig : serviceResConfigs.getChildren()) {

				className = srConfig.getAttribute("class");

				if (className != null) {
					try {
						serResolver = (AbstractServiceResolver) Class.forName(className).newInstance();
						serviceResolvers.add(serResolver);
						logger.info("Succeed add ServiceResolver: " + serResolver.getName());
					} catch (Exception e) {
						logger.info("Failed to add ServiceResolver: " + className);
						e.printStackTrace();
					}
				} else {
					logger.info("Failed to add ServiceResolver for no className!");
				}
			}// end of for
		}

		IConfigElement operationResConfigs = handlerConfigElement.getFirstChildWithName("operationResolvers");

		if (operationResConfigs != null) {

			String className;
			AbstractOperationResolver opeResolver;

			for (IConfigElement opeConfig : operationResConfigs.getChildren()) {

				className = opeConfig.getAttribute("class");

				if (className != null) {
					try {
						opeResolver = (AbstractOperationResolver) Class.forName(className).newInstance();
						operationResolvers.add(opeResolver);
						logger.info("Succeed add OperationResolver: " + opeResolver.getName());
					} catch (Exception e) {
						logger.info("Failed to add OperationResolver: " + className);
						e.printStackTrace();
					}
				} else {
					logger.info("Failed to add OperationResolver for no className!");
				}
			}// end of for
		}
	}

	public void processIn(IMessageContext inMessageContext) {
		// IMessageContextװΪIServiceContext
		IServiceContext serviceContext = new ServiceContextInOutImpl(inMessageContext);

		String serviceName = specialSOAPRegistry.getServiceNameByRequestURI(inMessageContext.getUri());

		if (serviceName != null) {// ΪSpecialϢ
			serviceContext.setSpecialSoap(true);
			serviceContext.setXcService(new XcService(serviceName));
		}
		this.handlerIn(serviceContext);
	}

	/**
	 * ڴ˷ڽϢɷ鿴ϢӦķǷ
	 * 
	 * @param serviceContext
	 * @return
	 */
	public IServiceContext[] handlerIn(IServiceContext serviceContext) {

		IServiceContext[] resultServiceContext = null;

		// SOAPϢ
		if (serviceContext.isSpecialSoap()) {
			resultServiceContext = this.getNextInHandler().handlerIn(
					serviceContext);
			return resultServiceContext;
		}

		// ͨSOAPϢ

		// зƥ
		for (int i = 0; i < serviceResolvers.size(); i++) {
			AbstractServiceResolver serRes = serviceResolvers.get(i);
			try {
				serRes.process(serviceContext, xcServiceRegistry);
			} catch (ResolverProcessException e) {
				e.printStackTrace();
			}
		}

		if (serviceContext.getXcService() == null) { // ƥʧ
			// õķ񲻴
			// ɴϢ
			ISOAPEnvelope faultMessage = SOAPMessageUtil.creatStdSimpleFaultMessage(
					serviceContext.getInMessage().getVersion(), 
					SOAPConstants.FAULT_CODE_RECEIVER,
					"The Request Service " + serviceContext.getRequestURI() + " is not exit!");

			// MessageContextе滻faultMessage
			serviceContext.getMessageContext().setSoapMessage(faultMessage);
			serviceContext.getMessageContext().setSOAPFaultMessage(true); // һҪ
			serviceContext.setFaultOutMessage(true);
			resultServiceContext = this.handlerOutFault(serviceContext);
			return resultServiceContext;
		}

		// вƥ
		for (int i = 0; i < operationResolvers.size(); i++) {
			AbstractOperationResolver opeRes = operationResolvers.get(i);
			try {
				opeRes.process(serviceContext, xcServiceRegistry);
			} catch (ResolverProcessException e) {
				e.printStackTrace();
			}
		}

		if (serviceContext.getXcOperation() == null) { // ƥʧ
			ISOAPEnvelope faultMessage = SOAPMessageUtil.creatStdFaultMessage(
					serviceContext.getInMessage().getVersion(),
					SOAPConstants.FAULT_CODE_RECEIVER,
					"The request method is not exit in the request servcice "
							+ serviceContext.getXcService().getServiceName(),
					null, serviceContext.getInMessageBody());

			// MessageContextе滻faultMessage
			serviceContext.getMessageContext().setSoapMessage(faultMessage);
			serviceContext.getMessageContext().setSOAPFaultMessage(true); // һҪ
			serviceContext.setFaultOutMessage(true);
			resultServiceContext = this.handlerOutFault(serviceContext);
			return resultServiceContext;
		}

		// Ϣɷɹ
		resultServiceContext = this.getNextInHandler().handlerIn(serviceContext);

		return resultServiceContext;
	}

	/**
	 * 
	 */
	public IServiceContext[] handlerInFault(IServiceContext serviceContext) {
		return this.getNextInHandler().handlerInFault(serviceContext);
	}

	/**
	 * 
	 */
	public IServiceContext[] handlerOut(IServiceContext serviceContext) {
		try {
			serviceOut.put(serviceContext.getMessageContext());
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
		return new IServiceContext[] { serviceContext };
	}

	public IServiceContext[] handlerOutFault(IServiceContext serviceContext) {
		try {
			serviceOut.put(serviceContext.getMessageContext());
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
		return new IServiceContext[] { serviceContext };
	}

	public IDispatchTarget<IMessageContext> getServiceOut() {
		return serviceOut;
	}

	public void setServiceOut(IDispatchTarget<IMessageContext> serviceOut) {
		this.serviceOut = serviceOut;
	}

	public void setXcServiceRegistry(XcServiceRegistry xcServiceRegistry) {
		this.xcServiceRegistry = xcServiceRegistry;
	}

	public void setSpecialSOAPRegistry(SpecialSOAPRegistry specialSOAPRegistry) {
		this.specialSOAPRegistry = specialSOAPRegistry;
	}
}
