package org.act.xservices.rt.xcontainer.service.handler;

import org.act.xservices.rt.soap.ISOAPEnvelope;
import org.act.xservices.rt.soap.constants.SOAPConstants;
import org.act.xservices.rt.soap.util.SOAPMessageUtil;
import org.act.xservices.rt.xcontainer.framework.config.IConfigElement;
import org.act.xservices.rt.xcontainer.framework.handler.AbstractHandler;
import org.act.xservices.rt.xcontainer.framework.policy.IPolicyProcessor;
import org.act.xservices.rt.xcontainer.framework.policy.PolicyProcessException;
import org.act.xservices.rt.xcontainer.inf.context.IServiceContext;

/**
 * Դ
 * 
 * @LastModifyTime 2008-9-8
 * @CreationTime 2007-06-20
 * @author Lizq [lizq@act.buaa.edu.cn]
 * @since JDK1.5
 */
public final class PolicyHandler extends AbstractHandler<IServiceContext> {

	/**
	 * Policy
	 * 
	 * @param serviceContext
	 * @return
	 * @throws ClassNotFoundException
	 */
	public IServiceContext[] handlerIn(IServiceContext serviceContext) {

		// SOAPϢ
		if (serviceContext.isSpecialSoap()) {
			this.getNextInHandler().handlerIn(serviceContext);
			return new IServiceContext[] { serviceContext };
		}

		try {
			// ԴεøεĲԴдֱжϴ, by lizq 2010-07-02
			// ---XcService---
			if (serviceContext.getXcService().getPolicyProcessor() != null) {
				for (IPolicyProcessor p : serviceContext.getXcService().getPolicyProcessor()) {
					p.processIN(serviceContext);
				}
			}

			// ---XcEndpoint---
			if (serviceContext.getXcEndpoint().getPolicyProcessor() != null) {
				for (IPolicyProcessor p : serviceContext.getXcEndpoint().getPolicyProcessor()) {
					p.processIN(serviceContext);
				}
			}

			// ---Xcoperation---
			if (serviceContext.getXcOperation().getPolicyProcessor() != null) {
				for (IPolicyProcessor p : serviceContext.getXcOperation().getPolicyProcessor()) {
					p.processIN(serviceContext);
				}
			}

		} catch (PolicyProcessException e1) {
			e1.printStackTrace();

			// ԴɴϢ
			ISOAPEnvelope faultMessage = SOAPMessageUtil.creatStdSimpleFaultMessage(
					serviceContext.getInMessage().getVersion(),
					SOAPConstants.FAULT_CODE_RECEIVER,
					"Policy in process error: " + serviceContext.getXcService().getServiceName());

			// MessageContextе滻faultMessage
			serviceContext.getMessageContext().setSoapMessage(faultMessage);
			serviceContext.getMessageContext().setSOAPFaultMessage(true); // һҪ
			serviceContext.setFaultOutMessage(true);
			return this.handlerOutFault(serviceContext);

		}

		this.getNextInHandler().handlerIn(serviceContext);
		return new IServiceContext[] { serviceContext };
	}

	public IServiceContext[] handlerInFault(IServiceContext serviceContext) {

		this.getNextInHandler().handlerInFault(serviceContext);
		return new IServiceContext[] { serviceContext };
	}

	/**
	 * Policy
	 * 
	 * @param serviceContext
	 * @return
	 */
	public IServiceContext[] handlerOut(IServiceContext serviceContext) {

		// SOAPϢ
		if (serviceContext.isSpecialSoap()) {
			this.getNextOutHandler().handlerOut(serviceContext);
			return new IServiceContext[] { serviceContext };
		}

		try {
			// ԴεøεĲԴдֱжϴ, by lizq 2010-07-02
			if (serviceContext.getOutMessage() != null) { // InOnlyģʽûзϢ

				// ---XcService---
				if (serviceContext.getXcService().getPolicyProcessor() != null) {
					for (IPolicyProcessor p : serviceContext.getXcService()
							.getPolicyProcessor()) {
						p.processOUT(serviceContext);
					}
				}

				// ---XcEndpoint---
				if (serviceContext.getXcEndpoint().getPolicyProcessor() != null) {
					for (IPolicyProcessor p : serviceContext.getXcEndpoint()
							.getPolicyProcessor()) {
						p.processOUT(serviceContext);
					}
				}

				// ---Xcoperation---
				if (serviceContext.getXcOperation().getPolicyProcessor() != null) {
					for (IPolicyProcessor p : serviceContext.getXcOperation()
							.getPolicyProcessor()) {
						p.processOUT(serviceContext);
					}
				}
			}

		} catch (PolicyProcessException e1) {
			e1.printStackTrace();

			// ԴɴϢ
			ISOAPEnvelope faultMessage = SOAPMessageUtil.creatStdSimpleFaultMessage(
					serviceContext.getInMessage().getVersion(),
					SOAPConstants.FAULT_CODE_RECEIVER,
					"Policy out process error: " + serviceContext.getXcService().getServiceName());

			// MessageContextе滻faultMessage
			serviceContext.getMessageContext().setSoapMessage(faultMessage);
			serviceContext.getMessageContext().setSOAPFaultMessage(true); // һҪ
			serviceContext.setFaultOutMessage(true);
			return this.handlerOutFault(serviceContext);
		}

		this.getNextOutHandler().handlerOut(serviceContext);
		return new IServiceContext[] { serviceContext };
	}

	public IServiceContext[] handlerOutFault(IServiceContext serviceContext) {

		this.getNextOutHandler().handlerOutFault(serviceContext);
		return new IServiceContext[] { serviceContext };
	}

	/**
	 * ÷
	 * 
	 * @param handlerconfigurableElement
	 */
	public void config(IConfigElement handlerconfigurableElement) {
		this.setHandlerName(handlerconfigurableElement.getAttribute("name"));
		this.setHandlerDescription(handlerconfigurableElement.getAttribute("description"));
	}

}
