package org.act.xservices.rt.hs.server.ssl;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.security.GeneralSecurityException;
import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.Security;

import javax.net.ssl.KeyManagerFactory;
import javax.net.ssl.SSLContext;

/**
 * @LastModifyTime 2010-6-11
 * @author niyong [niyong@act.buaa.edu.cn]
 *
 * @since jdk1.5
 */
public class XcontainerSslContextFactory {
	/**
	 *  ʹõİȫЭ.
	 */
	private static final String PROTOCOL = "TLS";
	
	private static final String KEY_MANAGER_FACTORY_ALGORITHM;
	
    static {
		String algorithm = Security.getProperty("ssl.KeyManagerFactory.algorithm");
		if(algorithm == null) {
			algorithm = "SunX509";
		}
		
		KEY_MANAGER_FACTORY_ALGORITHM = algorithm;
	}
    
    /**
     *  Xcontainerȫ֤Կļ .
     */
    private static final String XCONTAINER_KEYSTORE = "act.keystore";
    
    /**
     *  XcontainerԿ.
     */
    private static final char[] XCONTAINER_PW = { 'a', 'c', 't', '1', '2', '3'};
    
    private static SSLContext serverInstance = null;
    private static SSLContext clientInstance = null;
    
    
    /**
     * ȡSSLContextĵ
     * @param server
     * @return
     * @throws GeneralSecurityException 
     */
    public static SSLContext getInstance(boolean server) throws GeneralSecurityException {
    	SSLContext retInstance = null;
    	if (server) {
    		synchronized(XcontainerSslContextFactory.class) {
    			if (serverInstance == null) {
    				try {
    					serverInstance = createXcontainerServerSslContext();
    				} catch (Exception ioe) {
    					throw new GeneralSecurityException("Can't create Server SSLContext:" + ioe);
    				}
    			}
    		}
    		retInstance = serverInstance;
    	} else {
    		synchronized(XcontainerSslContextFactory.class) {
    			if (clientInstance == null) {
    				clientInstance = createXcontainerClientSslContext();
    			}
    		}
    		retInstance = clientInstance;
    	}
    	
    	return retInstance;
    }
    
    
    private static SSLContext createXcontainerServerSslContext() throws GeneralSecurityException, IOException {
    	// Create keystore
    	KeyStore ks = KeyStore.getInstance("JKS");
    	InputStream in = null;
    	
    	String keyStorePath = System.getProperty("user.dir") + File.separator + "conf"+File.separator+"act.keystore";
    	try {
			in = new FileInputStream(keyStorePath);
			ks.load(in, XCONTAINER_PW);
		} finally {
            if (in != null) {
                try {
                    in.close();
                } catch (IOException ignored) {
                }
            }
        }
		
		// ʹøԿԿ
		KeyManagerFactory kmf = KeyManagerFactory.getInstance(KEY_MANAGER_FACTORY_ALGORITHM);
		kmf.init(ks, XCONTAINER_PW);
		
		SSLContext sslContext = SSLContext.getInstance(PROTOCOL);
		sslContext.init(kmf.getKeyManagers(), XcontainerTrustManagerFactory.X509_MANAGERS, null);
		
		return sslContext;
		
    }
    
    private static SSLContext createXcontainerClientSslContext() throws GeneralSecurityException {
    	SSLContext context = SSLContext.getInstance(PROTOCOL);
        context.init(null, XcontainerTrustManagerFactory.X509_MANAGERS, null);
        return context;
        }
}

















