package org.act.xservices.rt.hs.server.ssl;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.security.GeneralSecurityException;
import java.security.KeyStore;
import java.security.Security;

import javax.net.ssl.KeyManagerFactory;
import javax.net.ssl.SSLContext;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 *  ͨļSSLContextĹ
 * @LastModifyTime 2010-7-7
 * @author niyong [niyong@act.buaa.edu.cn]
 *
 * @since jdk1.5
 */
public class XcontainerSslContextFactory1 {

	private static final Logger LOG = LoggerFactory.getLogger(XcontainerSslContextFactory1.class);
	
	private static final String KEY_MANAGER_FACTORY_ALGORITHM;
	
    static {
		String algorithm = Security.getProperty("ssl.KeyManagerFactory.algorithm");
		if(algorithm == null) {
			algorithm = "SunX509";
		}
		
		KEY_MANAGER_FACTORY_ALGORITHM = algorithm;
	}
    
    
    private static SSLContext serverInstance = null;
    private static SSLContext clientInstance = null;
    
    
    /**
     * ȡSSLContextĵ
     * @param server
     * @return
     * @throws GeneralSecurityException 
     */
    public static SSLContext getInstance(boolean server,String keystoreFile,char[] keystorePass,String sslProtocol) throws GeneralSecurityException {
    	SSLContext retInstance = null;
    	if (server) {
    		synchronized(XcontainerSslContextFactory.class) {
    			if (serverInstance == null) {
    				try {
    					serverInstance = createXcontainerServerSslContext(keystoreFile,keystorePass,sslProtocol);
    				} catch (Exception ioe) {
    					throw new GeneralSecurityException("Can't create Server SSLContext:" + ioe);
    				}
    			}
    		}
    		retInstance = serverInstance;
    	} else {
    		synchronized(XcontainerSslContextFactory.class) {
    			if (clientInstance == null) {
    				clientInstance = createXcontainerClientSslContext(sslProtocol);
    			}
    		}
    		retInstance = clientInstance;
    	}
    	
    	return retInstance;
    }
    
    /**
     *  ˵SSLContext
     * @param keystoreFile
     * @param keystorePass
     * @param sslProtocol
     * @return  SSLContext serverSslContext
     * @throws GeneralSecurityException
     * @throws IOException
     */
    private static SSLContext createXcontainerServerSslContext(String keystoreFile,char[] keystorePass,String sslProtocol) throws GeneralSecurityException, IOException {
    	if( keystoreFile == null) {
    		LOG.debug("keystoreFile null error!!!!!!!!!!!!!");
    		return null;
    	} 
    	
    	if (keystorePass == null) {
    		LOG.debug("keystoreFile null specifid!!!!!!!!!");
    		return null;
    	}
    	
    	if (sslProtocol == null ) {
    		sslProtocol = "TLS";
    	}
    	
    	// JKS͵Կ
    	KeyStore ks = KeyStore.getInstance("JKS");
    	InputStream in = null;
    	
    	String keyStorePath = System.getProperty("user.dir") + File.separator + "conf"+File.separator + keystoreFile;
    	
    	try {
			in = new FileInputStream(keyStorePath);
			ks.load(in, keystorePass);
		} finally {
            if (in != null) {
                try {
                    in.close();
                } catch (IOException ignored) {
                }
            }
        }
		
		// ʹøԿԿ
		KeyManagerFactory kmf = KeyManagerFactory.getInstance(KEY_MANAGER_FACTORY_ALGORITHM);
		kmf.init(ks, keystorePass);
		
		SSLContext serverSslContext = SSLContext.getInstance(sslProtocol);
		serverSslContext.init(kmf.getKeyManagers(), XcontainerTrustManagerFactory.X509_MANAGERS, null);
		
		return serverSslContext;
		
    }
    
    /**
     * 
     * @param sslProtocol
     * @return SSLContext ͻ˵SSLContext:clientSslContext
     * @throws GeneralSecurityException
     */
    private static SSLContext createXcontainerClientSslContext(String sslProtocol) throws GeneralSecurityException {
    	
    	// ָsslProtocol(磺TLS")ͻSSLContext
    	SSLContext clientSslContext = SSLContext.getInstance(sslProtocol);
    	
    	//ʼ
    	clientSslContext.init(null, XcontainerTrustManagerFactory.X509_MANAGERS, null);
    	
        return clientSslContext;
        }
    
    
}
