package org.act.xservices.rt.xcontainer.transport.http;

import org.act.xservices.log.Logger;
import org.act.xservices.log.LoggerFactory;
import org.act.xservices.rt.xcontainer.framework.IConnector;
import org.act.xservices.rt.xcontainer.framework.config.IConfigElement;
import org.act.xservices.rt.xcontainer.framework.transport.ATransportProtocol;
import org.act.xservices.rt.xcontainer.framework.transport.ITransportReceiver;
import org.act.xservices.rt.xcontainer.framework.transport.ITransportSender;
import org.act.xservices.rt.xcontainer.inf.context.IATFTransportContext;
import org.act.xservices.rt.xcontainer.inf.context.IATFTransportSession;

/**
 * Represent the Http protocol
 * 
 * @LastModifyTime 2008-7-3
 * @author Lizq [lizq@act.buaa.edu.cn]
 * @since JDK1.5
 */
public class Http11TransportProtocol extends ATransportProtocol {

	final Logger logger = LoggerFactory.getLogger(Http11TransportProtocol.class);
	
	
	public static final String HTTP_TRANSPORT_IDENTIFIER = "http";

	private ITransportSender httpSender = new HttpTransportSender();

	public void initialize(IConfigElement transportProtocolConfig,
			IConnector<IATFTransportSession, IATFTransportContext> connector) throws Exception {

		super.initialize(transportProtocolConfig, connector);

		// config Dispatch source
		IConfigElement[] receiverConfigList = 
			transportProtocolConfig.getChildrenWithName("httpReceiver");
		
		//
		for (IConfigElement receiverConfig : receiverConfigList) {
			try {
				ITransportReceiver transportReceiver = (ITransportReceiver) Class
						.forName(receiverConfig.getAttribute("class")).newInstance();

				transportReceiver.initialize(receiverConfig, this);

				this.getTransportReceiverList().add(transportReceiver);

			} catch (InstantiationException e) {
				e.printStackTrace();
			} catch (IllegalAccessException e) {
				e.printStackTrace();
			} catch (ClassNotFoundException e) {
				e.printStackTrace();
			}
		}

	}

	/**
	 * Is one way channel protocol
	 * 
	 * @return
	 */
	public boolean isOneWay() {
		return false;
	}

	public ITransportSender getTransportSender() {
		return httpSender;
	}

	public void startup() {
		for (ITransportReceiver traReceiver : getTransportReceiverList()) {
			traReceiver.startup();
			// ־
			logger.info( "{} Listening to port {}", this.getDescription(), traReceiver.getListenPort());
		}
	}

	public void shutdown() {
		for (ITransportReceiver traReceiver : getTransportReceiverList()) {
			traReceiver.shutdown();
		}
	}

	public String getTransportIdentifier() {
		return HTTP_TRANSPORT_IDENTIFIER;
	}

}
