package org.act.xservices.rt.xcontainer.transport.http;

import java.io.File;
import java.net.InetSocketAddress;

import org.act.xservices.rt.hs.server.BasicServiceContainer;
import org.act.xservices.rt.hs.server.ContainerLifecycleException;
import org.act.xservices.rt.hs.server.HttpServiceHandler;
import org.act.xservices.rt.hs.server.ServiceContainer;
import org.act.xservices.rt.hs.server.resolver.PatternMatchResolver;
import org.act.xservices.rt.xcontainer.framework.config.IConfigElement;
import org.act.xservices.rt.xcontainer.framework.transport.ITransportProtocol;
import org.act.xservices.rt.xcontainer.framework.transport.ITransportReceiver;
import org.act.xservices.rt.xcontainer.transport.http.server.HttpServerService;
import org.apache.mina.transport.socket.SocketAcceptor;



/**
 * HTTP Message Receiver
 * @version 1.0 2008-6-5
 * @author Lizq [lizq@act.buaa.edu.cn]
 * @since JDK1.5
 */
public class HttpTransportReceiver extends AStatisticalTransportReceiver implements ITransportReceiver{

	private String discription;

	private int port = 0;
	
	private InetSocketAddress address;

	private ITransportProtocol protocol;
	
	private ATFMinaTransport atfMinaTransport;
	
	private ServiceContainer httpServiceContainer;
	

	public HttpTransportReceiver() {
		
	}
	
	public ITransportProtocol getTransportProtocol() {
		return protocol;
	}

//	public void setTransportProtocol(ITransportProtocol protocol) {
//		this.protocol = protocol;
//	}
	
	
	
	/**
	 * 
	 */
	private void configServiceContainer(){
		
		// Config ServiceContainer
		httpServiceContainer = new BasicServiceContainer();
		//((BasicServiceContainer)httpServiceContainer).setSendServerHeader(false);
		
		// This handler invoke the HttpService
		HttpServiceHandler httpServiceHandler = new HttpServiceHandler();
        
		WSHttpService wsHttpService = new WSHttpService(); //the service to handle Web Services
		wsHttpService.setInPool(this.protocol.getConnector().getInputTarget());
		wsHttpService.setContainerEnvironment(this.protocol.getContainerEnvironment());
		
		// add WSHttpService to HttpServiceHandler
        httpServiceHandler.addHttpService("WSHttpService",wsHttpService);
		
        // add FileService to HttpServiceHandler
        String basePath = System.getProperty("user.dir") + File.separator + "webapps";
        httpServiceHandler.addHttpService("fileHttpService", new HttpServerService("/web", basePath));
        

        // add ServiceResolver to HttpServiceHandler
//        FixedServiceResolver fixedResolver = new FixedServiceResolver();
//        fixedResolver.setService("WSHttpService");
//        httpServiceHandler.setServiceResolver(fixedResolver);
        PatternMatchResolver resolver = new PatternMatchResolver();
       
        resolver.addPatternMapping("(?!/web/).*", "WSHttpService");
        resolver.addPatternMapping("/web/.*", "fileHttpService");
        
        httpServiceHandler.setServiceResolver(resolver);
        
        // add HttpServiceHandler to HttpServiceContainer
        httpServiceContainer.addServiceFilter(httpServiceHandler);
	}
	
	
	/** 
	 * Config the dispatch source
	 * 
	 * @param dispatchSourceConfig
	 */
	public void initialize(IConfigElement receiverConfig, 
			ITransportProtocol protocol) throws Exception{
		
		this.protocol = protocol;
		
		this.discription = receiverConfig.getAttribute("description"); 
		this.port = Integer.parseInt(receiverConfig.getAttribute("port"));
		String hostname = receiverConfig.getAttribute("hostname");
		if("*".equals(hostname)){
			this.address = new InetSocketAddress(port);
		}else{
			this.address = new InetSocketAddress(hostname, port);
		}

		// config ServiceContainer
		configServiceContainer();
		
		// creat Transport whitch will add to the ServiceContainer
        IConfigElement serverSocketConfig = 
        	receiverConfig.getFirstChildWithName("serverSocket");
		
        atfMinaTransport = new ATFMinaTransport();
		
        atfMinaTransport.setServiceContainer(httpServiceContainer);
        atfMinaTransport.setPort(port);
        atfMinaTransport.setAddress(address);
		atfMinaTransport.config(serverSocketConfig);
 	
		//  handler Ϣ
		ATFHttpIoHandler atfIoHandler = new ATFHttpIoHandler();
		atfIoHandler.setReadIdle(600); //ע⣺˴˶ʱ(seconds)
		atfMinaTransport.setIoHandler(atfIoHandler);
		
        // add the Transport to ServiceContainer
        httpServiceContainer.addTransport(atfMinaTransport);
	}
	

	public boolean isStarted() {
		return atfMinaTransport.isStarted();
	}

	public void setListenAddress(InetSocketAddress address) {
		if (address != null) {
			if (isStarted())
				throw new IllegalStateException(
						"can't set listen address after acceptor started");
			this.address = address;
		}
	}

	public InetSocketAddress getListenAddress() {
		return address;
	}

	public void setListenPort(int port) {
		if (isStarted())
			throw new IllegalStateException(
					"can't set listen port after acceptor started");
		this.port = port;
	}

	public int getListenPort() {
		return port;
	}

	public int getBacklog() {
		return atfMinaTransport.getBacklog();
	}
	
	public String getDescription(){
		return discription;
	}
	
	public synchronized void startup() {
		if (isStarted())
			return;     
		try {
			httpServiceContainer.start();
		} catch (ContainerLifecycleException e) {
			e.printStackTrace();
		}
		
		//---test---
		//testStatistical();
	}
	
	
	public synchronized void shutdown() {
		httpServiceContainer.stop();
	}
	
	
	public SocketAcceptor getSocketAcceptor(){
		return atfMinaTransport.getSocketAcceptor();
	}
	
	
	//-----------------test---------------------
	@SuppressWarnings("unused")
	private void testStatistical(){
		Thread printT = new Thread(){
		
		public void run(){
			
			while(true){
			System.out.println("-------------------------------------------");
			System.out.println("---getManagedSessionCount " + getSocketAcceptor().getManagedSessionCount());
			System.out.println("---getLargestManagedSessionCount " + getSocketAcceptor().getStatistics().getLargestManagedSessionCount());
			System.out.println("---getCumulativeManagedSessionCount " + getSocketAcceptor().getStatistics().getCumulativeManagedSessionCount());
			System.out.println("---getActivationTime " + getSocketAcceptor().getActivationTime());
			System.out.println("---getLastIoTime " + getSocketAcceptor().getStatistics().getLastIoTime());
			System.out.println("---getLastReadTime " + getSocketAcceptor().getStatistics().getLastReadTime());
			System.out.println("---getLastWriteTime " + getSocketAcceptor().getStatistics().getLastWriteTime());
			System.out.println("---getReadBytes " + getSocketAcceptor().getStatistics().getReadBytes());
			System.out.println("---getWrittenBytes " + getSocketAcceptor().getStatistics().getWrittenBytes());
			System.out.println("---getReadMessages " + getSocketAcceptor().getStatistics().getReadMessages());
			System.out.println("---getWrittenMessages " + getSocketAcceptor().getStatistics().getWrittenMessages());
			System.out.println("---getReadBytesThroughput " + getSocketAcceptor().getStatistics().getReadBytesThroughput());
			System.out.println("---getWrittenBytesThroughput " + getSocketAcceptor().getStatistics().getWrittenBytesThroughput());
			System.out.println("---getReadMessagesThroughput " + getSocketAcceptor().getStatistics().getReadMessagesThroughput());
			System.out.println("---getWrittenMessagesThroughput " + getSocketAcceptor().getStatistics().getWrittenMessagesThroughput());
			System.out.println("---getLargestReadBytesThroughput " + getSocketAcceptor().getStatistics().getLargestReadBytesThroughput());
			System.out.println("---getLargestWrittenBytesThroughput " + getSocketAcceptor().getStatistics().getLargestWrittenBytesThroughput());
			System.out.println("---getLargestReadMessagesThroughput " + getSocketAcceptor().getStatistics().getLargestReadMessagesThroughput());
			System.out.println("---getLargestWrittenMessagesThroughput " + getSocketAcceptor().getStatistics().getLargestWrittenMessagesThroughput());
			System.out.println("---getThroughputCalculationInterval " + getSocketAcceptor().getStatistics().getThroughputCalculationInterval());
			System.out.println("---getScheduledWriteBytes " + getSocketAcceptor().getStatistics().getScheduledWriteBytes());
			System.out.println("---getScheduledWriteMessages " + getSocketAcceptor().getStatistics().getScheduledWriteMessages());

			System.out.println("-------------------------------------------");
			System.out.println();
			
			try {
				sleep(3000);
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
			}
			
		}
		
	};
	
	printT.start();
	}
	
}
