package org.act.xservices.rt.xcontainer.transport.http;

import java.io.File;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.nio.channels.FileChannel;
import java.security.InvalidParameterException;

import org.act.xservices.log.Logger;
import org.act.xservices.log.LoggerFactory;
import org.act.xservices.rt.hs.common.DefaultHttpResponse;
import org.act.xservices.rt.hs.common.HttpResponseStatus;
import org.act.xservices.rt.hs.common.MutableHttpResponse;
import org.act.xservices.rt.hs.server.HttpService;
import org.act.xservices.rt.hs.server.HttpServiceContext;
import org.act.xservices.rt.xcontainer.framework.IContainerEnvironment;
import org.act.xservices.rt.xcontainer.framework.objectpool.IDispatchTarget;
import org.act.xservices.rt.xcontainer.inf.context.IATFTransportContext;
import org.act.xservices.rt.xcontainer.inf.context.impl.ATFTransportContext;
import org.act.xservices.rt.xcontainer.transport.TransportConstants;
import org.act.xservices.rt.xcontainer.transport.http.codec.ATFHttpServiceContext;
import org.act.xservices.rt.xcontainer.transport.http.codec.ATFHttpTransportSession;
import org.apache.mina.core.buffer.IoBuffer;

/**
 * WebService Service, regist to ServiceContainer
 * 
 * @LastModifyTime 2008-11-30
 * @CreationTime 2008-11-30
 * @author Lizq [lizq@act.buaa.edu.cn]
 * @since JDK1.5
 */

public class WSHttpService implements HttpService{
	
	final Logger logger = LoggerFactory.getLogger(WSHttpService.class);
	
	private IContainerEnvironment containerEnvironment;
	
	private IDispatchTarget<IATFTransportContext> _inPool;
	
	private HttpTransportSender httpSender = new HttpTransportSender();
	
	
	public void setInPool(IDispatchTarget<IATFTransportContext> inPool) {
		this._inPool = inPool;
	}
	
	
	public void setContainerEnvironment(IContainerEnvironment containerEnvironment) {
		this.containerEnvironment = containerEnvironment;
	}
	
	
	
	public void handleRequest(HttpServiceContext context) throws Exception {
		
		ATFHttpServiceContext atfHttpcontext = (ATFHttpServiceContext) context;

    	if("GET".equalsIgnoreCase(atfHttpcontext.getRequest().getMethod().name())){
        	processGet(atfHttpcontext);           
        }
    	else if("POST".equalsIgnoreCase(atfHttpcontext.getRequest().getMethod().name())){
    		processPost(atfHttpcontext);
        }
		
	}

	
	//Get
	private void processGet(ATFHttpServiceContext atfHttpContext) throws Exception{
        
		final String wsdl_request = "?wsdl";
		
		final String rawPath = atfHttpContext.getRequest().getRequestUri().toString();

		MutableHttpResponse response = new DefaultHttpResponse();
		
		if (rawPath.endsWith(wsdl_request)) {
			
			String serviceName = rawPath.substring(0,rawPath.lastIndexOf(wsdl_request));
			
			logger.info("Get WSDL File of Service " + serviceName);
			
			File wsdlFile = containerEnvironment.getWSDLFileByURL(serviceName);		
			
            if (wsdlFile!=null && wsdlFile.exists() && wsdlFile.isFile()) {

				response.setStatus(HttpResponseStatus.OK);

				response.setHeader("Pragma", "no-cache");
				response.setHeader("Cache-Control", "no-cache");
				response.setHeader("Content-Type", "text/xml; charset=UTF-8");

				response.setContent(loadFile(wsdlFile));

			} else {
				response.setStatus(HttpResponseStatus.NOT_FOUND);
				response.setStatusReasonPhrase("WSDL File of Service \"" + serviceName + "\"");
			}
            
		} else {
            // the file is not found, we send the famous 400 error
            response.setStatus(HttpResponseStatus.BAD_REQUEST);
            response.setStatusReasonPhrase( rawPath );
        }
		
		atfHttpContext.commitResponse(response);  // Ϣ
		
	}
	
	// Post
	private final void processPost(ATFHttpServiceContext atfHttpContext) throws Exception{
		
		ATFHttpTransportSession atfSession = new ATFHttpTransportSession(atfHttpContext);
		
		atfSession.setTransportSender(httpSender); //˴ע⣬һҪ
		
		ATFTransportContext atfContext = new ATFTransportContext();
		
		atfContext.setATFTransportSession(atfSession);
		
		//TODO ñ壬˴鸴ƣǷӰܣ lizq2008-12-03
		IoBuffer bufferContent = atfHttpContext.getRequest().getContent();
		byte[] byteContent = new byte[bufferContent.remaining()];
		bufferContent.get(byteContent);	
		atfContext.setRequestContent(byteContent);
		
		atfContext.setRequestContentType(atfHttpContext.getRequest().getContentType());
		
		atfContext.setRequestUri(atfHttpContext.getRequest().getRequestUri());
		
		atfContext.setRequestSoapAction(getRealAction(atfHttpContext.getRequest().getHeader(
				TransportConstants.HTTP_HEADER_SOAPACTION)));
		
		_inPool.put(atfContext);
	}

	
	/**
	 * ȥSOAPActionпܴڵ
	 * 
	 * @param actionParameter
	 * @return
	 */
	private String getRealAction(String actionParameter){
		// SOAPAction
		if(actionParameter == null)
			return null;
		
		if("".equals(actionParameter))
			return "";
		
		String[] stemp = actionParameter.split("\'|\"");
		if(stemp.length != 0)
			return stemp[stemp.length-1].trim();

		return null;
	}

	private IoBuffer loadFile(File file) throws IOException {
		if(file.length()> Integer.MAX_VALUE)
				throw new InvalidParameterException("File "+file+" is too big for SimpleFileLoader try MmapFileLoader");
		IoBuffer buffer=IoBuffer.allocate((int)file.length());
		RandomAccessFile raf = new RandomAccessFile(file, "r");
        FileChannel fc = raf.getChannel();
        fc.read(buffer.buf());
		buffer.flip();
		fc.close();
		raf.close();
		return buffer;
	}
	
	public void start() {
		// TODO Auto-generated method stub
		
	}

	public void stop() {
		// TODO Auto-generated method stub
		
	}

	
	
}
