package org.act.xservices.rt.xcontainer.transport.http.mina;

import java.io.IOException;
import java.net.InetSocketAddress;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import org.act.xservices.rt.xcontainer.framework.config.IConfigElement;
import org.act.xservices.rt.xcontainer.framework.transport.ITransportProtocol;
import org.act.xservices.rt.xcontainer.framework.transport.ITransportReceiver;
import org.act.xservices.rt.xcontainer.inf.context.IATFTransportSession;
import org.act.xservices.rt.xcontainer.transport.http.mina.codec.HTTPServerProtocolCodecFactory;
import org.apache.mina.core.filterchain.IoFilter;
import org.apache.mina.core.service.IoServiceStatistics;
import org.apache.mina.filter.codec.ProtocolCodecFilter;
import org.apache.mina.filter.executor.ExecutorFilter;
import org.apache.mina.transport.socket.nio.NioSocketAcceptor;


/**
 * HTTPЭɷԴ
 * @version 1.0 2008-6-5
 * @author zhangdecai <zhangdc@act.buaa.edu.cn>
 * @author Lizq [lizq@act.buaa.edu.cn]
 * @since JDK1.5
 */
public class HTTPMINATransportReceiver implements ITransportReceiver{

	private String discription;

	private int port = 0;

	private int backlog;

	private boolean isReuseAddress = true;
	
	
	private InetSocketAddress address;
	
	private NioSocketAcceptor acceptor = new NioSocketAcceptor(
			Runtime.getRuntime().availableProcessors() + 1);

	private ITransportProtocol protocol;

	private ExecutorService executorFilterThreadpool;

	private IoServiceStatistics statistics;
	
	
	
	
	public HTTPMINATransportReceiver() {
		
	}
	
	public ITransportProtocol getTransportProtocol() {
		return protocol;
	}

//	public void setTransportProtocol(ITransportProtocol protocol) {
//		this.protocol = protocol;
//	}
	
	
	
	/** 
	 * ɷԴ
	 * @param receiverConfig
	 */
	public void initialize(IConfigElement receiverConfig, ITransportProtocol protocol) throws Exception{
		
		this.protocol = protocol;
		
		this.discription = receiverConfig.getAttribute("description"); 
		
		//ͳƼʱ
		int throughputCalculationInterval = Integer.parseInt(
				receiverConfig.getAttribute("throughputCalculationInterval"));
		
    	// ServerSocketϢ
        IConfigElement serverSocketConfig = 
        	receiverConfig.getFirstChildWithName("serverSocket");
        
        this.port = Integer.parseInt(serverSocketConfig.getAttribute("port")); 
		this.address = new InetSocketAddress(port);
        this.backlog = Integer.parseInt(serverSocketConfig.getAttribute("backlog"));
        
		acceptor.setBacklog(this.backlog);
		acceptor.setReuseAddress(this.isReuseAddress);
		acceptor.setDefaultLocalAddress(address);
		statistics = acceptor.getStatistics();
		statistics.setThroughputCalculationInterval(throughputCalculationInterval);

		
		//  handler Ϣ
		HTTPTransportHandler transportHandler = new HTTPTransportHandler();
		// Handlerм
		transportHandler.setContainerEnvironment(this.protocol.getContainerEnvironment());
		transportHandler.setInPool(this.protocol.getConnector().getInputTarget());
		acceptor.setHandler(transportHandler);
		
				
		// ùϢ

		// init filters,such as LogFilter.
		IConfigElement filterChainConfig = 
			receiverConfig.getFirstChildWithName("filterChain");
		
		IConfigElement[] filtersConfigList = 
			filterChainConfig.getChildren();

		for (IConfigElement filterConfig : filtersConfigList) {
			try {
				
				if ("true".equalsIgnoreCase(filterConfig.getAttribute("enabled"))) {

					if("HTTPProtocolFilter".equalsIgnoreCase(filterConfig.getAttribute("name"))){
						// HTTPЭ鴦Ǳӵ
				        acceptor.getFilterChain().addLast( "HTTPProtocolFilter",
				                new ProtocolCodecFilter(new HTTPServerProtocolCodecFactory()));
				        continue;
					}
					
					//Ӷ̹߳
					if ("ExecutorFilter".equalsIgnoreCase(filterConfig.getAttribute("name"))) {
						
						if(this.executorFilterThreadpool == null){
							if("false".equalsIgnoreCase(filterConfig.getAttribute("useCachedThreadPool"))){
								
								int executorThreadNumber = Integer.valueOf(
										filterConfig.getAttribute("ExecutorThreadNumber"));
								executorFilterThreadpool = Executors.newFixedThreadPool(
										executorThreadNumber);
							}else{
								executorFilterThreadpool = Executors.newCachedThreadPool();
							}
						}
				        acceptor.getFilterChain().addLast("ExecutorFilter", 
				        		new ExecutorFilter(executorFilterThreadpool));
				        continue;
					}

					IoFilter ioFilter = (IoFilter) Class.forName(
							filterConfig.getAttribute("class")).newInstance();

						// ԾFilterõɷԴֵ֧Ĵ
					acceptor.getFilterChain().addLast(
							filterConfig.getAttribute("name"), ioFilter);
					

				}
			} catch (Exception e) {
				e.printStackTrace();
			}
		}

	}
	

	public boolean isStarted() {
		return acceptor.isActive();
	}

	public void setListenAddress(InetSocketAddress address) {
		if (address != null) {
			if (isStarted())
				throw new IllegalStateException(
						"can't set listen address after acceptor started");
			this.address = address;
		}
	}

	public InetSocketAddress getListenAddress() {
		return address;
	}

	public void setListenPort(int port) {
		if (isStarted())
			throw new IllegalStateException(
					"can't set listen port after acceptor started");
		this.port = port;
		this.address = new InetSocketAddress(port);
	}

	public int getListenPort() {
		return port;
	}

	public int getBacklog() {
		return backlog;
	}

	public void setBacklog(int backlog) {
		if (isStarted())
			throw new IllegalStateException(
					"can't set backlog after acceptor started");
		this.backlog = backlog;
	}

	
	public String getDescription(){
		return discription;
	}
	
	
	public synchronized void startup() {
		if (isStarted())
			return;     
		try {
			acceptor.bind();
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		
//		Thread printT = new Thread(){
//		
//		public void run(){
//			
//			while(true){
//			System.out.println("-------------------------------------------");
//			System.out.println("---getManagedSessionCount " + acceptor.getManagedSessionCount());
//			System.out.println("---getLargestManagedSessionCount " + acceptor.getLargestManagedSessionCount());
//			System.out.println("---getCumulativeManagedSessionCount " + acceptor.getCumulativeManagedSessionCount());
//			System.out.println("---getActivationTime " + acceptor.getActivationTime());
//			System.out.println("---getLastIoTime " + acceptor.getLastIoTime());
//			System.out.println("---getLastReadTime " + acceptor.getLastReadTime());
//			System.out.println("---getLastWriteTime " + acceptor.getLastWriteTime());
//			System.out.println("---getReadBytes " + acceptor.getReadBytes());
//			System.out.println("---getWrittenBytes " + acceptor.getWrittenBytes());
//			System.out.println("---getReadMessages " + acceptor.getReadMessages());
//			System.out.println("---getWrittenMessages " + acceptor.getWrittenMessages());
//			System.out.println("---getReadBytesThroughput " + acceptor.getReadBytesThroughput());
//			System.out.println("---getWrittenBytesThroughput " + acceptor.getWrittenBytesThroughput());
//			System.out.println("---getReadMessagesThroughput " + acceptor.getReadMessagesThroughput());
//			System.out.println("---getWrittenMessagesThroughput " + acceptor.getWrittenMessagesThroughput());
//			System.out.println("---getLargestReadBytesThroughput " + acceptor.getLargestReadBytesThroughput());
//			System.out.println("---getLargestWrittenBytesThroughput " + acceptor.getLargestWrittenBytesThroughput());
//			System.out.println("---getLargestReadMessagesThroughput " + acceptor.getLargestReadMessagesThroughput());
//			System.out.println("---getLargestWrittenMessagesThroughput " + acceptor.getLargestWrittenMessagesThroughput());
//			System.out.println("---getThroughputCalculationInterval " + acceptor.getThroughputCalculationInterval());
//			System.out.println("---getScheduledWriteBytes " + acceptor.getScheduledWriteBytes());
//			System.out.println("---getScheduledWriteMessages " + acceptor.getScheduledWriteMessages());
//
//			System.out.println("-------------------------------------------");
//			System.out.println();
//			
//			try {
//				sleep(3000);
//			} catch (InterruptedException e) {
//				e.printStackTrace();
//			}
//			}
//			
//		}
//		
//	};
//	
//	printT.start();
	
	}
	
    
	public synchronized void shutdown() {
		acceptor.unbind();
		//ʹõ̳߳أҪ
		if(executorFilterThreadpool != null)
			executorFilterThreadpool.shutdown();
	}
	
	
	
	/** 
	 * ˷Ч
	 * @return
	 * @throws InterruptedException
	 */
	public IATFTransportSession get() throws InterruptedException {
		throw new InterruptedException("This method is invalid!");
	}
	//---------------------------------------------------
	

	
	//----------------------ͳϢ-----------------------
    /**
     * ܹյSocket
     * @return
     */
    public long getAcceptedSocketCount() {
        return statistics.getCumulativeManagedSessionCount();
    }
    
	
	/**
	 * ʾͳƹǷ
	 * @return
	 */
	public boolean isThroughputStatisticAvailable(){
		return true;
	}
	

    /**
     * Get average receive speed, in byte/s.
     * ƽٶ
     * 
     * @return average receive speed,or -1 means statistic Function Unavailable.
     */
    public double getAvgReceiveSpeed() {
    	if(isThroughputStatisticAvailable()){
    		return getReceivedBytes() * 1000.0 / getRunningTimeInMillis();
    	}
        return -1;
    }

    /**
     * Get average send speed, in byte/s.
     * ƽٶ
     * 
     * @return average send speed,or -1 means statistic Function Unavailable.
     */
    public double getAvgSendSpeed() {
    	if(isThroughputStatisticAvailable()){
    		return getSentBytes() * 1000.0 / getRunningTimeInMillis();
    	}
        return -1;
    }
    
    /**
     * Get instantaneous receive speed, in byte/s.
     * ʵʱĽٶ
     * 
     * @return received speed,or -1 means statistic Function Unavailable.
     */
    public double getReceiveSpeed() {
    	if(isThroughputStatisticAvailable()){
    		return statistics.getReadBytesThroughput();
    	}
        return -1;
    }

    /**
     * Get instantaneous send speed, in byte/s.
     * ʵʱķٶ
     * 
     * @return send speed,or -1 means statistic Function Unavailable.
     */
    public double getSendSpeed() {
    	if(isThroughputStatisticAvailable()){
    		return statistics.getWrittenBytesThroughput();
    	}
        return -1;
    }
    
    

    /**
     * Get elapsed time after session started and before session closed, in ms.
     * ʱ
     * 
     * @return elapsed time,or -1 means statistic Function Unavailable.
     */
    public long getRunningTimeInMillis() {
    	if(isThroughputStatisticAvailable()){
    		return System.currentTimeMillis() - acceptor.getActivationTime();
    	}
        return -1;
    }
    
    /**
     * ôɷԴʱ
     * @return
     */
    public long getStartTimeInMillis(){
    	return acceptor.getActivationTime();
    }
    
    
    /**
     * Get received bytes count. If session restart, the value will reset to 0.
     * ֽܹܵ
     * 
     * @return received bytes count,or -1 means statistic Function Unavailable.
     */
    public long getReceivedBytes() {
    	if(isThroughputStatisticAvailable()){
    		return acceptor.getStatistics().getReadBytes();
    	}
        return -1;
    }

    /**
     * Get sent bytes count. If session restart, the value will reset to 0.
     * ֽܹ
     * 
     * @return sent bytes count,or -1 means statistic Function Unavailable.
     */
    public long getSentBytes() {
    	if(isThroughputStatisticAvailable()){
    		return statistics.getWrittenBytes();
    	}
        return -1;
    }


    //---------------
	public double getAvgMessageReceiveSpeed() {
    	if(isThroughputStatisticAvailable()){
    		return getReceivedMessages() * 1000.0 / getRunningTimeInMillis();
    	}
        return -1;
	}


	public double getAvgMessageSendSpeed() {
    	if(isThroughputStatisticAvailable()){
    		return getSentMessages() * 1000.0 / getRunningTimeInMillis();
    	}
        return -1;
	}


	public long getLargestManagedSocketCount() {
    	if(isThroughputStatisticAvailable()){
    		return statistics.getLargestManagedSessionCount();
    	}
        return -1;
	}


	public double getLargestMessageReceiveSpeed() {
    	if(isThroughputStatisticAvailable()){
    		return statistics.getLargestReadMessagesThroughput();
    	}
        return -1;
	}


	public double getLargestMessageSendSpeed() {
    	if(isThroughputStatisticAvailable()){
    		return statistics.getLargestWrittenMessagesThroughput();
    	}
        return -1;
	}


	public double getLargestReceiveSpeed() {
    	if(isThroughputStatisticAvailable()){
    		return statistics.getLargestReadBytesThroughput();
    	}
        return -1;
	}


	public double getLargestSendSpeed() {
    	if(isThroughputStatisticAvailable()){
    		return statistics.getLargestWrittenBytesThroughput();
    	}
        return -1;
	}


	//TODO ˴õǿת
	public long getManagedSocketCount() {
    	if(isThroughputStatisticAvailable()){
    		return statistics.getCumulativeManagedSessionCount();//.getManagedSessionCount();
    	}
        return -1;
	}


	public double getMessageReceiveSpeed() {
    	if(isThroughputStatisticAvailable()){
    		return statistics.getReadMessagesThroughput();
    	}
        return -1;
	}


	public long getReceivedMessages() {
    	if(isThroughputStatisticAvailable()){
    		return statistics.getReadMessages();
    	}
        return -1;
	}


	public double getSendMessageSpeed() {
    	if(isThroughputStatisticAvailable()){
    		return statistics.getWrittenMessagesThroughput();
    	}
        return -1;
	}


	public long getSentMessages() {
    	if(isThroughputStatisticAvailable()){
    		return statistics.getWrittenMessages();
    	}
        return -1;
	}


	public int getThroughputCalculationInterval() {
    	if(isThroughputStatisticAvailable()){
    		return statistics.getThroughputCalculationInterval();
    	}
        return -1;
	}


	public void setThroughputCalculationInterval(int throughputCalculationInterval) {
		statistics.setThroughputCalculationInterval(throughputCalculationInterval);
		
	}
    
}
