package org.act.xservices.rt.xcontainer.transport.http.mina;

import java.io.File;
import java.io.FileInputStream;
import java.net.HttpURLConnection;

import org.act.xservices.log.Logger;
import org.act.xservices.log.LoggerFactory;
import org.act.xservices.rt.xcontainer.framework.IContainerEnvironment;
import org.act.xservices.rt.xcontainer.framework.objectpool.IDispatchTarget;
import org.act.xservices.rt.xcontainer.inf.context.IATFTransportContext;
import org.act.xservices.rt.xcontainer.inf.context.impl.ATFTransportContext;
import org.act.xservices.rt.xcontainer.transport.http.mina.codec.HTTPTransportSession;
import org.apache.mina.core.service.IoHandlerAdapter;
import org.apache.mina.core.session.IdleStatus;
import org.apache.mina.core.session.IoSession;

/**
 * HTTP
 * 
 * @version 1.0 2008-10-12
 * @author zhangdecai <zhangdc@act.buaa.edu.cn>
 * @author Lizq [lizq@act.buaa.edu.cn]
 * @since JDK1.5
 */

public class HTTPTransportHandler extends IoHandlerAdapter{

	final Logger logger = LoggerFactory.getLogger(HTTPTransportHandler.class);
	
	private IContainerEnvironment _containerEnvironment;
	
	private IDispatchTarget<IATFTransportContext> _inPool;
	
	
	public void setInPool(IDispatchTarget<IATFTransportContext> inPool) {
		this._inPool = inPool;
	}
	
	
	public void setContainerEnvironment(IContainerEnvironment containerContext) {
		this._containerEnvironment = containerContext;
	}
	
	

    public void sessionOpened(IoSession session) throws Exception {
        // set idle time to 60 seconds
    	session.getConfig().setIdleTime(IdleStatus.BOTH_IDLE, 120);
    }

    public void sessionClosed(IoSession session) throws Exception {
    	//TODO liujian 2008-10-05
    	//System.out.println("HTTPTransportHandler.sessionClosed():"+session);
    }

    public void sessionIdle(IoSession session, IdleStatus status)throws Exception {
        session.close(false);
    }


    //TODO  2008-10-05 ϢɺҪرsession߽ Զ̷ǿƹرӵĴ
    public void messageSent(IoSession session, Object message) throws Exception {
    	//session.close();
    }
    
    public void exceptionCaught(IoSession session, Throwable cause) throws Exception {  	
    	
    	//쳣. close ǰSocketChannel , cancel ǰSelectionKey 
    	System.out.println( "HTTPTransportHandler.exceptionCaught() : ");
    	cause.printStackTrace();
    	session.close(false);
    }
    
    public void messageReceived(IoSession session, Object message)throws Exception {
    	
    	//ԱĽHTTP߼
    	ATFTransportContext traContext = (ATFTransportContext) message;
    	HTTPTransportSession httpSession = (HTTPTransportSession) traContext.getATFTransportSession();
    	if("GET".equalsIgnoreCase(httpSession.getRequestMethod())){
        	processGet(traContext);           
        }
    	else if("POST".equalsIgnoreCase(httpSession.getRequestMethod())){
    		processPost(traContext);
        }
	}

	
	
	
	//Get
	private void processGet(ATFTransportContext traContext) throws Exception{
        
		String wsdl_request = "?wsdl";
		
		String rawPath = traContext.getRequestUriString();

		HTTPTransportSession session = (HTTPTransportSession) traContext.getATFTransportSession();
        
		//ATFTransportContext responseObject = new ATFTransportContext();

        //responseObject.setATFTransportSession(requestObject.getATFTransportSession());
        
		if (rawPath.endsWith(wsdl_request)) {
			
			String serviceName = rawPath.substring(0,rawPath.lastIndexOf(wsdl_request));
			
			logger.debug("Get WSDL File of Service "+serviceName);
			
			File wsdlFile = _containerEnvironment.getWSDLFileByURL(serviceName);
			
            if (wsdlFile!=null && wsdlFile.exists() && wsdlFile.isFile()) {
                session.setResponseCode(HttpURLConnection.HTTP_OK);

                //TODO  2008-10-05 Ĵ
				switch (session.getResponseCode()) {
					case HttpURLConnection.HTTP_OK:
						session.setReasonPhrase("OK");
						break;
					case HttpURLConnection.HTTP_NOT_FOUND:
						session.setReasonPhrase("Not Found");
						break;
				}

				traContext.setParameter("Content-Type", "text/xml; charset=UTF-8");
				traContext.setResponseContentType("text/xml; charset=UTF-8");
				
				FileInputStream is = new FileInputStream(wsdlFile);
				int length = is.available();
				byte[] bytes = new byte[length];
				is.read(bytes);
				
				//עⱨĳȲڱʱӣʹλͷһ
				//responseContext.setParameter("Content-Length", String.valueOf(length));
				
				traContext.setResponseContent(bytes);
            }   
        }
		
		send(traContext);
	}
	
	// Post
	private final void processPost(ATFTransportContext requestObject)throws Exception{		
		_inPool.put(requestObject);
	}


	//ͱ
	private IATFTransportContext send(IATFTransportContext transportObject) throws Exception {
		
		return transportObject.getATFTransportSession().getTransportSender().send(transportObject);
	}

}
