package org.act.xservices.rt.xcontainer.transport.http.mina.codec;

import java.net.URI;
import java.nio.charset.CharacterCodingException;
import java.nio.charset.Charset;
import java.nio.charset.CharsetDecoder;

import org.act.xservices.rt.xcontainer.framework.transport.ITransportSender;
import org.act.xservices.rt.xcontainer.inf.context.impl.ATFTransportContext;
import org.act.xservices.rt.xcontainer.transport.TransportConstants;
import org.apache.mina.core.buffer.IoBuffer;
import org.apache.mina.core.session.IoSession;
import org.apache.mina.filter.codec.ProtocolDecoderOutput;
import org.apache.mina.filter.codec.demux.MessageDecoderAdapter;
import org.apache.mina.filter.codec.demux.MessageDecoderResult;

/**
 *  ֽתΪ
 * 
 * @LastModifyTime 2008-10-11
 * @CreationTime
 * @author Lizq [lizq@act.buaa.edu.cn]
 * @since JDK1.5
 */
public class TransportContextDecoder extends MessageDecoderAdapter implements
		Cloneable {

	private static final byte[] TOKEN = "\r\n\r\n".getBytes();

	private static final byte[] CONTENT_LENGTH = new String("Content-Length:")
			.getBytes();

	private final String charsetName = "UTF-8";

	private CharsetDecoder decoder = Charset.forName(charsetName).newDecoder();

	private ATFTransportContext requestContext = null;

	private ITransportSender transportSender; // = new HTTPSender();

	public TransportContextDecoder() {

	}

	/**
	 * жϴ˽Ƿɽ˱
	 * 
	 * @param session
	 * @param in
	 * @return
	 */
	public MessageDecoderResult decodable(IoSession session, IoBuffer in) {
		// Return NEED_DATA if the whole header is not read yet.
		try {
			return messageComplete(in) ? MessageDecoderResult.OK
					: MessageDecoderResult.NEED_DATA;
		} catch (Exception ex) {
			ex.printStackTrace();
		}
		// TODO ˴дĽlizq
		return MessageDecoderResult.NOT_OK;
	}

	public MessageDecoderResult decode(IoSession session, IoBuffer in,
			ProtocolDecoderOutput out) throws Exception {
		// Try to decode body
		ATFTransportContext m = decodeBody(in, session);

		// Return NEED_DATA if the body is not fully read.
		if (m == null)
			return MessageDecoderResult.NEED_DATA;

		// ContentTypeAction
		requestContext.setRequestContentType(requestContext
				.getParameter(TransportConstants.HTTP_HEADER_CONTENTTYPE));

		// SOAPAction
		requestContext.setRequestSoapAction(getRealAction(requestContext
				.getParameter(TransportConstants.HTTP_HEADER_SOAPACTION)));

		out.write(m);

		return MessageDecoderResult.OK;
	}

	/**
	 * ȥSOAPActionпܴڵ
	 * 
	 * @param actionParameter
	 * @return
	 */
	private String getRealAction(String actionParameter) {
		// SOAPAction
		if (actionParameter == null)
			return null;

		if ("".equals(actionParameter))
			return "";

		String[] stemp = actionParameter.split("\'|\"");
		if (stemp.length != 0)
			return stemp[stemp.length - 1].trim();

		return null;
	}

	/**
	 * жϢǷ
	 * 
	 * @param in
	 * @return
	 * @throws Exception
	 */
	private boolean messageComplete(IoBuffer in) throws Exception {

		int last = in.remaining() - 1;
		if (in.remaining() < 4)
			return false;

		// to speed up things we check if the Http request is a GET or POST
		if (in.get(0) == (byte) 'G' && in.get(1) == (byte) 'E'
				&& in.get(2) == (byte) 'T') {

			// Http GET request therefore the last 4 bytes should be 0x0D 0x0A
			// 0x0D 0x0A
			return (in.get(last) == (byte) 0x0A
					&& in.get(last - 1) == (byte) 0x0D
					&& in.get(last - 2) == (byte) 0x0A && in.get(last - 3) == (byte) 0x0D);

		} else if (in.get(0) == (byte) 'P' && in.get(1) == (byte) 'O'
				&& in.get(2) == (byte) 'S' && in.get(3) == (byte) 'T') {

			// Http POST request
			// first the position of the 0x0D 0x0A 0x0D 0x0A bytes
			int eoh = -1;

			for (int i = 4; i <= in.remaining() - 4; i++) {
				if (in.get(i) == (byte) 0x0D && in.get(i + 1) == (byte) 0x0A
						&& in.get(i + 2) == (byte) 0x0D
						&& in.get(i + 3) == (byte) 0x0A) {

					eoh = i + 4;
					break;
				}
			}

			if (eoh == -1)
				return false;

			for (int i = 0; i < last; i++) {

				boolean found = false;

				for (int j = 0; j < CONTENT_LENGTH.length; j++) {
					if (in.get(i + j) != CONTENT_LENGTH[j]) {
						found = false;
						break;
					}
					found = true;
				}

				if (found) {
					// retrieve value from this position till next 0x0D 0x0A
					StringBuilder contentLength = new StringBuilder();
					for (int j = i + CONTENT_LENGTH.length; j < last; j++) {
						if (in.get(j) == 0x0D)
							break;
						contentLength.append(new String(
								new byte[] { in.get(j) }));
					}
					// if content-length worth of data has been received then
					// the message is complete
					return (Integer.parseInt(contentLength.toString().trim())
							+ eoh == in.remaining());
				}
			}
		}

		// the message is not complete and we need more data
		return false;
	}

	private ATFTransportContext decodeBody(IoBuffer in, IoSession session) {
		requestContext = new ATFTransportContext();

		HTTPTransportSession httpSession = new HTTPTransportSession(session);
		httpSession.setTransportSender(transportSender);

		requestContext.setATFTransportSession(httpSession);
		try {
			if (parseRequest(in, requestContext)) {

				// requestContext.setTransportProtocol(TransportConstants.PROTOCOL_HTTP);

				return requestContext;
			}

			return null;

		} catch (CharacterCodingException ex) {
			ex.printStackTrace();
			return null;
		}

	}

	private int getIndex(IoBuffer in, byte[] b) {
		if (b.length == 0)
			return in.position();

		int startIndex = in.position();
		int endIndex = startIndex + in.remaining() - b.length;
		byte first = b[0];

		Label: for (int i = startIndex; i <= endIndex; i++) {
			// look for the first character
			if (in.get(i) == first) {
				for (int j = 1; j < b.length; j++) {
					if (in.get(i + j) != b[j]) {
						continue Label;
					}
				}
				return i;
			}
		}// end of for
		return -1;
	}

	/**
	 * HTTP
	 * 
	 * @param
	 * @return
	 */
	private boolean parseRequest(IoBuffer in, ATFTransportContext requestContext)
			throws CharacterCodingException {

		int index = getIndex(in, TOKEN);
		if (index < 0) // HTTPͷ
			return false;

		String[] headers = in.getString(index + TOKEN.length, decoder).split(
				"\r\n");

		// Get request URL.
		String[] url = headers[0].split(" ");
		if (url.length < 3)
			return false;

		HTTPTransportSession session = (HTTPTransportSession) requestContext
				.getATFTransportSession();
		session.setRequestMethod(url[0].toUpperCase());
		requestContext.setRequestUri(URI.create(url[1]));
		session.setHttpVersion(url[2]);

		// Read header
		for (int i = 1; i < headers.length; i++) {
			String[] tokens = headers[i].split(":");
			requestContext.setParameter(tokens[0].trim(), tokens[1].trim());
		}

		// If method 'POST' then read Content-Length worth of data
		if (url[0].equalsIgnoreCase("POST")) {

			String bodylen_str = requestContext.getParameter("Content-Length");

			// ڱ壬ȡ
			if (bodylen_str != null && (!"0".equals(bodylen_str))) {

				int bodylen = Integer.parseInt(bodylen_str);
				byte[] content = new byte[bodylen];

				if (in.remaining() >= bodylen) {// ģֱӶȡ
					in.get(content);
					requestContext.setRequestContent(content);

				} else {// 岻޷Context
					return false;
				}
			}

		}
		return true;
	}

	public void setTransportSender(ITransportSender transportSender) {
		this.transportSender = transportSender;
	}

	@Override
	public Object clone() {
		try {
			return super.clone();
		} catch (CloneNotSupportedException e) {
			throw new RuntimeException(e);
		}
	}
}
