package org.act.xservices.rt.xcontainer.transport.http.mina.codec;

import java.nio.charset.CharacterCodingException;
import java.nio.charset.Charset;
import java.nio.charset.CharsetEncoder;
import java.util.Collections;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;
import java.util.Map.Entry;

import org.act.xservices.rt.xcontainer.inf.context.impl.ATFTransportContext;
import org.apache.mina.core.buffer.IoBuffer;
import org.apache.mina.core.session.IoSession;
import org.apache.mina.filter.codec.ProtocolEncoderOutput;
import org.apache.mina.filter.codec.demux.MessageEncoder;

/**
 * A MessageEncoder that encodes TransportObject
 * @LastModifyTime 2008-10-11
 * @CreationTime
 * @author Lizq [lizq@act.buaa.edu.cn]
 * @since JDK1.5
 */

public class TransportContextEncoder implements MessageEncoder {
    
	private static final Set<Class<?>> TYPES;

    static {
        Set<Class<?>> types = new HashSet<Class<?>>();
        types.add(ATFTransportContext.class);
        TYPES = Collections.unmodifiableSet(types);
    }

    private static final byte[] CRLF = new byte[] { 0x0D, 0x0A };

    public TransportContextEncoder() {
    }

    public void encode(IoSession session, Object message,
            ProtocolEncoderOutput out) throws Exception {
    	
    	ATFTransportContext msg = (ATFTransportContext) message;
    	
        IoBuffer buf = IoBuffer.allocate(512);
        // Enable auto-expand for easier encoding
        buf.setAutoExpand(true);

        try {
        	HTTPTransportSession httpSession = (HTTPTransportSession) msg.getATFTransportSession();
            // output all headers except the content length
            CharsetEncoder encoder = Charset.forName("UTF-8").newEncoder();
            
            buf.putString("HTTP/1.1 ", encoder);
            
            buf.putString(String.valueOf(httpSession.getResponseCode()), encoder);
            
            buf.putString(" "+httpSession.getReasonPhrase(), encoder);
            
            buf.put(CRLF);
            

            //ͷ
            for (Iterator it = msg.getParameters().entrySet().iterator(); it.hasNext();) {
                Entry entry = (Entry) it.next();
                buf.putString((String) entry.getKey(), encoder);
                buf.putString(": ", encoder);
                buf.putString((String) entry.getValue(), encoder);
                buf.put(CRLF);
            }
            
            // now the content length is the body length,עڴ˴ӱĳ
            buf.putString("Content-Length: ", encoder);
            buf.putString(String.valueOf(msg.getResponseContent().length), encoder);
            
            buf.put(CRLF);
            buf.put(CRLF);
            // add body
            buf.put(msg.getResponseContent());
            
            //System.out.println("\n+++++++");
            //for (int i=0; i<buf.position();i++)System.out.print(new String(new byte[]{buf.get(i)}));
            //System.out.println("\n+++++++");
            
        } catch (CharacterCodingException ex) {
            ex.printStackTrace();
        }

        buf.flip();
        out.write(buf);
    }

    /**
     * ˱ֵ֧Ϣ
     * @return
     */
    public Set<Class<?>> getMessageTypes() {
        return TYPES;
    }
}
