package org.act.xservices.rt.xcontainer.transport.https;

import java.io.IOException;
import java.net.InetSocketAddress;
import java.security.GeneralSecurityException;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import javax.net.ssl.SSLContext;


import org.act.xservices.rt.hs.common.codec.HttpCodecFactory;
import org.act.xservices.rt.hs.server.ServiceContainer;
import org.act.xservices.rt.hs.server.Transport;
import org.act.xservices.rt.hs.server.TransportException;
import org.act.xservices.rt.hs.server.ssl.XcontainerSslContextFactory;
import org.act.xservices.rt.hs.server.ssl.XcontainerSslContextFactory1;
import org.act.xservices.rt.hs.server.transport.mina.HttpIoHandler;
import org.act.xservices.rt.xcontainer.framework.config.IConfigElement;
import org.act.xservices.rt.xcontainer.transport.http.ATFHttpIoHandler;
import org.act.xservices.rt.xcontainer.transport.http.codec.ContextConverterFilter;
import org.apache.mina.core.filterchain.DefaultIoFilterChainBuilder;
import org.apache.mina.core.filterchain.IoFilter;
import org.apache.mina.filter.codec.ProtocolCodecFilter;
import org.apache.mina.filter.executor.ExecutorFilter;
import org.apache.mina.filter.executor.OrderedThreadPoolExecutor;
import org.apache.mina.filter.ssl.SslFilter;
import org.apache.mina.transport.socket.SocketAcceptor;
import org.apache.mina.transport.socket.nio.NioSocketAcceptor;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * @LastModifyTime 2010-6-11
 * @author niyong [niyong@act.buaa.edu.cn]
 *
 * @since jdk1.5
 */
public class ATFSSLTransport implements Transport {
	
	private static final Logger LOG = LoggerFactory.getLogger(ATFSSLTransport.class);

	/** Ĭϼ˿ */
	private static final int DEFAULT_PORT = 443;

	/** ĬIO߳ */
	private static final int DEFAULT_IO_THREADS = Runtime.getRuntime().availableProcessors();

	private static final int DEFAULT_EVENT_THREADS = 8;

	
	private int ioThreads = DEFAULT_IO_THREADS;
	
	private int eventThreads = DEFAULT_EVENT_THREADS;

	private SocketAcceptor acceptor;

	private List<ExecutorService> executorList = new ArrayList<ExecutorService>();

	private int port = DEFAULT_PORT;

	private InetSocketAddress address;

	private HttpIoHandler ioHandler;

	private ServiceContainer container;

	private int backlog = 100;

	private boolean isReuseAddress = true;
	
	/**
	 * SSLЭصļԿ⡢SSLЭ .  2010.7.7
	 */
	private String keystoreFile = null;
	
	private char[] keystorePass = null;
	
	private String sslProtocol = null;
	
	
	//    private boolean isLoggingTraffic;
	//    
	//    private LogLevel logLevel = LogLevel.WARN;
	
	
	public String getKeystoreFile() {
		return keystoreFile;
	}

	public void setKeystoreFile(String keystoreFile) {
		this.keystoreFile = keystoreFile;
	}

	public char[] getKeystorePass() {
		return keystorePass;
	}

	public void setKeystorePass(char[] keystorePass) {
		this.keystorePass = keystorePass;
	}

	public String getSslProtocol() {
		return sslProtocol;
	}

	public void setSslProtocol(String sslProtocol) {
		this.sslProtocol = sslProtocol;
	}

	public ATFSSLTransport(){
		
	}

	/**
	 * Sets the port this transport will listen on
	 *
	 * @param port  The port
	 */
	public void setPort(int port) {
		this.port = port;
	}

	/**
	 * Sets the address this transport will listen on
	 *
	 * @param address  The address to bind to.
	 *                 Specify <tt>null</tt> or <tt>"*"</tt> to listen to all
	 *                 NICs (Network Interface Cards).
	 */
	public void setAddress(InetSocketAddress address) {
		this.address = address;
	}

	public int getIoThreads() {
		return ioThreads;
	}

	/**
	 * Sets the number of worker threads employed by this transport.
	 * This should typically be a small number (2 is a good choice) -
	 * and is not tied to the number of concurrent connections you wish to
	 * support
	 *
	 * @param ioThreads  The number of worker threads to employ
	 */
	public void setIoThreads(int ioThreads) {
		this.ioThreads = ioThreads;
	}

	public int getEventThreads() {
		return eventThreads;
	}

	public void setEventThreads(int eventThreads) {
		this.eventThreads = eventThreads;
	}

	//    /**
	//     * Sets whether traffic received through this transport is
	//     * logged (off by default)
	//     *
	//     * @param isLoggingTraffic  <code>true</code> iff traffic should be logged
	//     */
	//    public void setIsLoggingTraffic( boolean isLoggingTraffic )
	//    {
	//        this.isLoggingTraffic = isLoggingTraffic;
	//    }
	//    
	//    
	//    public void setLogLevel( String logLevel )
	//    {
	//    	this.logLevel = LogLevel.valueOf( logLevel );
	//    }

	/**
	 * Sets the <code>ServiceContainer</code> to which we issue requests
	 *
	 * @param container  Our associated <code>ServiceContainer</code>
	 */
	public void setServiceContainer(ServiceContainer container) {
		this.container = container;
	}

	/**
	 * Sets the <code>HttpIOHandler</code> to be employed by this transport
	 *
	 * @param httpIOHandler  The handler to be employed by this transport
	 */
	public void setIoHandler(HttpIoHandler httpIOHandler) {
		this.ioHandler = httpIOHandler;
	}

	/**
	 * config 
	 * @param serverSocketConfig
	 */
	public void config(IConfigElement serverSocketConfig) {

        this.backlog = Integer.parseInt(serverSocketConfig.getAttribute("backlog"));
		
		// ͳƼʱ
        int throughputCalculationInterval = Integer.parseInt(
        		serverSocketConfig.getAttribute("throughputCalculationInterval"));
        
        if("true".equalsIgnoreCase(serverSocketConfig.getAttribute("isReuseAddress"))){
        	this.isReuseAddress = true;
        }else{
        	this.isReuseAddress = false;
        }
        
        if( ! "*".equals(serverSocketConfig.getAttribute("ioThreadNumber"))){
        	this.ioThreads = Integer.parseInt(serverSocketConfig.getAttribute("ioThreadNumber"));
        }
        
        //---config acceptor---
        acceptor = new NioSocketAcceptor(ioThreads);
        
		acceptor.setBacklog(backlog);
		acceptor.setReuseAddress(isReuseAddress);
		acceptor.getStatistics().setThroughputCalculationInterval(throughputCalculationInterval);

		//---ô---
		DefaultIoFilterChainBuilder chain = acceptor.getFilterChain();
		
		try {
			addSSLSupport(chain);
		} catch (GeneralSecurityException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}

		IConfigElement filterChainConfig = serverSocketConfig.getFirstChildWithName("filterChain");
		IConfigElement[] filtersConfigList = filterChainConfig.getChildren();

		for (int i=0; i<filtersConfigList.length; i++) {
			IConfigElement filterConfig = filtersConfigList[i];
			try {	
				if ("true".equalsIgnoreCase(filterConfig.getAttribute("enabled"))) {
					
					//̳߳
					if("eventExecutorFilter".equalsIgnoreCase(filterConfig.getAttribute("name"))){
						this.eventThreads = Integer.parseInt(filterConfig.getAttribute("eventThreadNumber"));
						
						//TODO ע˴ɱ˳̳߳, ˴ҪǷӣby lizq 2008-12-02
						ExecutorService eventExecutor = new OrderedThreadPoolExecutor(this.eventThreads);
						executorList.add(eventExecutor);
						chain.addFirst(filterConfig.getAttribute("name"), new ExecutorFilter(eventExecutor));
						continue;
					}
					
					//httpЭ
					if("httpCodecFilter".equalsIgnoreCase(filterConfig.getAttribute("name"))){
						chain.addLast(filterConfig.getAttribute("name"), new ProtocolCodecFilter(new HttpCodecFactory()));
						continue;
					}
					
					//Contextת
					if("ATFContextConverterFilter".equalsIgnoreCase(filterConfig.getAttribute("name"))){
						chain.addLast(filterConfig.getAttribute("name"), 
								new ContextConverterFilter(container));
						continue;
					}
					
					//Ӷ̹߳
					if ("ExecutorFilter".equalsIgnoreCase(filterConfig.getAttribute("name"))) {
						ExecutorService executorFilterThreadpool;
							if("false".equalsIgnoreCase(filterConfig.getAttribute("useCachedThreadPool"))){
								int executorThreadNumber = Integer.valueOf(filterConfig.getAttribute("ExecutorThreadNumber"));
								executorFilterThreadpool = Executors.newFixedThreadPool(executorThreadNumber);
							}else{
								executorFilterThreadpool = Executors.newCachedThreadPool();
							}
						executorList.add(executorFilterThreadpool);
						chain.addLast(filterConfig.getAttribute("name")+i, new ExecutorFilter(executorFilterThreadpool));
				        continue;
					}

					// 
					IoFilter ioFilter = (IoFilter) Class.forName(filterConfig.getAttribute("class")).newInstance();
					// ԾFilterõɷԴֵ֧Ĵ
					chain.addLast(filterConfig.getAttribute("name"), ioFilter);
					

				}
			} catch (Exception e) {
				e.printStackTrace();
			}
		}// end of for
		

		//Filter
		//        chain.addLast("mdc", new MdcInjectionFilter() );

		//־Filter
		//        if ( isLoggingTraffic )
		//        {
		//            LOG.debug( "Configuring traffic logging filter" );
		//            LoggingFilter filter = new LoggingFilter();
		//            filter.setLogLevel( IoEventType.CLOSE, logLevel );
		//            filter.setLogLevel( IoEventType.EXCEPTION_CAUGHT, logLevel );
		//            filter.setLogLevel( IoEventType.MESSAGE_RECEIVED, logLevel );
		//            filter.setLogLevel( IoEventType.MESSAGE_SENT, logLevel );
		//            filter.setLogLevel( IoEventType.SESSION_CLOSED, logLevel );
		//            filter.setLogLevel( IoEventType.SESSION_CREATED, logLevel );
		//            filter.setLogLevel( IoEventType.SESSION_IDLE, logLevel );
		//            filter.setLogLevel( IoEventType.SESSION_OPENED, logLevel );
		//            filter.setLogLevel( IoEventType.SET_TRAFFIC_MASK, logLevel );
		//            filter.setLogLevel( IoEventType.WRITE, logLevel );
		//            acceptor.getFilterChain().addLast( "logging", filter );
		//        }

	}

	/**
	 * Starts this transport
	 * 
	 * @throws TransportException
	 *             If the transport can not be started
	 */
	public void start() throws TransportException {

		//ʼ
		initIOHandler();
		acceptor.setHandler(ioHandler);
		
		boolean success = false;
		try {
			if (address != null) {
				acceptor.bind(address);
			} else {
				acceptor.bind(new InetSocketAddress(port));
			}
			success = true;
			LOG.debug("NIO HTTP Transport bound on port " + port);
		} catch (IOException e) {
			throw new TransportException("NIOTransport Failed to bind to port "
					+ port, e);
		} finally {
			if (!success) {
				acceptor.dispose();
				acceptor = null;
			}
		}
	}

	/**
	 * Stops this transport
	 */
	public void stop() throws TransportException {
		if (acceptor == null) {
			return;
		}
		acceptor.dispose();
		
		// õ̳߳
		for(ExecutorService es : executorList){
			es.shutdown();
		}
		acceptor = null;
		executorList.clear();
	}

	/**
	 * @return A string representation of this transport
	 */
	@Override
	public String toString() {
		return "NIOTransport [port=" + port + "]";
	}

	/**
	 * Initializes our handler - creating a new (default) handler if none has been specified
	 *
	 * @throws IllegalStateException If we have not yet been associated with a container
	 */
	private void initIOHandler() {
		if (container == null) {
			throw new IllegalStateException("Transport not associated with a container");
		}
		
		if (ioHandler == null) {
			LOG.info("No http IO Handler associated - using defaults");
			ioHandler = new ATFHttpIoHandler();
		}
		ioHandler.setContainer(container);
	}


	public void setBacklog(int backlog) {
		if (isStarted())
			throw new IllegalStateException(
					"can't set backlog after acceptor started");
		this.backlog = backlog;
	}
	
	public int getBacklog() {
		return backlog;
	}

	public void setReuseAddress(boolean isReuseAddress) {
		this.isReuseAddress = isReuseAddress;
	}
	
	public boolean isReuseAddress() {
		return isReuseAddress;
	}
	
	public boolean isStarted() {
		return acceptor.isActive();
	}

	public SocketAcceptor getSocketAcceptor() {
		return acceptor;
	}
	
	
	private void addSSLSupport(DefaultIoFilterChainBuilder chain) throws GeneralSecurityException
	{
		// ͨXcontainerSslContextFactorySSLContext
//		SSLContext sslContext = XcontainerSslContextFactory.getInstance(true);
		
		SSLContext sslContext = XcontainerSslContextFactory1.getInstance(true,keystoreFile,keystorePass,sslProtocol);		
		
		// sslContextsslFilter, sslFilterʵSSLЭľ幦ܣݼܽܣ֤
		XcontainerSslFilter sslFilter = new XcontainerSslFilter(sslContext);
		
		// sslFilter뵽
		chain.addLast("sslFilter", sslFilter);
		LOG.debug("start SslFiter!");
		
	}

}
