package org.act.xservices.rt.xcontainer.transport.smtp;

import java.util.ArrayList;
import java.util.List;
import java.util.Properties;

import javax.mail.Authenticator;
import javax.mail.Flags;
import javax.mail.Folder;
import javax.mail.Message;
import javax.mail.NoSuchProviderException;
import javax.mail.PasswordAuthentication;
import javax.mail.Session;
import javax.mail.Store;
import javax.mail.URLName;

import org.act.xservices.log.Logger;
import org.act.xservices.log.LoggerFactory;
import org.act.xservices.rt.xcontainer.framework.config.IConfigElement;
import org.act.xservices.rt.xcontainer.framework.dispatch.Dispatcher;
import org.act.xservices.rt.xcontainer.framework.transport.ITransportProtocol;
import org.act.xservices.rt.xcontainer.framework.transport.ITransportReceiver;
import org.act.xservices.rt.xcontainer.inf.context.IATFTransportSession;


/**
 * 
 * @author Lizq 
 * 2008-1-18
 */
public class SMTPTransportDispatchSource implements ITransportReceiver{
	
	final Logger logger = LoggerFactory.getLogger(SMTPTransportDispatchSource.class);
	
	private String sendHost;
	private String sendProtocol;
	private String sendUsername;
	private String sendPassword;
	
	private String receiveHost;
	private String receiveProtocol;
	private String receiveUsername;
	private String receivePassword;
	
	private int receiveIntervalTime;
	
	private Session smtpSession;  //ʾʼỰ
	private Store smtpStore;  //ʾʼעûĴ洢ռ
	
	private Authenticator myAuth;
	
	private boolean isStartup = false;
	
	private ITransportProtocol protocol;
	
	private SMTPTransportHandler transportHandler;
	
	
	//ɷԴӦĴɷб
	private List<Dispatcher> inTransportDisatchers = new ArrayList<Dispatcher>();
	
	//̣߳ÿ̶ʱʼܵʼ
	private Thread receiveThread = new Thread(){
		
		public void run() {
			try {
				String folderName = "inbox";

				while (isStartup) { // ڿʼ״̬
					
					logger.debug("********** Receive Mail Message **********");//
					
					// 
					smtpStore.connect(receiveHost, receiveUsername, receivePassword);
					Folder folder = smtpStore.getFolder(folderName);
					// жļǷ
					if (folder == null || !folder.exists()) {
						throw new Exception(folderName + " Folder is not exists!");
					}
					
					folder.open(Folder.READ_WRITE);
					// ʼ
					receiveMessage(folder);
	
					folder.close(true);//رļ		
					
					smtpStore.close();//ر

					Thread.sleep(receiveIntervalTime);//

				}// end of while
				
				
			} catch (InterruptedException interExc) {
				interExc.printStackTrace();
			} catch (Exception e) {
				e.printStackTrace();
			}

		}// end of run()
		
		
		//մļеĵʼ
		private void receiveMessage(Folder myFolder) throws Exception{

			Message[] messages = myFolder.getMessages();
			
			for(int i=0; i<messages.length; i++){
				if(messages[i].isSet(Flags.Flag.DELETED))
					continue;
				
				SMTPTransportSession smtpTS = new SMTPTransportSession(smtpSession);
				
				//ʼͷ
				smtpTS.setAttribute(SMTPConstants.MAIL_HEADER_FROM, messages[i].getFrom());
				smtpTS.setAttribute(SMTPConstants.MAIL_HEADER_REPLYTO, messages[i].getReplyTo());
				smtpTS.setAttribute(SMTPConstants.MAIL_HEADER_RECIPIENT_TO, 
						messages[i].getRecipients(Message.RecipientType.TO));
				
				smtpTS.setAttribute(SMTPConstants.MAIL_HEADER_RECIPIENT_CC, 
						messages[i].getRecipients(Message.RecipientType.CC));
				smtpTS.setAttribute(SMTPConstants.MAIL_HEADER_RECIPIENT_BCC, 
						messages[i].getRecipients(Message.RecipientType.BCC));
				smtpTS.setAttribute(SMTPConstants.MAIL_HEADER_RECEIVEDDATA, messages[i].getReceivedDate());
				smtpTS.setAttribute(SMTPConstants.MAIL_HEADER_SENDDATA, messages[i].getSentDate());
				smtpTS.setAttribute(SMTPConstants.MAIL_HEADER_SUBJECT, messages[i].getSubject());
				
				//
				smtpTS.setTransportSender(protocol.getTransportSender());
				
				
				//ʼ
				if(messages[i].getContentType().startsWith("text/plain")){
					smtpTS.setContentType(messages[i].getContentType());
					
//					InputStream inStream = messages[i].getInputStream();
//					ByteArrayOutputStream byteOutStream = new ByteArrayOutputStream();
//					int c;
//					while((c = inStream.read()) != -1){
//						byteOutStream.write(c);
//					}
//					smtpTS.setContent(byteOutStream.toByteArray());
					
					
					//smtpTS.setContent( ((String)messages[i].getDataHandler().getContent()).getBytes() );
					smtpTS.setContent( ((String)messages[i].getContent()).getBytes() );
				}
				
				transportHandler.buildTransportContext(smtpTS);
		
				messages[i].setFlag(Flags.Flag.DELETED, true); //Ǵʼ
			}
			
		}
		
	};
	
	
    
	public SMTPTransportHandler getTransportHandler() {
		return transportHandler;
	}
	
	/**
	 * 
	 * @param sendURL
	 * @param receiveURL
	 * @param receiveInterval
	 * @throws SMTPSourceException
	 */
	public void initialize(IConfigElement mailConfig, ITransportProtocol protocol) throws SMTPSourceException{

		this.protocol = protocol;
		
		
		URLName sendURLName = new URLName(mailConfig.getAttribute("sendURL"));
		URLName receiveURLName = new URLName(mailConfig.getAttribute("receiveURL"));
		
		sendHost = sendURLName.getHost();
		sendProtocol = sendURLName.getProtocol();
		sendUsername = sendURLName.getUsername();
		sendPassword = sendURLName.getPassword();
		
		receiveHost = receiveURLName.getHost();
		receiveProtocol = receiveURLName.getProtocol();
		receiveUsername = receiveURLName.getUsername();
		receivePassword = receiveURLName.getPassword();
		
		receiveIntervalTime = Integer.parseInt(mailConfig.getAttribute("receiveInterval"));
		
    	// handler Ϣ;
            
        transportHandler = new SMTPTransportHandler();
			
		//Handlerм
        transportHandler.setContainerEnvironment(this.protocol.getContainerEnvironment());	
        	
        transportHandler.setInPool(protocol.getConnector().getInputTarget());
        	
        transportHandler.setTransportSender(protocol.getTransportSender());

		
		
	    //ڷʼʱ֤
	    myAuth = new Authenticator(){
	        public PasswordAuthentication getPasswordAuthentication() {
	          return new PasswordAuthentication(sendUsername,sendPassword);
	        }
	    };
		
		
		//
	    Properties  props = new Properties();
	    props.put("mail.transport.protocol", sendProtocol);
	    props.put("mail.store.protocol", receiveProtocol);

	    props.put("mail.smtp.host", sendHost); //÷ʼ
	    props.put("mail.smtp.auth", "true"); //ʼҪ֤
	    

	    
	    
	    //Session
	    smtpSession = Session.getDefaultInstance(props,myAuth);
	    
	    //smtpSession.setDebug(true); //־

	    // Store
	    try {
			smtpStore = smtpSession.getStore(receiveProtocol);
		} catch (NoSuchProviderException e) {
			e.printStackTrace();
			throw new SMTPSourceException("NoSuchProviderException");
		}
		
		
	
	}//end of init() 
	
	
	
	
	public void startup() {
		isStartup = true;
		receiveThread.start(); //߳
	}
	
	
	
	public void shutdown() {
		isStartup = false;
	}
	
	
	public void destroy(){
	}

	

	public IATFTransportSession get() throws InterruptedException {	
		throw new InterruptedException("This method is invalid!");
	}
	
	
	
	public String getDescription(){
		return "SMTP TransportDispatchSource, receive mails form "+ receiveUsername+"@"+receiveHost;
	}
	
	
	public List<Dispatcher> getInTransportDisatchers() {
		return inTransportDisatchers;
	}




	public long getAcceptedSocketCount() {
		// TODO Զɷ
		return -1;
	}




	public double getAvgReceiveSpeed() {
		// TODO Զɷ
		return -1;
	}




	public double getAvgSendSpeed() {
		// TODO Զɷ
		return -1;
	}




	public long getRunningTimeInMillis() {
		// TODO Զɷ
		return -1;
	}




	public double getReceiveSpeed() {
		// TODO Զɷ
		return -1;
	}




	public long getReceivedBytes() {
		// TODO Զɷ
		return -1;
	}




	public double getSendSpeed() {
		// TODO Զɷ
		return -1;
	}




	public long getSentBytes() {
		// TODO Զɷ
		return -1;
	}




	public boolean isThroughputStatisticAvailable() {
		// TODO Զɷ
		return false;
	}




	public int getListenPort() {
		// TODO Զɷ
		return 0;
	}




	public ITransportProtocol getTransportProtocol() {
		return protocol;
	}

//	public void setTransportProtocol(ITransportProtocol protocol) {
//		this.protocol = protocol;
//	}

	public long getStartTimeInMillis() {
		// TODO Զɷ
		return -1;
	}

	public double getAvgMessageReceiveSpeed() {
		// TODO Զɷ
		return 0;
	}

	public double getAvgMessageSendSpeed() {
		// TODO Զɷ
		return 0;
	}

	public long getLargestManagedSocketCount() {
		// TODO Զɷ
		return 0;
	}

	public double getLargestMessageReceiveSpeed() {
		// TODO Զɷ
		return 0;
	}

	public double getLargestMessageSendSpeed() {
		// TODO Զɷ
		return 0;
	}

	public double getLargestReceiveSpeed() {
		// TODO Զɷ
		return 0;
	}

	public double getLargestSendSpeed() {
		// TODO Զɷ
		return 0;
	}

	public long getManagedSocketCount() {
		// TODO Զɷ
		return 0;
	}

	public double getMessageReceiveSpeed() {
		// TODO Զɷ
		return 0;
	}

	public long getReceivedMessages() {
		// TODO Զɷ
		return 0;
	}

	public double getSendMessageSpeed() {
		// TODO Զɷ
		return 0;
	}

	public long getSentMessages() {
		// TODO Զɷ
		return 0;
	}

	public int getThroughputCalculationInterval() {
		// TODO Զɷ
		return 0;
	}

	public void setThroughputCalculationInterval(int throughputCalculationInterval) {
		// TODO Զɷ
		
	}

	
}
