
package org.act.xservices.rt.xcontainer.transport;

import java.net.InetAddress;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.act.xservices.rt.xcontainer.framework.AConnector;
import org.act.xservices.rt.xcontainer.framework.IContainerEnvironment;
import org.act.xservices.rt.xcontainer.framework.config.IConfigElement;
import org.act.xservices.rt.xcontainer.framework.dispatch.ITaskFactory;
import org.act.xservices.rt.xcontainer.framework.lifecycle.LifecycleException;
import org.act.xservices.rt.xcontainer.framework.transport.ITransportProtocol;
import org.act.xservices.rt.xcontainer.framework.transport.ITransportReceiver;
import org.act.xservices.rt.xcontainer.inf.context.IATFTransportContext;
import org.act.xservices.rt.xcontainer.inf.context.IATFTransportSession;


/**
 * 
 * 
 * @LastModifyTime 2008-10-10
 * @CreationTime 2008-01-21
 * @author zhangdecai <zhangdc@act.buaa.edu.cn> 
 * @author Lizq [lizq@act.buaa.edu.cn]
 * @since JDK1.5
 */

public class TransportConnector extends AConnector<IATFTransportSession, IATFTransportContext> {
	
	private Map<String,ITransportProtocol> transportProtocols = 
		new HashMap<String,ITransportProtocol>();
    
  
    /**
	 *init protocols,such as http,smtp.
	 * 
	 * @param connectorConfigElement
	 */
	@SuppressWarnings("unchecked")
	protected void doInitLocal() {

		IContainerEnvironment containerEnv = 
			this.getConnectorEnvironment().getContainerEnvironment();
		
		// init protocols,such as http,smtp
		IConfigElement[] protocolsConfigList = this.connectorConfiguration
				.getFirstChildWithName("protocols").getChildren();

		// ÿЭ
		for (int i = 0; i < protocolsConfigList.length; i++) {

			try {
				IConfigElement protocolConfig = protocolsConfigList[i];
				
				//ʼÿһõЭ
				if (protocolConfig.getAttribute("enabled").equalsIgnoreCase("true")) {
					
					ITransportProtocol protocol = (ITransportProtocol) Class
							.forName(protocolConfig.getAttribute("class")).newInstance();
					//ʼ
					protocol.initialize(protocolConfig, this);

					addTransportProtocol(protocol);
					
					
					//---IPַͶ˿ں--- add by lizq, 2008-07-25
					String localIPAddress = InetAddress.getLocalHost().getHostAddress();
					List<String> localIPList = new ArrayList<String>();
					
					for(ITransportReceiver td : protocol.getTransportReceiverList()){
						localIPList.add(localIPAddress+":"+td.getListenPort());
					}

					//ѾͬЭ
					String protocolName = protocol.getName();
					if(containerEnv.getAttribute(protocolName) != null){
						((List<String>)containerEnv.getAttribute(protocolName)).addAll(localIPList);
					}else{
						containerEnv.setAttribute(protocolName, localIPList);
					}
					//----------------------------------------------------

				}

			} catch (InstantiationException e) {
				e.printStackTrace();
			} catch (IllegalAccessException e) {
				e.printStackTrace();
			} catch (ClassNotFoundException e) {
				e.printStackTrace();
			} catch (Exception e) {
				e.printStackTrace();
			}

		}// end of for

	}
	
    
    /**
	 * ʼɷ
	 * дĬʵ
	 * @param connectorConfigElement
	 */
	protected void doInitDispatcher() {
		// ɷɷб	
		this.doInitOutputDispatcher();
    }
    

    /**
     * startЭĹ
     * ԸstartأΪ⡣
     * startЭĹ
     */
    public void start() throws LifecycleException{
       
    	//The first - start all of the protocols
        Iterator<ITransportProtocol> it= transportProtocols.values().iterator();
        while(it.hasNext()){
        	ITransportProtocol protocol=it.next();
            protocol.startup();  //ÿúõЭ
        }       
        super.start(); //ɷ
    }
    
    
    /** 
     * д෽ڹرЭ顣 дǣby lizq 2009-10-02
     * @throws LifecycleException
     */
    public void stop() throws LifecycleException{
        this.stopProtocol();
    }
    
    /**
     * stop all of the protocols
     */
    private void stopProtocol(){
        for(ITransportProtocol protocol : transportProtocols.values()){
        	 protocol.shutdown();
        }
    }

    
    /**
     * شĴЭ飬޸ģ
     * @return
     */
    public Collection<ITransportProtocol> getTransportProtocols(){
        return  Collections.unmodifiableCollection(
        		(Collection<ITransportProtocol>)transportProtocols.values());       
    }
    
    /**
     * ЭƻATransportProtocolڣ򷵻null
     * @param name
     * @return
     */
    public ITransportProtocol getTransportProtocolByIdentifier(String transportIdentifier){
        return  transportProtocols.get(transportIdentifier);       
    }
    
    
    /**
     * ӴЭ
     * @param protocol
     */
    protected void addTransportProtocol(ITransportProtocol protocol){
    	transportProtocols.put(protocol.getTransportIdentifier(), protocol);
    }
    
    
    
    @Override
    public ITaskFactory<IATFTransportSession> getInputTaskFactory() {   	
        //¼ܹʹô˷2008-10-10 lizq
        return null;
    }

    @Override
    public ITaskFactory<IATFTransportContext> getOutputTaskFactory() {
    	TransportOutTaskFactory transportOutDispatchHandlerFactory = 
    		new TransportOutTaskFactory(this);
    	
    	return transportOutDispatchHandlerFactory;
    }


	public ITransportProtocol getTransportProtocol(String uri) {
		throw new RuntimeException("Method not implemented.");
	}

	
	
	/** 
	 * TODO дʵ֣by lizq 2009-10-02
	 */
	public void destroy() {
		// TODO Auto-generated method stub
	}


}
