package org.act.xservices.rt.xcontainer.parser.wsdl;

import java.io.File;
import java.net.URI;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.Vector;

import javax.wsdl.Binding;
import javax.wsdl.BindingFault;
import javax.wsdl.BindingInput;
import javax.wsdl.BindingOperation;
import javax.wsdl.BindingOutput;
import javax.wsdl.Definition;
import javax.wsdl.Fault;
import javax.wsdl.Import;
import javax.wsdl.Input;
import javax.wsdl.Message;
import javax.wsdl.Operation;
import javax.wsdl.OperationType;
import javax.wsdl.Output;
import javax.wsdl.Part;
import javax.wsdl.Port;
import javax.wsdl.PortType;
import javax.wsdl.Service;
import javax.wsdl.Types;
import javax.wsdl.WSDLElement;
import javax.wsdl.WSDLException;
import javax.wsdl.extensions.ExtensibilityElement;
import javax.wsdl.extensions.UnknownExtensibilityElement;
import javax.wsdl.extensions.schema.Schema;
import javax.wsdl.extensions.soap.SOAPAddress;
import javax.wsdl.extensions.soap.SOAPBinding;
import javax.wsdl.extensions.soap.SOAPBody;
import javax.wsdl.extensions.soap.SOAPFault;
import javax.wsdl.extensions.soap.SOAPOperation;
import javax.wsdl.extensions.soap12.SOAP12Address;
import javax.wsdl.extensions.soap12.SOAP12Binding;
import javax.wsdl.extensions.soap12.SOAP12Body;
import javax.wsdl.extensions.soap12.SOAP12Fault;
import javax.wsdl.extensions.soap12.SOAP12Operation;
import javax.wsdl.factory.WSDLFactory;
import javax.wsdl.xml.WSDLReader;

import org.act.xservices.rt.metaset.utils.MetasetUtil;
import org.act.xservices.rt.xcontainer.inf.service.wsdl.ExtenseElement;
import org.act.xservices.rt.xcontainer.inf.service.wsdl.IAttributeExtensible;
import org.act.xservices.rt.xcontainer.inf.service.wsdl.IElementExtensible;
import org.act.xservices.rt.xcontainer.inf.service.wsdl.WSDL11Constants;
import org.act.xservices.rt.xcontainer.inf.service.wsdl.WSDL20Constants;
import org.act.xservices.rt.xcontainer.inf.service.wsdl.WSDLConstants;
import org.act.xservices.rt.xcontainer.inf.service.wsdl.XcBinding;
import org.act.xservices.rt.xcontainer.inf.service.wsdl.XcBindingMessage;
import org.act.xservices.rt.xcontainer.inf.service.wsdl.XcBindingOperation;
import org.act.xservices.rt.xcontainer.inf.service.wsdl.XcEndpoint;
import org.act.xservices.rt.xcontainer.inf.service.wsdl.XcMessage;
import org.act.xservices.rt.xcontainer.inf.service.wsdl.XcMessagePart;
import org.act.xservices.rt.xcontainer.inf.service.wsdl.XcOperation;
import org.act.xservices.rt.xcontainer.inf.service.wsdl.XcPortType;
import org.act.xservices.rt.xcontainer.inf.service.wsdl.XcService;
import org.act.xservices.rt.xcontainer.inf.service.wsdl.schema.ServiceSchema;
import org.act.xservices.rt.xcontainer.wsdl.WSDLParseException;
import org.act.xservices.rt.xcontainer.wsdl.schemaImpl.SchemaAnalyze;
import org.act.xservices.rt.xom.QName;
import org.apache.ws.commons.schema.XmlSchema;
import org.apache.ws.commons.schema.XmlSchemaCollection;
import org.apache.ws.commons.schema.utils.NamespaceMap;
import org.w3c.dom.Element;

/**
 * WSDL1.1ĵXcService
 * 
 * WSDLĵ裺 1. WSDLĵֻһserviceԪ 2. һPortTypeԪֻӦһBindingԪ
 * 
 * @LastModifyTime 2009-4-5
 * @CreationTime
 * @author Lizq [lizq@act.buaa.edu.cn]
 * @since JDK1.5
 */
public class WSDL11ParserUSEWSDL4J extends AWSDL11Parser {

	private static final int COMPONENT_PORT_TYPE = 1;
	private static final int COMPONENT_MESSAGE = 2;
	private static final int COMPONENT_BINDING = 3;

	/**
	 * keep track of whether setup code related to the entire wsdl is complete.
	 * Note that WSDL11ToAllAxisServices will call setup multiple times, so this
	 * field is used to make subsequent calls no-ops.
	 */
	private boolean setupComplete = false;

	protected Definition wsdl4jDefinition = null;

	/** TypesеSchemaԪ */
	private List<Element> schemaMap = null;

	/** WSDLĵimportĵееportType */
	protected Map<QName, XcPortType> xcPortTypeMap = new HashMap<QName, XcPortType>();

	// ע˴Զwsdl4jPortʹͬһbindingXcBindingл
	protected Map<QName, XcBinding> xcBindingMap = new HashMap<QName, XcBinding>();

	public WSDL11ParserUSEWSDL4J(File wsdlFile) {
		super(wsdlFile);
	}

	// -----------------------------
	/**
	 * contains all code which gathers non-service specific information from the
	 * wsdl.
	 * <p/>
	 * After all the setup completes successfully, the setupComplete field is
	 * set so that any subsequent calls to setup() will result in a no-op. Note
	 * that subclass WSDL11ToAllAxisServicesBuilder will call populateService
	 * for each port in the WSDL. Separating the non-service specific
	 * information here allows WSDL11ToAllAxisServicesBuilder to only do this
	 * work 1 time per WSDL, instead of for each port on each service.
	 * 
	 * @throws WSDLException
	 *             if readInTheWSDLFile fails
	 */
	protected void setup() throws WSDLException {
		if (setupComplete) { // already setup, just do nothing and return
			return;
		}
		if (wsdl4jDefinition == null) {
			wsdl4jDefinition = readInTheWSDLFile(this.wsdlFile);
		}
		if (wsdl4jDefinition == null) {
			return; // can't continue without wsdl
		}

		// setup the schemaMap
		this.schemaMap = new ArrayList<Element>();
		populateSchemaMap(wsdl4jDefinition, new HashSet<String>());

		// TODO: lizq 2009-04-05
		// setPolicyRegistryFromService(axisService);

		setupComplete = true; // if any part of setup fails, don't mark
		// setupComplete
	}

	/**
	 * Read the WSDL file given the inputstream for the WSDL source
	 * 
	 * @param in
	 * @throws WSDLException
	 */
	private Definition readInTheWSDLFile(File wsdlFile) throws WSDLException {

		WSDLReader reader = WSDLFactory.newInstance().newWSDLReader();

		// switch off the verbose mode for all usecases
		reader.setFeature("javax.wsdl.verbose", false);
		reader.setFeature("javax.wsdl.importDocuments", true);

		Definition def = reader.readWSDL(wsdlFile.toURI().toString());
		def.setDocumentBaseURI(getBaseUri());
		return def;
	}

	/**
	 * Populate a map of targetNamespace vs DOM schema element This is used to
	 * grab the correct schema element when adding a new element
	 * 
	 * @param definition
	 */

	@SuppressWarnings("unchecked")
	private void populateSchemaMap(Definition definition,
			Set<String> visitedWSDLs) {

		visitedWSDLs.add(definition.getDocumentBaseURI());

		// first process the types in the given wsdl
		Types types = definition.getTypes();
		Object extensibilityElement;
		if (types != null) {
			for (Iterator iterator = types.getExtensibilityElements()
					.iterator(); iterator.hasNext();) {
				extensibilityElement = iterator.next();

				if (extensibilityElement instanceof Schema) {
					Element schemaElement = ((Schema) extensibilityElement)
							.getElement();
					schemaMap.add(schemaElement);
				} else if (extensibilityElement instanceof UnknownExtensibilityElement) {
					Element schema = ((UnknownExtensibilityElement) extensibilityElement)
							.getElement();
					schemaMap.add(schema);
				}
			}
		}

		// popualte the imports as well
		Iterator iter = definition.getImports().values().iterator();
		Vector values = null;
		Import wsdlImport = null;
		for (; iter.hasNext();) {
			values = (Vector) iter.next();
			for (Iterator valuesIter = values.iterator(); valuesIter.hasNext();) {
				wsdlImport = (Import) valuesIter.next();
				Definition innerDefinition = wsdlImport.getDefinition();
				if (!visitedWSDLs
						.contains(innerDefinition.getDocumentBaseURI())) {
					populateSchemaMap(innerDefinition, visitedWSDLs);
				}
			}
		}
	}

	protected XmlSchema getXMLSchema(Element element, String baseUri) {
		XmlSchemaCollection schemaCollection = new XmlSchemaCollection();
		if (baseUri != null) {
			schemaCollection.setBaseUri(baseUri);
		}
		return schemaCollection.read(element);
	}

	/**
	 * WSDLĵXcService
	 * 
	 * @return
	 * @throws DeploymentException
	 */
	@SuppressWarnings("unchecked")
	public XcService parseWSDL() throws WSDLParseException {

		try {
			// ʼWSDLĵwsdl4jDefinition
			// ȡWSDLĵеTypesеSchemaԪأschemaMap
			this.setup();
		} catch (WSDLException e) {
			throw new WSDLParseException(e);
		}

		if (wsdl4jDefinition == null) {
			return null; // can't go any further without the wsdl
		}

		if (wsdl4jDefinition.getServices().size() > 0) {
			Iterator wsdlServIter = wsdl4jDefinition.getServices().values()
					.iterator();

			// һServiceԪ
			Service service = (Service) wsdlServIter.next();

			// now that service are confirm, call up to process the WSDL Service
			// element
			return processService(service);

		} else {
			throw new WSDLParseException(
					"Error: No service was not found in the WSDL at "
							+ wsdl4jDefinition.getDocumentBaseURI()
							+ " with targetnamespace "
							+ wsdl4jDefinition.getTargetNamespace());
		}
	}

	/**
	 * WSDLĵеServiceԪأXcService
	 * 
	 * @return
	 * @throws WSDLParseException
	 */
	@SuppressWarnings("unchecked")
	public XcService processService(Service wsdl4jService)
			throws WSDLParseException {

		if (wsdl4jDefinition == null) {
			return null; // can't go any further without the wsdl
		}

		// ---1.WSDLĵеportTypeԪ,portTypeMap
		processPortTypes(wsdl4jDefinition);
		// ---end of 1---

		// ---2.XcServiceƿռ
		// ҪService QNameXcService
		// ServiceQName{http://ws.act.org/resource/example/counter}GenericDiskDriveService
		this.xcService = new XcService(MetasetUtil.jQNameToxQName(wsdl4jService
				.getQName()));
		this.xcService.setServiceName(wsdl4jService.getQName().getLocalPart());

		this.xcService.setTargetPrefix(wsdl4jDefinition
				.getPrefix(wsdl4jDefinition.getTargetNamespace()));
		this.xcService
				.setTargetNamespace(wsdl4jDefinition.getTargetNamespace());
		this.xcService.setNamespaceMap(new NamespaceMap(wsdl4jDefinition
				.getNamespaces()));

		Map importsMap = wsdl4jDefinition.getImports();
		if (importsMap != null) {
			List<String> imports = new ArrayList(importsMap.keySet());
			this.xcService.setImportedNamespaces(imports);
		}

		// TODO˴дǣby lizq 2009-04-07
		addExtensibleElement(wsdl4jDefinition, xcService);// չԪ
		addExtensibleElement(wsdl4jService, xcService);
		// ---end of 2---

		// ---3.XcServiceеXMLSchema
		// schemaMapڳʼWSDLĵʱ
		for (Element schema : schemaMap) {
			this.xcService.addSchema(getXMLSchema(schema, schema.getBaseURI()));
			String qualified = schema
					.getAttribute(WSDLConstants.ELEMENT_FORM_DEFAULT_OPTION_LONG);
			if (qualified.equals(WSDLConstants.FORM_DEFAULT_QUALIFIED)) {
				xcService.setElementFormDefault(true);
			} else if (qualified.equals(WSDLConstants.FORM_DEFAULT_UNQUALIFIED)) {
				xcService.setElementFormDefault(false);
			}
		}
		// ---end of 3---

		// ---4.ServiceԪеportԪ
		Map<String, Port> wsdl4jPortMap = wsdl4jService.getPorts();

		List<Port> wsdl4jPortsOfSOAP = filterPort(wsdl4jPortMap);

		if (wsdl4jPortsOfSOAP.size() == 0) {
			throw new WSDLParseException(
					"Error: SOAP Port was not found in the Service element: "
							+ wsdl4jService.getQName().toString());
		}

		// ÿPortԪ
		for (Port wsdl4jPort : wsdl4jPortsOfSOAP) {

			XcEndpoint xcEndpoint = processEndpoint(wsdl4jPort);
			xcEndpoint.setXcService(xcService);
			xcService.addXcEndpoint(xcEndpoint);

			final boolean isXcbindingExist = xcBindingMap.get(xcEndpoint
					.getXcBinding().getQName()) != null;
			if (!isXcbindingExist) { // Xcbinding

				xcBindingMap.put(xcEndpoint.getXcBinding().getQName(),
						xcEndpoint.getXcBinding());

				// xcBindingжXcOperationXcService
				for (XcOperation xcop : xcEndpoint.getXcBinding()
						.getXcPortType().getAllXcOperations()) {
					xcop.setXcService(xcService);
					xcService.addXcOperation(xcop);
				}
			}

		}
		// ---end of 4---

		// add by viking 2010 07 28. analyse the schema.
		SchemaAnalyze analyze = new SchemaAnalyze(xcService.getSchema());
		ServiceSchema serviceSchema = analyze.analyzeSchema(xcService);
		xcService.setServiceSchema(serviceSchema);

		return xcService;
	}

	/**
	 * ServiceԪеportԪأXcEndpoint
	 * 
	 * @param wsdl4jPort
	 * @return
	 * @throws WSDLParseException
	 */
	@SuppressWarnings("unchecked")
	private XcEndpoint processEndpoint(Port wsdl4jPort)
			throws WSDLParseException {

		XcEndpoint xcEndpoint = new XcEndpoint(new QName(this.xcService
				.getTargetNamespace(), wsdl4jPort.getName()));

		addExtensibleElement(wsdl4jPort, xcEndpoint);// չԪ

		// ѰSOAPAddress
		for (ExtensibilityElement extTm : (List<ExtensibilityElement>) wsdl4jPort
				.getExtensibilityElements()) {

			if (extTm instanceof SOAPAddress) {
				SOAPAddress soap11AddressT = (SOAPAddress) extTm;
				xcEndpoint.setEndpointURL(soap11AddressT.getLocationURI());

			} else if (extTm instanceof SOAP12Address) {
				SOAP12Address soap12AddressT = (SOAP12Address) extTm;
				xcEndpoint.setEndpointURL(soap12AddressT.getLocationURI());
			}
		}

		// XcBindingXcEndpoint
		XcBinding xcBinding = xcBindingMap.get(MetasetUtil
				.jQNameToxQName(wsdl4jPort.getBinding().getQName()));
		if (xcBinding == null) { // XcBindingڣҪɣʹѴڵXcBinding

			// ҴEndpointӦBindingԪ
			Definition currentBindingWSDL = getParentDefinition(
					wsdl4jDefinition, MetasetUtil.jQNameToxQName(wsdl4jPort
							.getBinding().getQName()), COMPONENT_BINDING,
					new HashSet<String>());
			Binding currentBinding = currentBindingWSDL.getBinding(wsdl4jPort
					.getBinding().getQName());
			xcBinding = processBinding(currentBinding, currentBindingWSDL);
		}

		// xcBinding.setXcEndpoint(xcEndpoint);
		// //˴XcBindingڱXcEndpointãlizq2009-10-04
		xcEndpoint.setXcBinding(xcBinding);

		return xcEndpoint;
	}

	/**
	 * BindingԪأXcBinding
	 * 
	 * @param wsdl4jBinding
	 * @param currentBindingWSDL
	 *            ǰBindingԪڵWSDLĵ
	 * @return
	 * @throws WSDLParseException
	 */
	@SuppressWarnings("unchecked")
	private XcBinding processBinding(Binding wsdl4jBinding,
			Definition currentBindingWSDL) throws WSDLParseException {

		if (wsdl4jBinding.isUndefined())
			throw new WSDLParseException("Error: Error Binding element: "
					+ wsdl4jBinding.getQName());

		QName bindingQName = MetasetUtil.jQNameToxQName(wsdl4jBinding
				.getQName());
		PortType portTypeForBinding = wsdl4jBinding.getPortType();

		if (bindingQName == null || portTypeForBinding.isUndefined())
			throw new WSDLParseException(
					"Error: no name or type attribute in binding element");

		// ҴBindingӦPortTypeԪ
		XcPortType xcPortTypeOfBinding = this.xcPortTypeMap.get(MetasetUtil
				.jQNameToxQName(portTypeForBinding.getQName()));
		if (xcPortTypeOfBinding == null) {
			throw new WSDLParseException("Error: no such portType "
					+ portTypeForBinding.getQName() + " for binding element "
					+ wsdl4jBinding.getQName());
		}
		// һPortTypeĿǰٶֻһBindingӦby lizq 2009-04-07
		if (!(xcPortTypeOfBinding.getXcBinding() == null)) {
			throw new WSDLParseException("Error: multi Binding for PortType: "
					+ portTypeForBinding.getQName());
		}

		//
		XcBinding xcBinding = new XcBinding(MetasetUtil
				.jQNameToxQName(wsdl4jBinding.getQName()));

		// BindingԪصԣnameType
		addExtensibleAttribute(wsdl4jBinding, xcBinding);
		addExtensibleElement(wsdl4jBinding, xcBinding);// չԪ

		// bindingԪضserBindingеİϢ
		List<ExtensibilityElement> extElementList = wsdl4jBinding
				.getExtensibilityElements();

		// ѰSOAPBinding
		ExtensibilityElement soapBingT = null;
		for (ExtensibilityElement extTm : extElementList) {
			if (extTm instanceof SOAPBinding || extTm instanceof SOAP12Binding) {
				soapBingT = extTm;
				break;
			}
		}
		// SOAPBindingϢ
		if (soapBingT != null && soapBingT instanceof SOAPBinding) {
			SOAPBinding soap11BingTT = (SOAPBinding) soapBingT;
			String soapb_Style = soap11BingTT.getStyle();
			if (soapb_Style == null) {
				xcBinding.setSoapb_Style(WSDL11Constants.STYLE_DOC);
			} else if (soapb_Style
					.equalsIgnoreCase(WSDL11Constants.ATTR_VALUE_RPC)) {
				xcBinding.setSoapb_Style(WSDL11Constants.STYLE_RPC);
			} else if (soapb_Style
					.equalsIgnoreCase(WSDL11Constants.ATTR_VALUE_DOCUMENT)) {
				xcBinding.setSoapb_Style(WSDL11Constants.STYLE_DOC);
			}
			xcBinding.setSoapb_Transport(soap11BingTT.getTransportURI());// Ϊnull

		} else if (soapBingT != null && soapBingT instanceof SOAP12Binding) {
			SOAP12Binding soap12BingTT = (SOAP12Binding) soapBingT;
			String soapb_Style = soap12BingTT.getStyle();
			if (soapb_Style == null) {
				xcBinding.setSoapb_Style(WSDL11Constants.STYLE_DOC);
			} else if (soapb_Style
					.equalsIgnoreCase(WSDL11Constants.ATTR_VALUE_RPC)) {
				xcBinding.setSoapb_Style(WSDL11Constants.STYLE_RPC);
			} else if (soapb_Style
					.equalsIgnoreCase(WSDL11Constants.ATTR_VALUE_DOCUMENT)) {
				xcBinding.setSoapb_Style(WSDL11Constants.STYLE_DOC);
			}
			xcBinding.setSoapb_Transport(soap12BingTT.getTransportURI());// Ϊnull
		}

		// BindingOperationԪ
		List<BindingOperation> wsdl4jBindingOperations = wsdl4jBinding
				.getBindingOperations();
		if (wsdl4jBindingOperations.isEmpty()) {
			throw new WSDLParseException(
					"Error: No operation found for the binding: "
							+ wsdl4jBinding.getQName());
		}

		for (BindingOperation wsdl4jBindingOperation : wsdl4jBindingOperations) {

			XcBindingOperation xcBindingOperation = new XcBindingOperation(
					new QName(currentBindingWSDL.getTargetNamespace(),
							wsdl4jBindingOperation.getName()));

			// BindingOperationе
			addExtensibleAttribute(wsdl4jBindingOperation, xcBindingOperation);
			addExtensibleElement(wsdl4jBindingOperation, xcBindingOperation);// չԪ

			// ҴBindingOperationӦXcOperation
			String operationName = wsdl4jBindingOperation.getOperation()
					.getName();
			if (operationName == null) {
				throw new WSDLParseException(
						"Error: no name attribute in operation element in binding element "
								+ wsdl4jBinding.getQName());
			}

			XcOperation xcOperationOfBinding = xcPortTypeOfBinding
					.getXcOperationByName(operationName);

			if (xcOperationOfBinding == null) {
				throw new WSDLParseException("Error: no such operation "
						+ operationName + " in PortType "
						+ xcPortTypeOfBinding.getQName() + " For Binding: "
						+ wsdl4jBinding.getQName());
			}

			// set SOAPOperation
			ExtensibilityElement soapOperationT = null;
			// bindingOperationԪضoperationеİϢ
			List<ExtensibilityElement> extElementBOList = wsdl4jBindingOperation
					.getExtensibilityElements();

			for (ExtensibilityElement extTm : extElementBOList) {
				if (extTm instanceof SOAPOperation
						|| extTm instanceof SOAP12Operation) {
					soapOperationT = extTm;
					break;
				}
			}

			// SOAPOperationϢ
			if (soapOperationT != null
					&& soapOperationT instanceof SOAPOperation) {
				SOAPOperation soap11OperTT = (SOAPOperation) soapOperationT;
				xcBindingOperation.setSoapAction(soap11OperTT
						.getSoapActionURI());

				String operationStyle = soap11OperTT.getStyle();
				if (operationStyle == null) {
					if (xcBinding.getSoapb_Style() == null)
						xcBindingOperation
								.setOperationStyle(WSDL11Constants.STYLE_DOC);
					else
						xcBindingOperation.setOperationStyle(xcBinding
								.getSoapb_Style());
				} else if (operationStyle
						.equalsIgnoreCase(WSDL11Constants.ATTR_VALUE_RPC)) {
					xcBindingOperation
							.setOperationStyle(WSDL11Constants.STYLE_RPC);
				} else if (operationStyle
						.equalsIgnoreCase(WSDL11Constants.ATTR_VALUE_DOCUMENT)) {
					xcBindingOperation
							.setOperationStyle(WSDL11Constants.STYLE_DOC);
				}

			} else if (soapOperationT != null
					&& soapOperationT instanceof SOAP12Operation) {
				SOAP12Operation soap12OperTT = (SOAP12Operation) soapOperationT;
				xcBindingOperation.setSoapAction(soap12OperTT
						.getSoapActionURI());

				String operationStyle = soap12OperTT.getStyle();
				if (operationStyle == null) {
					if (xcBinding.getSoapb_Style() == null)
						xcBindingOperation
								.setOperationStyle(WSDL11Constants.STYLE_DOC);
					else
						xcBindingOperation.setOperationStyle(xcBinding
								.getSoapb_Style());
				} else if (operationStyle
						.equalsIgnoreCase(WSDL11Constants.ATTR_VALUE_RPC)) {
					xcBindingOperation
							.setOperationStyle(WSDL11Constants.STYLE_RPC);
				} else if (operationStyle
						.equalsIgnoreCase(WSDL11Constants.ATTR_VALUE_DOCUMENT)) {
					xcBindingOperation
							.setOperationStyle(WSDL11Constants.STYLE_DOC);
				}
			}

			// XcOperation

			xcOperationOfBinding.setOperationStyle(xcBindingOperation
					.getOperationStyle());
			xcOperationOfBinding.setSoapAction(xcBindingOperation
					.getSoapAction());

			xcBindingOperation.setXcOperation(xcOperationOfBinding);
			xcOperationOfBinding.setXcBindingOperation(xcBindingOperation);

			// ϢbindingϢ
			// Input
			BindingInput bind_in = wsdl4jBindingOperation.getBindingInput();

			processBindingMessage(bind_in, xcBindingOperation,
					xcOperationOfBinding.getInputXcMessage(),
					WSDL11Constants.ELEM_INPUT);

			// output
			BindingOutput bind_out = wsdl4jBindingOperation.getBindingOutput();

			if (bind_out != null) { // add by viking, date 2010/6/30. Reason:
				// InOnly MEP is no bind out
				processBindingMessage(bind_out, xcBindingOperation,
						xcOperationOfBinding.getOutputXcMessage(),
						WSDL11Constants.ELEM_OUTPUT);
			}

			// fault
			Map<String, BindingFault> bind_faultMap = wsdl4jBindingOperation
					.getBindingFaults();
			for (BindingFault bindingFault : bind_faultMap.values()) {

				if (bindingFault.getName() == null) {
					throw new WSDLParseException(
							"Error: Binding name is null for the binding fault in binding operation:"
									+ wsdl4jBindingOperation.getName());
				}

				XcMessage faultMes = xcOperationOfBinding
						.getgetFaultMessageByName(bindingFault.getName());

				if (faultMes == null) {
					throw new WSDLParseException(
							"Error: no such fault message "
									+ bindingFault.getName());
				}
				processBindingMessage(bindingFault, xcBindingOperation,
						faultMes, WSDL11Constants.ELEM_FAULT);
			}

			// ɵxcBindingOperationӵxcBinding
			xcBindingOperation.setXcBinding(xcBinding);
			xcBinding.addXcBindingOperation(xcBindingOperation);

		}// end of each wsdl4jBindingOperation

		xcBinding.setXcPortType(xcPortTypeOfBinding);
		xcPortTypeOfBinding.setXcBinding(xcBinding);

		return xcBinding;
	}

	/**
	 * BindingMessageXcBindingMessage
	 * 
	 * @param wsdlElement
	 * @param xcBindingOperation
	 * @param xcOperationOfBinding
	 * @param mes_Type
	 * @throws WSDLParseException
	 */
	@SuppressWarnings("unchecked")
	private void processBindingMessage(WSDLElement wsdlElement,
			XcBindingOperation xcBindingOperation, XcMessage xcMessage,
			String mes_Type) throws WSDLParseException {

		if (xcMessage == null) {
			throw new WSDLParseException("Error: no XcMessage for "
					+ wsdlElement.toString());
		}

		// inputϢ
		if (mes_Type.equals(WSDL11Constants.ELEM_INPUT)) {

			XcBindingMessage xcBindingInMessage = new XcBindingMessage(
					xcMessage.getQName().getName());

			// ѰSOAPBody
			ExtensibilityElement soapBodyT = null;
			for (ExtensibilityElement extTm : (List<ExtensibilityElement>) wsdlElement
					.getExtensibilityElements()) {
				if (extTm instanceof SOAPBody || extTm instanceof SOAP12Body) {
					soapBodyT = extTm;
					break;
				}
			}

			// SOAPBodyϢ
			if (soapBodyT != null && soapBodyT instanceof SOAPBody) {
				SOAPBody soap11BodyTT = (SOAPBody) soapBodyT;
				xcBindingInMessage.setMesUSE(soap11BodyTT.getUse());

				if (soap11BodyTT.getEncodingStyles() != null) {
					StringBuffer encStyle = new StringBuffer();
					for (String styT : (List<String>) soap11BodyTT
							.getEncodingStyles()) {
						encStyle.append(styT);
						encStyle.append(" ");
					}
					xcBindingInMessage
							.setMesEncordingStyle(encStyle.toString());
				}
				xcBindingInMessage.setMesNamespace(soap11BodyTT
						.getNamespaceURI());

			} else if (soapBodyT != null && soapBodyT instanceof SOAP12Body) {
				SOAP12Body soap12BodyTT = (SOAP12Body) soapBodyT;
				xcBindingInMessage.setMesUSE(soap12BodyTT.getUse());
				xcBindingInMessage.setMesEncordingStyle(soap12BodyTT
						.getEncodingStyle());
				xcBindingInMessage.setMesNamespace(soap12BodyTT
						.getNamespaceURI());
			}

			addExtensibleElement(wsdlElement, xcBindingInMessage);// չԪ

			xcBindingInMessage.setXcMessage(xcMessage);
			xcBindingInMessage.setXcBindingOperation(xcBindingOperation);

			xcBindingOperation.setInputMessage(xcBindingInMessage);
			xcMessage.setXcBindingMessage(xcBindingInMessage);

			// end of inputϢ

		} else if (mes_Type.equals(WSDL11Constants.ELEM_OUTPUT)) {
			// output Ϣ

			XcBindingMessage xcBindingOutMessage = new XcBindingMessage(
					xcMessage.getQName().getName());

			// ѰSOAPBody
			ExtensibilityElement soapBodyT = null;
			for (ExtensibilityElement extTm : (List<ExtensibilityElement>) wsdlElement
					.getExtensibilityElements()) {
				if (extTm instanceof SOAPBody || extTm instanceof SOAP12Body) {
					soapBodyT = extTm;
					break;
				}
			}

			// SOAPBodyϢ
			if (soapBodyT != null && soapBodyT instanceof SOAPBody) {
				SOAPBody soap11BodyTT = (SOAPBody) soapBodyT;
				xcBindingOutMessage.setMesUSE(soap11BodyTT.getUse());

				if (soap11BodyTT.getEncodingStyles() != null) {
					StringBuffer encStyle = new StringBuffer();
					for (String styT : (List<String>) soap11BodyTT
							.getEncodingStyles()) {
						encStyle.append(styT);
						encStyle.append(" ");
					}
					xcBindingOutMessage.setMesEncordingStyle(encStyle
							.toString());
				}
				xcBindingOutMessage.setMesNamespace(soap11BodyTT
						.getNamespaceURI());

			} else if (soapBodyT != null && soapBodyT instanceof SOAP12Body) {
				SOAP12Body soap12BodyTT = (SOAP12Body) soapBodyT;
				xcBindingOutMessage.setMesUSE(soap12BodyTT.getUse());
				xcBindingOutMessage.setMesEncordingStyle(soap12BodyTT
						.getEncodingStyle());
				xcBindingOutMessage.setMesNamespace(soap12BodyTT
						.getNamespaceURI());
			}

			addExtensibleElement(wsdlElement, xcBindingOutMessage);// չԪ

			xcBindingOutMessage.setXcMessage(xcMessage);
			xcBindingOutMessage.setXcBindingOperation(xcBindingOperation);

			xcBindingOperation.setOutputMessage(xcBindingOutMessage);
			xcMessage.setXcBindingMessage(xcBindingOutMessage);

			// end of output Ϣ

		} else if (mes_Type.equals(WSDL11Constants.ELEM_FAULT)) {

			// fault Ϣ
			XcBindingMessage xcBindingFaultMessage = new XcBindingMessage(
					xcMessage.getQName().getName());

			// ѰSOAPfault
			ExtensibilityElement soapFaultT = null;
			for (ExtensibilityElement extTm : (List<ExtensibilityElement>) wsdlElement
					.getExtensibilityElements()) {
				if (extTm instanceof SOAPFault || extTm instanceof SOAP12Fault) {
					soapFaultT = extTm;
					break;
				}
			}

			// SOAPFaultϢ
			if (soapFaultT != null && soapFaultT instanceof SOAPFault) {
				SOAPFault soap11FaultTT = (SOAPFault) soapFaultT;
				xcBindingFaultMessage.setMesUSE(soap11FaultTT.getUse());

				if (soap11FaultTT.getEncodingStyles() != null) {
					StringBuffer encStyle = new StringBuffer();
					for (String styT : (List<String>) soap11FaultTT
							.getEncodingStyles()) {
						encStyle.append(styT);
						encStyle.append(" ");
					}
					xcBindingFaultMessage.setMesEncordingStyle(encStyle
							.toString());
				}
				xcBindingFaultMessage.setMesNamespace(soap11FaultTT
						.getNamespaceURI());

			} else if (soapFaultT != null && soapFaultT instanceof SOAP12Fault) {
				SOAP12Fault soap12FaultTT = (SOAP12Fault) soapFaultT;
				xcBindingFaultMessage.setMesUSE(soap12FaultTT.getUse());
				xcBindingFaultMessage.setMesEncordingStyle(soap12FaultTT
						.getEncodingStyle());
				xcBindingFaultMessage.setMesNamespace(soap12FaultTT
						.getNamespaceURI());
			}

			addExtensibleElement(wsdlElement, xcBindingFaultMessage);// չԪ

			xcBindingFaultMessage.setXcMessage(xcMessage);
			xcBindingFaultMessage.setXcBindingOperation(xcBindingOperation);

			xcBindingOperation.addFaultMessage(xcBindingFaultMessage);
			xcMessage.setXcBindingMessage(xcBindingFaultMessage);

		}// end of fault

	}

	/**
	 * ˳SOAP󶨵PortĿǰ֧SOAP
	 * 
	 * @return
	 */
	@SuppressWarnings("unchecked")
	private List<Port> filterPort(Map<String, Port> wsdl4jPortMap) {

		List<Port> portList = new ArrayList<Port>();

		for (Port port : wsdl4jPortMap.values()) {

			List extensibilityElements = port.getExtensibilityElements();

			for (int i = 0; i < extensibilityElements.size(); i++) {
				Object extElement = extensibilityElements.get(i);

				if (extElement instanceof SOAP12Address) {
					// SOAP 1.2 addressfound
					String location = ((SOAP12Address) extElement)
							.getLocationURI().trim();
					if ((location != null) && location.startsWith("http:")) {
						// i.e we have found an http port so return from here
						portList.add(port);
						break; // 鿴һPort
					}
				}

				if (extElement instanceof SOAPAddress) {
					// SOAP 1.1 addressfound
					String location = ((SOAPAddress) extElement)
							.getLocationURI().trim();
					if ((location != null) && location.startsWith("http:")) {
						// i.e we have found an http port so return from here
						portList.add(port);
						break; // 鿴һPort
					}
				}
			}//
		}//

		return portList;
	}

	/**
	 *  WSDLĵԼimportĵ еportTypeԪ, portTypeMap:
	 * PortTypeQName-->PortType
	 * 
	 * @param wsdlDefinition
	 * @throws WSDLParseException
	 */
	@SuppressWarnings("unchecked")
	private void processPortTypes(Definition wsdlDefinition)
			throws WSDLParseException {

		// ע˴QNameΪPortTypeQName
		// 磺{http://docs.oasis-open.org/wsrf/rpw-2}GetMultipleResourceProperties
		Map<QName, PortType> portTypeMap = wsdlDefinition.getAllPortTypes();

		if (portTypeMap.isEmpty())
			throw new WSDLParseException(
					"Error: no portType element in all WSDL file ");

		for (PortType portTypeTemp : portTypeMap.values()) { // ÿportType

			if (portTypeTemp.isUndefined()) {
				continue;
			}

			// ҴPortTypeڵWSDLĵ
			Definition portTypeWSDL = getParentDefinition(wsdlDefinition,
					MetasetUtil.jQNameToxQName(portTypeTemp.getQName()),
					COMPONENT_PORT_TYPE, new HashSet<String>());

			// PortType,XcPortType,XcPortTypeMap
			processPortType(portTypeTemp, portTypeWSDL);
		}

	}

	/**
	 * һPortTypeԪأXcPortTypeXcPortTypeMap
	 * 
	 * @param wsdl4jPortType
	 * @param portTypeWSDL
	 *            PortTypeԪڵWSDLĵ
	 * @throws WSDLParseException
	 */
	@SuppressWarnings("unchecked")
	private void processPortType(PortType wsdl4jPortType,
			Definition portTypeWSDL) throws WSDLParseException {

		XcPortType xcPortType = new XcPortType(MetasetUtil
				.jQNameToxQName(wsdl4jPortType.getQName()));

		addExtensibleAttribute(wsdl4jPortType, xcPortType); // portTypeеԣname
		addExtensibleElement(wsdl4jPortType, xcPortType); // չԪ

		// Operation
		List<Operation> wsdl4jOperationList = wsdl4jPortType.getOperations();
		if (wsdl4jOperationList.isEmpty()) //
			throw new WSDLParseException(
					"Error: no operation element in portType element "
							+ wsdl4jPortType.getQName());

		XcOperation xcOperation;
		for (Operation wsdl4jOperation : wsdl4jOperationList) { // ÿoperation

			if (wsdl4jOperation.isUndefined()) {
				continue;
			}

			xcOperation = processOperation(wsdl4jOperation, wsdl4jPortType,
					portTypeWSDL);

			xcOperation.setXcPortType(xcPortType);
			xcPortType.addXcOperation(xcOperation);

		}// end of ÿoperation

		xcPortTypeMap.put(xcPortType.getQName(), xcPortType);
	}

	/**
	 * OperationԪ
	 * 
	 * @param wsdl4jOperation
	 * @param wsdl4jPortType
	 * @param portTypeDifinition
	 * @return
	 * @throws WSDLParseException
	 */
	@SuppressWarnings("unchecked")
	private XcOperation processOperation(Operation wsdl4jOperation,
			PortType wsdl4jPortType, Definition portTypeDif)
			throws WSDLParseException {

		XcOperation xcOperation = new XcOperation(new QName(portTypeDif
				.getTargetNamespace(), wsdl4jOperation.getName()));

		// operationеԣname
		addExtensibleAttribute(wsdl4jOperation, xcOperation);
		addExtensibleElement(wsdl4jOperation, xcOperation);// չԪ

		// ôOperationϢģʽ
		String mepuri = getMEPURI(wsdl4jOperation);
		xcOperation.setMepURI(mepuri);

		// ÿܻеĲ˳
		List<String> parameterList = wsdl4jOperation.getParameterOrdering();
		if (parameterList != null) {
			StringBuffer paraS = new StringBuffer();
			for (String parT : parameterList) {
				paraS.append(parT);
				paraS.append("");
			}
			xcOperation.setParameterOrder(paraS.toString());
		}

		// 򷽷Ϣ
		addMessageToOperation(wsdl4jOperation, xcOperation);

		return xcOperation;
	}

	// creatServiceInterfaceMap ʹ
	@SuppressWarnings("unchecked")
	private void addMessageToOperation(Operation wsdl4jOperation,
			XcOperation xcOperation) throws WSDLParseException {

		// Ϣ
		Input wsdl4jInput = wsdl4jOperation.getInput();
		if (wsdl4jInput != null) {
			XcMessage inMes = creatXcMessage(wsdl4jInput.getMessage());

			// WSA_ActionϢ
			Object wsaActionObj = wsdl4jInput
					.getExtensionAttribute(new javax.xml.namespace.QName(
							WSDL11Constants.prefix_WSAW_NAMESPACEURI, "Action"));
			if (wsaActionObj != null) {
				inMes.setWSA_Action(((javax.xml.namespace.QName) wsaActionObj)
						.getLocalPart());
			}

			// Inputеչ
			addExtensibleAttribute(wsdl4jInput, inMes);
			addExtensibleElement(wsdl4jInput, inMes);// չԪ

			inMes.setXcOperation(xcOperation);
			xcOperation.setInputXcMessage(inMes);
		}

		// Ϣ
		Output wsdl4jOutput = wsdl4jOperation.getOutput();
		if (wsdl4jOutput != null) {
			XcMessage outMes = creatXcMessage(wsdl4jOutput.getMessage());

			// WSA_ActionϢ
			Object wsaActionObj = wsdl4jOutput
					.getExtensionAttribute(new javax.xml.namespace.QName(
							WSDL11Constants.prefix_WSAW_NAMESPACEURI, "Action"));
			if (wsaActionObj != null) {
				outMes.setWSA_Action(((javax.xml.namespace.QName) wsaActionObj)
						.getLocalPart());
			}

			// Outputеչ
			addExtensibleAttribute(wsdl4jOutput, outMes);
			addExtensibleElement(wsdl4jOutput, outMes);

			outMes.setXcOperation(xcOperation);
			xcOperation.setOutputXcMessage(outMes);
		}

		// ӴϢ
		Map<String, Fault> faultMap = wsdl4jOperation.getFaults();
		if (!faultMap.isEmpty()) {

			for (Fault wsdl4jFault : faultMap.values()) {

				XcMessage faultMes = creatXcMessage(wsdl4jFault.getMessage());

				// WSA_ActionϢ
				Object wsaActionObj = wsdl4jFault
						.getExtensionAttribute(new javax.xml.namespace.QName(
								WSDL11Constants.prefix_WSAW_NAMESPACEURI,
								"Action"));
				if (wsaActionObj != null) {
					faultMes
							.setWSA_Action(((javax.xml.namespace.QName) wsaActionObj)
									.getLocalPart());
				}

				// Faultеչ
				addExtensibleAttribute(wsdl4jFault, faultMes);
				addExtensibleElement(wsdl4jFault, faultMes);

				faultMes.setXcOperation(xcOperation);
				xcOperation.addFaultXcMessage(faultMes);
			}
		}
	}

	/**
	 * WSDL4Jģ͹MessageImpl
	 * 
	 * @param wsdl4jMessage
	 * @return
	 */
	@SuppressWarnings("unchecked")
	private XcMessage creatXcMessage(Message wsdl4jMessage)
			throws WSDLParseException {

		XcMessage xcMessage = new XcMessage(MetasetUtil
				.jQNameToxQName(wsdl4jMessage.getQName()));// Messageԭ

		Map<String, Part> msgParts = wsdl4jMessage.getParts();

		if (!msgParts.isEmpty()) { // ע⣺MessageûpartԪ

			for (Part msgPartTemp : msgParts.values()) { // ÿMessagePart

				String partName = msgPartTemp.getName();

				QName partType = null;
				if (msgPartTemp.getTypeName() != null) {
					partType = MetasetUtil.jQNameToxQName(msgPartTemp
							.getTypeName()); // ΪTypeʱ
				} else if (msgPartTemp.getElementName() != null) {
					partType = MetasetUtil.jQNameToxQName(msgPartTemp
							.getElementName()); // ΪElementsʱ
				}

				if (partName == null || partType == null)
					throw new WSDLParseException(
							"Error: error part element in message element "
									+ wsdl4jMessage.getQName());

				XcMessagePart mesPart = new XcMessagePart(new QName(
						wsdl4jMessage.getQName().getNamespaceURI(), partName));

				mesPart.setParameterType(partType);

				mesPart.setXcMessage(xcMessage);

				xcMessage.addXcMessagePart(mesPart);

			}// end of ÿMessagePart
		}//

		// Message,name
		addExtensibleAttribute(wsdl4jMessage, xcMessage);
		addExtensibleElement(wsdl4jMessage, xcMessage);// չԪ

		return xcMessage;
	}

	/**
	 * Guess the MEP based on the order of messages
	 * 
	 * @param operation
	 * @throws AxisFault
	 */
	private String getMEPURI(Operation operation) throws WSDLParseException {
		OperationType operationType = operation.getStyle();

		if (operationType != null) {
			if (operationType.equals(OperationType.REQUEST_RESPONSE)) {
				return WSDL20Constants.MEP_URI_IN_OUT;
			}

			if (operationType.equals(OperationType.ONE_WAY)) {
				if (operation.getFaults().size() > 0) {
					return WSDL20Constants.MEP_URI_ROBUST_IN_ONLY;
				}
				return WSDL20Constants.MEP_URI_IN_ONLY;
			}

			if (operationType.equals(OperationType.NOTIFICATION)) {
				return WSDL20Constants.MEP_URI_OUT_ONLY;
			}

			if (operationType.equals(OperationType.SOLICIT_RESPONSE)) {
				return WSDL20Constants.MEP_URI_OUT_IN;
			}
			throw new WSDLParseException("Cannot Determine the MEP");
		}

		throw new WSDLParseException("Cannot Determine the MEP");
	}

	/**
	 * wsdlElementеչӵobjectEle
	 * 
	 * @param objectEle
	 * @param attrTemp
	 * @param exceptAttriutes
	 */
	@SuppressWarnings("unchecked")
	private void addExtensibleAttribute(WSDLElement wsdlElement,
			IAttributeExtensible objectEle) {

		// lizq2009-04-05
		// <wsdl:portType name="toLowerCase" tns:aaa="ccccccc">
		// wsdlElement.getExtensionAttributes() --> Map<QName, QName>
		// {http://act.org}aaa --> {http://schemas.xmlsoap.org/wsdl/}ccccccc
		// tns --> {http://act.org}
		// wsdl --> {http://schemas.xmlsoap.org/wsdl/}

		Map extAttributeMap = wsdlElement.getExtensionAttributes();
		for (Object temp : extAttributeMap.keySet()) {
			objectEle.setExtensionAttribute(MetasetUtil
					.jQNameToxQName((javax.xml.namespace.QName) temp),
					((javax.xml.namespace.QName) extAttributeMap.get(temp))
							.getLocalPart());
		}
	}

	/**
	 * WSDLеչԪ
	 * 
	 * @param wsdlElement
	 * @param objectEle
	 * @return
	 */
	@SuppressWarnings("unchecked")
	private int addExtensibleElement(WSDLElement wsdlElement,
			IElementExtensible objectEle) {
		int num = 0;

		for (ExtensibilityElement extEm : (List<ExtensibilityElement>) wsdlElement
				.getExtensibilityElements()) {

			if (extEm instanceof UnknownExtensibilityElement) {// չԪ

				UnknownExtensibilityElement unExtEle = (UnknownExtensibilityElement) extEm;

				ExtenseElement extEle = new ExtenseElement();
				extEle.setElementType(MetasetUtil.jQNameToxQName(unExtEle
						.getElementType()));

				if (unExtEle.getRequired() != null) {
					extEle.setRequired(unExtEle.getRequired());
				}

				extEle.setExtElement(unExtEle.getElement());

				if (objectEle.addExtensibilityElement(extEle))
					num++;
			}
		}// end of for

		return num;
	}

	// -------

	/**
	 * returns the wsld defintion for the given component.
	 * 
	 * @param definition
	 * @param qname
	 *            the QName of the given component
	 * @param componentType
	 * @param visitedWSDLs
	 * @return definition containing the component.
	 */
	@SuppressWarnings("unchecked")
	private Definition getParentDefinition(Definition definition, QName qname,
			int componentType, Set<String> visitedWSDLs) {

		visitedWSDLs.add(definition.getDocumentBaseURI());

		Definition newParentDefinition = null;

		// first find through imports
		Iterator iter = definition.getImports().values().iterator();

		List values = null;
		Import wsdlImport = null;

		while (iter.hasNext()) {
			values = (List) iter.next();

			for (Iterator valuesIter = values.iterator(); valuesIter.hasNext();) {

				wsdlImport = (Import) valuesIter.next();

				Definition innerDefinition = wsdlImport.getDefinition();

				if (!visitedWSDLs
						.contains(innerDefinition.getDocumentBaseURI())) {
					newParentDefinition = getParentDefinition(innerDefinition,
							qname, componentType, visitedWSDLs);

					if (newParentDefinition != null) {
						break;
					}
				}
			}//

			if (newParentDefinition != null) {
				break;
			}
		}//

		// if it not available in imports we check for the current definition.
		if (newParentDefinition == null) {
			// this can be in a imported wsdl
			if (isComponetAvailable(definition, qname, componentType)) {
				newParentDefinition = definition;
			}
		}

		return newParentDefinition;
	}

	private boolean isComponetAvailable(Definition definition, QName qname,
			int componentType) {
		boolean isAvailable = false;
		switch (componentType) {
		case COMPONENT_BINDING: {
			isAvailable = (definition.getBinding(MetasetUtil
					.xQNameTojQName(qname)) != null)
					&& (definition
							.getBinding(MetasetUtil.xQNameTojQName(qname))
							.getPortType() != null);
			break;
		}
		case COMPONENT_PORT_TYPE: {
			isAvailable = (definition.getPortType(MetasetUtil
					.xQNameTojQName(qname)) != null);
			break;
		}
		case COMPONENT_MESSAGE: {
			isAvailable = (definition.getMessage(MetasetUtil
					.xQNameTojQName(qname)) != null);
			break;
		}
		}
		return isAvailable;
	}

	public static void main(String[] args) throws Exception {

		// ----------
		// URL url = new URL("http://localhost:8091/a/b/c/Prospect?pl=YFZDZS");
		// System.out.println("url.getPath() : "+ url.getPath()); // =
		// url.getPath() : /a/b/c/Prospect
		// --------

		// ----------
		URI uri1 = new URI("http://localhost:8091/a/b/c/Prospect?pl=client");
		System.out.println(" 1 : " + uri1.getFragment());
		System.out.println(" 2 : " + uri1.getHost());
		System.out.println(" 3 : " + uri1.getPath());
		System.out.println(" 4 : " + uri1.getQuery());
		System.out.println(" 5 : " + uri1.getPort());
		System.out.println(" 6 : " + uri1.getRawPath());
		System.out.println(" 7 : " + uri1.getUserInfo());
		System.out.println(" 8 : " + uri1.getFragment());
		System.out.println(" 9 : " + uri1.getFragment());

		// --------

		// WSDL11ParserUSEWSDL4J wsdlParser = new WSDL11ParserUSEWSDL4J(
		// new File("WSR_Example/Prospect.bar/Prospect.wsdl"));
		//    	
		// XcService xcService = wsdlParser.parseWSDL();
		//    	
		// System.out.println("---"+xcService.getQName());
		//    	
		// File ff = new File("WSR_ExampleWSDL/GenericDiskDrive.wsdl");
		//    	
		// System.out.println("----ff.getAbsolutePath()----"+ff.getAbsolutePath());
		// System.out.println("----ff.getCanonicalPath()----"+ff.getCanonicalPath());
		// System.out.println("----ff.getPath()----"+ff.getPath());
		// System.out.println("----ff.toURI().toString()----"+ff.toURI().toString());
		// System.out.println("----ff.toURL().toString()----"+ff.toURL().toString());

	}

}
