package org.act.xservices.rt.xcontainer.wsdl.schemaImpl;

import java.io.File;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import java.util.Vector;

import javax.wsdl.Definition;
import javax.wsdl.Import;
import javax.wsdl.Service;
import javax.wsdl.Types;
import javax.wsdl.WSDLException;
import javax.wsdl.extensions.UnknownExtensibilityElement;
import javax.wsdl.extensions.schema.Schema;
import javax.wsdl.factory.WSDLFactory;
import javax.wsdl.xml.WSDLReader;

import org.act.xservices.rt.metaset.utils.MetasetUtil;
import org.act.xservices.rt.xcontainer.inf.service.wsdl.WSDLConstants;
import org.act.xservices.rt.xcontainer.inf.service.wsdl.XcService;
import org.act.xservices.rt.xcontainer.inf.service.wsdl.schema.ElementType;
import org.act.xservices.rt.xcontainer.inf.service.wsdl.schema.MessageSchema;
import org.act.xservices.rt.xcontainer.inf.service.wsdl.schema.ServiceSchema;
import org.act.xservices.rt.xcontainer.parser.wsdl.AWSDL11Parser;
import org.act.xservices.rt.xcontainer.wsdl.WSDLParseException;

import org.apache.ws.commons.schema.XmlSchema;
import org.apache.ws.commons.schema.XmlSchemaCollection;

import org.w3c.dom.Element;

import com.ibm.wsdl.MessageImpl;
import com.ibm.wsdl.PartImpl;

/**
 * 
 * @author viking
 * @date 2010-8-4
 * @since JDK 1.5.0
 * @version 1.0.0
 * 
 */
public class MessagePartParser extends AWSDL11Parser {

	/** TypesеSchemaԪ */
	private List<Element> schemaList = null;
	protected Definition wsdl4jDefinition = null;

	public MessagePartParser(String wsdlfile) {
		this(new File(wsdlfile));
	}

	public MessagePartParser(File file) {
		super(file);
	}

	public Definition readWSDL(File file) throws WSDLException {

		WSDLReader reader = WSDLFactory.newInstance().newWSDLReader();

		// switch off the verbose mode for all usecases
		reader.setFeature("javax.wsdl.verbose", false);
		reader.setFeature("javax.wsdl.importDocuments", true);

		Definition def = reader.readWSDL(file.toURI().toString());
		def.setDocumentBaseURI(getBaseUri());
		return def;
	}

	/**
	 * Populate a map of targetNamespace vs DOM schema element This is used to
	 * grab the correct schema element when adding a new element
	 * 
	 * @param definition
	 */

	@SuppressWarnings("unchecked")
	public XcService parseWSDL() throws WSDLParseException {
		try {
			wsdl4jDefinition = readWSDL(wsdlFile);
		} catch (WSDLException e) {
			throw new WSDLParseException(e);
		}

		// setup the schemaMap
		this.schemaList = new ArrayList<Element>();
		populateSchemaMap(wsdl4jDefinition, new HashSet<String>());
		if (wsdl4jDefinition == null) {
			return null; // can't go any further without the wsdl
		}

		if (wsdl4jDefinition.getServices().size() > 0) {
			Iterator wsdlServIter = wsdl4jDefinition.getServices().values()
					.iterator();

			// һServiceԪ
			Service service = (Service) wsdlServIter.next();

			// now that service are confirm, call up to process the WSDL Service
			// element
			return processService(service);

		} else {
			throw new WSDLParseException(
					"Error: No service was not found in the WSDL at "
							+ wsdl4jDefinition.getDocumentBaseURI()
							+ " with targetnamespace "
							+ wsdl4jDefinition.getTargetNamespace());
		}
	}

	public XcService processService(Service wsdl4jService)
			throws WSDLParseException {
		// ---2.XcServiceƿռ
		// ҪService QNameXcService
		// ServiceQName{http://ws.act.org/resource/example/counter}GenericDiskDriveService
		this.xcService = new XcService(MetasetUtil.jQNameToxQName(wsdl4jService
				.getQName()));
		this.xcService.setServiceName(wsdl4jService.getQName().getLocalPart());

		// ---3.XcServiceеXMLSchema
		// schemaMapڳʼWSDLĵʱ
		for (Element schema : schemaList) {
			this.xcService.addSchema(getXMLSchema(schema, schema.getBaseURI()));
			String qualified = schema
					.getAttribute(WSDLConstants.ELEMENT_FORM_DEFAULT_OPTION_LONG);
			if (qualified.equals(WSDLConstants.FORM_DEFAULT_QUALIFIED)) {
				xcService.setElementFormDefault(true);
			} else if (qualified.equals(WSDLConstants.FORM_DEFAULT_UNQUALIFIED)) {
				xcService.setElementFormDefault(false);
			}
		}
		wsdl4jDefinition.getMessages();
		SchemaAnalyze analyze = new SchemaAnalyze(xcService.getSchema());
		ServiceSchema serviceSchema = SchemaHelper.newServiceSchema();

		for (Object msgObj : wsdl4jDefinition.getMessages().values()) {
			MessageImpl message = (MessageImpl) msgObj;
			String name = message.getQName().getLocalPart();
			MessageSchema messageSchema = SchemaHelper.newMessageSchema(name);
			for (Object partObj : message.getOrderedParts(null)) {
				PartImpl part = (PartImpl) partObj;
				String partname = part.getName();
				javax.xml.namespace.QName type = part.getTypeName();
				if (type == null) {
					type = part.getElementName();
				}
				ElementType element = analyze.analyzeType(partname, type);
				messageSchema.addElement(element);
			}
			serviceSchema.addMessageSchema(messageSchema);
		}

		xcService.setServiceSchema(serviceSchema);

		return xcService;
	}

	protected XmlSchema getXMLSchema(Element element, String baseUri) {
		XmlSchemaCollection schemaCollection = new XmlSchemaCollection();
		if (baseUri != null) {
			schemaCollection.setBaseUri(baseUri);
		}
		return schemaCollection.read(element);
	}

	@SuppressWarnings("unchecked")
	private void populateSchemaMap(Definition definition,
			Set<String> visitedWSDLs) {

		visitedWSDLs.add(definition.getDocumentBaseURI());

		// first process the types in the given wsdl
		Types types = definition.getTypes();
		Object extensibilityElement;
		if (types != null) {
			for (Iterator iterator = types.getExtensibilityElements()
					.iterator(); iterator.hasNext();) {
				extensibilityElement = iterator.next();

				if (extensibilityElement instanceof Schema) {
					Element schemaElement = ((Schema) extensibilityElement)
							.getElement();
					schemaList.add(schemaElement);
				} else if (extensibilityElement instanceof UnknownExtensibilityElement) {
					Element schema = ((UnknownExtensibilityElement) extensibilityElement)
							.getElement();
					schemaList.add(schema);
				}
			}
		}

		// popualte the imports as well
		Iterator iter = definition.getImports().values().iterator();
		Vector values = null;
		Import wsdlImport = null;
		for (; iter.hasNext();) {
			values = (Vector) iter.next();
			for (Iterator valuesIter = values.iterator(); valuesIter.hasNext();) {
				wsdlImport = (Import) valuesIter.next();
				Definition innerDefinition = wsdlImport.getDefinition();
				if (!visitedWSDLs
						.contains(innerDefinition.getDocumentBaseURI())) {
					populateSchemaMap(innerDefinition, visitedWSDLs);
				}
			}
		}
	}

}
