package org.act.xservices.rt.xcontainer.wsdl.schemaImpl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.xml.namespace.QName;

import org.act.xservices.rt.metaset.utils.MetasetUtil;
import org.act.xservices.rt.xcontainer.inf.service.wsdl.WSDLConstants;
import org.act.xservices.rt.xcontainer.inf.service.wsdl.XcMessage;
import org.act.xservices.rt.xcontainer.inf.service.wsdl.XcMessagePart;
import org.act.xservices.rt.xcontainer.inf.service.wsdl.XcOperation;
import org.act.xservices.rt.xcontainer.inf.service.wsdl.XcService;
import org.act.xservices.rt.xcontainer.inf.service.wsdl.schema.ComplexType;
import org.act.xservices.rt.xcontainer.inf.service.wsdl.schema.ElementType;
import org.act.xservices.rt.xcontainer.inf.service.wsdl.schema.MessageSchema;
import org.act.xservices.rt.xcontainer.inf.service.wsdl.schema.OperationSchema;
import org.act.xservices.rt.xcontainer.inf.service.wsdl.schema.SchemaType;
import org.act.xservices.rt.xcontainer.inf.service.wsdl.schema.ServiceSchema;
import org.act.xservices.rt.xcontainer.wsdl.WSDLParseException;
import org.act.xservices.rt.xcontainer.wsdl.utils.SchemaConstants;
import org.act.xservices.rt.xcontainer.wsdl.utils.SchemaUtil;
import org.apache.ws.commons.schema.XmlSchema;
import org.apache.ws.commons.schema.XmlSchemaAll;
import org.apache.ws.commons.schema.XmlSchemaAny;
import org.apache.ws.commons.schema.XmlSchemaChoice;
import org.apache.ws.commons.schema.XmlSchemaComplexContent;
import org.apache.ws.commons.schema.XmlSchemaComplexContentExtension;
import org.apache.ws.commons.schema.XmlSchemaComplexContentRestriction;
import org.apache.ws.commons.schema.XmlSchemaComplexType;
import org.apache.ws.commons.schema.XmlSchemaContent;
import org.apache.ws.commons.schema.XmlSchemaContentModel;
import org.apache.ws.commons.schema.XmlSchemaElement;
import org.apache.ws.commons.schema.XmlSchemaGroup;
import org.apache.ws.commons.schema.XmlSchemaGroupBase;
import org.apache.ws.commons.schema.XmlSchemaGroupRef;
import org.apache.ws.commons.schema.XmlSchemaObject;
import org.apache.ws.commons.schema.XmlSchemaObjectCollection;
import org.apache.ws.commons.schema.XmlSchemaParticle;
import org.apache.ws.commons.schema.XmlSchemaSequence;
import org.apache.ws.commons.schema.XmlSchemaSimpleContent;
import org.apache.ws.commons.schema.XmlSchemaSimpleContentExtension;
import org.apache.ws.commons.schema.XmlSchemaSimpleContentRestriction;
import org.apache.ws.commons.schema.XmlSchemaSimpleType;
import org.apache.ws.commons.schema.XmlSchemaSimpleTypeContent;
import org.apache.ws.commons.schema.XmlSchemaSimpleTypeList;
import org.apache.ws.commons.schema.XmlSchemaSimpleTypeRestriction;
import org.apache.ws.commons.schema.XmlSchemaSimpleTypeUnion;
import org.apache.ws.commons.schema.XmlSchemaType;

/**
 * 
 * @author viking
 * @date 2010-7-29
 * @since JDK 1.5.0
 * @version 1.0.0
 * 
 *          This class is used to analyze the schema of wsdl.
 * 
 */
public class SchemaAnalyze {

	// to keep the XMLScheam getting either from WSDL
	private List<XmlSchema> schemaList = null;

	// record the schema that has found. this is used to accelerate to search.
	private Map<QName, XmlSchemaObject> schemaSearchMap = new HashMap<QName, XmlSchemaObject>();

	private Map<QName, ComplexType> complexTypeMap = new HashMap<QName, ComplexType>();

	public SchemaAnalyze() {

	}

	public SchemaAnalyze(XmlSchema schema) {
		List<XmlSchema> list = new ArrayList<XmlSchema>();
		list.add(schema);
		setSchemaList(list);
	}

	public SchemaAnalyze(List<XmlSchema> schemaList) {
		setSchemaList(schemaList);
	}

	public void setSchemaList(List<XmlSchema> schemaList) {
		this.schemaList = schemaList;
	}

	/**
	 * Analyze every operation of the service. One operation contains an input
	 * message or an output message or both.
	 * 
	 * @param service
	 * @return
	 * @throws WSDLParseException
	 */
	public ServiceSchema analyzeSchema(XcService service)
			throws WSDLParseException {
		if (service == null) {
			throw new WSDLParseException("The service is null.");
		}
		this.schemaList = service.getSchema();
		// new ServiceSchema
		ServiceSchema serviceSchema = SchemaHelper.newServiceSchema();

		// Get all operations
		for (XcOperation operation : service.getAllXcOperation()) {
			OperationSchema operationSchema = analyzeOperation(operation);
			// add operation schema
			serviceSchema.addOperation(operationSchema);
			MessageSchema input = operationSchema.getInputMessage();
			if (input != null) {
				serviceSchema.addMessageSchema(input);
			}
			MessageSchema output = operationSchema.getOutputMessage();
			if (output != null) {
				serviceSchema.addMessageSchema(output);
			}
		}
		return serviceSchema;
	}

	public OperationSchema analyzeOperation(XcOperation operation) {
		// Operation name
		String opName = operation.getQName().getName();
		OperationSchema operationSchema = SchemaHelper
				.newOperationSchema(opName);
		// deal with input message
		XcMessage inputMsg = operation.getInputXcMessage();
		if (inputMsg != null) {
			MessageSchema input = analyzeMessage(inputMsg);
			// add input message schema
			operationSchema.setInputMessage(input);

		}
		// deal with output message
		XcMessage outputMsg = operation.getOutputXcMessage();
		if (outputMsg != null) {
			MessageSchema output = analyzeMessage(outputMsg);
			// add out message schema
			operationSchema.setOutputMessage(output);
		}
		return operationSchema;
	}

	/**
	 * Analyze every part of message.A message contains several parts. Analyze
	 * message part.Every part has a type or element attribute which indicates
	 * the schema of the part.No matter which attribute the part contains, the
	 * part preserve as an ElementType.
	 * 
	 * The part like this <part name="part1" element="element1" /> will be
	 * replaced by ElementType "element1".
	 * 
	 * The part like this <part name="part1" type="type1" /> will be replaced by
	 * <element name="part1" type="type1"/>.
	 * 
	 * So every part can be represent as ElementType.
	 * 
	 * @param msg
	 * @return
	 */
	public MessageSchema analyzeMessage(XcMessage msg) {
		String msgName = msg.getQName().getName();
		MessageSchema msgSchema = SchemaHelper.newMessageSchema(msgName);
		for (XcMessagePart part : msg.getXcMessagePartMap().values()) {
			// Analyze Message part
			// get part name
			String name = part.getName().getName();
			// Get type
			QName type = MetasetUtil.xQNameTojQName(part.getParameterType());
			ElementType element = analyzeType(name, type);
			msgSchema.addElement(element);
		}
		return msgSchema;
	}

	/**
	 * Analyze type.
	 * 
	 * @param name
	 *            variable name
	 * @param type
	 *            variable type
	 * @return
	 * 
	 */
	public ElementType analyzeType(String name, QName type) {
		// Create ElementSchema
		ElementType element = SchemaHelper.newElementSchema(name);

		// is simple type
		if (!SchemaUtil.isXSDSimpleType(type.getLocalPart())) {
			// analyze this type. It can be element type or complex type.
			SchemaType schema = analyzeReferenceType(type);
			// part type is element
			if (schema instanceof ElementType) {
				element = (ElementType) schema;
			}
			// part type is complex type
			else if (schema instanceof ComplexType) {
				element.setComplex((ComplexType) schema);
			}
		}
		element.addType(type);
		return element;
	}

	/**
	 * Analyze reference type, element type , complex type or group reference.
	 * The reference type can't be simple type. Simple type is not necessary to
	 * analyze.
	 * 
	 * @param type
	 *            Reference type can't be simple type, and it must be defined in
	 *            schema.
	 * @return
	 */
	public SchemaType analyzeReferenceType(QName type) {
		// find type
		XmlSchemaObject schemaobj = findSchema(type);
		if (schemaobj != null) {
			// analyze element
			if (schemaobj instanceof XmlSchemaElement) {
				return analyzeElement((XmlSchemaElement) schemaobj);
			}
			// analyze complex
			else if (schemaobj instanceof XmlSchemaComplexType) {
				return analyzeComplexType((XmlSchemaComplexType) schemaobj);
			}
			// analyze group
			else if (schemaobj instanceof XmlSchemaGroup) {
				return analyzeGroupType((XmlSchemaGroup) schemaobj);
			}
		}
		return null;
	}

	/**
	 * Analyze Element type
	 * 
	 * @param schemaElement
	 * @param service
	 * @return
	 */
	public ElementType analyzeElement(XmlSchemaElement schemaElement) {
		if (schemaElement != null) {
			// Get the schema type.
			XmlSchemaType schema = schemaElement.getSchemaType();
			// Get element name
			String name = schemaElement.getName();
			// Create element type
			ElementType element = SchemaHelper.newElementSchema(name);
			// complex type
			if (schema instanceof XmlSchemaComplexType) {
				XmlSchemaComplexType complexType = (XmlSchemaComplexType) schema;
				// analyze complex type
				ComplexType complexSchema = analyzeComplexType(complexType);
				element.setComplex(complexSchema);
			}
			// simple type
			else if (schema instanceof XmlSchemaSimpleType) {
				XmlSchemaSimpleType simpleType = (XmlSchemaSimpleType) schema;
				javax.xml.namespace.QName type = simpleType.getQName();
				if (type == null) {
					// base type
					XmlSchemaSimpleTypeContent content = simpleType
							.getContent();
					if (content instanceof XmlSchemaSimpleTypeList) {
						XmlSchemaSimpleTypeList list = (XmlSchemaSimpleTypeList) content;
						type = list.getItemTypeName();
					} else if (content instanceof XmlSchemaSimpleTypeRestriction) {
						XmlSchemaSimpleTypeRestriction restriction = (XmlSchemaSimpleTypeRestriction) content;
						type = restriction.getBaseTypeName();
					} else if (content instanceof XmlSchemaSimpleTypeUnion) {
						XmlSchemaSimpleTypeUnion union = (XmlSchemaSimpleTypeUnion) content;
						QName[] types = union.getMemberTypesQNames();
						type = types[0];
					}
				}
				element.addType(type);

			}
			// reference to complex type
			else if (schemaElement.getSchemaTypeName() != null) {
				// Get type
				QName type = schemaElement.getSchemaTypeName();
				if (!type.getLocalPart().equals(SchemaConstants.ANYTYPE)) {
					// analyze reference type
					SchemaType refSchema = analyzeReferenceType(type);

					// complex type
					if (refSchema instanceof ComplexType) {
						element.setComplex((ComplexType) refSchema);
					}
				}
				element.addType(type);
			}
			// reference to element type
			else if (schemaElement.getRefName() != null) {
				// Get type
				QName type = schemaElement.getRefName();
				SchemaType refSchema = analyzeReferenceType(type);
				if (refSchema instanceof ComplexType) {
					element.setComplex((ComplexType) refSchema);
					element.addType(type);
				} else if (refSchema instanceof ElementType) {
					element = (ElementType) refSchema;
				}
			}
			// get max occur attribute
			long max = schemaElement.getMaxOccurs();
			element.setMaxOccurs(max);

			// get nillable attribute
			element.setIsNillable(schemaElement.isNillable());
			return element;
		}
		return null;
	}

	/**
	 * Analyze group type. In fact the group type is a complex type.
	 * 
	 * @param groupType
	 * @return
	 */
	public ComplexType analyzeGroupType(XmlSchemaGroup groupType) {
		QName type = groupType.getName();
		ComplexType schemaType = complexTypeMap.get(type);
		if (schemaType != null) {
			return schemaType;
		}
		// get complex type name
		String name = type.getLocalPart();
		ComplexType complex = SchemaHelper.newComplexTypeSchema(name);
		// get complex particle
		XmlSchemaParticle particle = groupType.getParticle();
		String particleName = null;
		if (particle != null) {
			// analyze particle
			particleName = analyzeParticle(particle, complex);
		}
		complex.setName(particleName);
		// save the analyzed complex type.
		complexTypeMap.put(type, complex);
		return complex;
	}

	/**
	 * Analyze Complex type
	 * 
	 * @param complexType
	 * @param service
	 * @return
	 */
	public ComplexType analyzeComplexType(XmlSchemaComplexType complexType) {

		// get complex type name
		String name = complexType.getName();
		// search complex type.
		if (name != null) {
			ComplexType complex = complexTypeMap.get(complexType.getQName());
			if (complex != null) {
				return complex;
			}
		}
		ComplexType complex = SchemaHelper.newComplexTypeSchema(name);

		// get complex particle
		XmlSchemaParticle particle = complexType.getParticle();
		String particleName = null;
		if (particle != null) {
			// analyze particle
			particleName = analyzeParticle(particle, complex);
		} else {
			XmlSchemaContentModel model = complexType.getContentModel();
			if (model != null) {
				// analyze particle
				// simple content model
				if (model instanceof XmlSchemaSimpleContent) {
					XmlSchemaSimpleContent simpleModel = (XmlSchemaSimpleContent) model;
					XmlSchemaContent content = simpleModel.getContent();
					if (content instanceof XmlSchemaSimpleContentExtension) {
						XmlSchemaSimpleContentExtension simpleContent = (XmlSchemaSimpleContentExtension) content;
						complex.setType(simpleContent.getBaseTypeName());
					} else if (content instanceof XmlSchemaSimpleContentRestriction) {
						XmlSchemaSimpleContentRestriction simpleRes = (XmlSchemaSimpleContentRestriction) content;
						complex.setType(simpleRes.getBaseTypeName());
					}
				}
				// complex content model
				else if (model instanceof XmlSchemaComplexContent) {
					XmlSchemaComplexContent complexModel = (XmlSchemaComplexContent) model;
					XmlSchemaContent content = complexModel.getContent();
					particleName = analyzeComplexContent(content, complex);
				}
			}
		}

		// save complex type.
		if (name != null) {
			complexTypeMap.put(complexType.getQName(), complex);
		}
		complex.setComplexParticle(particleName);
		return complex;
	}

	/**
	 * Analyze particle
	 * 
	 * @param particle
	 * @param complex
	 * @return
	 */
	public String analyzeParticle(XmlSchemaParticle particle,
			ComplexType complex) {
		List<ElementType> elements = complex.getElementTypes();
		XmlSchemaGroupBase schemaGroup = null;
		String particleName = null;
		// sequence
		if (particle instanceof XmlSchemaSequence) {
			schemaGroup = (XmlSchemaSequence) particle;
			particleName = SchemaConstants.SEQUENCE;
		}
		// all
		else if (particle instanceof XmlSchemaAll) {
			schemaGroup = (XmlSchemaAll) particle;
			particleName = SchemaConstants.ALL;
		}
		// choice
		else if (particle instanceof XmlSchemaChoice) {
			schemaGroup = (XmlSchemaChoice) particle;
			particleName = SchemaConstants.CHOICE;
		}
		if (schemaGroup != null) {
			// get all element
			XmlSchemaObjectCollection collection = schemaGroup.getItems();
			for (int i = 0; i < collection.getCount(); i++) {
				XmlSchemaObject obj = collection.getItem(i);
				// element
				if (obj instanceof XmlSchemaElement) {
					ElementType element = analyzeElement((XmlSchemaElement) obj);
					elements.add(element);
				}
				// any
				else if (obj instanceof XmlSchemaAny) {
					XmlSchemaAny any = (XmlSchemaAny) obj;
					String namespace = any.getNamespace();
					QName type = new QName(WSDLConstants.URI_2001_SCHEMA_XSD,
							namespace == null ? "" : namespace);
					ElementType element = SchemaHelper
							.newElementSchema(SchemaConstants.ANY);
					element.setMaxOccurs(any.getMaxOccurs());
					element.addType(type);
					elements.add(element);
				}
				// group reference
				else if (obj instanceof XmlSchemaGroupRef) {
					XmlSchemaGroupRef groupRef = (XmlSchemaGroupRef) obj;
					QName type = groupRef.getRefName();
					SchemaType schema = analyzeReferenceType(type);
					if (schema instanceof ComplexType) {
						ElementType element = SchemaHelper.newElementSchema();
						element.addType(type);
						element.setComplex((ComplexType) schema);
						element.setMaxOccurs(groupRef.getMaxOccurs());
						element.setGroupType(true);
						elements.add(element);
					}
				}
			}
		}
		return particleName;
	}

	/**
	 * Analyze complex content model.There are tow contents,
	 * ComoplexContentRestriction ,ComplexContentExtension.
	 * 
	 * @param content
	 *            complex Content
	 * @param elements
	 *            record every element.
	 * @return complex particle, sequence ,all, choice.
	 */
	public String analyzeComplexContent(XmlSchemaContent content,
			ComplexType complex) {
		List<ElementType> elements = complex.getElementTypes();
		// particle name
		String particleName = null;
		if (content != null) {
			XmlSchemaParticle particle = null;
			QName type = null;
			if (content instanceof XmlSchemaComplexContentExtension) {
				XmlSchemaComplexContentExtension complexContent = (XmlSchemaComplexContentExtension) content;
				type = complexContent.getBaseTypeName();
				particle = complexContent.getParticle();
			} else if (content instanceof XmlSchemaComplexContentRestriction) {
				XmlSchemaComplexContentRestriction complexRes = (XmlSchemaComplexContentRestriction) content;
				complex.setType(complexRes.getBaseTypeName());
				particle = complexRes.getParticle();
			}
			// base type
			if (type != null) {
				// analyze type
				SchemaType schemaType = analyzeReferenceType(type);
				if (schemaType instanceof ElementType) {
					elements.add((ElementType) schemaType);
					particleName = SchemaConstants.SEQUENCE;
				} else if (schemaType instanceof ComplexType) {
					ComplexType complexType = (ComplexType) schemaType;
					for (ElementType element : complexType.getElementTypes()) {
						elements.add(element);
					}
					particleName = complex.getComplexParticle();
				}
			}
			if (particle != null) {
				// analyze particle
				particleName = analyzeParticle(particle, complex);
			}
		}
		return particleName;
	}

	/**
	 * Find complex type ,element type or group schema.
	 * 
	 * @param type
	 *            type should not be simple type.
	 * @return
	 */
	private XmlSchemaObject findSchema(QName type) {
		if (type != null) {
			// search schema
			XmlSchemaObject schemaObj = schemaSearchMap.get(type);
			if (schemaObj != null) {
				return schemaObj;
			}
			for (XmlSchema schema : schemaList) {
				String target = schema.getTargetNamespace();
				if (target == null || target.equals("")
						|| target.equals(type.getNamespaceURI())) {
					// find complex type
					XmlSchemaType schemaType = schema.getTypeByName(type);
					if (schemaType != null) {
						schemaSearchMap.put(type, schemaType);
						return schemaType;
					}
					XmlSchemaObject group = schema.getGroups().getItem(type);
					if (group != null) {
						schemaSearchMap.put(type, group);
						return group;
					}
					// find element type
					XmlSchemaElement element = schema.getElementByName(type);
					if (element != null) {
						schemaSearchMap.put(type, element);
						return element;
					}
				}
			}
		}
		return null;
	}
}
