package org.act.xservices.rt.xcontainer.wsdl.types;

import java.io.ObjectStreamException;
import java.math.BigInteger;
import java.util.Random;

/**
 * Custom class for supporting primitive XSD data type nonNegativeInteger
 * 
 * @see <a href="http://www.w3.org/TR/xmlschema-2/#nonNegativeInteger">XML
 *      Schema 3.3.20</a>
 */
public class NonNegativeInteger extends BigInteger {

	private static final long serialVersionUID = -484577860351406054L;

	public NonNegativeInteger(byte[] val) {
		super(val);
		checkValidity();
	} // ctor

	public NonNegativeInteger(int signum, byte[] magnitude) {
		super(signum, magnitude);
		checkValidity();
	} // ctor

	public NonNegativeInteger(int bitLength, int certainty, Random rnd) {
		super(bitLength, certainty, rnd);
		checkValidity();
	} // ctor

	public NonNegativeInteger(int numBits, Random rnd) {
		super(numBits, rnd);
		checkValidity();
	} // ctor

	public NonNegativeInteger(String val) {
		super(val);
		checkValidity();
	}

	public NonNegativeInteger(String val, int radix) {
		super(val, radix);
		checkValidity();
	} // ctor

	/** validate the value against the xsd definition */
	private BigInteger zero = new BigInteger("0");

	private void checkValidity() {
		if (compareTo(zero) < 0) {
			throw new NumberFormatException(
			// Messages.getMessage("badNonNegInt00") +
					":  " + this);
		}
	} // checkValidity

	/**
	 * Work-around for
	 * http://developer.java.sun.com/developer/bugParade/bugs/4378370.html
	 * 
	 * @return BigIntegerRep
	 * @throws ObjectStreamException
	 */
	public Object writeReplace() throws ObjectStreamException {
		return new BigIntegerRep(toByteArray());
	}

	protected static class BigIntegerRep implements java.io.Serializable {
		private static final long serialVersionUID = -6135065605514678689L;
		private byte[] array;

		protected BigIntegerRep(byte[] array) {
			this.array = array;
		}

		protected Object readResolve() throws java.io.ObjectStreamException {
			return new NonNegativeInteger(array);
		}
	}
} // class NonNegativeInteger
