package org.act.xservices.rt.xcontainer.wsdl.types;

/**
 * Custom class for supporting primitive XSD data type UnsignedByte
 * 
 * @see <a href="http://www.w3.org/TR/xmlschema-2/#unsignedByte">XML Schema
 *      3.3.24</a>
 */
public class UnsignedByte extends UnsignedShort {

	private static final long serialVersionUID = 4631754787145526759L;

	public UnsignedByte() {

	}

	/**
	 * ctor for UnsignedByte
	 * 
	 * @throws Exception
	 *             will be thrown if validation fails
	 */
	public UnsignedByte(long sValue) throws NumberFormatException {
		setValue(sValue);
	}

	public UnsignedByte(String sValue) throws NumberFormatException {
		setValue(Long.parseLong(sValue));
	}

	/**
	 * validates the data and sets the value for the object.
	 * 
	 * @param sValue
	 *            the number to set
	 */
	public void setValue(long sValue) throws NumberFormatException {
		if (!UnsignedByte.isValid(sValue))
			throw new NumberFormatException(
			// Messages.getMessage("badUnsignedByte00") +
					String.valueOf(sValue) + "]");
		lValue = new Long(sValue);
	}

	/**
	 * validate the value against the xsd value space definition
	 * 
	 * @param sValue
	 *            number to check against range
	 */
	public static boolean isValid(long sValue) {
		return !((sValue < 0L) || (sValue > 255L));
	}

}
