package org.act.xservices.rt.xcontainer.rpc.util;

import java.util.ArrayList;
import java.util.List;

import org.act.xservices.rt.soap.ISOAPEnvelope;
import org.act.xservices.rt.soap.factory.SOAPFactory;
import org.act.xservices.rt.xcontainer.framework.provider.ServiceProcessException;
import org.act.xservices.rt.xcontainer.inf.context.IServiceContext;
import org.act.xservices.rt.xcontainer.inf.service.wsdl.WSDL11Constants;
import org.act.xservices.rt.xcontainer.inf.service.wsdl.WSDL20Constants;
import org.act.xservices.rt.xcontainer.inf.service.wsdl.WSDLConstants;
import org.act.xservices.rt.xcontainer.inf.service.wsdl.XcMessage;
import org.act.xservices.rt.xcontainer.inf.service.wsdl.XcMessagePart;
import org.act.xservices.rt.xcontainer.inf.service.wsdl.XcOperation;
import org.act.xservices.rt.xcontainer.inf.service.wsdl.XcService;
import org.act.xservices.rt.xcontainer.rpc.SerializationException;
import org.act.xservices.rt.xcontainer.util.JavaObjectSerializeUtil;
import org.act.xservices.rt.xcontainer.wsdl.utils.ParameterNameUtil;
import org.act.xservices.rt.xom.Metaset;
import org.act.xservices.rt.xom.Namespace;
import org.act.xservices.rt.xom.QName;

/**
 * 
 * @author viking
 * @date 2010-7-3
 * @since JDK 1.5.0
 * @version 1.0.0
 * 
 */
public class RPCProcessResponseUtil {

	public static IServiceContext processResonse(
			IServiceContext serviceContext, Object resObject)
			throws ServiceProcessException {
		// get the operation of the web service
		XcOperation xcOperation = serviceContext.getXcOperation();
		XcService service = serviceContext.getXcService();
		try {
			// get the response soap message
			XcMessage outXcMessage = xcOperation.getOutputXcMessage();
			QName operation = null;
			if (outXcMessage != null && outXcMessage.getQName() != null) {
				operation = outXcMessage.getQName();
			}else{
				operation=xcOperation.getQName();
			}
			if (xcOperation.getOperationStyle().equals(
					WSDL11Constants.STYLE_DOC)) {
				for (XcMessagePart part : outXcMessage.getXcMessagePartMap()
						.values()) {
					operation = part.getParameterType();
					break;
				}
			}
			
			// get the soap factory
			String nsURI = serviceContext.getInMessage().getNamespaceURI();
			SOAPFactory factory = SOAPFactory.getSOAPFactory(nsURI);		
			ISOAPEnvelope envelope = factory.getDefaultEnvelope();

			// if the method is inonly
			if (WSDL20Constants.MEP_URI_ROBUST_IN_ONLY.equals(xcOperation
					.getMepURI())) {
				Metaset bodyChild = factory.createElement(new QName(service
						.getServiceName(), operation.getNamespace()));
				envelope.getBody().add(bodyChild);
				serviceContext.setOutMessage(envelope);
				return serviceContext;
			}

			Metaset[] bodyContents = null;
			Object generateBare = service
					.getParameter(WSDLConstants.DOC_LIT_BARE_PARAMETER);

			String[] argNames = ParameterNameUtil
					.getResponseArgNames(xcOperation.getOutputXcMessage());
			if (generateBare != null && "true".equals(generateBare)) {
				// 
				bodyContents = RPCProcessResponseUtil
						.processResonseAsDocLitBare(operation, resObject,
								argNames, service.isElementFormDefault());
			} else {
				// wrapped the response result
				bodyContents = RPCProcessResponseUtil
						.processResponseAsDocLitWrapped(operation, resObject,
								argNames, service.isElementFormDefault());
			}
			for (Metaset bodyContent : bodyContents) {
				envelope.getBody().add(bodyContent);
			}
			serviceContext.setOutMessage(envelope);
			return serviceContext;
		} catch (SerializationException e) {
			throw new ServiceProcessException(e);
		}
	}

	/**
	 * 
	 * @param opQName
	 * @param resObj
	 * @param argNames
	 * @param qualified
	 * @return
	 * @throws SerializationException
	 */
	public static Metaset[] processResponseAsDocLitWrapped(QName opQName,
			Object resObj, String[] argNames, boolean qualified)
			throws SerializationException {
		Metaset result = JavaObjectSerializeUtil.serializeJavaObject(opQName,
				new Object[] { resObj }, argNames, qualified);
		return new Metaset[] { result };
	}

	/**
	 * 
	 * @param opQName
	 * @param resObject
	 * @param argNames
	 * @param qualified
	 * @return
	 * @throws SerializationException
	 */
	public static Metaset[] processResonseAsDocLitBare(QName opQName,
			Object resObject, String[] argNames, boolean qualified)
			throws SerializationException {

		Namespace itemNS = null;
		if (qualified) {
			itemNS = opQName.getNamespace();
		}
		// parameter count
		int parameterCount = 0;
		List<Metaset> operation = new ArrayList<Metaset>();
		if (resObject != null) {

			String itemName = null;
			if (argNames != null && parameterCount < argNames.length) {
				itemName = argNames[parameterCount++];
			}
			// serialize each parameter
			Metaset[] args = JavaObjectSerializeUtil.serializeParameter(
					resObject, itemName, itemNS, qualified);
			for (Metaset arg : args) {
				// add serialized parameter
				operation.add(arg);
			}
		}
		return operation.toArray(new Metaset[operation.size()]);
	}
}
