package org.act.xservices.rt.xcontainer.util;

import java.beans.BeanInfo;
import java.beans.IntrospectionException;
import java.beans.Introspector;
import java.beans.PropertyDescriptor;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;

import org.act.xservices.rt.xcontainer.rpc.DeserializationException;
import org.act.xservices.rt.xcontainer.rpc.SerializationException;
import org.act.xservices.rt.xom.Metaset;
import org.act.xservices.rt.xom.Namespace;
import org.codehaus.jam.JClass;
import org.codehaus.jam.JProperty;
import org.codehaus.jam.JamClassIterator;
import org.codehaus.jam.JamService;
import org.codehaus.jam.JamServiceFactory;
import org.codehaus.jam.JamServiceParams;

/**
 * 
 * @author viking
 * @date 2010-7-26
 * @since JDK 1.5.0
 * @version 1.0.0
 * 
 *          This Class is used to serialize and deserialize Java bean.
 * 
 */
public class JavaBeanUtil {

	public static Metaset serializeJavaBean(Object javaBean, Namespace itemNS,
			boolean qualified) throws SerializationException {

		String beanName = javaBean.getClass().getSimpleName();
		Metaset javaBeanMetaset = JavaObjectSerializeUtil.getItemMetaset(
				beanName, itemNS, qualified);
		// properties from JAM
		JProperty[] properties = getProperteies(javaBean);

		// properties map from bean
		HashMap<String, PropertyDescriptor> propertMap = null;
		try {
			propertMap = getPropertiesMap(javaBean.getClass());
		} catch (Exception e) {
			throw new SerializationException(e);
		}
		for (int itemCount = 0; itemCount < properties.length; itemCount++) {
			JProperty property = properties[itemCount];
			String propertyName = property.getSimpleName().toLowerCase();
			PropertyDescriptor propDesc = propertMap
					.get(getCorrectName(propertyName));
			if (propDesc == null) {
				propDesc = propertMap.get(propertyName);
			}
			if (propDesc == null) {
				continue;
			}

			// get property value
			Object value = getPropertyValue(propDesc, javaBean, property);

			Metaset[] items = JavaObjectSerializeUtil.serializeParameter(value,
					propDesc.getName(), itemNS, qualified);
			for (Metaset item : items) {
				javaBeanMetaset.add(item);
			}
		}
		return javaBeanMetaset;
	}

	public static Object deserializeJavaBean(Metaset beanElement,
			Class<?> javaBeanClass, MultirefHelper helper)
			throws DeserializationException {

		try {
			// properties map from bean
			HashMap<String, PropertyDescriptor> propertMap = getPropertiesMap(javaBeanClass);
			// java bean instance
			Object beanObj = javaBeanClass.newInstance();

			// java bean parts
			List<Metaset> parts = beanElement.elements();
			for (int i = 0; i < parts.size();) {
				// part
				Metaset part = parts.get(i);
				Metaset[] items = JavaObjectDeserializeUtil.getSameItems(i,
						parts);
				i += items.length;
				// part name
				String partLocalName = part.getName();
				// get java bean property
				PropertyDescriptor prty = (PropertyDescriptor) propertMap
						.get(partLocalName.toLowerCase());

				Type type = prty.getReadMethod().getGenericReturnType();
				// deserialize part
				Object partObj = JavaObjectDeserializeUtil
						.deserializeParameter(items, type, helper);
				Object[] parms = new Object[] { partObj };
				Method writeMethod = prty.getWriteMethod();
				if (writeMethod != null) {
					writeMethod.setAccessible(true);
					writeMethod.invoke(beanObj, parms);
				}
			}
			return beanObj;
		} catch (Exception e) {
			throw new DeserializationException(e);
		}
	}

	private static JProperty[] getProperteies(Object beanObject)
			throws SerializationException {

		// get JClass from JavaBean instance
		JClass jClass = getJClass(beanObject);

		// add all properties hat not exclude from bean
		ArrayList<JProperty> propertyList = new ArrayList<JProperty>();
		JProperty properties[] = jClass.getDeclaredProperties();
		for (int i = 0; i < properties.length; i++) {
			JProperty property = properties[i];
			propertyList.add(property);
		}

		// add all properties hat not exclude from super class
		JClass supClass = jClass.getSuperclass();
		while (!supClass.getQualifiedName().startsWith("java.")) {
			properties = supClass.getDeclaredProperties();
			for (int i = 0; i < properties.length; i++) {
				JProperty property = properties[i];
				propertyList.add(property);
			}
			supClass = supClass.getSuperclass();
		}
		properties = new JProperty[propertyList.size()];
		for (int i = 0; i < propertyList.size(); i++) {
			JProperty jProperty = (JProperty) propertyList.get(i);
			properties[i] = jProperty;
		}
		Arrays.sort(properties);
		return properties;
	}

	/**
	 * Get JClass from Object
	 * 
	 * @param beanObject
	 * @return
	 * @throws SerializationException
	 */
	private static JClass getJClass(Object beanObject)
			throws SerializationException {
		JamServiceFactory factory = JamServiceFactory.getInstance();
		JamServiceParams jam_service_parms = factory.createServiceParams();
		// get the class loader
		ClassLoader cl = beanObject.getClass().getClassLoader();
		if (cl == null)
			cl = ClassLoader.getSystemClassLoader();
		jam_service_parms.addClassLoader(cl);
		jam_service_parms.includeClass(beanObject.getClass().getName());
		JamService service;
		try {
			service = factory.createService(jam_service_parms);
		} catch (IOException e) {
			throw new SerializationException(e);
		}
		JamClassIterator jClassIter = service.getClasses();
		JClass jClass = null;
		if (jClassIter.hasNext()) {
			jClass = (JClass) jClassIter.next();
		} else {
			throw new SerializationException(
					"No service class found , exception from JAM");
		}
		return jClass;
	}

	/**
	 * JAM convert first name of an attribute into UpperCase as an example if
	 * there is a instance variable called foo in a bean , then Jam give that as
	 * Foo so this method is to correct that error
	 * 
	 * @param wrongName
	 * @return the right name, using english as the locale for case conversion
	 */
	private static String getCorrectName(String wrongName) {
		if (wrongName.length() > 1) {
			return wrongName.substring(0, 1).toLowerCase(Locale.ENGLISH)
					+ wrongName.substring(1, wrongName.length());
		} else {
			return wrongName.substring(0, 1).toLowerCase(Locale.ENGLISH);
		}
	}

	/**
	 * 
	 * @param beanObject
	 * @return
	 * @throws Exception
	 */
	private static HashMap<String, PropertyDescriptor> getPropertiesMap(
			Class<?> beanObject) throws Exception {
		BeanInfo beanInfo;
		HashMap<String, PropertyDescriptor> propertMap = new HashMap<String, PropertyDescriptor>();
		try {
			beanInfo = Introspector.getBeanInfo(beanObject);
			PropertyDescriptor[] propDescs = beanInfo.getPropertyDescriptors();
			for (int i = 0; i < propDescs.length; i++) {
				PropertyDescriptor propDesc = propDescs[i];
				if (propDesc.getName().equals("class")) {
					continue;
				}
				propertMap.put(propDesc.getName().toLowerCase(), propDesc);
			}
		} catch (IntrospectionException e) {
			throw new Exception(e);
		}

		return propertMap;
	}

	/**
	 * 
	 * @param propDesc
	 * @param beanObject
	 * @param property
	 * @return
	 * @throws SerializationException
	 */
	private static Object getPropertyValue(PropertyDescriptor propDesc,
			Object beanObject, JProperty property)
			throws SerializationException {
		Method readMethod = propDesc.getReadMethod();
		Object value = null;
		if (readMethod != null) {
			if (property.getGetter() != null && property.getGetter().isPublic()) {
				readMethod.setAccessible(true);
			}
			// get the property value
			try {
				value = readMethod.invoke(beanObject, new Object[] {});
			} catch (IllegalArgumentException e) {
				throw new SerializationException(e);
			} catch (IllegalAccessException e) {
				throw new SerializationException(e);
			} catch (InvocationTargetException e) {
				throw new SerializationException(e);
			}
		} else {
			throw new SerializationException("can not find read method for : "
					+ propDesc.getName());
		}
		return value;
	}
}
