package org.act.xservices.rt.xcontainer.util;

import java.lang.reflect.Array;
import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.act.xservices.rt.xcontainer.framework.provider.ProviderProcessException;
import org.act.xservices.rt.xcontainer.rpc.DeserializationException;
import org.act.xservices.rt.xcontainer.wsdl.typemapping.SimpleTypeMapper;
import org.act.xservices.rt.xom.Attribute;
import org.act.xservices.rt.xom.Metaset;

import com.sun.org.apache.xerces.internal.impl.dv.util.Base64;

/**
 * 
 * @author viking
 * @date 2010-7-26
 * @since JDK 1.5.0
 * @version 1.0.0 This class is used to deserialize metaset. Convert metaset to
 *          java type.
 * 
 * 
 */
public class JavaObjectDeserializeUtil {
	/**
	 * 
	 * @param response
	 * @param javaTypes
	 * @return
	 * @throws DeserializationException
	 */
	public static Object[] deserializeJavaObject(Metaset response,
			Type[] javaTypes) throws DeserializationException {
		/*
		 * Take the number of parameters in the method and , only take that much
		 * of child elements from the Metaset , other are ignore , as an example
		 * if the method is , foo(String a , int b) and if the Metaset <foo>
		 * <arg0>Val1</arg0> <arg1>Val2</arg1> <arg2>Val3</arg2>
		 * 
		 * only the val1 and Val2 take into account
		 */
		if (javaTypes == null) {
			return null;
		}
		int length = javaTypes.length;

		Object[] retObjs = new Object[length];

		/*
		 * If the body first child contains , then there can not be any other
		 * element without refs , so I can assume if the first child of the body
		 * first element has ref then the message has to handle as mutiref
		 * message. as an exmple if the body is like below <foo> <arg0
		 * href="#0"/> </foo>
		 * 
		 * then there can not be any element without refs , meaning following we
		 * are not handling <foo> <arg0 href="#0"/> <arg1>absbsbs</arg1> </foo>
		 */

		// get all child of the soap body
		List<Metaset> paramters = response.elements();
		// to handle multirefs
		// have to check the instanceof
		MultirefHelper helper = new MultirefHelper((Metaset) response
				.getParent());

		int eleCount = 0;
		for (int i = 0; i < length; i++) {
			Type parameterType = javaTypes[i];
			if (eleCount < paramters.size()) {
				Metaset[] items = getSameItems(eleCount, paramters);
				eleCount += items.length;
				retObjs[i] = deserializeParameter(items, parameterType, helper);
			}
		}
		// Ensure that we have at least a zero element array
		for (int i = 0; i < length; i++) {
			Type type = javaTypes[i];
			if (type instanceof Class<?>) {
				Class<?> clazz = (Class<?>) type;
				if (retObjs[i] == null && clazz.isArray()) {
					retObjs[i] = Array.newInstance(clazz.getComponentType(), 0);
				}
			}
		}

		helper.clean();
		return retObjs;
	}

	public static Object deserializeParameter(Metaset[] items, Type paramType,
			MultirefHelper helper) throws DeserializationException {
		// if there is a reference
		boolean hasRef = false;
		// get the reference attribute
		Attribute attr = MultirefHelper.processRefAtt(items[0]);
		String ref = null;
		if (attr != null) {
			hasRef = true;
			// get the reference name
			ref = MultirefHelper.getAttvalue(attr);
		}
		if (paramType instanceof Class<?>) {
			Class<?> paramClass = (Class<?>) paramType;
			if (Metaset.class.isAssignableFrom(paramClass)) {
				// process Metaset
				return deserializeMetaset(items[0], hasRef, ref, helper);
			} else if (hasRef) {
				if (helper.getObject(ref) != null) {
					return helper.getObject(ref);
				} else {
					try {
						return helper.processRef(paramClass, ref);
					} catch (ProviderProcessException e) {
						throw new DeserializationException(e);
					} catch (InstantiationException e) {
						throw new DeserializationException(e);
					}
				}
			} else if (items[0].attribute(SerializeConstants.NIL_QNAME) != null) {
				return null;
			} else if (SimpleTypeMapper.isSimpleType(paramClass)) {
				// simple type
				return SimpleTypeMapper.getSimpleTypeObject(paramClass,
						items[0]);
			} else if (byte[].class.isAssignableFrom(paramClass)) {
				// byte[]
				return Base64.decode(items[0].getText());
			} else if (paramClass.isArray()) {
				// array
				return deserializeArray(items, paramClass, helper);
			} else if (SimpleTypeMapper.isDataHandler(paramClass)) {
				return SimpleTypeMapper.getDataHandler(items[0]);
			} else if (SimpleTypeMapper.isCollection(paramClass)) {
				throw new DeserializationException(
						"The Colleciton must has parameter type, so the type should be ParameterizedType.");
			} else if (SimpleTypeMapper.isMap(paramClass)) {
				throw new DeserializationException(
						"The Map must has parameter type, so the type should be ParameterizedType.");
			}
			return JavaBeanUtil.deserializeJavaBean(items[0], paramClass,
					helper);
		}
		if (paramType instanceof ParameterizedType) {
			Type type = ((ParameterizedType) paramType).getRawType();
			if (SimpleTypeMapper.isCollection((Class<?>) type)) {
				// collection
				return deserializeArrayList(items[0], paramType, helper);
			} else if (SimpleTypeMapper.isMap((Class<?>) type)) {
				// map
				return deserializeMap(items[0], paramType, helper);
			}
		}
		return null;
	}

	/**
	 * Deserialize Metaset
	 * 
	 * @param item
	 * @param hasRef
	 * @param ref
	 * @param helper
	 * @return
	 * @throws DeserializationException
	 */
	private static Metaset deserializeMetaset(Metaset item, boolean hasRef,
			String ref, MultirefHelper helper) throws DeserializationException {
		// the result has been wrapped .so the first element is the result.
		item = item.elements().get(0);
		if (hasRef) {
			// get reference metaset
			item = helper.getMetasetElement(ref);
			if (item == null) {
				try {
					item = helper.processElementRef(ref);
				} catch (ProviderProcessException e) {
					throw new DeserializationException(e);
				}
			}
		}
		item.detach();
		return item;
	}

	/**
	 * Process array, make attention that
	 * int,float,long,double,short,char,byte,boolean are not Object.
	 * 
	 * @param arrayMetaset
	 * @param parameterType
	 * @param classLoader
	 * @param helper
	 * @return
	 * @throws DeserializationException
	 */
	private static Object deserializeArray(Metaset[] arrayMetaset,
			Class<?> parameterType, MultirefHelper helper)
			throws DeserializationException {
		// array length
		int length = arrayMetaset.length;
		// array type
		Class<?> arrayType = parameterType.getComponentType();
		// instance an array
		Object array = Array.newInstance(arrayType, length);

		// array type name
		String arrayTypeName = arrayType.getName();
		for (int i = 0; i < length; i++) {
			// array item
			Metaset arrayItem = arrayMetaset[i];

			// deserialize array item
			Object item = deserializeParameter(new Metaset[] { arrayItem },
					arrayType, helper);
			// set array item value
			SimpleTypeMapper.setArrayItem(array, i, item, arrayTypeName);
		}
		return array;
	}

	@SuppressWarnings("unchecked")
	private static Object deserializeArrayList(Metaset arrayMetaset,
			Type parameterType, MultirefHelper helper)
			throws DeserializationException {
		try {
			Class<?> classType = (Class<?>) ((ParameterizedType) parameterType)
					.getRawType();
			Type type = ((ParameterizedType) parameterType)
					.getActualTypeArguments()[0];

			Collection<Object> collection = null;

			if (classType.isInterface()) {
				collection = new ArrayList<Object>();

			} else {
				collection = (Collection<Object>) classType.newInstance();
			}
			Iterator<Metaset> items = arrayMetaset.elementIterator();

			while (items.hasNext()) {
				Metaset listItem = items.next();
				Object item = deserializeParameter(new Metaset[] { listItem },
						type, helper);
				collection.add(item);
			}
			return collection;
		} catch (InstantiationException e) {
			throw new DeserializationException(e);
		} catch (IllegalAccessException e) {
			throw new DeserializationException(e);
		}
	}

	@SuppressWarnings("unchecked")
	private static Object deserializeMap(Metaset mapMetaset,
			Type parameterType, MultirefHelper helper)
			throws DeserializationException {
		try {
			Map<Object, Object> map = null;
			Class<?> classType = (Class<?>) ((ParameterizedType) parameterType)
					.getRawType();
			if (classType.isInterface()) {
				map = new HashMap<Object, Object>();

			} else {
				map = (Map<Object, Object>) classType.newInstance();
			}
			Iterator<Metaset> items = mapMetaset.elementIterator();
			Type[] types = ((ParameterizedType) parameterType)
					.getActualTypeArguments();
			Type keyClass = types[0];
			Type valueClass = types[1];

			while (items.hasNext()) {
				Object key = null;
				Object value = null;
				Metaset keyItem = items.next();
				key = deserializeParameter(new Metaset[] { keyItem }, keyClass,
						helper);

				if (!items.hasNext()) {
					throw new DeserializationException(
							"deserialize failed, invalid soap message.");
				} else {
					Metaset valueItem = items.next();
					value = deserializeParameter(new Metaset[] { valueItem },
							valueClass, helper);
				}
				map.put(key, value);
			}
			return map;
		} catch (InstantiationException e) {
			throw new DeserializationException(e);
		} catch (IllegalAccessException e) {
			throw new DeserializationException(e);
		}
	}

	public static Metaset[] getSameItems(int eleCount, List<Metaset> parameters) {
		Metaset item = parameters.get(eleCount++);
		List<Metaset> list = new ArrayList<Metaset>();
		list.add(item);
		while (eleCount < parameters.size()
				&& item.getName().equals(parameters.get(eleCount).getName())) {
			list.add(parameters.get(eleCount));
			eleCount++;
		}
		Metaset[] items = list.toArray(new Metaset[list.size()]);
		return items;
	}
}
