package org.act.xservices.rt.xcontainer.util;

import org.act.xservices.rt.soap.ISOAPBody;
import org.act.xservices.rt.soap.ISOAPEnvelope;
import org.act.xservices.rt.soap.factory.SOAPFactory;
import org.act.xservices.rt.xcontainer.framework.provider.ProviderProcessException;
import org.act.xservices.rt.xcontainer.rpc.DeserializationException;
import org.act.xservices.rt.xcontainer.wsdl.typemapping.SimpleTypeMapper;
import org.act.xservices.rt.xom.Attribute;
import org.act.xservices.rt.xom.Metaset;
import org.act.xservices.rt.xom.Node;
import org.act.xservices.rt.xom.QName;
import org.act.xservices.rt.xom.Text;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

/**
 * 
 * @author viking
 * 
 */
public class MultirefHelper {

	public static final String SOAP12_REF_ATTR = "ref";
	public static final String SOAP11_REF_ATTR = "href";

	private boolean filledTable;

	private Metaset parent;

	private HashMap<String, Object> objectmap = new HashMap<String, Object>();
	private HashMap<String, Metaset> elementMap = new HashMap<String, Metaset>();
	private HashMap<String, Metaset> metasetElementMap = new HashMap<String, Metaset>();

	public MultirefHelper(Metaset parent) {
		this.parent = parent;
	}

	public Object getObject(String id) {
		return objectmap.get(id);
	}

	public Metaset getMetasetElement(String id) {
		return (Metaset) metasetElementMap.get(id);
	}

	public Metaset processElementRef(String id) throws ProviderProcessException {
		if (!filledTable) {
			readallChildElements();
		}
		Metaset val = elementMap.get(id);
		if (val == null) {
			throw new ProviderProcessException("Invalid reference :" + id);
		} else {
			Metaset ele = processElementforRefs(val);
			Metaset cloneele = ele.createCopy();
			metasetElementMap.put(id, cloneele);
			return cloneele;
		}
	}

	public Metaset processElementforRefs(Metaset elemnts)
			throws ProviderProcessException {
		Iterator<Metaset> itr = elemnts.elementIterator();
		while (itr.hasNext()) {
			Metaset element = itr.next();
			Attribute attri = processRefAtt(element);
			if (attri != null) {
				String ref = getAttvalue(attri);
				Metaset tempele = getMetasetElement(ref);
				if (tempele == null) {
					tempele = processElementRef(ref);
				}
				Metaset ele2 = tempele.createCopy();
				Iterator<Metaset> itrChild = ele2.elementIterator();
				while (itrChild.hasNext()) {
					Metaset item = itrChild.next();
					item.detach();
					element.add(item);
				}
			}
		}
		return elemnts;
	}

	public Object processRef(Class<?> javatype, String id)
			throws ProviderProcessException, InstantiationException,
			DeserializationException {
		if (!filledTable) {
			readallChildElements();
		}
		Metaset val = (Metaset) elementMap.get(id);
		if (val == null) {
			throw new ProviderProcessException("Invalid reference :" + id);
		} else {
			if (SimpleTypeMapper.isSimpleType(javatype)) {
				/**
				 * in this case OM element can not contains more child, that is
				 * no way to get the value as an exp , <refernce id="12">
				 * <value>foo</value> </refernce> the above one is not valid ,
				 * that should always be like below <refernce
				 * id="12">foo</refernce>
				 */
				Object valObj = SimpleTypeMapper.getSimpleTypeObject(javatype,
						val);
				objectmap.put(id, valObj);
				return valObj;
			} else if (SimpleTypeMapper.isCollection(javatype)) {
				Object valobj = val.elements();
				objectmap.put(id, valobj);
				return valobj;
			} else {
				Object obj = JavaBeanUtil.deserializeJavaBean(val, javatype,
						this);

				objectmap.put(id, obj);
				return obj;
			}
		}
	}

	private void readallChildElements() {
		Iterator<Metaset> childs = parent.elementIterator();
		while (childs.hasNext()) {
			Metaset metaset = childs.next();
			Attribute id = metaset.attribute(new QName("id"));
			if (id != null) {
				elementMap.put(id.getValue(), (Metaset) metaset.detach());
			}
		}
		filledTable = true;
	}

	public static String getAttvalue(Attribute omatribute) {
		String ref;
		ref = omatribute.getValue();
		if (ref != null) {
			if (ref.charAt(0) == '#') {
				ref = ref.substring(1);
			}
		}
		return ref;
	}

	public static Attribute processRefAtt(Metaset metaset) {
		Attribute attri = metaset.attribute(new QName(SOAP11_REF_ATTR));
		if (attri == null) {
			attri = metaset.attribute(new QName(SOAP12_REF_ATTR));
		}
		return attri;
	}

	public void clean() {
		elementMap.clear();
		objectmap.clear();
	}

	/**
	 * this method is used to process the href attributes which may comes with
	 * the incomming soap mesaage <soap:body> <operation> <arg1 href="#obj1"/>
	 * </operation> <multiref id="obj1"> <name>the real argument</name>
	 * <color>blue</color> </multiref> </soap:body> here we assume first child
	 * of the soap body has the main object structure and others contain the
	 * multiref parts. Soap spec says that those multiref parts must be top
	 * level elements.
	 * 
	 * @param soapEnvelope
	 */

	public static void processHrefAttributes(ISOAPEnvelope soapEnvelope)
			throws ProviderProcessException {
		// first populate the multiref parts to a hash table.
		ISOAPBody soapBody = soapEnvelope.getBody();
		// first build the whole tree

		Metaset metaset = null;
		Attribute idAttribute = null;
		Map<String, Metaset> idAndOMElementMap = new HashMap<String, Metaset>();
		for (Iterator<Metaset> iter = soapBody.elementIterator(); iter
				.hasNext();) {
			metaset = iter.next();
			// the attribute id is an unqualified attribute
			idAttribute = metaset.attribute(new QName(null, "id"));
			if (idAttribute != null) {
				// for the first element there may not have an id
				idAndOMElementMap.put(idAttribute.getValue(), metaset);
			}
		}

		// start processing from the first child
		processHrefAttributes(idAndOMElementMap, soapBody.elements().get(0));

	}

	public static void processHrefAttributes(
			Map<String, Metaset> idAndOMElementMap, Metaset elementToProcess)
			throws ProviderProcessException {

		// first check whether this element has an href value.
		// href is also an unqualifed attribute
		Attribute hrefAttribute = elementToProcess.attribute(new QName(null,
				"href"));
		if (hrefAttribute != null) {
			// i.e this has an href attribute
			String hrefAttributeValue = hrefAttribute.getValue();
			if (!hrefAttributeValue.startsWith("#")) {
				throw new ProviderProcessException("In valid href ==> "
						+ hrefAttributeValue + " does not starts with #");
			} else {
				Metaset referedOMElement = (Metaset) idAndOMElementMap
						.get(hrefAttributeValue.substring(1));
				if (referedOMElement == null) {
					throw new ProviderProcessException("In valid href ==> "
							+ hrefAttributeValue + " can not find"
							+ "the matching element");
				} else {
					// now we have to remove the hrefAttribute and add all the
					// child elements to the
					// element being proccesed
					elementToProcess.remove(hrefAttribute);
					Metaset clonedReferenceElement = getClonedOMElement(referedOMElement);
					Node omNode = null;
					for (Iterator<Node> iter = clonedReferenceElement
							.nodeIterator(); iter.hasNext();) {
						omNode = (Node) iter.next();
						elementToProcess.add(omNode.detach());
					}

					// add attributes
					Attribute omAttribute = null;
					for (Iterator<Attribute> iter = clonedReferenceElement
							.attributeIterator(); iter.hasNext();) {
						omAttribute = (Attribute) iter.next();
						// we do not have to populate the id attribute
						if (!omAttribute.getName().equals("id")) {
							elementToProcess.add(omAttribute);
						}
					}
				}
			}
		}

		// call recursively to proces all elements
		Metaset childOMElement = null;
		for (Iterator<Metaset> iter = elementToProcess.elementIterator(); iter
				.hasNext();) {
			childOMElement = iter.next();
			processHrefAttributes(idAndOMElementMap, childOMElement);
		}
	}

	/**
	 * returns an cloned om element for this Metaset
	 * 
	 * @param Metaset
	 * @return cloned Metaset
	 */
	public static Metaset getClonedOMElement(Metaset metaset)
			throws ProviderProcessException {
		SOAPFactory omFactory = SOAPFactory.getInstance();
		Metaset newOMElement = omFactory.createElement(metaset.getQName());

		// copying attributes
		Attribute omAttribute = null;
		Attribute newOMAttribute = null;
		for (Iterator<Attribute> iter = metaset.attributeIterator(); iter
				.hasNext();) {
			omAttribute = iter.next();
			if (!omAttribute.getValue().equals("id")) {
				newOMAttribute = omFactory.createAttribute(null, new QName(
						omAttribute.getName(), omAttribute.getNamespace()),
						omAttribute.getValue());
				newOMElement.add(newOMAttribute);
			}
		}
		Node omNode = null;
		Text omText = null;
		for (Iterator<Node> iter = metaset.nodeIterator(); iter.hasNext();) {
			omNode = (Node) iter.next();
			if (omNode instanceof Text) {
				omText = (Text) omNode;
				newOMElement.add(omFactory.createText(omText.getText()));
			} else if (omNode instanceof Metaset) {
				newOMElement.add(((Metaset) omNode).createCopy());
			} else {
				throw new ProviderProcessException(
						"Unknown child element type ==> "
								+ omNode.getClass().getName());
			}
		}
		return newOMElement;
	}

}
