
package org.act.xservices.rt.xnotification;

import java.util.Calendar;
import java.util.HashSet;
import java.util.Set;
import java.util.concurrent.Callable;
import java.util.concurrent.FutureTask;

import org.act.xservices.rt.xnotification.base.fault.NotificationBaseException;
import org.act.xservices.rt.xnotification.base.impl.ANotificationConsumer;
import org.act.xservices.rt.xnotification.base.impl.NotificationProducer;
import org.act.xservices.rt.xnotification.base.impl.NotificationPublisher;
import org.act.xservices.rt.xnotification.broker.impl.NotificationBroker;
import org.act.xservices.rt.xnotification.client.http.HttpCallClient;
import org.act.xservices.rt.xnotification.message.base.Message;
import org.act.xservices.rt.xnotification.message.base.MessageFactory;
import org.act.xservices.rt.xom.Metaset;

/**
 * ڴɷϢ,֪ͨʵֵеϢߡ
 * ϢɷӦĽɫ
 * 
 * @version 1.0 2008-3-12
 * @author cover_eye wangyg@act.buaa.edu.cn 2006-6-13
 * @author Lizq [lizq@act.buaa.edu.cn]
 * @since JDK1.5
 */

public class NotifyMessageBus {
	
    public final static int IN_MODE=0;
    public final static int OUT_MODE=1;
  
    private static NotifyMessageBus INSTANCE = new NotifyMessageBus();
    
    //ֵ֧Ϣ
    private Set<String> messageType = new HashSet<String>();
    

    
    private NotifyMessageBus() {
    	
    	
    	//---ⶩ---
		messageType.add("Subscribe"); 
		messageType.add("SubscribeResponse");
		
		messageType.add("Unsubscribe");
		messageType.add("UnsubscribeResponse");
		
		messageType.add("Renew");
		messageType.add("RenewResponse");
		
		messageType.add("PauseSubscription");
		messageType.add("PauseSubscriptionResponse");
		
		messageType.add("ResumeSubscription");
		messageType.add("ResumeSubscriptionResponse");
		
		
		
		//---ø֪ͨϢ---
		messageType.add("GetCurrentMessage");
		messageType.add("GetCurrentMessageResponse");

		//---֪ͨϢ---
		messageType.add("Notify");
		
		//---עᷢ---
		messageType.add("RegisterPublisher");
		messageType.add("RegisterPublisherResponse");
		
		messageType.add("DestroyRegistration");
		messageType.add("DestroyRegistrationResponse");
		
		
		//---б---
		messageType.add("GetTopicList");
		
		
		//---Pullpoint---
		messageType.add("DestroyPullPoint");
		messageType.add("CreatePullPoint");
		messageType.add("GetMessages");

	}
    
    public static NotifyMessageBus getNotifyMessageBus() {
        return INSTANCE;
    }
    
    
    /**
     * Ƿ֪ͨЭֵ֧Ϣ
     * @param Name
     * @return
     */
    public boolean isNotificationMessage(String name) {
        return messageType.contains(name);
    }

    
    
    
    //---------------------------------
    /**
     * Ϣ֪ͨߴ
     * 
     * @param requestMessage
     * @param webServiceName
     * @param methodName
     * @param publisher
     * @return
     */
    public Metaset forward2Publisher(Metaset requestMessage,String webServiceName, 
    		String methodName, NotificationPublisher publisher){
    	
    	//߽ɫ κϢ
    	NotificationBaseException excep = 
    		new NotificationBaseException("UnSupportedOperation",Calendar.getInstance(),
    				"The Operation "+methodName+" is unsupported for publisher");
    	
    	return MessageFactory.getExceptionResponseMessage(excep).toMetaset();
    	
    }
    
    /**
     *  Ϣ֪ͨߴ
     *  ֵ֧ϢУ
     *  SubscribeUnsubscribeRenewPauseSubscriptionResumeSubscription
     *  GetCurrentMessageGetTopicList
     *  
     * @param requestMessage
     * @param webServiceName
     * @param methodName
     * @param producer
     * @return
     */
    public Metaset forward2Producer(Metaset requestMessage,String webServiceName, 
    		String methodName, NotificationProducer producer){
		
    	
    	if (methodName.equals("Subscribe")) {
			// Ϣ
    		Message response = producer.handleSubscribe(MessageFactory.getInputMessage(requestMessage));
			return response.toMetaset();

		}else if(methodName.equals("Unsubscribe")){
    		//ȡ Ϣ
			Message response = producer.handleUnSubscribe(MessageFactory.getInputMessage(requestMessage));
			return response.toMetaset();
			
		}else if(methodName.equals("Renew")){
    		//ĸ Ϣ
			Message response = producer.handleRenew(MessageFactory.getInputMessage(requestMessage));
			return response.toMetaset();
			
		}else if(methodName.equals("PauseSubscription")){
    		//ͣ Ϣ
			Message response = producer.handlePause(MessageFactory.getInputMessage(requestMessage));
			return response.toMetaset();
			
		}else if(methodName.equals("ResumeSubscription")){
    		//ظ Ϣ
			Message response = producer.handleResume(MessageFactory.getInputMessage(requestMessage));
			return response.toMetaset();
			
		}else if(methodName.equals("GetCurrentMessage")){
    		//֪ͨ Ϣ
			Message response = producer.GetCurrentMessage(MessageFactory.getInputMessage(requestMessage));
			return response.toMetaset();
			
		}else if (methodName.equals("GetTopicList")){
    		// Ϣ
			Message response = producer.getTopicTreesSet().getTopicListResponse();
			return response.toMetaset();
			
		}else{
			//͵Ϣݲ֧
			return MessageFactory.getExceptionResponseMessage(
					new NotificationBaseException("UnsupportedOperation",Calendar.getInstance(),
							"NotificationProducer Wait for support the operation of " 
							+ methodName)).toMetaset();
			
		}
    }
    
    
    /**
     *  Ϣ֪ͨ
     *  ֵ֧ϢУ
     *  SubscribeUnsubscribeRenewPauseSubscriptionResumeSubscription
     *  GetCurrentMessageGetTopicList
     *  NotifyRegisterPublisherDestroyRegistration
     * 
     * @param requestMessage
     * @param webServiceName
     * @param methodName
     * @param broker
     * @return
     */
    public Metaset forward2Broker(Metaset requestMessage,String webServiceName, 
    		String methodName, NotificationBroker broker){
    	
    	if (methodName.equals("Subscribe")) {
			// Ϣ
    		Message response = broker.handleSubscribe(MessageFactory.getInputMessage(requestMessage));
			return response.toMetaset();

		}else if(methodName.equals("Unsubscribe")){
    		//ȡ Ϣ
			Message response = broker.handleUnSubscribe(MessageFactory.getInputMessage(requestMessage));
			return response.toMetaset();
			
		}else if(methodName.equals("Renew")){
    		//ĸ Ϣ
			Message response = broker.handleRenew(MessageFactory.getInputMessage(requestMessage));
			return response.toMetaset();
			
		}else if(methodName.equals("PauseSubscription")){
    		//ͣ Ϣ
			Message response = broker.handlePause(MessageFactory.getInputMessage(requestMessage));
			return response.toMetaset();
			
		}else if(methodName.equals("ResumeSubscription")){
    		//ظ Ϣ
			Message response = broker.handleResume(MessageFactory.getInputMessage(requestMessage));
			return response.toMetaset();
			
		}else if(methodName.equals("GetCurrentMessage")){
    		//֪ͨ Ϣ
			Message response = broker.GetCurrentMessage(MessageFactory.getInputMessage(requestMessage));
			return response.toMetaset();
			
		}else if (methodName.equals("GetTopicList")){
    		// Ϣ
			Message response = broker.getTopicTreesSet().getTopicListResponse();
			return response.toMetaset();
			
		}else if(methodName.equals("RegisterPublisher")){
			//ע֪ͨ Ϣ
			Message response = broker.handleRegisterPublisher(
					MessageFactory.getInputMessage(requestMessage));
			return response.toMetaset();
			
		}else if(methodName.equals("DestroyRegistration")){
			//֪ͨ Ϣ
			Message response = broker.handleDestroyRegistration(
					MessageFactory.getInputMessage(requestMessage));
			return response.toMetaset();
			
		}else if(methodName.equals("Notify")){
			//֪ͨ Ϣ
			System.out.println("---NotifyMessageBus.forward2Broker---Notify---");
			broker.handleNotification(requestMessage);
			return null; //ҪӦϢ
			
		}else{
			//͵Ϣݲ֧
			return MessageFactory.getExceptionResponseMessage(
					new NotificationBaseException("UnsupportedOperation",Calendar.getInstance(),
							"NotificationProducer Wait for support the operation of " 
							+ methodName)).toMetaset();
			
		}
    }
    
    /**
     * 
     * @param requestMessage
     * @param webServiceName
     * @param methodName
     * @param consumer
     * @return
     */
    public Metaset forward2Consumer(Metaset requestMessage,String webServiceName, 
    		String methodName, ANotificationConsumer consumer){
    	
    	//߽ɫ κϢ
    	NotificationBaseException excep = 
    		new NotificationBaseException("UnSupportedOperation",Calendar.getInstance(),
    				"The Operation "+methodName+" is wait for supported for Consumer");
    	
    	return MessageFactory.getExceptionResponseMessage(excep).toMetaset();
    }
    
    //---------------------------------------------
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    

 

    /**
	 * NotificationProducerҪǴϢʵϢҪתNotificationManager
	 * 
	 * @param requestServices
	 * @param webServiceName
	 * @param methodName
	 * @return
	 */
//
// һrequest
// POST /pubscribe/services/filesystem HTTP/1.1Content-Length: 1289Content-Type:
// text/xml; charset=UTF-8SOAPAction: User-Agent: Java/1.5.0_06Host:
// 127.0.0.1:8081Accept: text/html, image/gif, image/jpeg, *; q=.2, */*;
// q=.2Connection: keep-alive
//<Envelope xmlns="http://schemas.xmlsoap.org/soap/envelope/"      
//    xmlns:fs="http://ws.apache.org/resource/example/filesystem"    
//    xmlns:wsa="http://schemas.xmlsoap.org/ws/2003/03/addressing"   
//    xmlns:wsnt="http://docs.oasis-open.org/wsn/2004/06/wsn-WS-BaseNotification-1.2-draft-01.xsd"          >   
//   <Header>      
//      <wsa:To mustUnderstand="1">http://localhost:8080/pubscribe/services/filesystem</wsa:To>      
//      <wsa:Action mustUnderstand="1">http://ws.apache.org/resource/example/filesystem/FileSystemPortType/yourWsdlRequestName</wsa:Action>      
//      <fs:ResourceIdentifier mustUnderstand="1">/dev/vg00/lvol1</fs:ResourceIdentifier>   
//   </Header>   
//   <Body>      
//      <wsnt:Subscribe>                                           
//         <wsnt:ConsumerReference>            
//            <!-- make this filesystem for now... -->            
//               <wsa:Address>http://localhost:8080/pubscribe/services/filesystem</wsa:Address>       
//               <wsa:ReferenceProperties/>                            
//            </wsnt:ConsumerReference>                  
//            <wsnt:TopicExpression Dialect="http://docs.oasis-open.org/wsn/2004/06/TopicExpression/Simple">fs:MountPointDirectory</wsnt:TopicExpression>                  
//            <wsnt:UseNotify>true</wsnt:UseNotify>                
//         </wsnt:Subscribe>   
//      </Body>
//   </Envelope>
//
//response
//HTTP/1.1 200 OKContent-Type: text/xml;charset=utf-8Transfer-Encoding: chunkedDate: Fri, 30 Jun 2006 05:43:35 GMTServer: Apache-Coyote/1.1740<?xml version="1.0" encoding="UTF-8"?>
//   <soapenv:Envelope xmlns:soapenv="http://schemas.xmlsoap.org/soap/envelope/" xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"> 
//      <soapenv:Header>  
//         <wsa:Action soapenv:actor="http://schemas.xmlsoap.org/soap/actor/next" soapenv:mustUnderstand="0" xmlns:wsa="http://schemas.xmlsoap.org/ws/2003/03/addressing">http://schemas.xmlsoap.org/ws/2003/03/addressing/anonymous</wsa:Action>  
//         <wsa:To soapenv:actor="http://schemas.xmlsoap.org/soap/actor/next" soapenv:mustUnderstand="0" xmlns:wsa="http://schemas.xmlsoap.org/ws/2003/03/addressing">http://schemas.xmlsoap.org/ws/2003/03/addressing/anonymous</wsa:To> 
//      </soapenv:Header> 
//      <soapenv:Body>  
//         <wsn:SubscribeResponse xmlns:wsn="http://docs.oasis-open.org/wsn/2004/06/wsn-WS-BaseNotification-1.2-draft-01.xsd">   
//            <wsn:SubscriptionReference>    
//               <add:Address xmlns:add="http://schemas.xmlsoap.org/ws/2003/03/addressing">http://192.168.2.65:8080/pubscribe/services/SubscriptionManager</add:Address>    
//               <add:ReferenceProperties xmlns:add="http://schemas.xmlsoap.org/ws/2003/03/addressing">     
//                  <sub:ResourceIdentifier xmlns:sub="http://ws.apache.org/notification/base/service/SubscriptionManager">ec88efd2-6aac-4027-9fa9-7b55a5b8f96b</sub:ResourceIdentifier>    
//               </add:ReferenceProperties>    
//               <add:PortType xmlns:add="http://schemas.xmlsoap.org/ws/2003/03/addressing" xmlns:sub="http://ws.apache.org/notification/base/service/SubscriptionManager">sub:SubscriptionManagerPortType</add:PortType>    
//               <add:ServiceName PortName="SubscriptionManager" xmlns:add="http://schemas.xmlsoap.org/ws/2003/03/addressing" xmlns:sub="http://ws.apache.org/notification/base/service/SubscriptionManager">sub:SubscriptionManager</add:ServiceName>   
//            </wsn:SubscriptionReference>  
//         </wsn:SubscribeResponse> 
//      </soapenv:Body>
//   </soapenv:Envelope>0
    
    
   //getcurrent Message...
//    OST /pubscribe/services/filesystem HTTP/1.1Content-Length: 886Content-Type: text/xml; charset=UTF-8SOAPAction: User-Agent: Java/1.5.0_06Host: 127.0.0.1:8081Accept: text/html, image/gif, image/jpeg, *; q=.2, */*; q=.2Connection: keep-alive<Envelope xmlns="http://schemas.xmlsoap.org/soap/envelope/"           xmlns:fs="http://ws.apache.org/resource/example/filesystem"          xmlns:wsnt="http://docs.oasis-open.org/wsn/2004/06/wsn-WS-BaseNotification-1.2-draft-01.xsd">   
//    <Header xmlns:wsa="http://schemas.xmlsoap.org/ws/2004/03/addressing">      
//       <wsa:To mustUnderstand="1">http://localhost:8080/pubscribe/services/filesystem</wsa:To>      
//       <wsa:Action mustUnderstand="1">http://ws.apache.org/resource/example/filesystem/FileSystemPortType/yourWsdlRequestName</wsa:Action>      
//       <fs:ResourceIdentifier mustUnderstand="1">/dev/vg00/lvol1</fs:ResourceIdentifier>   
//    </Header>   
//    <Body>      
//       <wsnt:GetCurrentMessage>         
//          <wsnt:Topic Dialect="http://docs.oasis-open.org/wsn/2004/06/TopicExpression/Simple">fs:MountPointDirectory</wsnt:Topic>      
//       </wsnt:GetCurrentMessage>   
//    </Body></Envelope>
//
// .response
// HTTP/1.1 200 OKContent-Type: text/xml;charset=utf-8Transfer-Encoding: chunkedDate: Fri, 30 Jun 2006 05:50:21 GMTServer: Apache-Coyote/1.160c<?xml version="1.0" encoding="UTF-8"?>
//    <soapenv:Envelope xmlns:soapenv="http://schemas.xmlsoap.org/soap/envelope/" xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"> 
//       <soapenv:Header>  
//          <wsa:Action soapenv:actor="http://schemas.xmlsoap.org/soap/actor/next" soapenv:mustUnderstand="0" xmlns:wsa="http://schemas.xmlsoap.org/ws/2004/03/addressing">http://schemas.xmlsoap.org/ws/2004/03/addressing/anonymous</wsa:Action>  
//          <wsa:To soapenv:actor="http://schemas.xmlsoap.org/soap/actor/next" soapenv:mustUnderstand="0" xmlns:wsa="http://schemas.xmlsoap.org/ws/2004/03/addressing">http://schemas.xmlsoap.org/ws/2004/03/addressing/anonymous</wsa:To> 
//       </soapenv:Header> 
//       <soapenv:Body>  
//          <wsn:GetCurrentMessageResponse xmlns:wsn="http://docs.oasis-open.org/wsn/2004/06/wsn-WS-BaseNotification-1.2-draft-01.xsd">   
//             <wsrf:ResourcePropertyValueChangeNotification xmlns:wsrf="http://docs.oasis-open.org/wsrf/2004/06/wsrf-WS-ResourceProperties-1.2-draft-01.xsd">    
//                <wsrf:OldValue>     
//                   <fil:MountPointDirectory xmlns:fil="http://ws.apache.org/resource/example/filesystem">/usr</fil:MountPointDirectory>    
//                </wsrf:OldValue>    
//                <wsrf:NewValue>     
//                   <fs:MountPointDirectory xmlns:fs="http://ws.apache.org/resource/example/filesystem" xmlns:wsrp="http://docs.oasis-open.org/wsrf/2004/06/wsrf-WS-ResourceProperties-1.2-draft-01.xsd">/var/tmp/foo</fs:MountPointDirectory>    
//                </wsrf:NewValue>   
//             </wsrf:ResourcePropertyValueChangeNotification>  
//          </wsn:GetCurrentMessageResponse> 
//       </soapenv:Body>
//    </soapenv:Envelope>
    

    
    //---------------------------------------------------
	/**
	 * һӦϢFutureTask
	 * 
	 * @param metaset
	 * @param ipAddress
	 * @return
	 */
	public FutureTask<Metaset> sendMessageWithResponse(Metaset metaset, String ipAddress) {

		class clientWithRes implements Callable<Metaset> {
			
			Metaset metaset = null;
			String ipaddr = null;

			public clientWithRes(Metaset metaset, String ipaddr) {
				this.metaset = metaset;
				this.ipaddr = ipaddr;
			}

			public Metaset call() {
				System.err.println("---sendMessageWithResponse()---Ϣ͵" + ipaddr);//////
				
				HttpCallClient client = new HttpCallClient();
				Metaset meta = client.call(ipaddr, metaset);  //ϢӦ
				return meta; //ӦϢ
			}
		}
		
		FutureTask<Metaset> task = new FutureTask<Metaset>(new clientWithRes(metaset, ipAddress));
		
		Thread thread = new Thread(task);
		thread.start();
		
		return task; //Ϣtask
	}
	
	
	/**
	 * Զ̶˵㷢һϢӦϢ
	 * @param metaset
	 * @param ipAddress
	 */
	public void sendMessageWithoutResponse(Metaset metaset, String ipAddress){
		
		class client implements Runnable {
			Metaset metaset = null;
			String ipaddr = null;

			public client(Metaset metaset, String ipaddr) {
				this.metaset = metaset;
				this.ipaddr = ipaddr;
			}

			public void run() {
				System.err.println("---NotifyMessageBus.sendMessageWithoutResponse()---Ϣ͵" + ipaddr);//////
				System.err.println("---NotifyMessageBus.sendMessageWithoutResponse()---\n" + metaset.toFormatXML());//////
				
				HttpCallClient client = new HttpCallClient();
				
				client.callWithoutResponse(ipaddr, metaset); //Ϣ
				
			}
		}
		
		Thread thread = new Thread(new client(metaset, ipAddress));
		thread.start();
		
	}

}
