package org.act.xservices.rt.xnotification.base.impl;

import java.util.concurrent.ExecutionException;
import java.util.concurrent.FutureTask;

import org.act.xservices.rt.xnotification.NotifyMessageBus;
import org.act.xservices.rt.xnotification.base.INotificationSubscriber;
import org.act.xservices.rt.xnotification.message.base.Message;
import org.act.xservices.rt.xnotification.message.base.MessageFactory;
import org.act.xservices.rt.xom.Metaset;

/**
 * 
 * @version 1.0 2008-3-24
 * @author Lizq [lizq@act.buaa.edu.cn]
 * @since JDK1.5
 */
public class NotificationSubscriber implements INotificationSubscriber{
	
	
	/** 
	 * 
	 * @param topicExpressions
	 * @param subManagerAddress
	 * @param consumerAddress
	 * @return
	 * @see org.act.xservices.rt.xnotification.base.INotificationSubscriber#subscribe(org.act.xservices.rt.metaset.Metaset[], java.lang.String, java.lang.String)
	 */
	public Metaset subscribe(Metaset[] topicExpressions, String subManagerAddress, String consumerAddress) {

		Message subReqMessage = 
			MessageFactory.getSubscribeRequestMessage(topicExpressions, consumerAddress);
		
		return this.subscribe(subReqMessage.toMetaset(), subManagerAddress);
	}

	
	
	/** 
	 * 
	 * @param subscribeSOAPMessage
	 * @param subManagerAddress
	 * @return
	 * @see org.act.xservices.rt.xnotification.base.INotificationSubscriber#subscribe(org.act.xservices.rt.metaset.Metaset, java.lang.String)
	 */
	public Metaset subscribe(Metaset subscribeSOAPMessage, String subManagerAddress) {
		
		FutureTask<Metaset> task = NotifyMessageBus.getNotifyMessageBus().sendMessageWithResponse(
				subscribeSOAPMessage, subManagerAddress);
		
		try {
			System.out.println("---NotificationSubscriber.subscribe()--Response--\n"+ task.get().toXML());
			return task.get();
		} catch (InterruptedException e) {
			e.printStackTrace();
		} catch (ExecutionException e) {
			e.printStackTrace();
		}
		return null;
	}



	public Metaset UnSubscribe(Metaset[] topicExpressions, String subManagerAddress, String consumerAddress) {
		
		Message unsubReqMessage = 
			MessageFactory.getUnsubscribeRequestMessage(topicExpressions, consumerAddress);
		
		return this.subscribe(unsubReqMessage.toMetaset(), subManagerAddress);
	}



	public Metaset UnSubscribe(Metaset unSubscribeSOAPMessage, String subManagerAddress) {
		
		FutureTask<Metaset> task = NotifyMessageBus.getNotifyMessageBus().sendMessageWithResponse(
				unSubscribeSOAPMessage, subManagerAddress);
		
		try {
			System.out.println("---NotificationSubscriber.UnSubscribe()--Response--\n"+ task.get().toXML());
			return task.get();
		} catch (InterruptedException e) {
			e.printStackTrace();
		} catch (ExecutionException e) {
			e.printStackTrace();
		}
		return null;
	}
	
	
	
	/**
	 * 
	 * <s:Envelope xmlns="http://schemas.xmlsoap.org/soap/envelope/"
	 * xmlns:s="http://schemas.xmlsoap.org/soap/envelope/"
	 * xmlns:fs="http://ws.apache.org/resource/example/filesystem"
	 * xmlns:wsa="http://schemas.xmlsoap.org/ws/2003/03/addressing"
	 * xmlns:wsnt="http://docs.oasis-open.org/wsn/2004/06/wsn-WS-BaseNotification-1.2-draft-01.xsd" >
	 * 
	 * <s:Header> <wsa:To
	 * mustUnderstand="1">http://localhost:8080/pubscribe/services/filesystem</wsa:To>
	 * <wsa:Action
	 * mustUnderstand="1">http://ws.apache.org/resource/example/filesystem/FileSystemPortType/yourWsdlRequestName</wsa:Action>
	 * <fs:ResourceIdentifier mustUnderstand="1">/dev/vg00/lvol1</fs:ResourceIdentifier>
	 * </s:Header>
	 * 
	 * <s:Body>
	 * 
	 * <wsnt:Subscribe> 
	 * 
	 * <wsnt:ConsumerReference>
	 * <wsa:Address>http://localhost:8090/ConsumerService</wsa:Address>
	 * </wsnt:ConsumerReference> 
	 * 
	 * <wsnt:Filter> 
	 * <wsnt:TopicExpression Dialect=
	 * "http://docs.oasis-open.org/wsn/t-1/TopicExpression/Simple">fs:MountPointDirectory
	 * </wsnt:TopicExpression>
	 * 
	 * <wsnt:MessageContent Dialect="http://www.w3.org/TR/1999/REC-xpath-19991116">
	 * boolean(ncex:Producer="15") 
	 * </wsnt:MessageContent> 
	 * </wsnt:Filter>
	 * 
	 * <wsnt:InitialTerminationTime> 2005-12-25T00:00:00.00000Z
	 * </wsnt:InitialTerminationTime> 
	 * 
	 * </wsnt:Subscribe>
	 * 
	 * </s:Body> 
	 * </s:Envelope>
	 */
	

}
