package org.act.xservices.rt.xnotification.base.impl;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.act.xservices.rt.xnotification.base.IPublisherTopicSet;
import org.act.xservices.rt.xnotification.base.ITopicTreesSet;
import org.act.xservices.rt.xnotification.base.fault.InvalidTopicExpressionException;
import org.act.xservices.rt.xnotification.base.fault.TopicNotSupportedException;
import org.act.xservices.rt.xnotification.topic.ITopic;
import org.act.xservices.rt.xnotification.topic.exception.TopicExpressionException;
import org.act.xservices.rt.xnotification.topic.exception.UnsupportedTopicExpressionException;
import org.act.xservices.rt.xnotification.topic.expression.ITopicExpression;
import org.act.xservices.rt.xnotification.topic.expression.TopicCompileEngine;
import org.act.xservices.rt.xnotification.topic.expression.TopicExpression;

/**
 * ⼯ʵ
 * 
 * @version 1.0 2007-7-31
 * @author cover_eye [wangyg@act.buaa.edu.cn]
 * @author Lizq [lizq@act.buaa.edu.cn]
 * @since JDK1.5
 */
public class PublisherTopicSet implements IPublisherTopicSet {


	/**άһNotificationPublisherֵ֧Topic */
	private final Set<ITopic> topicSet = new HashSet<ITopic>();
	
	
	/** 
	 * 
	 * @param topics
	 * @see org.act.xservices.rt.xnotification.base.IPublisherTopicSet#addTopics(org.act.xservices.rt.xnotification.topic.ITopic[])
	 */
	public void addTopics(ITopic[] topics) {
		for (int i = 0; i < topics.length; i++) {
		      topicSet.add(topics[i]);
		}
	}
	
	/** 
	 * 
	 * @param topic
	 * @see org.act.xservices.rt.xnotification.base.IPublisherTopicSet#addTopics(org.act.xservices.rt.xnotification.topic.ITopic[])
	 */
	public void addTopic(ITopic topic) {
		topicSet.add(topic);
	}
	
	
	/** 
	 * 
	 * @param topics
	 * @see org.act.xservices.rt.xnotification.base.IPublisherTopicSet#removeTopics(org.act.xservices.rt.xnotification.topic.ITopic[])
	 */
	public void removeTopics(ITopic[] topics){
		for (int i = 0; i < topics.length; i++) {
		      topicSet.remove(topics[i]);
		}
	}
	
	/** 
	 * ڸнʽȻӽ
	 * 
	 * @param topicExpr
	 * @throws TopicExpressionException
	 * @see org.act.xservices.rt.xnotification.base.IPublisherTopicSet#addTopicWithExpression(org.act.xservices.rt.xnotification.topic.expression.TopicExpression)
	 */
	public void addTopicWithExpression(TopicExpression topicExpr,ITopicTreesSet topicTreeSet)
			throws TopicExpressionException {
	    
		ITopic[] topics;
		try {
			topics = TopicCompileEngine.getTopicEngine().compile(topicExpr,topicTreeSet);
			for (int i = 0; i < topics.length; i++) {
			      topicSet.add(topics[i]);
			}
		} catch (InvalidTopicExpressionException e) {
			e.printStackTrace();
		} catch (TopicNotSupportedException e) {
			e.printStackTrace();
		} catch (UnsupportedTopicExpressionException e) {
			e.printStackTrace();
		}
	}


	/** 
	 * 
	 * @param topic
	 * @return
	 * @see org.act.xservices.rt.xnotification.base.IPublisherTopicSet#containsTopic(org.act.xservices.rt.xnotification.topic.ITopic)
	 */
	public boolean containsTopic(ITopic topic) {
		return topicSet.contains(topic);
	}


	/**
	 * ʽָģұֵ֪֧ͨ
	 * @param topicExpr
	 * @return
	 * @throws TopicExpressionException
	 * @see org.act.xservices.rt.xnotification.base.IPublisherTopicSet#evaluateTopicExpression(org.act.xservices.rt.xnotification.topic.expression.TopicExpression)
	 */
	public ITopic[] evaluateTopicExpression(ITopicExpression topicExpr, ITopicTreesSet topicTreeSet)
			throws TopicExpressionException {
		ITopic[] topics;
		try {
			topics = 
				TopicCompileEngine.getTopicEngine().compile(topicExpr, topicTreeSet);
			return getSupportedTopics(topics);
		} catch (InvalidTopicExpressionException e) {
			e.printStackTrace();
		} catch (TopicNotSupportedException e) {
			e.printStackTrace();
		} catch (UnsupportedTopicExpressionException e) {
			e.printStackTrace();
		}
		return null;
	}

	private ITopic[] getSupportedTopics(ITopic[] topics) {
		List<ITopic> supportedTopics = new ArrayList<ITopic>();
		for (int i = 0; i < topics.length; i++) {
			ITopic topic = topics[i];
			if (containsTopic(topic)) {
				supportedTopics.add(topic);
			}
		}
		return supportedTopics.toArray(new ITopic[supportedTopics.size()]);
	}



	/** 
	 * طֵ֧
	 * @return
	 * @see org.act.xservices.rt.xnotification.base.IPublisherTopicSet#getTopics()
	 */
	public ITopic[] getAllTopics() {
		return topicSet.toArray(new ITopic[topicSet.size()]);
	}


	public boolean isFixed() {
		return false;
	}


	public void setFixed(boolean fixed) {
		
	}
	

	
	/** 
	 * 
	 * @return
	 * @see org.act.xservices.rt.xnotification.base.IPublisherTopicSet#isEmpty()
	 */
	public boolean isEmpty(){
		return (topicSet.size()==0);
	}
	
	

}
