package org.act.xservices.rt.xnotification.base.impl;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import org.act.xservices.rt.xcontainer.addressing.IEndpointReference;
import org.act.xservices.rt.xcontainer.addressing.impl.EndpointReference;
import org.act.xservices.rt.xcontainer.addressing.impl.MessageAddressingProperties;
import org.act.xservices.rt.xnotification.NotificationUtil;
import org.act.xservices.rt.xnotification.base.IPublisherTopicSet;
import org.act.xservices.rt.xnotification.base.ISubscriptionContext;
import org.act.xservices.rt.xnotification.base.ISubscriptionManager;
import org.act.xservices.rt.xnotification.base.ITopicTreesSet;
import org.act.xservices.rt.xnotification.base.fault.InvalidTopicExpressionException;
import org.act.xservices.rt.xnotification.base.fault.NotificationBaseException;
import org.act.xservices.rt.xnotification.base.fault.SubscribeCreationFailedFault;
import org.act.xservices.rt.xnotification.base.fault.TopicNotSupportedException;
import org.act.xservices.rt.xnotification.message.base.Filter;
import org.act.xservices.rt.xnotification.message.base.Message;
import org.act.xservices.rt.xnotification.message.base.MessageFactory;
import org.act.xservices.rt.xnotification.topic.ITopic;
import org.act.xservices.rt.xnotification.topic.exception.TopicExpressionException;
import org.act.xservices.rt.xnotification.topic.exception.UnsupportedTopicExpressionException;
import org.act.xservices.rt.xnotification.topic.expression.ITopicExpression;
import org.act.xservices.rt.xnotification.topic.expression.TopicCompileEngine;
import org.act.xservices.rt.xom.Metaset;


/**
 * ⶩĹ
 * 
 * @version 1.0 2008-3-14
 * @author Lizq [lizq@act.buaa.edu.cn]
 * @since JDK1.5
 */
public class SubscriptionManager implements ISubscriptionManager {

	
	/** */
	private ITopicTreesSet topicTreesSet;
	
	/**⼯ */
	private IPublisherTopicSet publisherTopicSet;
	
	
	/**˶ĹߵĶ˵ */
	protected IEndpointReference subscriptionManagerEPR;
	
	
	
	/**
	 * άTopicSubscriptionбĶӦ
	 */
	private final Map<ITopic, List<ISubscriptionContext>> topic2SubscriptionListMap = 
		new ConcurrentHashMap<ITopic, List<ISubscriptionContext>>();
	
	
	
	
	
	public SubscriptionManager(IEndpointReference subscriptionManagerEPR, 
			ITopicTreesSet topicTreesSet,IPublisherTopicSet publisherTopicSet){
		this.subscriptionManagerEPR = subscriptionManagerEPR;
		this.topicTreesSet = topicTreesSet;
		this.publisherTopicSet = publisherTopicSet;
	}
	
	

	public IEndpointReference getEPR() {
		return subscriptionManagerEPR;
	}
	

	

	
	//---δʵ
	public Message handlePause(Message pauseMessage) {
		// TODO Auto-generated method stub
		return MessageFactory.getExceptionResponseMessage(
				new NotificationBaseException("UnsupportedOperation",
						Calendar.getInstance(),
						"Wait for support the operation of Pause"));
	}
	
	//---δʵ
	public Message handleRenew(Message renewMessage) {
		// TODO Auto-generated method stub
		return MessageFactory.getExceptionResponseMessage(
				new NotificationBaseException("UnsupportedOperation",
						Calendar.getInstance(),
						"Wait for support the operation of Renew"));
	}
	
	//---δʵ
	public Message handleResume(Message resumeMessage) {
		// TODO Auto-generated method stub
		return MessageFactory.getExceptionResponseMessage(
				new NotificationBaseException("UnsupportedOperation",
						Calendar.getInstance(),
						"Wait for support the operation of Resume"));
	}

	

	/**
	 *  Ϣ
	 * 
	 * @param subscriptionMessage
	 * @return ӦϢ
	 */
	public Message handleSubscribe(Message subscriptionMessage) {
        
		ISubscriptionContext subContext=null;
		
		try {
			subContext = new SubscriptionContext(subscriptionMessage);
		} catch (InvalidTopicExpressionException e1) {
			e1.printStackTrace();
			return MessageFactory.getExceptionResponseMessage(e1);
		}

        if (subContext.validate() == false) {
            // ǴĽģôֵΪfalse
            return MessageFactory.getExceptionResponseMessage(new SubscribeCreationFailedFault(
                    "SubscribeCreationFailedFault", 
                    Calendar.getInstance(), 
                    "SubscribeCreationFailedFault"));
        }
        
        ArrayList<ITopic> topicsForSub = new ArrayList<ITopic>();

        try {
        	//νÿʽ
        	for(ITopicExpression topExpTemp : subContext.getFilters().getTopicExpressions()){
        		//ҵ⣬鿴Ƿ⼯
        		ITopic[] topicsT = publisherTopicSet.evaluateTopicExpression(topExpTemp, topicTreesSet);
        		
        		for(ITopic top : topicsT)
        			topicsForSub.add(top); //ӵ
        	}
          
        } catch (InvalidTopicExpressionException e) {
			e.printStackTrace();
			return MessageFactory.getExceptionResponseMessage(e);

		} catch (TopicNotSupportedException e) {
			e.printStackTrace();
			return MessageFactory.getExceptionResponseMessage(e);

		} catch (TopicExpressionException e) {
			e.printStackTrace();
		}
      

		try {
			System.out.println("SubscriptionManager.subscribe():topic "
					+ topicsForSub.toString() + "Context:" + subContext);
			
			//ÿĶбӴ˶
			this.addSubscription(subContext, topicsForSub.toArray(new ITopic[0]));
			
		} catch (TopicNotSupportedException e) {
			e.printStackTrace();
			return MessageFactory.getExceptionResponseMessage(e);
		}

		
		//ӦϢͨء(IN-OUTģʽ)
		Metaset body = NotificationUtil.getMetasetOfPrefix_WSNT("SubscribeResponse");
		
		//ӶĹߵĶ˵
		Metaset smEPRMeta = this.subscriptionManagerEPR.toMetaset();
		
		Metaset subResponse = NotificationUtil.getMetasetOfPrefix_WSNT("SubscriptionReference");
		
		List list = smEPRMeta.elements();
		for (int i = 0; i < list.size(); i++) {
			subResponse.add((Metaset) list.get(i));
		}

		body.add(subResponse);
		
		// <wsnt:SubscribeResponse>
		//   <wsnt:SubscriptionReference> 760
		//     <wsa:Address> 761
		//       http://www.example.org/SubscriptionManager 762
		//     </wsa:Address> 763
		//   </wsnt:SubscriptionReference> 764
		// </wsnt:SubscribeResponse>

		Message message = MessageFactory.getSubscribeResponseMessage(body);
		
		//---
		if(subContext.getMessageAddressingProperties().getReplyTo() != null){
			Metaset to = NotificationUtil.getMetasetOfPrefix_WSA("To");
			to.setText(subContext.getMessageAddressingProperties().getReplyTo().getAddress());
			message.getHeader().add(to);
			System.out.println("---SubscriptionManager.handleSubscribe()---\n"+message.toMetaset().toXML());
		}
		
		return message;
	}


	
	
	/**
	 * һ
	 * 
	 * @param subscribe
	 *            һ
	 * @param topics
	 *            飬ڴÿĶбһʾĶ
	 * @throws TopicNotSupportedException
	 */
	private void addSubscription(ISubscriptionContext subscribe,
			ITopic[] topics) throws TopicNotSupportedException {

		List<ISubscriptionContext> subscriptionlist = null;

		for (int i = 0; i < topics.length; i++) {

			subscriptionlist = 
				(List<ISubscriptionContext>) topic2SubscriptionListMap.get(topics[i]);

			if (subscriptionlist != null) {
				subscriptionlist.add(subscribe);
				System.out.println("ڴ˶ĶеĳΪ: " + subscriptionlist.size());
			} else {
				List<ISubscriptionContext> sub = new LinkedList<ISubscriptionContext>();
				sub.add(subscribe);

				System.out.println("ڴ˶ĶеĳΪ: " + sub.size());
				topic2SubscriptionListMap.put(topics[i], sub);
			}
		}

	}
	
	
	/**
	 *  ȡϢ
	 * 
	 * @param unSubscribeMessage
	 * @return ӦϢ
	 * @see org.act.xservices.rt.xnotification.base.ISubscriptionManager#handleUnSubscribe(org.act.xservices.rt.xnotification.message.base.Message)
	 */
	public Message handleUnSubscribe(Message unSubscribeMessage) {

		MessageAddressingProperties mih = new MessageAddressingProperties(unSubscribeMessage.getHeader());
		
		IEndpointReference consumerEPR = null;
		Filter ft = null;
		
		Metaset message = unSubscribeMessage.getBody().filter(".\\Unsubscribe").car();

		if (message != null) {
			Metaset consumer = message.filter(".\\ConsumerReference");
			if (consumer != null) {
				consumerEPR = new EndpointReference(consumer.car());
			}

			if (message.filter(".\\Filter") != null) {
				// this is special...
				try {
					ft = new Filter(unSubscribeMessage);
				} catch (InvalidTopicExpressionException e) {
					e.printStackTrace();
				}
			}
		}
		
		//ȡϢ漰
        ArrayList<ITopic> topicsForUnSub = new ArrayList<ITopic>();
		try {
        	//νÿʽ
        	for(ITopicExpression topExpTemp : ft.getTopicExpressions()){
        		ITopic[] topicsT = 
        			TopicCompileEngine.getTopicEngine().compile(topExpTemp, topicTreesSet);
        		
        		for(ITopic top : topicsT)
        			topicsForUnSub.add(top); //ӵ
        	}
		} catch (UnsupportedTopicExpressionException e) {
			e.printStackTrace();
			return MessageFactory.getExceptionResponseMessage(
					new TopicNotSupportedException(
							"TopicNotSupportedFault",Calendar.getInstance(),"TopicNotSupportedFault"));
		} catch (InvalidTopicExpressionException e) {
			e.printStackTrace();
			return MessageFactory.getExceptionResponseMessage(e);
		} catch (TopicNotSupportedException e) {
			e.printStackTrace();
			return MessageFactory.getExceptionResponseMessage(e);
		}

		
		boolean[] successfulRemoveFlag = new boolean[topicsForUnSub.size()];
		
		for (int i=0; i< topicsForUnSub.size(); i++) {	
			//ôӦĶĶ
			List<ISubscriptionContext> subscriptionlist = 
				(List<ISubscriptionContext>) topic2SubscriptionListMap.get(topicsForUnSub.get(i));
			
			if(subscriptionlist == null){//ûдصĶļ¼
				continue;
			}
			//ӶĶвҴ֪ͨ߶ӦĶļ¼
			for (int num = 0; num < subscriptionlist.size(); num++) {
				
				if (subscriptionlist.get(num).getNotificationConsumer().
						getAddress().equals(consumerEPR.getAddress())) {
					//ҵɾμ¼ע⣺ֻɾһΣʹظļ¼
					subscriptionlist.remove(num);
					successfulRemoveFlag[i] = true;
					break;
				}//end of if
			}//end of for
			
		}//end of for 
		
		//ռ޷
		StringBuffer failedTopics = new StringBuffer();
		for(int a=0; a<successfulRemoveFlag.length; a++){
			if(!successfulRemoveFlag[a]){
				failedTopics.append(topicsForUnSub.get(a).toString()+";");
			}
		}
		
		
		//ӦϢͨء(IN-OUTģʽ)
		Metaset body = NotificationUtil.getMetasetOfPrefix_WSNT("UnsubscribeResponse");
		
		if (failedTopics.length() == 0) {
			body.add(message.filter(".\\Filter").car());
		}else {
			body.setText("Can not find the subscription of the consumer on topics of "+ 
					failedTopics.toString());
		}
		
		Message unsubscribeResponse = MessageFactory.getUnsubscribeResponseMessage(body);
		
		//---
		if(mih.getReplyTo() != null){
			Metaset to = NotificationUtil.getMetasetOfPrefix_WSA("To");
			to.setText(mih.getReplyTo().getAddress());
			unsubscribeResponse.getHeader().add(to);
			System.out.println("---SubscriptionManager.handleUnSubscribe()---\n"+unsubscribeResponse.toMetaset().toXML());
		}
		
		return unsubscribeResponse;
	}


//	/**
//	 * ɾһ
//	 * 
//	 * @param subscribe һ
//	 * @param topics 飬ڴÿĶбɾһʾĶ
//	 * @return
//	 * @throws UnsupportedTopicExpressionException
//	 * @throws InvalidTopicExpressionException
//	 * @throws TopicNotSupportedException
//	 * @see org.act.ws.notification.base.ISubscriptionManager#removeSubscription(org.act.ws.notification.base.ISubscriptionContext, org.act.ws.notification.topic.ITopic[])
//	 */
//	private boolean removeSubscription(ISubscriptionContext subscribe,
//			ITopic[] topics) throws UnsupportedTopicExpressionException,
//			InvalidTopicExpressionException, TopicNotSupportedException {
//		
//		//ÿ
//		for (int i = 0; i < topics.length; i++) {
//			//ôӦĶб
//			List<ISubscriptionContext> subscriptionlist = 
//				(List<ISubscriptionContext>) topic2SubscriptionListMap.get(topics[i]);
//			
//			//Ӷбɾ˶ģûд˶ļ¼򱨴
//			
//			if(! subscriptionlist.remove(subscribe)){
//				throw new TopicNotSupportedException(
//						"TopicNotSupportedException",Calendar.getInstance(),"No such subscription!");
//			}
//		}//end of for
//		
//		return true;
//	}

	
	/** 
	 * ɾصж
	 * 
	 * @param topic
	 * @see org.act.xservices.rt.xnotification.base.ISubscriptionManager#removeSubscritionByTopic(org.act.xservices.rt.xnotification.topic.ITopic)
	 */
	@SuppressWarnings("unused")
	private void removeSubscritionByTopic(ITopic topic) {
		topic2SubscriptionListMap.remove(topic);
	}
	
	
	/** 
	 * ӦĶб
	 * עⷵصǰװ飬иĲӰԭб
	 * @param topic
	 * @return
	 * @see org.act.xservices.rt.xnotification.base.ISubscriptionManager#getSubscription(org.act.xservices.rt.xnotification.topic.ITopic)
	 */
	public ISubscriptionContext[] getSubscription(ITopic topic) {
		List<ISubscriptionContext> subscriptionlist = 
			(List<ISubscriptionContext>) topic2SubscriptionListMap.get(topic);

		if (subscriptionlist != null) {
			return subscriptionlist
					.toArray(new ISubscriptionContext[subscriptionlist.size()]);
		} else {
			return new ISubscriptionContext[0];
		}
	}

}
