package org.act.xservices.rt.xnotification.broker.impl;

import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.act.xservices.rt.xcontainer.addressing.IEndpointReference;
import org.act.xservices.rt.xnotification.NotificationConstants;
import org.act.xservices.rt.xnotification.NotificationUtil;
import org.act.xservices.rt.xnotification.WSNRuntime;
import org.act.xservices.rt.xnotification.base.INotificationSubscriber;
import org.act.xservices.rt.xnotification.base.ISubscriptionContext;
import org.act.xservices.rt.xnotification.base.fault.InvalidTopicExpressionException;
import org.act.xservices.rt.xnotification.base.fault.TopicNotSupportedException;
import org.act.xservices.rt.xnotification.base.impl.NotificationProducer;
import org.act.xservices.rt.xnotification.broker.INotificationBroker;
import org.act.xservices.rt.xnotification.broker.IPublisherRegistrationManager;
import org.act.xservices.rt.xnotification.message.base.Message;
import org.act.xservices.rt.xnotification.message.base.MessageFactory;
import org.act.xservices.rt.xnotification.message.base.NotifyMessage;
import org.act.xservices.rt.xnotification.message.base.NotifyMessage.NotificationMessage;
import org.act.xservices.rt.xnotification.topic.ITopic;
import org.act.xservices.rt.xnotification.topic.ITopicNamespace;
import org.act.xservices.rt.xnotification.topic.exception.UnsupportedTopicExpressionException;
import org.act.xservices.rt.xom.Metaset;
import org.act.xservices.rt.xom.QName;


/**
 * ֪ͨ
 * @version 1.0 2008-3-21
 * @author Lizq [lizq@act.buaa.edu.cn]
 * @since JDK1.5
 */

public class NotificationBroker extends NotificationProducer implements	INotificationBroker {
	
	
	/**߶Ĺ */
	private IPublisherRegistrationManager publisherRegistrationManager;
	
	/**ⶩ */
	private INotificationSubscriber notificationSubscriber;
	
	
	public NotificationBroker(IEndpointReference producerEPR) {
		super(producerEPR);
		publisherRegistrationManager = new PublisherRegistrationManager(this.publisherEPR,
				this.topicTreesSet, this.publisherTopicSet);
	}

	
	
	public Message handleRegisterPublisher(Message message) {
		// TODO Auto-generated method stub
		return publisherRegistrationManager.handleRegisterPublisher(message);
	}
	
	
	public Message handleDestroyRegistration(Message message) {
		// TODO Auto-generated method stub
		return publisherRegistrationManager.handleDestroyRegistration(message);
	}


	public Set<ITopic> getTopicsOfProducer(String producerEPR) {
		return publisherRegistrationManager.getTopicsOfProducer(producerEPR);
	}


	public IPublisherRegistrationManager getPublisherRegistrationManager() {
		return publisherRegistrationManager;
	}
	
	/** 
	 * ֪ͨ ֪ͨϢ
	 * Ҫת
	 * 
	 * @param notificationMessage
	 * @see org.act.xservices.rt.xnotification.base.INotificationConsumer#handleNotification(org.act.xservices.rt.metaset.Metaset)
	 */
	public void handleNotification(Metaset notifyMessageM) {
		
		System.out.println("---NotificationBroker.handleNotification()---\n"+notifyMessageM.toXML());
		
		Message inputMes = MessageFactory.getInputMessage(notifyMessageM);
		NotifyMessage notifyMessage = new NotifyMessage(inputMes);
		
		List<NotificationMessage> list = notifyMessage.getNotifycontext().getNotificationMessagelist();
		
		// NotifyCallBack
		for (Iterator<NotificationMessage> iter = list.iterator(); iter.hasNext();) {
			NotificationMessage notificationMessage = (NotificationMessage) iter.next();

			System.out.println("---NotificationBroker---handleNotification()---\n"+notificationMessage.toMetaset().toXML());
			
			//֪ͨϢдڸ
			if(notificationMessage.getTopicExpression() != null){
				//תÿ֪ͨϢ
				notifyCallBack(notificationMessage);
			}
		}
	}
	
	
	/**
	 * ֪ͨϢת˵
	 * @param notificationMessage
	 */
	private void notifyCallBack(NotificationMessage notificationMessage){
		
		try {
			//Brokerн֪ͨϢʽӦ
			ITopic[] supportedTopicsOfBroker = 
				WSNRuntime.getRuntime().getTopicEngine().compile(
						notificationMessage.getTopicExpression(), this.topicTreesSet);
			
			if(supportedTopicsOfBroker == null){
				System.err.println("---notifyCallBack()---Unsupported TopicExpress\n" +
						notificationMessage.getTopicExpression());
				return ;
			}
			
			Set<ITopic> supportedTopicsOfProducer = 
				publisherRegistrationManager.getTopicsOfProducer(
						notificationMessage.getProducerReference().getAddress());
			
			if(supportedTopicsOfProducer == null){
				System.err.println("---notifyCallBack()---Unsupported Producer\n" +
						notificationMessage.getProducerReference());
				return ;
			}
			
			for(ITopic topicTem : supportedTopicsOfBroker){
				//Ƿָ߶Ӧ
				if(supportedTopicsOfProducer.contains(topicTem)){
					//-----------------------------------------------
					
					//ע⣡˴Ҫ޸תϢĶĹַΪ֪ͨߵַ
					Metaset notiMessage = 
						NotificationUtil.getMetasetOfPrefix_WSNT("NotificationMessage");

					// ӶĹϢ
					Metaset subsciptionEPR = this.publisherEPR.toMetaset();//Ϊ֪ͨߵַ
					subsciptionEPR.setQName(new QName(
							NotificationConstants.URI_NS_WSNT,"SubscriptionReference","wsnt"));
					
					notiMessage.add(subsciptionEPR);

					
					//֪ͨϢӦϢ
					Metaset topicMetaset = NotificationUtil.getMetasetOfPrefix_WSNT("Topic");
					ITopicNamespace topicNS = topicTem.getTopicNamespace();
					topicMetaset.add(NotificationUtil.getNewAttribute(
							"Dialect", "http://docs.oasis-open.org/wsn/t-1/TopicExpression/Concrete"));
					topicMetaset.add(NotificationUtil.getNewAttribute(
							"xmlns:" + topicNS.getName(), topicNS.getTargetNamespace()));
					topicMetaset.setText(topicTem.toString());
					notiMessage.add(topicMetaset);
					
					// ֪ͨߵϢ
					Metaset producerEPR = notificationMessage.getProducerReference().toMetaset();
					producerEPR.setQName(new QName(
							NotificationConstants.URI_NS_WSNT,"ProducerReference","wsnt"));
					notiMessage.add(producerEPR);

					//֪ͨϢ
					notiMessage.add(notificationMessage.getMessageContent());
					
					
			        Metaset notify = NotificationUtil.getMetasetOfPrefix_WSNT("Notify");
			        notify.add(notiMessage);

			        Message message = MessageFactory.getNofitifyMessage(notify);
			        		        
			        //עϢ֪ͨ	
			        
					ISubscriptionContext[] subspContext = subscriptionManager.getSubscription(topicTem);
		            Metaset to = NotificationUtil.getMetasetOfPrefix_WSA("To");
		            message.getHeader().add(to);
		            
		            System.out.println("--notifyCallBack()-һҪ͵Ϣһ---" + subspContext.length);
			        
			        for (int i = 0; i < subspContext.length; i++) {
			 
			            String consumerAddress = subspContext[i].getNotificationConsumer().getAddress();
			            to.setText(consumerAddress);

			            //
			            WSNRuntime.getRuntime().getNotifyMessageBus().sendMessageWithoutResponse(
			            		message.toMetaset(), consumerAddress);
			        }//end of for
					
				}//end of if
			}

		} catch (InvalidTopicExpressionException e) {
			e.printStackTrace();
		} catch (TopicNotSupportedException e) {
			e.printStackTrace();
		} catch (UnsupportedTopicExpressionException e) {
			e.printStackTrace();
		}

	}




	//----дһʵ---add by lizq 2008-03-24

	/** 
	 * ߵݽⶩ
	 * 
	 * @param topicExpressionsM
	 * @param subManagerAddress
	 * @param consumerAddress
	 * @return
	 * @see org.act.xservices.rt.xnotification.base.INotificationSubscriber#subscribe(org.act.xservices.rt.metaset.Metaset[], java.lang.String, java.lang.String)
	 */
	public Metaset subscribe(Metaset[] topicExpressionsM, String subManagerAddress, String consumerAddress) {
		// TODO Auto-generated method stub
		return notificationSubscriber.subscribe(topicExpressionsM, subManagerAddress, consumerAddress);
	}



	public Metaset subscribe(Metaset subscribeSOAPMessage, String subManagerAddress) {
		// TODO Auto-generated method stub
		return notificationSubscriber.subscribe(subscribeSOAPMessage, subManagerAddress);
	}



	public Metaset UnSubscribe(Metaset[] topicExpressionsM, String subManagerAddress, String consumerAddress) {
		// TODO Auto-generated method stub
		return null;
	}



	public Metaset UnSubscribe(Metaset unSubscribeSOAPMessage, String subManagerAddress) {
		// TODO Auto-generated method stub
		return null;
	}

}
