package org.act.xservices.rt.xnotification.broker.impl;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.act.xservices.rt.xcontainer.addressing.IEndpointReference;
import org.act.xservices.rt.xcontainer.addressing.IMessageAddressingProperties;
import org.act.xservices.rt.xcontainer.addressing.impl.EndpointReference;
import org.act.xservices.rt.xcontainer.addressing.impl.MessageAddressingProperties;
import org.act.xservices.rt.xnotification.broker.IPublisherRegistrationContext;
import org.act.xservices.rt.xnotification.message.base.Message;
import org.act.xservices.rt.xnotification.topic.ITopic;
import org.act.xservices.rt.xnotification.topic.expression.ITopicExpression;
import org.act.xservices.rt.xnotification.topic.expression.TopicExpressionFactory;
import org.act.xservices.rt.xom.Metaset;




/**
 * ע¼
 * 
 * @version 1.0 2008-3-11
 * @author Lizq [lizq@act.buaa.edu.cn]
 * @since JDK1.5
 */
public class PublisherRegistrationContext implements IPublisherRegistrationContext {
	
	private IEndpointReference publisherReferenceEPR = null;

	private ITopicExpression[] topicExpressions = null;
	
	private Set<ITopic> topicsSet= new HashSet<ITopic>();

	private boolean demand = false;

	private Calendar initialTerminationTime = null;

	private IMessageAddressingProperties mesAP;
	
	
    
    
	
    /**
     * add by cover_eye:2007-7-27
     * ע󴴽עᷢ 
     * @param message
     */
    public PublisherRegistrationContext(Message message) {
        super();
        this.buildPublisherRegistrationContext(message);
    }    
	

    /**
     * @return  demand
     */
    public boolean isDemand() {
        return demand;
    }


    /**
     * @param demand Ҫõ demand
     */
    public void setDemand(boolean demand) {
        this.demand = demand;
    }
    
    
    /**
     * @return  initialTerminationTime
     */
    public Calendar getInitialTerminationTime() {
        return initialTerminationTime;
    }


    /**
     * @param initialTerminationTime Ҫõ initialTerminationTime
     */
    public void setInitialTerminationTime(Calendar initialTerminationTime) {
        this.initialTerminationTime = initialTerminationTime;
    }
    
    

    /**
     * @return  publisherReferenceEPR
     */
    public IEndpointReference getPublisherReferenceEPR() {
        return publisherReferenceEPR;
    }


    /**
     * @param publisherReferenceEPR Ҫõ publisherReferenceEPR
     */
    public void setPublisherReferenceEPR(IEndpointReference publisherReferenceEPR) {
        this.publisherReferenceEPR = publisherReferenceEPR;
    }
    
    
    /**
     * @return  topicExpression
     */
    public ITopicExpression[] getTopicExpressions() {
        return topicExpressions;
    }


    /**
     * @param topicExpression Ҫõ topicExpression
     */
    public void setTopicExpressions(ITopicExpression[] topicExpressions) {
        this.topicExpressions = topicExpressions;
    }
    
	public IMessageAddressingProperties getMesAP() {
		return mesAP;
	}
	
    
    private void setMessageAddressingProperties(
			IMessageAddressingProperties mesAP) {
		this.mesAP = mesAP;

	}
    
    
    /**
     * ߶
     * 
    <s:Envelope ...>
	<s:Header>
		<wsa:Action>
			http://docs.oasis-open.org/wsn/brw-2/RegisterPublisher/RegisterPublisherRequest
		</wsa:Action>
		...
	</s:Header>
	<s:Body>
		<wsn-br:RegisterPublisher>
			
			<wsn-br:PublisherReference>
				<wsa:Address>
					http://www.example.org/PublisherEndpoint
				</wsa:Address>
				<wsa:ReferenceParameters>
					<npex: NPResourceDisambiguator>
						uuid:84decd55-7d3f-65ad-ac44-675d9fce5d22
					</npex:NPResourceDisambiguator>
				</wsa:ReferenceParameters>
			</wsn-br:PublisherReference>
			
			<wsn-br:Topic
				Dialect="http://docs.oasis-open.org/wsn/t-1/TopicExpression/Simple">
				npex:SomeTopic
			</wsn-br:Topic>*
			
			<wsn-br:Demand>true</wsn-br:Demand>
			
			<wsn-br:InitialTerminationTime>
				2003-12-25T00:00:00.00000
			</wsn-br:InitialTerminationTime>
			
		</wsn-br:RegisterPublisher>
	</s:Body>
	</s:Envelope>  
    */
    
    
    public void buildPublisherRegistrationContext(Message registrerMessage) {

		MessageAddressingProperties messageAP = new MessageAddressingProperties(registrerMessage.getHeader());
		
		this.setMessageAddressingProperties(messageAP);

		Metaset message = registrerMessage.getBody().filter(".\\RegisterPublisher").car();

		if (message != null) {
			Metaset publisher = message.filter(".\\PublisherReference");
			if (publisher != null) {
				EndpointReference publisherEpr = new EndpointReference(publisher.car());
				this.setPublisherReferenceEPR(publisherEpr);
			}
			
			//ʽעϢпжעʽ
			Metaset topicExpressionsM = message.filter(".\\Topic");
			if (topicExpressionsM != null) {
				
				List<ITopicExpression> topicExpList = new ArrayList<ITopicExpression>();
				for(Object tp : topicExpressionsM.elements()){
					topicExpList.add( TopicExpressionFactory
						.getTopicExpression((Metaset)tp, registrerMessage.getEnvelopeNameSpace()) );
				}
				this.setTopicExpressions(topicExpList.toArray(new ITopicExpression[0]));
			}

			Metaset demandM = message.filter(".\\Demand");
			if (demandM != null) {
				// this is special...
				if (demandM.getText() == "false") {
					this.setDemand(false);
				} else {
					this.setDemand(true);
				}
			}
			
			//δʵֶʱĴ޸ġ add by lizq
			Metaset termin = message.filter(".\\InitialTerminationTime");
			if (termin != null) {
				//FIXME: this is in correct...to ûдʱ.ô
				//                Date date=null;
				this.setInitialTerminationTime(null);
			}
		}
	}


	public Set<ITopic> getTopicsSet() {
		return topicsSet;
	}


	public void setTopicsSet(Set<ITopic> topicsSet) {
		this.topicsSet = topicsSet;
	}



}
