package org.act.xservices.rt.xnotification.broker.impl;

import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;

import org.act.xservices.rt.xcontainer.addressing.IEndpointReference;
import org.act.xservices.rt.xcontainer.addressing.impl.EndpointReference;
import org.act.xservices.rt.xcontainer.addressing.impl.MessageAddressingProperties;
import org.act.xservices.rt.xnotification.NotificationUtil;
import org.act.xservices.rt.xnotification.base.IPublisherTopicSet;
import org.act.xservices.rt.xnotification.base.ITopicTreesSet;
import org.act.xservices.rt.xnotification.base.fault.InvalidTopicExpressionException;
import org.act.xservices.rt.xnotification.base.fault.TopicNotSupportedException;
import org.act.xservices.rt.xnotification.broker.IPublisherRegistrationContext;
import org.act.xservices.rt.xnotification.broker.IPublisherRegistrationManager;
import org.act.xservices.rt.xnotification.message.base.Message;
import org.act.xservices.rt.xnotification.message.base.MessageFactory;
import org.act.xservices.rt.xnotification.topic.ITopic;
import org.act.xservices.rt.xnotification.topic.exception.UnsupportedTopicExpressionException;
import org.act.xservices.rt.xnotification.topic.expression.ITopicExpression;
import org.act.xservices.rt.xnotification.topic.expression.TopicCompileEngine;
import org.act.xservices.rt.xom.Metaset;


public class PublisherRegistrationManager implements
		IPublisherRegistrationManager {

	
	private IEndpointReference pubRegManagerEPR;
	
	
	/**עӳ  pulisherWSAddress --> PublisherTopicSet */
	private Map<String, IPublisherRegistrationContext> publisherToRegContextMap = 
		new ConcurrentHashMap<String, IPublisherRegistrationContext>();
	
	
	/**˷ע֪ͨе⼯ */
	protected IPublisherTopicSet brokerTopicSet;
	
	/**˷ע֪ͨе */
	protected ITopicTreesSet brokerTopicTreesSet;
	
	
	public PublisherRegistrationManager(IEndpointReference pubRegManagerEPR, 
			ITopicTreesSet topicTreesSet, IPublisherTopicSet brokerTopicSet){
		this.pubRegManagerEPR = pubRegManagerEPR;
		this.brokerTopicTreesSet = topicTreesSet;
		this.brokerTopicSet = brokerTopicSet;
	}
	
	
	
	
	public Set<ITopic> getTopicsOfProducer(String producerEPR){
		return publisherToRegContextMap.get(producerEPR).getTopicsSet();
	}
	
	
	
	
	/** 
	 * շעϢ
	 * ע⣺˴ע֮ǰ鿴ǷǰѾעᣬѾעᣬɾǰļ¼ע
	 * ˴д޸ģadd by lizq
	 * 
	 * @param message
	 * @return עϢӦϢ
	 * @see org.act.xservices.rt.xnotification.broker.IPublisherRegistrationManager#handleRegisterPublisher(org.act.xservices.rt.xnotification.message.base.Message)
	 */
	public Message handleRegisterPublisher(Message registPublishermessage) {
		
		//עϢ ע
		IPublisherRegistrationContext pubRegContext_new = 
			new PublisherRegistrationContext(registPublishermessage);
		
		String publisherAddressS = pubRegContext_new.getPublisherReferenceEPR().getAddress();
		
		//鿴Ƿǰע
		IPublisherRegistrationContext pubRegContext_old = 
			publisherToRegContextMap.get(publisherAddressS);
		if(pubRegContext_old != null){
			//broker ɾǰע
			removeRegisterTopics(pubRegContext_old);
			//ӳɾע¼
			publisherToRegContextMap.remove(publisherAddressS);
		}

		
		if(pubRegContext_new.getTopicExpressions() != null){
			//µ ⼯
			addRegisterTopics(pubRegContext_new);
			publisherToRegContextMap.put(publisherAddressS, pubRegContext_new);
			
			//ӦϢͨء(IN-OUTģʽ)

			Metaset body = NotificationUtil.getMetasetOfPrefix_WSNBR("RegisterPublisherResponse");
			body.add(pubRegManagerEPR.toMetaset());
			Message message = MessageFactory.getRegisterPublisherResponseMessage(body);
			
			Metaset to = NotificationUtil.getMetasetOfPrefix_WSA("To");
			to.setText(pubRegContext_new.getMesAP().getReplyTo().getAddress());
			message.getHeader().add(to);
			
			System.out.println("----handleRegisterPublisher----\n"+message.toMetaset().toFormatXML());
			return message;
			
		}else{
			//˷ûзκ
			
			//ӦϢͨء(IN-OUTģʽ)

			Metaset body = NotificationUtil.getMetasetOfPrefix_WSNBR("RegisterPublisherResponse");
			body.add(pubRegManagerEPR.toMetaset());
			
			Metaset wrong = NotificationUtil.getMetasetNoPrefix("Error");
			wrong.setText("There is no TopicExpress in the requestMessage, can not Register Publisher!");
			
			body.add(wrong);
			
			Message message = MessageFactory.getRegisterPublisherResponseMessage(body);
			
			Metaset to = NotificationUtil.getMetasetOfPrefix_WSA("To");
			to.setText(pubRegContext_new.getMesAP().getReplyTo().getAddress());
			message.getHeader().add(to);

			System.out.println("----handleRegisterPublisher----\n"+message.toMetaset().toFormatXML());
			return message;
			
		}

	}
	
	
	/**
	 * ⼯ ɾǰע 
	 * @param pubRegContext_old
	 */
	private void removeRegisterTopics(IPublisherRegistrationContext pubRegContext_old) {
		try {
			if (pubRegContext_old != null) {
				// ɾǰע
				for (ITopicExpression topExp : pubRegContext_old.getTopicExpressions()) {
					TopicCompileEngine.getTopicEngine().removeTopicTryBest(topExp, brokerTopicTreesSet);
				}
				brokerTopicSet.removeTopics(pubRegContext_old.getTopicsSet().toArray(new ITopic[0]));
			}
		} catch (InvalidTopicExpressionException e) {
			e.printStackTrace();
		} catch (TopicNotSupportedException e) {
			e.printStackTrace();
		} catch (UnsupportedTopicExpressionException e) {
			e.printStackTrace();
		}
	}
	
	
	/**
	 * ⼯  ע 
	 * @param pubRegContext_old
	 */
	private void addRegisterTopics(IPublisherRegistrationContext pubRegContext_new) {
		try {
			if (pubRegContext_new != null) {
				//µ 
				Set<ITopic> regTopics = new HashSet<ITopic>();
				for(ITopicExpression topExpTemp : pubRegContext_new.getTopicExpressions()){
					ITopic topTemp = TopicCompileEngine.getTopicEngine().addTopicTryBest(
							topExpTemp, brokerTopicTreesSet);
					topTemp.addNotificationProducer(pubRegContext_new.getPublisherReferenceEPR());
					regTopics.add(topTemp);
				}
				//µ ⼯
				brokerTopicSet.addTopics(regTopics.toArray(new ITopic[0]));
				
				pubRegContext_new.setTopicsSet(regTopics);
			}
		} catch (InvalidTopicExpressionException e) {
			e.printStackTrace();
		} catch (TopicNotSupportedException e) {
			e.printStackTrace();
		} catch (UnsupportedTopicExpressionException e) {
			e.printStackTrace();
		}
	}
	
	
	/**
	 * ٷעϢ
	<s:Envelope ... >
	  <s:Header>
	    <wsa:Action>
	      http://docs.oasis-open.org/wsn/brw-2/PublisherRegistrationManager/DestroyRegistrationRequest
	    </wsa:Action>
	    ...
	  </s:Header>
	  <s:Body>
	    <wsn-br:DestroyRegistration>
	       <wsn-br:PublisherReference>
             <wsa:Address>
                http://www.example.org/PublisherEndpoint
             </wsa:Address>
           </wsn-br:PublisherReference>
	    <wsn-br:DestroyRegistration/>
	  </s:Body>
	</s:Envelope>
	*/
	
	public Message handleDestroyRegistration(Message requestMessage) {
		MessageAddressingProperties messageAP = new MessageAddressingProperties(
				requestMessage.getHeader());
		
		Metaset destroyMessage = requestMessage.getBody().filter(".\\DestroyRegistration").car();

		if (destroyMessage != null) {
			Metaset publisherM = destroyMessage.filter(".\\PublisherReference");
			if (publisherM != null) {
				EndpointReference publisherEpr = new EndpointReference(publisherM.car());
				
				String publisherAddressS = publisherEpr.getAddress();
				
				//鿴Ƿǰע
				IPublisherRegistrationContext pubRegContext_old = 
					publisherToRegContextMap.get(publisherAddressS);
				if(pubRegContext_old != null){
					//ɾǰע
					removeRegisterTopics(pubRegContext_old);
					//ɾע¼
					publisherToRegContextMap.remove(pubRegContext_old);
					
					//ӦϢͨء(IN-OUTģʽ)
					Metaset body = NotificationUtil.getMetasetOfPrefix_WSNBR("DestroyRegistrationResponse");
					body.setText("Success Destroy PublisherRegistration");
					Message responseMessage = MessageFactory.getRegisterPublisherResponseMessage(body);
					Metaset to = NotificationUtil.getMetasetOfPrefix_WSA("To");
					to.setText(messageAP.getReplyTo().getAddress());
					responseMessage.getHeader().add(to);
					System.out.println(responseMessage.toMetaset().toXML());
					return responseMessage;
				}else{
					//޷ҵ˷ߵע¼
					//ӦϢͨء(IN-OUTģʽ)
					Metaset body = NotificationUtil.getMetasetOfPrefix_WSNBR("DestroyRegistrationResponse");
					body.setText("Can not find the PublisherRegistration of "+ publisherAddressS);
					Message responseMessage = MessageFactory.getRegisterPublisherResponseMessage(body);
					Metaset to =  NotificationUtil.getMetasetOfPrefix_WSA("To");
					to.setText(messageAP.getReplyTo().getAddress());
					responseMessage.getHeader().add(to);
					System.out.println(responseMessage.toMetaset().toXML());
					return responseMessage;
				}				
			}		
		}//
		
		//ӦϢͨء(IN-OUTģʽ)
		//ٷעϢʽ
		Metaset body = NotificationUtil.getMetasetOfPrefix_WSNBR("DestroyRegistrationResponse");
		body.setText("Error format of PublisherRegistration  RequestMessage !");
		Message responseMessage = MessageFactory.getRegisterPublisherResponseMessage(body);
		Metaset to = NotificationUtil.getMetasetOfPrefix_WSA("To");
		to.setText(messageAP.getReplyTo().getAddress());
		responseMessage.getHeader().add(to);
		System.out.println(responseMessage.toMetaset().toXML());
		return responseMessage;
	}


	
	
//	
//
//	private void register(IEndpointReference publisher,ITopic[]topics){
//		
//	}
//	
//	private void unRegister(IEndpointReference publisher,ITopic[]topics){
//		
//	}
	
}
