package org.act.xservices.rt.xnotification.client.util;

import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.InterruptedIOException;
import java.net.ServerSocket;
import java.net.Socket;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;
import java.util.TimeZone;

import org.act.xservices.rt.xom.Metaset;


/**
 * 
 * @version 1.0 2008-3-26
 * @author leon 2006-9-25
 * @author Lizq [lizq@act.buaa.edu.cn]
 * @since JDK1.5
 */

public class MiniServer {

	public void startListen(int PORT) {
		int i = 1;
		ServerSocket server = null;
		Socket client = null;
		try {
			server = new ServerSocket(PORT);
			System.out.println("Notification consumer is listening on port "
					+ server.getLocalPort());
			for (;;) {
				client = server.accept(); // ܿͻ
				client.setSoTimeout(2000);
				new ConnectionThread(client, i).start();
				i++;
			}
		} catch (Exception e) {
			System.out.println(e);
		}
	}

	public static void main(String args[]) {
		MiniServer mini = new MiniServer();
		mini.startListen(8095);
	}

}//



/** ConnnectionThreadһWebͨ */
class ConnectionThread extends Thread {

	Socket client; // Websocket

	int counter; // 

	/**
	 * the buffer to store the incoming message
	 */
	@SuppressWarnings("unused")
	private Metaset m_incomingMessage;

	private static final DateFormat HTTP_DATE_FORMAT = new SimpleDateFormat(
			"EEE, dd MMM yyyy HH:mm:ss zzz", Locale.US);

	static {
		HTTP_DATE_FORMAT.setTimeZone(TimeZone.getTimeZone("GMT"));
	}

	private static final String HTTP_SERVER_NAME = "WS-NotificationConsumer/1.0";

	private static final String HTTP_VERSION = "1.0";

	private static final String HTTP_STATUS = "202 Accepted";

	public ConnectionThread() {

	}

	public ConnectionThread(Socket cl, int c) {
		client = cl;
		counter = c;
	}

	/**
	 * Build a one-way HTTP response message that complies to the WS-I Basic Profile 1.0, section 5.6.10
	 * - see http://www.ws-i.org/Profiles/BasicProfile-1.0-2004-04-16.html#refinement16651160
	 *
	 * @return a WS-I-compliant Notify response message
	 */
	private String buildResponse() {
		Date timeNow = new Date();
		Date sameTimeTomorrow = new Date(timeNow.getTime()
				+ (24 * 60 * 60 * 1000));
		StringBuffer responseBuf = new StringBuffer("HTTP/");
		responseBuf.append(HTTP_VERSION);
		responseBuf.append(" ");
		responseBuf.append(HTTP_STATUS);
		responseBuf.append("\r\n");

		responseBuf.append("Date: ");
		responseBuf.append(HTTP_DATE_FORMAT.format(timeNow));
		responseBuf.append("\r\n");

		responseBuf.append("Server: ");
		responseBuf.append(HTTP_SERVER_NAME);
		responseBuf.append("\r\n");

		responseBuf.append("Cache-Control: ");
		responseBuf.append("max-age=86400");
		responseBuf.append("\r\n");

		responseBuf.append("Expires: ");
		responseBuf.append(HTTP_DATE_FORMAT.format(sameTimeTomorrow));
		responseBuf.append("\r\n");

		return responseBuf.toString();
	}

	
	public void run(){
		try {
			String destIP = client.getInetAddress().toString(); // ͻIPַ
			int destport = client.getPort(); // ͻ˿ں
			System.out.println("Connection " + counter + ":connected to "
					+ destIP + " on port " + destport + ".");

			StringBuffer retIncomingMessage = new StringBuffer();

			InputStream soIn = client.getInputStream();// ///////////
			BufferedReader bufIn = new BufferedReader(new InputStreamReader(soIn));

			BufferedOutputStream bufOut = new BufferedOutputStream(client.getOutputStream());

			byte[] responseBytes = buildResponse().toString().getBytes();	

			try {
				//while (!bufIn.readLine().equals("")); // skip empty lines
				
				char[] buffer = new char[10240];
				int length = -1;
				while (soIn.available() != 0) {
					length = bufIn.read(buffer, 0, buffer.length);
					retIncomingMessage.append(buffer, 0, length);
				}
				
			} catch (InterruptedIOException ignored) {
				ignored.printStackTrace();
			}

			bufOut.write(responseBytes, 0, responseBytes.length);
			bufOut.flush();
			bufIn.close();
			bufOut.close();

			System.out.println("-----Received Notification:-----\n"+ retIncomingMessage.toString());

			client.close();
			
		} catch (Exception e) {
			e.printStackTrace();
		}
	}//end of run
	
}