
package org.act.xservices.rt.xnotification.message.base;

import org.act.xservices.rt.xcontainer.addressing.IEndpointReference;
import org.act.xservices.rt.xnotification.NotificationConstants;
import org.act.xservices.rt.xnotification.NotificationUtil;
import org.act.xservices.rt.xnotification.util.ConfigException;
import org.act.xservices.rt.xom.Attribute;
import org.act.xservices.rt.xom.Metaset;
import org.act.xservices.rt.xom.QName;

/**
 * ⶩ Ϣ
 * @version 1.0 2008-3-16
 * @author <a href="liwu@act.buaa.edu.cn">liwu </a> 2006-9-20
 * @author cover_eye 
 * @author Lizq [lizq@act.buaa.edu.cn]
 * @since JDK1.5
 */

public class SubscribeRequestMessage extends Message {

	private Metaset subRequestBody = null;

	private String wSATo = null;

	private String wSAResourceIdentifier = null;

	private IEndpointReference consumerReference = null;
	
    //޸bycover_eye:2007.3.9
	//private Filter filter = null;
	
	private Metaset filter = null;

	private String initialTerminationTime = null;
	
	private String actionValue = 
		"http://docs.oasis-open.org/wsn/bw-2/NotificationProducer/SubscribeRequest";

	
	
	/**
	 * Ϣģ
	<wsnt:Subscribe>
	  <wsnt:ConsumerReference>
	    wsa:EndpointReferenceType
	  </wsnt:ConsumerReference>
	  <wsnt:Filter>
	    [ <wsnt:TopicExpression Dialect="xsd:anyURI"> 
	        {any} ?
	      </wsnt:TopicExpression> |
	      <wsnt:ProducerProperties Dialect="xsd:anyURI"> 
	        {any} ?
	      </wsnt:ProducerProperties> |
	      <wsnt:MessageContent Dialect="xsd:anyURI"> 
	        {any} ?
	      </wsnt:MessageContent> |
	      {any} *
	    ] *
	  </wsnt:Filter> ?
	  <wsnt:InitialTerminationTime> 
	    [xsd:dateTime | xsd:duration]
	  </wsnt:InitialTerminationTime> ?
	  <wsnt:SubscriptionPolicy>
	    [ <wsnt:UseRaw/> |
	      {any}
	    ] *
	  </wsnt:SubscriptionPolicy> ?
	  {any}*
	</wsnt:Subscribe>
	*/
	
	public SubscribeRequestMessage() {
		this.buildMessageSkeleton();
		
		buildSubscribeHeader();
		
		buildSubscribeRequestBody();
		
		this.getWSAAction().setText(actionValue);
	}

	public SubscribeRequestMessage(
			IEndpointReference consumerReference,
			Metaset subFilter, 
			String initialTerminationTime) {
		
		this("", "", consumerReference, subFilter, initialTerminationTime);
	}

	public SubscribeRequestMessage(
			String toValue, 
			String resourceIdendifier, 
			IEndpointReference consumerReference,
			Metaset subFilter, 
			String initialTerminationTime) {
		
		this.buildMessageSkeleton();

		buildSubscribeHeader(toValue, actionValue, resourceIdendifier);

		buildSubscribeRequestBody(consumerReference, subFilter,initialTerminationTime);


	}

	public void config(Metaset config) throws ConfigException {
		// TODO Զɷ

	}

	/**
	 * ԴҪĲ.
	 */
	private void buildSubscribeHeader() {
		Metaset header = this.getHeader();
		
		Metaset wsaTo = NotificationUtil.getMetasetOfPrefix_WSA("To");
		header.add(wsaTo);

		Metaset wsaResourceID = NotificationUtil.getMetasetOfPrefix_WSA("ResourceIdentifier");
		header.add(wsaResourceID);

	}

	private void buildSubscribeHeader(String toValue, String actionValue,String resourceIdendifier) {
		// header 
		buildSubscribeHeader();
		
		setWSATo(toValue);
		
		this.getWSAAction().setText(actionValue);
		
		setWSAResourceIdentifier(resourceIdendifier);
	}

	/**
	 * TODO: : һַHeaderдһд.ȻĲԷڹ캯ﴫ,ҲͨһϢ֮.ȽһЩ
	 * 
	 * @param consumerRef
	 * @param subFilter
	 * @param initialTerminaTime
	 * @return
	 * @throws Exception
	 */
	private void buildSubscribeRequestBody() {
		
		subRequestBody = NotificationUtil.getMetasetOfPrefix_WSNT("Subscribe");

		Metaset initialTerminationTime = NotificationUtil.getMetasetOfPrefix_WSNT("InitialTerminationTime");
		subRequestBody.add(initialTerminationTime);
		
		this.getBody().add(subRequestBody);
	}

	
	private void buildSubscribeRequestBody(
			IEndpointReference consumerRef,
			Metaset subFilter, 
			String initialTerminaTime){
		
		this.buildSubscribeRequestBody();
		
		setConsumerReference(consumerRef);
		setFilter(subFilter);
		setInitialTerminationTime(initialTerminaTime);
	}

	/**
	 * @return  consumerReference
	 */
	public IEndpointReference getConsumerReference() {
		return consumerReference;
	}

	
	/**
	 * @param consumerReference
	 *            Ҫõ consumerReference
	 */
	public void setConsumerReference(IEndpointReference consumerReference) {

		this.getBody().removeElements(".\\Subscribe\\ConsumerReference");

		// xiugai by cover_eye:
		// ΪIEndpointReferenceΪwsaIEndpointReference
		// subscribewsnt:ConsumerReference
		
		Metaset conRefMetaset = consumerReference.toMetaset();
		
		conRefMetaset.setQName(new QName(
				NotificationConstants.URI_NS_WSNT,"ConsumerReference","wsnt"));


		this.getBody().filter(".\\Subscribe").car().add(conRefMetaset);
		this.consumerReference = consumerReference;
	}

	
	/**
	 * @return  filter
	 */
	public Metaset getFilter() {
		return filter;
	}

	public void setFilter(Metaset filter) {
		this.filter = filter;
        //޸bycover_eye:Metasetremove⡣
		//this.getBody().remove(".\\Subscribe\\Filter");
		
		this.getBody().filter(".\\Subscribe").car().add(filter);
	}

	
	/**
	 * @return  initialTerminationTime
	 */
	public String getInitialTerminationTime() {
		return initialTerminationTime;
	}

	/**
	 * @param initialTerminationTime
	 *            Ҫõ initialTerminationTime
	 */
	public void setInitialTerminationTime(String initialTerminationTime) {

		this.getBody().filter(".\\Subscribe\\InitialTerminationTime").car().setText(
				initialTerminationTime);
		
		this.initialTerminationTime = initialTerminationTime;
	}
	
	

	/**
	 * @return  wSAResourceIdentifier
	 */
	public String getWSAResourceIdentifier() {
		return wSAResourceIdentifier;
	}

	/**
	 * @param resourceIdentifier
	 *            Ҫõ wSAResourceIdentifier
	 */
	public void setWSAResourceIdentifier(String resourceIdentifier) {
		this.getHeader().filter(".\\ResourceIdentifier").car().setText(
				resourceIdentifier);
		wSAResourceIdentifier = resourceIdentifier;

	}

	/**
	 * @return  wSATo
	 */
	public String getWSATo() {
		return wSATo;
	}

	/**
	 * @param to
	 *            Ҫõ wSATo
	 */
	public void setWSATo(String to) {
		this.getHeader().filter(".\\To").car().setText(to);
		wSATo = to;
	}


	/**
	 * ֿռ䡣ӵenvelopnamespace
	 * 
	 * @param qName
	 * @param value
	 */
	public void addNameSpace(String qName, String value) {
		Attribute newAttribute = NotificationUtil.getNewAttribute(qName, value);
		this.addNameSpaceToEnvelope(newAttribute);
	}
}
