package org.act.xservices.rt.xnotification.message.base;

import org.act.xservices.rt.xcontainer.addressing.IEndpointReference;
import org.act.xservices.rt.xnotification.NotificationConstants;
import org.act.xservices.rt.xnotification.NotificationUtil;
import org.act.xservices.rt.xnotification.util.ConfigException;
import org.act.xservices.rt.xom.Attribute;
import org.act.xservices.rt.xom.Metaset;
import org.act.xservices.rt.xom.QName;

/**
 * ȡ Ϣ
 * 
 * @version 1.0 2008-3-16
 * @author liwu 2006-11-7
 * @author Lizq [lizq@act.buaa.edu.cn]
 * @since JDK1.5
 */

public class UnsubscribeRequestMessage extends Message {

	private Metaset unSubRequestBody = null;

	private String wSATo = null;

	private String wSAResourceIdentifier = null;

	private IEndpointReference consumerReference = null;
	
	private Metaset filter = null;


	
	private String actionValue = 
		"http://docs.oasis-open.org/wsn/bw-2/SubscriptionManager/UnsubscribeRequest";

	
	
	/**
	 * ȡ Ϣģ
	<wsnt:UnSubscribe>
	
	  <wsnt:ConsumerReference>
	    wsa:EndpointReferenceType
	  </wsnt:ConsumerReference>
	  
	  <wsnt:Filter>
	    [ <wsnt:TopicExpression Dialect="xsd:anyURI"> 
	        {any} ?
	      </wsnt:TopicExpression> |
	      <wsnt:ProducerProperties Dialect="xsd:anyURI"> 
	        {any} ?
	      </wsnt:ProducerProperties> |
	      <wsnt:MessageContent Dialect="xsd:anyURI"> 
	        {any} ?
	      </wsnt:MessageContent> |
	      {any} *
	    ] *
	  </wsnt:Filter> ?
	  
	  {any}*
	  
	</wsnt:UnSubscribe>
	*/
	
	
	public UnsubscribeRequestMessage() {
		this.buildMessageSkeleton();
		
		buildUnSubscribeHeader();
		
		buildUnSubscribeRequestBody();
		
		this.getWSAAction().setText(actionValue);
	}

	public UnsubscribeRequestMessage(IEndpointReference consumerReference, Metaset subFilter) {
		
		this("", "", consumerReference, subFilter);
	}

	public UnsubscribeRequestMessage(String toValue, String resourceIdendifier, 
			IEndpointReference consumerReference, Metaset subFilter) {
		
		this.buildMessageSkeleton();

		buildUnSubscribeHeader(toValue, actionValue, resourceIdendifier);

		buildUnSubscribeRequestBody(consumerReference, subFilter);


	}

	public void config(Metaset config) throws ConfigException {
		// TODO Զɷ

	}

	/**
	 * ԴҪĲ.
	 */
	private void buildUnSubscribeHeader() {
		Metaset header = this.getHeader();
		
		Metaset wsaTo = NotificationUtil.getMetasetOfPrefix_WSA("To");
		header.add(wsaTo);

		Metaset wsaResourceID = NotificationUtil.getMetasetOfPrefix_WSA("ResourceIdentifier");
		header.add(wsaResourceID);

	}

	private void buildUnSubscribeHeader(String toValue, String actionValue,String resourceIdendifier) {
		// header 
		buildUnSubscribeHeader();
		
		setWSATo(toValue);
		
		this.getWSAAction().setText(actionValue);
		
		setWSAResourceIdentifier(resourceIdendifier);
	}

	/**
	 * TODO: : һַHeaderдһд.ȻĲԷڹ캯ﴫ,ҲͨһϢ֮.ȽһЩ
	 * 
	 * @param consumerRef
	 * @param subFilter
	 * @param initialTerminaTime
	 * @return
	 * @throws Exception
	 */
	private void buildUnSubscribeRequestBody() {
		
		unSubRequestBody = NotificationUtil.getMetasetOfPrefix_WSNT("Unsubscribe");
		
		this.getBody().add(unSubRequestBody);
	}

	
	private void buildUnSubscribeRequestBody(
			IEndpointReference consumerRef,
			Metaset subFilter){
		
		this.buildUnSubscribeRequestBody();
		
		setConsumerReference(consumerRef);
		
		setFilter(subFilter);

	}

	/**
	 * @return  consumerReference
	 */
	public IEndpointReference getConsumerReference() {
		return consumerReference;
	}

	
	/**
	 * @param consumerReference
	 *            Ҫõ consumerReference
	 */
	public void setConsumerReference(IEndpointReference consumerReference) {

		this.getBody().removeElements(".\\Unsubscribe\\ConsumerReference");

		// xiugai by cover_eye:
		// ΪIEndpointReferenceΪwsaIEndpointReference
		// subscribewsnt:ConsumerReference
		
		Metaset conRefMetaset = consumerReference.toMetaset();
		conRefMetaset.setQName(new QName(
				NotificationConstants.URI_NS_WSNT,"ConsumerReference","wsnt"));

		this.getBody().filter(".\\Unsubscribe").car().add(conRefMetaset);
		this.consumerReference = consumerReference;
	}

	
	/**
	 * @return  filter
	 */
	public Metaset getFilter() {
		return filter;
	}

	public void setFilter(Metaset filter) {
		this.filter = filter;
        //޸bycover_eye:Metasetremove⡣
		//this.getBody().remove(".\\Subscribe\\Filter");
		
		this.getBody().filter(".\\Unsubscribe").car().add(filter);
	}


	/**
	 * @return  wSAResourceIdentifier
	 */
	public String getWSAResourceIdentifier() {
		return wSAResourceIdentifier;
	}

	/**
	 * @param resourceIdentifier
	 *            Ҫõ wSAResourceIdentifier
	 */
	public void setWSAResourceIdentifier(String resourceIdentifier) {
		this.getHeader().filter(".\\ResourceIdentifier").car().setText(
				resourceIdentifier);
		wSAResourceIdentifier = resourceIdentifier;

	}

	/**
	 * @return  wSATo
	 */
	public String getWSATo() {
		return wSATo;
	}

	/**
	 * @param to
	 *            Ҫõ wSATo
	 */
	public void setWSATo(String to) {
		this.getHeader().filter(".\\To").car().setText(to);
		wSATo = to;
	}


	/**
	 * ֿռ䡣ӵenvelopnamespace
	 * 
	 * @param qName
	 * @param value
	 */
	public void addNameSpace(String qName, String value) {
		Attribute newAttribute = NotificationUtil.getNewAttribute(qName, value);
		this.addNameSpaceToEnvelope(newAttribute);
	}
}
