package org.act.xservices.rt.xnotification.resourcecallback;

import java.util.ArrayList;
import java.util.List;

import org.act.xservices.rt.xnotification.NotificationUtil;
import org.act.xservices.rt.xnotification.WSNRuntime;
import org.act.xservices.rt.xnotification.base.INotificationPublisher;
import org.act.xservices.rt.xnotification.base.fault.InvalidTopicExpressionException;
import org.act.xservices.rt.xnotification.base.fault.TopicNotSupportedException;
import org.act.xservices.rt.xnotification.topic.ITopic;
import org.act.xservices.rt.xnotification.topic.ITopicNamespace;
import org.act.xservices.rt.xnotification.topic.TopicNamespaceFactory;
import org.act.xservices.rt.xnotification.topic.exception.UnsupportedTopicExpressionException;
import org.act.xservices.rt.xnotification.topic.expression.ITopicExpression;
import org.act.xservices.rt.xnotification.topic.expression.TopicCompileEngine;
import org.act.xservices.rt.xnotification.topic.expression.TopicExpressionFactory;
import org.act.xservices.rt.xom.Metaset;
import org.act.xservices.rt.xresource.resource.IResourceCallBack;


/**
 * ֪ͨԴ
 * ֪ͨԴı仯֪ͨעԴӦ
 * ÿ֪ͨԴӦһ
 * 
 * @version 1.0 2008-3-5
 * @author Lizq [lizq@act.buaa.edu.cn]
 * @since JDK1.5
 */
public class NotificationResourceMonitor implements IResourceCallBack{

	/** */
	private static final long serialVersionUID = 1L;
	
	/**ԴӦڵƿռ */
	@SuppressWarnings("unused")
	private ITopicNamespace topicNamespace;
	
	/**֪ͨԴ֪ͨ */
	private INotificationPublisher notificationPublisher;
	
	/**ԴӦһЩ */
	private ITopic[] topics;
	
	/**ԴӦһЩʽ */
	private List<ITopicExpression> topicExpressions;
	
	
	
	//----Ҫ֪ͨԴлӦ⣬֪ͨ߶˵
	public NotificationResourceMonitor(Metaset configMetaset) throws Exception{
		//ô֪ͨԴ
		this.config(configMetaset);
		
		//򷢲ע֪ͨԴֵ֧
		this.registerTopicsToPublisher();
		
	}
	
	
	
	/**
	 * ļʵ
	<NotificationResourceConfig
	xmlns="http://schemas.xmlsoap.org/soap/envelope/"
	xmlns:wstop="http://docs.oasis-open.org/wsn/t-1"
	xmlns:newsSpace="http://www.newsSpace.com"
	xmlns:wsa="http://schemas.xmlsoap.org/ws/2003/03/addressing"
	xmlns:wsnt="http://docs.oasis-open.org/wsn/2004/06/wsn-WS-BaseNotification-1.2-draft-01.xsd">

	<wsnt:PublisherReference>
		<wsa:Address>
			http://localhost:8080/wsn/services/news
		</wsa:Address>
	</wsnt:PublisherReference>

	<wstop:TopicNamespace name="newsSpace"
		targetNamespace="http://www.newsSpace.com" />

	<wsnt:TopicExpression
		Dialect="http://docs.oasis-open.org/wsn/t-1/TopicExpression/Simple">
		newsSpace:news
	</wsnt:TopicExpression>*
    </NotificationResourceConfig>
    *
    */
	
	
	/**
	 * NotificationResourceMonitor
	 * 1.notificationPublisherEPRnotificationPublisher
	 * 2.ڵռ
	 * 3.ʽ
	 * 
	 * @param configMetaset
	 * @throws Exception
	 */
	public void config(Metaset configMetaset)throws Exception{
		
		//ȡԴӦ֪ͨ
		String pubAddress = configMetaset.filter(".\\PublisherReference\\Address").car().getText();
		notificationPublisher = WSNRuntime.getRuntime().getPublisherByAddress(pubAddress);
		if(notificationPublisher == null)
			throw new Exception("No such publisher for "+pubAddress);
		
		//ƿռ
		String tnName = configMetaset.selectorAttributeValue(".\\TopicNamespace", "name");	
		String tnNamespace = configMetaset.selectorAttributeValue(".\\TopicNamespace", "targetNamespace");
		if(tnName == null || tnNamespace == null)
			throw new Exception("TopicNamespace Error !");
		topicNamespace = TopicNamespaceFactory.getInstance().getTopicNamespace(
				tnName, tnNamespace, notificationPublisher.getTopicTreesSet());
		
		//ʽ
		Metaset tpExpM = configMetaset.filter(".\\TopicExpression");
		if(tpExpM == null)
			throw new Exception("TopicExpression Error !");
		topicExpressions = new ArrayList<ITopicExpression>();
		//TopicExpression
		for(Object te : tpExpM.elements()){
			ITopicExpression topicExpre = TopicExpressionFactory.getTopicExpression(
					(Metaset)te, configMetaset.attributes());
			topicExpressions.add(topicExpre);
		}
	}
	
	
	/**
	 * ֪ͨעԴ漰
	 * 
	 * @return
	 */
	public ITopic[] registerTopicsToPublisher() {
		
		ITopic topic = null;
		List<ITopic> topicList = new ArrayList<ITopic>();
		try {
			//ÿ
			for(ITopicExpression tpExp: topicExpressions){
				
				topic = TopicCompileEngine.getTopicEngine().addTopicTryBest(tpExp,
						notificationPublisher.getTopicTreesSet());
				topic.addNotificationProducer(notificationPublisher.getEPR()); //Ӵ
				topicList.add(topic);
			}
			
			topics = topicList.toArray(new ITopic[0]);
			//עֵ֧ᷢ
			notificationPublisher.getTopicSet().addTopics(topics);
			
			//仯¼
			notificationPublisher.supportedTopicChanged();
			
			return topics;
			
		} catch (UnsupportedTopicExpressionException e) {
			e.printStackTrace();
		} catch (InvalidTopicExpressionException e) {
			e.printStackTrace();
		} catch (TopicNotSupportedException e) {
			e.printStackTrace();
		}
		return null;
	}

	
	/**
	 * ԴԷ仯
	 * 
	 * @param oldValue
	 * @param newValue
	 */
	public void fireChangeNotification(Object oldValue, Object newValue) {

		// ÿ֪ⷢͨϢ
		for (ITopic topicTemp : topics) {

			System.out.println("--NotificationResourceMonitor--α仯Ϊ"+ topicTemp);

			Metaset message = NotificationUtil.getMetasetOfPrefix_WSNT("Message");

			if (oldValue instanceof Metaset && newValue instanceof Metaset) {
				Metaset _old = NotificationUtil.getMetasetNoPrefix("oldValue");
				_old.add((Metaset) oldValue);
				message.add(_old);

				Metaset _new = NotificationUtil.getMetasetNoPrefix("newValue");
				_new.add((Metaset) newValue);
				message.add(_new);
			}

			// ùڴһ֪ͨϢд by lizq
			topicTemp.setContent(message);

			// ֪ͨϢ
			System.out.println("NotificationResourceMonitor.topicChanged()"
					+ topicTemp);
			System.out.println(message.toFormatXML());
			
			notificationPublisher.sendNotification(topicTemp);

		}

	}
	
	
	/**
	 * ɾԴӦ
	 */
	public void fireDeleteNotification() {
		
		//ڷɾԴӦ
		for (ITopicExpression topicExp : topicExpressions) {
			try {
				//Ŭɾ
				TopicCompileEngine.getTopicEngine().removeTopicTryBest(topicExp,
						notificationPublisher.getTopicTreesSet());
			} catch (InvalidTopicExpressionException e) {
				e.printStackTrace();
			} catch (TopicNotSupportedException e) {
				e.printStackTrace();
			} catch (UnsupportedTopicExpressionException e) {
				e.printStackTrace();
			}
			
			System.out.println("--NotificationResourceMonitor--"+ topicExp+" has been deleted!!!");
		}//end of for
		
		//ڷ⼯ɾԴӦ
		notificationPublisher.getTopicSet().removeTopics(topics);
		
		topicExpressions = null; //ɾ
		topics = null;
		
		//仯¼
		notificationPublisher.supportedTopicChanged();

	}

	
	
	
	//----------------- IResourceCallBack ӿеķ-------------------

	public void deleteResourceProperties() {
		this.fireDeleteNotification();
		
	}

	public void destroyResourceProperty() {
		this.fireDeleteNotification();
		
	}

	public void getMultipleResourceProperties() {
		// TODO Auto-generated method stub
		
	}

	public void getResourceProperty() {
		// TODO Auto-generated method stub
		
	}

	public void getResourcePropertyDocument() {
		// TODO Auto-generated method stub
		
	}

	public void insertResourceProperties(Metaset oldValue, Metaset newValue) {
		System.out.println("ResourceCallBack.updateResourceProperties()");
		this.fireChangeNotification(oldValue,newValue);
		
	}

	public void putResourcePropertyDocument(Metaset oldValue, Metaset newValue) {
		
		System.out.println("ResourceCallBack.updateResourceProperties()");
		this.fireChangeNotification(oldValue,newValue);
		
	}

	public void queryResourceProperties() {
		// TODO Auto-generated method stub
		
	}

	public void setResourceProperties(Metaset oldValue, Metaset newValue) {
		System.out.println("ResourceCallBack.updateResourceProperties()");
		this.fireChangeNotification(oldValue,newValue);
		
	}

	public void updateResourceProperties(Metaset oldValue, Metaset newValue) {
		System.out.println("ResourceCallBack.updateResourceProperties()");
		this.fireChangeNotification(oldValue,newValue);
		
	}
	
	//----------------- end of IResourceCallBack ӿеķ-------------------
	
	
	
	

}
