package org.act.xservices.rt.xnotification.topic;

import org.act.xservices.rt.xom.Metaset;


/**
 * A topic namespace (a hierarchy of topics within a particular namespace), 
 * as defined by the WS-Topics 1.3 specification.
 * 
 * @version 1.0 2007-8-9
 * @author cover_eye wangyg@act.buaa.edu.cn
 * @author Lizq [lizq@act.buaa.edu.cn]
 * @since JDK1.5
 */
public interface ITopicNamespace extends ITopicNode {
	/**
	 * Sets this topic namespace's finality.
	 *
	 * @param value
	 */
	public void setFinal(boolean value);

	/**
	 * If the value is <code>true</code> it indicates that NotificationProducers cannot use topics whose root topics do
	 * not appear in this topic namespace. The default value is <code>false</code>.
	 *
	 * @return true if this topic namespace is final, or false if it is not
	 */
	public boolean isFinal();

	/**
	 * Sets this topic namespace's (optional) name.
	 *
	 * @param name the name
	 */
	public void setName(String name);

	/**
	 * Returns this topic namespace's name, or null if no name is defined.
	 *
	 * @return this topic namespace's name, or null if no name is defined
	 */
	public String getName();

	/**
	 * Returns this topic namespace's target namespace.
	 *
	 * @return this topic namespace's target namespace
	 */
	public String getTargetNamespace();
	
	/**
	 * Returns this topic namespace as a Metaset 
	 * @return
	 */
	public Metaset toMetaset();
}
