package org.act.xservices.rt.xnotification.topic;

import java.util.Iterator;

import org.act.xservices.rt.xnotification.base.fault.InvalidTopicExpressionException;
import org.act.xservices.rt.xnotification.base.fault.TopicNotSupportedException;

/**
 *  A set of Topics
 * 
 * @version 1.0 2007-8-9
 * @author cover_eye wangyg@act.buaa.edu.cn
 * @author Lizq [lizq@act.buaa.edu.cn]
 * @since JDK1.5
 */

public interface ITopicNode {
	
	/**
	 * Gets the topic with the specified name.
	 * 
	 * @param name the name of the topic to get
	 * 
	 * @return the topic, or null if no topic with the given name exists in this
	 *         set
	 */
	public ITopic getTopic(String name);

	/**
	 * Creates a new Topic object with the specified name and adds it to this
	 * set. If this set already contains a Topic with the specified name, it
	 * will be replaced.
	 * 
	 * @param name the name of the topic to be added
	 * @return the new topic
	 * @throws InvalidTopicExpressionException
	 */
	public ITopic addTopic(String name) throws InvalidTopicExpressionException;

	/**
	 * Returns <tt>true</tt> if this set contains a topic with the specified
	 * name.
	 * 
	 * @param name the name of the topic to be checked
	 * 
	 * @return <tt>true</tt> if this set contains a topic with the specified
	 *         name
	 */
	public boolean containsTopic(String name);

	/**
	 * Removes the topic with the specified name from this set.
	 * 
	 * @param name the name of the topic to be removed
	 * @throws TopicNotSupportedException
	 */
	public void removeTopic(String name) throws TopicNotSupportedException;

	/**
	 * Removes the specified topic from this set.
	 * 
	 * @param topic the topic to be removed
	 * @throws TopicNotSupportedException
	 */
	public void removeTopic(ITopic topic) throws TopicNotSupportedException;

	/**
	 * Gets an iterator of the topics in this set.
	 * 
	 * @return the iterator
	 */
	public Iterator topicIterator();
}
