
package org.act.xservices.rt.xnotification.topic;

import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.act.xservices.rt.xcontainer.addressing.IEndpointReference;
import org.act.xservices.rt.xnotification.NotificationUtil;
import org.act.xservices.rt.xnotification.base.fault.InvalidTopicExpressionException;
import org.act.xservices.rt.xnotification.base.fault.TopicNotSupportedException;
import org.act.xservices.rt.xom.Metaset;

/**
 * 
 * @version 1.0 2008-3-13
 * @author cover_eye wangyg@act.buaa.edu.cn 2007-8-9
 * @author Lizq [lizq@act.buaa.edu.cn]
 * @since JDK1.5
 */
public class TopicImpl implements ITopic {

	private String name;

	/**ӳ */
	private Map<String, ITopic> subTopicsMap = new HashMap<String, ITopic>();

	private boolean finalflag = false;

	private Map<String, IEndpointReference> producerMap = new HashMap<String, IEndpointReference>();

	private Object content = null;

	/**ŵǱ· */
	private List<String> topicPath = new LinkedList<String>();

	private ITopicNamespace topicNs;
	

	/**
	 * Create a topic with the given name
	 *
	 * @param name The name of the created topic
	 */
	public TopicImpl(String topicName) {
		this.name = topicName;
	}
	
	public TopicImpl(String topicName, ITopicNamespace topicNamespace) {
		this.name = topicName;
		this.topicNs = topicNamespace;
	}
	


	public void addNotificationProducer(IEndpointReference notificationEPR) {
		producerMap.put(notificationEPR.getAddress(), notificationEPR);
	}

	
	public boolean containNotificationProducer(IEndpointReference notificationEPR) {
		return producerMap.containsKey(notificationEPR.getAddress());
	}

	public Object getContect() {
		return content;
	}

	public String getName() {
		return name;
	}


	public String getParent() {
		return null;
	}

	public Iterator getProducerIterator() {
		return producerMap.values().iterator();
	}

	
	public ITopicNamespace getTopicNamespace() {
		return topicNs;
	}

	
	public List<String> getTopicPath() {
		return topicPath;
	}

	public boolean isFinal() {
		return finalflag;
	}

	
	public void setContent(Object content) {
		this.content = content;

	}


	public void setFinal(boolean value) {
		this.finalflag = value;

	}


	public void setParent(String concretTopicExpression) {
		// TODO Զɷ

	}


	public void setTopicNamespace(ITopicNamespace topicNs) {
		this.topicNs = topicNs;

	}


	public void setTopicPath(List<String> topicPath) {
		this.topicPath = topicPath;

	}


	public Metaset toMetaset() {
		
		Metaset space = NotificationUtil.getMetasetNoPrefix("Topic");

		space.add(NotificationUtil.getNewAttribute("name", this.getName()));

		Iterator topics = this.topicIterator();
		while (topics.hasNext()) {
			ITopic temp = (ITopic) topics.next();
			space.add(temp.toMetaset());
		}
		return space;
	}


	
	/** 
	 * Ϊֱ
	 * @param name
	 * @return ӵ
	 * @throws InvalidTopicExpressionException
	 * @see org.act.xservices.rt.xnotification.topic.ITopicNode#addTopic(java.lang.String)
	 */
	public ITopic addTopic(String name) throws InvalidTopicExpressionException {
		
		ITopic topicForAdd = new TopicImpl(name);
		
		subTopicsMap.put(topicForAdd.getName(), topicForAdd);
		topicForAdd.setTopicNamespace(topicNs);
		List<String> subTopicPath = new LinkedList<String>();
		subTopicPath.addAll(this.topicPath);
		subTopicPath.add(topicForAdd.getName());
		topicForAdd.setTopicPath(subTopicPath);
		return topicForAdd;
	}
	
	


	public boolean containsTopic(String topicName) {
		return subTopicsMap.containsKey(topicName);
	}


	public ITopic getTopic(String topicName) {
		return subTopicsMap.get(topicName);
	}


	public void removeTopic(String name) throws TopicNotSupportedException {
		subTopicsMap.remove(name);

	}


	public void removeTopic(ITopic topic) throws TopicNotSupportedException {
		subTopicsMap.remove(topic.getName());
	}


	public Iterator topicIterator() {
		return subTopicsMap.values().iterator();
	}



	/** 
	 * ·ͬʱ⼴Ϊ
	 * @param obj
	 * @return
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	public boolean equals(Object obj) {
		if((obj != null) && obj instanceof TopicImpl){
			return ((TopicImpl)obj).getAbsolutePath().equals(
					this.getAbsolutePath());
		}
		return false;
	}


	public int hashCode() {
		return getAbsolutePath().hashCode();
	}
	
	private String getAbsolutePath(){
		StringBuffer strBuf = new StringBuffer();
		if ((topicPath != null) && !topicPath.isEmpty()) {
			
			if(topicNs != null){//ƿռ
				strBuf.append(topicNs.getTargetNamespace()+":");
			}	
			for (String path : topicPath) {
				strBuf.append(path);
				strBuf.append('/');
			}	
			strBuf.setLength(strBuf.length() - 1); // chop off trailing slash
		}
		return strBuf.toString();
	}
	
	
	public String toString() {
		StringBuffer strBuf = new StringBuffer();
		if ((topicPath != null) && !topicPath.isEmpty()) {
			if(topicNs != null){//ƿռ
				strBuf.append(topicNs.getName()+":");
			}	
			for (String path : topicPath) {
				strBuf.append(path);
				strBuf.append('/');
			}	
			strBuf.setLength(strBuf.length() - 1); // chop off trailing slash
		}
		return strBuf.toString();
	}
	
	
	/**
	 * ôConcrete ͵ıʽ
	 * @return
	 */
	public Metaset getConcreteExpression(){
		
		Metaset topicExpression = NotificationUtil.getMetasetNoPrefix("Topic");
		
		topicExpression.add(
				NotificationUtil.getNewAttribute("Dialect", TopicConstants.CONCRETE_TOPICEXPRESSION));
		
		topicExpression.add(
				NotificationUtil.getNewAttribute("xmlns:"+this.getTopicNamespace().getName(), 
				this.getTopicNamespace().getTargetNamespace()));
				
		topicExpression.setText(this.toString());

		return topicExpression;
	}


	/**
	 * @param args
	 */
	public static void main(String[] args) {

	}

}
