package org.act.xservices.rt.xnotification.topic;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import org.act.xservices.rt.xnotification.NotificationUtil;
import org.act.xservices.rt.xnotification.base.fault.InvalidTopicExpressionException;
import org.act.xservices.rt.xnotification.base.fault.TopicNotSupportedException;
import org.act.xservices.rt.xom.Metaset;

/**
 * ƿռʵ
 * 
 * @version 1.0 2008-3-14
 * @author cover_eye wangyg@act.buaa.edu.cn 2007-8-9
 * @author Lizq [lizq@act.buaa.edu.cn]
 * @since JDK1.5
 */
public class TopicNamespaceImpl implements ITopicNamespace {

	private String name;

	private String targetNamespace;

	private Map<String, ITopic> rootTopicsMap = new ConcurrentHashMap<String, ITopic>();

	/**
	 * If the value is true it indicates that 
	 * any Topic which appears in a NotificationProducers Topic Set 
	 * and uses this target namespace MUST have its root explicitly defined in the TopicNamespace
	 */
	private boolean finalflag;
	

	/**
	 * Creates a topic namespace with the specified target namespace.
	 *
	 * @param targetNs 
	 */
	public TopicNamespaceImpl(String targetNs) {
		if (targetNs == null) {
			throw new IllegalArgumentException(
					"Target namespace must not be null (\"\" should be used to represent the default namespace).");
		}
		targetNamespace = targetNs;
	}


	public String getName() {
		return name;
	}


	public String getTargetNamespace() {
		return targetNamespace;
	}


	public boolean isFinal() {
		return finalflag;
	}


	public void setFinal(boolean value) {
		finalflag = value;

	}


	public void setName(String name) {
		this.name = name;

	}


	public Metaset toMetaset() {
			Metaset space = NotificationUtil.getMetasetNoPrefix("TopicSpace");
			
			space.add(NotificationUtil.getNewAttribute("name",this.getName()));
			
			space.add(
					NotificationUtil.getNewAttribute("targetNamespace",this.getTargetNamespace()));
			
			space.add(
					NotificationUtil.getNewAttribute("xmlns",
							"http://docs.oasis-open.org/wsn/2004/06/wsn-WS-Topics-1.2-draft-01.xsd"));
			
			Iterator topics=this.topicIterator();
			
			while (topics.hasNext()) {
				ITopic temp=(ITopic)topics.next();
				space.add(temp.toMetaset());
			}
			
			return space;
	}


	/** 
	 * һΪ
	 * һ
	 * 
	 * @param name
	 * @return
	 * @throws InvalidTopicExpressionException
	 * @see org.act.xservices.rt.xnotification.topic.ITopicNode#addTopic(java.lang.String)
	 */
	public ITopic addTopic(String name) throws InvalidTopicExpressionException {
		
		ITopic topicForAdd = new TopicImpl(name);
		
		rootTopicsMap.put(topicForAdd.getName(), topicForAdd);
		topicForAdd.setTopicNamespace(this);
		List<String> topicPath = new LinkedList<String>();
		topicPath.add(topicForAdd.getName());
		topicForAdd.setTopicPath(topicPath);
		
		return topicForAdd;
	}
	


	public boolean containsTopic(String name) {
		return rootTopicsMap.containsKey(name);
	}


	public ITopic getTopic(String name) {
		return rootTopicsMap.get(name);
	}


	public void removeTopic(String name) throws TopicNotSupportedException {
		rootTopicsMap.remove(name);

	}


	public void removeTopic(ITopic topic) throws TopicNotSupportedException {
		rootTopicsMap.remove(topic.getName());
	}


	public Iterator topicIterator() {
		return rootTopicsMap.values().iterator();
	}

	/**
	 * Override some usable methods of the Object
	 */


	public boolean equals(Object obj) {
		return ( obj != null ) && obj instanceof TopicNamespaceImpl && ( obj.hashCode(  ) == hashCode(  ) );
	}


	@Override
	public int hashCode() {
		return targetNamespace.hashCode(  );
	}


	
	/** 
	 * 
	 * @return a String representation of this TopicNamespace
	 */
	public String toString() {
		StringBuffer strBuf = new StringBuffer();
		if (targetNamespace != null) {
			strBuf.append('{');
			strBuf.append(targetNamespace);
			strBuf.append('}');
		}
		if (name != null) {
			strBuf.append(name);
		}
		return strBuf.toString();
	}
	

}
