package org.act.xservices.rt.xnotification.topic.expression;

import java.util.Calendar;
import java.util.List;
import java.util.Map;

import org.act.xservices.rt.xnotification.base.ITopicTreesSet;
import org.act.xservices.rt.xnotification.base.fault.InvalidTopicExpressionException;
import org.act.xservices.rt.xnotification.base.fault.TopicNotSupportedException;
import org.act.xservices.rt.xnotification.topic.ITopic;
import org.act.xservices.rt.xnotification.topic.ITopicNamespace;
import org.act.xservices.rt.xnotification.topic.TopicConstants;
import org.act.xservices.rt.xnotification.topic.TopicNamespaceFactory;

/**
 * 
 * @version 1.0 2008-3-4
 * @author cover_eye wangyg@act.buaa.edu.cn 2006-8-31
 * @author Lizq [lizq@act.buaa.edu.cn]
 * @since JDK1.5
 */
public class ConcreteTopicExpressionCompiler implements
		ITopicExpressionCompiler {

	public ConcreteTopicExpressionCompiler() {
		super();
	}

	public String getSupportedDialectName() {
		return TopicConstants.CONCRETE_TOPICEXPRESSION;
	}

	/**
	 * вҴ˱ʽӦ
	 * Concrete ͵ıʽһʽָһ
	 * Ҳ򷵻null
	 * @param expression
	 * @return
	 * @throws InvalidTopicExpressionException
	 * @throws TopicNotSupportedException
	 * @see org.act.xservices.rt.xnotification.topic.expression.ITopicExpressionCompiler#compile(org.act.xservices.rt.xnotification.topic.expression.TopicExpression)
	 */
	public ITopic[] compile(ITopicExpression expression,
			ITopicTreesSet topicTreesSet)
			throws InvalidTopicExpressionException, TopicNotSupportedException {

		Map<String,String> tpNSMap = expression.getTopicNamespaceMap();
		
		// ѣ
		String[] s = TopicExpressionHelper.splitExpression(expression.getContent());

		String namespaceuri = null;
		// жֿռlocalǲǶ
		namespaceuri = tpNSMap.get(s[0]);
		
		System.out.println("ConcreteTopicExpressionCompiler.compile():"+ namespaceuri );
		
		
		if (namespaceuri == null) {
			throw new TopicNotSupportedException("TopicNotSupportedException",
					Calendar.getInstance(),
					"TopicNotSupportedException in namespace in soap Message");
			// return null;
		}

		List topiclist = TopicExpressionHelper.splitExpressionConcrete(s[1]);

		ITopicNamespace namespace = topicTreesSet.getTopicTreeByTopicNamespace(namespaceuri);

		if (namespace == null) {
			// ׳쳣ʾִֿ֧ռ䣮
			throw new TopicNotSupportedException("TopicNotSupportedException",
					Calendar.getInstance(),
					"TopicNotSupportedException in namespace");
		}

		if (topiclist.size() == 0) {
			// ׳쳣ʾִ֧˱ʽ
			throw new TopicNotSupportedException("TopicNotSupportedException",
					Calendar.getInstance(),
					"TopicNotSupportedException in Topic Expression");
		}
		
		String root = (String) topiclist.get(0);
		// жûroot
		ITopic topic = namespace.getTopic(root);
		if(topic == null){
			throw new TopicNotSupportedException(
					"TopicNotSupportedException", Calendar.getInstance(),
					"TopicNotSupportedException in compiling"+ root);
		}
		
		for (int i = 1; i < topiclist.size(); i++) {

			String topicname = (String) topiclist.get(i);
			if (topic.containsTopic(topicname)) {
				topic = topic.getTopic(topicname);
			} else {
				// ׳쳣ʾ֧
				throw new TopicNotSupportedException(
						"TopicNotSupportedException", Calendar.getInstance(),
						"TopicNotSupportedException in compiling");
			}

		}
		
		if(topic == null){
			return null;
		}
		//Concrete ͵ıʽһʽָһ
		return new ITopic[]{topic};
	}

	
	

	/** 
	 * 
	 * @param topicExpression
	 * @param topicTreesSet
	 * @return
	 * @throws InvalidTopicExpressionException
	 * @throws TopicNotSupportedException
	 * @see org.act.xservices.rt.xnotification.topic.expression.ITopicExpressionCompiler#add(org.act.xservices.rt.xnotification.topic.expression.ITopicExpression, org.act.xservices.rt.xnotification.base.ITopicTreesSet)
	 */
	public ITopic add(ITopicExpression topicExpression,ITopicTreesSet topicTreesSet)
			throws InvalidTopicExpressionException, TopicNotSupportedException {


		Map<String,String> tpNSMap = topicExpression.getTopicNamespaceMap();
		//ѣ
		String[] s = TopicExpressionHelper.splitExpression(topicExpression.getContent());

		String namespaceuri = null;

		namespaceuri = tpNSMap.get(s[0]);
		
		System.out.println("ConcreteTopicExpressionCompiler.add():"+ topicExpression.getContent() );
		
		if (namespaceuri == null) {
			throw new TopicNotSupportedException("TopicNotSupportedException",Calendar.getInstance(),
					"TopicNotSupportedException in namespace in soap Message");
			// return null;
		}

		ITopicNamespace namespace = TopicNamespaceFactory.getInstance()
				.getTopicNamespace(s[0], namespaceuri, topicTreesSet);

		if (namespace == null) {
			throw new TopicNotSupportedException("TopicNotSupportedException",
					Calendar.getInstance(),"TopicNotSupportedException in namespace");
		}

		List topiclist = TopicExpressionHelper.splitExpressionConcrete(s[1]);

		if (topiclist.size() == 0) {
			// ׳쳣ʾִ֧˱ʽ
			throw new TopicNotSupportedException("TopicNotSupportedException",
					Calendar.getInstance(), "ûдκεTopicTopic Expression");
		}

		String root = (String) topiclist.get(0);
		//Ӹ
		if (topiclist.size() == 1) {
			ITopic topicroot = namespace.addTopic(root);
			return topicroot;
		}

		//һ⣬ĳⲻ򱨴
		ITopic topic = namespace.getTopic(root);
		if(topic == null){
			throw new TopicNotSupportedException(
					"TopicNotSupportedException", Calendar.getInstance(),
					"TopicNotSupportedException in compiling"+ root);
		}
		for (int i = 1; i < topiclist.size() - 1; i++) {
			String topicname = (String) topiclist.get(i);
			if (topic.containsTopic(topicname)) {
				topic = topic.getTopic(topicname);
			} else {
				// ׳쳣ʾ֧
				throw new TopicNotSupportedException(
						"TopicNotSupportedException", Calendar.getInstance(),
						"TopicNotSupportedException in compiling");
			}

		}
		
		return topic.addTopic((String) topiclist.get(topiclist.size() - 1));
	}
	
	
	/**
	 * TopicTreesSetʽָ
	 * ڸ⣬й
	 * 
	 * @param expression
	 * @param topicTreesSet
	 * @return
	 * @throws InvalidTopicExpressionException
	 * @throws TopicNotSupportedException
	 */
	public ITopic addTryBest(ITopicExpression topicExpression, ITopicTreesSet topicTreesSet)
	throws InvalidTopicExpressionException, TopicNotSupportedException{
		
		Map<String,String> tpNSMap = topicExpression.getTopicNamespaceMap();
		//ѣ
		String[] s = TopicExpressionHelper.splitExpression(topicExpression.getContent());

		String namespaceuri = null;

		namespaceuri = tpNSMap.get(s[0]);
		
		System.out.println("ConcreteTopicExpressionCompiler.addTryBest():"+ topicExpression.getContent() );
		
		if (namespaceuri == null) {
			throw new TopicNotSupportedException("TopicNotSupportedException",Calendar.getInstance(),
					"TopicNotSupportedException in namespace in soap Message");
			// return null;
		}

		ITopicNamespace namespace = TopicNamespaceFactory.getInstance()
				.getTopicNamespace(s[0], namespaceuri, topicTreesSet);

		if (namespace == null) {
			throw new TopicNotSupportedException("TopicNotSupportedException",
					Calendar.getInstance(),"TopicNotSupportedException in namespace");
		}

		List topiclist = TopicExpressionHelper.splitExpressionConcrete(s[1]);

		if (topiclist.size() == 0) {
			// ׳쳣ʾִ֧˱ʽ
			throw new TopicNotSupportedException("TopicNotSupportedException",
					Calendar.getInstance(), "ûдκεTopicTopic Expression");
		}

		String root = (String) topiclist.get(0);
		//Ӹ
		if (topiclist.size() == 1) {
			ITopic topicroot = namespace.addTopic(root);
			return topicroot;
		}

		//һ⣬ĳⲻ
		ITopic topic = namespace.getTopic(root);
		if(topic == null){
			topic = namespace.addTopic(root);
//			throw new TopicNotSupportedException(
//					"TopicNotSupportedException", Calendar.getInstance(),
//					"TopicNotSupportedException in compiling"+ root);
		}
		for (int i = 1; i < topiclist.size() - 1; i++) {
			String topicname = (String) topiclist.get(i);
			if (topic.containsTopic(topicname)) {
				topic = topic.getTopic(topicname);
			} else {
				//ڣй
				topic = topic.addTopic(topicname);
			}

		}
		return topic.addTopic((String) topiclist.get(topiclist.size() - 1));
	}
	



	
	/** 
	 * ɾʽ
	 * @param topicExpression
	 * @param topicTreesSet
	 * @return ɾʧ
	 * @throws TopicNotSupportedException
	 * @throws InvalidTopicExpressionException
	 * @see org.act.xservices.rt.xnotification.topic.expression.ITopicExpressionCompiler#remove(org.act.xservices.rt.xnotification.topic.expression.ITopicExpression, org.act.xservices.rt.xnotification.base.ITopicTreesSet)
	 */
	public boolean remove(ITopicExpression topicExpression,
			ITopicTreesSet topicTreesSet) throws TopicNotSupportedException,
			InvalidTopicExpressionException {

		//topiclistŵȫǽtopic
		Map<String,String> tpNSMap = topicExpression.getTopicNamespaceMap();
		//ѣ
		String[] s = TopicExpressionHelper.splitExpression(topicExpression.getContent());

		String namespaceuri = null;

		namespaceuri = tpNSMap.get(s[0]);
		
		System.out.println("ConcreteTopicExpressionCompiler.remove():"+ namespaceuri );
		
		if (namespaceuri == null) {
			throw new TopicNotSupportedException("TopicNotSupportedException",
					Calendar.getInstance(),"TopicNotSupportedException in namespace in soap Message");
		}

		ITopicNamespace namespace = topicTreesSet.getTopicTreeByTopicNamespace(namespaceuri);

		if (namespace == null) {
			throw new TopicNotSupportedException("TopicNotSupportedException",
					Calendar.getInstance(),
					"TopicNotSupportedException in namespace");
		}

		List<String> topiclist = TopicExpressionHelper.splitExpressionConcrete(s[1]);

		if (topiclist.size() == 0) {
			// ׳쳣ʾִ֧˱ʽ
			throw new TopicNotSupportedException("TopicNotSupportedException",
					Calendar.getInstance(), "ûдκεTopicTopic Expression");
		}

		String root = topiclist.get(0);
		
		if (topiclist.size() == 1) {//
			namespace.removeTopic(root);
			return true;
		}

		
		ITopic topic = namespace.getTopic(root);
		if(topic == null){
			throw new TopicNotSupportedException(
					"TopicNotSupportedException", Calendar.getInstance(),
					"TopicNotSupportedException in compiling"+ root);
		}
		//·
		for (int i = 1; i < topiclist.size() - 1; i++) {
			String topicname = topiclist.get(i);
			if (topic.containsTopic(topicname)) {
				topic = topic.getTopic(topicname);

			} else {
				// ׳쳣ʾ֧
				throw new TopicNotSupportedException(
						"TopicNotSupportedException", Calendar.getInstance(),
						"TopicNotSupportedException in compiling");
			}
		}
		
		//鿴ҪɾǷ
		if(!topic.containsTopic(topiclist.get(topiclist.size() - 1))){
			return false; //Ҫɾⲻڣɾʧ
		}
		
		//ɾʽ
		topic.removeTopic((String) topiclist.get(topiclist.size() - 1));
		return true;
	}
	
	
	
	/**
	 * ڸTopicTreesSet Ŭ ɾ ʽ 
	 * Ҳ⣬trueⲻ
	 * @param topicExpression
	 * @param namespace
	 * @param topic
	 * @return
	 * @throws InvalidTopicExpressionException
	 * @throws TopicNotSupportedException
	 */
	public boolean removeTryBest(ITopicExpression topicExpression, ITopicTreesSet topicTreesSet)
			throws InvalidTopicExpressionException, TopicNotSupportedException{
		
		//topiclistŵȫǽtopic
		Map<String,String> tpNSMap = topicExpression.getTopicNamespaceMap();
		//ѣ
		String[] s = TopicExpressionHelper.splitExpression(topicExpression.getContent());

		String namespaceuri = null;

		namespaceuri = tpNSMap.get(s[0]);
		
		System.out.println("ConcreteTopicExpressionCompiler.removeTryBest():"+ namespaceuri );
		
		if (namespaceuri == null) {
			throw new TopicNotSupportedException("TopicNotSupportedException",
					Calendar.getInstance(),"TopicNotSupportedException in namespace in soap Message");
		}

		ITopicNamespace namespace = topicTreesSet.getTopicTreeByTopicNamespace(namespaceuri);

		if (namespace == null) {
			throw new TopicNotSupportedException("TopicNotSupportedException",
					Calendar.getInstance(),
					"TopicNotSupportedException in namespace");
		}

		List<String> topiclist = TopicExpressionHelper.splitExpressionConcrete(s[1]);

		if (topiclist.size() == 0) {
			// ׳쳣ʾִ֧˱ʽ
			throw new TopicNotSupportedException("TopicNotSupportedException",
					Calendar.getInstance(), "ûдκεTopicTopic Expression");
		}

		String root = topiclist.get(0);
		if (topiclist.size() == 1) {
			namespace.removeTopic(root);
			return true;
		}

		
		ITopic topic = namespace.getTopic(root);
		if(topic == null){
			return true;
		}
		
		//·
		for (int i = 1; i < topiclist.size() - 1; i++) {
			String topicname = topiclist.get(i);
			if (topic.containsTopic(topicname)) {
				topic = topic.getTopic(topicname);

			} else {
				return true; //·ʧܣʾⲻ
			}
		}
		
		//鿴ҪɾǷ
		if(!topic.containsTopic(topiclist.get(topiclist.size() - 1))){
			return true; //Ҫɾⲻ
		}
		
		//ɾʽ
		topic.removeTopic((String) topiclist.get(topiclist.size() - 1));
		return true;
	}
	
	
	

}
