
package org.act.xservices.rt.xnotification.topic.expression;

import java.util.HashMap;
import java.util.Map;

import org.act.xservices.rt.xnotification.base.ITopicTreesSet;
import org.act.xservices.rt.xnotification.base.fault.InvalidTopicExpressionException;
import org.act.xservices.rt.xnotification.base.fault.TopicNotSupportedException;
import org.act.xservices.rt.xnotification.topic.ITopic;
import org.act.xservices.rt.xnotification.topic.exception.UnsupportedTopicExpressionException;


/**
 * 
 * Ĺڸ͵עᣮ
 * 
 * Ҫ̰߳ȫҪǵЩԴֵЩ
 * ˴ǶTopicڵģңԲû⣮
 * ȡCompilerĹܻ̿漰⣮
 * 
 * @version 1.0 2008-3-4
 * @author cover_eye wangyg@act.buaa.edu.cn 2006-6-14
 * @author Lizq [lizq@act.buaa.edu.cn]
 * @since JDK1.5
 */

public class TopicCompileEngine implements ITopicCompileEngine {
	
	private static TopicCompileEngine INSTANCE = new TopicCompileEngine();

	private Map<String, ITopicExpressionCompiler> topicExpressionCompilerMap = 
		new HashMap<String, ITopicExpressionCompiler>();

	/**
	 * ڹ췽ѾעĬֵ֧ģԣţ壮 
	 * ͨķµģԣ̣壮
	 */
	private TopicCompileEngine() {
		registerTopicCompiler(new SimpleTopicExpressionCompiler());
		registerTopicCompiler(new ConcreteTopicExpressionCompiler());
	}

	public static TopicCompileEngine getTopicEngine() {
		return INSTANCE;
	}

	
	/** 
	 * ڸTopicTreesSetϽʽ 
	 * 
	 * @param expression
	 * @param topicTreesSet
	 * @return
	 * @throws UnsupportedTopicExpressionException
	 * @throws InvalidTopicExpressionException
	 * @throws TopicNotSupportedException
	 * @see org.act.xservices.rt.xnotification.topic.expression.ITopicCompileEngine#compile(org.act.xservices.rt.xnotification.topic.expression.ITopicExpression, org.act.xservices.rt.xnotification.base.ITopicTreesSet)
	 */
	public ITopic[] compile(ITopicExpression expression,
			ITopicTreesSet topicTreesSet)
			throws UnsupportedTopicExpressionException,
			InvalidTopicExpressionException, TopicNotSupportedException {

		String name = expression.getDialect();
		ITopicExpressionCompiler compiler = (ITopicExpressionCompiler) topicExpressionCompilerMap
				.get(name);
		if (compiler == null) {
			throw new UnsupportedTopicExpressionException("ûֵ֧Topic"
					+ name);
		}

		return compiler.compile(expression, topicTreesSet);
	}

	
	/** 
	 * ע
	 * @param compiler
	 * @see org.act.xservices.rt.xnotification.topic.expression.ITopicCompileEngine#registerTopicCompiler(org.act.xservices.rt.xnotification.topic.expression.ITopicExpressionCompiler)
	 */
	public void registerTopicCompiler(ITopicExpressionCompiler compiler) {
		topicExpressionCompilerMap.put(compiler.getSupportedDialectName(),compiler);

	}

	/**
	 * ôֵ֧
	 * @return
	 */
	public ITopicExpressionCompiler[] getSupportedTopicCompiler() {
		return (ITopicExpressionCompiler[]) topicExpressionCompilerMap
				.entrySet().toArray();
	}

	
	/** 
	 * Ӵɾ
	 * @param expression
	 * @return
	 * @see org.act.xservices.rt.xnotification.topic.expression.ITopicCompileEngine#removeTopicCompiler(org.act.xservices.rt.xnotification.topic.expression.ITopicExpressionCompiler)
	 */
	public boolean removeTopicCompiler(ITopicExpressionCompiler expression) {

		Object ob = topicExpressionCompilerMap.remove(expression
				.getSupportedDialectName());
		if (ob == null)
			return false;
		return true;
	}

	
	public String[] getSupportedTopicCompilerName() {
		return (String[]) topicExpressionCompilerMap.keySet().toArray();
	}

	
	public ITopic addTopic(ITopicExpression expression, ITopicTreesSet topicTreesSet)
			throws UnsupportedTopicExpressionException,InvalidTopicExpressionException, 
			TopicNotSupportedException {
		
		String name = expression.getDialect();
		ITopicExpressionCompiler compiler = (ITopicExpressionCompiler)topicExpressionCompilerMap.get(name);
		
		if (compiler == null) {
			throw new UnsupportedTopicExpressionException("ûֵ֧Topic" + name);
		}
		
		return compiler.add(expression, topicTreesSet);
	}
	
	
	/**
	 * TopicTreesSetʽָ
	 * ڸ⣬й
	 * ˴صעܹ޷ɾ⣬Ϊûбʽָдǣ by lizq
	 * @param expression
	 * @param topicTreesSet
	 * @return
	 * @throws UnsupportedTopicExpressionException
	 * @throws InvalidTopicExpressionException
	 * @throws TopicNotSupportedException
	 */
	public ITopic addTopicTryBest(ITopicExpression expression, ITopicTreesSet topicTreesSet)
	throws UnsupportedTopicExpressionException,InvalidTopicExpressionException,TopicNotSupportedException{
		String name = expression.getDialect();
		ITopicExpressionCompiler compiler = (ITopicExpressionCompiler)topicExpressionCompilerMap.get(name);
		if (compiler == null) {
			throw new UnsupportedTopicExpressionException("ûֵ֧Topic" + name);
		}
		return compiler.addTryBest(expression, topicTreesSet);
	}
	
	

	
	/** 
	 * 
	 * @param topicExpression
	 * @param topicTreesSet
	 * @return
	 * @throws UnsupportedTopicExpressionException
	 * @throws InvalidTopicExpressionException
	 * @throws TopicNotSupportedException
	 * @see org.act.xservices.rt.xnotification.topic.expression.ITopicCompileEngine#removeTopic(org.act.xservices.rt.xnotification.topic.expression.ITopicExpression, org.act.xservices.rt.xnotification.base.ITopicTreesSet)
	 */
	public boolean removeTopic(ITopicExpression topicExpression,
			ITopicTreesSet topicTreesSet)
			throws UnsupportedTopicExpressionException,
			InvalidTopicExpressionException, TopicNotSupportedException {
		String name = topicExpression.getDialect();
		ITopicExpressionCompiler compiler = 
			(ITopicExpressionCompiler) topicExpressionCompilerMap.get(name);
		if (compiler == null) {
			throw new UnsupportedTopicExpressionException("ûֵ֧Topic"+ name);
		}
		return compiler.remove(topicExpression, topicTreesSet);
	}
	
	
	/**
	 * TopicTreesSetɾʽָ
	 * ɾⲻڣtrue
	 * 
	 * @param ITopicExpression
	 * @param topic
	 * @return
	 * @throws UnsupportedITopicExpressionException
	 * @throws TopicNotSupportedException
	 * @throws InvalidITopicExpressionException
	 */
	public boolean removeTopicTryBest(ITopicExpression topicExpression,
			ITopicTreesSet topicTreesSet)
			throws UnsupportedTopicExpressionException,
			InvalidTopicExpressionException, TopicNotSupportedException{
		
		String name = topicExpression.getDialect();
		ITopicExpressionCompiler compiler = 
			(ITopicExpressionCompiler) topicExpressionCompilerMap.get(name);
		if (compiler == null) {
			throw new UnsupportedTopicExpressionException("ûֵ֧Topic"+ name);
		}
		return compiler.removeTryBest(topicExpression, topicTreesSet);
	}
	
}
