package org.act.xservices.rt.soap;

import java.util.List;

/**
 * @author Viking.Liu
 * @since JDK 1.5
 * @date 2010-6-5
 * @version 1.0.0
 * 
 */
/**
 * This interface represents the thing which provides us with the SOAP roles in
 * a given context. Used to search through SOAP header blocks.
 */
public interface IRolePlayer {
	/**
	 * Get a list of the roles supported. NOTE: the "next" role is automatically
	 * supported, and the "none" role (for SOAP 1.2) is automatically
	 * disallowed. The roles returned should only be the non-standard ones,
	 * since the ultimate destination role is also handled by the
	 * isUltimateDestination method below.
	 * 
	 * @return a List containing URI Strings, one per custom role supported, or
	 *         null
	 */
	List<String> getRoles();

	/**
	 * Are we the ultimate destination?
	 * 
	 * @return true if this is the ultimate destination, false if an
	 *         intermediary.
	 */
	boolean isUltimateDestination();
	
	/**
	 * Add a role URI
	 * @param role
	 */
	void addRole(String role);
}
