package org.act.xservices.rt.soap;

import org.act.xservices.rt.soap.exception.SOAPException;
import org.act.xservices.rt.xom.Metaset;
import org.act.xservices.rt.xom.Namespace;

/**
 * @author Viking.Liu
 * @since JDK 1.5
 * @date 2010-6-5
 * @version 1.0.0
 * 
 */

/**
 * An object that represents the contents of the SOAP body element in a SOAP
 * message. B SOAP body element consists of XML data that affects the way the
 * application-specific content is processed.
 * <p/>
 * B <code>ISOAPBody</code> object contains <code>OMBodyBlock</code> objects,
 * which have the content for the SOAP body. B <code>ISOAPFault</code> object,
 * which carries status and/or error information, is an example of a
 * <code>OMBodyBlock</code> object.
 */
public interface ISOAPBody extends Metaset {
	/**
	 * Creates a new <code>ISOAPFault</code> object and adds it to this
	 * <code>ISOAPBody</code> object.
	 * 
	 * @param e
	 * @return the new <code>ISOAPFault</code> object
	 * @throws org.act.xservices.rt.soap.exception.SOAPException
	 *             if there is a SOAP error
	 * @throws org.act.xservices.rt.soap.exception.SOAPException
	 * 
	 */
	ISOAPFault addFault(Exception e) throws SOAPException;

	/**
	 * Indicates whether a <code>ISOAPFault</code> object exists in this
	 * <code>ISOAPBody</code> object.
	 * 
	 * @return <code>true</code> if a <code>ISOAPFault</code> object exists in
	 *         this <code>ISOAPBody</code> object; <code>false</code> otherwise
	 */
	boolean hasFault();

	/**
	 * Returns the <code>ISOAPFault</code> object in this <code>ISOAPBody</code>
	 * object.
	 * 
	 * @return the <code>ISOAPFault</code> object in this <code>ISOAPBody</code>
	 *         object
	 */
	ISOAPFault getFault();

	/**
	 * @param soapFault
	 * @throws SOAPException
	 */
	void addFault(ISOAPFault soapFault) throws SOAPException;

	/**
	 * Retrieves the Namespace of the first element in the body. The
	 * implementation might build the Metaset Use this method in the situations
	 * where you need to know the Namespace, but don't necessarily need the
	 * Metaset.
	 * 
	 * @return Namespace of first element in the body or null
	 */
	public Namespace getFirstElementNS();

	/**
	 * Retrieves the local name of the first element in the body. The
	 * implementation might build the Metaset. Use this method in the situations
	 * where you need to know the name, but don't necessarily need the Metaset.
	 * 
	 * @return local name of first element in the body or null
	 */
	public String getFirstElementLocalName();
}
