package org.act.xservices.rt.soap;

import org.act.xservices.rt.soap.exception.SOAPException;
import org.act.xservices.rt.xom.Metaset;
import org.act.xservices.rt.xom.Namespace;

/**
 * @author Viking.Liu
 * @since JDK 1.5
 * @date 2010-6-5
 * @version 1.0.0
 * 
 */

/** Interface SOAPEnvelope */
public interface ISOAPEnvelope extends Metaset {
	/**
	 * Returns the <CODE>ISOAPHeader</CODE> object for this
	 * <CODE> ISOAPEnvelope</CODE> object.
	 * <P>
	 * This ISOAPHeader will just be a container for all the headers in the
	 * <CODE>ISOAPMessage</CODE>
	 * </P>
	 * 
	 * @return the <CODE>ISOAPHeader</CODE> object or <CODE> null</CODE> if
	 *         there is none
	 * @throws org.act.xservices.rt.soap.exception.SOAPException
	 *             if there is a problem obtaining the <CODE>ISOAPHeader</CODE>
	 *             object
	 */
	ISOAPHeader getHeader() throws SOAPException;

	/**
	 * 
	 * @param header
	 */
	void addHeader(ISOAPHeader header)throws SOAPException;

	/**
	 * Returns the <CODE>SOAPBody</CODE> object associated with this
	 * <CODE>ISOAPEnvelope</CODE> object.
	 * <P>
	 * This SOAPBody will just be a container for all the BodyElements in the
	 * <CODE>ISOAPMessage</CODE>
	 * </P>
	 * 
	 * @return the <CODE>SOAPBody</CODE> object for this
	 *         <CODE> ISOAPEnvelope</CODE> object or <CODE>null</CODE> if there
	 *         is none
	 * @throws SOAPException
	 *             if there is a problem obtaining the <CODE>SOAPBody</CODE>
	 *             object
	 */
	ISOAPBody getBody() throws SOAPException;

	/**
	 * 
	 * @param body
	 */
	void addBody(ISOAPBody body)throws SOAPException;

	/**
	 * Gets the SOAP version SOAP1.1 or SOAP1.2
	 * 
	 * @return SOAP11Version or SOAP12Version
	 */
	ISOAPVersion getVersion();

	/**
	 * Sets the SOAP Version
	 * 
	 * @param version
	 *            SOAP11Version or SOAP12Version
	 */
	void setVersion(ISOAPVersion version);

	/**
	 * Returns true if there is a SOAPFault in the body. The implementation may
	 * choose to get this information by building the dom tree or use parser
	 * provided information.
	 * 
	 * @return true if SOAPFault in the body
	 */
	boolean hasFault();

	/**
	 * Retrieves the Namespace of the first element in the body. The
	 * implementation might build the Metaset . Use this method in the
	 * situations where you need to know the Namespace, but don't necessarily
	 * need the Metaset.
	 * 
	 * @return Namespace of first element in the body or null
	 */
	Namespace getSOAPBodyFirstElementNS();

	/**
	 * Retrieves the local name of the first element in the body. The
	 * implementation might build the Metaset or it may obtain this information
	 * from the builder/parser without building the Metaset. Use this method in
	 * the situations where you need to know the name, but don't necessarily
	 * need the Metaset.
	 * 
	 * @return local name of first element in the body or null
	 */
	String getSOAPBodyFirstElementLocalName();
}
