package org.act.xservices.rt.soap;

import org.act.xservices.rt.soap.exception.SOAPException;
import org.act.xservices.rt.soap.exception.SOAPProcException;
/**
 * @author Viking.Liu
 * @since JDK 1.5
 * @date 2010-6-5
 * @version 1.0.0
 * 
 */
import org.act.xservices.rt.xom.Metaset;

/**
 * An element in the <CODE>ISOAPBody</CODE> object that contains error and/or status information.
 * This information may relate to errors in the <CODE>ISOAPMessage</CODE> object or to problems that
 * are not related to the content in the message itself. Problems not related to the message itself
 * are generally errors in processing, such as the inability to communicate with an upstream
 * server.
 * <p/>
 * The <CODE>ISOAPFault</CODE> interface provides methods for retrieving the information contained in
 * a <CODE> ISOAPFault</CODE> object and for setting the fault code, the fault actor, and a string
 * describing the fault. B fault code is one of the codes defined in the SOAP 1.1 specification that
 * describe the fault. An actor is an intermediate recipient to whom a message was routed. The
 * message path may include one or more actors, or, if no actors are specified, the message goes
 * only to the default actor, which is the final intended recipient.
 */
public interface ISOAPFault extends Metaset {

    /**
     * SOAPFaultCode is a mandatory item in a Fault, in SOAP 1.2 specification
     *
     * @param soapFaultCode
     */
    void addFaultCode(ISOAPFaultCode soapFaultCode) throws SOAPProcException;

    /**
     * Gets the SOAPFaultCode 
     * @return
     */
    ISOAPFaultCode getFaultCode();

    /**
     * SOAPFaultReason is a mandatory item in a Fault, in SOAP 1.2 specification
     *
     * @param reason
     */
    void addFaultReason(ISOAPFaultReason soapFaultReason) throws SOAPProcException;
    
    ISOAPFaultReason getFaultReason();

    /**
     * SOAPFaultNode is an optional item in a Fault, in SOAP 1.2 specification
     *
     * @param soapFaultNode
     */
    void addFaultNode(ISOAPFaultNode soapFaultNode) throws SOAPProcException;

    ISOAPFaultNode getFaultNode();

    /**
     * SOAPFaultRoleImpl is an optional item in a Fault, in SOAP 1.2 specification
     *
     * @param soapFaultRole
     */
    void addFaultRole(ISOAPFaultRole soapFaultRole) throws SOAPProcException;

    ISOAPFaultRole getFaultRole();

    /**
     * SOAPFaultRoleImpl is an optional item in a Fault, in SOAP 1.2 specification
     *
     * @param soapFaultDetail
     */
    void addFaultDetail(ISOAPFaultDetail soapFaultDetail) throws SOAPProcException;

    ISOAPFaultDetail getFaultDetail();

    /**
     * Returns Exception if there is one in the SOAP fault.
     * <p/>
     * If the exception is like; <ISOAPFault> <Detail> <Exception> stack trace goes here </Exception>
     * </Detail> </ISOAPFault>
     *
     * @return Returns Exception.
     * @throws org.act.xservices.rt.soap.exception.SOAPException
     *
     */
    Exception getException() throws SOAPException;

    void setException(Exception e) throws SOAPException;
}
