package org.act.xservices.rt.soap;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.act.xservices.rt.soap.exception.SOAPException;
import org.act.xservices.rt.xom.Metaset;
import org.act.xservices.rt.xom.Namespace;

/**
 * @author Viking.Liu
 * @since JDK 1.5
 * @date 2010-6-5
 * @version 1.0.0
 * 
 */
/** Interface SOAPHeader */
public interface ISOAPHeader extends Metaset {
	/**
	 * Creates a new <CODE>ISOAPHeaderBlock</CODE> object initialized with the
	 * specified name and adds it to this <CODE>ISOAPHeader</CODE> object.
	 * 
	 * @param localName
	 * @param ns
	 * @return the new <CODE>ISOAPHeaderBlock</CODE> object that was inserted
	 *         into this <CODE>ISOAPHeader</CODE> object
	 * @throws org.act.xservices.rt.soap.exception.SOAPException
	 *             if a SOAP error occurs
	 */
	ISOAPHeaderBlock addHeaderBlock(String localName, Namespace ns)
			throws SOAPException;

	/**
	 * Get the appropriate set of headers for a RolePlayer.
	 * 
	 * The RolePlayer indicates whether it is the ultimate destination (in which
	 * case headers with no role or the explicit UltimateDestination role will
	 * be included), and any non-standard roles it supports. Headers targeted to
	 * "next" will always be included, and those targeted to "none" (for SOAP
	 * 1.2) will never be included.
	 * 
	 * @return an Iterator over all the HeaderBlocks this RolePlayer should
	 *         process.
	 */
	Iterator<ISOAPHeaderBlock> getHeadersToProcess(IRolePlayer rolePlayer);

	/**
	 * Returns a list of all the <CODE>ISOAPHeaderBlock</CODE> objects in this
	 * <CODE>ISOAPHeader</CODE> object that have the the specified role. An role
	 * is a global attribute that indicates the intermediate parties to whom the
	 * message should be sent. An role receives the message and then sends it to
	 * the next role. The default role is the ultimate intended recipient for
	 * the message, so if no role attribute is included in a
	 * <CODE>ISOAPHeader</CODE> object, the message is sent to its ultimate
	 * destination.
	 * 
	 * @param role
	 *            a <CODE>String</CODE> giving the URI of the role for which to
	 *            search
	 * @return Returns an <CODE>Iterator</CODE> object over all the
	 *         <CODE> ISOAPHeaderBlock</CODE> objects that contain the specified
	 *         role
	 * @see #extractHeaderBlocks(String) extractHeaderBlocks(java.lang.String)
	 */
	Iterator<ISOAPHeaderBlock> examineHeaderBlocks(String role);

	/**
	 * Returns a list of all the <CODE>ISOAPHeaderBlock</CODE> objects in this
	 * <CODE>ISOAPHeader</CODE> object that have the the specified role and
	 * detaches them from this <CODE>ISOAPHeader</CODE> object.
	 * <p/>
	 * This method allows an role to process only the parts of the
	 * <CODE> ISOAPHeader</CODE> object that apply to it and to remove them
	 * before passing the message on to the next role.
	 * 
	 * @param role
	 *            a <CODE>String</CODE> giving the URI of the role for which to
	 *            search
	 * @return Returns an <CODE>Iterator</CODE> object over all the
	 *         <CODE> ISOAPHeaderBlock</CODE> objects that contain the specified
	 *         role
	 * @see #examineHeaderBlocks(String) examineHeaderBlocks(java.lang.String)
	 */
	Iterator<ISOAPHeaderBlock> extractHeaderBlocks(String role);

	/**
	 * Returns an <code>Iterator</code> over all the
	 * <code>ISOAPHeaderBlock</code> objects in this <code>ISOAPHeader</code>
	 * object that have the specified role and that have a MustUnderstand
	 * attribute whose value is equivalent to <code>true</code>.
	 * 
	 * @param role
	 *            a <code>String</code> giving the URI of the role for which to
	 *            search
	 * @return Returns an <code>Iterator</code> object over all the
	 *         <code>ISOAPHeaderBlock</code> objects that contain the specified
	 *         role and are marked as MustUnderstand.
	 */
	Iterator<ISOAPHeaderBlock> examineMustUnderstandHeaderBlocks(String role);

	/**
	 * Returns an <code>Iterator</code> over all the
	 * <code>ISOAPHeaderBlock</code> objects in this <code>ISOAPHeader</code>
	 * object.
	 * 
	 * @return an <code>Iterator</code> object over all the
	 *         <code>ISOAPHeaderBlock</code> objects contained by this
	 *         <code>ISOAPHeader</code>. If there are no header blocks then an
	 *         empty iterator is returned.
	 */
	Iterator<ISOAPHeaderBlock> examineAllHeaderBlocks();

	/**
	 * Returns an <code>Iterator</code> over all the
	 * <code>ISOAPHeaderBlock</code> objects in this <code>ISOAPHeader </code>
	 * object and detaches them from this <code>ISOAPHeader</code> object.
	 * 
	 * @return Returns an <code>Iterator</code> object over all the
	 *         <code>ISOAPHeaderBlock</code> objects contained by this
	 *         <code>ISOAPHeader</code>
	 */
	Iterator<ISOAPHeaderBlock> extractAllHeaderBlocks();

	/**
	 * Return all the Headers that has the Namespace URI to given NS URI.
	 * 
	 * @param nsURI
	 * @return Returns ArrayList.
	 */
	ArrayList<ISOAPHeaderBlock> getHeaderBlocksWithNSURI(String nsURI);

	/**
	 * Get the appropriate set of headers for a RolePlayer in a particular
	 * namespace
	 * <p/>
	 * The RolePlayer indicates whether it is the ultimate destination (in which
	 * case headers with no role or the explicit UltimateDestination role will
	 * be included), and any non-standard roles it supports. Headers targeted to
	 * "next" will always be included, and those targeted to "none" (for SOAP
	 * 1.2) will never be included.
	 * <p/>
	 * This version of the API allows us to iterate only once over the headers
	 * searching for a particular namespace for headers targeted at "us".
	 * 
	 * @param rolePlayer
	 *            a RolePlayer containing our role configuration
	 * @param namespace
	 *            if specified, we'll only return headers from this namespace
	 * @return an Iterator over all the HeaderBlocks this RolePlayer should
	 *         process.
	 */
	Iterator<ISOAPHeaderBlock> getHeadersToProcess(IRolePlayer rolePlayer,
			String namespace);
	
	List<ISOAPHeaderBlock> getAllHeaderBlocks();
}
