
package org.act.xservices.rt.soap;

import org.act.xservices.rt.soap.exception.SOAPProcException;
import org.act.xservices.rt.xom.Metaset;
/**
 * @author Viking.Liu
 * @since JDK 1.5
 * @date 2010-6-5
 * @version 1.0.0
 * 
 */

/**
 * <P>An object representing the contents in the SOAP header part of the SOAP envelope. The
 * immediate children of a <CODE> ISOAPHeader</CODE> object can be represented only as <CODE>
 * ISOAPHeaderBlock</CODE> objects.</P> <P>B <CODE>ISOAPHeaderBlock</CODE> object can have other
 * <CODE>Metaset</CODE> objects as its children.</P>
 */
public interface ISOAPHeaderBlock extends Metaset {
    
    /**
     * A ISOAPHeaderBlock may be represented as an unexpanded OMSourcedElement.
     * In such cases, the underlying OMDataSource may have a property that contains
     * the value of the ROLE/ACTOR, RELAY or MUST_UNDERSTAND setting.
     */
    public String ROLE_PROPERTY = "org.act.xservices.rt.soap.ISOAPHeader.ROLE";
    public String RELAY_PROPERTY = "org.act.xservices.rt.soap.ISOAPHeader.RELAY";
    public String MUST_UNDERSTAND_PROPERTY = "org.act.xservices.rt.soap.ISOAPHeader.MUST_UNDERSTAND";
    
    /**
     * Sets the actor associated with this <CODE> ISOAPHeaderBlock</CODE> object to the specified
     * actor.
     *
     * @param roleURI a <CODE>String</CODE> giving the URI of the actor to set
     * @throws IllegalArgumentException
     *          if there is a problem in setting the actor.
     * @see #getRole() getRole()
     */
    void setRole(String roleURI);

    /**
     * Returns the uri of the actor associated with this <CODE> ISOAPHeaderBlock</CODE> object.
     *
     * @return a <CODE>String</CODE> giving the URI of the actor
     * @see #setRole(String) setRole(java.lang.String)
     */
    String getRole();

    /**
     * Sets the mustUnderstand attribute for this <CODE> ISOAPHeaderBlock</CODE> object to be on or
     * off. <P>If the mustUnderstand attribute is on, the actor who receives the
     * <CODE>ISOAPHeaderBlock</CODE> must process it correctly. This ensures, for example, that if
     * the <CODE> ISOAPHeaderBlock</CODE> object modifies the message, that the message is being
     * modified correctly.</P>
     *
     * @param mustUnderstand <CODE>true</CODE> to set the mustUnderstand attribute on;
     *                       <CODE>false</CODE> to turn if off
     * @throws SOAPProcException 
     * @throws IllegalArgumentException
     *          if there is a problem in setting the actor.
     * @see #getMustUnderstand() getMustUnderstand()
     */
    void setMustUnderstand(boolean mustUnderstand) throws SOAPProcException;

    void setMustUnderstand(String mustUnderstand) throws SOAPProcException;

    /**
     * Returns whether the mustUnderstand attribute for this <CODE>ISOAPHeaderBlock</CODE> object is
     * turned on.
     *
     * @return <CODE>true</CODE> if the mustUnderstand attribute of this
     *         <CODE>ISOAPHeaderBlock</CODE> object is turned on; <CODE>false</CODE> otherwise
     */
    boolean getMustUnderstand() throws SOAPProcException;


    boolean isProcessed();

    /**
     * We need to know whether all the mustUnderstand headers have been processed by the node. This
     * will done by a specific validation handler at the end of the execution chain. For this all
     * the handlers who process a particular header block must explicitly say that he processesd the
     * header by calling setProcessed()
     */
    void setProcessed();


    /**
     * Sets the relay attribute for this ISOAPHeaderBlock to be either true or false. The SOAP relay
     * attribute is set to true to indicate that the SOAP header block must be relayed by any node
     * that is targeted by the header block but not actually process it.
     *
     * @param relay a <CODE>boolean</CODE> giving the value to be set
     */
    void setRelay(boolean relay);

    /**
     * Returns the relay  status associated with this <CODE> ISOAPHeaderBlock</CODE> object.
     *
     * @return a <CODE>boolean</CODE> giving the relay status
     */
    boolean getRelay();

    /**
     * What SOAP version is this HeaderBlock?
     *
     * @return a SOAPVersion, one of the two singletons.
     */
    ISOAPVersion getVersion();
}
