package org.act.xservices.rt.soap;

import org.act.xservices.rt.xom.QName;

/**
 * @author Viking.Liu
 * @since JDK 1.5
 * @date 2010-6-5
 * @version 1.0.0
 * 
 */
/**
 * A generic way to get at SOAP-version-specific values. As long as we can get
 * one of these from a SOAP element, we can get at the right
 */
public interface ISOAPVersion {
	/**
	 * Obtain the envelope namespace for this version of SOAP
	 */
	String getEnvelopeURI();

	/**
	 * Obtain the encoding namespace for this version of SOAP
	 */
	String getEncodingURI();

	/**
	 * Obtain the QName for the role attribute (actor/role)
	 */
	QName getRoleAttributeQName();

	/**
	 * Obtain the "next" role/actor URI
	 */
	String getNextRoleURI();

	/**
	 * Obtain the QName for the MustUnderstand fault code
	 */
	QName getMustUnderstandFaultCode();

	/**
	 * Obtain the QName for the Sender fault code
	 * 
	 * @return Sender fault code as a QName
	 */
	QName getSenderFaultCode();

	/**
	 * Obtain the QName for the Receiver fault code
	 * 
	 * @return Receiver fault code as a QName
	 */
	QName getReceiverFaultCode();

	/**
	 * Obtain the QName for the fault reason element
	 * 
	 * @return
	 */
	QName getFaultReasonQName();

	/**
	 * Obtain the QName for the fault code element
	 * 
	 * @return
	 */
	QName getFaultCodeQName();

	/**
	 * Obtain the QName for the fault detail element
	 * 
	 * @return
	 */
	QName getFaultDetailQName();

	/**
	 * Obtain the QName for the fault role/actor element
	 * 
	 * @return
	 */
	QName getFaultRoleQName();
}
