package org.act.xservices.rt.soap.builder;

import java.io.InputStream;
import java.io.Reader;

import org.act.xservices.rt.soap.ISOAPMessage;
import org.act.xservices.rt.soap.exception.SOAPException;
import org.act.xservices.rt.soap.factory.SOAPFactory;

/**
 * 
 * @author Viking.Liu
 * @date 2010-6-10
 * @since JDK 1.5
 * @version 1.0.0
 *
 */
/**
 * This interface provide common method for SOAPBuilder. The
 * <Code>SOAPSAXBuilder</Code> and <Code>SOAPStAXBuilder</Code> implement this
 * interface.
 */
public interface ISOAPBuilder {
	/**
	 * 
	 * @param in
	 *            java.io.InputStream. SOAP message input stream .
	 * @return SOAP message
	 * @throws SOAPException
	 */
	ISOAPMessage readSOAPMessage(InputStream in) throws SOAPException;

	/**
	 * 
	 * @param filepath
	 *            SOAP message file path.
	 * @return SOAP message
	 * @throws SOAPException
	 */
	ISOAPMessage readSOAPMessage(String filepath) throws SOAPException;

	/**
	 * 
	 * @param reader
	 *            SOAP message reader.
	 * @return SOAP message
	 * @throws SOAPException
	 */
	ISOAPMessage readSOAPMessage(Reader reader) throws SOAPException;

	/**
	 * Sets the SOAPFactory. There are two SOAPFactory. One is
	 * <Code>SOAP11Factory</Code> ,and another is <Code>SOAP12Factory</Code>.
	 * 
	 * @param soapFactory
	 */
	void setSOAPFactory(SOAPFactory soapFactory);
}
