package org.act.xservices.rt.soap.builder;

import org.act.xservices.rt.soap.ISOAPHeader;
import org.act.xservices.rt.soap.ISOAPMessage;
import org.act.xservices.rt.soap.constants.SOAP11Constants;
import org.act.xservices.rt.soap.constants.SOAP12Constants;
import org.act.xservices.rt.soap.exception.SOAPProcException;
import org.act.xservices.rt.soap.factory.SOAPFactory;
import org.act.xservices.rt.soap.impl.SOAPMessageImpl;
import org.act.xservices.rt.soap.util.SOAPBuilderhelper;
import org.act.xservices.rt.xom.Branch;
import org.act.xservices.rt.xom.DocumentFactory;
import org.act.xservices.rt.xom.ElementHandler;
import org.act.xservices.rt.xom.Metaset;
import org.act.xservices.rt.xom.QName;
import org.act.xservices.rt.xom.io.SAXContentHandler;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

/**
 * 
 * @author Viking.Liu
 * @date 2010-6-5
 * @since JDK 1.5
 * @version 1.0.0
 * 
 */
public class SOAPSAXContentHandler extends SAXContentHandler {

	/**
	 * SOAP Factory is used to create SOAP Element
	 */
	private SOAPFactory soapFactory;

	/** indicate the deep of the header */
	private int headerDeep = 0;

	/**
	 * SOAP Message
	 */
	private ISOAPMessage message;

	public SOAPSAXContentHandler() {
		super();
	}

	public SOAPSAXContentHandler(SOAPFactory soapFactory,
			ElementHandler elementHandler) {
		super(DocumentFactory.getInstance(), elementHandler);
		this.soapFactory = soapFactory;
	}

	public SOAPSAXContentHandler(SOAPFactory soapFactory) {
		super(DocumentFactory.getInstance());
		this.soapFactory = soapFactory;
	}

	/**
	 * Override DefaultHandler's method startElement to create SOAP Element,not
	 * Dom element.
	 * 
	 */
	@Override
	public void startElement(String namespaceURI, String localName,
			String qualifiedName, Attributes attributes) {
		if (mergeAdjacentText && textInTextBuffer) {
			completeCurrentTextNode();
		}

		QName qName = namespaceStack.getQName(namespaceURI, localName,
				qualifiedName);

		Branch branch = currentElement;

		if (branch == null) {
			branch = getSOAPMessage();
		}
		try {
			// Create a soap Factory
			if (soapFactory == null) {
				if (qName.getNamespaceURI().equals(
						SOAP11Constants.SOAP_ENVELOPE_NAMESPACE_URI)) {
					soapFactory = SOAPFactory.getSOAP11Factory();
				} else if (qName.getNamespaceURI().equals(
						SOAP12Constants.SOAP_ENVELOPE_NAMESPACE_URI)) {
					soapFactory = SOAPFactory.getSOAP12Factory();
				}
			}
			// Create SOAP Element
			Metaset element = SOAPBuilderhelper.createElement(qName,
					soapFactory);

			if (headerDeep != 0)
				headerDeep++;

			if (element instanceof ISOAPHeader) {
				headerDeep = 1;
			} else if (headerDeep == 2) {
				element = soapFactory.createSOAPHeaderBlock(qName);
			}

			// add the new element to it's parent.
			branch.add(element);

			// add all declared namespaces
			addDeclaredNamespaces(element);

			// now lets add all attribute values
			addAttributes(element, attributes);

			elementStack.pushElement(element);
			currentElement = element;

			entity = null; // fixes bug527062

			if (elementHandler != null) {
				elementHandler.onStart(elementStack);
			}
		} catch (SOAPProcException e) {
			e.printStackTrace();
		}
	}

	@Override
	public void endElement(String namespaceURI, String localName, String qName)
			throws SAXException {
		if (mergeAdjacentText && textInTextBuffer) {
			completeCurrentTextNode();
		}

		if (headerDeep != 0)
			headerDeep--;// ջ

		if ((elementHandler != null) && (currentElement != null)) {
			elementHandler.onEnd(elementStack);
		}

		elementStack.popElement();
		currentElement = elementStack.peekElement();
	}

	/**
	 * Add this method to get a SOAP Message
	 * 
	 * @return SOAP Message
	 */
	public ISOAPMessage getSOAPMessage() {
		if (message == null) {
			String encoding = getEncoding();
			message = new SOAPMessageImpl(encoding);
			// set the EntityResolver
			message.setEntityResolver(entityResolver);

			if (inputSource != null) {
				message.setName(inputSource.getSystemId());
			}
		}
		return message;
	}

	/**
	 * Override this method to return a SOAP Message.
	 */
	@Override
	public ISOAPMessage getDocument() {
		return getSOAPMessage();
	}
}
