package org.act.xservices.rt.soap.builder;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.Reader;


import org.act.xservices.rt.soap.ISOAPHeader;
import org.act.xservices.rt.soap.ISOAPMessage;
import org.act.xservices.rt.soap.constants.SOAP11Constants;
import org.act.xservices.rt.soap.constants.SOAP12Constants;
import org.act.xservices.rt.soap.exception.SOAPException;
import org.act.xservices.rt.soap.factory.SOAPFactory;
import org.act.xservices.rt.soap.util.SOAPBuilderhelper;
import org.act.xservices.rt.xom.DocumentException;
import org.act.xservices.rt.xom.DocumentFactory;
import org.act.xservices.rt.xom.Metaset;
import org.act.xservices.rt.xom.QName;
import org.act.xservices.rt.xom.io.XPP3Reader;
import org.xmlpull.v1.XmlPullParser;
import org.xmlpull.v1.XmlPullParserException;

public class SOAPXPPBuilder extends XPP3Reader implements ISOAPBuilder {

	private SOAPFactory soapFactory;

	private int headerDeep = 0;

	public SOAPXPPBuilder() {
		super();
		// TODO Auto-generated constructor stub
	}

	public SOAPXPPBuilder(SOAPFactory factory) {
		super(factory);
		this.soapFactory = factory;
		this.setDocumentFactory(soapFactory);
	}

	public ISOAPMessage readSOAPMessage(InputStream in) throws SOAPException {
		try {
			return readSOAPMessage(createReader(in));
		} catch (IOException e) {
			throw new SOAPException(e);
		}
	}

	public ISOAPMessage readSOAPMessage(String filepath) throws SOAPException {
		try {
			FileInputStream input = new FileInputStream(new File(filepath));
			return readSOAPMessage(input);
		} catch (FileNotFoundException e) {
			throw new SOAPException(e);
		}
	}

	public ISOAPMessage readSOAPMessage(Reader reader) throws SOAPException {
		try {
			getXPPParser().setInput(reader);
			return parseDocument();
		} catch (XmlPullParserException e) {
			throw new SOAPException(e);
		} catch (DocumentException e) {
			throw new SOAPException(e);
		} catch (IOException e) {
			throw new SOAPException(e);
		}
	}

	@Override
	protected ISOAPMessage parseDocument() throws DocumentException,
			IOException, XmlPullParserException {

		DocumentFactory df = getDocumentFactory();
		ISOAPMessage document = SOAPFactory.getInstance().createSOAPMessage();
		Metaset parent = null;
		XmlPullParser pp = getXPPParser();
		pp.setFeature(XmlPullParser.FEATURE_PROCESS_NAMESPACES, true);

		while (true) {
			int type = pp.nextToken();

			switch (type) {
			case XmlPullParser.PROCESSING_INSTRUCTION: {
				String text = pp.getText();
				int loc = text.indexOf(" ");

				if (loc >= 0) {
					String target = text.substring(0, loc);
					String txt = text.substring(loc + 1);
					document.addProcessingInstruction(target, txt);
				} else {
					document.addProcessingInstruction(text, "");
				}

				break;
			}

			case XmlPullParser.COMMENT: {
				if (parent != null) {
					parent.addComment(pp.getText());
				} else {
					document.addComment(pp.getText());
				}

				break;
			}

			case XmlPullParser.CDSECT: {
				if (parent != null) {
					parent.addCDATA(pp.getText());
				} else {
					String msg = "Cannot have text content outside of the "
							+ "root document";
					throw new DocumentException(msg);
				}

				break;
			}

			case XmlPullParser.ENTITY_REF:
				break;

			case XmlPullParser.END_DOCUMENT:
				return document;

			case XmlPullParser.START_TAG: {
				QName qname = (pp.getPrefix() == null) ? df.createQName(pp
						.getName(), pp.getNamespace()) : df.createQName(pp
						.getName(), pp.getPrefix(), pp.getNamespace());

				// Create a soap Factory
				if (qname.getNamespaceURI().equals(
						SOAP11Constants.SOAP_ENVELOPE_NAMESPACE_URI)) {
					soapFactory = SOAPFactory.getSOAP11Factory();
				} else if (qname.getNamespaceURI().equals(
						SOAP12Constants.SOAP_ENVELOPE_NAMESPACE_URI)) {
					soapFactory = SOAPFactory.getSOAP12Factory();
				}
				if (headerDeep != 0) {
					headerDeep++;
				}
				Metaset newElement = null;
				if (headerDeep == 2) {
					newElement = soapFactory.createSOAPHeaderBlock(qname);
				} else {

					// Create a SOAP Element
					newElement = SOAPBuilderhelper.createElement(qname,
							soapFactory);
				}
				if (newElement instanceof ISOAPHeader) {
					headerDeep = 1;
				}

				int nsStart = pp.getNamespaceCount(pp.getDepth() - 1);
				int nsEnd = pp.getNamespaceCount(pp.getDepth());

				for (int i = nsStart; i < nsEnd; i++) {
					if (pp.getNamespacePrefix(i) != null) {
						newElement.addNamespace(pp.getNamespacePrefix(i), pp
								.getNamespaceUri(i));
					}
				}

				for (int i = 0; i < pp.getAttributeCount(); i++) {
					QName qa = (pp.getAttributePrefix(i) == null) ? df
							.createQName(pp.getAttributeName(i)) : df
							.createQName(pp.getAttributeName(i), pp
									.getAttributePrefix(i), pp
									.getAttributeNamespace(i));
					newElement.addAttribute(qa, pp.getAttributeValue(i));
				}

				if (parent != null) {
					parent.add(newElement);
				} else {
					document.add(newElement);
				}

				parent = newElement;

				break;
			}

			case XmlPullParser.END_TAG: {
				if (parent != null) {
					parent = parent.getParent();
				}

				if (headerDeep != 0) {
					headerDeep--;
				}
				break;
			}

			case XmlPullParser.TEXT: {
				String text = pp.getText();

				if (parent != null) {
					parent.addText(text);
				} else {
					String msg = "Cannot have text content outside of the "
							+ "root document";
					throw new DocumentException(msg);
				}

				break;
			}

			default:
				break;
			}
		}
	}

	public void setSOAPFactory(SOAPFactory soapFactory) {
		if (soapFactory != null) {
			this.soapFactory = soapFactory;
		} else {
			this.soapFactory = SOAPFactory.getInstance();
			this.setDocumentFactory(soapFactory);
		}

	}

}
