package org.act.xservices.rt.soap.factory;

import org.act.xservices.rt.soap.ISOAPBody;
import org.act.xservices.rt.soap.ISOAPFault;
import org.act.xservices.rt.soap.ISOAPFaultCode;
import org.act.xservices.rt.soap.ISOAPFaultDetail;
import org.act.xservices.rt.soap.ISOAPFaultNode;
import org.act.xservices.rt.soap.ISOAPFaultReason;
import org.act.xservices.rt.soap.ISOAPFaultRole;
import org.act.xservices.rt.soap.ISOAPFaultSubCode;
import org.act.xservices.rt.soap.ISOAPFaultText;
import org.act.xservices.rt.soap.ISOAPFaultValue;
import org.act.xservices.rt.soap.ISOAPHeader;
import org.act.xservices.rt.soap.ISOAPHeaderBlock;
import org.act.xservices.rt.soap.ISOAPVersion;
import org.act.xservices.rt.soap.constants.SOAP11Constants;
import org.act.xservices.rt.soap.constants.SOAP11Version;
import org.act.xservices.rt.soap.exception.SOAPProcException;
import org.act.xservices.rt.soap11.impl.SOAP11BodyImpl;
import org.act.xservices.rt.soap11.impl.SOAP11FaultActorImpl;
import org.act.xservices.rt.soap11.impl.SOAP11FaultCodeImpl;
import org.act.xservices.rt.soap11.impl.SOAP11FaultDetailImpl;
import org.act.xservices.rt.soap11.impl.SOAP11FaultImpl;
import org.act.xservices.rt.soap11.impl.SOAP11FaultStringImpl;
import org.act.xservices.rt.soap11.impl.SOAP11HeaderBlockImpl;
import org.act.xservices.rt.soap11.impl.SOAP11HeaderImpl;
import org.act.xservices.rt.xom.Namespace;
import org.act.xservices.rt.xom.QName;

/**
 * 
 * @author Viking.Liu
 * @date 2010-6-5
 * @since JDK 1.5
 * @version 1.0.0
 * 
 */
public class SOAP11Factory extends SOAPFactory {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	@Override
	public ISOAPBody createSOAPBody() throws SOAPProcException {
		return new SOAP11BodyImpl();
	}

	@Override
	public ISOAPBody createSOAPBody(QName qname) throws SOAPProcException {
		return new SOAP11BodyImpl(qname);
	}

	@Override
	public ISOAPFault createSOAPFault() throws SOAPProcException {
		return new SOAP11FaultImpl();
	}

	@Override
	public ISOAPFault createSOAPFault(QName qname) throws SOAPProcException {
		return new SOAP11FaultImpl(qname);
	}

	@Override
	public ISOAPFaultCode createSOAPFaultCode() throws SOAPProcException {
		return new SOAP11FaultCodeImpl();
	}

	@Override
	public ISOAPFaultCode createSOAPFaultCode(QName qname)
			throws SOAPProcException {
		return new SOAP11FaultCodeImpl(qname);
	}

	@Override
	public ISOAPFaultDetail createSOAPFaultDetail() throws SOAPProcException {
		return new SOAP11FaultDetailImpl();
	}

	@Override
	public ISOAPFaultDetail createSOAPFaultDetail(QName qname)
			throws SOAPProcException {
		return new SOAP11FaultDetailImpl(qname);
	}

	@Override
	public ISOAPFaultNode createSOAPFaultNode() throws SOAPProcException {
		throw new SOAPProcException("SOAP1.1 does not support this method.");
	}

	@Override
	public ISOAPFaultNode createSOAPFaultNode(QName qname)
			throws SOAPProcException {
		throw new SOAPProcException("SOAP1.1 does not support this method.");
	}

	@Override
	public ISOAPFaultReason createSOAPFaultReason() throws SOAPProcException {
		return new SOAP11FaultStringImpl();
	}

	@Override
	public ISOAPFaultReason createSOAPFaultReason(QName qname)
			throws SOAPProcException {
		return new SOAP11FaultStringImpl(qname);
	}

	@Override
	public ISOAPFaultRole createSOAPFaultRole() throws SOAPProcException {
		return new SOAP11FaultActorImpl();
	}

	@Override
	public ISOAPFaultRole createSOAPFaultRole(QName qname)
			throws SOAPProcException {
		return new SOAP11FaultActorImpl(qname);
	}

	@Override
	public ISOAPFaultSubCode createSOAPFaultSubCode() throws SOAPProcException {
		throw new SOAPProcException("SOAP1.1 does not support this method.");
	}

	@Override
	public ISOAPFaultSubCode createSOAPFaultSubCode(QName qname)
			throws SOAPProcException {
		throw new SOAPProcException("SOAP1.1 does not support this method.");
	}

	@Override
	public ISOAPFaultText createSOAPFaultText() throws SOAPProcException {
		throw new SOAPProcException("SOAP1.1 does not support this method.");
	}

	@Override
	public ISOAPFaultText createSOAPFaultText(QName qname)
			throws SOAPProcException {
		throw new SOAPProcException("SOAP1.1 does not support this method.");
	}

	@Override
	public ISOAPFaultValue createSOAPFaultValue() throws SOAPProcException {
		throw new SOAPProcException("SOAP1.1 does not support this method.");
	}

	@Override
	public ISOAPFaultValue createSOAPFaultValue(QName qname)
			throws SOAPProcException {
		throw new SOAPProcException("SOAP1.1 does not support this method.");
	}

	@Override
	public ISOAPHeaderBlock createSOAPHeaderBlock(String localName, Namespace ns)
			throws SOAPProcException {		
		return new SOAP11HeaderBlockImpl(localName,ns);
	}

	@Override
	public ISOAPHeaderBlock createSOAPHeaderBlock(QName qname)
			throws SOAPProcException {		 
		return new SOAP11HeaderBlockImpl(qname);
	}

	@Override
	public ISOAPFault createSOAPFault(Exception e) throws SOAPProcException {
		return new SOAP11FaultImpl(e);
	}

	@Override
	public ISOAPVersion getSOAPVersion() {
		return SOAP11Version.getSingleton();
	}

	@Override
	public String getSoapVersionURI() {
		return SOAP11Constants.SOAP_ENVELOPE_NAMESPACE_URI;
	}

	@Override
	public Namespace getNamespace() {
		return createNamespace(
				SOAP11Constants.SOAP_DEFAULT_NAMESPACE_PREFIX,
				SOAP11Constants.SOAP_ENVELOPE_NAMESPACE_URI);
	}

	@Override
	public ISOAPHeader createSOAPHeader() throws SOAPProcException {
		return new SOAP11HeaderImpl();
	}

	@Override
	public ISOAPHeader createSOAPHeader(QName qname) throws SOAPProcException {
		return new SOAP11HeaderImpl(qname);
	}
}
