package org.act.xservices.rt.soap.factory;

import org.act.xservices.rt.soap.ISOAPBody;
import org.act.xservices.rt.soap.ISOAPFault;
import org.act.xservices.rt.soap.ISOAPFaultCode;
import org.act.xservices.rt.soap.ISOAPFaultDetail;
import org.act.xservices.rt.soap.ISOAPFaultNode;
import org.act.xservices.rt.soap.ISOAPFaultReason;
import org.act.xservices.rt.soap.ISOAPFaultRole;
import org.act.xservices.rt.soap.ISOAPFaultSubCode;
import org.act.xservices.rt.soap.ISOAPFaultText;
import org.act.xservices.rt.soap.ISOAPFaultValue;
import org.act.xservices.rt.soap.ISOAPHeader;
import org.act.xservices.rt.soap.ISOAPHeaderBlock;
import org.act.xservices.rt.soap.ISOAPVersion;
import org.act.xservices.rt.soap.constants.SOAP12Constants;
import org.act.xservices.rt.soap.constants.SOAP12Version;
import org.act.xservices.rt.soap.exception.SOAPProcException;
import org.act.xservices.rt.soap12.impl.SOAP12BodyImpl;
import org.act.xservices.rt.soap12.impl.SOAP12FaultCodeImpl;
import org.act.xservices.rt.soap12.impl.SOAP12FaultDetailImpl;
import org.act.xservices.rt.soap12.impl.SOAP12FaultImpl;
import org.act.xservices.rt.soap12.impl.SOAP12FaultNodeImpl;
import org.act.xservices.rt.soap12.impl.SOAP12FaultReasonImpl;
import org.act.xservices.rt.soap12.impl.SOAP12FaultRoleImpl;
import org.act.xservices.rt.soap12.impl.SOAP12FaultSubCodeImpl;
import org.act.xservices.rt.soap12.impl.SOAP12FaultTextImpl;
import org.act.xservices.rt.soap12.impl.SOAP12FaultValueImpl;
import org.act.xservices.rt.soap12.impl.SOAP12HeaderBlockImpl;
import org.act.xservices.rt.soap12.impl.SOAP12HeaderImpl;
import org.act.xservices.rt.xom.Namespace;
import org.act.xservices.rt.xom.QName;

/**
 * 
 * @author Viking.Liu
 * @date 2010-6-5
 * @since JDK 1.5
 * @version 1.0.0
 * 
 */
public class SOAP12Factory extends SOAPFactory {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	@Override
	public ISOAPBody createSOAPBody() throws SOAPProcException {
		return new SOAP12BodyImpl();
	}

	@Override
	public ISOAPBody createSOAPBody(QName qname) throws SOAPProcException {
		return new SOAP12BodyImpl(qname);
	}

	@Override
	public ISOAPFault createSOAPFault() throws SOAPProcException {
		return new SOAP12FaultImpl();
	}

	@Override
	public ISOAPFault createSOAPFault(Exception e) throws SOAPProcException {
		return new SOAP12FaultImpl(e);
	}

	@Override
	public ISOAPFault createSOAPFault(QName qname) throws SOAPProcException {
		return new SOAP12FaultImpl(qname);
	}

	@Override
	public ISOAPFaultCode createSOAPFaultCode() throws SOAPProcException {
		return new SOAP12FaultCodeImpl();
	}

	@Override
	public ISOAPFaultCode createSOAPFaultCode(QName qname) {
		return new SOAP12FaultCodeImpl(qname);
	}

	@Override
	public ISOAPFaultDetail createSOAPFaultDetail() throws SOAPProcException {
		return new SOAP12FaultDetailImpl();
	}

	@Override
	public ISOAPFaultDetail createSOAPFaultDetail(QName qname)
			throws SOAPProcException {
		return new SOAP12FaultDetailImpl(qname);
	}

	@Override
	public ISOAPFaultNode createSOAPFaultNode() throws SOAPProcException {
		return new SOAP12FaultNodeImpl();
	}

	@Override
	public ISOAPFaultNode createSOAPFaultNode(QName qname)
			throws SOAPProcException {
		return new SOAP12FaultNodeImpl(qname);
	}

	@Override
	public ISOAPFaultReason createSOAPFaultReason() throws SOAPProcException {
		return new SOAP12FaultReasonImpl();
	}

	@Override
	public ISOAPFaultReason createSOAPFaultReason(QName qname)
			throws SOAPProcException {
		return new SOAP12FaultReasonImpl(qname);
	}

	@Override
	public ISOAPFaultRole createSOAPFaultRole() throws SOAPProcException {
		return new SOAP12FaultRoleImpl();
	}

	@Override
	public ISOAPFaultRole createSOAPFaultRole(QName qname)
			throws SOAPProcException {
		return new SOAP12FaultRoleImpl(qname);
	}

	@Override
	public ISOAPFaultSubCode createSOAPFaultSubCode() throws SOAPProcException {
		return new SOAP12FaultSubCodeImpl();
	}

	@Override
	public ISOAPFaultSubCode createSOAPFaultSubCode(QName qname)
			throws SOAPProcException {
		return new SOAP12FaultSubCodeImpl(qname);
	}

	@Override
	public ISOAPFaultText createSOAPFaultText() throws SOAPProcException {
		return new SOAP12FaultTextImpl();
	}

	@Override
	public ISOAPFaultText createSOAPFaultText(QName qname)
			throws SOAPProcException {
		return new SOAP12FaultTextImpl(qname);
	}

	@Override
	public ISOAPFaultValue createSOAPFaultValue() throws SOAPProcException {
		return new SOAP12FaultValueImpl();
	}

	@Override
	public ISOAPFaultValue createSOAPFaultValue(QName qname)
			throws SOAPProcException {
		return new SOAP12FaultValueImpl();
	}

	@Override
	public ISOAPHeaderBlock createSOAPHeaderBlock(String localName, Namespace ns)
			throws SOAPProcException {
		return new SOAP12HeaderBlockImpl(localName, ns);
	}

	@Override
	public ISOAPHeaderBlock createSOAPHeaderBlock(QName qname)
			throws SOAPProcException {
		return new SOAP12HeaderBlockImpl(qname);
	}

	@Override
	public ISOAPVersion getSOAPVersion() {
		return SOAP12Version.getSingleton();
	}

	@Override
	public String getSoapVersionURI() {
		return SOAP12Constants.SOAP_ENVELOPE_NAMESPACE_URI;
	}

	@Override
	public Namespace getNamespace() {
		return createNamespace(
				SOAP12Constants.SOAP_DEFAULT_NAMESPACE_PREFIX,
				SOAP12Constants.SOAP_ENVELOPE_NAMESPACE_URI);
	}

	@Override
	public ISOAPHeader createSOAPHeader() throws SOAPProcException {		
		return new SOAP12HeaderImpl();
	}

	@Override
	public ISOAPHeader createSOAPHeader(QName qname) throws SOAPProcException {
		return new SOAP12HeaderImpl(qname);
	}

}
