package org.act.xservices.rt.soap.factory;

import org.act.xservices.rt.metaset.MetasetBuilder;
import org.act.xservices.rt.soap.ISOAPBody;
import org.act.xservices.rt.soap.ISOAPEnvelope;
import org.act.xservices.rt.soap.ISOAPFault;
import org.act.xservices.rt.soap.ISOAPFaultCode;
import org.act.xservices.rt.soap.ISOAPFaultDetail;
import org.act.xservices.rt.soap.ISOAPFaultNode;
import org.act.xservices.rt.soap.ISOAPFaultReason;
import org.act.xservices.rt.soap.ISOAPFaultRole;
import org.act.xservices.rt.soap.ISOAPFaultSubCode;
import org.act.xservices.rt.soap.ISOAPFaultText;
import org.act.xservices.rt.soap.ISOAPFaultValue;
import org.act.xservices.rt.soap.ISOAPHeader;
import org.act.xservices.rt.soap.ISOAPHeaderBlock;
import org.act.xservices.rt.soap.ISOAPMessage;
import org.act.xservices.rt.soap.ISOAPVersion;
import org.act.xservices.rt.soap.constants.SOAP11Constants;
import org.act.xservices.rt.soap.constants.SOAP11Version;
import org.act.xservices.rt.soap.constants.SOAP12Constants;
import org.act.xservices.rt.soap.constants.SOAP12Version;
import org.act.xservices.rt.soap.constants.SOAPConstants;
import org.act.xservices.rt.soap.exception.SOAPException;
import org.act.xservices.rt.soap.exception.SOAPProcException;
import org.act.xservices.rt.soap.impl.SOAPEnvelopeImpl;
import org.act.xservices.rt.soap.impl.SOAPMessageImpl;
import org.act.xservices.rt.xom.Namespace;
import org.act.xservices.rt.xom.QName;

/**
 * 
 * @author Viking.Liu
 * @date 2010-6-5
 * @since JDK 1.5
 * @version 1.0.0
 * 
 */
/**
 * The Default SOAPFactory is SOAP 1.2
 */
public abstract class SOAPFactory extends MetasetBuilder {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private static SOAPFactory instance;
	private static SOAPFactory soap11Factory;
	private static SOAPFactory soap12Factory;

	/**
	 * Default SOAPFactory is SOAP11Factory
	 * 
	 * @return
	 */
	public static SOAPFactory getInstance() {
		if (instance == null) {
			instance = new SOAP11Factory();
		}
		return instance;
	}

	public static SOAPFactory getSOAP11Factory() {
		if (soap11Factory == null) {
			soap11Factory = new SOAP11Factory();
		}
		return soap11Factory;
	}

	public static SOAPFactory getSOAP12Factory() {
		if (soap12Factory == null) {
			soap12Factory = new SOAP12Factory();
		}
		return soap12Factory;
	}

	public static SOAPFactory getSOAPFactory(String nsURI){
		if (SOAP12Constants.SOAP_ENVELOPE_NAMESPACE_URI.equals(nsURI)) {
			return getSOAP11Factory();
		} else if (SOAP11Constants.SOAP_ENVELOPE_NAMESPACE_URI.equals(nsURI)) {
			return getSOAP11Factory();
		} else {
			throw new SOAPException("invalidSOAPversion");
		}
	}
	
	public static SOAPFactory getSOAPFactory(ISOAPVersion version){
		 
		if(version==null||version instanceof SOAP11Version){
			return getSOAP11Factory();
		}else if(version instanceof SOAP12Version){
			return getSOAP11Factory();
		}
		return null;
	}
	/** =============== SOAP Message===================== */
	public ISOAPMessage createSOAPMessage(String encodingScheme) {
		return new SOAPMessageImpl(encodingScheme);
	}

	public ISOAPMessage createSOAPMessage() {
		return new SOAPMessageImpl();
	}

	/** =============== SOAP Envelope===================== */
	public ISOAPEnvelope createSOAPEnvelope() throws SOAPProcException {
		return new SOAPEnvelopeImpl();
	}

	public ISOAPEnvelope createSOAPEnvelope(Namespace ns) {
		return new SOAPEnvelopeImpl(SOAPConstants.SOAP_ENVELOPE_NAME, ns);
	}

	public ISOAPEnvelope createSOAPEnvelope(QName qname)
			throws SOAPProcException {
		return new SOAPEnvelopeImpl(qname);
	}

	/** =============== SOAP Default Envelope===================== */
	public ISOAPEnvelope getDefaultEnvelope() throws SOAPProcException {
		ISOAPEnvelope envelope = new SOAPEnvelopeImpl(
				SOAPConstants.SOAP_ENVELOPE_NAME, getNamespace());
		ISOAPHeader header = this.createSOAPHeader(new QName(
				SOAPConstants.SOAP_HEADER_NAME, getNamespace()));
		ISOAPBody body = this.createSOAPBody(new QName(
				SOAPConstants.SOAP_BODY_NAME, getNamespace()));
		envelope.addHeader(header);
		envelope.addBody(body);
		return envelope;
	}

	public ISOAPEnvelope getDefaultFaultEnvelope() throws SOAPProcException {
		ISOAPEnvelope envelope = this.getDefaultEnvelope();
		ISOAPBody body = envelope.getBody();
		ISOAPFault fault = this.createSOAPFault();
		ISOAPFaultCode faultCode = this.createSOAPFaultCode();
		ISOAPFaultReason faultReason = this.createSOAPFaultReason();
		fault.addFaultCode(faultCode);
		fault.addFaultReason(faultReason);
		body.addFault(fault);
		return envelope;
	}

	/** =============== Abstract Method ===================== */
	public abstract ISOAPBody createSOAPBody() throws SOAPProcException;

	public abstract ISOAPBody createSOAPBody(QName qname)
			throws SOAPProcException;

	public abstract ISOAPFault createSOAPFault() throws SOAPProcException;

	public abstract ISOAPFault createSOAPFault(QName qname)
			throws SOAPProcException;

	public abstract ISOAPFault createSOAPFault(Exception e)
			throws SOAPProcException;

	public abstract ISOAPFaultCode createSOAPFaultCode()
			throws SOAPProcException;

	public abstract ISOAPFaultCode createSOAPFaultCode(QName qname)
			throws SOAPProcException;

	public abstract ISOAPFaultDetail createSOAPFaultDetail()
			throws SOAPProcException;

	public abstract ISOAPFaultDetail createSOAPFaultDetail(QName qname)
			throws SOAPProcException;

	public abstract ISOAPFaultNode createSOAPFaultNode()
			throws SOAPProcException;

	public abstract ISOAPFaultNode createSOAPFaultNode(QName qname)
			throws SOAPProcException;

	public abstract ISOAPFaultReason createSOAPFaultReason()
			throws SOAPProcException;

	public abstract ISOAPFaultReason createSOAPFaultReason(QName qname)
			throws SOAPProcException;

	public abstract ISOAPFaultRole createSOAPFaultRole()
			throws SOAPProcException;

	public abstract ISOAPFaultRole createSOAPFaultRole(QName qname)
			throws SOAPProcException;

	public abstract ISOAPFaultSubCode createSOAPFaultSubCode()
			throws SOAPProcException;

	public abstract ISOAPFaultSubCode createSOAPFaultSubCode(QName qname)
			throws SOAPProcException;

	public abstract ISOAPFaultText createSOAPFaultText()
			throws SOAPProcException;

	public abstract ISOAPFaultText createSOAPFaultText(QName qname)
			throws SOAPProcException;

	public abstract ISOAPFaultValue createSOAPFaultValue()
			throws SOAPProcException;

	public abstract ISOAPFaultValue createSOAPFaultValue(QName qname)
			throws SOAPProcException;

	public abstract ISOAPHeaderBlock createSOAPHeaderBlock(String localName,
			Namespace ns) throws SOAPProcException;

	public abstract ISOAPHeaderBlock createSOAPHeaderBlock(QName qname)
			throws SOAPProcException;

	public abstract ISOAPVersion getSOAPVersion();

	public abstract String getSoapVersionURI();

	public abstract Namespace getNamespace();

	/** =============== SOAP Header===================== */

	public abstract ISOAPHeader createSOAPHeader() throws SOAPProcException;

	public abstract ISOAPHeader createSOAPHeader(QName qname)
			throws SOAPProcException;
}
