package org.act.xservices.rt.soap.impl;

import org.act.xservices.rt.soap.ISOAPBody;
import org.act.xservices.rt.soap.ISOAPFault;
import org.act.xservices.rt.soap.constants.SOAPConstants;
import org.act.xservices.rt.soap.exception.SOAPException;
import org.act.xservices.rt.xom.Metaset;
import org.act.xservices.rt.xom.Namespace;
import org.act.xservices.rt.xom.QName;
import org.act.xservices.rt.xom.tree.DefaultElement;

/**
 * 
 * @author Viking.Liu
 * @date 2010-6-3
 * @since JDK 1.5
 * @version 1.0.0
 * 
 */
public abstract class SOAPBodyImpl extends DefaultElement implements ISOAPBody {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private boolean hasFault = false;

	public SOAPBodyImpl() {
		super(SOAPConstants.SOAP_BODY_NAME);
	}

	public SOAPBodyImpl(QName qname) {
		super(qname);
		if (qname == null) {
			throw new SOAPException("The qname is null.");
		}
		if (!qname.getName().equals(SOAPConstants.SOAP_BODY_NAME)) {
			throw new SOAPException(
					"The SOAPBody name should be Body.But it is "
							+ qname.getName());
		}
	}

	public SOAPBodyImpl(String name, Namespace namespace) {
		super(name, namespace);
		if (name == null || !name.equals(SOAPConstants.SOAP_BODY_NAME)) {
			throw new SOAPException(
					"The SOAPBody name should be Body.But it is " + name);
		}
	}

	/**
	 * @param soapFault
	 * @throws org.act.xservices.rt.soap.exception.SOAPException
	 */
	public void addFault(ISOAPFault soapFault) throws SOAPException {
		if (this.nodeCount() > 0) {
			throw new SOAPException(
					"SOAP Fault should be the first child of SOAP Body, but there already exist other children.");
		}
		getFault();
		if (hasFault) {
			throw new SOAPException(
					"SOAP Body already has a SOAP Fault and there can not be "
							+ "more than one SOAP fault");
		}
		hasFault = true;
		this.add(soapFault);
	}

	/**
	 * Returns the <code>SOAPFault</code> object in this <code>SOAPBody</code>
	 * object.
	 * 
	 * @return the <code>SOAPFault</code> object in this <code>SOAPBody</code>
	 *         object
	 */
	public ISOAPFault getFault() {
		
		Metaset fault = this.element(SOAPConstants.SOAP_BODY_FAULT_NAME);
		if (fault != null) {
			hasFault = true;
			return (ISOAPFault) fault;
		}
		return null;
	}

	public String getFirstElementLocalName() {
		Metaset first = getFirstElement();
		if (first != null) {
			return first.getName();
		}
		return null;
	}

	public Namespace getFirstElementNS() {
		Metaset first = getFirstElement();
		if (first != null) {
			return first.getNamespace();
		}
		return null;
	}

	/**
	 * Indicates whether a <code>SOAPFault</code> object exists in this
	 * <code>SOAPBody</code> object.
	 * 
	 * @return <code>true</code> if a <code>SOAPFault</code> object exists in
	 *         this <code>SOAPBody</code> object; <code>false</code> otherwise
	 */
	public boolean hasFault() {
		if (!hasFault) {
			getFault();
		}
		return hasFault;
	}
}
