package org.act.xservices.rt.soap.impl;

import java.util.List;

import org.act.xservices.rt.soap.ISOAPBody;
import org.act.xservices.rt.soap.ISOAPEnvelope;
import org.act.xservices.rt.soap.ISOAPHeader;
import org.act.xservices.rt.soap.ISOAPVersion;
import org.act.xservices.rt.soap.constants.SOAPConstants;
import org.act.xservices.rt.soap.exception.SOAPException;
import org.act.xservices.rt.xom.Metaset;
import org.act.xservices.rt.xom.Namespace;
import org.act.xservices.rt.xom.Node;
import org.act.xservices.rt.xom.QName;
import org.act.xservices.rt.xom.tree.DefaultElement;

/**
 * 
 * @author Viking.Liu
 * @date 2010-6-2
 * @since JDK 1.5
 * @version 1.0.0
 */
public class SOAPEnvelopeImpl extends DefaultElement implements ISOAPEnvelope {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private ISOAPVersion version;

	private boolean hasHeader = false;
	private boolean hasBody = false;

	public SOAPEnvelopeImpl() {
		super(SOAPConstants.SOAP_ENVELOPE_NAME);
	}

	public SOAPEnvelopeImpl(QName qname) {
		super(qname);
		if (qname == null) {
			throw new SOAPException("The qname is null.");
		}
		if (!qname.getName().equals(SOAPConstants.SOAP_ENVELOPE_NAME)) {
			throw new SOAPException(
					"The SOAPEnvelope name should be Envelope.But it is "
							+ qname.getName());
		}
	}

	public SOAPEnvelopeImpl(String name, Namespace namespace) {
		super(name, namespace);
		if (name == null || !name.equals(SOAPConstants.SOAP_ENVELOPE_NAME)) {
			throw new SOAPException(
					"The SOAPEnvelope name should be Envelope.But it is "
							+ name);
		}
	}

	/**
	 * Returns the <CODE>SOAPBody</CODE> object associated with this
	 * <CODE>SOAPEnvelope</CODE> object.
	 * <p/>
	 * This SOAPBody will just be a container for all the BodyElements in the
	 * <CODE>SOAPMessage</CODE>
	 * </P>
	 * 
	 * @return the <CODE>SOAPBody</CODE> object for this
	 *         <CODE> SOAPEnvelope</CODE> object or <CODE>null</CODE> if there
	 *         is none
	 * @throws org.act.xservices.rt.soap.exception.SOAPException
	 *             if there is a problem obtaining the <CODE>SOAPBody</CODE>
	 *             object
	 * @throws SOAPException
	 */

	public ISOAPBody getBody() throws SOAPException {
		Metaset soapBody = element(SOAPConstants.SOAP_BODY_NAME);
		if (soapBody == null) {
			throw new SOAPException(
					"The SOAP body is null.The envelope must has the body element.");
		}
		hasBody = true;
		return (ISOAPBody) soapBody;
	}

	/**
	 * Returns the <CODE>SOAPHeader</CODE> object for this
	 * <CODE> SOAPEnvelope</CODE> object.
	 * <p/>
	 * This SOAPHeader will just be a container for all the headers in the
	 * <CODE>SOAPMessage</CODE>
	 * </P>
	 * 
	 * @return the <CODE>SOAPHeader</CODE> object or <CODE> null</CODE> if there
	 *         is none
	 * @throws org.act.xservices.rt.soap.exception.SOAPException
	 *             if there is a problem obtaining the <CODE>SOAPHeader</CODE>
	 *             object
	 * @throws SOAPException
	 */
	public ISOAPHeader getHeader() throws SOAPException {
		Metaset header = element(SOAPConstants.SOAP_HEADER_NAME);
		if (header == null) {
			return null;
		}
		hasHeader = true;
		return (ISOAPHeader) header;
	}

	public String getSOAPBodyFirstElementLocalName() throws SOAPException {
		return getBody().getFirstElementLocalName();
	}

	public Namespace getSOAPBodyFirstElementNS() throws SOAPException {
		return getBody().getFirstElementNS();
	}

	public ISOAPVersion getVersion() {
		return version;
	}

	public boolean hasFault() {
		return getBody().hasFault();
	}

	public void addBody(ISOAPBody body) throws SOAPException {
		Metaset soapBody = element(SOAPConstants.SOAP_BODY_NAME);
		if (soapBody != null) {
			hasBody = true;
		}
		if (hasBody) {
			throw new SOAPException(
					"SOAP Envelope already has a SOAP Body, and there can not be more than one SOAP Body.");
		}
		List<Metaset> children = elements();
		if (children.size() > 1) {
			throw new SOAPException(
					"SOAP Envelope must has a SOAP Body after SOAP Header if there is a SOAP Header,so there can not be more than tow child.");
		} else if (children.size() == 1) {
			Node header = children.get(0);
			if (!(header instanceof ISOAPHeader)) {
				throw new SOAPException(
						"SOAP Envelope first child must be SOAP Header if there is a SOAP Header.");
			}
		}
		hasBody = true;
		add(body);
	}

	public void addHeader(ISOAPHeader header) throws SOAPException {
		if (hasHeader || getFirstElement() != null) {
			throw new SOAPException(
					"SOAP Envelope already has some children. The SOAP Header must be the first child of the SOAP Envelope.");
		}
		hasHeader = true;
		add(header);
	}

	public void setVersion(ISOAPVersion version) {
		this.version = version;
	}
}
