package org.act.xservices.rt.soap.util;

import org.act.xservices.rt.soap.ISOAPEnvelope;
import org.act.xservices.rt.soap.constants.SOAP11Constants;
import org.act.xservices.rt.soap.constants.SOAP12Constants;
import org.act.xservices.rt.soap.constants.SOAPConstants;
import org.act.xservices.rt.soap.exception.SOAPProcException;
import org.act.xservices.rt.soap.factory.SOAPFactory;
import org.act.xservices.rt.xom.DocumentFactory;
import org.act.xservices.rt.xom.Metaset;
import org.act.xservices.rt.xom.QName;

public class SOAPBuilderhelper {

	/**
	 * Create SOAP Element SOAPEnvelope SOAPBody SOAPHeader SOAPBody SOAPFault
	 * SOAP11 faultcode faultstring faultactor detail SOAP12 Code Reason Node
	 * Role Detail Value Text SubCode
	 * 
	 * @param elemName
	 * @return
	 * @throws SOAPProcException
	 */
	public static Metaset createElement(QName qname, SOAPFactory soapFactory)
			throws SOAPProcException {
		Metaset elem = null;
		if (soapFactory == null) {
			throw new SOAPProcException("The SOAP Factory is null!");
		}
		if (qname.getName().equals(SOAPConstants.SOAP_ENVELOPE_NAME)) {
			// Envelope
			ISOAPEnvelope envelope = soapFactory.createSOAPEnvelope(qname);
			envelope.setVersion(soapFactory.getSOAPVersion());
			elem = envelope;
		} else if (qname.getName().equals(SOAPConstants.SOAP_HEADER_NAME)) {
			// Header
			elem = soapFactory.createSOAPHeader(qname);
		} else if (qname.getName().equals(SOAPConstants.SOAP_BODY_NAME)) {
			// Body
			elem = soapFactory.createSOAPBody(qname);
		} else if (qname.getName().equals(SOAPConstants.SOAP_BODY_FAULT_NAME)) {
			// Fault
			elem = soapFactory.createSOAPFault(qname);
		} else if (qname.getName().equals(SOAP11Constants.SOAP_FAULT_CODE_NAME)) {
			// SOAP11 FaultCode
			elem = soapFactory.createSOAPFaultCode(qname);
		} else if (qname.getName().equals(
				SOAP11Constants.SOAP_FAULT_STRING_NAME)) {
			// SOAP11 FaultString
			elem = soapFactory.createSOAPFaultReason(qname);
		} else if (qname.getName()
				.equals(SOAP11Constants.SOAP_FAULT_ACTOR_NAME)) {
			// SOAP11 SOAPActor
			elem = soapFactory.createSOAPFaultRole(qname);
		} else if (qname.getName().equals(
				SOAP11Constants.SOAP_FAULT_DETAIL_NAME)) {
			// SOAP11 SOAPDetail
			elem = soapFactory.createSOAPFaultDetail(qname);
		} else if (qname.getName().equals(SOAP12Constants.SOAP_FAULT_CODE_NAME)) {
			// SOAP12 Code
			elem = soapFactory.createSOAPFaultCode(qname);
		} else if (qname.getName().equals(
				SOAP12Constants.SOAP_FAULT_REASON_NAME)) {
			// SOAP12 Reason
			elem = soapFactory.createSOAPFaultReason(qname);
		} else if (qname.getName().equals(SOAP12Constants.SOAP_FAULT_ROLE_NAME)) {
			// SOAP12 Role
			elem = soapFactory.createSOAPFaultRole(qname);
		} else if (qname.getName().equals(
				SOAP12Constants.SOAP_FAULT_DETAIL_NAME)) {
			// SOAP12 Detail
			elem = soapFactory.createSOAPFaultDetail(qname);
		} else if (qname.getName().equals(
				SOAP12Constants.SOAP_FAULT_SUB_CODE_NAME)) {
			// SOAP12 SubCode
			elem = soapFactory.createSOAPFaultSubCode(qname);
		} else if (qname.getName()
				.equals(SOAP12Constants.SOAP_FAULT_VALUE_NAME)) {
			// SOAP12 Value
			elem = soapFactory.createSOAPFaultValue(qname);
		} else if (qname.getName().equals(SOAP12Constants.SOAP_FAULT_TEXT_NAME)) {
			// SOAP12 Text
			elem = soapFactory.createSOAPFaultText(qname);
		} else if (qname.getName().equals(SOAP12Constants.SOAP_FAULT_NODE_NAME)) {
			elem = soapFactory.createSOAPFaultNode(qname);
		} else {
			elem = DocumentFactory.getInstance().createElement(qname);
		}
		return elem;
	}
}
