package org.act.xservices.rt.soap.util;

import java.io.StringReader;
import java.util.List;

import org.act.xservices.rt.soap.ISOAPBody;
import org.act.xservices.rt.soap.ISOAPEnvelope;
import org.act.xservices.rt.soap.ISOAPFault;
import org.act.xservices.rt.soap.ISOAPFaultDetail;
import org.act.xservices.rt.soap.ISOAPFaultRole;
import org.act.xservices.rt.soap.ISOAPFaultText;
import org.act.xservices.rt.soap.ISOAPFaultValue;
import org.act.xservices.rt.soap.ISOAPHeader;
import org.act.xservices.rt.soap.ISOAPHeaderBlock;
import org.act.xservices.rt.soap.ISOAPMessage;
import org.act.xservices.rt.soap.ISOAPVersion;
import org.act.xservices.rt.soap.builder.ISOAPBuilder;
import org.act.xservices.rt.soap.builder.SOAPBuilderFactory;
import org.act.xservices.rt.soap.constants.SOAP11Version;
import org.act.xservices.rt.soap.constants.SOAP12Version;
import org.act.xservices.rt.soap.factory.SOAPFactory;
import org.act.xservices.rt.xom.Metaset;

public class SOAPMessageUtil {

	public static ISOAPEnvelope creatStdSimpleFaultMessage(
			ISOAPVersion version, String faultCodeReceiver, String string) {

		SOAPFactory factory = getSOAPFactory(version);
		ISOAPEnvelope envelope = factory.getDefaultFaultEnvelope();
		ISOAPFault fault = envelope.getBody().getFault();
		processFault(fault, version, faultCodeReceiver, string);

		return envelope;
	}

	public static ISOAPEnvelope creatResponseMessage(ISOAPVersion version,
			Metaset responseMetaset, List<ISOAPHeaderBlock> inMessageHeaders) {
		SOAPFactory factory = getSOAPFactory(version);
		ISOAPEnvelope envelope = factory.getDefaultEnvelope();
		ISOAPHeader header = envelope.getHeader();
		if (inMessageHeaders != null) {
			for (ISOAPHeaderBlock headerBlock : inMessageHeaders) {
				header.add(headerBlock.createCopy());
			}
		}
		ISOAPBody body = envelope.getBody();
		body.add(responseMetaset);
		return envelope;
	}

	public static ISOAPEnvelope creatStdFaultMessage(ISOAPVersion version,
			String faultCodeReceiver, String string, String actorstring,
			Metaset detailContent) {
		ISOAPEnvelope envelope = creatStdSimpleFaultMessage(version,faultCodeReceiver, string);

		ISOAPFault fault = envelope.getBody().getFault();
		processFault(fault, version, actorstring, detailContent);
		return envelope;
	}

	public static ISOAPEnvelope convertMetasetToSOAPEnvelope(Metaset metaset) {
		if (metaset != null) {
			String xml = metaset.asXML();
			ISOAPBuilder builder = SOAPBuilderFactory.getSOAPBuilder();

			ISOAPMessage message = builder
					.readSOAPMessage(new StringReader(xml));
			return message.getSOAPEnvelope();
		}
		return null;
	}

	private static SOAPFactory getSOAPFactory(ISOAPVersion version) {
		SOAPFactory factory = null;
		if (version == null || version instanceof SOAP11Version) {
			factory = SOAPFactory.getSOAP11Factory();
		} else if (version instanceof SOAP12Version) {
			factory = SOAPFactory.getSOAP12Factory();
		}
		return factory;
	}

	private static void processFault(ISOAPFault fault, ISOAPVersion version,
			String faultCodeReceiver, String reason) {

		faultCodeReceiver = processStringNull(faultCodeReceiver);
		reason = processStringNull(reason);

		if (version == null || (version instanceof SOAP11Version)) {
			fault.getFaultCode().setText(faultCodeReceiver);
			fault.getFaultReason().setText(reason);
		} else if (version instanceof SOAP12Version) {
			ISOAPFaultValue faultValue = SOAPFactory.getSOAP12Factory().createSOAPFaultValue();
			faultValue.setText(faultCodeReceiver);
			ISOAPFaultText faultText = SOAPFactory.getSOAP12Factory().createSOAPFaultText();
			faultText.setText(reason);

			fault.getFaultCode().addFaultValue(faultValue);
			fault.getFaultReason().addFaultText(faultText);
		}
	}

	private static void processFault(ISOAPFault fault, ISOAPVersion version,
			String actorstring, Metaset detailContent) {
		actorstring = processStringNull(actorstring);
		SOAPFactory factory = null;
		if (version == null || version instanceof SOAP11Version) {
			factory = SOAPFactory.getSOAP11Factory();

		} else if (version instanceof SOAP12Version) {
			factory = SOAPFactory.getSOAP12Factory();
		}

		ISOAPFaultRole actor = factory.createSOAPFaultRole();
		actor.setText(actorstring);
		fault.addFaultRole(actor);
		ISOAPFaultDetail detail = factory.createSOAPFaultDetail();
		detailContent.detach();
		detail.add(detailContent);
		fault.addFaultDetail(detail);
	}

	private static String processStringNull(String value) {
		if (value == null) {
			return "";
		}
		return value;
	}
}
