package org.act.xservices.rt.soap11.impl;

import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.List;

import org.act.xservices.rt.soap.ISOAPFaultCode;
import org.act.xservices.rt.soap.ISOAPFaultDetail;
import org.act.xservices.rt.soap.ISOAPFaultNode;
import org.act.xservices.rt.soap.ISOAPFaultReason;
import org.act.xservices.rt.soap.ISOAPFaultRole;
import org.act.xservices.rt.soap.constants.SOAP11Constants;
import org.act.xservices.rt.soap.constants.SOAPConstants;
import org.act.xservices.rt.soap.exception.SOAPException;
import org.act.xservices.rt.soap.exception.SOAPProcException;
import org.act.xservices.rt.soap.impl.SOAPFaultImpl;
import org.act.xservices.rt.xom.DocumentHelper;
import org.act.xservices.rt.xom.Metaset;
import org.act.xservices.rt.xom.Namespace;
import org.act.xservices.rt.xom.QName;

/**
 * 
 * @author Viking.Liu
 * @date2010-6-9
 * @since JDK 1.5
 * @version 1.0.0
 * 
 */
public class SOAP11FaultImpl extends SOAPFaultImpl {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	private boolean hasCode = false;
	private boolean hasReason = false;
	private boolean hasRole = false;
	private boolean hasDetail = false;

	public SOAP11FaultImpl() {
		super();
	}

	public SOAP11FaultImpl(QName qname) {
		super(qname);
	}

	public SOAP11FaultImpl(String name, Namespace namespace) {
		super(name, namespace);
	}

	public SOAP11FaultImpl(Exception e) {
		super();
		setException(e);
	}

	public ISOAPFaultCode getFaultCode() {
		Metaset code = this.element(SOAP11Constants.SOAP_FAULT_CODE_NAME);
		if (code != null) {
			hasCode = true;
			return (ISOAPFaultCode) code;
		}
		return null;
	}

	public ISOAPFaultDetail getFaultDetail() {
		Metaset detail = this.element(SOAP11Constants.SOAP_FAULT_DETAIL_NAME);
		if (detail != null) {
			hasDetail = true;
			return (ISOAPFaultDetail) detail;
		}
		return null;
	}

	public ISOAPFaultNode getFaultNode() {
		throw new SOAPException("SOAP11 has not the Node! ");
	}

	public ISOAPFaultReason getFaultReason() {
		Metaset reason = element(SOAP11Constants.SOAP_FAULT_STRING_NAME);
		if (reason != null) {
			hasReason = true;
			return (ISOAPFaultReason) reason;
		}
		return null;
	}

	public ISOAPFaultRole getFaultRole() {
		Metaset role = element(SOAP11Constants.SOAP_FAULT_ACTOR_NAME);
		if (role != null) {
			hasRole = true;
			return (ISOAPFaultRole) role;
		}
		return null;
	}

	public void addFaultNode(ISOAPFaultNode node) throws SOAPProcException {
		throw new SOAPException("SOAP11 Fault has not the Node Child! ");
	}

	public void addFaultCode(ISOAPFaultCode soapFaultCode)
			throws SOAPProcException {
		if (hasCode || getFirstElement()!=null) {
			throw new SOAPProcException(
					"The faultcode must be the first child of Fault."
							+ "The SOAP Fault already has children.");
		}
		hasCode = true;
		add(soapFaultCode);
	}

	public void addFaultReason(ISOAPFaultReason soapFaultReason)
			throws SOAPProcException {
		if (hasReason || elements().size() > 1) {
			throw new SOAPProcException(
					"The faultstring must be the second child of the SOAP Fault,"
							+ " but now SOAP Fault already has more than one child.");
		}
		getFaultCode();
		if (!hasCode) {
			throw new SOAPProcException(
					"SOAP Fault first child is not faultcode.");
		}
		hasReason = true;
		add(soapFaultReason);
	}

	public void addFaultRole(ISOAPFaultRole soapFaultrole)
			throws SOAPProcException {
		verify();
		getFaultRole();
		if (hasRole) {
			throw new SOAPProcException(
					"SOAP Fault already has SOAP Fault faultactor, and it can only has one faultactor.");
		}
		hasRole = true;
		add(soapFaultrole);
	}

	public void addFaultDetail(ISOAPFaultDetail soapFaultDetail)
			throws SOAPProcException {
		verify();
		getFaultDetail();
		if (hasDetail) {
			throw new SOAPProcException(
					"SOAP Fault already has SOAP Fault detail, and it can only has one detail.");
		}
		hasDetail = true;
		add(soapFaultDetail);
	}

	private void verify() throws SOAPProcException {
		List<Metaset> children=elements();
		if (children.size() < 2) {
			throw new SOAPProcException(
					"SOAP Fault first child should be faultcode ,and the second child should be faultstring. But it has less than tow children.");
		}
		if (!(children.get(0) instanceof ISOAPFaultCode)) {
			throw new SOAPProcException(
					"SOAP Fault first child should be faultcode ,but it is not.");
		}
		if (!(children.get(1) instanceof ISOAPFaultReason)) {
			throw new SOAPProcException(
					"SOAP Fault second child should be faultstring ,but it is not.");
		}
	}

	public void setException(Exception e) throws SOAPException {
		this.exception = e;
		if (getFaultCode() == null) {
			add(new SOAP11FaultCodeImpl());
		}
		if (getFaultReason() == null) {
			add(new SOAP11FaultStringImpl());
		}
		ISOAPFaultDetail detail = getFaultDetail();
		if (detail == null) {
			detail = new SOAP11FaultDetailImpl();
			add(detail);
		}
		StringWriter sw = new StringWriter();
		e.printStackTrace(new PrintWriter(sw));
		sw.flush();
		Metaset detailEntity = DocumentHelper
				.createElement(SOAPConstants.SOAP_FAULT_DETAIL_EXCEPTION_ENTRY);
		detailEntity.setText(sw.toString());
		detail.add(detailEntity);
	}
}
