package org.act.xservices.rt.soap11.impl;

import org.act.xservices.rt.soap.ISOAPVersion;
import org.act.xservices.rt.soap.constants.SOAP11Constants;
import org.act.xservices.rt.soap.constants.SOAP11Version;
import org.act.xservices.rt.soap.constants.SOAPConstants;
import org.act.xservices.rt.soap.exception.SOAPProcException;
import org.act.xservices.rt.soap.impl.SOAPHeaderBlockImpl;
import org.act.xservices.rt.xom.Attribute;
import org.act.xservices.rt.xom.Namespace;
import org.act.xservices.rt.xom.QName;

/**
 * 
 * @author Viking.Liu
 * @date 2010-6-11
 * @since JDK 1.5
 * @version 1.0.0
 * 
 */
public class SOAP11HeaderBlockImpl extends SOAPHeaderBlockImpl {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	public SOAP11HeaderBlockImpl(QName qname) {
		super(qname);
	}

	public SOAP11HeaderBlockImpl(String name, Namespace namespace) {
		super(name, namespace);
	}

	public SOAP11HeaderBlockImpl(String name) {
		super(name);
	}

	public boolean getMustUnderstand() throws SOAPProcException {
		Attribute attri = attribute(SOAPConstants.ATTR_MUSTUNDERSTAND);
		if (attri != null) {
			String value = attri.getValue();
			if (value.equals(SOAPConstants.ATTR_MUSTUNDERSTAND_0)
					|| value.equals(SOAPConstants.ATTR_MUSTUNDERSTAND_FALSE)) {
				return false;
			} else if (value.equals(SOAPConstants.ATTR_MUSTUNDERSTAND_1)
					|| value.equals(SOAPConstants.ATTR_MUSTUNDERSTAND_TRUE)) {
				return true;
			}
		}
		return false;
	}

	public void setRole(String roleURI) {
		QName qName = new QName(SOAP11Constants.SOAP_ENVELOPE_NAMESPACE_URI,
				SOAP11Constants.ATTR_ACTOR, this.getEnvelopePrefix());
		addAttribute(qName, roleURI);
	}

	public String getRole() {
		Attribute attri = attribute(SOAP11Constants.ATTR_ACTOR);
		if (attri != null) {
			return attri.getValue();
		}
		return null;
	}

	public ISOAPVersion getVersion() {
		return SOAP11Version.getSingleton();
	}

	public void setMustUnderstand(boolean mustUnderstand) {
		QName qName = new QName(SOAP11Constants.SOAP_ENVELOPE_NAMESPACE_URI,
				SOAPConstants.ATTR_MUSTUNDERSTAND, getEnvelopePrefix());

		String value = mustUnderstand ? SOAPConstants.ATTR_MUSTUNDERSTAND_TRUE
				: SOAPConstants.ATTR_MUSTUNDERSTAND_FALSE;
		addAttribute(qName, value);
	}

	public void setMustUnderstand(String mustUnderstand)
			throws SOAPProcException {
		if (SOAPConstants.ATTR_MUSTUNDERSTAND_TRUE.equals(mustUnderstand)
				|| SOAPConstants.ATTR_MUSTUNDERSTAND_FALSE
						.equals(mustUnderstand)
				|| SOAPConstants.ATTR_MUSTUNDERSTAND_0.equals(mustUnderstand)
				|| SOAPConstants.ATTR_MUSTUNDERSTAND_1.equals(mustUnderstand)) {
			QName qName = new QName(
					SOAP11Constants.SOAP_ENVELOPE_NAMESPACE_URI,
					SOAPConstants.ATTR_MUSTUNDERSTAND, getEnvelopePrefix());

			addAttribute(qName, mustUnderstand);
		} else {
			throw new SOAPProcException(
					"mustUndertand should be one of \"true\", "
							+ "\"false\", \"0\" or \"1\" ");
		}

	}

	public void setRelay(boolean relay) {
		throw new UnsupportedOperationException("Not supported for SOAP 1.1");

	}

	public boolean getRelay() {
		throw new UnsupportedOperationException("Not supported for SOAP 1.1");
	}

}
