package org.act.xservices.rt.soap12.impl;

import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.List;

import org.act.xservices.rt.soap.ISOAPFaultCode;
import org.act.xservices.rt.soap.ISOAPFaultDetail;
import org.act.xservices.rt.soap.ISOAPFaultNode;
import org.act.xservices.rt.soap.ISOAPFaultReason;
import org.act.xservices.rt.soap.ISOAPFaultRole;
import org.act.xservices.rt.soap.constants.SOAP12Constants;
import org.act.xservices.rt.soap.constants.SOAPConstants;
import org.act.xservices.rt.soap.exception.SOAPException;
import org.act.xservices.rt.soap.exception.SOAPProcException;
import org.act.xservices.rt.soap.impl.SOAPFaultImpl;
import org.act.xservices.rt.xom.DocumentHelper;
import org.act.xservices.rt.xom.Metaset;
import org.act.xservices.rt.xom.Namespace;
import org.act.xservices.rt.xom.QName;

/**
 * 
 * @author Viking.Liu
 * @date2010-6-9
 * @since JDK 1.5
 * @version 1.0.0
 * 
 */
public class SOAP12FaultImpl extends SOAPFaultImpl {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	private boolean hasCode = false;
	private boolean hasReason = false;
	private boolean hasNode = false;
	private boolean hasRole = false;
	private boolean hasDetail = false;

	public SOAP12FaultImpl() {
		super();
	}

	public SOAP12FaultImpl(QName qname) {
		super(qname);
	}

	public SOAP12FaultImpl(String name, Namespace namespace) {
		super(name, namespace);
	}

	public SOAP12FaultImpl(Exception e) {
		super();
		this.exception = e;
		this.setException(e);
	}

	public ISOAPFaultCode getFaultCode() {
		Metaset code = this.element(SOAP12Constants.SOAP_FAULT_CODE_NAME);
		if (code != null) {
			hasCode = true;
			return (ISOAPFaultCode) code;
		}
		return null;
	}

	public ISOAPFaultDetail getFaultDetail() {
		Metaset detail = this.element(SOAP12Constants.SOAP_FAULT_DETAIL_NAME);
		if (detail != null) {
			hasDetail = true;
			return (ISOAPFaultDetail) detail;
		}
		return null;
	}

	public ISOAPFaultNode getFaultNode() {
		Metaset node = this.element(SOAP12Constants.SOAP_FAULT_NODE_NAME);
		if (node != null) {
			hasNode = true;
			return (ISOAPFaultNode) node;
		}
		return null;
	}

	public ISOAPFaultReason getFaultReason() {
		Metaset reason = this.element(SOAP12Constants.SOAP_FAULT_REASON_NAME);
		if (reason != null) {
			hasReason = true;
			return (ISOAPFaultReason) reason;
		}
		return null;
	}

	public ISOAPFaultRole getFaultRole() {
		Metaset role = this.element(SOAP12Constants.SOAP_FAULT_ROLE_NAME);
		if (role != null) {
			hasRole = true;
			return (ISOAPFaultRole) role;
		}
		return null;
	}

	public void addFaultCode(ISOAPFaultCode soapFaultCode)
			throws SOAPProcException {
		if (hasCode || this.elements().size() > 0) {
			throw new SOAPProcException(
					"The Code must be the first child of Fault."
							+ "The SOAP Fault already has children.");
		}
		hasCode = true;
		add(soapFaultCode);
	}

	public void addFaultReason(ISOAPFaultReason soapFaultReason)
			throws SOAPProcException {
		if (hasReason || this.elements().size() > 1) {
			throw new SOAPProcException(
					"The Reason must be the second child of the SOAP Fault,"
							+ " but now SOAP Fault already has more than one child.");
		}
		this.getFaultCode();
		if (!hasCode) {
			throw new SOAPProcException("SOAP Fault first child is not Code.");
		}
		hasReason = true;
		add(soapFaultReason);
	}

	public void addFaultNode(ISOAPFaultNode node) throws SOAPProcException {
		verify();
		this.getFaultNode();
		if (hasNode) {
			throw new SOAPProcException(
					"SOAP Fault already has SOAP Fault Node, and it can only has one Node.");
		}
		add(node);
	}

	public void addFaultRole(ISOAPFaultRole soapFaultrole)
			throws SOAPProcException {
		verify();
		this.getFaultRole();
		if (hasRole) {
			throw new SOAPProcException(
					"SOAP Fault already has SOAP Fault Role, and it can only has one Role.");
		}
		hasRole = true;
		add(soapFaultrole);
	}

	public void addFaultDetail(ISOAPFaultDetail soapFaultDetail)
			throws SOAPProcException {
		verify();
		this.getFaultDetail();
		if (hasDetail) {
			throw new SOAPProcException(
					"SOAP Fault already has SOAP Fault Detail, and it can only has one Detail.");
		}
		hasDetail = true;
		add(soapFaultDetail);
	}

	public void setException(Exception e) throws SOAPException {
		this.exception = e;		
		if (element(SOAP12Constants.SOAP_FAULT_CODE_NAME) == null) {
			ISOAPFaultCode code=new SOAP12FaultCodeImpl();
			code.addFaultValue(new SOAP12FaultValueImpl());
			add(code);
		}
		if (element(SOAP12Constants.SOAP_FAULT_REASON_NAME) == null) {
			add(new SOAP12FaultReasonImpl());
		}
		ISOAPFaultDetail detail = getFaultDetail();
		if (detail == null) {
			detail = new SOAP12FaultDetailImpl();
			add(detail);
		}
		StringWriter sw = new StringWriter();
		e.printStackTrace(new PrintWriter(sw));
		sw.flush();
		Metaset detailEntity = DocumentHelper
				.createElement(SOAPConstants.SOAP_FAULT_DETAIL_EXCEPTION_ENTRY);
		detailEntity.setText(sw.toString());
		detail.add(detailEntity);
	}

	private void verify() throws SOAPProcException {
		List<Metaset> children=elements();
		if (children.size() < 2) {
			throw new SOAPProcException(
					"SOAP Fault first child should be Code ,and the second child should be Reason. But it has less than tow children.");
		}
		if (!(children.get(0)instanceof ISOAPFaultCode)) {
			throw new SOAPProcException(
					"SOAP Fault first child should be Code ,but it is not.");
		}
		if (!(children.get(1) instanceof ISOAPFaultReason)) {
			throw new SOAPProcException(
					"SOAP Fault second child should be Reason ,but it is not.");
		}
	}
}
