package org.act.xservices.rt.soap12.impl;

import java.util.List;

import org.act.xservices.rt.soap.ISOAPFaultText;
import org.act.xservices.rt.soap.constants.SOAP12Constants;
import org.act.xservices.rt.soap.exception.SOAPException;
import org.act.xservices.rt.soap.exception.SOAPProcException;
import org.act.xservices.rt.soap.impl.SOAPFaultReasonImpl;
import org.act.xservices.rt.xom.Attribute;
import org.act.xservices.rt.xom.Metaset;
import org.act.xservices.rt.xom.Namespace;
import org.act.xservices.rt.xom.Node;
import org.act.xservices.rt.xom.QName;
import org.act.xservices.rt.xom.tree.BackedList;

/**
 * 
 * @author Viking.Liu
 * @date2010-6-9
 * @since JDK 1.5
 * @version 1.0.0
 * 
 */
public class SOAP12FaultReasonImpl extends SOAPFaultReasonImpl {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	public SOAP12FaultReasonImpl(QName qname) {
		super(qname);
		if (qname == null) {
			throw new SOAPException("The qname is null.");
		}
		if (!qname.getName().equals(SOAP12Constants.SOAP_FAULT_REASON_NAME)) {
			throw new SOAPException(
					"The SOAP Fault Reason name should be SOAP12 Reason. But it is "
							+ qname.getName());
		}
	}

	public SOAP12FaultReasonImpl(String name, Namespace namespace) {
		super(name, namespace);
		if (name == null) {
			throw new SOAPException(
					"The Name should not be null.But it is null.");
		}
		if (!name.equals(SOAP12Constants.SOAP_FAULT_REASON_NAME)) {
			throw new SOAPException(
					"The SOAP Fault Reason name should  be Reason. But it is "
							+ name);
		}
	}

	public SOAP12FaultReasonImpl() {
		super(SOAP12Constants.SOAP_FAULT_REASON_NAME);
	}

	public void addFaultText(ISOAPFaultText soapFaultText)
			throws SOAPProcException {
		add(soapFaultText);
	}

	public List<ISOAPFaultText> getAllFaultTexts() {
		List<Node> list = contentList();
		BackedList<ISOAPFaultText> answer = createResultList();
		int size = list.size();
		for (int i = 0; i < size; i++) {
			Node object = list.get(i);
			if (object instanceof ISOAPFaultText) {
				answer.addLocal((ISOAPFaultText) object);
			}
		}
		return answer;
	}

	public ISOAPFaultText getFirstFaultText() {
		Metaset firstText=getFirstElement();	
		return firstText==null?null:(ISOAPFaultText)firstText;
	}

	public ISOAPFaultText getFaultText(String language) {
		for (Metaset metaset : elements()) {
			for (Attribute attribute : metaset.attributes()) {
				if (attribute.getName().equals(
						SOAP12Constants.SOAP_FAULT_TEXT_LANG_ATTR_NAME)) {
					return (ISOAPFaultText) metaset;
				}
			}
		}
		return null;
	}
}
