package org.act.xservices.rt.soap12.impl;

import org.act.xservices.rt.soap.ISOAPVersion;
import org.act.xservices.rt.soap.constants.SOAP12Constants;
import org.act.xservices.rt.soap.constants.SOAP12Version;
import org.act.xservices.rt.soap.constants.SOAPConstants;
import org.act.xservices.rt.soap.exception.SOAPProcException;
import org.act.xservices.rt.soap.impl.SOAPHeaderBlockImpl;
import org.act.xservices.rt.xom.Attribute;
import org.act.xservices.rt.xom.Namespace;
import org.act.xservices.rt.xom.QName;

/**
 * 
 * @author Viking.Liu
 * @date2010-6-9
 * @since JDK 1.5
 * @version 1.0.0
 * 
 */
public class SOAP12HeaderBlockImpl extends SOAPHeaderBlockImpl {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	public SOAP12HeaderBlockImpl(QName qname) {
		super(qname);
	}

	public SOAP12HeaderBlockImpl(String name, Namespace namespace) {
		super(name, namespace);
	}

	public SOAP12HeaderBlockImpl(String name) {
		super(name);
	}

	public ISOAPVersion getVersion() {
		return SOAP12Version.getSingleton();
	}

	public void setMustUnderstand(boolean mustUnderstand)
			throws SOAPProcException {
		QName qName = new QName(SOAP12Constants.SOAP_ENVELOPE_NAMESPACE_URI,
				SOAPConstants.ATTR_MUSTUNDERSTAND, this.getEnvelopePrefix());

		String value = mustUnderstand ? SOAPConstants.ATTR_MUSTUNDERSTAND_TRUE
				: SOAPConstants.ATTR_MUSTUNDERSTAND_FALSE;
		this.addAttribute(qName, value);
	}

	public void setMustUnderstand(String mustUnderstand)
			throws SOAPProcException {
		if (SOAPConstants.ATTR_MUSTUNDERSTAND_TRUE.equals(mustUnderstand)
				|| SOAPConstants.ATTR_MUSTUNDERSTAND_FALSE
						.equals(mustUnderstand)
				|| SOAPConstants.ATTR_MUSTUNDERSTAND_0.equals(mustUnderstand)
				|| SOAPConstants.ATTR_MUSTUNDERSTAND_1.equals(mustUnderstand)) {
			QName qName = new QName(SOAP12Constants.SOAP_ENVELOPE_NAMESPACE_URI,
					SOAPConstants.ATTR_MUSTUNDERSTAND, this.getEnvelopePrefix());

			this.addAttribute(qName, mustUnderstand);
		} else {
			throw new SOAPProcException(
					"mustUndertand should be one of \"true\", "
							+ "\"false\", \"0\" or \"1\" ");
		}
	}

	public void setRelay(boolean relay) {
		QName qName = new QName(SOAP12Constants.SOAP_ENVELOPE_NAMESPACE_URI,
				SOAP12Constants.SOAP_RELAY, this.getEnvelopePrefix());
        String value=relay?"true":"false";
        this.addAttribute(qName, value);
	}

	public void setRole(String roleURI) {
		QName qName = new QName(SOAP12Constants.SOAP_ENVELOPE_NAMESPACE_URI,
				SOAP12Constants.SOAP_ROLE, this.getEnvelopePrefix());        
        this.addAttribute(qName, roleURI);
	}

	public boolean getMustUnderstand() throws SOAPProcException {
		Attribute attri = this.attribute(SOAPConstants.ATTR_MUSTUNDERSTAND);
		if (attri != null) {
			String value = attri.getValue();
			if (value.equals(SOAPConstants.ATTR_MUSTUNDERSTAND_0)
					|| value.equals(SOAPConstants.ATTR_MUSTUNDERSTAND_FALSE)) {
				return false;
			} else if (value.equals(SOAPConstants.ATTR_MUSTUNDERSTAND_1)
					|| value.equals(SOAPConstants.ATTR_MUSTUNDERSTAND_TRUE)) {
				return true;
			}
			else{
				throw new SOAPProcException("The mustUnderstand value is invalid.");
			}
		}
		return false;
	}

	public boolean getRelay() {
		Attribute attri = this.attribute(SOAP12Constants.SOAP_RELAY);
		if (attri != null) {
			String value = attri.getValue();
			return Boolean.parseBoolean(value);
		}
		return false;
	}

	public String getRole() {
		Attribute attri = this.attribute(SOAP12Constants.SOAP_ROLE);
		if (attri != null) {
			return attri.getValue();
		}
		return null;
	}
}
