package org.act.xservices.rt.soap;

import static org.junit.Assert.*;

import java.util.Iterator;

import org.junit.BeforeClass;
import org.junit.Test;

public class EmtyBodymessage extends SOAPBaseTest {
	private static ISOAPMessage soapmessage = null;
	private static ISOAPEnvelope envelope = null;
	private static ISOAPHeader header = null;
	private static ISOAPBody body = null;

	@BeforeClass
	public static void init() {
		soapmessage = readSOAPMessage("soap/emptyBodymessage.xml");
		envelope = soapmessage.getSOAPEnvelope();
		header = envelope.getHeader();
		body = envelope.getBody();
	}

	@Test
	public void TestSOAPEnvelope() {

		assertEquals("Envelope", envelope.getName());

		assertEquals(0, envelope.attributes().size());
		assertEquals(4, envelope.declaredNamespaces().size());

		assertEquals("soapenv", envelope.getNamespacePrefix());
		assertEquals("http://schemas.xmlsoap.org/soap/envelope/", envelope
				.getNamespaceURI());

		assertEquals("http://schemas.xmlsoap.org/soap/envelope/", envelope
				.declaredNamespaces().get(0).getURI());
		assertEquals("http://www.w3.org/2001/XMLSchema", envelope
				.declaredNamespaces().get(1).getURI());
		assertEquals("http://www.w3.org/2001/XMLSchema-instance", envelope
				.declaredNamespaces().get(2).getURI());
		assertEquals("http://schemas.xmlsoap.org/ws/2004/03/addressing",
				envelope.declaredNamespaces().get(3).getURI());

		assertEquals("soapenv", envelope.declaredNamespaces().get(0)
				.getPrefix());
		assertEquals("xsd", envelope.declaredNamespaces().get(1).getPrefix());
		assertEquals("xsi", envelope.declaredNamespaces().get(2).getPrefix());
		assertEquals("wsa", envelope.declaredNamespaces().get(3).getPrefix());

	}

	@Test
	public void TestSOAPHeader() {
		assertEquals("Header", header.getName());
		assertEquals("soapenv", header.getNamespacePrefix());
		assertEquals("http://schemas.xmlsoap.org/soap/envelope/", header
				.getNamespaceURI());
		assertEquals(3, header.elements().size());
		Iterator<ISOAPHeaderBlock> headerBlocks = header
				//.examineAllHeaderBlocks();
		        .examineAllHeaderBlocks();
		    
		ISOAPHeaderBlock[] headerList = new ISOAPHeaderBlock[3];
		int index = 0;
		while (headerBlocks.hasNext()) {
			headerList[index++] = headerBlocks.next();
		}
		assertEquals("MessageID", headerList[0].getName());
		assertEquals("To", headerList[1].getName());
		assertEquals("From", headerList[2].getName());
		
		assertFalse(headerList[0].getMustUnderstand());
		assertFalse(headerList[1].getMustUnderstand());
		assertFalse(headerList[2].getMustUnderstand());
		
		assertEquals("wsa",headerList[0].getNamespacePrefix());
		assertEquals("wsa",headerList[1].getNamespacePrefix());
		assertEquals("wsa",headerList[2].getNamespacePrefix());
		
		assertEquals("http://schemas.xmlsoap.org/ws/2004/03/addressing",headerList[0].getNamespaceURI());
		assertEquals("http://schemas.xmlsoap.org/ws/2004/03/addressing",headerList[1].getNamespaceURI());
		assertEquals("http://schemas.xmlsoap.org/ws/2004/03/addressing",headerList[2].getNamespaceURI());
	}
	
	public void TestSOAPBody(){
		assertEquals("Body", body.getName());
		assertEquals("soapenv", body.getNamespacePrefix());
		assertEquals("http://schemas.xmlsoap.org/soap/envelope/", body
				.getNamespaceURI());
		
	}
}
