package org.act.xservices.rt.soap;

import static org.junit.Assert.*;

import java.util.Iterator;

import org.act.xservices.rt.soap.constants.SOAP11Constants;
import org.act.xservices.rt.soap.constants.SOAPConstants;
import org.act.xservices.rt.soap.exception.SOAPProcException;
import org.junit.BeforeClass;
import org.junit.Test;

public class SOAP11Test extends SOAPBaseTest {

	
	private static ISOAPMessage soapmessage = null;
	private static ISOAPEnvelope envelope = null;	
	private static ISOAPHeader header = null;
	private static ISOAPBody body = null;
	private static ISOAPFault fault=null;
	
	@BeforeClass
	public static void init() throws SOAPProcException{
		soapmessage = readSOAPMessage("soap/soap11/message.xml");		
		envelope = soapmessage.getSOAPEnvelope();
		header = envelope.getHeader();
		body = envelope.getBody();
		fault=body.getFault();
	}
	
	@Test
	public void testSOAPMessage() {
		//assertEquals("utf-8", soapmessage.getXMLEncoding());
		System.out.println(soapmessage.asXML());
	}
	

	@Test
	public void testSOAPEnvelope() {	
		assertTrue(envelope != null);
		assertEquals(SOAPConstants.SOAP_DEFAULT_NAMESPACE_PREFIX,envelope.getNamespacePrefix());
		assertEquals(SOAP11Constants.SOAP_ENVELOPE_NAMESPACE_URI,envelope.getNamespaceURI());
		assertEquals(SOAP11Constants.SOAP_ENVELOPE_NAME,envelope.getName());
		assertTrue(envelope.hasFault());
		assertEquals(SOAPConstants.SOAP_BODY_FAULT_NAME,envelope.getSOAPBodyFirstElementLocalName());
		assertEquals(envelope.getNamespace(),envelope.getSOAPBodyFirstElementNS());
	}

	@Test
	public void testSOAPHeader() {
		assertTrue(header != null);
		assertEquals(SOAPConstants.SOAP_HEADER_NAME,header.getName());
		assertEquals(SOAPConstants.SOAP_DEFAULT_NAMESPACE_PREFIX,header.getNamespacePrefix());
		
	}

	@Test
	public void testSOAPHeaderBlock() {
		Iterator<ISOAPHeaderBlock> headerblocks=header.examineAllHeaderBlocks();
		while(headerblocks.hasNext()){
			ISOAPHeaderBlock headerblock=headerblocks.next();
			try {
				System.out.println(headerblock.getMustUnderstand());
				System.out.println(headerblock.getRole());
			} catch (SOAPProcException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}

	@Test
	public void testSOAPBody() {
		assertTrue(body != null);
	}
	
	@Test
	public void testSOAPFault(){
		ISOAPFaultCode code=fault.getFaultCode();
		ISOAPFaultReason string=fault.getFaultReason();
		ISOAPFaultRole actor=fault.getFaultRole();
		ISOAPFaultDetail detail=fault.getFaultDetail();
		System.out.println(code.getText());
		System.out.println(string.getText());
		System.out.println(actor.getText());
		System.out.println(code.getText());
		System.out.println(detail.asXML());
	}
}
