package org.act.xservices.rt.soap;

import static org.junit.Assert.*;

import java.util.Iterator;

import org.act.xservices.rt.soap.constants.SOAP12Constants;
import org.act.xservices.rt.soap.constants.SOAPConstants;
import org.act.xservices.rt.soap.exception.SOAPProcException;
import org.junit.BeforeClass;
import org.junit.Test;

public class SOAP12Test extends SOAPBaseTest {
	private static ISOAPMessage soapmessage = null;
	private static ISOAPEnvelope envelope = null;
	private static ISOAPHeader header = null;
	private static ISOAPBody body = null;
	private static ISOAPFault fault = null;

	@BeforeClass
	public static void setUpBeforeClass() throws Exception {
		soapmessage = readSOAPMessage("soap/soap12/message.xml");
		envelope = soapmessage.getSOAPEnvelope();
		header = envelope.getHeader();
		body = envelope.getBody();
		fault = body.getFault();
	}

	@Test
	public void testSOAPMessage() {
		//assertNotNull(soapmessage.getXMLEncoding());
		System.out.println(soapmessage.asXML());
	}

	@Test
	public void testSOAPEnvelope() {
		assertTrue(envelope != null);
		assertEquals("env", envelope.getNamespacePrefix());
		assertEquals(SOAP12Constants.SOAP_ENVELOPE_NAMESPACE_URI, envelope
				.getNamespaceURI());
		assertEquals(SOAP12Constants.SOAP_ENVELOPE_NAME, envelope.getName());
	}

	@Test
	public void testSOAPBody() {
		assertTrue(body != null);
	}

	@Test
	public void testSOAPHeader() {
		assertTrue(header != null);
		assertEquals(SOAPConstants.SOAP_HEADER_NAME, header.getName());
		assertEquals("env", header.getNamespacePrefix());

	}

	@Test
	public void testSOAPHeaderBlock() {
		Iterator<ISOAPHeaderBlock> headerblocks = header
				.examineAllHeaderBlocks();
		System.out.println("Header block!");
		while (headerblocks.hasNext()) {
			ISOAPHeaderBlock headerblock = headerblocks.next();
			try {
				System.out.println(headerblock.asXML());
				System.out.println(headerblock.getMustUnderstand());
				System.out.println(headerblock.getRole());
			} catch (SOAPProcException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}

	@Test
	public void testSOAPFault() {
		ISOAPFaultCode code = fault.getFaultCode();
		ISOAPFaultReason reason = fault.getFaultReason();
		ISOAPFaultNode node = fault.getFaultNode();
		ISOAPFaultRole role = fault.getFaultRole();
		ISOAPFaultDetail detail = fault.getFaultDetail();
		assertNotNull(code);
		assertNotNull(reason);
		assertNotNull(node);
		assertNotNull(role);
		assertNotNull(detail);
	}

	@Test
	public void testCode() throws SOAPProcException {
		ISOAPFaultCode code = fault.getFaultCode();
		ISOAPFaultValue value = code.getFaultValue();
		System.out.println(value.getText());

		ISOAPFaultSubCode subcode = code.getFaultSubCode();

		System.out.println(subcode.getFaultValue().getText());
		System.out.println(subcode.getFaultSubCode().getFaultValue().getText());
		System.out.println(subcode.getFaultSubCode().getFaultSubCode()
				.getFaultValue().getText());
	}

	@Test
	public void testReason() {
		ISOAPFaultReason reason = fault.getFaultReason();
		ISOAPFaultText text = reason.getFirstFaultText();
		System.out.println(text.getText());
	}

	@Test
	public void testNode() {

	}

	@Test
	public void testRole() {

	}

	@Test
	public void testDetail() {

	}
}
