package org.act.xservices.rt.soap.util;

import java.util.ArrayList;
import java.util.List;

import org.act.xservices.rt.soap.ISOAPEnvelope;
import org.act.xservices.rt.soap.ISOAPHeaderBlock;
import org.act.xservices.rt.soap.ISOAPVersion;
import org.act.xservices.rt.soap.constants.SOAP11Version;
import org.act.xservices.rt.soap.constants.SOAP12Version;
import org.act.xservices.rt.soap.factory.SOAPFactory;
import org.act.xservices.rt.xom.Metaset;
import org.act.xservices.rt.xom.QName;
import org.junit.BeforeClass;
import org.junit.Test;

public class SOAPMessageUtilTest {

	private ISOAPVersion version11 = new SOAP11Version();
	private ISOAPVersion version12 = new SOAP12Version();
	private ISOAPVersion versionNull = null;
	private String faultCodeReceiver = "Server";
	private String string = "something wrong";

	@BeforeClass
	public static void init() {

	}

	@Test
	public void creatStdSimpleFaultMessageTest() {

		ISOAPEnvelope envelope = SOAPMessageUtil.creatStdSimpleFaultMessage(
				version11, faultCodeReceiver, string);
		System.out.println("creatStdSimpleFaultMessage:" + envelope.asXML());
		System.out.println();
		envelope = SOAPMessageUtil.creatStdSimpleFaultMessage(version12,
				faultCodeReceiver, string);
		System.out.println("creatStdSimpleFaultMessage:" + envelope.asXML());
		System.out.println();
		envelope = SOAPMessageUtil.creatStdSimpleFaultMessage(versionNull,
				faultCodeReceiver, string);
		System.out.println("creatStdSimpleFaultMessage:" + envelope.asXML());
		System.out.println();
	}

	@Test
	public void creatResponseMessageTest() {

		SOAPFactory factory = SOAPFactory.getInstance();

		Metaset responseMetaset = factory.createElement("Add");
		Metaset returnMetaset = factory.createElement("return");
		responseMetaset.add(returnMetaset);
		returnMetaset.setText("14");

		List<ISOAPHeaderBlock> inMessageHeaders = new ArrayList<ISOAPHeaderBlock>();
		inMessageHeaders.add(factory.createSOAPHeaderBlock(new QName(
				"http://act/svn", "from", "wsa")));
		inMessageHeaders.add(factory.createSOAPHeaderBlock(new QName(
				"http://act/svn", "to", "wsa")));
		ISOAPEnvelope envelope = SOAPMessageUtil.creatResponseMessage(
				version11, responseMetaset, inMessageHeaders);
		System.out.println("creatResponseMessage:" + envelope.asXML());
		System.out.println();
	}

	@Test
	public void creatStdFaultMessageTest() {
		SOAPFactory factory = SOAPFactory.getInstance();
		String actorString = null;
		Metaset inMessageBody = factory.createElement("Add");
		Metaset returnMetaset = factory.createElement("return");
		inMessageBody.add(returnMetaset);
		returnMetaset.setText("14");

		ISOAPEnvelope envelope = SOAPMessageUtil.creatStdFaultMessage(
				version12, faultCodeReceiver, string, actorString,
				inMessageBody);
		System.out.println("creatStdFaultMessage:" + envelope.asXML());
		System.out.println();
	}

	@Test
	public void convertMetasetToSOAPEnvelopeTest() {
		SOAPFactory factory = SOAPFactory.getInstance();
		ISOAPEnvelope envelope = factory.getDefaultFaultEnvelope();
		ISOAPEnvelope resultEnvelope = SOAPMessageUtil
				.convertMetasetToSOAPEnvelope(envelope);
		System.out.println("convertMetasetToSOAPEnvelope:"
				+ resultEnvelope.asXML());
		System.out.println();

	}
}
