package org.act.xservices.rt.soap11.impl;

import static org.junit.Assert.*;

import org.act.xservices.rt.soap.constants.SOAPConstants;
import org.act.xservices.rt.soap.exception.SOAPException;
import org.act.xservices.rt.xom.DocumentFactory;
import org.act.xservices.rt.xom.Metaset;
import org.act.xservices.rt.xom.Namespace;
import org.act.xservices.rt.xom.QName;
import org.junit.Test;

public class SOAP11BodyImplTest {

	@Test
	public void testSOAP11BodyImpl() {
		SOAP11BodyImpl body = new SOAP11BodyImpl();
		assertEquals(SOAPConstants.SOAP_BODY_NAME, body.getName());
	}

	@Test
	public void testSOAP11BodyImplQName() {
		SOAP11BodyImpl body = null;
		QName qname = null;
		// qname = new QName("", "Other", "");
		qname = new QName("", "Body", "");
		try {
			body = new SOAP11BodyImpl(qname);
		} catch (SOAPException e) {
			e.printStackTrace();
			return;
		}
		assertEquals(SOAPConstants.SOAP_BODY_NAME, body.getName());
	}

	@Test
	public void testSOAP11BodyImplStringNamespace() {
		SOAP11BodyImpl body = null;
		Namespace ns = new Namespace("envlope", "a");
		String name = null;
		// name="";
		name = "Body";
		try {
			body = new SOAP11BodyImpl(name, ns);
		} catch (SOAPException e) {
			e.printStackTrace();
			return;
		}
		assertEquals(SOAPConstants.SOAP_BODY_NAME, body.getName());
	}

	@Test
	public void testAddFaultException() {
		Exception e = new Exception("there are some wrong!");
		SOAP11BodyImpl body = new SOAP11BodyImpl();
		body.addFault(e);
        System.out.println(body.asXML());
		assertEquals(e, body.getFault().getException());
	}

	@Test
	public void testAddFault() {
		SOAP11FaultImpl fault = new SOAP11FaultImpl();
		SOAP11BodyImpl body = new SOAP11BodyImpl();
		Metaset node = DocumentFactory.getInstance().createElement("Test");
		node.setText("test node");
		// body.add(node);
		try {
			body.addFault(fault);
		} catch (SOAPException e) {
			e.printStackTrace();
			return;
		}
		assertEquals(fault, body.getFault());
	}

	@Test
	public void testGetFault() {
		SOAP11BodyImpl body = new SOAP11BodyImpl();
		assertNull(body.getFault());
		SOAP11FaultImpl fault = new SOAP11FaultImpl();
		body.addFault(fault);
		assertEquals(fault, body.getFault());
	}

	@Test
	public void testGetFirstElementLocalName() {
		SOAP11BodyImpl body = new SOAP11BodyImpl();
		assertNull(body.getFirstElementLocalName());
		SOAP11FaultImpl fault = new SOAP11FaultImpl();
		body.addFault(fault);
		assertEquals(SOAPConstants.SOAP_BODY_FAULT_NAME, body
				.getFirstElementLocalName());
	}

	@Test
	public void testGetFirstElementNS() {
		SOAP11BodyImpl body = new SOAP11BodyImpl();		
		SOAP11FaultImpl fault = new SOAP11FaultImpl();
		Namespace ns = new Namespace("envelope", "aaaaaaaa");
		assertNull(body.getFirstElementNS());
		body.addFault(fault);
		
		fault.setNamespace(ns);
		assertEquals(ns, body.getFirstElementNS());
	}

	@Test
	public void testHasFault() {
		SOAP11FaultImpl fault = new SOAP11FaultImpl();
		SOAP11BodyImpl body = new SOAP11BodyImpl();
		body.addFault(fault);
		assertTrue(body.hasFault());
	}
}
