package org.act.xservices.rt.soap12.impl;

import static org.junit.Assert.*;

import java.util.List;

import org.act.xservices.rt.soap.ISOAPFaultText;
import org.act.xservices.rt.soap.constants.SOAP12Constants;
import org.act.xservices.rt.soap.exception.SOAPException;
import org.act.xservices.rt.soap.exception.SOAPProcException;
import org.act.xservices.rt.soap.factory.SOAPFactory;
import org.act.xservices.rt.xom.Attribute;
import org.act.xservices.rt.xom.Namespace;
import org.act.xservices.rt.xom.QName;
import org.junit.Test;

public class SOAP12FaultReasonImplTest {

	@Test
	public void testSOAP12FaultReasonImplQName() {
		SOAP12FaultReasonImpl reason = null;
		QName qname = null;
		// qname = new QName("", "", "");
		qname = new QName("", "Reason", "");
		try {
			reason = new SOAP12FaultReasonImpl(qname);
		} catch (SOAPException e) {
			e.printStackTrace();
			return;
		}
		assertEquals(SOAP12Constants.SOAP_FAULT_REASON_NAME, reason
				.getName());
	}

	@Test
	public void testSOAP12FaultReasonImplStringNamespace() {
		SOAP12FaultReasonImpl reason = null;
		String name = null;
		Namespace ns = new Namespace("", "");
		// name="reason";
		name = "Reason";
		try {
			reason = new SOAP12FaultReasonImpl(name, ns);
		} catch (SOAPException e) {
			e.printStackTrace();
			return;
		}
		assertEquals(SOAP12Constants.SOAP_FAULT_REASON_NAME, reason
				.getName());
		assertEquals(ns, reason.getNamespace());
	}

	@Test
	public void testSOAP12FaultReasonImpl() {
		SOAP12FaultReasonImpl reason = new SOAP12FaultReasonImpl();
		assertEquals(SOAP12Constants.SOAP_FAULT_REASON_NAME, reason
				.getName());
	}

	@Test
	public void testAddFaultText() throws SOAPProcException {
		SOAP12FaultTextImpl text=new SOAP12FaultTextImpl();
		SOAP12FaultReasonImpl reason = new SOAP12FaultReasonImpl();
		reason.addFaultText(text);		
	}

	@Test
	public void testGetAllFaultTexts() throws SOAPProcException {
		SOAP12FaultReasonImpl reason = new SOAP12FaultReasonImpl();
		SOAP12FaultTextImpl text0=new SOAP12FaultTextImpl();
		SOAP12FaultTextImpl text1=new SOAP12FaultTextImpl();
		reason.addFaultText(text0);
		reason.addFaultText(text1);
		List<ISOAPFaultText> texts=reason.getAllFaultTexts();
		assertEquals(text0,texts.get(0));
		assertEquals(text1,texts.get(1));
	}

	@Test
	public void testGetFirstFaultText() throws SOAPProcException {
		SOAP12FaultTextImpl text=new SOAP12FaultTextImpl();
		SOAP12FaultReasonImpl reason = new SOAP12FaultReasonImpl();
		reason.addFaultText(text);	
		
		assertEquals(text,reason.getFirstFaultText());
	}

	@Test
	public void testGetFaultText() throws SOAPProcException {
		SOAP12FaultReasonImpl reason = new SOAP12FaultReasonImpl();
		SOAP12FaultTextImpl text=new SOAP12FaultTextImpl();
		Attribute attri = SOAPFactory.getInstance().createAttribute(
				text,
				new QName(SOAP12Constants.SOAP_FAULT_TEXT_LANG_ATTR_NS_URI,
						"lang",
						SOAP12Constants.SOAP_FAULT_TEXT_LANG_ATTR_NS_PREFIX),
				"en");
		text.add(attri);
		reason.addFaultText(text);
		
		assertEquals(text,reason.getFaultText("en"));
	}

}
