package org.act.xservices.rt.metaset;

import java.io.ByteArrayInputStream;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;

import javax.xml.stream.XMLStreamException;

import org.act.xservices.rt.metaset.exception.MetasetParsingException;
import org.act.xservices.rt.xom.Document;
import org.act.xservices.rt.xom.Metaset;
import org.act.xservices.rt.xom.io.STAXEventReader;
/**
 * 
 * @author viking
 *
 */
public class MetasetStAXParser implements MetasetParser {

	private STAXEventReader stax = new STAXEventReader(MetasetBuilder.getInstance());
	
	/**
	 * Parse an InputStream and return a ServiceSet
	 * 
	 * @param in
	 * @return
	 * @throws MetasetParsingException
	 * @throws XMLStreamException
	 */
	public Metaset parse(InputStream in) throws MetasetParsingException {
		try {
			Document doc = stax.readDocument(in);
			return doc.getRootElement();
		} catch (XMLStreamException e) {
			e.printStackTrace();
		}
		return null;
	}

	/**
	 * Construct a Metaset from a xml file
	 * 
	 * @param filePath
	 *            String the xml file full path
	 * @return Metaset the <code>Metaset</code> constructed from the xml file
	 * @throws Exception
	 *             when exception occured.
	 */
	public Metaset parse(String filePath) throws MetasetParsingException {
		try {
			Metaset results = parse(new FileInputStream(filePath));
			return results;
		} catch (FileNotFoundException e) {
			throw new MetasetParsingException(
					"Malformat error with Metaset parser for "
							+ e.getMessage());
		}
	}

	/**
	 * Construct a Metaset from a String
	 * 
	 * @param xmlString
	 * @return
	 * @throws Exception
	 */
	public Metaset parseXMLString(String xmlString) throws MetasetParsingException {
		InputStream in = new ByteArrayInputStream(xmlString.getBytes());
		Metaset results = parse(in);
		return results;
	}

}
