package org.act.xservices.rt.metaset.utils;

import java.util.StringTokenizer;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.act.xservices.rt.metaset.MetasetBuilder;
import org.act.xservices.rt.metaset.exception.ElementExpressionException;
import org.act.xservices.rt.metaset.exception.QNameExpressionException;
import org.act.xservices.rt.metaset.query.PathExpConstant;
import org.act.xservices.rt.xom.Attribute;
import org.act.xservices.rt.xom.Namespace;
import org.act.xservices.rt.xom.QName;


public class ExpressionUtil {
	
	private static final MetasetBuilder metBuilder = MetasetBuilder.getInstance();
	
	/**
	 * ƥQName Stringʽ 
	 * ʽ
	 * 1. act:book
	 * 2. {http://act.buaa.edu.cn}book
	 * 3. book
	 * 4. *
	 * 5. .
	 */
	private static final Pattern QNamePattern = Pattern.compile(
			"((([^\\s\\{\\}]+:)|(\\{[^\\s\\{\\}\\\\]+\\}))?[^\\s:][^\\s:\\{\\}=]+)|\\*|\\.");
	
	
	
	/**ƥԵʽ */
	private static final Pattern attrPattern = Pattern.compile(
			"(((([^\\s\\{\\}]+:)|(\\{[^\\s\\{\\}\\\\]+\\}))?[^\\s:][^\\s:\\{\\}=]+)|\\*)(=\\'(([^\\'\\\"]+)|\\*)\\')?");
	
	
	
	
	/**
	 * QNameString,ʽact:book  {http://act.buaa.edu.cn}book  book
	 * 
	 * @param qnStrT
	 * @return õQName׳쳣
	 * @throws QNameExpressionException
	 * 
	 */
	public static QName parseQNameString(String qnStr) throws QNameExpressionException{
		
		String qnStrT = qnStr.trim();
		
		//ͨʽ֤QNameƽ֤
		Matcher qNameMatcher = QNamePattern.matcher(qnStrT);
		if(!qNameMatcher.matches()){
			throw new QNameExpressionException(
					"Error when match the QName Pattern. For " + qnStrT);
		}
		
		final int ns_Start = qnStrT.indexOf(PathExpConstant.NAMESPACEURI_START);
		final int ns_end = qnStrT.indexOf(PathExpConstant.NAMESPACEURI_END);	
		final boolean hasNamespaceStart = ns_Start != -1;
		final boolean hasNamespaceEnd = ns_end != -1;
		final boolean hasPrefixSeperator = qnStrT.indexOf(PathExpConstant.PREFIX_SEPERATOR) != -1;
		
		if( hasNamespaceStart && hasNamespaceEnd ){
			//ΪURIʽ		
			if( ns_Start == ns_end-1 || qnStrT.length() == ns_end+1 )
				throw new QNameExpressionException("Error QName for " + qnStrT);//ʽûURIlocalpart
			
			String nsURI = qnStrT.substring(ns_Start+1, ns_end);
			String localpart = qnStrT.substring(ns_end+1);			 
			return new QName(localpart, new Namespace("",nsURI));//ע˴URIΪ
			
		}else if( hasPrefixSeperator && !hasNamespaceStart && !hasNamespaceEnd ){
			//Ϊǰ׺ʽ
			StringTokenizer st = new StringTokenizer(qnStrT, PathExpConstant.PREFIX_SEPERATOR);
			
			if(st.countTokens() != 2){
				throw new QNameExpressionException("Error QName for " + qnStrT); //ʽ
			}
			
			String prefix = st.nextToken();
			String localpart = st.nextToken();			
			return new QName(localpart,new Namespace( prefix,""));//ע˴URIΪ
			
		}else{
			//Ϊʽ
			return new QName(qnStrT);
		}
	}
	

	/**
	 * Աʽ
	 * {http://xiao.org}name='java'
	 * 
	 * @param attrString
	 * @return
	 * @throws ElementExpressionException
	 */
	public static Attribute parseAttributeString(String attrString) throws ElementExpressionException{
		
		String attrStringT = attrString.trim();
		
		//ͨʽ֤,ַ֤
		Matcher attrMatcher = attrPattern.matcher(attrStringT);
		if(!attrMatcher.matches()){
			throw new ElementExpressionException(
					"Error when match the Attribute Pattern. For " + attrStringT);
		}
		
		int pos = attrStringT.indexOf(PathExpConstant.SEPERATOR_OF_ATTRIBUTE_VALUE);
		
		//жǷֵ
		if (pos != -1) { //ֵ
			
			if(pos == (attrStringT.length()-1))//ֵΪ
				throw new ElementExpressionException("Error Attribute for " + attrStringT);
			
			String temp_Name = attrStringT.substring(0, pos); //{http://xiao.org}name
			String temp_attrValue = attrStringT.substring(pos+1); //'java'
			
			QName qn = ExpressionUtil.parseQNameString(temp_Name);	
			
			int pos_Value_Seperator_Start = temp_attrValue.indexOf(PathExpConstant.Value_SEPERATOR);
			int pos_Value_Seperator_End = temp_attrValue.lastIndexOf(PathExpConstant.Value_SEPERATOR);
			
			if( pos_Value_Seperator_Start!=0 || 
					pos_Value_Seperator_End!=(temp_attrValue.length()-1) ){
				throw new ElementExpressionException("Error Attribute Value String for " + temp_attrValue);
			}
			
			String attrValue = 
				temp_attrValue.substring(pos_Value_Seperator_Start+1, pos_Value_Seperator_End);
	
			return metBuilder.newAttribute(qn, attrValue);
			
		} else {//ûֵ
			if(attrStringT.indexOf(PathExpConstant.Value_SEPERATOR) != -1)
				throw new ElementExpressionException("Error Attribute name for " + attrStringT);
			
			QName qn = ExpressionUtil.parseQNameString(attrStringT);
			return metBuilder.newAttribute(qn, PathExpConstant.NULL_ATTRIBUTE_VALUE);
		}
	}
	
	
	/**
	 * ƥAttributeQNameǷ
	 * QNameַʽact:book  {http://act.buaa.edu.cn}book  book
	 * 
	 * @param attrStrQName һQNameַQName
	 * @param attrMatchedQName ƥQName
	 * @return
	 */
	public static boolean matchAttributeQName(QName attrStrQName, QName attrMatchedQName){
	
		final boolean hasPrefix = !attrStrQName.getNamespacePrefix().equals("");
		final boolean hasNamespaceURI = !attrStrQName.getNamespaceURI().equals("");
		

		if (attrStrQName.getName().equals(PathExpConstant.ALL_MATCH_CODE)) {
			return true;

		} else if (attrStrQName.getName().equals(attrMatchedQName.getName())) {

			if (!hasPrefix && !hasNamespaceURI) {
				// localpart
				return true;

			} else if (hasPrefix && 
					!hasNamespaceURI && 
					attrStrQName.getNamespacePrefix().equals(attrMatchedQName.getNamespacePrefix())) {
				// prefix
				return true;

			} else if (hasNamespaceURI && 
					!hasPrefix &&
					attrStrQName.getNamespaceURI().equals(attrMatchedQName.getNamespaceURI())) {
				// NamespaceURI
				return true;
			} else if(hasPrefix && 
					hasNamespaceURI && 
					attrStrQName.getNamespacePrefix().equals(attrMatchedQName.getNamespacePrefix()) &&
					attrStrQName.getNamespaceURI().equals(attrMatchedQName.getNamespaceURI())){
				return true;
			}
		}
		
		return false;

	}

}
