package org.act.xservices.rt.xom.impl.xpath;

import java.io.IOException;
import java.io.InputStream;
import java.util.List;

import org.act.xservices.rt.metaset.MetasetBuilder;
import org.act.xservices.rt.metaset.MetasetParserFactory;
import org.act.xservices.rt.metaset.exception.MetasetParsingException;
import org.act.xservices.rt.xom.AbstractTestCase;
import org.act.xservices.rt.xom.Attribute;
import org.act.xservices.rt.xom.DocumentHelper;
import org.act.xservices.rt.xom.Metaset;
import org.act.xservices.rt.xom.Node;
import org.act.xservices.rt.xom.XPath;
import org.act.xservices.rt.xom.xpath.DefaultXPath;
import org.jaxen.JaxenException;
import org.jaxen.SimpleNamespaceContext;
import org.xml.sax.SAXException;

import junit.framework.TestCase;

public class BaseXPathTest extends TestCase {

	private Metaset metaset;
	private static MetasetBuilder builder;

	public BaseXPathTest(String name) {
		super(name);
	}

	protected void setUp() throws Exception {
		builder = new MetasetBuilder();
		metaset = builder.newMetaset("root");
	}

	@SuppressWarnings("unchecked")
	public void testParentOfSelection() throws JaxenException {
		/*
		 * html a img a <- return that node img <- select this node
		 */
		XPath xpath = DocumentHelper
				.createXPath("(.//html/a/img[contains(@src,'gif')])[2]");

		Metaset html = builder.newMetaset("html");
		Metaset a1 = builder.newMetaset("a");
		Metaset a2 = builder.newMetaset("a");
		Metaset img1 = builder.newMetaset("img");
		Attribute img1_src = builder.newAttribute("src", "1.gif");
		Metaset img2 = builder.newMetaset("img");
		Attribute img2_src = builder.newAttribute("src", "2.gif");

		img1.add(img1_src);
		img2.add(img2_src);
		a1.add(img1);
		a2.add(img2);
		html.add(a1);
		html.add(a2);
		metaset.add(html);

		List result = xpath.selectNodes(metaset);
		assertEquals(1, result.size());
		// assertEquals(a2, result.get(0));
	}

	@SuppressWarnings("unchecked")
	public void testAncestorAxis() throws JaxenException {

		XPath xpath = DocumentHelper.createXPath("ancestor::*");

		Metaset root = builder.newMetaset("root");
		Metaset parent = builder.newMetaset("parent");
		metaset.add(root);

		Metaset child = builder.newMetaset("child");
		root.add(parent);
		parent.add(child);

		List result = xpath.selectNodes(child);
		result.remove(metaset); // remove the empty node(document node)
		assertEquals(2, result.size());
		assertEquals(root, result.get(0));
		assertEquals(parent, result.get(1));

		metaset.elements().clear();
	}

	@SuppressWarnings("unchecked")
	public void testPrecedingSiblingAxisIsInDocumentOrder()
			throws JaxenException {

		XPath xpath = DocumentHelper.createXPath("preceding-sibling::*");

		Metaset root = builder.newMetaset("root");
		metaset.add(root);
		root.setParent(metaset);
		Metaset child1 = builder.newMetaset("child1");
		root.add(child1);
		child1.setParent(root);
		Metaset child2 = builder.newMetaset("child2");
		root.add(child2);
		child2.setParent(root);
		Metaset child3 = builder.newMetaset("child3");
		root.add(child3);
		child3.setParent(root);

		List result = xpath.selectNodes(child3);
		assertEquals(2, result.size());
		assertEquals(child1, result.get(0));
		assertEquals(child2, result.get(1));

		metaset.elements().clear();

	}

	@SuppressWarnings("unchecked")
	public void testPrecedingAxisIsInDocumentOrder() throws JaxenException {

		XPath xpath = DocumentHelper.createXPath("preceding::*");

		Metaset root = builder.newMetaset("root");
		metaset.add(root);
		root.setParent(metaset);
		Metaset parent1 = builder.newMetaset("parent1");
		root.add(parent1);
		parent1.setParent(root);
		Metaset parent2 = builder.newMetaset("parent2");
		root.add(parent2);
		parent2.setParent(root);
		Metaset child1 = builder.newMetaset("child1");
		parent2.add(child1);
		child1.setParent(parent2);
		Metaset child2 = builder.newMetaset("child2");
		parent2.add(child2);
		child2.setParent(parent2);
		Metaset child3 = builder.newMetaset("child3");
		parent2.add(child3);
		child3.setParent(parent2);

		List result = xpath.selectNodes(child3);
		assertEquals(3, result.size());
		assertEquals(parent1, result.get(0));
		assertEquals(child1, result.get(1));
		assertEquals(child2, result.get(2));

		metaset.elements().clear();

	}

	@SuppressWarnings("unchecked")
	public void testPrecedingAxisWithPositionalPredicate()
			throws JaxenException {

		XPath xpath = DocumentHelper.createXPath("preceding::*[1]");

		Metaset root = builder.newMetaset("root");
		metaset.add(root);
		root.setParent(metaset);
		Metaset child1 = builder.newMetaset("child1");
		root.add(child1);
		child1.setParent(root);
		Metaset child2 = builder.newMetaset("child2");
		root.add(child2);
		child2.setParent(root);
		Metaset child3 = builder.newMetaset("child3");
		root.add(child3);
		child3.setParent(root);

		List result = xpath.selectNodes(child3);
		assertEquals(1, result.size());
		assertEquals(child2, result.get(0));

		metaset.elements().clear();

	}

	@SuppressWarnings("unchecked")
	public void testAncestorAxisWithPositionalPredicate() throws JaxenException {

		XPath xpath = DocumentHelper.createXPath("ancestor::*[1]");

		Metaset root = builder.newMetaset("root");
		metaset.add(root);
		root.setParent(metaset);
		Metaset child1 = builder.newMetaset("child1");
		root.add(child1);
		child1.setParent(root);
		Metaset child2 = builder.newMetaset("child2");
		child1.add(child2);
		child2.setParent(child1);
		Metaset child3 = builder.newMetaset("child3");
		child2.add(child3);
		child3.setParent(child2);

		List result = xpath.selectNodes(child3);
		assertEquals(1, result.size());
		assertEquals(child2, result.get(0));

		metaset.elements().clear();

	}

	@SuppressWarnings("unchecked")
	public void testAncestorOrSelfAxis() throws JaxenException {

		XPath xpath = DocumentHelper.createXPath("ancestor-or-self::*");

		Metaset root = builder.newMetaset("root");
		Metaset parent = builder.newMetaset("parent");
		metaset.add(root);
		root.setParent(metaset);
		Metaset child = builder.newMetaset("child");
		root.add(parent);
		parent.setParent(root);
		parent.add(child);
		child.setParent(parent);

		List result = xpath.selectNodes(child);
		result.remove(metaset); // remove the empty node(document node)
		assertEquals(3, result.size());
		assertEquals(root, result.get(0));
		assertEquals(parent, result.get(1));
		assertEquals(child, result.get(2));

		metaset.elements().clear();

	}

	@SuppressWarnings("unchecked")
	public void testAbbreviatedDoubleSlashAxis() throws JaxenException {

		XPath xpath = DocumentHelper.createXPath(".//x");

		Metaset a = builder.newMetaset("a");
		Metaset b = builder.newMetaset("b");
		;
		metaset.add(a);
		Metaset x1 = builder.newMetaset("x");
		a.add(x1);
		a.add(b);
		Metaset x2 = builder.newMetaset("x");
		Metaset x3 = builder.newMetaset("x");
		Metaset x4 = builder.newMetaset("x");
		a.add(x4);
		b.add(x2);
		b.add(x3);

		List result = xpath.selectNodes(metaset);
		assertEquals(4, result.size());
		assertEquals(x1, result.get(0));
		assertEquals(x2, result.get(1));
		assertEquals(x3, result.get(2));
		assertEquals(x4, result.get(3));

		metaset.elements().clear();
	}

	@SuppressWarnings("unchecked")
	public void testAncestorFollowedByChildren() throws JaxenException {

		XPath xpath = DocumentHelper
				.createXPath("./a/b/x/ancestor::*/child::x");

		Metaset a = builder.newMetaset("a");
		Metaset b = builder.newMetaset("b");
		;
		metaset.add(a);
		a.setParent(metaset);
		Metaset x1 = builder.newMetaset("x");
		a.add(x1);
		x1.setParent(a);
		a.add(b);
		b.setParent(a);
		Metaset x2 = builder.newMetaset("x");
		Metaset x3 = builder.newMetaset("x");
		Metaset x4 = builder.newMetaset("x");
		a.add(x4);
		x4.setParent(a);
		b.add(x2);
		x2.setParent(b);
		b.add(x3);
		x3.setParent(b);

		List result = xpath.selectNodes(metaset);
		assertEquals(4, result.size());
		assertEquals(x1, result.get(0));
		assertEquals(x2, result.get(1));
		assertEquals(x3, result.get(2));
		assertEquals(x4, result.get(3));

		metaset.elements().clear();
	}

	@SuppressWarnings("unchecked")
	public void testDescendantAxis() throws JaxenException {

		XPath xpath = DocumentHelper.createXPath("./descendant::x");

		Metaset a = builder.newMetaset("a");
		Metaset b = builder.newMetaset("b");
		;
		metaset.add(a);
		a.setParent(metaset);
		Metaset x1 = builder.newMetaset("x");
		a.add(x1);
		x1.setParent(a);
		a.add(b);
		b.setParent(a);
		Metaset x2 = builder.newMetaset("x");
		Metaset x3 = builder.newMetaset("x");
		Metaset x4 = builder.newMetaset("x");
		a.add(x4);
		x4.setParent(a);
		b.add(x2);
		x2.setParent(b);
		b.add(x3);
		x3.setParent(b);

		List result = xpath.selectNodes(metaset);
		assertEquals(4, result.size());
		assertEquals(x1, result.get(0));
		assertEquals(x2, result.get(1));
		assertEquals(x3, result.get(2));
		assertEquals(x4, result.get(3));

		metaset.elements().clear();

	}

	@SuppressWarnings("unchecked")
	public void testDescendantAxisWithAttributes() throws JaxenException {

		XPath xpath = DocumentHelper.createXPath("./descendant::x/@*");

		Metaset a = builder.newMetaset("a");
		Metaset b = builder.newMetaset("b");
		metaset.add(a);
		a.setParent(metaset);
		Metaset x1 = builder.newMetaset("x");
		a.add(x1);
		x1.setParent(a);
		a.add(b);
		b.setParent(a);
		Metaset x2 = builder.newMetaset("x");
		Metaset x3 = builder.newMetaset("x");
		Metaset x4 = builder.newMetaset("x");
		a.add(x4);
		x4.setParent(a);
		b.add(x2);
		x2.setParent(b);
		b.add(x3);
		x3.setParent(b);

		Attribute a1 = builder.newAttribute("name", "1");
		x1.add(a1);
		Attribute a2 = builder.newAttribute("name", "2");
		x2.add(a2);
		Attribute a3 = builder.newAttribute("name", "3");
		x3.add(a3);
		Attribute a4 = builder.newAttribute("name", "4");
		x4.add(a4);

		List result = xpath.selectNodes(metaset);
		assertEquals(4, result.size());
		assertEquals(a1.getQName(), ((Attribute) result.get(0)).getQName()); // the
																				// return
																				// list
																				// is
																				// AttributeEX,
		assertEquals(a1.getValue(), ((Attribute) result.get(0)).getValue()); // so
																				// it
																				// can't
																				// be
																				// written
																				// as
		assertEquals(a2.getQName(), ((Attribute) result.get(1)).getQName()); // assertEquals(a1,
																				// result.get(0))
		assertEquals(a2.getValue(), ((Attribute) result.get(1)).getValue());
		assertEquals(a3.getQName(), ((Attribute) result.get(2)).getQName());
		assertEquals(a3.getValue(), ((Attribute) result.get(2)).getValue());
		assertEquals(a4.getQName(), ((Attribute) result.get(3)).getQName());
		assertEquals(a4.getValue(), ((Attribute) result.get(3)).getValue());

		metaset.elements().clear();

	}

	@SuppressWarnings("unchecked")
	public void testMultipleAttributesOnElement() throws JaxenException {

		XPath xpath = DocumentHelper.createXPath("./descendant::x/@*");

		Metaset a = builder.newMetaset("a");
		Metaset b = builder.newMetaset("b");
		metaset.add(a);
		a.setParent(metaset);
		Metaset x1 = builder.newMetaset("x");
		a.add(x1);
		x1.setParent(a);
		a.add(b);
		b.setParent(a);

		Attribute a1 = builder.newAttribute("name1", "1");
		x1.add(a1);
		Attribute a2 = builder.newAttribute("name2", "2");
		x1.add(a2);
		Attribute a3 = builder.newAttribute("name3", "3");
		x1.add(a3);
		Attribute a4 = builder.newAttribute("name4", "4");
		x1.add(a4);

		List result = xpath.selectNodes(metaset);
		assertEquals(4, result.size());
		assertEquals(a1.getQName(), ((Attribute) result.get(3)).getQName()); // the
																				// return
																				// list
																				// is
																				// AttributeEX,
		assertEquals(a1.getValue(), ((Attribute) result.get(3)).getValue()); // so
																				// it
																				// can't
																				// be
																				// written
																				// as
		assertEquals(a2.getQName(), ((Attribute) result.get(2)).getQName()); // assertEquals(a1,
																				// result.get(0)).
		assertEquals(a2.getValue(), ((Attribute) result.get(2)).getValue()); // because
																				// the
																				// result
																				// is
																				// in
																				// reverse
		assertEquals(a3.getQName(), ((Attribute) result.get(1)).getQName()); // order,
																				// so
																				// a1=result.get(3)...
		assertEquals(a3.getValue(), ((Attribute) result.get(1)).getValue());
		assertEquals(a4.getQName(), ((Attribute) result.get(0)).getQName());
		assertEquals(a4.getValue(), ((Attribute) result.get(0)).getValue());

		metaset.elements().clear();

	}

	@SuppressWarnings("unchecked")
	public void testXMLNamespaceAttributeOrderOnAncestorAxis()
			throws JaxenException {

		Metaset superroot = builder.newMetaset("superroot");
		metaset.add(superroot);
		superroot.setParent(metaset);
		Metaset root = builder.newMetaset("root");
		superroot.add(root);
		root.setParent(superroot);

		Attribute p0 = builder.newAttribute(
				"http://www.w3.org/XML/1998/namespace", "id", "xml", "p0");
		superroot.add(p0);
		Attribute p1 = builder.newAttribute(
				"http://www.w3.org/XML/1998/namespace", "id", "xml", "p1");
		root.add(p1);

		Metaset child = builder.newMetaset("child312");
		root.add(child);

		XPath xpath = DocumentHelper.createXPath("ancestor::*/@xml:*");

		List result = xpath.selectNodes(child);
		assertEquals(2, result.size());
		assertEquals(p0.getQName(), ((Attribute) result.get(0)).getQName());
		assertEquals(p0.getValue(), ((Attribute) result.get(0)).getValue());
		assertEquals(p1.getQName(), ((Attribute) result.get(1)).getQName());
		assertEquals(p1.getValue(), ((Attribute) result.get(1)).getValue());

		metaset.elements().clear();

	}

	public void testDescendantAxisWithAttributesAndChildren()
			throws JaxenException {

		XPath xpath = DocumentHelper
				.createXPath(".//descendant::x/@* | .//descendant::x");

		Metaset a = builder.newMetaset("a");
		Metaset b = builder.newMetaset("b");
		metaset.add(a);
		a.setParent(metaset);
		Metaset x1 = builder.newMetaset("x");
		a.add(x1);
		x1.setParent(a);
		a.add(b);
		b.setParent(a);
		Metaset x2 = builder.newMetaset("x");
		Metaset x3 = builder.newMetaset("x");
		Metaset x4 = builder.newMetaset("x");
		a.add(x4);
		x4.setParent(a);
		b.add(x2);
		x2.setParent(b);
		b.add(x3);
		x3.setParent(b);

		Attribute a1 = builder.newAttribute("name", "1");
		x1.add(a1);
		Attribute a2 = builder.newAttribute("name", "2");
		x2.add(a2);
		Attribute a3 = builder.newAttribute("name", "3");
		x3.add(a3);
		Attribute a4 = builder.newAttribute("name", "4");
		x4.add(a4);

		List result = xpath.selectNodes(metaset);
		assertEquals(8, result.size());
		assertEquals(x1, result.get(0));
		assertEquals(a1.getValue(), ((Attribute) result.get(1)).getValue());
		assertEquals(a1.getQName(), ((Attribute) result.get(1)).getQName());
		assertEquals(x2, result.get(2));
		assertEquals(a2.getValue(), ((Attribute) result.get(3)).getValue());
		assertEquals(a2.getQName(), ((Attribute) result.get(3)).getQName());
		assertEquals(x3, result.get(4));
		assertEquals(a3.getValue(), ((Attribute) result.get(5)).getValue());
		assertEquals(a3.getQName(), ((Attribute) result.get(5)).getQName());
		assertEquals(x4, result.get(6));
		assertEquals(a4.getValue(), ((Attribute) result.get(7)).getValue());
		assertEquals(a4.getQName(), ((Attribute) result.get(7)).getQName());

		metaset.elements().clear();

	}

	@SuppressWarnings("unchecked")
	public void testAncestorAxisWithAttributes() throws JaxenException {

		XPath xpath = DocumentHelper.createXPath("ancestor::*/@*");

		Metaset a = builder.newMetaset("a");
		Metaset b = builder.newMetaset("b");
		metaset.add(a);
		a.setParent(metaset);
		a.add(b);
		b.setParent(a);
		Metaset x3 = builder.newMetaset("x");
		b.add(x3);
		x3.setParent(b);

		Attribute a1 = builder.newAttribute("name", "1");
		a.add(a1);
		Attribute a2 = builder.newAttribute("name", "2");
		b.add(a2);
		Attribute a3 = builder.newAttribute("name", "3");
		x3.add(a3);

		List result = xpath.selectNodes(x3);
		assertEquals(2, result.size());
		assertEquals(a1.getValue(), ((Attribute) result.get(0)).getValue());
		assertEquals(a1.getQName(), ((Attribute) result.get(0)).getQName());
		assertEquals(a2.getValue(), ((Attribute) result.get(1)).getValue());
		assertEquals(a2.getQName(), ((Attribute) result.get(1)).getQName());

		metaset.elements().clear();

	}

	@SuppressWarnings("unchecked")
	public void testDescendantOrSelfAxis() throws JaxenException {

		XPath xpath = DocumentHelper.createXPath(".//descendant-or-self::x");

		Metaset a = builder.newMetaset("a");
		Metaset b = builder.newMetaset("b");
		metaset.add(a);
		a.setParent(metaset);
		Metaset x1 = builder.newMetaset("x");
		a.add(x1);
		x1.setParent(a);
		a.add(b);
		b.setParent(a);
		Metaset x2 = builder.newMetaset("x");
		Metaset x3 = builder.newMetaset("x");
		Metaset x4 = builder.newMetaset("x");
		a.add(x4);
		x4.setParent(a);
		b.add(x2);
		x2.setParent(b);
		b.add(x3);
		x3.setParent(b);

		List result = xpath.selectNodes(metaset);
		assertEquals(4, result.size());
		assertEquals(x1, result.get(0));
		assertEquals(x2, result.get(1));
		assertEquals(x3, result.get(2));
		assertEquals(x4, result.get(3));

		metaset.elements().clear();

	}

	@SuppressWarnings("unchecked")
	public void testDuplicateNodes() throws JaxenException {

		XPath xpath = DocumentHelper.createXPath(".//x | .//*");

		Metaset a = builder.newMetaset("a");
		Metaset b = builder.newMetaset("b");
		metaset.add(a);
		a.setParent(metaset);
		Metaset x1 = builder.newMetaset("x");
		a.add(x1);
		x1.setParent(a);
		a.add(b);
		b.setParent(a);
		Metaset x2 = builder.newMetaset("x");
		Metaset x3 = builder.newMetaset("x");
		Metaset x4 = builder.newMetaset("x");
		a.add(x4);
		x4.setParent(a);
		b.add(x2);
		x2.setParent(b);
		b.add(x3);
		x3.setParent(b);

		List result = xpath.selectNodes(metaset);
		assertEquals(6, result.size());

		metaset.elements().clear();

	}

	@SuppressWarnings("unchecked")
	public void testJaxen107() throws JaxenException {

		Metaset a = builder.newMetaset("http://www.a.com/", "foo", "a");
		metaset.add(a);
		a.setParent(metaset);
		Metaset b = builder.newMetaset("http://www.b.com/", "bar", "b");
		a.add(b);
		b.setParent(a);

		XPath xpath = DocumentHelper.createXPath(".//a:foo/b:bar");

		SimpleNamespaceContext context1 = new SimpleNamespaceContext();
		context1.addNamespace("a", "http://www.a.com/");
		context1.addNamespace("b", "http://www.b.com/");
		xpath.setNamespaceContext(context1);
		List result = xpath.selectNodes(metaset);
		assertEquals(1, result.size());
		assertEquals(b, result.get(0));

		metaset.elements().clear();

	}

	public void testJaxen107FromFile() throws JaxenException, SAXException,
			IOException, MetasetParsingException {		
		InputStream in = BaseXPathTest.class.getClassLoader()
				.getResourceAsStream("textFile/xpath/testNamespaces.xml");
		Metaset root = MetasetParserFactory.getStAXParser().parse(in);
		metaset.add(root);

		XPath xpath = DocumentHelper.createXPath(".//Template/Application2");

		List result = xpath.selectNodes(metaset);
		assertEquals(1, result.size());

		metaset.elements().clear();

	}

}
