package org.act.xservices.rt.xom.query;

import org.act.xservices.rt.metaset.exception.ElementExpressionException;
import org.act.xservices.rt.metaset.query.ElementExpression;

import junit.framework.TestCase;

public class ElementExpressionTest extends TestCase {

	
	public ElementExpressionTest(String name) {
		super(name);
	}

	protected void setUp() throws Exception {
		super.setUp();
	}

	protected void tearDown() throws Exception {
		super.tearDown();
	}
	
	
	
	public void testParseForNSPrefix(){
		try {
			ElementExpression eleExp = 
				new ElementExpression("act:book@actAttr:name='java'");
			
			assertEquals("book", eleExp.getElementQName().getName());
			assertEquals("act", eleExp.getElementQName().getNamespacePrefix());
			assertTrue(eleExp.getElementQName().getNamespaceURI().equals(""));
			
			assertEquals(1,eleExp.getAttrList().size());
			assertEquals("actAttr", eleExp.getAttrList().get(0).getQName().getNamespacePrefix());
			assertEquals("name", eleExp.getAttrList().get(0).getQName().getName());
			assertTrue(eleExp.getAttrList().get(0).getQName().getNamespaceURI().equals(""));
			
			assertEquals("java", eleExp.getAttrList().get(0).getValue());
			
		} catch (ElementExpressionException e) {
			// TODO Զ catch 
			fail("Error");
		}
		
	}
	
	public void testParseForNSURI(){
		try {
			ElementExpression eleExp = 
				new ElementExpression("{http://act.buaa.edu.cn}book@{http://xiao.org}nameй='java'");
			
			assertEquals("book", eleExp.getElementQName().getName());
			assertEquals("http://act.buaa.edu.cn", eleExp.getElementQName().getNamespaceURI());
			assertEquals(1,eleExp.getAttrList().size());
			assertEquals("http://xiao.org", eleExp.getAttrList().get(0).getQName().getNamespaceURI());
			assertEquals("nameй", eleExp.getAttrList().get(0).getQName().getName());
			assertEquals("java", eleExp.getAttrList().get(0).getValue());
		} catch (ElementExpressionException e) {
			// TODO Զ catch 
			fail("Error");
		}
		
	}
	
	public void testParseForNSURI1(){
		try {
			ElementExpression eleExp = 
				new ElementExpression("{http://act.buaa.edu.cn}book@actAttr:name='java'");
			
			assertEquals("book", eleExp.getElementQName().getName());
			assertEquals("http://act.buaa.edu.cn", eleExp.getElementQName().getNamespaceURI());
			
			assertEquals(1,eleExp.getAttrList().size());
			assertEquals("actAttr", eleExp.getAttrList().get(0).getQName().getNamespacePrefix());
			assertEquals("name", eleExp.getAttrList().get(0).getQName().getName());
			assertTrue(eleExp.getAttrList().get(0).getQName().getNamespaceURI().equals(""));
			
			assertEquals("java", eleExp.getAttrList().get(0).getValue());
		} catch (ElementExpressionException e) {
			// TODO Զ catch 
			fail("Error");
		}
		
	}
	
	public void testParse(){
		try {
			ElementExpression eleExp = new ElementExpression(".");
			
			assertEquals(".", eleExp.getElementQName().getName());

		} catch (ElementExpressionException e) {
			// TODO Զ catch 
			fail("Error");
		}
		
	}
	
	
	public void testParseForException(){
		try {
			
			new ElementExpression("{}book@actAttr:name='java'");
			fail("should have thrown an exception !");
			
		} catch (ElementExpressionException e) {
			// TODO Զ catch 
			assertTrue(true);
		}
		
	}
	
	public void testParseForException1(){
		try {
			
			new ElementExpression("{book@:name='java'");
			fail("should have thrown an exception !");
			
		} catch (ElementExpressionException e) {
			// TODO Զ catch 
			assertTrue(true);
		}
		
	}
	
	public void testParseForException2(){
		try {
			
			new ElementExpression("{:::}book@actAttr:='java'");
			fail("should have thrown an exception !");
			
		} catch (ElementExpressionException e) {
			// TODO Զ catch 
			assertTrue(true);
		}
		
	}
	
	public void testParseForException3(){
		try {
			
			new ElementExpression("{:::}book@actAttr:name'java'");
			fail("should have thrown an exception !");
			
		} catch (ElementExpressionException e) {
			// TODO Զ catch 
			assertTrue(true);
		}
		
	}
	

}
