package org.act.xservices.rt.xom.util;

import static org.junit.Assert.*;

import org.act.xservices.rt.metaset.MetasetBuilder;
import org.act.xservices.rt.metaset.MetasetParserFactory;
import org.act.xservices.rt.metaset.exception.MetasetParsingException;
import org.act.xservices.rt.xom.Metaset;
import org.junit.Before;
import org.junit.Test;

public class MetasetHelperTest {
	private String xmlsoap;

	@Before
	public void setUp() throws Exception {
		xmlsoap = "<soapenv:Envelope xmlns:soapenv=\"http://schemas.xmlsoap.org/soap/envelope/\" xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\">";
		xmlsoap += "<soapenv:Header>";
		xmlsoap += "<wsa:Action soapenv:actor=\"http://schemas.xmlsoap.org/soap/actor/next\" soapenv:mustUnderstand=\"0\" xmlns:wsa=\"http://schemas.xmlsoap.org/ws/2004/03/addressing\">http://docs.oasis-open.org/wsrf/rpw-2/GetMultipleResourceProperties/GetMultipleResourcePropertiesRequest</wsa:Action>";
		xmlsoap += "<wsa:To soapenv:actor=\"http://schemas.xmlsoap.org/soap/actor/next\" soapenv:mustUnderstand=\"0\" xmlns:wsa=\"http://schemas.xmlsoap.org/ws/2004/03/addressing\">http://192.168.2.100:8091/wsrf/services/JvmResourceService</wsa:To>";
		xmlsoap += "<ex:MessageId soapenv:actor=\"http://schemas.xmlsoap.org/soap/actor/next\" soapenv:mustUnderstand=\"0\" xmlns:ex=\"http://ws.act.org/resource/example/JvmResourceService\">9288</ex:MessageId>";
		xmlsoap += "</soapenv:Header>";
	    xmlsoap += "<soapenv:Body>";
	    xmlsoap += "<wsrf-rp:GetMultipleResourceProperties xmlns:wsrf-rp=\"http://docs.oasis-open.org/wsrf/rp-2\" xmlns:tns=\"http://ws.act.org/resource/example/JvmResourceService\">";
	    xmlsoap += "<wsrf-rp:ResourceProperty>tns:NumberOfBlocks</wsrf-rp:ResourceProperty>";
	    xmlsoap += "<wsrf-rp:ResourceProperty>tns:BlockSize</wsrf-rp:ResourceProperty>";
	    xmlsoap += "</wsrf-rp:GetMultipleResourceProperties>";
	    xmlsoap += "</soapenv:Body>";
	    xmlsoap += "</soapenv:Envelope>";
	}

	@Test
	public void testGetTypeOfMetaset() throws Exception {
		MetasetHelper metHelper = MetasetHelper.getInstance();
		
		Metaset xmlMet = MetasetParserFactory.getMetasetParser().parseXMLString(xmlsoap);
		Metaset metHeader = xmlMet.element("Header");
		assertEquals(metHelper.getTypeOfMetaset(metHeader), MetasetHelper.ELEMENT_NODE);

		Metaset metAction = metHeader.element("Action");
		assertEquals(metHelper.getTypeOfMetaset(metAction), MetasetHelper.TEXT_NODE);
	}
	
	@Test 
	public void testIsEqual() throws Exception{
		String xmlString1 = "<soapenv:Envelope xmlns:soapenv=\"http://schemas.xmlsoap.org/soap/envelope/\" xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\">";
		xmlString1 += "<soapenv:Body>";
		xmlString1 += "<wsrf-rp1:GetMultipleResourceProperties xmlns:wsrf-rp1=\"http://docs.oasis-open.org/wsrf/rp-2\" xmlns:tns=\"http://ws.act.org/resource/example/JvmResourceService\" soapenv:age=\"11\">";
		xmlString1 += "<wsrf-rp1:ResourceProperty>tns:NumberOfBlocks</wsrf-rp1:ResourceProperty>";
		xmlString1 += "<wsrf-rp1:ResourceProperty>tns:BlockSize</wsrf-rp1:ResourceProperty>";
		xmlString1 += "</wsrf-rp1:GetMultipleResourceProperties>";
		xmlString1 += "</soapenv:Body>";
		xmlString1 += "</soapenv:Envelope>";
		
		String xmlString2 = "<soapenv1:Envelope xmlns:soapenv1=\"http://schemas.xmlsoap.org/soap/envelope/\" xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\">";
		xmlString2 += "<soapenv1:Body>";
		xmlString2 += "<wsrf-rp:GetMultipleResourceProperties xmlns:wsrf-rp=\"http://docs.oasis-open.org/wsrf/rp-2\" xmlns:tns=\"http://ws.act.org/resource/example/JvmResourceService\" soapenv1:age=\"11\">";
		xmlString2 += "<wsrf-rp:ResourceProperty>tns:NumberOfBlocks</wsrf-rp:ResourceProperty>";
		xmlString2 += "<wsrf-rp:ResourceProperty>tns:BlockSize</wsrf-rp:ResourceProperty>";
		xmlString2 += "</wsrf-rp:GetMultipleResourceProperties>";
		xmlString2 += "</soapenv1:Body>";
		xmlString2 += "</soapenv1:Envelope>";
		
		MetasetHelper metHelper = MetasetHelper.getInstance();
		Metaset xmlMet1 = MetasetParserFactory.getMetasetParser().parseXMLString(xmlString1);
		Metaset xmlMet2 = MetasetParserFactory.getMetasetParser().parseXMLString(xmlString2);
		
		assertEquals(metHelper.isEqual(xmlMet1, xmlMet2), true);
	}

}
