package org.act.xservices.rt.xresource.addressing;

import java.net.URI;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.act.xservices.rt.soap.ISOAPHeader;
import org.act.xservices.rt.soap.ISOAPHeaderBlock;
import org.act.xservices.rt.soap.factory.SOAPFactory;
import org.act.xservices.rt.xcontainer.addressing.EndpointReference;
import org.act.xservices.rt.xcontainer.addressing.MessageHeaders;
import org.act.xservices.rt.xcontainer.addressing.uitl.SoapConstants;
import org.act.xservices.rt.xcontainer.addressing.uitl.WsaConstants;
import org.act.xservices.rt.xom.Attribute;
import org.act.xservices.rt.xom.Metaset;
import org.act.xservices.rt.xom.QName;
import org.act.xservices.rt.xom.util.MetasetHelper;
import org.act.xservices.rt.xresource.util.TnsConstants;

/**
 * Addressing for Resource, the following part should present in <Header> section
 * <tns:InstanceId>uuid:1254-1259-5656-2228</tns:InstanceId>
 * @author douyl
 *
 */
public class ResourceMessageHeaders extends MessageHeaders  {
	/**
	 * Instance Id
	 */
	private Metaset instanceIdMet;
	
	/**
	 * InstanceId String
	 */
	private String instanceID;
	
	
	public ResourceMessageHeaders(Metaset soapHeaders) throws Exception {
		super(soapHeaders);
		instanceIdMet = soapHeaders.element(TnsConstants.INSTANCE_ID).createCopy();
		if(instanceIdMet == null){
			throw new Exception("Soap Header does not contain InstanceId");
		}
		
		this.instanceID = instanceIdMet.getTextTrim();
		if(instanceID == null || instanceID.length() == 0){
			throw new Exception("InstanceId should not be empty");
		}
	}
	
	public ResourceMessageHeaders(EndpointReference faultTo, String faultUri) {
		super(faultTo, faultUri);
	}

	/**
     * 
     * һڸöWS-AϢ; ظһЯϢʱ, صheadersԱ
     * ʹáThe headersλwsa:FaultTo EPR WS-A fault action URI. 
     * ûwsa:FaultTo, wsa:FromWS-A anonymous EPRᱻʹá
     * <br><br>
     * <b>ע:</b> ÷Ӧñڴ"normal" responsesheaders - ˹Ӧʹ
     * createReplyHeaders(URI).
     *
     * @return A new MessageHeaders where:
     *         <ul>
     *         <li> wsa:ToǸö wsa:FaultTo,wsa:From, or the anonymous EPRURI</li>
     *         <li>the wsa:From is this object's wsa:To EPR</li>
     *         <li>the wsa:From Ǹö wsa:To EPR</li>
     *         <li>the wsa:RelatesTo Ǹöĵ wsa:MessageID URI</li>
     *         <li>the wsa:Action Ǳ׼ WS-A fault URI.</li>
     *         </ul>
     * 
     * @see #createReplyHeaders()
     * @see #getFaultToAddress()
     * @see #setFaultToAddress(EndpointReference)
     * @see WsaConstants#FAULT_URI
     *
     */
    public ResourceMessageHeaders createFaultHeaders()
    {
        EndpointReference faultTo = getFaultToAddress();
        String messageID = getMessageID();
        
        //
        // wsa:FaultTo  wsa:Fromڣֻܷfault headers
        //
        if (faultTo == null)
        {
            EndpointReference from = getFromAddress();
            
            if (from == null)
                from = WsaConstants.ANONYMOUS_EPR;
            
            faultTo = from;
        }
        
        //
        // 1. Ҫ wsa:FaultTo  headers
        // 2. ʹrequest MessageIDheaders
        // 3. wsa:Toеsource
        //
        ResourceMessageHeaders faultHeaders = 
            new ResourceMessageHeaders(faultTo, WsaConstants.FAULT_URI);
        
        if (messageID != null)
            faultHeaders.setRelationship(messageID);
        
        faultHeaders.setFromAddress(getToAddress());
        faultHeaders.setInstanceId(instanceID);
        faultHeaders.setInstanceIdMet(instanceIdMet);
        
        return faultHeaders;
    }
    
    /**
     * 
     * ڸö,һµWS-AϢͷ; ظһϢ(Ǵ)ʱ÷صheaders 
     * ʹ. The headerswsa:ReplyTo EPR. ûwsa:ReplyTo,
     * wsa:From or the WS-A anonymous EPRᱻʹ.
     * <br><br>
     * <b>ע:</b> This method should not be used to create headers for 
     * responses where a fault has occurred - that should be done with 
     * createFaultHeaders().
     * дʱ, ÷Ӧñڴresponses headers - Ӧ
     * createFaultHeaders()
     *
     * @return A new MessageHeaders where:
     *         <ul>
     *         <li> wsa:To Ǹö wsa:ReplyTo,wsa:From, or the anonymous EPRURI</li>
     *         <li>the wsa:From Ǹö wsa:To EPR</li>
     *         <li>the wsa:RelatesTo is the URI of this object's 
     *         wsa:MessageID</li>
     *         <li>wsa:RelatesToǸöwsa:MessageIDURI</li>
     *         <li>the wsa:Action ʹ"Response"ǰ׺ĵǰURI</li>
     *         </ul>
     * 
     * @see #getReplyToAddress()
     * @see #setReplyToAddress(EndpointReference)
     *
     */
    public ResourceMessageHeaders createReplyHeaders()
    {
        EndpointReference replyTo = getReplyToAddress();
        String messageID = getMessageID();

        //
        // wsa:ReplyTo or wsa:From,ֻܷϢͷ
        //
        if (replyTo == null)
        {
            EndpointReference from = getFromAddress();
            
            if (from == null)
                from = WsaConstants.ANONYMOUS_EPR;
            
            replyTo = from;
        }
        
        String requestAction = getAction().toString();
        String replyAction = null;
        
        int requestSuffix = requestAction.lastIndexOf(_REQUEST);
        
        if (requestSuffix >= 0)
            replyAction = requestAction.substring(0, requestSuffix) + _RESPONSE;
        
        else
            replyAction = requestAction + _RESPONSE;
        
        //
        // 1. һ wsa:ReplyTo ͷ
        // 2. headersMessageID
        // 3. wsa:Toеsource
        //
        ResourceMessageHeaders replyHeaders = new ResourceMessageHeaders(replyTo, replyAction);
        
        
        if (messageID != null)
            replyHeaders.setRelationship(messageID);
        
        replyHeaders.setFromAddress(getToAddress());
        replyHeaders.setInstanceId(instanceID);
        replyHeaders.setInstanceIdMet(instanceIdMet);
        
        return replyHeaders;
    }

	public String getInstanceID() {
		return instanceID;
	}

	public void setInstanceId(String instanceID) {
		this.instanceID = instanceID;
	}
	
	public Metaset getInstanceIdMet() {
		return instanceIdMet;
	}

	public void setInstanceIdMet(Metaset instanceIdMet) {
		this.instanceIdMet = instanceIdMet;
	}

	@Override
	public Metaset toMetaset(){
		Metaset soapHeaders = super.toMetaset();
		soapHeaders.add(this.instanceIdMet);
		return soapHeaders;
	}
	
	public ISOAPHeader toSOAPHeader(){
		SOAPFactory soapFact = SOAPFactory.getInstance();
		ISOAPHeader soapHeaders = soapFact.createSOAPHeader(SoapConstants.HEADER_QNAME);
		this.populateIntoMetaset(soapHeaders);
		return soapHeaders;
	}
	
	public List<ISOAPHeaderBlock> toHeaderBlocks(){
		List<ISOAPHeaderBlock> headerBlockList = new ArrayList<ISOAPHeaderBlock>();
		SOAPFactory soapFact = SOAPFactory.getInstance();
		
        //
        // wsa:To
        //
        URI toURI = getToAddress().getAddress();
        ISOAPHeaderBlock to = soapFact.createSOAPHeaderBlock(WsaConstants.TO_QNAME);
        to.setText(toURI.toString());
        headerBlockList.add(to);

        //
        // wsa:Action
        //
        ISOAPHeaderBlock actionMet = soapFact.createSOAPHeaderBlock(WsaConstants.ACTION_QNAME);
        actionMet.setText(getAction());
        headerBlockList.add(actionMet);
        
        //
        // wsa:MessageID
        //
        String messageID = getMessageID();
        if (messageID != null){
        	ISOAPHeaderBlock msgIdMet = soapFact.createSOAPHeaderBlock(WsaConstants.MESSAGE_ID_QNAME);
            msgIdMet.setText(messageID);
            headerBlockList.add(msgIdMet);
        }
        
        //
        // wsa:RelatesTo (optional)
        //
        String relatesTo = getRelationship();
        if (relatesTo != null)
        {
        	ISOAPHeaderBlock relationshipMet = soapFact.createSOAPHeaderBlock(WsaConstants.RELATES_TO_QNAME);
            relationshipMet.setText(relatesTo);
            
            String type = MetasetHelper.getInstance().convertQNameToString(getRelationshipType());
            // TODO: setAttributeNS
            relationshipMet.addAttribute(WsaConstants.RELATIONSHIP_TYPE, type);
            headerBlockList.add(relationshipMet);
        }
        
        //
        // wsa:From, wsa:ReplyTo, and wsa:FaultTo (optional)
        //
        
        EndpointReference[] eprs = new EndpointReference[] {
            getFromAddress(), getReplyToAddress(), getFaultToAddress() 
        };
        
        for (int n = 0; n < eprs.length; ++n)
        {
            if (eprs[n] != null)
            {
            	Metaset xml = eprs[n].toMetaset();
                ISOAPHeaderBlock tempHB = soapFact.createSOAPHeaderBlock(xml.getQName());
                List<Metaset> elemList = xml.elements();
                for(int i = 0; i < elemList.size(); i++){
                	tempHB.add(elemList.get(i).createCopy());
                }
                headerBlockList.add(tempHB);
            }
        }
        
        //
        // wsa:ReferenceParameters (optional)
        //
        
        Metaset[] parameters = getToAddress().getParameters();
        
        for (int n = 0; n < parameters.length; ++n)
        {
        	Metaset copy = parameters[n];
        	ISOAPHeaderBlock tempHB = soapFact.createSOAPHeaderBlock(copy.getQName());
            List<Metaset> elemList = copy.elements();
            for(int i = 0; i < elemList.size(); i++){
            	tempHB.add(elemList.get(i).createCopy());
            }
        	
        	Attribute irpAttr = copy.attribute(WsaConstants.IS_REFERENCE_PARAMETER_QNAME);
        	if(irpAttr == null){
        		irpAttr = soapFact.createAttribute(copy,WsaConstants.IS_REFERENCE_PARAMETER_QNAME, "true");
        	} else {
				irpAttr.setValue("true");
			}
            
        	headerBlockList.add(tempHB);
        	
        }
        
        //
        // non-WS-A headers
        //
        Iterator i = getCustomHeaderNames().iterator();
        
        while (i.hasNext())
        {
            QName qname = (QName)i.next();
            Iterator j = getCustomHeaders(qname).iterator();
            
            while (j.hasNext())
            {
                Metaset next = (Metaset)j.next();
                ISOAPHeaderBlock tempHB = soapFact.createSOAPHeaderBlock(next.getQName());
                List<Metaset> elemList = next.elements();
                for(int n = 0; n < elemList.size(); n++){
                	tempHB.add(elemList.get(n).createCopy());
                }
            }
        }
		return headerBlockList;
		
	}
}
