package org.act.xservices.rt.xresource.basefault;

import java.util.Date;
import java.util.List;

import org.act.xservices.rt.metaset.MetasetBuilder;
import org.act.xservices.rt.xcontainer.addressing.EndpointReference;
import org.act.xservices.rt.xom.Metaset;
import org.act.xservices.rt.xom.QName;
import org.act.xservices.rt.xom.util.MetasetHelper;
import org.act.xservices.rt.xresource.util.WsbfConstants;
import org.act.xservices.rt.xresource.util.WsrfConstants;

/**
 * <BaseFault> {any}* <Timestamp>xsd:dateTime</Timestamp>
 * <OriginatorReference>wsa:EndpointReferenceType</OriginatorReference>?
 * <ErrorCode dialect="anyURI">xsd:anyType</ErrorCode>*
 * <Description>{any}</Description>? </BaseFault>
 * 
 * @author Administrator
 * 
 */

public class BaseFault extends Exception {
	/**
	 * 
	 */
	private static final long serialVersionUID = 8131756819717925081L;

	/**
	 * Tools for build the basefault
	 */
	private MetasetHelper metHelper = MetasetHelper.getInstance();
	private MetasetBuilder metBuilder = MetasetBuilder.getInstance();

	/**
	 * BaseFaultType includes open element extensibility.
	 */
	private Metaset extension;

	/**
	 * REQUIRED element, must be time at which the fault occured. Must be only
	 * one
	 */
	private Date timestamp;

	/**
	 * OPTIONAL element, is a WS-Addressing EndpointReference of the webservice
	 * that generated the fault.
	 */
	private EndpointReference originatorReference;

	/**
	 * OPTIONAL element, provides convenient support for legacy fault reporting
	 * system
	 */
	private String errorCode;

	/**
	 * OPTIONAL element, contains a plain language description of the fault.
	 * This description is expected to be helpful in explaining the fault to
	 * users. There MAY be any number of description elements.
	 */
	private List<String> description;

	/**
	 * OPTIONAL element, if present, MUST contain a BaseFault or an element
	 * whose type extends the BaseFaultType that describes an underlying cause
	 * of this fault
	 */
	private Metaset faultCause;

	/**
	 * REQUIRED element, if not present, use BaseFault E.g ResourceUnknowFault
	 */
	private String baseFaultType;

	/**
	 * Default constructor, not recommended to use
	 */
	public BaseFault() {
		this.timestamp = new Date(System.currentTimeMillis()); 
	}

	/**
	 * A baseFaultType is needed, e.g ResourceUnknowFault
	 * 
	 * @param baseFaultType
	 */
	public BaseFault(String baseFaultType) {
		this.baseFaultType = baseFaultType;
		this.timestamp = new Date(System.currentTimeMillis()); 
	}

	/**
	 * Constructor
	 * 
	 * @param baseFaultType
	 *            A BaseFaultType, e.g ResourceUnknowFault
	 * @param description
	 *            Description about the fault, e.g Resource Unknown
	 */
	public BaseFault(String baseFaultType, List<String> description) {
		this.baseFaultType = baseFaultType;
		this.description = description;
		this.timestamp = new Date(System.currentTimeMillis()); 
	}

	/**
	 * 
	 * @param baseFaultType
	 *            A BaseFaultType, e.g ResourceUnknowFault
	 * @param description
	 *            Description about the fault, e.g Resource Unknown
	 * @param faultCause
	 *            If present, MUST contain a BaseFault or an element whose type
	 *            extends
	 */
	public BaseFault(String baseFaultType, List<String> description,
			Metaset faultCause) {
		this.baseFaultType = baseFaultType;
		this.description = description;
		this.faultCause = faultCause;
		this.timestamp = new Date(System.currentTimeMillis()); 
	}

	/**
	 * 
	 * @param baseFaultType
	 *            A BaseFaultType, e.g ResourceUnknowFault
	 * @param description
	 *            Description about the fault, e.g Resource Unknown
	 * @param originatorReference
	 *            A WS-Addressing EndpointReference of the webservice that
	 *            generated the fault.
	 */
	public BaseFault(String baseFaultType, List<String> description,
			EndpointReference originatorReference) {
		this.baseFaultType = baseFaultType;
		this.description = description;
		this.originatorReference = originatorReference;
		this.timestamp = new Date(System.currentTimeMillis()); 
	}

	/**
	 * 
	 * @param baseFaultType
	 *            A BaseFaultType, e.g ResourceUnknowFault
	 * @param description
	 *            Description about the fault, e.g Resource Unknown
	 * @param originatorReference
	 *            A WS-Addressing EndpointReference of the webservice that
	 *            generated the fault.
	 * @param faultCause
	 *            If present, MUST contain a BaseFault or an element whose type
	 *            extends
	 */
	public BaseFault(String baseFaultType, List<String> description,
			EndpointReference originatorReference, Metaset faultCause) {
		this.baseFaultType = baseFaultType;
		this.description = description;
		this.originatorReference = originatorReference;
		this.faultCause = faultCause;
		this.timestamp = new Date(System.currentTimeMillis()); 
	}

	public Metaset getExtension() {
		return extension;
	}

	public void setExtension(Metaset extension) {
		this.extension = extension;
	}

	public Date getTimestamp() {
		return timestamp;
	}

	public void setTimestamp(Date timestamp) {
		this.timestamp = timestamp;
	}

	public EndpointReference getOriginatorReference() {
		return originatorReference;
	}

	public void setOriginatorReference(EndpointReference originatorReference) {
		this.originatorReference = originatorReference;
	}

	public String getErrorCode() {
		return errorCode;
	}

	public void setErrorCode(String errorCode) {
		this.errorCode = errorCode;
	}

	public List<String> getDescription() {
		return description;
	}

	public void setDescription(List<String> description) {
		this.description = description;
	}

	public Metaset getFaultCause() {
		return faultCause;
	}

	public void setFaultCause(Metaset faultCause) {
		this.faultCause = faultCause;
	}

	public String getBaseFaultType() {
		return baseFaultType;
	}

	public void setBaseFaultType(String baseFaultType) {
		this.baseFaultType = baseFaultType;
	}

	public static long getSerialversionuid() {
		return serialVersionUID;
	}

	/**
	 * Get the metset representation of the BaseFault
	 * @return
	 */
	public Metaset toMetaset() {
		// Note the namespace & prefix, there is a little difference when you
		// use BaseFault & WSRF Faults
		QName faultRootQName = this.baseFaultType == null ? WsbfConstants.BASE_FAULT_QNAME
				: new QName(WsrfConstants.NAMESPACE_URI, this.baseFaultType,
						WsrfConstants.PREFIX);
		Metaset faultMet = metBuilder.newMetaset(faultRootQName);
		
		// Add the extension
		if(this.extension != null){
			faultMet.add(extension);
		}
		
		// Build and add timestamp
		Metaset timestampMet = metBuilder.newMetaset(WsbfConstants.TIMESTAMP_QNAME);
		timestampMet.setText(metHelper.getLocalTimeString(timestamp));
		faultMet.add(timestampMet);
		
		// Add the OriginatorReference
		if(this.originatorReference != null){
			Metaset oepr = metBuilder.createElement(WsbfConstants.ORIGINATOR_QNAME);
			oepr.add(this.originatorReference.toMetaset());
			faultMet.add(oepr);
		}
		
		// Add the errorCode
		// TODO: XServices will not use this property now
		
		// Add description
		if(this.description != null){
			for(int i = 0; i < this.description.size(); i++){
				Metaset tempDesMet = metBuilder.createElement(WsbfConstants.DESCRIPTION_QNAME);
				tempDesMet.setText(this.description.get(i));
				faultMet.add(tempDesMet);
			}
		}
		
		// Add faultCause
		if(this.faultCause != null){
			Metaset faultCauseMet = metBuilder.createElement(WsbfConstants.FAULT_CAUSE_QNAME);
			faultCauseMet.add(this.faultCause);
			faultMet.add(faultCauseMet);
		}

		return faultMet;
	}
	
	public String toString(){
		return this.toMetaset().asXML();
	}
}
