/* 
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 *
 */

package org.act.xservices.rt.xresource.faults.ext;

import java.util.List;

import org.act.xservices.rt.xcontainer.addressing.EndpointReference;
import org.act.xservices.rt.xom.Metaset;
import org.act.xservices.rt.xresource.basefault.BaseFault;

/**
 * 
 * @author douyl
 *
 */

public class InvalidMessageFormatFault extends BaseFault
{
    private static final long serialVersionUID = 4874022523783312375L;

    private final static String FAULT_TYPE = "InvalidMessageFormatFault";
    /**
	 * A baseFaultType is needed, e.g ResourceUnknowFault
	 * 
	 * @param baseFaultType
	 */
    public InvalidMessageFormatFault(){
    	super(FAULT_TYPE);
    }
    
    /**
	 * Constructor
	 * 
	 * @param baseFaultType
	 *            A BaseFaultType, e.g ResourceUnknowFault
	 * @param description
	 *            Description about the fault, e.g Resource Unknown
	 */
	public InvalidMessageFormatFault(List<String> description) {
		super(FAULT_TYPE, description);
	}

	/**
	 * 
	 * @param baseFaultType
	 *            A BaseFaultType, e.g ResourceUnknowFault
	 * @param description
	 *            Description about the fault, e.g Resource Unknown
	 * @param faultCause
	 *            If present, MUST contain a BaseFault or an element whose type
	 *            extends
	 */
	public InvalidMessageFormatFault(String baseFaultType, List<String> description,
			Metaset faultCause) {
		super(FAULT_TYPE, description, faultCause);
	}

	/**
	 * 
	 * @param baseFaultType
	 *            A BaseFaultType, e.g ResourceUnknowFault
	 * @param description
	 *            Description about the fault, e.g Resource Unknown
	 * @param originatorReference
	 *            A WS-Addressing EndpointReference of the webservice that
	 *            generated the fault.
	 */
	public InvalidMessageFormatFault(String baseFaultType, List<String> description,
			EndpointReference originatorReference) {
		super(FAULT_TYPE, description, originatorReference);
	}

	/**
	 * 
	 * @param baseFaultType
	 *            A BaseFaultType, e.g ResourceUnknowFault
	 * @param description
	 *            Description about the fault, e.g Resource Unknown
	 * @param originatorReference
	 *            A WS-Addressing EndpointReference of the webservice that
	 *            generated the fault.
	 * @param faultCause
	 *            If present, MUST contain a BaseFault or an element whose type
	 *            extends
	 */
	public InvalidMessageFormatFault(String baseFaultType, List<String> description,
			EndpointReference originatorReference, Metaset faultCause) {
		super(FAULT_TYPE, description, originatorReference, faultCause );
	}
}
