package org.act.xservices.rt.xresource.resource.impl;

/**
 * @author duanjiaqin
 */
import java.io.Serializable;
import java.util.Calendar;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Map;

import org.act.xservices.rt.xresource.resource.IResource;
import org.act.xservices.rt.xresource.resource.IResourceHome;
import org.act.xservices.rt.xresource.resource.IResourceLifetime;
import org.act.xservices.rt.xresource.resource.ResourceException;
import org.act.xservices.rt.xresource.resource.transaction.ResourceTransaction;
import org.prevayler.Prevayler;

import commonj.timers.Timer;
import commonj.timers.TimerListener;

/**
 * @author   Administrator
 */
public class ResourceCleaner implements TimerListener,Serializable {
	
    protected Map resources;
    protected IResourceHome home;

    /**
     *
     * @param resources must be synchronized map
     */
    public ResourceCleaner(IResourceHome home, Map resources) {
        this.home = home;
        this.resources = resources;
    }

    public void timerExpired(Timer timer) {
      
        Calendar currentTime = Calendar.getInstance();
        String key;
        IResource resource;
        LinkedList list = new LinkedList();

        synchronized(this.resources) {
            Iterator keyIterator = this.resources.keySet().iterator();
            while(keyIterator.hasNext()) {
                key = (String)keyIterator.next();
                try {
                    resource = ResourceTransactionManager.getInstance().doGet(home, key);
                    if (resource != null && isExpired(resource, currentTime)) {
                        list.add(key);
                    }
                } catch (ResourceException e) {
                    e.printStackTrace();
                } catch (Exception e) {
					e.printStackTrace();
				}
            }
        }

        Iterator iter = list.iterator();
        while(iter.hasNext()) {
            key = (String)iter.next();
            try {
            	ResourceTransactionManager.getInstance().doRemove(home, key);
            } catch (Exception e) {
            	
				e.printStackTrace();
			}
        }
    }

    protected IResource getResource(String key) throws Exception {
    	return ResourceTransactionManager.getInstance().doGet(home, key);
    }
    
    protected boolean isExpired(Object resource, Calendar currentTime) {
        if (!(resource instanceof IResourceLifetime)) {
            return false;
        }
        return isExpired((IResourceLifetime)resource, currentTime);
    }
    
    public static boolean isExpired(IResource resource) {
        if (!(resource instanceof IResourceLifetime)) {
            return false;
        }
        return isExpired((IResourceLifetime)resource, Calendar.getInstance());
    }
    
    public static boolean isExpired(IResourceLifetime resource,
                                    Calendar currentTime) {
        Calendar terminationTime = resource.getTerminationTime();
        return (terminationTime != null &&
                terminationTime.before(currentTime));
    }
    
}
