package org.act.xservices.rt.xresource.resource.impl;

import java.io.Serializable;
import java.util.Map;

import org.act.xservices.log.Logger;
import org.act.xservices.log.LoggerFactory;
import org.act.xservices.rt.xom.Metaset;
import org.act.xservices.rt.xresource.resource.IResource;
import org.act.xservices.rt.xresource.resource.IResourceContext;
import org.act.xservices.rt.xresource.resource.IResourceHome;
import org.act.xservices.rt.xresource.resource.ResourceException;
import org.act.xservices.rt.xresource.resource.ResourceMap;
import org.prevayler.Prevayler;
import org.prevayler.PrevaylerFactory;

import commonj.timers.Timer;
import commonj.timers.TimerManager;

/**
 * There are two mechanism in this class.<br/>
 * 1. Prevayler to take snapshot If the interval since last snapshot time less
 * than 5 seconds, then ignore the snapshot. Else take a snapshot. 2. Schedule
 * remove. Query Delete
 * 
 * @author Administrator
 */
public class ResourceHome implements IResourceHome, Serializable {
	private static final long serialVersionUID = -7624859140436447517L;

	Logger logger = LoggerFactory.getLogger(ResourceHome.class);
	
	protected Class resourceClass;
	protected Class resourceHomeClass;

	@Deprecated
	private boolean resourceIsPersistent;
	
	private boolean initialized = false;

	@Deprecated
	private Sweeper sweeper;

	// Prevayler
	private Prevayler prevayler;

	// private transient Prevayler prevayler;
	// private ResourceTransactionControl control;

	// Ĭÿ1Sweeperһ
	private long delay = 60000;
	private String endpoint;

	/**
	 * @return Returns the initialized.
	 * @uml.property name="initialized"
	 */
	public boolean isInitialized() {
		return this.initialized;
	}

	/**
	 * @param resourceClass
	 *            The resourceClass to set.
	 * @uml.property name="resourceClass"
	 */
	public void setResourceClass(Class resourceClass) {
		if ((resourceClass != null)
				&& !IResource.class.isAssignableFrom(resourceClass)) {
			throw new IllegalArgumentException("Specified resource class '"
					+ resourceClass.getName() + "' does not implement the "
					+ IResource.class.getName() + " interface.");
		}
		
		if(this.prevayler != null){
			ResourceMap tempMap = (ResourceMap)prevayler.prevalentSystem();
			tempMap.setResourceClass(resourceClass);
		}
		
		this.resourceClass = resourceClass;
	}

	public Class getResourceClass() {
		return resourceClass;
	}

	public void setResourceClassName(String resourceClassName) {
		try {
			this.resourceClass = Class.forName(resourceClassName);
		} catch (ClassNotFoundException e) {
			throw new RuntimeException(e);
		}
	}

	public String getResourceClassName() {
		return (resourceClass != null) ? resourceClass.getName() : null;
	}

	/**
	 * @param resourceHomeClass
	 *            The resourceHomeClass to set.
	 * @uml.property name="resourceHomeClass"
	 */
	public void setResourceHomeClass(Class resourceHomeClass) {
		if ((resourceHomeClass != null)
				&& !IResourceHome.class.isAssignableFrom(resourceHomeClass)) {
			throw new IllegalArgumentException(
					"Specified resource home class '"
							+ resourceHomeClass.getName()
							+ "' does not implement the "
							+ IResourceHome.class.getName() + " interface.");
		}
	}

	public void setResourceHomeClassName(String resourceHomeClassName) {
		try {
			this.resourceHomeClass = Class.forName(resourceHomeClassName);
		} catch (ClassNotFoundException e) {
			throw new RuntimeException(e);
		}
	}

	public String getResourceHomeClassName() {
		return (resourceHomeClass != null) ? resourceHomeClass.getName() : null;
	}

	/**
	 * @return Returns the delay.
	 * @uml.property name="delay"
	 */
	@Deprecated
	public long getDelay() {
		return this.delay;
	}

	/**
	 * @param delay
	 *            The delay to set.
	 * @uml.property name="delay"
	 */
	@Deprecated
	public void setDelay(long delay) {
		this.delay = delay;
	}

	public Prevayler getPrevayler() {
		return prevayler;
	}

	// Ըhomeгʼ
	public synchronized void initialize() throws Exception {
		System.out.println("Begin to initialize....");
		if (isInitialized()) {
			return;
		}
		if (resourceClass == null) {
			throw new Exception("The resourceClass has not Configured");
		}
		if (!IResource.class.isAssignableFrom(resourceClass)) {
			throw new Exception("The resourceClass configured is not supported");
		}

		// Config Prevayler
		String baseDirectory = "persistent_data/" + resourceClass.getName();
		ResourceMap resourceMap = new ResourceMap(resourceClass);
		prevayler = PrevaylerFactory.createPrevayler(resourceMap, baseDirectory);
		
		this.initialized = true;
		System.out.println("The initialize finished!");
	}


	/**
	 * SOAP EnvelopeнResource Identifier
	 */

	public String extractResourceIdentifier(IResourceContext ctx) {
		Metaset message = ctx.getRequestMessage();
		Metaset instanceIDMet = message.filter(".\\Header\\InstanceID").car();

		return instanceIDMet.getTextTrim();
	}

	/**
	 * Դʵ
	 */
	public int getResourceCount() {
		Map resources = (Map)prevayler.prevalentSystem();
		return resources.size();
	}

	/**
	 * This ResourceSweeper implementation just returns the resources currently
	 * stored in the map. The reason is that the sweeper doesn't have to
	 * reactivate/reload a persistent resource if the resource object was
	 * reclaimed. So lifetime checks are not done on reclained resources.
	 * Lifetime checks have to be done on resource load.
	 */
	@Deprecated
	private static class Sweeper extends ResourceCleaner implements Serializable {

		private TimerManager timerManager;

		private Timer timer;

		private long delay;

		public Sweeper(ResourceHome home, Map resources,
				TimerManager timerManager, long delay) {
			super(home, resources);
			this.timerManager = timerManager;
			this.delay = delay;
		}

		private synchronized void resetSweeper() {
			this.timer = null;
		}

		synchronized void scheduleSweeper() {
			System.err.println("Sweeper is running");
			if (this.timer == null) {
				this.timer = this.timerManager.schedule(this, this.delay);

			}
		}

		protected IResource getResource(Object key) throws ResourceException {
			return (IResource) this.resources.get(key);
		}

		public void timerExpired(Timer timer) {
			super.timerExpired(timer);
			resetSweeper();
			if (!this.resources.isEmpty()) {
				scheduleSweeper();
			}
		}
	}

}
