package org.act.xservices.rt.xresource.resource.impl.properties;

import java.rmi.RemoteException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.act.xservices.log.Logger;
import org.act.xservices.log.LoggerFactory;
import org.act.xservices.rt.metaset.MetasetBuilder;
import org.act.xservices.rt.metaset.MetasetParserFactory;
import org.act.xservices.rt.soap.ISOAPBody;
import org.act.xservices.rt.soap.ISOAPEnvelope;
import org.act.xservices.rt.soap.ISOAPHeader;
import org.act.xservices.rt.soap.ISOAPHeaderBlock;
import org.act.xservices.rt.soap.factory.SOAPFactory;
import org.act.xservices.rt.xcontainer.addressing.uitl.SoapConstants;
import org.act.xservices.rt.xom.Metaset;
import org.act.xservices.rt.xom.Namespace;
import org.act.xservices.rt.xom.Node;
import org.act.xservices.rt.xom.QName;
import org.act.xservices.rt.xom.xpath.DefaultXPath;
import org.act.xservices.rt.xresource.basefault.BaseFault;
import org.act.xservices.rt.xresource.faults.InvalidResourcePropertyQNameFault;
import org.act.xservices.rt.xresource.faults.ResourceUnknownFault;
import org.act.xservices.rt.xresource.faults.ext.InvalidMessageFormatFault;
import org.act.xservices.rt.xresource.resource.IResource;
import org.act.xservices.rt.xresource.resource.IResourceProperties;
import org.act.xservices.rt.xresource.resource.NoSuchResourceException;
import org.act.xservices.rt.xresource.resource.ResourceException;
import org.act.xservices.rt.xresource.resource.impl.ResourceContext;
import org.act.xservices.rt.xresource.util.ResourceConstants;
import org.act.xservices.rt.xresource.util.ResourceUtil;
import org.act.xservices.rt.xresource.util.WsrpConstants;

/**
 * Get ResourceProperty from Resource
 * 
 * For the request soap message
 * SOAPHeader part:
 * wsa:Action & wsa:To is needed
 * 
 * SOAPBody part:
 * <wsrf-rp:GetResourceProperty>
 * QName
 * </wsrf-rp:GetResourceProperty>
 * 
 * For the response soap message
 * SOAPBody part:
 * <wsrf-rp:GetResourcePropertyResponse>
 * {any}*
 * </wsrf-rp:GetResourcePropertyResponse>
 * 
 * @author Administrator
 *
 */
public class GetResourcePropertyPortType extends PropertyPortTypeOperation {
	final static Logger logger = LoggerFactory.getLogger(GetResourcePropertyPortType.class);

	private String prefix;
	private String propertyName;
	
	@Override
	protected void validateSOAPMsg(ISOAPEnvelope soapEnv) throws BaseFault {
		
		// Get the propertyName requested by client
		Metaset propertyMet = soapEnv.getBody().element(WsrpConstants.GET_QNAME);
		if(propertyMet == null){
			throw new InvalidMessageFormatFault();
		}
		
		String text = propertyMet.getText();
		if(text == null || text.trim().length() == 0){
			throw new InvalidMessageFormatFault();
		}
		
		int index = text.indexOf(':');
		if(index <= 0){
			throw new InvalidMessageFormatFault();
		}
		
		// Check prefix
		prefix = text.substring(0, index);
		if( propertyMet.getNamespaceForPrefix(prefix) == null ){
			throw new InvalidResourcePropertyQNameFault();
		}

		propertyName = text.substring(index+1, text.length());
		if(propertyName == null || propertyName.trim().length() == 0){
			throw new InvalidResourcePropertyQNameFault();
		}
	}

	@Override
	protected Metaset generateResponseSOAPBody(ISOAPEnvelope soapEnv)
			throws BaseFault, RemoteException {
		// Get Resource
		IResource resource = null;
		try{
			ResourceContext ctx = new ResourceContext(soapEnv);
			resource = ctx.getResource();
			
		}catch(NoSuchResourceException e){
			throw new ResourceUnknownFault();
			
		}catch(ResourceException e){
			throw new ResourceUnknownFault();
			
		}catch(Exception e){
			throw new RemoteException("resourceDisoveryFailed",e);
		}
		
		IResourceProperties irp = resource.getResourceProperties();
		String propertyValue = irp.getProperty(propertyName);

		// If propertyValue is null, then fault it
		if(propertyValue == null){
			throw new InvalidResourcePropertyQNameFault();
		}
		
		// Build soapBody
		MetasetBuilder metBuilder = MetasetBuilder.getInstance();
		Metaset responseMetaset = metBuilder.createElement(WsrpConstants.GET_RESPONSE_QNAME);

		Metaset actionMS = soapEnv.getBody().element(WsrpConstants.GET_QNAME);
		Namespace propertyNS = actionMS.getNamespaceForPrefix(prefix);
		Metaset metProperty = metBuilder.createElement(new QName(propertyNS.getURI(), propertyName, propertyNS.getPrefix()));
		metProperty.setText(propertyValue);
		responseMetaset.add(metProperty);
		
		return responseMetaset;
	}
	
	public static void main(String[] args) throws Exception{
		//String xmlsoap = "<soapenv:Envelope xmlns:soapenv=\"http://schemas.xmlsoap.org/soap/envelope/\" xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"><soapenv:Header><wsa:Action xmlns:wsa=\"http://schemas.xmlsoap.org/ws/2004/03/addressing\" soapenv:actor=\"http://schemas.xmlsoap.org/soap/actor/next\" soapenv:mustUnderstand=\"0\">http://docs.oasis-open.org/wsrf/rpw-2/GetResourceProperty/GetResourcePropertyResponse</wsa:Action><wsa:To xmlns:wsa=\"http://schemas.xmlsoap.org/ws/2004/03/addressing\" soapenv:actor=\"http://schemas.xmlsoap.org/soap/actor/next\" soapenv:mustUnderstand=\"0\">http://192.168.2.100:8091/wsrf/services/JvmResourceService</wsa:To><ex:MessageId xmlns:ex=\"http://ws.act.org/resource/example/JvmResourceService\" soapenv:actor=\"http://schemas.xmlsoap.org/soap/actor/next\" soapenv:mustUnderstand=\"0\">9288</ex:MessageId></soapenv:Header><soapenv:Body><wsrf-rp:GetResourceProperty xmlns:wsrf-rp=\"http://docs.oasis-open.org/wsrf/rp-2\" xmlns:tns=\"http://ws.act.org/resource/example/JvmResourceService\">tns:NumberOfBlocks</wsrf-rp:GetResourceProperty></soapenv:Body></soapenv:Envelope>";

		//		String rootPath = System.getProperty("user.dir");
//		String filePath = rootPath + "/target/classes/data.xml";
//		Metaset met = MetasetParserFactory.getMetasetParser().parse(filePath);
		
		String xmlsoap = "<tns:JvmResourceProperties xmlns:tns=\"http://ws.act.org/resource/example/ResourceProperties\"><tns:NumberOfBlocks>22</tns:NumberOfBlocks><tns:BlockSize>1024</tns:BlockSize><tns:Manufacturer>DrivesRUs</tns:Manufacturer></tns:JvmResourceProperties>";
		Metaset met = MetasetParserFactory.getMetasetParser().parseXMLString(xmlsoap);
		
		System.out.println(met.asXML());
		
		
		
		//DefaultXPath xpath=new DefaultXPath(".//wsrf-rp:GetResourceProperty");
		DefaultXPath xpath=new DefaultXPath(".//tns:NumberOfBlocks");
		Map<String, String> map = new HashMap<String, String>();
		
		// List<Namespace> nsList = met.element("Body").element("GetDocument").declaredNamespaces();
		List<Namespace> nsList = met.declaredNamespaces();
		System.out.println(nsList.size());
		for(int i=0; i<nsList.size(); i++){
			Namespace tempNS = nsList.get(i);
			map.put(tempNS.getPrefix(), tempNS.getURI());
		}
		
		xpath.setNamespaceURIs(map);
		
		Node node = xpath.selectSingleNode(met);
		System.out.println("node: " + node.asXML());
//		
//		ISOAPEnvelope soapEnv = SOAPMessageUtil.convertMetasetToSOAPEnvelope(met);
//		Metaset mets = soapEnv.getBody().element("GetResourceProperty");
//		System.out.println("mets: " + mets.asXML());
	}
}
