package org.act.xservices.rt.xresource.resource.impl.properties;

import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Iterator;
import java.util.List;

import org.act.xservices.log.Logger;
import org.act.xservices.log.LoggerFactory;
import org.act.xservices.rt.metaset.MetasetBuilder;
import org.act.xservices.rt.metaset.utils.MetasetUtil;
import org.act.xservices.rt.soap.ISOAPBody;
import org.act.xservices.rt.soap.ISOAPEnvelope;
import org.act.xservices.rt.soap.ISOAPHeader;
import org.act.xservices.rt.soap.ISOAPHeaderBlock;
import org.act.xservices.rt.xom.Metaset;
import org.act.xservices.rt.xom.QName;
import org.act.xservices.rt.xresource.basefault.BaseFault;
import org.act.xservices.rt.xresource.faults.InvalidModificationFault;
import org.act.xservices.rt.xresource.faults.InvalidResourcePropertyQNameFault;
import org.act.xservices.rt.xresource.faults.ResourceUnknownFault;
import org.act.xservices.rt.xresource.faults.UnableToModifyResourcePropertyFault;
import org.act.xservices.rt.xresource.faults.ext.InvalidMessageFormatFault;
import org.act.xservices.rt.xresource.resource.IPersistentResource;
import org.act.xservices.rt.xresource.resource.IResource;
import org.act.xservices.rt.xresource.resource.IResourceProperties;
import org.act.xservices.rt.xresource.resource.IResourceWithCallback;
import org.act.xservices.rt.xresource.resource.NoSuchResourceException;
import org.act.xservices.rt.xresource.resource.ResourceException;
import org.act.xservices.rt.xresource.resource.impl.ResourceContext;
import org.act.xservices.rt.xresource.util.ResourceConstants;
import org.act.xservices.rt.xresource.util.SOAPUtil;
import org.act.xservices.rt.xresource.util.WsrpConstants;
import org.act.xservices.rt.xresource.util.XMLPersistenceUtil;

/**
 * @author duanjiaqin
 * 
 *         һԴ Ϣıĸʽ£ <s11:Envelope
 *         xmlns:s11="http://schemas.xmlsoap.org/soap/envelope/"
 *         xmlns:wsa="http://www.w3.org/2005/08/addressing"
 *         xmlns:wsrf-rp="http://docs.oasis-open.org/wsrf/rp-2"
 *         xmlns:ex="http://examples.com/exampleNS"> <s11:Header>
 *         <wsa:Action>http
 *         ://docs.oasis-open.org/wsrf/rpw-2/InsertResourceProperties
 *         /InsertResourcePropertiesRequest</wsa:Action> <wsa:To
 *         s11:mustUnderstand
 *         ="1">http://www.provider.org/ProviderEndpoint</wsa:To>
 *         <ex:ResourceDiambiguator
 *         >uuid:84decd55-7d3f-65ad-ac44-675d9fce5d22</ex:ResourceDiambiguator>
 *         </s11:Header> <s11:Body> <wsrf-rp:InsertResourceProperties
 *         xmlns:tns="http://example.com/diskDrive"> <wsrf-rp:Insert>
 *         <tns:StorageCapability>
 *         <tns:NoSinglePointOfFailure>true</tns:NoSinglePointOfFailure>
 *         </tns:StorageCapability> <tns:StorageCapability>
 *         <tns:DataRedundancyMax>42</tns:DataRedundancyMax>
 *         </tns:StorageCapability> </wsrf-rp:Insert>
 *         </wsrf-rp:InsertResourceProperties> </s11:Body> </s11:Envelope>
 *         صӦĸʽ£ <s11:Envelope
 *         xmlns:s11="http://schemas.xmlsoap.org/soap/envelope/"
 *         xmlns:wsa="http://www.w3.org/2005/08/addressing"
 *         xmlns:wsrf-rp="http://docs.oasis-open.org/wsrf/rp-2"> <s11:Header>
 *         <wsa:Action>
 *         http://docs.oasis-open.org/wsrf/rpw-2/InsertResourceProperties
 *         /InsertResourcePropertiesResponse </wsa:Action> <s11:Body>
 *         <wsrf-rp:InsertResourcePropertiesResponse>
 *         </wsrf-rp:InsertResourcePropertiesResponse> </s11:Body>
 *         </s11:Envelope>
 */

public class InsertResourcePropertiesPortType extends PropertyPortTypeOperation {
	final static Logger logger = LoggerFactory
			.getLogger(InsertResourcePropertiesPortType.class);

	@Override
	protected void validateSOAPMsg(ISOAPEnvelope soapEnv) throws BaseFault {
		// Get the propertyName requested by client
		Metaset propertyMet = soapEnv.getBody().element(
				WsrpConstants.INSERT_RESOURCE_PROPERTIES_ACTION);
		if (propertyMet == null) {
			throw new InvalidMessageFormatFault();
		}

		Metaset insertMet = propertyMet.element("Insert");
		if (insertMet == null) {
			throw new InvalidMessageFormatFault();
		}

		List<Metaset> elemList = insertMet.elements();
		if (elemList == null || elemList.size() == 0) {
			throw new InvalidMessageFormatFault();
		}

		QName tnsQName = elemList.get(0).getQName();
		for (int i = 1; i < elemList.size(); i++) {
			if (!elemList.get(i).getQName().equals(tnsQName)) {
				List<String> faultList = new ArrayList<String>();
				String faultStr = "If there are multiple child elements of the Insert component,";
				faultStr += " each MUST have the same namespace and name.";
				faultList.add(faultStr);
				throw new InvalidResourcePropertyQNameFault(faultList);
			}
		}
	}

	@Override
	protected Metaset generateResponseSOAPBody(ISOAPEnvelope soapEnv)
			throws BaseFault, RemoteException {
		// Get Resource
		IResource resource = null;
		try {
			ResourceContext ctx = new ResourceContext(soapEnv);
			resource = ctx.getResource();

		} catch (NoSuchResourceException e) {
			throw new ResourceUnknownFault();

		} catch (ResourceException e) {
			throw new ResourceUnknownFault();

		} catch (Exception e) {
			throw new RemoteException("resourceDisoveryFailed", e);
		}

		IResourceProperties irp = resource.getResourceProperties();
		Metaset rpDoc = irp.getdocument();

		// Start the set processing
		List<Metaset> insertMetList = soapEnv.getBody().getFirstElement().element(WsrpConstants.INSERT_QNAME).elements();
		Iterator<Metaset> iter = insertMetList.iterator();
		while (iter.hasNext()) {
			Metaset insertItem = iter.next();
			rpDoc.add(insertItem.createCopy());
		}

		// If you can go here, you get a sucessful operation
		// Then what you do is to return a response message
		// TODO: Thread safe
		return MetasetBuilder.getInstance().newMetaset(
				WsrpConstants.INSERT_RESPONSE_QNAME);

	}
	
	
	// public Metaset insertResourceProperties(Metaset request)throws
	// RemoteException,
	// ResourceUnknownFault,
	// InvalidModificationFault,
	// UnableToModifyResourcePropertyFault,
	// InsertResourcePropertiesRequestFailedFault{
	// IResource resource = null;
	// ResourceContext ctx = null;
	//		
	// try{
	// ctx = new ResourceContext(request);
	// resource = ctx.getResource();
	// }catch(NoSuchResourceException e){
	// Calendar timestamp = Calendar.getInstance();
	// ResourceUnknownFault fault = new ResourceUnknownFault(timestamp,
	// ResourceConstants.RESOURCE_UNKNOWN_FAULT);
	// throw fault;
	// }catch(ResourceException e){
	// Calendar timestamp = Calendar.getInstance();
	// ResourceUnknownFault fault = new ResourceUnknownFault(timestamp,
	// ResourceConstants.RESOURCE_UNKNOWN_FAULT);
	// throw fault;
	// }catch(Exception e){
	// Calendar timestamp = Calendar.getInstance();
	// InsertResourcePropertiesRequestFailedFault fault = new
	// InsertResourcePropertiesRequestFailedFault(timestamp,
	// ResourceConstants.INSERT_RESOURCE_PROPERTIES_REQUEST_FAILED_FAULT);
	// throw fault;
	// }
	//		
	// if(resource == null){
	// Calendar timestamp = Calendar.getInstance();
	// ResourceUnknownFault fault = new ResourceUnknownFault(timestamp,
	// ResourceConstants.RESOURCE_UNKNOWN_FAULT);
	// throw fault;
	// }
	//		
	// //췵صsoap
	// String action =
	// "http://docs.oasis-open.org/wsrf/rpw-2/InsertResourceProperties/InsertResourcePropertiesResponse";
	//
	// IResourceProperties pro = resource.getResourceProperties();
	// Metaset rpDocument = pro.getdocument();
	//		
	// //add by cover_eye:07-12-7
	// Metaset old=null;
	// old = (Metaset) rpDocument.clone();
	//
	// //end by cover_eye:07-12-7
	// String actualnamespace = rpDocument.getNamespaceURI();
	//		
	// Metaset insertProperties = getInsertProperties(request);
	//		
	// //ԪΪ
	// if(insertProperties == null){
	// Calendar timestamp = Calendar.getInstance();
	// InsertResourcePropertiesRequestFailedFault fault = new
	// InsertResourcePropertiesRequestFailedFault(timestamp,
	// ResourceConstants.INSERT_RESOURCE_PROPERTIES_REQUEST_FAILED_FAULT);
	// throw fault;
	// }
	//		
	// //жϲÿǲǺϷ
	// List childs = insertProperties.elements();
	// Iterator iter = childs.iterator();
	// while(iter.hasNext()){
	// Metaset child = (Metaset)iter.next();
	// String namespace =
	// MetasetUtil.findNsURIofPrefix(child,child.getNamespacePrefix());
	// if(!namespace.equals(actualnamespace)){
	// Calendar timestamp = Calendar.getInstance();
	// InvalidResourcePropertyQNameFault fault = new
	// InvalidResourcePropertyQNameFault(timestamp,
	// ResourceConstants.INVALID_RESOURCE_PROPERTY_QNAME_FAULT);
	// throw fault;
	// }
	// }
	//		
	// //û׳쳣,еԼԴĵ
	// Iterator iterator = childs.iterator();
	// while(iterator.hasNext()){
	// Metaset element = (Metaset)iterator.next();
	// //ֿռ,ǰ׺ͬ,򽫸ôԪصǰ׺ΪԴĵǰ׺
	// if(element.getName().equals(rpDocument.getName()))
	// {
	// // TODO: need to resolved
	// // element.setTargetNamespace(rpDocument.getName());
	// }
	// rpDocument.add(element.createCopy());
	// }
	//		
	// //add by cover_eye:07-12-7
	// if(resource instanceof IResourceWithCallback &&
	// ((IResourceWithCallback)resource).getResourceCallBack()!=null){
	// ((IResourceWithCallback)resource).getResourceCallBack().insertResourceProperties(old,rpDocument);
	// //System.err.println("wo shi xiaoxiao !");
	// //System.err.println(old.toXML());
	// //System.err.println(rpDocument.toXML());
	// }
	// //end by cover_eye:07-12-7
	//		
	//		
	//		
	//		
	// //Դǳ־ûģִtrancation
	// if(IPersistentResource.class.isAssignableFrom(resource.getClass())){
	// // ResourceHome home = (ResourceHome)ctx.getResourceHome();
	// // ResourceProperties rpDoc = new
	// ResourceProperties(rpDocument.getNamespace(),rpDocument.getName());
	// // rpDoc = SOAPUtil.addChilds(rpDoc,rpDocument);
	// // Object key = home.extractResourceIdentifier(ctx);
	// // home.changePersistentResourceState(key,rpDoc);
	// Object key = resource.getID();
	// try{
	// XMLPersistenceUtil helper = new XMLPersistenceUtil(resource.getClass());
	// helper.store(key,rpDocument);
	// }catch(Exception e){
	// e.printStackTrace();
	// }
	// }
	// MetasetBuilder metBuilder = MetasetBuilder.getInstance();
	// Metaset response = metBuilder.newMetaset(ResourceConstants.NS_WSRF_RP
	// ,"InsertResourcePropertiesResponse", "wsrf-rp");
	//	
	// Metaset insertResourcePropertyDocumentResponse =
	// SOAPUtil.generateSOAP(action,response);
	//		
	// // if(resource.getResourceCallBack()!=null){
	// // resource.getResourceCallBack().insertResourceProperties();
	// // }
	//		
	// return insertResourcePropertyDocumentResponse;
	// }
	//	
	// //شԴ
	// private static Metaset getInsertProperties(Metaset message){
	// Metaset insert =
	// message.filter(".\\Body\\InsertResourceProperties\\Insert").car();
	// return insert;
	// }

}
