package org.act.xservices.rt.xresource.resource.impl.properties;

import java.rmi.RemoteException;
import java.util.List;

import org.act.xservices.log.Logger;
import org.act.xservices.log.LoggerFactory;
import org.act.xservices.rt.soap.ISOAPBody;
import org.act.xservices.rt.soap.ISOAPEnvelope;
import org.act.xservices.rt.soap.ISOAPHeader;
import org.act.xservices.rt.soap.ISOAPHeaderBlock;
import org.act.xservices.rt.soap.factory.SOAPFactory;
import org.act.xservices.rt.soap.util.SOAPMessageUtil;
import org.act.xservices.rt.xcontainer.addressing.uitl.WsaConstants;
import org.act.xservices.rt.xom.Metaset;
import org.act.xservices.rt.xom.QName;
import org.act.xservices.rt.xresource.basefault.BaseFault;
import org.act.xservices.rt.xresource.util.ResourceConstants;
import org.act.xservices.rt.xresource.util.ResourceUtil;
import org.act.xservices.rt.xresource.util.WsrpConstants;

/**
 * Defines the operation flow in resource process
 * This class exposes only processRequestSOAPMsg method to outside, and the same time, 
 * exposes three other methods in its subclass which used to process the flows
 * 
 * @author Administrator
 * @version 1.0
 *
 */
public abstract class PropertyPortTypeOperation {
	final static Logger logger = LoggerFactory.getLogger(PropertyPortTypeOperation.class);
	/**
	 * The only interface exposed to outside, process the operation
	 * @param soapEnv
	 * @return
	 * @throws BaseFault
	 */
	public ISOAPEnvelope processRequestSOAPMsg(ISOAPEnvelope soapEnv) throws BaseFault, RemoteException{
		
		validateSOAPMsg(soapEnv);
		List<ISOAPHeaderBlock> inMessageHeaders = generateResponseSOAPHeader(soapEnv);
		Metaset responseMetaset = generateResponseSOAPBody(soapEnv);
		
		return SOAPMessageUtil.creatResponseMessage(soapEnv.getVersion(), responseMetaset, inMessageHeaders);
	}
	
	/**
	 * Validate the soapMsg, if format is invalid, an Fault thrown
	 * @param soapEnv
	 * @throws BaseFault
	 */
	protected abstract void validateSOAPMsg(final ISOAPEnvelope soapEnv) throws BaseFault;
	
	/**
	 * Generates response soapHeader while received a soapEnvelope
	 * @param soapEnv
	 * @return
	 * @throws BaseFault
	 */
	protected  List<ISOAPHeaderBlock> generateResponseSOAPHeader(final ISOAPEnvelope soapEnv) throws BaseFault{
		return ResourceUtil.getInstance().generateResponseSOAPHeader(soapEnv.getHeader());
	}
	
	
	/**
	 * Generates response soapBody while received a soapEnvelope
	 * @param soapEnv
	 * @return
	 * @throws BaseFault
	 */
	protected abstract Metaset generateResponseSOAPBody(final ISOAPEnvelope soapEnv) throws BaseFault, RemoteException;
}
