/*
 * Created on 2006-4-10
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package org.act.xservices.rt.xresource.resource.impl.properties;

import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.act.xservices.rt.metaset.MetasetBuilder;
import org.act.xservices.rt.soap.ISOAPBody;
import org.act.xservices.rt.soap.ISOAPEnvelope;
import org.act.xservices.rt.soap.ISOAPHeader;
import org.act.xservices.rt.soap.ISOAPHeaderBlock;
import org.act.xservices.rt.soap.factory.SOAPFactory;
import org.act.xservices.rt.xcontainer.addressing.uitl.SoapConstants;
import org.act.xservices.rt.xom.Metaset;
import org.act.xservices.rt.xom.Namespace;
import org.act.xservices.rt.xom.Node;
import org.act.xservices.rt.xom.QName;
import org.act.xservices.rt.xom.xpath.DefaultXPath;
import org.act.xservices.rt.xresource.basefault.BaseFault;
import org.act.xservices.rt.xresource.faults.QueryEvaluationErrorFault;
import org.act.xservices.rt.xresource.faults.ResourceUnknownFault;
import org.act.xservices.rt.xresource.faults.UnknownQueryExpressionDialectFault;
import org.act.xservices.rt.xresource.faults.ext.InvalidMessageFormatFault;
import org.act.xservices.rt.xresource.resource.IResource;
import org.act.xservices.rt.xresource.resource.IResourceProperties;
import org.act.xservices.rt.xresource.resource.NoSuchResourceException;
import org.act.xservices.rt.xresource.resource.ResourceException;
import org.act.xservices.rt.xresource.resource.impl.ResourceContext;
import org.act.xservices.rt.xresource.util.ResourceConstants;
import org.act.xservices.rt.xresource.util.ResourceUtil;
import org.act.xservices.rt.xresource.util.WsrpConstants;

/**
 * Query ResourceProperties
 * If a WS-Resource supports the QueryResourceProperty operation and supports
 * dialects in addition to the required XPath 1.0 dialect, then it MUST include 
 * a wsrf-rp:QueryExpressionDialect resource property if it supports XPath 1.0 dialect.
 * 
 * Currently, we only support XPath, and will support other dialects in the future.
 * To support XPath, the following dialect is required:
 * http://www.w3.org/TR/1999/REC-xpath-19991116
 * 
 * The request soap messge should be:
 * <wsrf-rp:QueryResourceProperties>
 * 	<wsrf-rp:QueryExpression Dialect="http://....">
 *  </wsrf-rp:QueryExpression>
 * <wsrf-rp:QueryResourceProperties>
 * 
 * Note: The namspaces in your expression MUST declared in the wsrf-rp:QueryExpression
 * 
 * @author Administrator
 */
public class QueryResourcePropertiesPortType extends PropertyPortTypeOperation {
	// xpath dialect
	private final String DIALECT_XPATH = "http://www.w3.org/TR/1999/REC-xpath-19991116";
	private String queryExpressionStr;
	
	@Override
	protected void validateSOAPMsg(ISOAPEnvelope soapEnv) throws BaseFault {
		// Check if body content is valid
		Metaset bodyMet = soapEnv.getBody().element(WsrpConstants.QUERY_QNAME);
		if(bodyMet == null){
			throw new InvalidMessageFormatFault();
		}
		
		Metaset qeMet = bodyMet.element("QueryExpression");
		if(qeMet == null){
			throw new InvalidMessageFormatFault();
		}
		
		String dialect = qeMet.attributeValue("Dialect");
		if(dialect == null || dialect.length() == 0 || !dialect.equals(DIALECT_XPATH)){
			throw new UnknownQueryExpressionDialectFault();
		}
		
		queryExpressionStr = qeMet.getTextTrim();
		if(queryExpressionStr == null || queryExpressionStr.length() == 0){
			throw new InvalidMessageFormatFault();
		}
	}
	
	@Override
	protected Metaset generateResponseSOAPBody(ISOAPEnvelope soapEnv)
			throws BaseFault, RemoteException {
		// Get Resource
		IResource resource = null;
		try{
			ResourceContext ctx = new ResourceContext(soapEnv);
			resource = ctx.getResource();
			
		}catch(NoSuchResourceException e){
			throw new ResourceUnknownFault();
			
		}catch(ResourceException e){
			throw new ResourceUnknownFault();
			
		}catch(Exception e){
			throw new RemoteException("resourceDisoveryFailed",e);
		}
		
		// Get resource document
		IResourceProperties irp = resource.getResourceProperties();
		DefaultXPath xpath=new DefaultXPath(queryExpressionStr);
		Map<String, String> map = new HashMap<String, String>();
		
		logger.debug("queryExpressionStr: " + queryExpressionStr);
		// Add the namespace from request soap message into map
		List<Namespace> nsList = irp.getdocument().declaredNamespaces();
		nsList = nsList == null? new ArrayList<Namespace>() : nsList;
		nsList.add(irp.getdocument().getNamespace());
		for( int i = 0; i < nsList.size(); i++ ){
			Namespace tempNS = nsList.get(i);
			map.put(tempNS.getPrefix(), tempNS.getURI());
		}
		
		xpath.setNamespaceURIs(map);
		Node resultNode = null;
		try{
			resultNode = xpath.selectSingleNode(irp.getdocument());
		}catch(Exception ex){
			throw new QueryEvaluationErrorFault();
		}
		
		logger.debug("resultNode: " + resultNode);
		// Build the response soapBody
		// Build soapBody
		MetasetBuilder metBuilder = MetasetBuilder.getInstance();
		Metaset responseMetaset = metBuilder.createElement(WsrpConstants.QUERY_RESPONSE_QNAME);
		
		responseMetaset.add(((Metaset)resultNode).createCopy());
		return responseMetaset;
	}
}
