/*
 * Created on 2006-5-10
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package org.act.xservices.rt.xresource.util;

import java.io.File;
import java.io.FilenameFilter;
import java.io.IOException;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

/**
 * @author   Administrator
 */
public class FilePersistenceUtil implements Serializable{
	private static final String FILE_PERSISTENCE_DIR =
        System.getProperty("user.dir") + File.separatorChar + "resource" + File.separatorChar + 
        ".act" + File.separatorChar + "persisted";

    protected Class beanClass;
    protected File storageDir;
    protected String fileSuffix;

    public static String getDefaultStorageDir(Class beanClass) 
        throws IOException {
        if (beanClass == null) {
            return null;
        }
        String fullClassName = beanClass.getName();
        String className = fullClassName.substring(
                      beanClass.getPackage().getName().length() + 1);       
               
        String dir = FILE_PERSISTENCE_DIR + File.separatorChar + className;

        File baseDir = new File(FILE_PERSISTENCE_DIR);
        
        File storageDir = new File(dir);
        if (!storageDir.getCanonicalPath().startsWith(baseDir.getCanonicalPath())) {
          //  throw new IOException(i18n.getMessage("invalidStorageDir", dir));
        	throw new IOException("invalidStorageDir");
        }
        System.out.println(dir);
        
        return dir;
    }

    /**
     * Creates FilePersistenceHelper with default storage directory based on
     * the beanClass name and specified suffix.
     */
    public FilePersistenceUtil(Class beanClass,
                                 String suffix)
        throws IOException {
        this(beanClass, getDefaultStorageDir(beanClass), suffix);
    }

    /**
     * Creates FilePersistenceHelper with specific storage directory and
     * file suffix.
     */
    public FilePersistenceUtil(Class beanClass,
                                 String storageDir,
                                 String suffix)
        throws IOException {
        if (beanClass == null) {
        	 throw new IllegalArgumentException(
                    ("nullArgumentbeanClass"));
        }
        if (storageDir == null) {
        	throw new IllegalArgumentException(
                    ("nullArgument storageDir"));
        }
        if (suffix == null) {
            throw new IllegalArgumentException(
                    ("nullArgument suffix"));
        }
        this.beanClass = beanClass;
        this.fileSuffix = suffix;
        setStorageDirectory(storageDir);
    }

    protected void setStorageDirectory(String strDir)
        throws IOException {
        File dir = new File(strDir);
        synchronized (FilePersistenceUtil.class) {
            if (!dir.exists()) {
                if (!dir.mkdirs()) {
                	throw new IOException("storDirFailed dir");
                }
            } else {
                if (!dir.canWrite() || !dir.canRead()) {
                	throw new IOException("storDirPerm " +
                         " dir");
                }
            }
        }
        this.storageDir = dir;
    }

    public File getStorageDirectory() {
        return this.storageDir;
    }

    /**
	 * @return   Returns the fileSuffix.
	 * @uml.property   name="fileSuffix"
	 */
    public String getFileSuffix() {
		return this.fileSuffix;
	}

    /**
	 * @return   Returns the beanClass.
	 * @uml.property   name="beanClass"
	 */
    public Class getBeanClass() {
		return this.beanClass;
	}

    /**
     * Create a file object based on the key supplied in parameter.
     * The file name will follow the format:<p>
     * file name :== (class name)_(key scalar value).xml<p>
     * where:
     * <ul>
     * <li>(class name) is class name of the implementation resource bean.</li>
     * <li>(key scalar value) is the toString value of the key passed in
     *     parameter. If the key is an instance of ResourceKey, its value is
     *     the result of key.getValue().</li>
     *
     * @param key the key of the object
     * @return File
     */
    public File getKeyAsFile(Object key) {

        Object keyValue = key;

        File file = new File(getStorageDirectory(),
                             keyValue + this.fileSuffix);
        return file;
    }

    public void remove(Object key)
        throws Exception {
        File f = getKeyAsFile(key);
        f.delete();
    }

    /**
     * This function returns the keys of the objects that have been stored.
     * This should be used by the home in order to recover state,
     * by listing all the stored object keys and adding them to the
     * map of resources. The consumer code must test for null.
     *
     * @return List the list of key values
     */
    public List list()
        throws IOException {

        //read list of persistence resource files
        File persistenceDirectory = getStorageDirectory();
        File [] keyFiles =
            persistenceDirectory.listFiles(new Filter(this.fileSuffix));
        List keyValueList = null;
        if (keyFiles != null) {
            keyValueList = new ArrayList(keyFiles.length);
            for (int i = 0; i < keyFiles.length; i++) {
                String fileName = keyFiles[i].getName();
                int keyEnd = fileName.lastIndexOf(this.fileSuffix);
                String keyValue = fileName.substring(0, keyEnd);
                keyValueList.add(keyValue);
            }
        } else {
           // logger.debug("Persistence directory does not exist yet");
            keyValueList = new ArrayList(0);
        }

        return keyValueList;
    }

    /**
     * Removes all stored objects from file system.
     */
    public void removeAll()
        throws IOException {
        File persistenceDirectory = getStorageDirectory();
        File [] keyFiles =
            persistenceDirectory.listFiles(new Filter(this.fileSuffix));
        for (int i = 0; i < keyFiles.length; i++) {
            keyFiles[i].delete();
        }
    }

    private static class Filter implements FilenameFilter {

        private String suffix;

        public Filter(String suffix) {
            this.suffix = suffix;
        }

        public boolean accept(File dir, String name) {
            return (name.endsWith(this.suffix));
        }
    }

}
