package org.act.xservices.rt.xresource.util;

import java.util.ArrayList;
import java.util.List;

import org.act.xservices.rt.soap.ISOAPHeader;
import org.act.xservices.rt.soap.ISOAPHeaderBlock;
import org.act.xservices.rt.soap.factory.SOAPFactory;
import org.act.xservices.rt.soap.impl.SOAPHeaderBlockImpl;
import org.act.xservices.rt.xcontainer.addressing.uitl.SoapConstants;
import org.act.xservices.rt.xom.Metaset;
import org.act.xservices.rt.xom.QName;
import org.act.xservices.rt.xresource.addressing.ResourceMessageHeaders;
import org.act.xservices.rt.xresource.basefault.BaseFault;
import org.act.xservices.rt.xresource.faults.InvalidResourcePropertyQNameFault;
import org.act.xservices.rt.xresource.faults.ext.InvalidMessageFormatFault;

public class ResourceUtil {
	private static ResourceUtil instance = null;
	
	/**
	 * Private access, used for Singleton Pattern
	 */
	private ResourceUtil(){
		
	}
	
	/**
	 * Signleton pattern
	 * @return
	 * 		A instance of ResourceUtil
	 */
	public static ResourceUtil getInstance(){
		if(instance == null){
			synchronized(ResourceUtil.class){
				if(instance == null){
					instance = new ResourceUtil();
				}
			}
		}
		return instance;
	}
	
	/**
	 * For some circumstance, xom will not check tns:NumberBlock text.
	 * So this method add the check mechanism
	 * @param propertyMet
	 * @return NumberBlock
	 * @throws InvalidRequestFormatFault 
	 * @throws InvalidResourcePropertyQNameFault 
	 */
	public String getPropertyValue(Metaset propertyMet) throws InvalidMessageFormatFault, InvalidResourcePropertyQNameFault{
		String text = propertyMet.getText();
		if(text == null || text.trim().length() == 0){
			throw new InvalidMessageFormatFault();
		}
		
		int index = text.indexOf(':');
		if(index <= 0){
			throw new InvalidMessageFormatFault();
		}
		
		// Check prefix
		String prefix = text.substring(0, index);
		if( propertyMet.getNamespaceForPrefix(prefix) == null ){
			throw new InvalidResourcePropertyQNameFault();
		}

		String propertyName = text.substring(index+1, text.length());
		if(propertyName == null || propertyName.trim().length() == 0){
			throw new InvalidResourcePropertyQNameFault();
		}
		
		return propertyName;
	}
	
	
	/**
	 * Generate a soapHeader according to the soapHeader received from client
	 * Some headBlocks in request header will be converted into response soapHeader
	 * An action is needed, which will be added into response soapHeader
	 * 
	 * @param requestHeader
	 * 				The soapHeader received from client
	 * @return ISOAPHeader
	 * 				The soapHeader generated for response
	 * @throws BaseFault 
	 */
	public List<ISOAPHeaderBlock> generateResponseSOAPHeader(ISOAPHeader requestHeader) throws BaseFault{
		ResourceMessageHeaders rmHeaders = null;
		try {
			rmHeaders = new ResourceMessageHeaders(requestHeader);
		} catch (Exception e) {
			throw new BaseFault(e.getMessage());
		}
		return rmHeaders.createReplyHeaders().toHeaderBlocks();
	}
}
