package org.act.xservices.rt.xsecurity.handler;

import java.io.File;
import java.io.IOException;
import java.security.InvalidKeyException;
import java.security.Key;
import java.security.KeyPair;
import java.security.KeyPairGenerator;
import java.security.NoSuchAlgorithmException;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.SecureRandom;
import java.util.Date;
import java.util.List;

import javax.crypto.BadPaddingException;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.NoSuchPaddingException;
import javax.crypto.SecretKey;
import javax.crypto.spec.SecretKeySpec;

import org.act.xservices.rt.metaset.MetasetBuilder;
import org.act.xservices.rt.metaset.MetasetParserFactory;
import org.act.xservices.rt.xom.Attribute;
import org.act.xservices.rt.xom.Metaset;
import org.act.xservices.rt.xom.QName;
import org.act.xservices.rt.xsecurity.enc.EncryptedData;
import org.act.xservices.rt.xsecurity.enc.XMLEncDataBuilder;
import org.act.xservices.rt.xsecurity.enc.algfactory.AsymDec;
import org.act.xservices.rt.xsecurity.enc.algfactory.AsymDecFactory;
import org.act.xservices.rt.xsecurity.enc.algfactory.AsymEnc;
import org.act.xservices.rt.xsecurity.enc.algfactory.AsymEncFactory;
import org.act.xservices.rt.xsecurity.enc.algorithm.RSA_Dec;
import org.act.xservices.rt.xsecurity.exception.CipherDataNotFoundException;
import org.act.xservices.rt.xsecurity.exception.DataObjectNotFoundException;
import org.act.xservices.rt.xsecurity.exception.DecryptException;
import org.act.xservices.rt.xsecurity.exception.EncryptException;
import org.act.xservices.rt.xsecurity.exception.KeyResolverException;
import org.act.xservices.rt.xsecurity.exception.TransformationException;
import org.act.xservices.rt.xsecurity.util.MetaPrint;
import org.act.xservices.rt.xsecurity.util.tConst;

public class DecryptionHandler {
	private Metaset soap;

	private String algorithm;

	private String encedDataUri;

	private String Data2Dec;

	private Metaset header;

	private Metaset body;

	private Key symKey;

	private Key asymPriKey;

	private String symKeyEnced;

	private List<Attribute> attributesList;

	private String decResultHeader;

	private String decResultBody;

	private Metaset decMeta;

	private String asymKey;

	private String algName;

	private String psw;

	/**
	 * @param asymKey
	 */
	public DecryptionHandler(String asymKey) {
		super();
		this.asymKey = asymKey;
	}

	/**
	 * 
	 */
	public DecryptionHandler() {
		// TODO Auto-generated constructor stub
	}

	public void init(Metaset soapMt, String data2Sign, String asymKey,
			List<Attribute> attributesList) {
		this.soap = soapMt;

		this.asymKey = asymKey;
		this.attributesList = attributesList;
	}

	public void init(String algname, String asykey,
			List<Attribute> attributeslist, String data2dec) {
		this.algName = algname;
		this.asymKey = asykey;
		this.attributesList = attributeslist;
		this.psw = "password";
		this.Data2Dec = data2dec;
	}

	public void init(Metaset wsdl, Metaset wsdd, List<Attribute> attributesList) {
		try {
			this.algName = wsdl.filter(tConst.dec_algName).car().getText();
		} catch (Exception e) {
			this.algName = "http://www.w3.org/2000/09/xmldsig#rsa";
		}
		try {
			this.asymKey = wsdl.filter(tConst.dec_asymKey).car().getText()
					+ ".keystore";
		} catch (Exception e) {
			this.asymKey = System.getProperty("user.dir") + File.separator
					+ "conf\\WSSecureFiles\\files\\" + "server.keystore";
		}
		try {
			this.psw = wsdd.filter(tConst.dec_psw).car().getText();
		} catch (Exception e) {
			this.psw = "password";
		}
		this.attributesList = attributesList;
	}

	public Metaset process(Metaset soapMt) throws Exception {
		// ֽ⴫Soap
		getInput(soapMt);

		// ȡ㷨ͱݵid
		analyze();

		// ܶԳԿ
		DecKey();

		// ʹöԳԿ
		Metaset mt = DecData();

		return mt;
	}

	public Metaset process(Metaset headerMt, Metaset bodyMt) throws Exception {
		Metaset soapMt = reConsSoap(headerMt, bodyMt);
		// ֽ⴫Soap
		getInput(soapMt);

		// ȡ㷨ͱݵid
		analyze();

		// ܶԳԿ
		DecKey();

		// ʹöԳԿ
		Metaset mt = DecData();

		return mt;
	}

	public Metaset process(Metaset headerMt, Metaset bodyMt, QName soapname)
			throws Exception {
		Metaset soapMt = reConsSoap(headerMt, bodyMt, soapname);
		// ֽ⴫Soap
		getInput(soapMt);

		// ȡ㷨ͱݵid
		analyze();

		// ܶԳԿ
		DecKey();

		// ʹöԳԿ
		Metaset mt = DecData();

		return mt;
	}

	public Metaset process(Metaset headerMt, Metaset bodyMt, QName soapname,
			String algname) throws Exception {
		Metaset soapMt = reConsSoap(headerMt, bodyMt, soapname);
		// ֽ⴫Soap
		getInput(soapMt);

		// ȡ㷨ͱݵid
		analyze();

		// ܶԳԿ
		DecKey(algname);

		// ʹöԳԿ
		Metaset mt = DecData(algname, this.Data2Dec);

		return mt;
	}
	
	public Metaset process(Metaset soapMt,
			String algname) throws Exception {
//		Metaset soapMt = reConsSoap(headerMt, bodyMt, soapname);
		// ֽ⴫Soap
		getInput(soapMt);

		// ȡ㷨ͱݵid
		analyze();

		// ܶԳԿ
		DecKey(algname);

		// ʹöԳԿ
		Metaset mt = DecData(algname, this.Data2Dec);

		return mt;
	}
	
	

	public Metaset process() throws Exception {
		// ֽ⴫Soap
		getInput(soap);

		// ȡ㷨ͱݵid
		analyze();

		// ܶԳԿ
		DecKey();

		// ʹöԳԿ
		Metaset mt = DecData();

		return mt;
	}

	public Metaset getInput(Metaset soapMt) throws Exception {
		soap = soapMt;
		
//		header = soap.filter(".\\Header").car();
		header =(Metaset) soap.elements().get(0);
		return soap;
	}

	private void analyze() {
		// algorithm = header.selector(
		// ".\\Security\\EncryptedKey\\EncryptionMethod", "Algorithm");
		// encedDataUri = header.selector(
		// ".\\Security\\EncryptedKey\\ReferenceList\\DataReference",
		// "URI");

		algorithm = header.selectorAttributeValue(
				".\\Security\\EncryptedKey\\EncryptionMethod", "Algorithm");
		encedDataUri = header.selectorAttributeValue(
				".\\Security\\EncryptedKey\\ReferenceList\\DataReference",
				"URI");
		// System.out.println("wewwwwwwwwwwwwwwwwwwwwwwwwww\n"+algorithm+"\nwwwwwwwwwwwwwwwwwwwwwww\n"+encedDataUri);

	}

	private void DecKey() throws Exception {
		// System.out.println("wewwwwwwwwwwwwwwwwwwwwwwwwww\n"+soap.toFormatXML());
		symKeyEnced = soap.filter(tConst.cipherValue).car().getText();

		AsymDecFactory asymDecFactory = new AsymDecFactory();
		AsymDec asymDec = asymDecFactory.getDec(algName);
		// System.out
		// .println("##################################################\n"
		// + asymKey);
		String protoStr4SymKey = asymDec.encWithPri(symKeyEnced, asymKey, psw);
		if (protoStr4SymKey.equals("false")) {
			System.out.println("Wrong Keystore Password...");
			symKey = null;
		} else {
			Bridge bridge = new Bridge();
			symKey = bridge.creatSymKey(protoStr4SymKey);
		}

	}

	private void DecKey(String a) throws Exception {
		// System.out.println("wewwwwwwwwwwwwwwwwwwwwwwwwww\n"+soap.toFormatXML());
		symKeyEnced = soap.filter(tConst.cipherValue).car().getText();

		AsymDecFactory asymDecFactory = new AsymDecFactory();
		AsymDec asymDec = asymDecFactory.getDec(algName);
		// System.out
		// .println("##################################################\n"
		// + asymKey);
		String protoStr4SymKey = asymDec.encWithPri(symKeyEnced, asymKey, psw);

		if (protoStr4SymKey.equals("false")) {
			System.out.println("Wrong Keystore Password...");
			symKey = null;
		} else {

			Bridge bridge = new Bridge();
			symKey = bridge.creatSymKey(protoStr4SymKey, a);
		}

	}

	public Metaset DecData() {
		String encId = encedDataUri;
		Metaset souceData;
		String attr = "";
		try {

			EncryptedData DecData = new EncryptedData();

			// if the Keystore Password is wrong,return undecrpyted soap
			if (symKey == null) {

				return soap;
			}
            //  **********0806Ķ
			decResultHeader = DecData.decrypt(soap, "Header", symKey,
					tConst.des);

			decResultBody = DecData.decrypt(soap, "Body", symKey, tConst.des);
			// MetasetBuilderImpl decResultHeaderMtbuilder = new
			// MetasetBuilderImpl();
			for (int i = 0; i < this.attributesList.size(); i++) {

				attr = attr + this.attributesList.get(i).toString() + " ";
			}
			Metaset decResultHeaderMt = null;
			if (!decResultHeader.equals("")) {
				// decResultHeaderMt = Metaset.create(decResultHeader, "");
				// int a = attr.indexOf("Envelope");
				// String coa = attr.substring(a);
				// String cob = attr.substring(0, a);
				// System.out.println("&&&&&&&&&&&&&&&&&&&&&&&&&&&\n"+coa+"$$$$$$$$$$$$$$$$$$$$$$$$$$$$$\n"+cob);

				decResultHeader = "<Envelope "
						+ "xmlns:ds=\"http://www.w3.org/2000/09/xmldsig#\" "
						+ attr + ">" + decResultHeader + "</Envelope>";
				// System.out.println("**************************************\n"+decResultHeader);
				
				decResultHeaderMt = MetasetParserFactory.getMetasetParser().parseXMLString(decResultHeader).car();
			} else {
				// decResultHeaderMt = new Metaset ("Header");
				decResultHeaderMt = MetasetBuilder.getInstance()
						.newMetaset("Header");

			}
			// Metaset decResultBodyMt = Metaset.create(decResultBody,
			// "").car();
			decResultBody = "<Envelope " + attr + ">" + decResultBody
					+ "</Envelope>";
			Metaset decResultBodyMt = MetasetParserFactory.getMetasetParser().parseXMLString(decResultBody).car().car();
			Metaset headMt = soap.filter(".\\Header");
			// Metaset soapMt = new Metaset();
			Metaset soapMt = MetasetBuilder.getInstance().newMetaset();

			int i = soap.elements().size();

			// soap.remove((Metaset)soap.getChilds().get(1));

			while (soap.filter(".\\Header").car().elements().size() > 0) {
				for (int j = 0; j < soap.filter(".\\Header").car()
						.elements().size(); j++) {
					soap.car().removeChildren(
							soap.filter(".\\Header").car().elements()
									.get(j));

				}
			}

			// soap.car().remove(soap.car().filter(".\\Header").car());

			// ԭsoapBodyǩ
			// hdbodyMt--<SOAP-ENV:Body id="#xiaoshan"> </SOAP-ENV:Body
			// id="#xiaoshan">
			Metaset hdbodyMt = soap.elements().get(1);
			hdbodyMt.removeChildren(soap.filter(".\\Body").car().car());
			// if(decResultHeaderMt==null){
			// System.out.println("yyyyyyyyyyyyyyyyyyyyyyyyyy\n");
			// System.exit(0);
			// }

			for (int j = 0; j < decResultHeaderMt.elements().size(); j++) {
				soap.addChildren(".\\Header", decResultHeaderMt.elements()
						.get(j));
			}

			// soap.add(".\\", decResultHeaderMt);

			hdbodyMt.add(decResultBodyMt);
			// Metaset a2 = new Metaset("Body");
			// a2.add(decResultBodyMt);

			// soap.car().remove(soap.filter(".\\Header").car().car());

			// soap.add(hdbodyMt);

			decMeta = soap;

			// decMeta = soapMt;
			return decMeta;

		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return null;
	}

	public Metaset DecData(String algname, String datadec) {
		String encId = encedDataUri;
		Metaset souceData;
		String attr = "";
		try {

			EncryptedData DecData = new EncryptedData();

			// if the Keystore Password is wrong,return undecrpyted soap
			if (symKey == null) {

				return soap;
			}

			if (datadec.equals("soap")) {
				decResultHeader = DecData.decrypt(soap, "Header", symKey,
						algname);
                //
//				decResultHeader = decResultHeader.replaceAll("Header", "soapenv:Header");
				decResultBody = DecData.decrypt(soap, "Body", symKey, algname);
//				decResultBody = decResultBody.replaceAll("Body", "soapenv:Body");
				// MetasetBuilderImpl decResultHeaderMtbuilder = new
				// MetasetBuilderImpl();
				for (int i = 0; i < this.attributesList.size(); i++) {

					attr = attr + this.attributesList.get(i).toString() + " ";
				}
				
				Metaset decResultHeaderMt = null;

				if (!decResultHeader.equals("")) {
					// decResultHeaderMt = Metaset.create(decResultHeader, "");
					// int a = attr.indexOf("Envelope");
					// String coa = attr.substring(a);
					// String cob = attr.substring(0, a);
					// System.out.println("&&&&&&&&&&&&&&&&&&&&&&&&&&&\n"+coa+"$$$$$$$$$$$$$$$$$$$$$$$$$$$$$\n"+cob);
//0806Ķ
					decResultHeader = "<soapenv:Envelope "
							+ "xmlns:ds=\"http://www.w3.org/2000/09/xmldsig#\" "
							+"xmlns:wsse=\"http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd\" "
							+"xmlns:soapenv=\"http://schemas.xmlsoap.org/soap/envelope/\""
							+ attr + ">" + decResultHeader + "</soapenv:Envelope>";
					// System.out.println("**************************************\n"+decResultHeader);

					decResultHeaderMt = MetasetParserFactory.getMetasetParser().parseXMLString(decResultHeader).car();
				} else {
					// decResultHeaderMt = new Metaset ("Header");
					decResultHeaderMt = MetasetBuilder.getInstance().newMetaset("Header");

				}
				// Metaset decResultBodyMt = Metaset.create(decResultBody,
				// "").car();
				Metaset decResultBodyMt;
				if(!decResultBody.equals("")){
					decResultBody = "<soapenv:Envelope " +"xmlns:wsu=\"http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-utility-1.0.xsd\" " 
					+"xmlns:soapenv=\"http://schemas.xmlsoap.org/soap/envelope/\""
					+attr + ">" + decResultBody
					+ "</soapenv:Envelope>";
			decResultBodyMt = MetasetParserFactory.getMetasetParser().parseXMLString(decResultBody).car().car();
				}
				else{
					decResultBodyMt = MetasetBuilder.getInstance().newMetaset("Body");
				}
				
				Metaset headMt = soap.filter(".\\Header");
				// Metaset soapMt = new Metaset();
				Metaset soapMt = MetasetBuilder.getInstance().newMetaset();

				int i = soap.elements().size();

				// soap.remove((Metaset)soap.getChilds().get(1));

				while (soap.filter(".\\Header").car().elements().size() > 0) {
					for (int j = 0; j < soap.filter(".\\Header").car()
							.elements().size(); j++) {
						soap.car().removeChildren(
								soap.filter(".\\Header").car().elements()
										.get(j));

					}
				}

				// soap.car().remove(soap.car().filter(".\\Header").car());

				// ԭsoapBodyǩ
				// hdbodyMt--<SOAP-ENV:Body id="#xiaoshan"> </SOAP-ENV:Body
				// id="#xiaoshan">
				Metaset hdbodyMt = soap.elements().get(1);
				hdbodyMt.removeChildren(soap.filter(".\\Body").car().car());
//				soap.removeChildren(soap.filter(".\\Body").car());
//				Metaset bodymt = decResultBodyMt;
//				bodymt.detach();
//				soap.add(bodymt);
				// if(decResultHeaderMt==null){
				// System.out.println("yyyyyyyyyyyyyyyyyyyyyyyyyy\n");
				// System.exit(0);
				// }

				for (int j = 0; j < decResultHeaderMt.elements().size(); j++) {
					soap.addChildren(".\\Header", decResultHeaderMt
							.elements().get(j));
				}

				// soap.add(".\\", decResultHeaderMt);

				Metaset bodymt = decResultBodyMt;
				bodymt.detach();
				hdbodyMt.add(bodymt);
//				hdbodyMt.removeAttributeByName("xmlns");
				
				// Metaset a2 = new Metaset("Body");
				// a2.add(decResultBodyMt);

				// soap.car().remove(soap.filter(".\\Header").car().car());

				// soap.add(hdbodyMt);
				if (algname.equals("http://www.w3.org/2001/04/xmlenc#aes-cbcc")) {
					Metaset firstSoapMt = MetasetParserFactory.getMetasetParser().parse("conf\\WSSecureFiles\\files\\smallSoap.xml");
					Metaset HandlerLinks = MetasetBuilder.getInstance().newMetaset("HandlerLinks");
					Metaset hder = firstSoapMt.filter(".\\Header").car();
					List<Attribute> attributesList = firstSoapMt.attributes();
					for (int l = 0; l< hder.elements().size(); l++) {
						HandlerLinks.add(hder.elements().get(l));
					}
					// Metaset operation = new Metaset("operation");
					
					Metaset operation = MetasetBuilder.getInstance().newMetaset("operation");
					Metaset bdy = firstSoapMt.filter(".\\Body").car().car();
					operation.add(bdy);
			
					SinatureHandler sinatureHandler = new SinatureHandler();
					sinatureHandler.init("http://www.w3.org/2000/09/xmldsig#rsa", ".\\Body","http://www.w3.org/2000/09/xmldsig#sha1", attributesList); 
					decMeta = sinatureHandler.process(HandlerLinks, operation,firstSoapMt.getQName());

					
					
				} else {
					decMeta = soap;
				}

			} else if (datadec.equals("header")) {
				decResultHeader = DecData.decrypt(soap, "Header", symKey,
						algname);
				for (int i = 0; i < this.attributesList.size(); i++) {

					attr = attr + this.attributesList.get(i).toString() + " ";
				}
				Metaset decResultHeaderMt = null;
				if (!decResultHeader.equals("")) {
					decResultHeader = "<Envelope "
							+ "xmlns:ds=\"http://www.w3.org/2000/09/xmldsig#\" "
							+ attr + ">" + decResultHeader + "</Envelope>";
					// System.out.println("**************************************\n"+decResultHeader);

					decResultHeaderMt = MetasetParserFactory.getMetasetParser().parseXMLString(decResultHeader).car();
				} else {
					// decResultHeaderMt = new Metaset ("Header");
					decResultHeaderMt = MetasetBuilder.getInstance()
							.newMetaset("Header");

				}
				// Metaset soapMt = new Metaset();
				Metaset soapMt = MetasetBuilder.getInstance().newMetaset();

				int i = soap.elements().size();

				// soap.remove((Metaset)soap.getChilds().get(1));

				while (soap.filter(".\\Header").car().elements().size() > 0) {
					for (int j = 0; j < soap.filter(".\\Header").car()
							.elements().size(); j++) {
						soap.car().removeChildren(
								soap.filter(".\\Header").car().elements()
										.get(j));

					}
				}
				for (int j = 0; j < decResultHeaderMt.elements().size(); j++) {
					soap.addChildren(".\\Header", decResultHeaderMt
							.elements().get(j));
				}

			} else {

				decResultBody = DecData.decrypt(soap, "Body", symKey, algname);
				for (int i = 0; i < this.attributesList.size(); i++) {

					attr = attr + this.attributesList.get(i).toString() + " ";
				}
				decResultBody = "<Envelope " + attr + ">" + decResultBody
						+ "</Envelope>";
				Metaset decResultBodyMt = MetasetParserFactory.getMetasetParser().parseXMLString(decResultBody).car().car();
				Metaset headMt = soap.filter(".\\Header");
				Metaset soapMt = MetasetBuilder.getInstance().newMetaset();


				// ԭsoapBodyǩ
				// hdbodyMt--<SOAP-ENV:Body id="#xiaoshan"> </SOAP-ENV:Body
				// id="#xiaoshan">
				Metaset hdbodyMt = soap.elements().get(1);

				hdbodyMt.removeChildren(soap.filter(".\\Body").car().car());

				hdbodyMt.add(decResultBodyMt);
				// Metaset a2 = new Metaset("Body");
				// a2.add(decResultBodyMt);

				// soap.car().remove(soap.filter(".\\Header").car().car());

				// soap.add(hdbodyMt);

				decMeta = soap;
			}

			// decMeta = soapMt;
			return decMeta;

		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return null;
	}

	/**
	 * @return the asymPriKey
	 */
	public Key getAsymPriKey() {
		return asymPriKey;
	}

	/**
	 * @param asymPriKey
	 *            the asymPriKey to set
	 */
	public void setAsymPriKey(Key asymPriKey) {
		this.asymPriKey = asymPriKey;
	}

	/**
	 * @return the symKeyEnced
	 */
	public String getSymKeyEnced() {
		return symKeyEnced;
	}

	/**
	 * @param asymKeyEnced
	 *            the symKeyEnced to set
	 */
	public void setSymKeyEnced(String asymKeyEnced) {
		this.symKeyEnced = asymKeyEnced;
	}

	/**
	 * @return the decResultBody
	 */
	public String getDecResultBody() {
		return decResultBody;
	}

	/**
	 * @param decResultBody
	 *            the decResultBody to set
	 */
	public void setDecResultBody(String decResultBody) {
		this.decResultBody = decResultBody;
	}

	/**
	 * @return the decResultHeader
	 */
	public String getDecResultHeader() {
		return decResultHeader;
	}

	/**
	 * @param decResultHeader
	 *            the decResultHeader to set
	 */
	public void setDecResultHeader(String decResultHeader) {
		this.decResultHeader = decResultHeader;
	}

	/**
	 * @return the decMeta
	 */
	public Metaset getDecMeta() {
		return decMeta;
	}

	/**
	 * @param decMeta
	 *            the decMeta to set
	 */
	public void setDecMeta(Metaset decMeta) {
		this.decMeta = decMeta;
	}

	public Metaset reConsSoap(Metaset soapHeadMt, Metaset soapBodyMt) {
		QName rpQName = new QName("S", "Header", "s");
		QName rpQName2 = new QName("Body");
		// Metaset soap = new Metaset(new QName("SOAP-ENV", "Envelope",
		// "SOAP-ENV"));
		// MetasetBuilderImpl soapbuilder = new MetasetBuilderImpl();
		Metaset soap = MetasetBuilder.getInstance().newMetaset(new QName("SOAP-ENV", "Envelope",
				"SOAP-ENV"));
		// Metaset header = new Metaset(rpQName);
		Metaset header = MetasetBuilder.getInstance().newMetaset(rpQName);

		for (int i = 0; i < soapHeadMt.elements().size(); i++) {
			header.addChildren(".\\", soapHeadMt.elements().get(i));
		}

		// Metaset body = new Metaset(rpQName2);
		Metaset body = MetasetBuilder.getInstance().newMetaset(rpQName2);
		body.addChildren(".\\", soapBodyMt.elements().get(0));
		soap.add(header);
		soap.add(body);
		// System.out.println("%%%%%%%%%%%%%%%%%%%%%%%%%\n"+soap.toFormatXML());
		return soap;
	}

	public Metaset reConsSoap(Metaset soapHeadMt, Metaset soapBodyMt,
			QName soapname) {

		QName rpQName;
		QName rpQName2;

		if (soapname.getNamespaceURI().equals(
				"http://schemas.xmlsoap.org/soap/envelope/")) {
			rpQName = new QName("http://schemas.xmlsoap.org/soap/envelope/",
					"Header", soapname.getNamespacePrefix());
			rpQName2 = new QName("http://schemas.xmlsoap.org/soap/envelope/",
					"Body", soapname.getNamespacePrefix());
		} else if (soapname.getNamespaceURI().equals(
				"http://www.w3.org/2003/05/soap-envelope")) {
			rpQName = new QName("http://www.w3.org/2003/05/soap-envelope",
					"Header", soapname.getNamespacePrefix());
			rpQName2 = new QName("http://www.w3.org/2003/05/soap-envelope",
					"Body", soapname.getNamespacePrefix());
		} else {
			rpQName = new QName("http://schemas.xmlsoap.org/soap/envelope/",
					"Header", soapname.getNamespacePrefix());
			rpQName2 = new QName("http://schemas.xmlsoap.org/soap/envelope/",
					"Body", soapname.getNamespacePrefix());
		}

		Metaset soap = MetasetBuilder.getInstance().newMetaset(soapname);
		// Metaset header = new Metaset(rpQName);
		Metaset header = MetasetBuilder.getInstance().newMetaset(rpQName);

		for (int i = 0; i < soapHeadMt.elements().size(); i++) {
			header.addChildren(".\\", soapHeadMt.elements().get(i));
		}

		// Metaset body = new Metaset(rpQName2);
//		Metaset body = MetasetBuilder.getInstance().newMetaset(rpQName2);
//		body.addChildren(".\\", soapBodyMt.elements().get(0));
		Metaset metabody = soapBodyMt.element("Body");
		metabody.detach();
		soap.add(header);
		
		soap.add(metabody);

		for (int j = 0; j < this.attributesList.size(); j++) {
			soap.add(this.attributesList.get(j));
		}
		if (soapname.getNamespaceURI().equals(
				"http://schemas.xmlsoap.org/soap/envelope/")) {
			soap.add(MetasetBuilder.getInstance().newAttribute(new QName(
					"http://schemas.xmlsoap.org/soap/envelope/", "SOAP-ENV",
					"xmlns"), "http://schemas.xmlsoap.org/soap/envelope/"));
		} else if (soapname.getNamespaceURI().equals(
				"http://www.w3.org/2003/05/soap-envelope")) {
			soap.add(MetasetBuilder.getInstance().newAttribute(new QName(
					"http://www.w3.org/2003/05/soap-envelope", "SOAP-ENV",
					"xmlns"), "http://www.w3.org/2003/05/soap-envelope"));
		}
		// System.out.println("%%%%%%%%%%%%%%%%%%%%%%%%%\n"+soap.toFormatXML());
		return soap;
	}

}
