/**
 * Attribute.java
 */
package org.act.xservices.rt.xsecurity.xacml.context;

import java.net.URI;

import org.act.xservices.rt.xom.Attribute;
import org.act.xservices.rt.xom.Metaset;
import org.act.xservices.rt.xsecurity.xacml.AXacmlElement;
import org.act.xservices.rt.xsecurity.xacml.exception.DataTypeMappingException;
import org.act.xservices.rt.xsecurity.xacml.exception.PolicySyntaxErrorException;
import org.act.xservices.rt.xsecurity.xacml.policy.UtilMetaset;
import org.act.xservices.rt.xsecurity.xacml.policy.target.AttributeValue;

/**
 * ʵеԪءOASIS XACML淶2.06.7ڡ <xs:element name="Attribute"
 * type="xacml-context:AttributeType"/> <xs:complexType name="AttributeType">
 * <xs:sequence> <xs:element ref="xacml-context:AttributeValue"
 * maxOccurs="unbounded"/> </xs:sequence> <xs:attribute name="AttributeId"
 * type="xs:anyURI" use="required"/> <xs:attribute name="DataType"
 * type="xs:anyURI" use="required"/> <xs:attribute name="Issuer"
 * type="xs:string" use="optional"/> </xs:complexType>
 * 
 * 
 */
public class ContextAttribute extends AXacmlElement {
	public static final String ATTRIBUTE_ID = "AttributeId";

	public static final String DATA_TYPE = "DataType";

	public static final String ISSUER = "Issuer";

	public static final String ATTRIBUTE_VALUE = "AttributeValue";

	private URI attributeId = null;

	private URI dataType = null;

	private String issuer = null;

	private AttributeValue attributeValue = null;

	public ContextAttribute() {
	}

	public ContextAttribute(URI attributeId, URI dataType, String issuer,
			AttributeValue attributeValue) {
		this.attributeId = attributeId;
		this.dataType = dataType;
		this.issuer = issuer;
		this.attributeValue = attributeValue;
		this.attributeValue.setParent(this);
	}

	public ContextAttribute(Metaset xmlElement)
			throws PolicySyntaxErrorException, DataTypeMappingException {
		super(xmlElement);
		Attribute[] idAttrs = UtilMetaset.extractAttributesByName(xmlElement,
				ATTRIBUTE_ID);
		UtilMetaset.checkEmpty(idAttrs, ATTRIBUTE_ID);
		UtilMetaset.checkNumber(idAttrs, 1, ATTRIBUTE_ID);
		String idStr = idAttrs[0].getValue().trim();
		this.attributeId = URI.create(idStr);

		Attribute[] typeAttrs = UtilMetaset.extractAttributesByName(xmlElement,
				DATA_TYPE);
		UtilMetaset.checkEmpty(typeAttrs, DATA_TYPE);
		UtilMetaset.checkNumber(typeAttrs, 1, DATA_TYPE);
		String typeStr = typeAttrs[0].getValue().trim();
		this.dataType = URI.create(typeStr);

		Attribute[] issuerAttrs = UtilMetaset.extractAttributesByName(
				xmlElement, ISSUER);
		if (issuerAttrs != null && issuerAttrs.length != 0) {
			UtilMetaset.checkNumber(issuerAttrs, 1, ISSUER);
			this.issuer = issuerAttrs[0].getValue().trim();
		}

		Metaset[] valueChildren = UtilMetaset.extractChildrenByName(xmlElement,
				ATTRIBUTE_VALUE);
		UtilMetaset.checkEmpty(valueChildren, ATTRIBUTE_VALUE);
		UtilMetaset.checkNumber(valueChildren, 1, ATTRIBUTE_VALUE);
		String value = valueChildren[0].getText().trim();
		this.attributeValue = AttributeValue.newInstance(dataType, value);
		this.attributeValue.setParent(this);
	}

	public URI getAttributeId() {
		return attributeId;
	}

	public void setAttributeId(URI attributeId) {
		this.attributeId = attributeId;
	}

	public AttributeValue getAttributeValue() {
		return attributeValue;
	}

	public void setAttributeValue(AttributeValue attributeValue) {
		this.attributeValue = attributeValue;
		this.attributeValue.setParent(this);
	}

	public URI getDataType() {
		return dataType;
	}

	public void setDataType(URI dataType) {
		this.dataType = dataType;
	}

	public String getIssuer() {
		return issuer;
	}

	public void setIssuer(String issuer) {
		this.issuer = issuer;
	}
}
