/**
 * Request.java
 */
package org.act.xservices.rt.xsecurity.xacml.context;

import java.net.URI;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.act.xservices.rt.xom.Metaset;
import org.act.xservices.rt.xsecurity.xacml.AXacmlElement;
import org.act.xservices.rt.xsecurity.xacml.exception.DataTypeMappingException;
import org.act.xservices.rt.xsecurity.xacml.exception.IndeterminateException;
import org.act.xservices.rt.xsecurity.xacml.exception.PolicySyntaxErrorException;
import org.act.xservices.rt.xsecurity.xacml.policy.UtilMetaset;
import org.act.xservices.rt.xsecurity.xacml.policy.target.AttributeValue;
import org.act.xservices.rt.xsecurity.xacml.util.Constants;

/**
 * ʵһ(OASIS XACML淶2.06.1) <xs:element name="Request"
 * type="xacml-context:RequestType"/> <xs:complexType name="RequestType">
 * <xs:sequence> <xs:element ref="xacml-context:Subject" maxOccurs="unbounded"/>
 * <xs:element ref="xacml-context:Resource" maxOccurs="unbounded"/> <xs:element
 * ref="xacml-context:Action"/> <xs:element ref="xacml-context:Environment"/>
 * </xs:sequence> </xs:complexType> ʵֲֶ֧ԶԴȨֻܰһResource.
 * (XACML淶2.0УԶԴȨǿѡ)
 * 
 * 
 */
public class Request extends AXacmlElement implements IRequest {
	public static final String SUBJECT = "Subject";

	public static final String RESOURCE = "Resource";

	public static final String ACTION = "Action";

	public static final String ENVIRONMENT = "Environment";

	private ContextSubject[] subjects = null;

	private ContextResource resource = null;

	private ContextAction action = null;

	private ContextEnvironment environment = null;

	// idֵӳ䡣idԵ֣һжֵ
	private Map<URI, List<AttributeValue>> id2AttributeValues = null;

	public Request() {
	}

	public Request(ContextSubject[] subjects, ContextResource resource,
			ContextAction action, ContextEnvironment environment) {
		this.subjects = subjects;
		this.resource = resource;
		this.action = action;
		this.environment = environment;

		this.init();
	}

	public Request(Metaset xmlElement) throws PolicySyntaxErrorException,
			DataTypeMappingException {
		super(xmlElement);

		Metaset[] subjectChildren = UtilMetaset.extractChildrenByName(
				xmlElement, SUBJECT);
		UtilMetaset.checkEmpty(subjectChildren, SUBJECT);
		this.subjects = new ContextSubject[subjectChildren.length];
		for (int i = 0; i < this.subjects.length; i++) {
			this.subjects[i] = new ContextSubject(subjectChildren[i]);
		}

		Metaset[] resChildren = UtilMetaset.extractChildrenByName(xmlElement,
				RESOURCE);
		UtilMetaset.checkEmpty(resChildren, RESOURCE);
		UtilMetaset.checkNumber(resChildren, 1, RESOURCE);
		this.resource = new ContextResource(resChildren[0]);

		Metaset[] actionChildren = UtilMetaset.extractChildrenByName(
				xmlElement, ACTION);
		UtilMetaset.checkEmpty(actionChildren, ACTION);
		UtilMetaset.checkNumber(actionChildren, 1, ACTION);
		this.action = new ContextAction(actionChildren[0]);

		Metaset[] envChildren = UtilMetaset.extractChildrenByName(xmlElement,
				ENVIRONMENT);
		UtilMetaset.checkEmpty(envChildren, ENVIRONMENT);
		UtilMetaset.checkNumber(envChildren, 1, ENVIRONMENT);
		this.environment = new ContextEnvironment(envChildren[0]);

		this.init();
	}

	private void init() {
		for (int i = 0; i < this.subjects.length; i++) {
			this.subjects[i].setParent(this);
		}
		this.resource.setParent(this);
		this.action.setParent(this);
		this.environment.setParent(this);

		this.id2AttributeValues = new HashMap<URI, List<AttributeValue>>();
		for (int i = 0; i < this.subjects.length; i++) {
			ContextAttribute[] attributes = this.subjects[i].getAttributes();
			for (int j = 0; j < attributes.length; j++) {
				this.addAttribute(attributes[j].getAttributeId(), attributes[j]
						.getAttributeValue());
			}
		}
		ContextAttribute[] ResAttributes = this.resource.getAttributes();
		for (int j = 0; j < ResAttributes.length; j++) {
			this.addAttribute(ResAttributes[j].getAttributeId(),
					ResAttributes[j].getAttributeValue());
		}
		ContextAttribute[] actionAttributes = this.action.getAttributes();
		for (int j = 0; j < actionAttributes.length; j++) {
			this.addAttribute(actionAttributes[j].getAttributeId(),
					actionAttributes[j].getAttributeValue());
		}
		ContextAttribute[] envAttributes = this.environment.getAttributes();
		for (int j = 0; j < envAttributes.length; j++) {
			this.addAttribute(envAttributes[j].getAttributeId(),
					envAttributes[j].getAttributeValue());
		}
	}

	private void addAttribute(URI id, AttributeValue value) {
		List<AttributeValue> list = this.id2AttributeValues.get(id);
		if (list == null) {
			list = new ArrayList<AttributeValue>();
			this.id2AttributeValues.put(id, list);
		}
		list.add(value);
	}

	public AttributeValue[] queryAttributeValues(URI attributeId, URI dataType,
			String issuer, URI subjectCategory) {
		// TODO Auto-generated method stub
		List<AttributeValue> sameIdList = this.id2AttributeValues
				.get(attributeId);
		List<AttributeValue> hit = new ArrayList<AttributeValue>();
		if (sameIdList != null) {
			for (AttributeValue v : sameIdList) {
				ContextAttribute parent = (ContextAttribute) v.getParent();

				if (parent.getDataType().equals(dataType)) {
					if (subjectCategory != null) {
						ContextSubject sub = (ContextSubject) parent
								.getParent();
						if (sub.getCategory().equals(subjectCategory)) {
							hit.add(v);
						}
					} else {
						hit.add(v);
					}
				}
			}
		}
		return hit.toArray(new AttributeValue[0]);
	}

	public AttributeValue[] getAttributeValuesByXPath(
			String requestContextPath, URI dataType)
			throws IndeterminateException {
		// TODO Auto-generated method stub
		return null;
	}

	public String getResourceId() {
		// TODO Auto-generated method stub
		return null;
	}

	public ContextAction getAction() {
		return action;
	}

	public void setAction(ContextAction action) {
		this.action = action;
	}

	public ContextEnvironment getEnvironment() {
		return environment;
	}

	public void setEnvironment(ContextEnvironment environment) {
		this.environment = environment;
	}

	public ContextResource getResource() {
		return resource;
	}

	public void setResource(ContextResource resource) {
		this.resource = resource;
	}

	public ContextSubject[] getSubjects() {
		return subjects;
	}

	public void setSubjects(ContextSubject[] subjects) {
		this.subjects = subjects;
	}

	public AContextEntity[] getAllContextEntities() {
		// TODO Auto-generated method stub
		List<AContextEntity> result = new ArrayList<AContextEntity>();
		if (this.subjects != null) {
			for (int i = 0; i < this.subjects.length; i++) {
				result.add(this.subjects[i]);
			}
		}
		if (this.resource != null) {
			result.add(this.resource);
		}
		if (this.action != null) {
			result.add(this.action);
		}
		if (this.environment != null) {
			result.add(this.environment);
		}
		return result.toArray(new AContextEntity[0]);

	}

}
