/**
 * AttributeIndex.java
 */
package org.act.xservices.rt.xsecurity.xacml.pap;

import java.net.URI;

import org.act.xservices.rt.xsecurity.xacml.policy.target.AAttributeDesignator;
import org.act.xservices.rt.xsecurity.xacml.policy.target.AttributeValue;

/**
 * 
 */
class AttributeIndex {
    private int hashCode;
    private String str;
    private URI attrId;
    private URI dataType;
    private Object value;

    public AttributeIndex(URI attrId, URI dataType) {
        this.attrId = attrId;
        this.dataType = dataType;
        generateHashCode();
    }

    public AttributeIndex(URI attrId, URI dataType, Object value) {
        this.attrId = attrId;
        this.dataType = dataType;
        this.value = value;
        generateHashCode();
    }

    public AttributeIndex(AAttributeDesignator designator, AttributeValue attrValue) {
    	
        this.attrId = designator.getAttributeId();
        this.dataType = designator.getDataType();
        this.value = attrValue.getValue();
        generateHashCode();
    }

    private void generateHashCode() {
        hashCode = this.attrId.hashCode() + hashCode * 13;
        hashCode = this.dataType.hashCode() + hashCode * 13;
        hashCode = (value == null ? 0 : value.hashCode()) + hashCode * 13;
        str = attrId.toString() + "(" + dataType.toString() + "," + (value == null ? "null" : value) + ")";
    }

    public URI getAttributeId() {
        return attrId;
    }

    public URI getDataType() {
        return dataType;
    }

    public Object getValue() {
        return value;
    }

    public int hashCode() {
        return hashCode;
    }

    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }

        if (o != null && o.getClass() == this.getClass()) {
            AttributeIndex other = (AttributeIndex)o;
            if (this.attrId.equals(other.attrId) && this.dataType.equals(other.dataType) &&
               compareObject(value, other.value)) {
                return true;
            }
        }
        return false;
    }

    public boolean compareObject(Object a,Object b){
    	 if (a == b) {
             return true;
         }
         else if (a != null) {
             return a.equals(b);
         }
         else if (b != null) {
             return b.equals(a);
         }
         return false;
    }
    
    public String toString() {
        return str;
    }
}