/**
 * DeployMonitor.java
 */
package org.act.xservices.rt.xsecurity.xacml.pap;

import java.io.File;
import java.io.FileFilter;
import java.net.URI;
import java.util.HashMap;
import java.util.Map;
import java.util.logging.Logger;

import org.act.xservices.rt.metaset.exception.MetasetParsingException;
import org.act.xservices.rt.xsecurity.xacml.exception.XacmlException;
import org.act.xservices.rt.xsecurity.xacml.util.LoggerFactory;

/**
 * 
 */
public class DeployMonitor implements Runnable {
	
	private static final String LOCAL_FILE = "local_file";
	
	private static  Logger logger=LoggerFactory.getLogger(DeployMonitor.class);

	//ϴβʱԿ״̬
	private Map<String,URI> last_deployed = null; 
	
	private String policy_file_dir=null;
	
	//ʱ(Ϊλ)
	private int update_interval;
	
	private PolicyCache cache=null;
	
	/* (non-Javadoc)
	 * @see java.lang.Runnable#run()
	 */
	
	public DeployMonitor(String policy_file_dir,int update_interval,PolicyCache cache){
		this.last_deployed=new HashMap<String,URI>();
		this.policy_file_dir=policy_file_dir;
		this.update_interval=update_interval;
		this.cache=cache;
	}	
	public PolicyCache getCache() {
		return cache;
	}
	public void setCache(PolicyCache cache) {
		this.cache = cache;
	}
	public Map<String, URI> getLast_deployed() {
		return last_deployed;
	}
	public void setLast_deployed(Map<String, URI> last_deployed) {
		this.last_deployed = last_deployed;
	}
	public String getPolicy_file_dir() {
		return policy_file_dir;
	}
	public void setPolicy_file_dir(String policy_file_dir) {
		this.policy_file_dir = policy_file_dir;
	}
	public int getUpdate_interval() {
		return update_interval;
	}
	public void setUpdate_interval(int update_interval) {
		this.update_interval = update_interval;
	}
	public void run() {
		// TODO Auto-generated method stub
		try{
			while(true){
				logger.info("DepolyMonitor starts to detect updates...");
				long before=System.currentTimeMillis();
				
				int count=0;
				
				//¼ɨʱԿ״̬
				Map<String,URI> temp=new HashMap<String,URI>();
				File dir=new File(this.policy_file_dir);
				if(dir.isDirectory()){
					File[] policies= dir.listFiles(new FileFilter(){

						public boolean accept(File pathname) {
							// TODO Auto-generated method stub
							String name=pathname.getName();
							if(name.endsWith(".xml")){
								return true;
							}
							return false;
						}
					});		
					//µĲ
					for(File p:policies){
						if(!exist(p.getAbsolutePath())){
							PolicyCacheableItem itm=new PolicyCacheableItem(p.getAbsolutePath(),LOCAL_FILE,this.cache);
							URI id=this.cache.AddPolicyCacheableItem(itm);
							//¼Ĳ
							temp.put(p.getAbsolutePath(), id);
							count++;
							logger.info("\tDeploy policy [PolicyId="+id+", PolicyFilePath="+p.getAbsolutePath()+"] successfully");
						}else{
							//¼ϴδڵĲ
							temp.put(p.getAbsolutePath(), this.last_deployed.get(p.getAbsolutePath()));
						}
					}
					this.last_deployed=temp;
					
					long after=System.currentTimeMillis();
					String count_msg=count+" new "+((count==1||count==0)?"policy was":"policies were")+" deployed.";
					long time=after-before;
					String time_msg=time + ((time==0||time==1)?" millisecond":" millseconds")+" elapsed.";
					logger.info("Done. "+count_msg+" "+time_msg	);
				}
				
				Thread.sleep(this.update_interval*1000);
			}
		}
		catch(InterruptedException e){
			e.printStackTrace();
		}
		catch(XacmlException e){
			e.printStackTrace();
			
		} catch (MetasetParsingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	private boolean exist(String abPath){
		if(this.last_deployed.get(abPath)==null){
			return false;
		}
		return true;
	}
	
	public static void main(String[] args) throws InterruptedException{
		String dir = "policies";
		int interval=10;
		PolicyCache cache=new PolicyCache(new PolicyLoader());
		
		DeployMonitor monitor=new DeployMonitor(dir,interval,cache);
		
		new Thread(monitor).start();
		
		Thread.currentThread().join();
		
	}
}
