/**
 * PolicyCache.java
 */
package org.act.xservices.rt.xsecurity.xacml.pap;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.net.URI;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Formatter;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.logging.Logger;

import org.act.xservices.rt.metaset.exception.MetasetParsingException;
import org.act.xservices.rt.xsecurity.xacml.Cache;
import org.act.xservices.rt.xsecurity.xacml.context.AContextEntity;
import org.act.xservices.rt.xsecurity.xacml.context.ContextAttribute;
import org.act.xservices.rt.xsecurity.xacml.context.IRequest;
import org.act.xservices.rt.xsecurity.xacml.exception.CacheOverflowException;
import org.act.xservices.rt.xsecurity.xacml.exception.DataTypeMappingException;
import org.act.xservices.rt.xsecurity.xacml.exception.IndeterminateException;
import org.act.xservices.rt.xsecurity.xacml.exception.PolicySyntaxErrorException;
import org.act.xservices.rt.xsecurity.xacml.exception.UnsuportedLocationTypeException;
import org.act.xservices.rt.xsecurity.xacml.policy.AXacmlPolicy;
import org.act.xservices.rt.xsecurity.xacml.policy.target.AAttributeDesignator;
import org.act.xservices.rt.xsecurity.xacml.policy.target.AEntityMatch;
import org.act.xservices.rt.xsecurity.xacml.policy.target.Actions;
import org.act.xservices.rt.xsecurity.xacml.policy.target.AttributeValue;
import org.act.xservices.rt.xsecurity.xacml.policy.target.ConjunctiveMatch;
import org.act.xservices.rt.xsecurity.xacml.policy.target.DisjunctiveMatch;
import org.act.xservices.rt.xsecurity.xacml.policy.target.Environments;
import org.act.xservices.rt.xsecurity.xacml.policy.target.IAttributeLocator;
import org.act.xservices.rt.xsecurity.xacml.policy.target.Resources;
import org.act.xservices.rt.xsecurity.xacml.policy.target.Subjects;
import org.act.xservices.rt.xsecurity.xacml.policy.target.Target;
import org.act.xservices.rt.xsecurity.xacml.util.Constants;
import org.act.xservices.rt.xsecurity.xacml.util.LoggerFactory;

/**
 * 
 * Իάӳһб 1. ԱʶԵӳ䣻 2. Իӳ䣻 3. targetԻб
 * 
 * һӳάϢڶӳб3ڲҲԡ
 * 
 * 
 */
public class PolicyCache extends Cache{
	
	private static Logger logger=LoggerFactory.getLogger(PolicyCache.class);

	// cachedPolicyCountڴʱ̣߳ͷŹڵĲԡ
	private int max_cached_policy_count = 1200;

	// ԤĻռ䣬ǰĲcachedPolicyCount+preparedNum >=
	// max_cached_policy_countʱ߳
	private int preparedNum = 10;

	// ǿemptyĲԵĻʱ(Ϊλ)ڲԻi
	// (System.currentTimeMillis()-i.getLastUseTime())/1000 > expire_interval
	// ʱiΪڻ´߳ʱеĲԡ
	// -1ʾ
	private long nonempty_expire_interval = 20;

	// emptyĲԵĻʱ(Ϊλ)ڿemptyĲκǵĻʱӦñȷǿյĳ
	// -1ʾ
	private long empty_expire_interval = 40;


	// ӳ1ԱʶԵӳ
	private Map<URI, List<AttributeIndex>> invertedTable = null;

	// ӳ2Իӳ
	private Map<AttributeIndex, List<PolicyCacheableItem>> cache = null;

	// б3targetԻб
	private List<PolicyCacheableItem> emptyTargetPolicyCache = null;

	private int item_count;

	private int cachedPolicyCount;

	private PolicyLoader policyLoader = null;

	

	public PolicyCache(PolicyLoader loader) {
		this.cache = new HashMap<AttributeIndex, List<PolicyCacheableItem>>();
		this.emptyTargetPolicyCache = new ArrayList<PolicyCacheableItem>();
		this.invertedTable = new HashMap<URI, List<AttributeIndex>>();
		this.item_count = 0;
		this.cachedPolicyCount = 0;
		this.policyLoader = loader;
	}

	/**
	 * һԻ
	 * 
	 * ԻаĲһtargetԣӵб3У a. Ե
	 * b.ѲԱʶӵӳ1 c. Ѹ͸ĲԻһһӳ2
	 * 
	 * @return
	 * @throws UnsuportedLocationTypeException
	 * @throws IndeterminateException
	 * @throws DataTypeMappingException
	 * @throws PolicySyntaxErrorException
	 * @throws MetasetParsingException
	 * @throws CacheOverflowException
	 * @throws
	 * @param PolicyCacheableItem
	 */
	public URI AddPolicyCacheableItem(PolicyCacheableItem policyCacheableItem)
			throws MetasetParsingException, PolicySyntaxErrorException,
			DataTypeMappingException, IndeterminateException,
			UnsuportedLocationTypeException, CacheOverflowException {

		try {

			this.lock();

			this.refreshOne(policyCacheableItem);
			AXacmlPolicy xacmlPolicy = policyCacheableItem.getXacmlPolicy();

			// ˲ԻѾڻ
			if (this.invertedTable.get(xacmlPolicy.getId()) != null
					|| this.emptyTargetPolicyCache.contains(xacmlPolicy)) {
				this.releaseOne(policyCacheableItem);
				return null;
			}

			policyCacheableItem.setOwner(this);

			// ԻĲԵ
			List<AttributeIndex> indexList = extractIndiceFromXacmlPolicy(xacmlPolicy);
			// ѲԱʶӵӳ1.

			if (indexList == null || indexList.size() == 0) {
				if (!this.emptyTargetPolicyCache.contains(policyCacheableItem)) {
					this.emptyTargetPolicyCache.add(policyCacheableItem);
				}
			} else {
				this.invertedTable.put(xacmlPolicy.getId(), indexList);

				AttributeIndex[] indice = indexList
						.toArray(new AttributeIndex[0]);

				// ÿ͸ĲԻӵ
				if (indice != null && indice.length > 0) {
					for (int i = 0; i < indice.length; i++) {
						List<PolicyCacheableItem> items = this.cache
								.get(indice[i]);
						if (items == null) {
							items = new ArrayList<PolicyCacheableItem>();
							this.cache.put(indice[i], items);
						}
						if (!items.contains(policyCacheableItem)) {
							items.add(policyCacheableItem);
						}
					}
				}
			}
			this.item_count++;
			return policyCacheableItem.getPolicyId();
		} finally {
			this.unlock();
		}
	}

	/**
	 * ȡtargetаĸԣΪAttributeIndex
	 * 
	 * @param xacmlPolicy
	 * @return
	 */
	private List<AttributeIndex> extractIndiceFromXacmlPolicy(
			AXacmlPolicy xacmlPolicy) {
		List<AttributeIndex> result = new ArrayList<AttributeIndex>();
		Target target = xacmlPolicy.getTarget();
		// ȡԵtargetаĸʵ弯()弯Դͻ
		DisjunctiveMatch[] entitySets = extractMatchesFromTarget(target);
		for (int i = 0; i < entitySets.length; i++) {
			DisjunctiveMatch entitySet = entitySets[i];
			ConjunctiveMatch[] entities = (ConjunctiveMatch[]) entitySet
					.getMatches();
			// ʵ弯иʵ壬гȡ
			for (int j = 0; j < entities.length; j++) {
				ConjunctiveMatch entity = entities[j];
				AEntityMatch[] matches = (AEntityMatch[]) entity.getMatches();
				for (int k = 0; k < matches.length; k++) {
					AEntityMatch match = matches[k];
					IAttributeLocator locator = match.getAttributeLocator();
					AttributeValue attrValue=match.getAttributeValue();
					Object obj=attrValue.getValue();
					if (locator instanceof AAttributeDesignator) {
						// 
						AAttributeDesignator designator = (AAttributeDesignator) locator;
						URI attrId = designator.getAttributeId();
						URI attrType = designator.getDataType();
						AttributeIndex index = new AttributeIndex(attrId,
								attrType,obj);
						result.add(index);
					}
				}
			}
		}
		return result;
	}

	/**
	 * ȡԵtargetаĸʵ弯()弯Դͻ
	 * 
	 * @param target
	 * @return
	 */
	private DisjunctiveMatch[] extractMatchesFromTarget(Target target) {
		List<DisjunctiveMatch> result = new ArrayList<DisjunctiveMatch>();

		Subjects subjects = target.getSubjects();
		if (subjects != null) {
			result.add(subjects);
		}

		Resources resources = target.getResources();
		if (resources != null) {
			result.add(resources);
		}

		Actions actions = target.getActions();
		if (actions != null) {
			result.add(actions);
		}

		Environments environments = target.getEnvironments();
		if (environments != null) {
			result.add(environments);
		}

		return result.toArray(new DisjunctiveMatch[0]);
	}

	private void ensureCapacity() throws CacheOverflowException {

		// cachedPolicyCount+preparedNum>max_cached_policy_countʱͿʼ̣߳ÿζһЩ
		// ԣᱻ㡣ǣеĲԶûеڣÿ̲߳δʵԣֱ㣬
		// 쳣
		if (this.cachedPolicyCount >= this.max_cached_policy_count) {
			throw new CacheOverflowException(Constants.STATUS_PROCESSINGERROR,
					"PolicyCache overflows!", null);
		}
		
		if (this.cachedPolicyCount + this.preparedNum >= this.max_cached_policy_count) {
			//(Ԥռ <= preparedNum/2)еڲ
			if(this.cachedPolicyCount+this.preparedNum/2 >= this.max_cached_policy_count){
				cleanupExpired(true);
			}
			//߳治ţֻշǿtargetĵڲ
			else{
				cleanupExpired(false);
			}
			
		}
	}
	
	protected void cleanup(boolean emergent){
		try{
			lock();
			for(List<PolicyCacheableItem> list:cache.values()){
				for(PolicyCacheableItem itm:list){
					if(!itm.isExpired()){
						long lut=itm.getLastUseTime();
						long now=System.currentTimeMillis();
						long span=(now-lut)/1000;
						if(span > nonempty_expire_interval){
							releaseOne(itm);
						}
					}
				}
			}
			//ֻлռŵʱŻտtargetĵڲ
			if(emergent){
				for(PolicyCacheableItem itm:emptyTargetPolicyCache){
					if(!itm.isExpired()){
						long lut=itm.getLastUseTime();
						long now=System.currentTimeMillis();
						long span=(now-lut)/1000;
						if(span > empty_expire_interval){
							releaseOne(itm);
						}
					}
				}
			}
		}
		finally{
			unlock();
		}
	}

	/**
	 * ѯڸĲԡ ҹ򣺸ݸаԣtargetʹЩԵĲԡ⣬targetԡ
	 * ע⣺ҵĲԲһáǽһɸѡõĲԡ˵һõĲԣPDPЧʡ
	 * 
	 * @return
	 * @throws MetasetParsingException
	 * @throws PolicySyntaxErrorException
	 * @throws DataTypeMappingException
	 * @throws IndeterminateException
	 * @throws UnsuportedLocationTypeException
	 * @throws CacheOverflowException
	 */
	public AXacmlPolicy[] getXacmlPolicies(IRequest request)
			throws MetasetParsingException, PolicySyntaxErrorException,
			DataTypeMappingException, IndeterminateException,
			UnsuportedLocationTypeException, CacheOverflowException {
		List<AXacmlPolicy> result = new ArrayList<AXacmlPolicy>();

		try {
			this.lock();
			// targetĲʺκ԰ӵ
			for (PolicyCacheableItem itm : this.emptyTargetPolicyCache) {
				this.refreshOne(itm);
				AXacmlPolicy policy = itm.getXacmlPolicy();
				result.add(policy);
			}

			// лȡʵ壬塢Դ
			AContextEntity[] contextEntities = request.getAllContextEntities();
			if (contextEntities != null && contextEntities.length > 0) {
				for (int i = 0; i < contextEntities.length; i++) {
					AContextEntity contextEntity = contextEntities[i];
					// ӸʵлȡԼ
					ContextAttribute[] attrs = contextEntity.getAttributes();
					if (attrs != null && attrs.length > 0) {
						for (int j = 0; j < attrs.length; j++) {
							// ÿԹ
							ContextAttribute attr = attrs[j];
							URI attrId = attr.getAttributeId();
							URI attrType = attr.getDataType();
							Object value=attr.getAttributeValue().getValue();
							
							AttributeIndex index = new AttributeIndex(attrId,
									attrType,value);
							// ݹӳ2.
							List<PolicyCacheableItem> items = this.cache
									.get(index);
							// ЩлĲԿܱͷţرͷŵĲԡ
							refreshAll(items);
							// ѸеĲӵ
							if(items!=null)
							for (PolicyCacheableItem itm : items) {
								AXacmlPolicy p = itm.getXacmlPolicy();
								if (!result.contains(p)) {
									result.add(p);
								}
							}
						}
					}
				}
			}
		} finally {
			this.unlock();
		}
		return result.toArray(new AXacmlPolicy[0]);
	}

	private void refreshOne(PolicyCacheableItem item)
			throws MetasetParsingException, PolicySyntaxErrorException,
			DataTypeMappingException, IndeterminateException,
			UnsuportedLocationTypeException, CacheOverflowException {
		ensureCapacity();
		if (item.refresh()) {
			this.cachedPolicyCount++;
		}
	}

	private void refreshAll(List<PolicyCacheableItem> items)
			throws MetasetParsingException, PolicySyntaxErrorException,
			DataTypeMappingException, IndeterminateException,
			UnsuportedLocationTypeException, CacheOverflowException {
		if(items!=null)
		for (PolicyCacheableItem itm : items) {
			if(itm!=null)
				refreshOne(itm);
		}
	}

	private void releaseOne(PolicyCacheableItem item) {
		if (item.release()) {
			this.cachedPolicyCount--;
		}
	}

	private void releaseAll(List<PolicyCacheableItem> items) {
		for (PolicyCacheableItem itm : items) {
			releaseOne(itm);
		}
	}

	/**
	 * ͷһռõڴռ䡣ע⣬ͷһԲͬͷŲڴռ䲻ٲڻеϢ١
	 * 
	 * @return
	 * @throws
	 * @param policyId
	 */
	public void releasePolicy(URI policyId) {
		try {
			this.lock();

			PolicyCacheableItem item = null;

			// ӿtargetвҴ˲
			for (PolicyCacheableItem itm : this.emptyTargetPolicyCache) {
				if (itm.getPolicyId().equals(policyId)) {
					item = itm;
					break;
				}
			}
			if (item == null) {
				// ӷлȡ˲԰
				List<AttributeIndex> indexList = this.invertedTable
						.get(policyId);
				// ÿܰڶͬĲУÿӦĲԲԻҳҪͷŵǸ
				// ע⣬AttributeIndexͬһPolicyCacheableItemӦʱֻһPolicyCacheableItemĶ
				// ԣҵһֹͣ
				for (AttributeIndex index : indexList) {
					List<PolicyCacheableItem> list = this.cache.get(index);
					boolean find = false;
					for (PolicyCacheableItem itm : list) {
						if (itm.getPolicyId().equals(policyId)) {
							item = itm;
							find = true;
							break;
						}
					}
					if (find) {
						break;
					}
				}
			}

			if (item != null) {
				releaseOne(item);
			}
		} finally {
			this.unlock();
		}
	}

	/**
	 * һԣע⣬ͷһڴռ䲻ͬͷŲڴռ䲻ٲڻеϢ١
	 * 
	 * @return
	 * @throws
	 * @param policyId
	 */
	public void undepolyPolicy(URI policyId) {
		try {
			this.lock();

			PolicyCacheableItem item = null;
			// ڿtargetвҸ
			for (PolicyCacheableItem itm : this.emptyTargetPolicyCache) {
				if (itm.getPolicyId().equals(policyId)) {
					item = itm;
					break;
				}
			}
			// targetҵɾ
			if (item != null) {
				releaseOne(item);
				this.emptyTargetPolicyCache.remove(item);
				this.item_count--;
			}
			// cacheɾ
			else {
				// ӷɾ
				List<AttributeIndex> indexList = this.invertedTable
						.remove(policyId);
				if (indexList != null) {
					// ɾԵĸӳ
					for (AttributeIndex index : indexList) {
						List<PolicyCacheableItem> items = this.cache.get(index);
						PolicyCacheableItem i = null;
						for (PolicyCacheableItem itm : items) {
							if (itm.getPolicyId().equals(policyId)) {
								i = itm;
								break;
							}
						}
						if (i != null) {
							releaseOne(i);
							items.remove(i);
							if (items.size() == 0) {
								this.cache.remove(index);
							}
						}
					}
					this.item_count--;
				}
			}
		} finally {
			this.unlock();
		}
	}

	public PolicyLoader getPolicyLoader() {
		return policyLoader;
	}

	public void setPolicyLoader(PolicyLoader policyLoader) {
		this.policyLoader = policyLoader;
	}

	public Map<URI, List<AttributeIndex>> getInvertedTable() {
		return invertedTable;
	}

	public void setInvertedTable(Map<URI, List<AttributeIndex>> invertedTable) {
		this.invertedTable = invertedTable;
	}
	
	public Map<AttributeIndex, List<PolicyCacheableItem>> getCache() {
		return cache;
	}

	public void setCache(Map<AttributeIndex, List<PolicyCacheableItem>> cache) {
		this.cache = cache;
	}

	public List<PolicyCacheableItem> getEmptyTargetPolicyCache() {
		return emptyTargetPolicyCache;
	}

	public void setEmptyTargetPolicyCache(
			List<PolicyCacheableItem> emptyTargetPolicyCache) {
		this.emptyTargetPolicyCache = emptyTargetPolicyCache;
	}

	public long getEmpty_expire_interval() {
		return empty_expire_interval;
	}

	public void setEmpty_expire_interval(long empty_expire_interval) {
		this.empty_expire_interval = empty_expire_interval;
	}

	public long getNonempty_expire_interval() {
		return nonempty_expire_interval;
	}

	public void setNonempty_expire_interval(long nonempty_expire_interval) {
		this.nonempty_expire_interval = nonempty_expire_interval;
	}

	public void dump_count(OutputStream out) throws IOException{
		try{
			lock();
			
			boolean isDef = false;
			if (out == null) {
				isDef = true;
				out = new FileOutputStream(new File("dump.txt"));
			}

			PrintWriter pw = new PrintWriter(out);
			pw.println("item_count = " + this.item_count);
			pw.println("cachedPolicyCount = " + this.cachedPolicyCount);
			
			if (isDef) {
				pw.close();
				out.close();			
			}
		}
		finally{
			unlock();
		}
	}
	
	public void dump_detail(OutputStream out) throws IOException {

		try{
			lock();
			boolean isDef = false;
			if (out == null) {
				isDef = true;
				out = new FileOutputStream(new File("dump.txt"));
			}

			PrintWriter pw = new PrintWriter(out);
			pw.println("item_count = " + this.item_count);
			pw.println("cachedPolicyCount = " + this.cachedPolicyCount);

			pw.println("_____________________non-empty-target policy____________________");
			pw.flush();
			for (AttributeIndex i : this.cache.keySet()) {
				pw.println("AttributeIndex: (attributeId=" + i.getAttributeId()
						+ ", attributeType=" + i.getDataType() + ")");
				List<PolicyCacheableItem> list = this.cache.get(i);
				for (PolicyCacheableItem itm : list) {
					printItem(pw,itm);
				}
			}

			pw.println("_______________________empty-target policy______________________");
			pw.flush();
			for(PolicyCacheableItem itm:this.emptyTargetPolicyCache){
				printItem(pw,itm);
			}
			
			// ֻرԼ򿪵ļ
			if (isDef) {
				pw.close();
				out.close();
			}
		}
		finally{
			unlock();
		}
	}
	
	private void printItem(PrintWriter pw,PolicyCacheableItem itm){
		pw.println("\tPolicyCacheableItem:\n\t\tpolicyId="
				+ itm.getPolicyId() + "\n\t\texpired="
				+ itm.isExpired() + "\n\t\tfileLocation="
				+ itm.getFileLocation());
		Formatter fmt = new Formatter();
		Calendar c = Calendar.getInstance();
		c.setTimeInMillis(itm.getCreateTime());
		fmt.format("%tc", c);
		pw.println("\t\tcreateTime=" + fmt);
		Calendar l = Calendar.getInstance();
		l.setTimeInMillis(itm.getLastUseTime());
		Formatter fmt1 = new Formatter();
		fmt1.format("%tc", l);
		pw.println("\t\tlastUseTime=" + fmt1);
		
		pw.flush();
	}

	
	public int getCachedPolicyCount() {
		return cachedPolicyCount;
	}

	public void setCachedPolicyCount(int cachedPolicyCount) {
		this.cachedPolicyCount = cachedPolicyCount;
	}

	public static void main(String[] args) throws MetasetParsingException,
			PolicySyntaxErrorException, DataTypeMappingException,
			IndeterminateException, UnsuportedLocationTypeException,
			IOException, CacheOverflowException {
		String dir = "policies";
		PolicyLoader loader = new PolicyLoader();

		PolicyCache cache = new PolicyCache(loader);

		File file = new File(dir);
		if (file.isDirectory()) {
			File[] policies = file.listFiles();
			for (int i = 0; i < policies.length; i++) {
				File policy = policies[i];
				String path = policy.getAbsolutePath();
				String type = "local_file";
				
				try {
					Thread.sleep(4500);
				} catch (InterruptedException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				
				cache.AddPolicyCacheableItem(new PolicyCacheableItem(path,
						type, cache));
				
				cache.dump_detail(System.out);
				System.out.println("\n\n\n\n\n");
				
			}
		}
	}
}
