/**
 * PolicyLoader.java
 */
package org.act.xservices.rt.xsecurity.xacml.pap;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.logging.Logger;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;


import org.act.xservices.rt.metaset.MetasetBuilder;
import org.act.xservices.rt.metaset.MetasetParser;
import org.act.xservices.rt.metaset.MetasetParserFactory;
import org.act.xservices.rt.metaset.exception.MetasetParsingException;
import org.act.xservices.rt.xom.Metaset;
import org.act.xservices.rt.xsecurity.xacml.context.Request;
import org.act.xservices.rt.xsecurity.xacml.context.Result;
import org.act.xservices.rt.xsecurity.xacml.exception.DataTypeMappingException;
import org.act.xservices.rt.xsecurity.xacml.exception.IndeterminateException;
import org.act.xservices.rt.xsecurity.xacml.exception.PolicySyntaxErrorException;
import org.act.xservices.rt.xsecurity.xacml.exception.UnsuportedLocationTypeException;
import org.act.xservices.rt.xsecurity.xacml.policy.AXacmlPolicy;
import org.act.xservices.rt.xsecurity.xacml.policy.Policy;
import org.act.xservices.rt.xsecurity.xacml.policy.PolicySet;
import org.act.xservices.rt.xsecurity.xacml.util.Constants;
import org.act.xservices.rt.xsecurity.xacml.util.LoggerFactory;
import org.w3c.dom.Document;
import org.xml.sax.ErrorHandler;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

/**
 * 
 */
public class PolicyLoader {

	private static final String LOCAL_FILE = "local_file";

	private static MetasetBuilder builder =MetasetBuilder.getInstance(); 

	private static MetasetParser parser=MetasetParserFactory.getMetasetParser();
	
	private static String POLICY = "Policy";

	private static String POLICY_SET = "PolicySet";

	private static Logger logger = LoggerFactory.getLogger(PolicyLoader.class);

	public AXacmlPolicy loadXacmlPolicy(String location, String locationType)
			throws MetasetParsingException, PolicySyntaxErrorException,
			DataTypeMappingException, IndeterminateException,
			UnsuportedLocationTypeException {
		if (locationType==null || locationType.equals(LOCAL_FILE)) {
		
			if(!validate(location)){
				throw new PolicySyntaxErrorException(Constants.STATUS_UNKNOWNERROR,
						"Validation failed.", null);
			}
			
			Metaset msPolicy = parser.parse(location);
			if (msPolicy.getName().equals(POLICY)) {
				Policy policy = new Policy(msPolicy);
				return policy;
			} else if (msPolicy.getName().equals(POLICY_SET)) {
				PolicySet policySet = new PolicySet(msPolicy);
				return policySet;
			} else {
				throw new PolicySyntaxErrorException(
						Constants.STATUS_SYNTAXERROR,
						"The root element of the policy file is not \"Policy\" or \"PolicySet\"!",
						null);
			}
		} else {
			throw new UnsuportedLocationTypeException(locationType,
					Constants.STATUS_UNKNOWNERROR,
					"Can not process the location type", null);
		}
	}
	
	
	private boolean validate(String loc){
		
		File schemaFile=new File("C:\\HGJ Workspace\\XServices-3.50-091007\\XServices-3.50-091007\\AccessControl\\xacml-2.0-policy.xsd");
		
		 DocumentBuilderFactory factory =
             DocumentBuilderFactory.newInstance();
         factory.setIgnoringComments(true);

         DocumentBuilder db = null;

         // as of 1.2, we always are namespace aware
         factory.setNamespaceAware(true);
         
         factory.setValidating(true);

         factory.setAttribute( "http://java.sun.com/xml/jaxp/properties/schemaLanguage", "http://www.w3.org/2001/XMLSchema");
         factory.setAttribute( "http://java.sun.com/xml/jaxp/properties/schemaSource", schemaFile);
         
         try {
			db = factory.newDocumentBuilder();
		} catch (ParserConfigurationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return false;
		}
         db.setErrorHandler(new MyHandler());
         try {
			Document doc = db.parse(new FileInputStream(loc));
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return false;
		} catch (SAXException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return false;
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return false;
		}

		return true;
	}

	public static void main(String[] args) throws MetasetParsingException,
			PolicySyntaxErrorException, DataTypeMappingException,
			IndeterminateException {

		
		MetasetParser builder =MetasetParserFactory.getMetasetParser();

		Metaset metaset0 = builder.parse(args[0]);
		Policy policy0 = new Policy(metaset0);

		Metaset metasetRequest0 = builder.parse(args[1]);
		Request request0 = new Request(metasetRequest0);

		Result r = policy0.evaluate(request0);

		System.out.println(r.getDecision());
		System.out.println(r.getStatus().getStatusCode().getValue());
	}
}


class MyHandler implements ErrorHandler{

		public void error(SAXParseException exception) throws SAXException {
			// TODO Auto-generated method stub
			exception.printStackTrace();
		}

		public void fatalError(SAXParseException exception) throws SAXException {
			// TODO Auto-generated method stub
			exception.printStackTrace();
		}

		public void warning(SAXParseException exception) throws SAXException {
			// TODO Auto-generated method stub
			exception.printStackTrace();
		}
}
