/**
 * PDP.java
 */
package org.act.xservices.rt.xsecurity.xacml.pdp;

import java.net.URI;
import java.util.logging.Logger;

import org.act.xservices.rt.metaset.exception.MetasetParsingException;
import org.act.xservices.rt.xsecurity.xacml.context.IRequest;
import org.act.xservices.rt.xsecurity.xacml.context.Result;
import org.act.xservices.rt.xsecurity.xacml.exception.CacheOverflowException;
import org.act.xservices.rt.xsecurity.xacml.exception.DataTypeMappingException;
import org.act.xservices.rt.xsecurity.xacml.exception.IndeterminateException;
import org.act.xservices.rt.xsecurity.xacml.exception.PolicySyntaxErrorException;
import org.act.xservices.rt.xsecurity.xacml.exception.UnsuportedLocationTypeException;
import org.act.xservices.rt.xsecurity.xacml.exception.XacmlFunctionInvokeException;
import org.act.xservices.rt.xsecurity.xacml.exception.XacmlFunctionLoadException;
import org.act.xservices.rt.xsecurity.xacml.pap.PolicyCache;
import org.act.xservices.rt.xsecurity.xacml.pdp.function.FunctionRegistry;
import org.act.xservices.rt.xsecurity.xacml.pdp.function.XacmlFunction;
import org.act.xservices.rt.xsecurity.xacml.policy.AXacmlPolicy;
import org.act.xservices.rt.xsecurity.xacml.util.Constants;
import org.act.xservices.rt.xsecurity.xacml.util.LoggerFactory;

/**
 * 
 */
public class PDP {
	
	private static Logger logger=LoggerFactory.getLogger(PDP.class);
	
	private PolicyCache cache=null;
	private URI policyCombAlgID=null;
	private XacmlFunction combAlg=null;
	private URI def_combAlg=Constants.ALG_POLICY_PERMITOVERRIDES;
	
	public PDP(PolicyCache cache,URI policyCombAlg) throws XacmlFunctionLoadException{
		this.cache=cache;
		this.policyCombAlgID=policyCombAlg;
		if(this.policyCombAlgID!=null){
			this.combAlg=FunctionRegistry.getInstance().lookup(this.policyCombAlgID);
		}
		if(this.combAlg==null){
			this.combAlg=FunctionRegistry.getInstance().lookup(this.def_combAlg);
		}
	}

	public Result evaluate(IRequest request) throws MetasetParsingException, PolicySyntaxErrorException, DataTypeMappingException, IndeterminateException, UnsuportedLocationTypeException, CacheOverflowException, XacmlFunctionInvokeException{
		
		
//		long start = System.currentTimeMillis();
		AXacmlPolicy[] policies=this.cache.getXacmlPolicies(request);
//		long end = System.currentTimeMillis();
		
//		System.out.println("Find "+policies.length +"applicable policies, time cost: "+(end-start)+" milliseconds.");
		
		Result r=null;
		
//		start= System.currentTimeMillis();
		r=(Result)this.combAlg.invoke(request, policies);
//		end = System.currentTimeMillis();
		
//		System.out.println("Evaluation time : "+(end-start)+" milliseconds.");
		return r;
	}

	public PolicyCache getCache() {
		return cache;
	}

	public void setCache(PolicyCache cache) {
		this.cache = cache;
	}
}
