package org.act.xservices.rt.xsecurity.xacml.pdp;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.net.URI;
import java.util.ArrayList;
import java.util.List;

import org.act.xservices.rt.metaset.exception.MetasetParsingException;
import org.act.xservices.rt.xom.Metaset;
import org.act.xservices.rt.xsecurity.xacml.context.Decision;
import org.act.xservices.rt.xsecurity.xacml.context.IRequest;
import org.act.xservices.rt.xsecurity.xacml.context.Result;
import org.act.xservices.rt.xsecurity.xacml.exception.CacheOverflowException;
import org.act.xservices.rt.xsecurity.xacml.exception.DataTypeMappingException;
import org.act.xservices.rt.xsecurity.xacml.exception.IndeterminateException;
import org.act.xservices.rt.xsecurity.xacml.exception.PolicySyntaxErrorException;
import org.act.xservices.rt.xsecurity.xacml.exception.UnsuportedLocationTypeException;
import org.act.xservices.rt.xsecurity.xacml.exception.XacmlFunctionLoadException;
import org.act.xservices.rt.xsecurity.xacml.pap.PolicyCache;
import org.act.xservices.rt.xsecurity.xacml.pap.PolicyCacheableItem;
import org.act.xservices.rt.xsecurity.xacml.pap.PolicyLoader;
import org.act.xservices.rt.xsecurity.xacml.util.GlobalMetasetBuilder;


public class PolicyEngine {
	
	private static PolicyEngine instance=null;
	
	private PDP pdp=null;

	private PolicyEngine() {
	
		try {
			
			Metaset config = GlobalMetasetBuilder.builder.parse(new FileInputStream("C:\\HGJ Workspace\\XServices-3.50-091007\\XServices-3.50-091007\\org.act.xservices.rt.xcontainer.container\\conf\\XacmlEngineConfig\\config.xml"));

			
			String policyLibPath=config.filter(".\\PolicyLibPath").car().getText();
//			String policyLibPath = "performance_test1\\size10";//дԼϣ20100624Ϊ̬
			File policyLib=new File(policyLibPath);
			
			List<String> paths=new ArrayList<String>();
			if (policyLib.isDirectory()) {
				File[] policies = policyLib.listFiles();
				for (int i = 0; i < policies.length; i++) {
					File policy = policies[i];
					paths.add(policy.getAbsolutePath());
				}
			}
			
			PolicyCache cache = new PolicyCache(new PolicyLoader());
			for(String path:paths){
				cache.AddPolicyCacheableItem(new PolicyCacheableItem(path,"local_file",cache));
			}
			
			URI policyCombAlg=URI.create(config.filter(".\\PolicyCombAlg").car().getText());
			
			this.pdp=new PDP(cache,policyCombAlg);
			
		} catch (MetasetParsingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (PolicySyntaxErrorException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (DataTypeMappingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IndeterminateException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (UnsuportedLocationTypeException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (CacheOverflowException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (XacmlFunctionLoadException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public static PolicyEngine getInstance(){
		if(instance==null){
			synchronized(PolicyEngine.class){
				if(instance==null){
					instance=new PolicyEngine();
				}
			}
		}
		return instance;
	}
	
	public Result evaluate(IRequest request){
		Result r=null;
		try {
			r=this.pdp.evaluate(request);
		} catch (Exception e) {
			// TODO Auto-generated catch block
			r=new Result();
			r.setDecision(Decision.INDETERMINATE);
		} 
		return r;
	}


	public PDP getPdp() {
		return pdp;
	}

	public void setPdp(PDP pdp) {
		this.pdp = pdp;
	}
	
	
	/**
	 * @param args
	 * @throws FileNotFoundException 
	 * @throws MetasetParsingException 
	 */
	public static void main(String[] args) throws MetasetParsingException, FileNotFoundException {
		// TODO Auto-generated method stub
	}

}
