/**
 * FunctionRegistry.java
 */
package org.act.xservices.rt.xsecurity.xacml.pdp.function;

import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import org.act.xservices.rt.xsecurity.xacml.context.IRequest;
import org.act.xservices.rt.xsecurity.xacml.exception.IndeterminateException;
import org.act.xservices.rt.xsecurity.xacml.exception.XacmlFunctionInvokeException;
import org.act.xservices.rt.xsecurity.xacml.exception.XacmlFunctionLoadException;
import org.act.xservices.rt.xsecurity.xacml.policy.target.AttributeValue;
import org.act.xservices.rt.xsecurity.xacml.util.ClassFileLoader;

/**
 * XACMLעXACML淶ĺԼûԶĺעᵽXACMLעСʱɰʶȡִС
 * 
 * 
 */
public class FunctionRegistry {
	public static String XACML_FUNC_IMPL_PKG = "org.act.xacml.pdp.function.impl";

	private Map<URI, XacmlFunction> registry = null;

	private static FunctionRegistry instance = null;

	private FunctionRegistry() throws XacmlFunctionLoadException {
		this.registry = new HashMap<URI, XacmlFunction>();
		this.init();
	}

	public static synchronized FunctionRegistry getInstance()
			throws XacmlFunctionLoadException {
		if (instance == null) {
			instance = new FunctionRegistry();
		}
		return instance;
	}

	private void init() throws XacmlFunctionLoadException {
		// TODO Auto-generated method stub
		Set<Class<?>> providerSet = ClassFileLoader
				.loadClasses(XACML_FUNC_IMPL_PKG);

		for (Class<?> provider : providerSet) {
			// ֻXacmlFunctionProviderAnnotationǵࡣʵXACML淶ĺûԶĺҲҪøעͱǡ
			if (provider
					.isAnnotationPresent(XacmlFunctionProviderAnnotation.class)) {
				Method[] methods = provider.getMethods();
				for (Method currentMethod : methods) {
					// ѴXacmlFunctionAnnotationǵķװXacmlFunction뵽עСûԶĺҲҪXacmlFunctionAnnotationǡ
					XacmlFunctionAnnotation funcAnn = currentMethod
							.getAnnotation(XacmlFunctionAnnotation.class);
					if (funcAnn != null) {
						if (!Modifier.isStatic(currentMethod.getModifiers())) {
							throw new XacmlFunctionLoadException(null,
									"XACML functions must be static !",null);
						}
						String[] funcURIs = funcAnn.funcURIs();
						if (funcURIs.length == 0) {
							// XACMLûXacmlFunctionAnnotationעṩúURIΪURI
							funcURIs = new String[] { currentMethod.getName() };
						}
						for (String funcURI : funcURIs) {
							if (funcURI != null && funcURI.trim().length() > 0) {

								URI functionId;
								try {
									functionId = new URI(funcURI);
									if (registry.get(functionId) != null) {
										throw new XacmlFunctionLoadException(null,
												"FunctionID conflict, multiple functions have the same ID!",null);
									}
									XacmlFunction function = new XacmlFunction(
											currentMethod, functionId);
									this.register(function);
								} catch (URISyntaxException e) {
									// TODO Auto-generated catch block
									e.printStackTrace();
								}

							}
						}
					}
				}
			}
		}
	}
	public void register(XacmlFunction function) {
		this.registry.put(function.getMethodId(), function);
	}
	
	public XacmlFunction lookup(URI functionId){
		return this.registry.get(functionId);
	}
	//ɾָXACMLɹرɾĺ򣬷null
	public XacmlFunction unRegister(XacmlFunction function){
		return unRegister(function.getMethodId());
	}
	public XacmlFunction unRegister(URI functionId){
		XacmlFunction func=this.lookup(functionId);
		if(func!=null){
			this.registry.remove(functionId);
		}
		return func;
	}
	
	public Map<URI, XacmlFunction> getRegistry() {
		return registry;
	}

	public void setRegistry(Map<URI, XacmlFunction> registry) {
		this.registry = registry;
	}

	public static void main(String[] args) throws XacmlFunctionLoadException, XacmlFunctionInvokeException{
		FunctionRegistry reg=FunctionRegistry.getInstance();
		Map<URI,XacmlFunction> map=reg.getRegistry();
		System.out.println(map.keySet().size());
		for(URI id:map.keySet()){
			System.out.print(id.toString()+" : ");
			XacmlFunction f=reg.lookup(id);
			System.out.println(f.getMethod().getName());
		}
	}
}