/**
 * ArgumentCheckUtil.java
 */
package org.act.xservices.rt.xsecurity.xacml.pdp.function.impl;

import java.net.URI;

import javax.xml.datatype.Duration;
import javax.xml.datatype.XMLGregorianCalendar;

import org.act.xservices.rt.xsecurity.xacml.IExpression;
import org.act.xservices.rt.xsecurity.xacml.exception.IndeterminateException;
import org.act.xservices.rt.xsecurity.xacml.policy.target.AttributeValue;
import org.act.xservices.rt.xsecurity.xacml.util.Constants;

/**
 * 
 */
public class ArgumentCheckUtil {
	public static void checkArguments(Object[] params, int expectedNumber)
			throws IndeterminateException {
		checkNull(params);
		if (expectedNumber > 0 && params.length != expectedNumber) {
			throw new IndeterminateException(Constants.STATUS_PROCESSINGERROR,
					"Incorrect argument number [actualNumber=" + params.length
							+ "; expectedNumber=" + expectedNumber + "]!", null);
		}
		for (Object param : params) {
			if (param != null && !(param instanceof AttributeValue)
					&& !(param instanceof AttributeValue[])) {
				throw new IndeterminateException(
						Constants.STATUS_PROCESSINGERROR,
						"Incorrect argument Type [Only Attribute type and Attribute[] type are legal]!",
						null);
			}
		}
	}

	public static void checkNull(Object[] params) throws IndeterminateException {
		for (Object param : params) {
			checkNull(param);
		}
	}

	public static void checkNull(Object arg) throws IndeterminateException {
		if (arg == null) {
			throw new IndeterminateException(Constants.STATUS_PROCESSINGERROR,
					"Null argument error!", null);
		}
	}

	public static void checkArgumentType(AttributeValue attrVal,
			URI expectedType) throws IndeterminateException {
		URI actualType = attrVal.getDataType();
		if (!actualType.equals(expectedType)) {
			throw new IndeterminateException(Constants.STATUS_PROCESSINGERROR,
					"Incorrect AttributeValue type [actualType=" + actualType
							+ "; expectedType=" + expectedType + "]!", null);
		}
	}

	public static void checkArgumentNumberType(AttributeValue value)
			throws IndeterminateException {
		if (!(value.getValue() instanceof Number)) {
			throw new IndeterminateException(Constants.STATUS_PROCESSINGERROR,
					"Expected a number type, but got a "
							+ value.getValue().getClass().getName(), null);
		}
	}

	public static void checkArgumentExpressionType(Object param)
			throws IndeterminateException {
		if (!(param instanceof IExpression)) {
			throw new IndeterminateException(Constants.STATUS_PROCESSINGERROR,
					"Incorrect type error [ExpectedType=IExpression, actualType="
							+ param.getClass() + "]!", null);
		}
	}

	public static void checkArgumentDateType(AttributeValue attrVal)
			throws IndeterminateException {
		Object actualAttr = attrVal.getValue();
		if (!(actualAttr instanceof XMLGregorianCalendar)) {
			throw new IndeterminateException(Constants.STATUS_PROCESSINGERROR,
					"AttributeValue data type erro [expectedType=XMLGregorianCalendar, actualType="
							+ actualAttr.getClass().getName() + "]!", null);
		}
	}

	public static void checkArgumentDurationType(AttributeValue attrVal)
			throws IndeterminateException {
		Object actualAttr = attrVal.getValue();
		if (!(actualAttr instanceof Duration)) {
			throw new IndeterminateException(Constants.STATUS_PROCESSINGERROR,
					"AttributeValue data type erro [expectedType=Duration, actualType="
							+ actualAttr.getClass().getName() + "]!", null);
		}
	}
}
