/**
 * HighOrderBagFunctions.java
 */
package org.act.xservices.rt.xsecurity.xacml.pdp.function.impl;

import org.act.xservices.rt.xsecurity.xacml.context.IRequest;
import org.act.xservices.rt.xsecurity.xacml.exception.IndeterminateException;
import org.act.xservices.rt.xsecurity.xacml.exception.XacmlFunctionInvokeException;
import org.act.xservices.rt.xsecurity.xacml.pdp.function.XacmlFunction;
import org.act.xservices.rt.xsecurity.xacml.pdp.function.XacmlFunctionAnnotation;
import org.act.xservices.rt.xsecurity.xacml.pdp.function.XacmlFunctionProviderAnnotation;
import org.act.xservices.rt.xsecurity.xacml.policy.rule.Function;
import org.act.xservices.rt.xsecurity.xacml.policy.target.AttributeValue;
import org.act.xservices.rt.xsecurity.xacml.util.Constants;

/**
 * ʵOASIS XACML淶2.0ĸ߽׺߽׺ĵһһ
 * 	Appendix A.3.12 Higher-order bag functions
 * 
 * 
 */
@XacmlFunctionProviderAnnotation
public class HighOrderBagFunctions {
	@XacmlFunctionAnnotation(funcURIs = { "urn:oasis:names:tc:xacml:1.0:function:any-of" })
	public static AttributeValue anyOf(IRequest request, Object[] params)
			throws IndeterminateException {
		ArgumentCheckUtil.checkArguments(params, 3);
		Function func = (Function) params[0];
		AttributeValue o = (AttributeValue) params[1];
		AttributeValue[] bag = (AttributeValue[]) params[2];
		XacmlFunction xacmlFunc = func.getFunction();
		try {
			for (AttributeValue each : bag) {
				AttributeValue evalResult;
				evalResult = (AttributeValue) xacmlFunc.invoke(request,
						new Object[] { o, each });
				if (evalResult == AttributeValue.TRUE) {
					return AttributeValue.TRUE;
				}
			}
			return AttributeValue.FALSE;
		} catch (XacmlFunctionInvokeException e) {
			// TODO Auto-generated catch block
			// e.printStackTrace();
			throw new IndeterminateException(Constants.STATUS_PROCESSINGERROR,
					"", e);
		}
	}

	@XacmlFunctionAnnotation(funcURIs = { "urn:oasis:names:tc:xacml:1.0:function:all-of" })
	public static AttributeValue allOf(IRequest request, Object[] params)
			throws IndeterminateException {
		ArgumentCheckUtil.checkArguments(params, 3);
		Function func = (Function) params[0];
		AttributeValue o = (AttributeValue) params[1];
		AttributeValue[] bag = (AttributeValue[]) params[2];
		XacmlFunction xacmlFunc = func.getFunction();
		try {
			for (AttributeValue each : bag) {
				AttributeValue evalResult;
				evalResult = (AttributeValue) xacmlFunc.invoke(request,
						new Object[] { o, each });
				if (evalResult == AttributeValue.FALSE) {
					return AttributeValue.FALSE;
				}
			}
			return AttributeValue.TRUE;
		} catch (XacmlFunctionInvokeException e) {
			// TODO Auto-generated catch block
			// e.printStackTrace();
			throw new IndeterminateException(Constants.STATUS_PROCESSINGERROR,
					"", e);
		}
	}

	@XacmlFunctionAnnotation(funcURIs = { "urn:oasis:names:tc:xacml:1.0:function:any-of-any" })
	public static AttributeValue anyOfAny(IRequest request,
			Object[] params) throws IndeterminateException {
		ArgumentCheckUtil.checkArguments(params, 3);
		Function func = (Function) params[0];
		AttributeValue[] bag1 = (AttributeValue[]) params[1];
		AttributeValue[] bag2 = (AttributeValue[]) params[2];
		XacmlFunction xacmlFunc = func.getFunction();
		try {
			for (AttributeValue each1 : bag1) {
				for (AttributeValue each2 : bag2) {
					AttributeValue evalResult;
					evalResult = (AttributeValue) xacmlFunc.invoke(request,
							new Object[] { each1, each2 });
					if (evalResult == AttributeValue.TRUE) {
						return AttributeValue.TRUE;
					}
				}
			}
			return AttributeValue.FALSE;
		} catch (XacmlFunctionInvokeException e) {
			// TODO Auto-generated catch block
			throw new IndeterminateException(Constants.STATUS_PROCESSINGERROR,
					"", e);
		}
	}

	@XacmlFunctionAnnotation( funcURIs={ "urn:oasis:names:tc:xacml:1.0:function:all-of-any" })
	public static AttributeValue allOfAny(IRequest request, Object[] params)
			throws IndeterminateException {
		ArgumentCheckUtil.checkArguments(params, 3);
		Function func = (Function) params[0];
		AttributeValue[] bag1 = (AttributeValue[]) params[1];
		AttributeValue[] bag2 = (AttributeValue[]) params[2];
		XacmlFunction xacmlFunc = func.getFunction();

		try {
			for (AttributeValue each1 : bag1) {
				boolean flag = false;
				for (AttributeValue each2 : bag2) {
					AttributeValue evalResult = (AttributeValue) xacmlFunc.invoke(
							request, new Object[] { each1, each2 });
					if (evalResult == AttributeValue.TRUE) {
						flag = true;
						break;
					}
				}
				if (!flag) {
					return AttributeValue.FALSE;
				}
			}
			return AttributeValue.TRUE;
		} catch (XacmlFunctionInvokeException e) {
			// TODO Auto-generated catch block
			throw new IndeterminateException(Constants.STATUS_PROCESSINGERROR,
					"", e);
		}
	}

	@XacmlFunctionAnnotation( funcURIs={ "urn:oasis:names:tc:xacml:1.0:function:any-of-all" })
	public static AttributeValue anyOfAll(IRequest request, Object[] params)
			throws IndeterminateException {
		ArgumentCheckUtil.checkArguments(params, 3);
		Function func = (Function) params[0];
		AttributeValue[] bag1 = (AttributeValue[]) params[1];
		AttributeValue[] bag2 = (AttributeValue[]) params[2];
		XacmlFunction xacmlFunc = func.getFunction();

		try {
			for (AttributeValue each1 : bag1) {
				boolean flag = true;
				for (AttributeValue each2 : bag2) {
					AttributeValue evalResult = (AttributeValue) xacmlFunc.invoke(
							request, new Object[] { each1, each2 });
					if (evalResult == AttributeValue.FALSE) {
						flag = false;
						break;
					}
				}
				if (flag) {
					return AttributeValue.TRUE;
				}
			}
			return AttributeValue.FALSE;
		} catch (XacmlFunctionInvokeException e) {
			// TODO Auto-generated catch block
			throw new IndeterminateException(Constants.STATUS_PROCESSINGERROR,
					"", e);
		}
	}

	@XacmlFunctionAnnotation( funcURIs={ "urn:oasis:names:tc:xacml:1.0:function:all-of-all" })
	public static AttributeValue allOfAll(IRequest request, Object[] params)
			throws IndeterminateException {
		ArgumentCheckUtil.checkArguments(params, 3);
		Function func = (Function) params[0];
		AttributeValue[] bag1 = (AttributeValue[]) params[1];
		AttributeValue[] bag2 = (AttributeValue[]) params[2];
		XacmlFunction xacmlFunc = func.getFunction();

		try {
			for (AttributeValue each1 : bag1) {
				for (AttributeValue each2 : bag2) {
					AttributeValue evalResult = (AttributeValue) xacmlFunc.invoke(
							request, new Object[] { each1, each2 });
					if (evalResult == AttributeValue.FALSE) {
						return AttributeValue.FALSE;
					}
				}
			}
			return AttributeValue.TRUE;
		} catch (XacmlFunctionInvokeException e) {
			// TODO Auto-generated catch block
			throw new IndeterminateException(Constants.STATUS_PROCESSINGERROR,
					"", e);
		}
	}

	@XacmlFunctionAnnotation( funcURIs={ "urn:oasis:names:tc:xacml:1.0:function:map" })
	public static AttributeValue[] map(IRequest request, Object[] params)
			throws IndeterminateException {
		ArgumentCheckUtil.checkArguments(params, 2);
		Function func = (Function) params[0];
		AttributeValue[] bag = (AttributeValue[]) params[1];
		XacmlFunction xacmlFunc = func.getFunction();

		AttributeValue[] result = new AttributeValue[bag.length];
		try {
			for (int i = 0; i < bag.length; i++) {
				result[i] = (AttributeValue) xacmlFunc.invoke(request,
						new Object[] { bag[i] });
			}
			return result;
		} catch (XacmlFunctionInvokeException e) {
			// TODO Auto-generated catch block
			throw new IndeterminateException(Constants.STATUS_PROCESSINGERROR,
					"", e);
		}
	}
}
