/**
 * LogicFunctions.java
 */
package org.act.xservices.rt.xsecurity.xacml.pdp.function.impl;

import java.math.BigInteger;
import java.net.URI;

import org.act.xservices.rt.xsecurity.xacml.IExpression;
import org.act.xservices.rt.xsecurity.xacml.context.IRequest;
import org.act.xservices.rt.xsecurity.xacml.exception.DataTypeMappingException;
import org.act.xservices.rt.xsecurity.xacml.exception.IndeterminateException;
import org.act.xservices.rt.xsecurity.xacml.pdp.function.XacmlFunctionAnnotation;
import org.act.xservices.rt.xsecurity.xacml.pdp.function.XacmlFunctionProviderAnnotation;
import org.act.xservices.rt.xsecurity.xacml.policy.target.AttributeValue;
import org.act.xservices.rt.xsecurity.xacml.util.Constants;

/**
 * ʵOASIS XACML淶2.0߼㺯 Appendix A.3.5 Logical functions
 * 
 * 
 */
@XacmlFunctionProviderAnnotation
public class LogicFunctions {
	@XacmlFunctionAnnotation(funcURIs = { "urn:oasis:names:tc:xacml:1.0:function:or" })
	public static AttributeValue or(IRequest request, Object[] params)
			throws IndeterminateException, DataTypeMappingException{
		if (params != null) {
			ArgumentCheckUtil.checkNull(params);
			for (int i = 0; i < params.length; i++) {
				ArgumentCheckUtil.checkArgumentExpressionType(params[i]);
				AttributeValue result = (AttributeValue) ((IExpression) params[i])
						.evaluate(request);
				if (result == AttributeValue.TRUE) {
					return AttributeValue.TRUE;
				}
			}
		}
		return AttributeValue.FALSE;
	}

	@XacmlFunctionAnnotation(funcURIs = { "urn:oasis:names:tc:xacml:1.0:function:and" })
	public static AttributeValue and(IRequest request, Object[] params)
			throws IndeterminateException, DataTypeMappingException {
		if (params != null) {
			ArgumentCheckUtil.checkNull(params);
			for (int i = 0; i < params.length; i++) {
				ArgumentCheckUtil.checkArgumentExpressionType(params[i]);
				AttributeValue result = (AttributeValue) ((IExpression) params[i])
						.evaluate(request);
				if (result == AttributeValue.FALSE) {
					return AttributeValue.FALSE;
				}
			}
		}
		return AttributeValue.TRUE;
	}

	@XacmlFunctionAnnotation(funcURIs = { "urn:oasis:names:tc:xacml:1.0:function:not" })
	public static AttributeValue not(IRequest request, Object[] params)
			throws IndeterminateException, DataTypeMappingException{
		if (params != null && params.length == 1) {
			ArgumentCheckUtil.checkNull(params);
			ArgumentCheckUtil.checkArgumentExpressionType(params[0]);
			AttributeValue arg = (AttributeValue) ((IExpression) params[0])
					.evaluate(request);
			ArgumentCheckUtil.checkArgumentType(arg, Constants.TYPE_BOOLEAN);
			return arg == AttributeValue.TRUE ? AttributeValue.FALSE
					: AttributeValue.TRUE;
		} else {
			throw new IndeterminateException(
					Constants.STATUS_PROCESSINGERROR,
					"Expected 1 parameter, but got null or more than 1 parameters.",
					null);
		}
	}

	@XacmlFunctionAnnotation(funcURIs = { "urn:oasis:names:tc:xacml:1.0:function:n-of" })
	public static AttributeValue nOf(IRequest request, Object[] params)
			throws IndeterminateException, DataTypeMappingException{
		if (params != null && params.length > 0) {
			ArgumentCheckUtil.checkNull(params);
			ArgumentCheckUtil.checkArgumentExpressionType(params[0]);

			// һʽֵõһ
			AttributeValue arg1 = (AttributeValue) ((IExpression) params[0])
					.evaluate(request);
			ArgumentCheckUtil.checkArgumentType(arg1, Constants.TYPE_INTEGER);
			Object value = arg1.getValue();
			int intArg1 = ((BigInteger) value).intValue();
			// һΪ0ֱӷtrue.
			if (intArg1 == 0) {
				return AttributeValue.TRUE;
			}

			// ʣµıʽСڵһָ׳쳣
			if (intArg1 > params.length - 1) {
				throw new IndeterminateException(
						Constants.STATUS_PROCESSINGERROR, "Expected at least "
								+ intArg1
								+ " boolean expressions, but got only "
								+ (params.length - 1) + ".", null);
			}

			int count = 0;
			for (int i = 1; i < params.length; i++) {
				ArgumentCheckUtil.checkArgumentExpressionType(params[i]);
				// ʽֱָtrueʽ
				AttributeValue result = (AttributeValue) ((IExpression) params[i])
						.evaluate(request);
				if (result == AttributeValue.TRUE) {
					count++;
					if (count == intArg1) {
						return AttributeValue.TRUE;
					}
				}
			}
			return AttributeValue.FALSE;
		} else {
			throw new IndeterminateException(Constants.STATUS_PROCESSINGERROR,
					"Expected at least 1 argument, but we got null or 0.", null);
		}
	}
}
