/**
 * MathFunctions.java
 */
package org.act.xservices.rt.xsecurity.xacml.pdp.function.impl;

import java.net.URI;

import javax.naming.ldap.LdapName;

import org.act.xservices.rt.xsecurity.xacml.context.IRequest;
import org.act.xservices.rt.xsecurity.xacml.exception.IndeterminateException;
import org.act.xservices.rt.xsecurity.xacml.pdp.function.XacmlFunctionAnnotation;
import org.act.xservices.rt.xsecurity.xacml.pdp.function.XacmlFunctionProviderAnnotation;
import org.act.xservices.rt.xsecurity.xacml.pdp.type.RFC822Name;
import org.act.xservices.rt.xsecurity.xacml.policy.target.AttributeValue;
import org.act.xservices.rt.xsecurity.xacml.util.Constants;

/**
 * ʵOASIS XACML淶2.0Ļڹʽƥ亯Լƥ亯
 *  Appendix A.3.13 Regular-expression-based functions
 *  Appendix A.3.14 Special match functions
 * 
 * 
 */
@XacmlFunctionProviderAnnotation
public class MatchFunctions {

	
	/**
	 * OASIS XACML淶2.0  XQuery 1.0 and XPath 2.0 Functions and Operators (http://www.w3.org/TR/xquery-operators/#func-matches)Section 7.6.2
	 * ַƥ亯Ķ塣Unless the metacharacters ^ and $ are used as anchors, the string is considered to match the pattern if any substring 
	 * matches the pattern. But if anchors are used, the anchors must match the start/end of the string (in string mode), or the start/end of a 
	 * line (in multiline mode).һַӴģʽƥ䣬ַģʽƥ䣬ʹԪַ"^""$"
	 * 
	 * @param params
	 * @param type1
	 * @param type2
	 * @return
	 * @throws IndeterminateException
	 */
	private static boolean stringMatch(Object[] params, URI type1, URI type2)
			throws IndeterminateException {
		ArgumentCheckUtil.checkArguments(params, 2);
		AttributeValue pattern = (AttributeValue) params[0];
		AttributeValue toBeMatched = (AttributeValue) params[1];
		ArgumentCheckUtil.checkArgumentType(pattern, type1);
		ArgumentCheckUtil.checkArgumentType(toBeMatched, type2);

		String strPattern = pattern.getValue().toString();
		String source = toBeMatched.getValue().toString();
		if (strPattern.charAt(0) != '^') {
			strPattern = ".*" + strPattern;
		}
		if (strPattern.charAt(strPattern.length() - 1) != '$') {
			strPattern = strPattern + ".*";
		}
		strPattern = strPattern.replaceAll("\\-\\[", "&&[^");

		return source.matches(strPattern);
	}
	
	@XacmlFunctionAnnotation(funcURIs={
        "urn:oasis:names:tc:xacml:1.0:function:string-regexp-match"
    })
    public static AttributeValue stringRegularExpressionMatch(IRequest request, Object[] params)
    throws IndeterminateException {
        return stringMatch(params, Constants.TYPE_STRING, Constants.TYPE_STRING) ? AttributeValue.TRUE : AttributeValue.FALSE;
    }

	@XacmlFunctionAnnotation(funcURIs={
        "urn:oasis:names:tc:xacml:2.0:function:anyURI-regexp-match"
    })
    public static AttributeValue uriRegularExpressionMatch(IRequest request, Object[] params)
    throws IndeterminateException {
        return stringMatch(params, Constants.TYPE_STRING, Constants.TYPE_ANYURI) ? AttributeValue.TRUE : AttributeValue.FALSE;
    }

	@XacmlFunctionAnnotation(funcURIs={
        "urn:oasis:names:tc:xacml:2.0:function:ipAddress-regexp-match"
    })
    public static AttributeValue ipAddressRegularExpressionMatch(IRequest request, Object[] params)
    throws IndeterminateException {
        return stringMatch(params, Constants.TYPE_STRING, Constants.TYPE_IPADDRESS) ? AttributeValue.TRUE : AttributeValue.FALSE;
    }

	@XacmlFunctionAnnotation(funcURIs={
        "urn:oasis:names:tc:xacml:2.0:function:dnsName-regexp-match"
    })
    public static AttributeValue dnsNameRegularExpressionMatch(IRequest request, Object[] params)
    throws IndeterminateException {
        return stringMatch(params, Constants.TYPE_STRING, Constants.TYPE_DNSNAME) ? AttributeValue.TRUE : AttributeValue.FALSE;
    }

	@XacmlFunctionAnnotation(funcURIs={
        "urn:oasis:names:tc:xacml:2.0:function:rfc822Name-regexp-match"
    })
    public static AttributeValue rfc822NameRegularExpressionMatch(IRequest request, Object[] params)
    throws IndeterminateException {
        return stringMatch(params, Constants.TYPE_STRING, Constants.TYPE_RFC822NAME) ? AttributeValue.TRUE : AttributeValue.FALSE;
    }

	@XacmlFunctionAnnotation(funcURIs={
        "urn:oasis:names:tc:xacml:2.0:function:x500Name-regexp-match"
    })
    public static AttributeValue x500NameRegularExpressionMatch(IRequest request, Object[] params)
    throws IndeterminateException {
        return stringMatch(params, Constants.TYPE_STRING, Constants.TYPE_X500NAME) ? AttributeValue.TRUE : AttributeValue.FALSE;
    }

	@XacmlFunctionAnnotation(funcURIs={
        "urn:oasis:names:tc:xacml:1.0:function:x500Name-match"
    })
    public static AttributeValue x500NameMatch(IRequest request, Object[] params)
    throws IndeterminateException {
		ArgumentCheckUtil.checkArguments(params, 2);
        AttributeValue pattern = (AttributeValue)params[0];
        AttributeValue toBeMatched = (AttributeValue)params[1];
        ArgumentCheckUtil.checkArgumentType(pattern, Constants.TYPE_X500NAME);
        ArgumentCheckUtil.checkArgumentType(toBeMatched, Constants.TYPE_X500NAME);

        LdapName x500Pattern = (LdapName)pattern.getValue();
        LdapName x500Source = (LdapName)toBeMatched.getValue();

        return x500Source.startsWith(x500Pattern) ? AttributeValue.TRUE : AttributeValue.FALSE;
    }

	@XacmlFunctionAnnotation(funcURIs={
        "urn:oasis:names:tc:xacml:1.0:function:rfc822Name-match"
    })
    public static AttributeValue rfc822NameMatch(IRequest request, Object[] params)
    throws IndeterminateException {
		ArgumentCheckUtil.checkArguments(params, 2);
        AttributeValue pattern = (AttributeValue)params[0];
        AttributeValue toBeMatched = (AttributeValue)params[1];
        ArgumentCheckUtil.checkArgumentType(pattern, Constants.TYPE_STRING);
        ArgumentCheckUtil.checkArgumentType(toBeMatched, Constants.TYPE_RFC822NAME);

        String strPattern = (String)pattern.getValue();
        RFC822Name rfc822Source = (RFC822Name)toBeMatched.getValue();

        int atIndex = strPattern.indexOf("@");
        if (atIndex > 0) {
            String local = strPattern.substring(0, atIndex);
            String domain = strPattern.substring(atIndex + 1);

            return rfc822Source.getLocalPart().equals(local) &&
                    rfc822Source.getDomain().equalsIgnoreCase(domain) ? AttributeValue.TRUE : AttributeValue.FALSE;
        }
        else if (strPattern.charAt(0) == '.') {
            return rfc822Source.getDomain().endsWith(strPattern.toLowerCase()) ? AttributeValue.TRUE : AttributeValue.FALSE;
        }
        else {
            return rfc822Source.getDomain().equalsIgnoreCase(strPattern) ? AttributeValue.TRUE : AttributeValue.FALSE;
        }
    }
	
	public static void main(String[] args){
		
		String strPattern = "^a*";
		String source = "aa";
		if (strPattern.charAt(0) != '^') {
			strPattern = ".*" + strPattern;
		}
		if (strPattern.charAt(strPattern.length() - 1) != '$') {
			strPattern = strPattern + ".*";
		}
		strPattern = strPattern.replaceAll("\\-\\[", "&&[^");
		
		System.out.println(strPattern);
		System.out.println(source.matches(strPattern));
	}
}
