/**
 * PolicyCombiningAlg.java
 */
package org.act.xservices.rt.xsecurity.xacml.pdp.function.impl;

import org.act.xservices.rt.xsecurity.xacml.context.Decision;
import org.act.xservices.rt.xsecurity.xacml.context.IRequest;
import org.act.xservices.rt.xsecurity.xacml.context.Result;
import org.act.xservices.rt.xsecurity.xacml.context.Status;
import org.act.xservices.rt.xsecurity.xacml.context.StatusCode;
import org.act.xservices.rt.xsecurity.xacml.exception.DataTypeMappingException;
import org.act.xservices.rt.xsecurity.xacml.exception.IndeterminateException;
import org.act.xservices.rt.xsecurity.xacml.pdp.function.XacmlFunctionAnnotation;
import org.act.xservices.rt.xsecurity.xacml.pdp.function.XacmlFunctionProviderAnnotation;
import org.act.xservices.rt.xsecurity.xacml.policy.AXacmlPolicy;

/**
 * ʵOASIS XACML淶2.0ĲԺϲ㷨
 * 	urn:oasis:names:tc:xacml:1.0:policy-combining-algorithm:deny-overrides,
 *	urn:oasis:names:tc:xacml:1.1:policy-combining-algorithm:ordered-deny-overrides
 *	urn:oasis:names:tc:xacml:1.0:policy-combining-algorithm:permit-overrides
 *	urn:oasis:names:tc:xacml:1.1:policy-combining-algorithm:ordered-permit-overrides
 *	urn:oasis:names:tc:xacml:1.0:policy-combining-algorithm:first-applicable
 *	urn:oasis:names:tc:xacml:1.0:policy-combining-algorithm:only-one-applicable
 *	ʵΪģdeny-overridesordered-deny-overridespermit-overridesordered-permit-overrides
 *  Ӧͬһʵ֡
 *  
 * 
 */
@XacmlFunctionProviderAnnotation
public class PolicyCombiningAlgs {
	/**
	 * OASIS XACML淶2.0Դ㷨ʾ
	 * Decision denyOverridesPolicyCombiningAlgorithm(Policy policy[]) {
     *     Boolean atLeastOnePermit = false;
     *     for(i=0 ; i < lengthOf(policy); i ++) {
     *         Decision decision = evaluate(policy[i]);
     *         if (decision == Deny) {
     *             return Deny;
     *         }
     *         if (decision == Permit) {
     *             atLeastOnePermit = true;
     *             continue;
     *         }
     *         if (decision == NotApplicable) {
     *             continue;
     *         }
     *         if (decision == Indeterminate) {
     *             return Deny;
     *         }
     *     }
     *     if (atLeastOnePermit) {
     *         return Permit;
     *     }
     *     return NotApplicable;
     * }
	 * 
	 *  οOASIS XACML淶2.0 Appendix C. Combining algorithms (normative)
	 * 
	 * @param request
	 * @param policies
	 * @return
	 * @throws DataTypeMappingException 
	 * @throws
	 */
	@XacmlFunctionAnnotation(funcURIs={
	"urn:oasis:names:tc:xacml:1.0:policy-combining-algorithm:deny-overrides",
	"urn:oasis:names:tc:xacml:1.1:policy-combining-algorithm:ordered-deny-overrides"
	})
	public static Result denyOverridesPolicyCombAlg(IRequest request, AXacmlPolicy[] policies) throws DataTypeMappingException{
		Result result = new Result();
		result.setResourceId(request.getResourceId());
		
		Boolean atLeastOnePermit=false;
		
		for(int i=0;i<policies.length;i++){
			Result r=policies[i].evaluate(request);
			Decision decision=r.getDecision();
			if(decision==Decision.DENY){
				result.setDecision(Decision.DENY);
				Status sts=new Status(StatusCode.STATUS_OK,"In the denyOverridesPolicyCombAlg: A policy evaluates to Deny!");
				result.setStatus(sts);
				return result;
			}
			if(decision==Decision.PERMIT){
				atLeastOnePermit=true;
				continue;
			}
			if(decision==Decision.NOTAPPLICABLE){
				continue;
			}
			if(decision==Decision.INDETERMINATE){
				result.setDecision(Decision.DENY);
				Status sts=new Status(r.getStatus().getStatusCode(),"In the denyOverridesPolicyCombAlg: A policy evaluates to Indeterminate!");
				result.setStatus(sts);
				return result;
			}
		}
		if(atLeastOnePermit){
			result.setDecision(Decision.PERMIT);
			Status sts=new Status(StatusCode.STATUS_OK,"In the denyOverridesPolicyCombAlg: At least one policy evaluates to Permit and others NotApplicable!");
			result.setStatus(sts);
			return result;
		}
		result.setDecision(Decision.NOTAPPLICABLE);
		Status sts=new Status(StatusCode.STATUS_OK,"In the denyOverridesPolicyCombAlg: All policies evaluate to NotApplicable!");
		result.setStatus(sts);
		return result;
	}
	
	/**
	 * OASIS XACML淶2.0Դ㷨ʾ
	 * Decision permitOverridesPolicyCombiningAlgorithm(Policy policy[]) {
     *     Boolean atLeastOneError = false;
     *     Boolean atLeastOneDeny  = false;
     *     for (i = 0; i < lengthOf(policy); i ++) {
     *         Decision decision = evaluate(policy[i]);
     *         if (decision == Deny) {
     *             atLeastOneDeny = true;
     *             continue;
     *         }
     *         if (decision == Permit) {
     *             return Permit;
     *         }
     *         if (decision == NotApplicable) {
     *             continue;
     *         }
     *         if (decision == Indeterminate) {
     *             atLeastOneError = true;
     *             continue;
     *         }
     *     }
     *     if (atLeastOneDeny) {
     *         return Deny;
     *     }
     *     if (atLeastOneError) {
     *         return Indeterminate;
     *     }
     *     return NotApplicable;
     * }
     * 
     *  οOASIS XACML淶2.0 Appendix C. Combining algorithms (normative)
     * 
	 * @param request
	 * @param policies
	 * @return
	 * @throws DataTypeMappingException 
	 * @throws
	 */
	@XacmlFunctionAnnotation(funcURIs={
	"urn:oasis:names:tc:xacml:1.0:policy-combining-algorithm:permit-overrides",
	"urn:oasis:names:tc:xacml:1.1:policy-combining-algorithm:ordered-permit-overrides"	
	})
	public static Result permitOverridesPolicyCombAlg(IRequest request, AXacmlPolicy[] policies) throws DataTypeMappingException{
		Result result = new Result();
		result.setResourceId(request.getResourceId());
		
		Boolean atLeastOneError=false;
		Boolean atLeastOneDeny=false;
		
		Status errorStatus=null;
		
		for(int i=0;i<policies.length;i++){
			Result r=policies[i].evaluate(request);
			Decision decision=r.getDecision();
			if(decision==Decision.DENY){
				atLeastOneDeny=true;
				continue;
			}
			if(decision==Decision.PERMIT){
				result.setDecision(Decision.PERMIT);
				Status sts=new Status(StatusCode.STATUS_OK,"In the permitOverridesPolicyCombAlg: A policy evaluates to Permit!");
				result.setStatus(sts);
				return result;
			}
			if(decision==Decision.NOTAPPLICABLE){
				continue;
			}
			if(decision==Decision.INDETERMINATE){
				atLeastOneError=true;
				errorStatus=r.getStatus();
				continue;
			}
		}
		if(atLeastOneDeny){
			result.setDecision(Decision.DENY);
			Status sts=new Status(StatusCode.STATUS_OK,"In the permitOverridesPolicyCombAlg: At least one policy evaluates to Deny, others NotApplicable!");
			result.setStatus(sts);
			return result;
		}
		if(atLeastOneError){
			result.setDecision(Decision.INDETERMINATE);
			Status sts=new Status(errorStatus.getStatusCode(),"In the permitOverridesPolicyCombAlg: At least one policy evaluates to Indeterminate!");
			result.setStatus(sts);
			return result;
		}
		result.setDecision(Decision.NOTAPPLICABLE);
		Status sts=new Status(StatusCode.STATUS_OK,"In the permitOverridesPolicyCombAlg: All policies evaluate to NotApplicable!");
		result.setStatus(sts);
		return result;
	}
	
	/**
	 * OASIS XACML淶2.0Դ㷨ʾ
	 * Decision firstApplicableEffectPolicyCombiningAlgorithm(Policy policy[]) {
     *     for (i = 0; i < lengthOf(policy); i ++) {
     *         Decision decision = evaluate(policy[i]);
     *         if(decision == Deny) {
     *             return Deny;
     *         }
     *         if(decision == Permit) {
     *             return Permit;
     *         }
     *         if (decision == NotApplicable) {
     *             continue;
     *         }
     *         if (decision == Indeterminate) {
     *             return Indeterminate;
     *         }
     *     }
     *     return NotApplicable;
     * }
     *  οOASIS XACML淶2.0 Appendix C. Combining algorithms (normative)
     * 
	 * @param request
	 * @param policies
	 * @return
	 * @throws DataTypeMappingException 
	 * @throws
	 */
	@XacmlFunctionAnnotation(funcURIs={
	"urn:oasis:names:tc:xacml:1.0:policy-combining-algorithm:first-applicable"
	})
	public static Result firstApplicablePolicyCombAlg(IRequest request, AXacmlPolicy[] policies) throws DataTypeMappingException{
		Result result = new Result();
		result.setResourceId(request.getResourceId());
		for(int i=0;i<policies.length;i++){
			Result r=policies[i].evaluate(request);
			Decision decision=r.getDecision();
			if(decision ==Decision.DENY){
				result.setDecision(Decision.DENY);
				Status sts=new Status(StatusCode.STATUS_OK,"In the firstApplicablePolicyCombAlg: A policy evaluates to Deny!");
				result.setStatus(sts);
				return result;
			}
			if(decision==Decision.PERMIT){
				result.setDecision(Decision.PERMIT);
				Status sts=new Status(StatusCode.STATUS_OK,"In the firstApplicablePolicyCombAlg: A policy evaluates to Permit!");
				result.setStatus(sts);
				return result;
			}
			if(decision==Decision.NOTAPPLICABLE){
				continue;
			}
			if(decision==Decision.INDETERMINATE){
				result.setDecision(Decision.INDETERMINATE);
				Status sts=new Status(r.getStatus().getStatusCode(),"In the firstApplicablePolicyCombAlg: A policy evaluates to Indeterminate!");
				result.setStatus(sts);
				return result;
			}
		}
		result.setDecision(Decision.NOTAPPLICABLE);
		Status sts=new Status(StatusCode.STATUS_OK,"In the firstApplicablePolicyCombAlg: All policies evaluate to NotApplicable!");
		result.setStatus(sts);
		return result;
	}
	
	/**
	 * OASIS XACML淶2.0Դ㷨ʾ
	 * Decision onlyOneApplicablePolicyPolicyCombiningAlogrithm(Policy policy[]) {
     *   Boolean          atLeastOne     = false;
     *   Policy           selectedPolicy = null;
     *   ApplicableResult appResult;
     * 
     *   for (i = 0; i < lengthOf(policy); i ++) {
     *      appResult = isApplicable(policy[I]);
     * 
     *      if (appResult == Indeterminate) {
     *          return Indeterminate;
     *      }
     *      if(appResult == Applicable) {
     *          if (atLeastOne) {
     *              return Indeterminate;
     *          }
     *          else {
     *              atLeastOne     = true;
     *              selectedPolicy = policy[i];
     *          }
     *      }
     *      if (appResult == NotApplicable) {
     *          continue;
     *      }
     *   }
     *   if (atLeastOne) {
     *       return evaluate(selectedPolicy);
     *   }
     *   else {
     *       return NotApplicable;
     *   }
     * }
     * 
     * οOASIS XACML淶2.0 Appendix C. Combining algorithms (normative)
	 * 
	 * @param request
	 * @param policies
	 * @return
	 * @throws DataTypeMappingException 
	 * @throws AttributeMissingException 
	 * @throws
	 */
	@XacmlFunctionAnnotation(funcURIs={
	"urn:oasis:names:tc:xacml:1.0:policy-combining-algorithm:only-one-applicable"
	})
	public static Result onlyOneApplicablePolicyCombAlg(IRequest request,
			AXacmlPolicy[] policies) throws DataTypeMappingException{
		
		Result result=new Result();
		result.setResourceId(request.getResourceId());
		
		Boolean atLeastOne=false;
		AXacmlPolicy selectedPolicy=null;
		
		Boolean appResult;
		
		for(int i=0;i<policies.length;i++){
			try {
				appResult=policies[i].match(request);
				if(appResult){
					if(atLeastOne){
						result.setDecision(Decision.INDETERMINATE);
						Status sts=new Status(StatusCode.STATUS_PROCESSING_ERROR,"In the onlyOneApplicablePolicyCombAlg: More than one policies applicable!");
						result.setStatus(sts);
						return result;
					}
					else{
						atLeastOne=true;
						selectedPolicy=policies[i];
					}
				}
			} catch (IndeterminateException e) {
				// TODO Auto-generated catch block
				result.setDecision(Decision.INDETERMINATE);
				StatusCode code=StatusCode.newStatusCode(e, null);
				Status sts=new Status(code,"In the onlyOneApplicablePolicyCombAlg: The target of a policy evaluates to Indeterminate!");
				result.setStatus(sts);
				return result;
			}
		}
		if(atLeastOne){
			return selectedPolicy.evaluate(request);
		}
		result.setDecision(Decision.NOTAPPLICABLE);
		Status sts=new Status(StatusCode.STATUS_OK,"In the onlyOneApplicablePolicyCombAlg: All policies evaluate to NotApplicable!");
		result.setStatus(sts);
		return result;
	}
}
