/**
 * CombiningAlg.java
 */
package org.act.xservices.rt.xsecurity.xacml.pdp.function.impl;

import org.act.xservices.rt.xsecurity.xacml.context.Decision;
import org.act.xservices.rt.xsecurity.xacml.context.IRequest;
import org.act.xservices.rt.xsecurity.xacml.context.Result;
import org.act.xservices.rt.xsecurity.xacml.context.Status;
import org.act.xservices.rt.xsecurity.xacml.context.StatusCode;
import org.act.xservices.rt.xsecurity.xacml.exception.DataTypeMappingException;
import org.act.xservices.rt.xsecurity.xacml.pdp.function.XacmlFunctionAnnotation;
import org.act.xservices.rt.xsecurity.xacml.pdp.function.XacmlFunctionProviderAnnotation;
import org.act.xservices.rt.xsecurity.xacml.policy.rule.Effect;
import org.act.xservices.rt.xsecurity.xacml.policy.rule.Rule;
import org.act.xservices.rt.xsecurity.xacml.util.Constants;

/**
 * ʵOASIS XACML淶2.0Ĺϲ㷨
 *  urn:oasis:names:tc:xacml:1.0:rule-combining-algorithm:deny-overrides
 *  urn:oasis:names:tc:xacml:1.1:rule-combining-algorithm:ordered-deny-overrides
 *  urn:oasis:names:tc:xacml:1.0:rule-combining-algorithm:permit-overrides
 *  urn:oasis:names:tc:xacml:1.1:rule-combining-algorithm:ordered-permit-overrides
 *  urn:oasis:names:tc:xacml:1.0:rule-combining-algorithm:first-applicable
 *  ʵΪģdeny-overridesordered-deny-overridespermit-overridesordered-permit-overrides
 *  Ӧͬһʵ֡
 *  
 * 
 */
@XacmlFunctionProviderAnnotation
public class RuleCombiningAlgs {
	/**
	 * OASIS XACML淶2.0Դ㷨ʾ
	 * Decision denyOverridesRuleCombiningAlgorithm(Rule rule[]) {
     *     Boolean atLeastOneError  = false;
     *     Boolean potentialDeny    = false;
     *     Boolean atLeastOnePermit = false;
     *     for (i = 0; i < lengthOf(rules); i ++) {
     *         Decision decision = evaluate(rule[i]);
     *         if (decision == Deny) {
     *             return Deny;
     *         }
     *         if (decision == Permit) {
     *             atLeastOnePermit = true;
     *             continue;
     *         }
     *         if (decision == NotApplicable) {
     *             continue;
     *         }
     *         if (decision == Indeterminate) {
     *             atLeastOneError = true;
     *  
     *             if (effect(rule[i]) == Deny) {
     *                 potentialDeny = true;
     *             }
     *             continue;
     *         }
     *     }
     *     if (potentialDeny) {
     *         return Indeterminate;
     *     }
     *     if (atLeastOnePermit) {
     *         return Permit;
     *     }
     *     if (atLeastOneError) {
     *         return Indeterminate;
     *     }
     *     return NotApplicable;
     * }
     * 
     *  οOASIS XACML淶2.0 Appendix C. Combining algorithms (normative)
	 * @param request
	 * @param params
	 * @throws DataTypeMappingException 
	 * @throws
	 * @return
	 */
	@XacmlFunctionAnnotation(funcURIs = {
		"urn:oasis:names:tc:xacml:1.0:rule-combining-algorithm:deny-overrides",
		"urn:oasis:names:tc:xacml:1.1:rule-combining-algorithm:ordered-deny-overrides" 
	})
	public static Result denyOverridesRuleCombAlg(IRequest request,
			Rule[] rules) throws DataTypeMappingException {
		
		Result result = new Result();
		result.setResourceId(request.getResourceId());
		
		Boolean atLeastOneError = false;
		Boolean potentialDeny = false;
		Boolean atLeastOnePermit = false;

		Status errorStatus=null;
		
		for (int i = 0; i < rules.length; i++) {
			Result r = rules[i].evaluate(request);
			Decision decision = r.getDecision();
			if (decision == Decision.DENY) {
				result.setDecision(Decision.INDETERMINATE);
				Status sts=new Status(StatusCode.STATUS_OK,"In the denyOverridesRuleCombAlg: A rule evaluates to Deny!");
				result.setStatus(sts);
				result.setDecision(Decision.DENY);
				return result;
			}
			if (decision == Decision.PERMIT) {
				atLeastOnePermit = true;
				continue;
			}
			if (decision == Decision.NOTAPPLICABLE) {
				continue;
			}
			if (decision == Decision.INDETERMINATE) {
				errorStatus=r.getStatus();
				atLeastOneError = true;
				if (rules[i].getEffect() == Effect.DENY) {
					potentialDeny = true;
				}
				continue;
			}
		}
		
		if (potentialDeny) {
			result.setDecision(Decision.INDETERMINATE);
			Status sts=new Status(errorStatus.getStatusCode(),"In the denyOverridesRuleCombAlg: A potential deny rule evaluates to Indeterminate!");
			result.setStatus(sts);
			return result;
		}
		if (atLeastOnePermit) {
			result.setDecision(Decision.PERMIT);
			result.setStatus(new Status(StatusCode.STATUS_OK,"In the denyOverridesRuleCombAlg: At least one rule evaluates to Permit, all others evaluate to NotApplicable!"));
			return result;
		}
		if (atLeastOneError) {
			result.setDecision(Decision.INDETERMINATE);
			Status sts=new Status(errorStatus.getStatusCode(),"In the denyOverridesRuleCombAlg: At least one rule evaluates to Indeterminate!");
			result.setStatus(sts);
			return result;
		}
		result.setDecision(Decision.NOTAPPLICABLE);
		Status sts=new Status(StatusCode.STATUS_OK,"In the denyOverridesRuleCombAlg: All rules evaluate to NotApplicable!");
		result.setStatus(sts);
		return result;
	}
	
	
	/**
	 * OASIS XACML淶2.0Դ㷨ʾ
	 * Decision permitOverridesRuleCombiningAlgorithm(Rule rule[]) {
     *     Boolean atLeastOneError  = false;
     *     Boolean potentialPermit  = false;
     *     Boolean atLeastOneDeny   = false;
     *     for (i=0 ; i < lengthOf(rule) ; i ++) {
     *         Decision decision = evaluate(rule[i]);
     *         if (decision == Deny) {
     *             atLeastOneDeny = true;
     *             continue;
     *         }
     *         if (decision == Permit) {
     *             return Permit;
     *         }
     *         if (decision == NotApplicable) {
     *             continue;
     *         }
     *         if (decision == Indeterminate) {
     *             atLeastOneError = true;
     * 
     *             if (effect(rule[i]) == Permit) {
     *                 potentialPermit = true;
     *             }
     *             continue;
     *         }
     *     }
     *     if (potentialPermit) {
     *         return Indeterminate;
     *     }
     *     if (atLeastOneDeny) {
     *         return Deny;
     *     }
     *     if (atLeastOneError) {
     *         return Indeterminate;
     *     }
     *     return NotApplicable;
     * }
     * 
     *  οOASIS XACML淶2.0 Appendix C. Combining algorithms (normative)
     * 
	 * @param request
	 * @param rules
	 * @return
	 * @throws DataTypeMappingException 
	 * @throws
	 */
	@XacmlFunctionAnnotation(funcURIs={
		"urn:oasis:names:tc:xacml:1.0:rule-combining-algorithm:permit-overrides",
		"urn:oasis:names:tc:xacml:1.1:rule-combining-algorithm:ordered-permit-overrides"
	})
	public static Result permitOverridesRuleCombAlg(IRequest request,
			Rule[] rules) throws DataTypeMappingException { 
		
		Result result = new Result();
		result.setResourceId(request.getResourceId());
		
		Boolean atLeastOneError=false;
		Boolean potentialPermit=false;
		Boolean atLeastOneDeny=false;
		
		Status errorStatus=null;
		
		for(int i=0;i<rules.length;i++){
			Result r=rules[i].evaluate(request);
			Decision decision=r.getDecision();
			if(decision==Decision.DENY){
				atLeastOneDeny=true;
				continue;
			}
			if(decision==Decision.PERMIT){
				result.setDecision(Decision.PERMIT);
				Status sts=new Status(StatusCode.STATUS_OK,"In the permitOverridesRuleCombAlg: A rule evaluates to Permit!");
				result.setStatus(sts);
				return result;
			}
			if(decision==Decision.NOTAPPLICABLE){
				continue;
			}
			if (decision == Decision.INDETERMINATE) { 
				 atLeastOneError = true;
				 errorStatus=r.getStatus();
			     if (rules[i].getEffect()==Effect.PERMIT) {
			    	 potentialPermit = true;
			     }
			     continue;
			}
		}
		if(potentialPermit){
			result.setDecision(Decision.INDETERMINATE);
			Status sts=new Status(errorStatus.getStatusCode(),"In the permitOverridesRuleCombAlg: A potential permit rule evaluates to Indeterminate!");
			result.setStatus(sts);
			return result;
		}
		if(atLeastOneDeny){
			result.setDecision(Decision.DENY);
			result.setStatus(new Status(StatusCode.STATUS_OK,"In the permitOverridesRuleCombAlg: At least one rule evaluates to Deny, all others evaluate to NotApplicable!"));
			return result;
		}
		if(atLeastOneError){
			result.setDecision(Decision.INDETERMINATE);
			Status sts=new Status(errorStatus.getStatusCode(),"In the permitOverridesRuleCombAlg: At least one rule evaluates to Indeterminate!");
			result.setStatus(sts);
			return result;
		}
		result.setDecision(Decision.NOTAPPLICABLE);
		Status sts=new Status(StatusCode.STATUS_OK,"In the permitOverridesRuleCombAlg: All rules evaluate to NotApplicable!");
		result.setStatus(sts);
		return result;
	}
	/**
	 * OASIS XACML淶2.0Դ㷨ʾ
	 * Decision firstApplicableEffectRuleCombiningAlgorithm(Rule rule[]) {
     *     for (i = 0 ; i < lengthOf(rule); i ++) {
     *         Decision decision = evaluate(rule[i]);
     *         if (decision == Deny) {
     *             return Deny;
     *         }
     *         if (decision == Permit) {
     *             return Permit;
     *         }
     *         if (decision == NotApplicable) {
     *             continue;
     *         }
     *         if (decision == Indeterminate) {
     *             return Indeterminate;
     *         }
     *     }
     *     return NotApplicable;
     * }
     * 
     *  οOASIS XACML淶2.0 Appendix C. Combining algorithms (normative)
     * 
	 * @param request
	 * @param rules
	 * @return
	 * @throws DataTypeMappingException 
	 * @throws
	 */
	@XacmlFunctionAnnotation(funcURIs={
		"urn:oasis:names:tc:xacml:1.0:rule-combining-algorithm:first-applicable"
	})
	public static Result firstApplicableRuleCombAlg(IRequest request,
			Rule[] rules) throws DataTypeMappingException{
		Result result = new Result();
		result.setResourceId(request.getResourceId());
		
		for(int i=0;i<rules.length;i++){
			Result r=rules[i].evaluate(request);
			Decision decision=r.getDecision();
			if(decision == Decision.DENY){
				result.setDecision(Decision.DENY);
				Status sts=new Status(StatusCode.STATUS_OK,"In the firstApplicableRuleCombAlg: A rule evaluates to Deny!");
				result.setStatus(sts);
				return result;
			}
			if(decision == Decision.PERMIT){
				result.setDecision(Decision.PERMIT);
				Status sts=new Status(StatusCode.STATUS_OK,"In the firstApplicableRuleCombAlg: A rule evaluates to Permit!");
				result.setStatus(sts);
				return result;
			}
			if(decision == Decision.NOTAPPLICABLE){
				continue;
			}
			if(decision == Decision.INDETERMINATE){
				result.setDecision(Decision.INDETERMINATE);
				Status sts=new Status(r.getStatus().getStatusCode(),"In the firstApplicableRuleCombAlg: A rule evaluates to Indeterminate!");
				result.setStatus(sts);
				return result;
			}
		}
		result.setDecision(Decision.NOTAPPLICABLE);
		Status sts=new Status(StatusCode.STATUS_OK,"In the firstApplicableRuleCombAlg: All rules evaluate to NotApplicable!");
		result.setStatus(sts);
		return result;
	}
}
