/**
 * StringFunctions.java
 */
package org.act.xservices.rt.xsecurity.xacml.pdp.function.impl;

import java.net.URI;

import org.act.xservices.rt.xsecurity.xacml.context.IRequest;
import org.act.xservices.rt.xsecurity.xacml.exception.IndeterminateException;
import org.act.xservices.rt.xsecurity.xacml.pdp.function.XacmlFunctionAnnotation;
import org.act.xservices.rt.xsecurity.xacml.pdp.function.XacmlFunctionProviderAnnotation;
import org.act.xservices.rt.xsecurity.xacml.policy.target.AttributeValue;
import org.act.xservices.rt.xsecurity.xacml.util.Constants;

/**
 * ʵOASIS XACML淶2.0ַصĺ
 * 	Appendix A.3.3 String conversion functions
 *  Appendix A.3.8 Non-numeric comparison functionsеַȽ
 * 	Appendix A.3.9 String functions
 * 
 */
@XacmlFunctionProviderAnnotation
public class StringFunctions {
	@XacmlFunctionAnnotation(funcURIs = { "urn:oasis:names:tc:xacml:1.0:function:string-normalize-space" })
	public static AttributeValue trim(IRequest rquest, Object[] params)
			throws IndeterminateException {
		ArgumentCheckUtil.checkArguments(params, 1);
		AttributeValue value = (AttributeValue) params[0];
		ArgumentCheckUtil.checkArgumentType(value, Constants.TYPE_STRING);

		try {
			String strValue = (String) value.getValue();
			return AttributeValue.newInstance(value.getDataType(), strValue
					.trim());
		} catch (Exception e) {
			throw new IndeterminateException(Constants.STATUS_PROCESSINGERROR,
					"Error occurs in string-normalize-space.", e);
		}
	}

	@XacmlFunctionAnnotation(funcURIs = { "urn:oasis:names:tc:xacml:1.0:function:string-normalize-to-lower-case" })
	public static AttributeValue toLowerCase(IRequest rquest,
			Object[] params) throws IndeterminateException {
		ArgumentCheckUtil.checkArguments(params, 1);
		AttributeValue value = (AttributeValue) params[0];
		ArgumentCheckUtil.checkArgumentType(value, Constants.TYPE_STRING);

		try {
			String strValue = (String) value.getValue();
			return AttributeValue.newInstance(value.getDataType(), strValue
					.toLowerCase());
		} catch (Exception e) {
			throw new IndeterminateException(Constants.STATUS_PROCESSINGERROR,
					"Error occurs in string-normalize-to-lower-case.", e);
		}
	}

	@XacmlFunctionAnnotation(funcURIs = { "urn:oasis:names:tc:xacml:1.0:function:string-greater-than" })
	public static AttributeValue stringGreaterThan(IRequest rquest,
			Object[] params) throws IndeterminateException {
		return compareString(params) > 0 ? AttributeValue.TRUE
				: AttributeValue.FALSE;
	}

	@XacmlFunctionAnnotation(funcURIs = { "urn:oasis:names:tc:xacml:1.0:function:string-greater-than-or-equal" })
	public static AttributeValue stringGreaterThanOrEqual(
			IRequest rquest, Object[] params)
			throws IndeterminateException {
		return compareString(params) >= 0 ? AttributeValue.TRUE
				: AttributeValue.FALSE;
	}

	@XacmlFunctionAnnotation(funcURIs = { "urn:oasis:names:tc:xacml:1.0:function:string-less-than" })
	public static AttributeValue stringLessThan(IRequest rquest,
			Object[] params) throws IndeterminateException {
		return compareString(params) < 0 ? AttributeValue.TRUE
				: AttributeValue.FALSE;
	}

	@XacmlFunctionAnnotation(funcURIs = { "urn:oasis:names:tc:xacml:1.0:function:string-less-than-or-equal" })
	public static AttributeValue stringLessThanOrEqual(IRequest rquest,
			Object[] params) throws IndeterminateException {
		return compareString(params) <= 0 ? AttributeValue.TRUE
				: AttributeValue.FALSE;
	}

	@XacmlFunctionAnnotation(funcURIs = { "urn:oasis:names:tc:xacml:2.0:function:string-concatenate" })
	public static AttributeValue stringConcat(IRequest rquest,
			Object[] params) throws IndeterminateException {
		ArgumentCheckUtil.checkArguments(params, -1);
		if (params.length < 2) {
			throw new IndeterminateException(Constants.STATUS_PROCESSINGERROR,
					"Invalid argument number. Expected 2 or more parameters, but got "
							+ params.length, null);
		}
		AttributeValue s1 = (AttributeValue) params[0];
		ArgumentCheckUtil.checkArgumentType(s1, Constants.TYPE_STRING);

		try {
			StringBuffer buf = new StringBuffer();
			buf.append((String) s1.getValue());
			for (int i = 1; i < params.length; i++) {
				AttributeValue s = (AttributeValue) params[i];
				ArgumentCheckUtil.checkArgumentType(s, Constants.TYPE_STRING);
				buf.append((String) s.getValue());
			}
			return AttributeValue.newInstance(s1.getDataType(), buf.toString());
		} catch (Exception e) {
			throw new IndeterminateException(Constants.STATUS_PROCESSINGERROR,
					"Error occurs in string-concatenate.", e);
		}
	}

	@XacmlFunctionAnnotation(funcURIs = { "urn:oasis:names:tc:xacml:2.0:function:uri-string-concatenate" })
	public static AttributeValue uriConcat(IRequest rquest,
			Object[] params) throws IndeterminateException {
		ArgumentCheckUtil.checkArguments(params, -1);
		if (params.length < 2) {
			throw new IndeterminateException(Constants.STATUS_PROCESSINGERROR,
					"Invalid argument number. Expected 2 or more parameters, but got "
							+ params.length, null);
		}
		AttributeValue s1 = (AttributeValue) params[0];
		ArgumentCheckUtil.checkArgumentType(s1, Constants.TYPE_ANYURI);

		try {
			StringBuffer buf = new StringBuffer();
			buf.append(s1.getValue().toString());
			for (int i = 1; i < params.length; i++) {
				AttributeValue s = (AttributeValue) params[i];
				ArgumentCheckUtil.checkArgumentType(s, Constants.TYPE_STRING);
				buf.append((String) s.getValue());
			}
			return AttributeValue.newInstance(s1.getDataType(), buf.toString());
		} catch (Exception e) {
			throw new IndeterminateException(Constants.STATUS_PROCESSINGERROR,
					"Error occurs in uri-string-concatenate.", e);
		}
	}

	private static int compareString(Object[] params)
			throws IndeterminateException {
		ArgumentCheckUtil.checkArguments(params, 2);
		AttributeValue s1 = (AttributeValue) params[0];
		AttributeValue s2 = (AttributeValue) params[1];
		ArgumentCheckUtil.checkArgumentType(s1, Constants.TYPE_STRING);
		ArgumentCheckUtil.checkArgumentType(s2, Constants.TYPE_STRING);
		String a = (String) s1.getValue();
		String b = (String) s2.getValue();
		return a.compareTo(b);
	}
}
