/**
 * RFC822Name.java
 */
package org.act.xservices.rt.xsecurity.xacml.pdp.type;

import org.act.xservices.rt.xsecurity.xacml.exception.InvalidDataFormatException;
import org.act.xservices.rt.xsecurity.xacml.util.Constants;

/**
 * 
 */
public class RFC822Name {

	private String localPart;

	private String domain;

	private String address;

	private int hashCode;

	public RFC822Name(String value) throws InvalidDataFormatException {
		if (value != null && value.length() > 0) {
			String[] parts = value.split("@");
			if (parts.length == 2) {
				localPart = parts[0];
				domain = parts[1].toLowerCase();
				address = localPart + "@" + domain;
				hashCode = address.hashCode();
			}
		} else {
			throw new InvalidDataFormatException(
					Constants.STATUS_PROCESSINGERROR, "The value '" + value
							+ "' is not an valid rfc822 name.", null);
		}
	}

	public String getLocalPart() {
		return localPart;
	}

	public String getDomain() {
		return domain;
	}

	public String toString() {
		return address;
	}

	public boolean equals(Object o) {
		if (this == o) {
			return true;
		}
		if (o != null && o.getClass() == this.getClass()) {
			return localPart.equals(((RFC822Name) o).getLocalPart())
					&& domain.equalsIgnoreCase(((RFC822Name) o).getDomain());
		}
		return false;
	}

	public int hashCode() {
		return hashCode;
	}

	public static RFC822Name valueOf(String value)
			throws InvalidDataFormatException {
		return new RFC822Name(value);
	}
}
