/**
 * AXacmlPolicy.java
 */
package org.act.xservices.rt.xsecurity.xacml.policy;

import java.net.URI;
import java.util.HashMap;
import java.util.Map;

import org.act.xservices.rt.xom.Attribute;
import org.act.xservices.rt.xom.Metaset;
import org.act.xservices.rt.xsecurity.xacml.AXacmlElement;
import org.act.xservices.rt.xsecurity.xacml.IEvaluateable;
import org.act.xservices.rt.xsecurity.xacml.IMatchable;
import org.act.xservices.rt.xsecurity.xacml.context.IRequest;
import org.act.xservices.rt.xsecurity.xacml.context.Result;
import org.act.xservices.rt.xsecurity.xacml.exception.DataTypeMappingException;
import org.act.xservices.rt.xsecurity.xacml.exception.IndeterminateException;
import org.act.xservices.rt.xsecurity.xacml.exception.PolicySyntaxErrorException;
import org.act.xservices.rt.xsecurity.xacml.policy.target.Target;
import org.act.xservices.rt.xsecurity.xacml.util.Constants;

/**
 * 
 */
public abstract class AXacmlPolicy extends AXacmlElement implements IMatchable,
		IEvaluateable {
	
	public static final String POLICY_ID="PolicyId";
	public static final String POLICY_SET_ID="PolicySetId";
	public static final String DESCRIPTION="Description";
	public static final String POLICY_DEFAULTS="PolicyDefaults";
	public static final String POLICY_SET_DEFAULTS="PolicySetDefaults";
	public static final String TARGET="Target";
	public static final String OBLIGATIONS="Obligations";
	public static final String VERSION="Version";

	protected URI id = null;

	protected String description = null;

	protected XacmlPolicyDefaults defaults = null;

	protected Target target = null;

	protected Map<String, String> namespaceMap = null;

	protected Obligations obligations = null;

	protected Version version = Constants.POLICY_DEFAULT_VERSION;

	/**
	 * PolicySetܰPolicySetPolicyãIdReferenceĶPolicySet֮ǰҪЩýΪʵʵ
	 * PolicySetPolicyֶڱǽǷɡPolicyĶûаõPolicySetĶΪtrue
	 */
	protected boolean resolved;

	public AXacmlPolicy() {
		this.namespaceMap = new HashMap<String, String>();
	}

	public AXacmlPolicy(URI id, String description,
			XacmlPolicyDefaults defaults, Target target, Obligations obligations) {
		this.id = id;
		this.description = description;
		this.defaults = defaults;
		((AXacmlElement)this.defaults).setParent(this);
		this.target = target;
		((AXacmlElement)this.target).setParent(this);
		this.namespaceMap = new HashMap<String, String>();
		this.obligations = obligations;
		this.obligations.setParent(this);
	}
	
	public AXacmlPolicy(Metaset xmlElement) throws PolicySyntaxErrorException, DataTypeMappingException{
		super(xmlElement);
		
		this.namespaceMap = UtilMetaset.extractNameSpaceMap(xmlElement);
		
		Attribute[] idAttrs=UtilMetaset.extractAttributesByName(xmlElement,POLICY_ID);
		if(idAttrs==null || idAttrs.length==0){
			idAttrs=UtilMetaset.extractAttributesByName(xmlElement, POLICY_SET_ID);
		}
		UtilMetaset.checkEmpty(idAttrs, POLICY_ID+"//"+POLICY_SET_ID);
		UtilMetaset.checkNumber(idAttrs, 1, POLICY_ID+"//"+POLICY_SET_ID);
		String idStr=idAttrs[0].getValue().trim();
		this.id=URI.create(idStr);
		
		Metaset[] descChildren=UtilMetaset.extractChildrenByName(xmlElement, DESCRIPTION);
		if(descChildren!=null && descChildren.length!=0){
			UtilMetaset.checkNumber(descChildren, 1, DESCRIPTION);
			this.description=descChildren[0].getText().trim();
		}
		
		Metaset[] dfChildren=UtilMetaset.extractChildrenByName(xmlElement, POLICY_DEFAULTS);
		if(dfChildren==null || dfChildren.length==0){
			dfChildren=UtilMetaset.extractChildrenByName(xmlElement, POLICY_SET_DEFAULTS);
		}
		if(dfChildren!=null && dfChildren.length!=0){
			UtilMetaset.checkNumber(dfChildren, 1, POLICY_DEFAULTS+"//"+POLICY_SET_DEFAULTS);
			this.defaults=new XacmlPolicyDefaults(dfChildren[0]);
			((AXacmlElement)this.defaults).setParent(this);
		}
		
		Metaset[] targetChildren=UtilMetaset.extractChildrenByName(xmlElement, TARGET);
		if(targetChildren!=null && targetChildren.length!=0){
			UtilMetaset.checkNumber(targetChildren, 1, TARGET);
			this.target=new Target(targetChildren[0]);
			((AXacmlElement)this.target).setParent(this);
		}
		
		Metaset[] oblChildren=UtilMetaset.extractChildrenByName(xmlElement, OBLIGATIONS);
		if(oblChildren!=null&&oblChildren.length!=0){
			UtilMetaset.checkNumber(oblChildren, 1, OBLIGATIONS);
			this.obligations=new Obligations(oblChildren[0]);
			this.obligations.setParent(this);
		}
		
		Attribute[] versionAttrs=UtilMetaset.extractAttributesByName(xmlElement, VERSION);
		if(versionAttrs!=null&&versionAttrs.length!=0){
			UtilMetaset.checkNumber(versionAttrs, 1, VERSION);
			String vStr=versionAttrs[0].getValue().trim();
			this.version=new Version(vStr);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.act.xacml.IMatchable#match(org.act.xacml.context.IRequestContext)
	 */
	public boolean match(IRequest request) throws IndeterminateException, DataTypeMappingException {
		// TODO Auto-generated method stub
		return this.target.match(request);
	}

	public abstract Result evaluate(IRequest request) throws DataTypeMappingException;

	public Map<String, String> getNamespaceMap() {
		return namespaceMap;
	}

	public void setNamespaceMap(Map<String, String> namespaceMap) {
		this.namespaceMap = namespaceMap;
	}

	public XacmlPolicyDefaults getDefaults() {
		return defaults;
	}

	public void setDefaults(XacmlPolicyDefaults defaults) {
		this.defaults = defaults;
		((AXacmlElement)this.defaults).setParent(this);
	}

	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public URI getId() {
		return id;
	}

	public void setId(URI id) {
		this.id = id;
	}

	public Obligations getObligations() {
		return obligations;
	}

	public void setObligations(Obligations obligations) {
		this.obligations = obligations;
		this.obligations.setParent(this);
	}

	public boolean isResolved() {
		return resolved;
	}

	public void setResolved(boolean resolved) {
		this.resolved = resolved;
	}

	public Target getTarget() {
		return target;
		
	}
	public void setTarget(Target target) {
		this.target = target;
		((AXacmlElement)this.target).setParent(this);
	}

	public Version getVersion() {
		return version;
	}

	public void setVersion(Version version) {
		this.version = version;
	}

}
