/**
 * AttributeAssignment.java
 */
package org.act.xservices.rt.xsecurity.xacml.policy;

import java.net.URI;

import org.act.xservices.rt.xom.Attribute;
import org.act.xservices.rt.xom.Metaset;
import org.act.xservices.rt.xsecurity.xacml.exception.DataTypeMappingException;
import org.act.xservices.rt.xsecurity.xacml.exception.PolicySyntaxErrorException;
import org.act.xservices.rt.xsecurity.xacml.policy.target.AttributeValue;

/**
 * 
 */
public class AttributeAssignment extends AttributeValue {

	public static final String ATTRIBUTE_ID = "AttributeId";

	private URI attributeId = null;

	protected AttributeAssignment(URI attributeId, URI dataType, Object value)
			throws DataTypeMappingException {
		super(dataType, value);
		this.attributeId = attributeId;
	}

	protected AttributeAssignment(URI attributeId, URI dataType, String value)
			throws DataTypeMappingException {
		super(dataType, value);
		this.attributeId = attributeId;
	}

	protected AttributeAssignment(Metaset xmlElement)
			throws PolicySyntaxErrorException, DataTypeMappingException {
		super(xmlElement);
		Attribute[] idAttrs = UtilMetaset.extractAttributesByName(xmlElement,
				ATTRIBUTE_ID);
		UtilMetaset.checkEmpty(idAttrs, ATTRIBUTE_ID);
		UtilMetaset.checkNumber(idAttrs, 1, ATTRIBUTE_ID);
		String idStr = idAttrs[0].getValue().trim();
		this.attributeId = URI.create(idStr);
	}

	public static AttributeAssignment newAttributeAssignment(URI attributeId,
			URI dataType, Object value) throws DataTypeMappingException {
		if (!checkParams(dataType, value)) {
			throw new DataTypeMappingException(null,
					"dataType or value is null!", null);
		}
		return new AttributeAssignment(attributeId, dataType, value);
	}

	public static AttributeAssignment newAttributeAssignment(URI attributeId,
			URI dataType, String value) throws DataTypeMappingException {
		if (!checkParams(dataType, value)) {
			throw new DataTypeMappingException(null,
					"dataType or value is null!", null);
		}
		return new AttributeAssignment(attributeId, dataType, value);
	}

	public static AttributeAssignment newAttributeAssignment(Metaset xmlElement)
			throws PolicySyntaxErrorException, DataTypeMappingException {
		Attribute[] attrs = UtilMetaset.extractAttributesByName(xmlElement,
				DATA_TYPE);
		UtilMetaset.checkNumber(attrs, 1, DATA_TYPE);
		URI dataType = URI.create(attrs[0].getValue());
		String dataValue = xmlElement.getText().trim();

		if (!checkParams(dataType, dataValue)) {
			throw new DataTypeMappingException(null,
					"dataType or value is null!", null);
		}
		return new AttributeAssignment(xmlElement);
	}

	public URI getAttributeId() {
		return attributeId;
	}

	public void setAttributeId(URI attributeId) {
		this.attributeId = attributeId;
	}
}
