/**
 * Obligation.java
 */
package org.act.xservices.rt.xsecurity.xacml.policy;

import java.net.URI;

import org.act.xservices.rt.xom.Attribute;
import org.act.xservices.rt.xom.Metaset;
import org.act.xservices.rt.xsecurity.xacml.AXacmlElement;
import org.act.xservices.rt.xsecurity.xacml.exception.DataTypeMappingException;
import org.act.xservices.rt.xsecurity.xacml.exception.PolicySyntaxErrorException;
import org.act.xservices.rt.xsecurity.xacml.policy.rule.Effect;
import org.act.xservices.rt.xsecurity.xacml.util.Constants;

/**
 * 
 */
public class Obligation extends AXacmlElement {

	public static final String OBLIGATION_ID = "ObligationId";

	public static final String FULFILL_ON = "FulfillOn";

	public static final String ATTR_ASSIGN = "AttributeAssignment";

	public static final String PERMIT = "Permit";

	public static final String DENY = "Deny";

	private Effect fulfillOn = null;

	private URI obligationId = null;

	private AttributeAssignment[] assignments = null;

	public Obligation(Effect fulfillOn, URI obligationId,
			AttributeAssignment[] assignments) {
		this.fulfillOn = fulfillOn;
		this.obligationId = obligationId;
		this.assignments = assignments;
		for (int i = 0; i < this.assignments.length; i++) {
			this.assignments[i].setParent(this);
		}
	}

	public Obligation(Metaset xmlElement) throws PolicySyntaxErrorException,
			DataTypeMappingException {
		super(xmlElement);
		Attribute[] idAttrs = UtilMetaset.extractAttributesByName(xmlElement,
				OBLIGATION_ID);
		UtilMetaset.checkEmpty(idAttrs, OBLIGATION_ID);
		UtilMetaset.checkNumber(idAttrs, 1, OBLIGATION_ID);
		String idStr = idAttrs[0].getValue().trim();
		this.obligationId = URI.create(idStr);

		Attribute[] fulfillAttrs = UtilMetaset.extractAttributesByName(
				xmlElement, FULFILL_ON);
		UtilMetaset.checkEmpty(fulfillAttrs, FULFILL_ON);
		UtilMetaset.checkNumber(fulfillAttrs, 1, FULFILL_ON);
		String fulfillValue = fulfillAttrs[0].getValue().trim();
		if (fulfillValue.equalsIgnoreCase(PERMIT)) {
			this.fulfillOn = Effect.PERMIT;
		} else if (fulfillValue.equalsIgnoreCase(DENY)) {
			this.fulfillOn = Effect.DENY;
		} else {
			throw new PolicySyntaxErrorException(Constants.STATUS_SYNTAXERROR,
					"Invalid effect value!", null);
		}

		Metaset[] assignChildren = UtilMetaset.extractChildrenByName(
				xmlElement, ATTR_ASSIGN);
		this.assignments = new AttributeAssignment[assignChildren.length];
		for (int i = 0; i < this.assignments.length; i++) {
			this.assignments[i] = AttributeAssignment
					.newAttributeAssignment(assignChildren[i]);
			this.assignments[i].setParent(this);
		}
	}

	public AttributeAssignment[] getAssignments() {
		return assignments;
	}

	public void setAssignments(AttributeAssignment[] assignments) {
		this.assignments = assignments;
		for (int i = 0; i < this.assignments.length; i++) {
			this.assignments[i].setParent(this);
		}
	}

	public Effect getFulfillOn() {
		return fulfillOn;
	}

	public void setFulfillOn(Effect fulfillOn) {
		this.fulfillOn = fulfillOn;
	}

	public URI getObligationId() {
		return obligationId;
	}

	public void setObligationId(URI obligationId) {
		this.obligationId = obligationId;
	}

}
