/**
 * Obligations.java
 */
package org.act.xservices.rt.xsecurity.xacml.policy;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.act.xservices.rt.xom.Metaset;
import org.act.xservices.rt.xsecurity.xacml.AXacmlElement;
import org.act.xservices.rt.xsecurity.xacml.context.Decision;
import org.act.xservices.rt.xsecurity.xacml.exception.DataTypeMappingException;
import org.act.xservices.rt.xsecurity.xacml.exception.PolicySyntaxErrorException;
import org.act.xservices.rt.xsecurity.xacml.policy.rule.Effect;

/**
 * 
 */
public class Obligations extends AXacmlElement {

	public static final String OBLIGATION = "Obligation";

	private Obligation[] obligations = null;

	private Map<Decision, Obligation[]> decisionOblMap = null;

	public Obligations(Obligation[] obligations) {
		this.obligations = obligations;
	
		List<Obligation> permitList=new ArrayList<Obligation>();
		List<Obligation> denyList=new ArrayList<Obligation>();
		
		for (int i = 0; i < this.obligations.length; i++) {
			this.obligations[i].setParent(this);
			if(this.obligations[i].getFulfillOn()==Effect.PERMIT){
				permitList.add(this.obligations[i]);
			}else if(this.obligations[i].getFulfillOn()==Effect.DENY){
				denyList.add(this.obligations[i]);
			}
		}
		this.decisionOblMap=new HashMap<Decision,Obligation[]>();
		this.decisionOblMap.put(Decision.PERMIT, permitList.toArray(new Obligation[0]));
		this.decisionOblMap.put(Decision.DENY, denyList.toArray(new Obligation[0]));
		
		
	}

	public Obligations(Metaset xmlElement) throws PolicySyntaxErrorException,
			DataTypeMappingException {
		super(xmlElement);
		
		List<Obligation> permitList=new ArrayList<Obligation>();
		List<Obligation> denyList=new ArrayList<Obligation>();
		
		Metaset[] oblChildren = UtilMetaset.extractChildrenByName(xmlElement,
				OBLIGATION);
		this.obligations = new Obligation[oblChildren.length];
		for (int i = 0; i < this.obligations.length; i++) {
			this.obligations[i] = new Obligation(oblChildren[i]);
			this.obligations[i].setParent(this);
			if(this.obligations[i].getFulfillOn()==Effect.PERMIT){
				permitList.add(this.obligations[i]);
			}else if(this.obligations[i].getFulfillOn()==Effect.DENY){
				denyList.add(this.obligations[i]);
			}
		}
		this.decisionOblMap=new HashMap<Decision,Obligation[]>();
		this.decisionOblMap.put(Decision.PERMIT, permitList.toArray(new Obligation[0]));
		this.decisionOblMap.put(Decision.DENY, denyList.toArray(new Obligation[0]));
	}
	
	public Obligation[] getObligations() {
		return obligations;
	}

	public void setObligations(Obligation[] obligations) {
		this.obligations = obligations;
		List<Obligation> permitList=new ArrayList<Obligation>();
		List<Obligation> denyList=new ArrayList<Obligation>();
		
		for (int i = 0; i < this.obligations.length; i++) {
			this.obligations[i].setParent(this);
			if(this.obligations[i].getFulfillOn()==Effect.PERMIT){
				permitList.add(this.obligations[i]);
			}else if(this.obligations[i].getFulfillOn()==Effect.DENY){
				denyList.add(this.obligations[i]);
			}
		}
		this.decisionOblMap=new HashMap<Decision,Obligation[]>();
		this.decisionOblMap.put(Decision.PERMIT, permitList.toArray(new Obligation[0]));
		this.decisionOblMap.put(Decision.DENY, denyList.toArray(new Obligation[0]));
	}

	public Obligation[] getObligationsByDecision(Decision decision) {
		Obligation[] result = this.decisionOblMap.get(decision);
		if (result == null) {
			result = new Obligation[0];
		}
		return result;
	}
}
