/**
 * Version.java
 */
package org.act.xservices.rt.xsecurity.xacml.policy;

import org.act.xservices.rt.xsecurity.xacml.AXacmlElement;
import org.act.xservices.rt.xsecurity.xacml.exception.PolicySyntaxErrorException;
import org.act.xservices.rt.xsecurity.xacml.util.Constants;

/**
 * 
 */
public class Version extends AXacmlElement implements Comparable<Version> {

	public static Version VERSION_1_0 = null;
	static {
		try {
			VERSION_1_0 = new Version("1.0");
		} catch (PolicySyntaxErrorException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	private String versionString = null;

	public static final String VERSION_PATTERN = "(\\d+\\.)*\\d+";

	public Version() {
	}

	public Version(String versionString) throws PolicySyntaxErrorException {
		if (versionString.matches(VERSION_PATTERN)) {
			this.versionString = versionString;
		} else {
			PolicySyntaxErrorException e = new PolicySyntaxErrorException(
					Constants.STATUS_SYNTAXERROR,
					"Version string does not match VERSION_PATTERN !", null);
			throw e;
		}
	}

	public boolean equals(Object v) {
		if (this == v) {
			return true;
		}
		if (v != null && v.getClass() == this.getClass()) {
			return this.versionString.equals(((Version) v).getVersionString());
		}
		return false;
	}

	public String getVersionString() {
		return versionString;
	}

	public void setVersionString(String versionString) {
		this.versionString = versionString;
	}

	public int compareTo(Version another) {
		String sep = "\\.";
		String[] thisVal = this.versionString.split(sep);
		String[] otherVal = another.getVersionString().split(sep);
		int min = Math.min(thisVal.length, otherVal.length);
		for (int i = 0; i < min; i++) {
			if (!thisVal[i].equals(otherVal[i])) {
				float result = 0;
				if (i == 0) {
					result = Float.parseFloat(thisVal[i])
							- Float.parseFloat(otherVal[i]);
				} else {
					result = Float.parseFloat("." + thisVal[i])
							- Float.parseFloat("." + otherVal[i]);
				}
				return result < 0 ? -1 : 1;
			}
		}
		return thisVal.length - otherVal.length;
	}

	public static void main(String[] args) throws PolicySyntaxErrorException {
		Version v1 = new Version("3.4");
		Version v2 = new Version("3.4.1");
		Version v3 = new Version("333.1.3.4");
		// Version v4=new Version("d3.1.3.4");
		System.out.println(v1.compareTo(v2));
		System.out.println(v1.compareTo(v3));
	}

}
