/**
 * Apply.java
 */
package org.act.xservices.rt.xsecurity.xacml.policy.rule;

import java.net.URI;
import java.util.List;

import org.act.xservices.rt.xom.Attribute;
import org.act.xservices.rt.xom.Metaset;
import org.act.xservices.rt.xsecurity.xacml.AXacmlElement;
import org.act.xservices.rt.xsecurity.xacml.IExpression;
import org.act.xservices.rt.xsecurity.xacml.context.IRequest;
import org.act.xservices.rt.xsecurity.xacml.exception.DataTypeMappingException;
import org.act.xservices.rt.xsecurity.xacml.exception.IndeterminateException;
import org.act.xservices.rt.xsecurity.xacml.exception.PolicySyntaxErrorException;
import org.act.xservices.rt.xsecurity.xacml.exception.XacmlFunctionInvokeException;
import org.act.xservices.rt.xsecurity.xacml.exception.XacmlFunctionLoadException;
import org.act.xservices.rt.xsecurity.xacml.pdp.function.FunctionRegistry;
import org.act.xservices.rt.xsecurity.xacml.pdp.function.XacmlFunction;
import org.act.xservices.rt.xsecurity.xacml.policy.UtilMetaset;
import org.act.xservices.rt.xsecurity.xacml.util.Constants;

/**
 * 
 */
public class Apply extends AXacmlElement implements IExpression {

	public static final String FUNCTION_ID = "FunctionId";

	private URI functionId = null;

	private IExpression[] expressions = null;

	public Apply() {
	}

	public Apply(URI functionId, IExpression[] expressions) {
		this.functionId = functionId;
		this.expressions = expressions;
		for (int i = 0; i < this.expressions.length; i++) {
			((AXacmlElement) this.expressions[i]).setParent(this);
		}
	}

	public Apply(Metaset xmlElement) throws PolicySyntaxErrorException,
			DataTypeMappingException, IndeterminateException {
		super(xmlElement);
		Attribute[] funcIdAttrs = UtilMetaset.extractAttributesByName(
				xmlElement, FUNCTION_ID);
		UtilMetaset.checkEmpty(funcIdAttrs, FUNCTION_ID);
		UtilMetaset.checkNumber(funcIdAttrs, 1, FUNCTION_ID);
		URI funcId = URI.create(funcIdAttrs[0].getValue().trim());
		this.functionId = funcId;

		List<Metaset> subElems = xmlElement.elements();
		IExpression[] subExps = new IExpression[subElems.size()];
		for (int i = 0; i < subExps.length; i++) {
			subExps[i] = ExpressionFactory.newExpression(subElems.get(i));
			((AXacmlElement) subExps[i]).setParent(this);
		}
		this.expressions = subExps;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.act.xacml.IExpression#evaluate(org.act.xacml.context.IRequestContext)
	 */
	public Object evaluate(IRequest request)
			throws IndeterminateException, DataTypeMappingException{
		// TODO Auto-generated method stub
		try {
			XacmlFunction func = FunctionRegistry.getInstance().lookup(
					this.functionId);
			Object[] params = new Object[this.expressions.length];
			for (int i = 0; i < this.expressions.length; i++) {
				params[i] = this.expressions[i].evaluate(request);
			}
			Object result = func.invoke(request, params);
			return result;
		} catch (XacmlFunctionLoadException e) {
			IndeterminateException ie = new IndeterminateException(
					Constants.STATUS_PROCESSINGERROR,
					"XacmlFunction load error!", e);
			throw ie;
		} catch (XacmlFunctionInvokeException e) {
			// TODO Auto-generated catch block
			// e.printStackTrace();
			IndeterminateException ie = new IndeterminateException(
					Constants.STATUS_PROCESSINGERROR,
					"XacmlFunction invocation error!", e);
			throw ie;
		}
	}

	public IExpression[] getExpressions() {
		return expressions;
	}

	public void setExpressions(IExpression[] expressions) {
		this.expressions = expressions;
		for (int i = 0; i < this.expressions.length; i++) {
			((AXacmlElement) this.expressions[i]).setParent(this);
		}
	}

	public URI getFunctionId() {
		return functionId;
	}

	public void setFunctionId(URI functionId) {
		this.functionId = functionId;
	}

}
