/**
 * Condition.java
 */
package org.act.xservices.rt.xsecurity.xacml.policy.rule;

import java.util.List;

import org.act.xservices.rt.xom.Metaset;
import org.act.xservices.rt.xsecurity.xacml.AXacmlElement;
import org.act.xservices.rt.xsecurity.xacml.IExpression;
import org.act.xservices.rt.xsecurity.xacml.context.IRequest;
import org.act.xservices.rt.xsecurity.xacml.exception.DataTypeMappingException;
import org.act.xservices.rt.xsecurity.xacml.exception.IndeterminateException;
import org.act.xservices.rt.xsecurity.xacml.exception.PolicySyntaxErrorException;
import org.act.xservices.rt.xsecurity.xacml.policy.UtilMetaset;
import org.act.xservices.rt.xsecurity.xacml.policy.target.AttributeValue;
import org.act.xservices.rt.xsecurity.xacml.util.Constants;

/**
 * 
 */
public class Condition extends AXacmlElement {

	public static final String EXPRESSION = "Expression";

	private IExpression expression = null;

	public Condition(IExpression expression) {
		this.expression = expression;
		((AXacmlElement) this.expression).setParent(this);
	}

	public Condition(Metaset xmlElement) throws PolicySyntaxErrorException,
			DataTypeMappingException, IndeterminateException {
		super(xmlElement);
		// ConditionӦֻһԪأԪһExpression֪Expression
		List<Metaset> expList = xmlElement.elements();
		Metaset[] expChildren = expList.toArray(new Metaset[0]);
		UtilMetaset.checkEmpty(expChildren, EXPRESSION);
		UtilMetaset.checkNumber(expChildren, 1, EXPRESSION);
		this.expression = ExpressionFactory.newExpression(expChildren[0]);
		((AXacmlElement) this.expression).setParent(this);
	}

	public boolean evaluate(IRequest request) throws IndeterminateException, DataTypeMappingException {
		// TODO Auto-generated method stub
		if (this.expression == null) {
			throw new IndeterminateException(Constants.STATUS_SYNTAXERROR,
					"The expression of condition is null!", null);
		}
		Object result = this.expression.evaluate(request);
		if (result == null) {
			throw new IndeterminateException(Constants.STATUS_PROCESSINGERROR,
					"The expression of condition evaluates to null!", null);
		}
		if (result == AttributeValue.TRUE) {
			return true;
		} else if (result == AttributeValue.FALSE) {
			return false;
		}

		throw new IndeterminateException(
				Constants.STATUS_PROCESSINGERROR,
				"The expression of condition evaluates to a non-Boolean result!",
				null);

	}

	public IExpression getExpression() {
		return expression;
	}

	public void setExpression(IExpression expression) {
		this.expression = expression;
		((AXacmlElement) this.expression).setParent(this);
	}

}
