/**
 * ExpressionFactory.java
 */
package org.act.xservices.rt.xsecurity.xacml.policy.rule;

import java.net.URI;
import java.util.List;

import org.act.xservices.rt.xom.Metaset;
import org.act.xservices.rt.xsecurity.xacml.AXacmlElement;
import org.act.xservices.rt.xsecurity.xacml.IExpression;
import org.act.xservices.rt.xsecurity.xacml.exception.DataTypeMappingException;
import org.act.xservices.rt.xsecurity.xacml.exception.IndeterminateException;
import org.act.xservices.rt.xsecurity.xacml.exception.PolicySyntaxErrorException;
import org.act.xservices.rt.xsecurity.xacml.policy.UtilMetaset;
import org.act.xservices.rt.xsecurity.xacml.policy.target.ActionAttributeDesignator;
import org.act.xservices.rt.xsecurity.xacml.policy.target.AttributeSelector;
import org.act.xservices.rt.xsecurity.xacml.policy.target.AttributeValue;
import org.act.xservices.rt.xsecurity.xacml.policy.target.EnvironmentAttributeDesignator;
import org.act.xservices.rt.xsecurity.xacml.policy.target.ResourceAttributeDesignator;
import org.act.xservices.rt.xsecurity.xacml.policy.target.SubjectAttributeDesignator;
import org.act.xservices.rt.xsecurity.xacml.util.Constants;

/**
 * 
 */
public class ExpressionFactory {

	public static final String APPLY = "Apply";

	public static final String ATTRIBUTE_SELECTOR = "AttributeSelector";

	public static final String ATTRIBUTE_VALUE = "AttributeValue";

	public static final String FUNCTION = "Function";

	public static final String VARIABLE_REFERENCE = "VariableReference";

	public static final String ACTION_ATTR_DSGN = "ActionAttributeDesignator";

	public static final String RES_ATTR_DSGN = "ResourceAttributeDesignator";

	public static final String SUB_ATTR_DSGN = "SubjectAttributeDesignator";

	public static final String ENV_ATTR_DSGN = "EnvironmentAttributeDesignator";

	public static IExpression newExpression(Metaset xmlElement)
			throws PolicySyntaxErrorException, DataTypeMappingException,
			IndeterminateException {
		IExpression resultExp = null;
		String expName = xmlElement.getName().trim();
		if (expName.equalsIgnoreCase(APPLY)) {
			resultExp = makeApply(xmlElement);
		} else if (expName.equalsIgnoreCase(ATTRIBUTE_SELECTOR)) {
			resultExp = makeAttributeSelector(xmlElement);
		} else if (expName.equalsIgnoreCase(ATTRIBUTE_VALUE)) {
			resultExp = makeAttributeValue(xmlElement);
		} else if (expName.equalsIgnoreCase(FUNCTION)) {
			resultExp = makeFunction(xmlElement);
		} else if (expName.equalsIgnoreCase(VARIABLE_REFERENCE)) {
			resultExp = makeVariableRef(xmlElement);
		} else if (expName.equalsIgnoreCase(ACTION_ATTR_DSGN)) {
			resultExp = makeActionAttrDesignator(xmlElement);
		} else if (expName.equalsIgnoreCase(RES_ATTR_DSGN)) {
			resultExp = makeResourceAttrDesignator(xmlElement);
		} else if (expName.equalsIgnoreCase(SUB_ATTR_DSGN)) {
			resultExp = makeSubjectAttrDesignator(xmlElement);
		} else if (expName.equalsIgnoreCase(ENV_ATTR_DSGN)) {
			resultExp = makeEnvironmentAttrDesignator(xmlElement);
		} else {
			throw new PolicySyntaxErrorException(Constants.STATUS_SYNTAXERROR,
					"Unknown expression name!", null);
		}
		return resultExp;
	}

	private static IExpression makeApply(Metaset xmlElement)
			throws PolicySyntaxErrorException, DataTypeMappingException,
			IndeterminateException {
		// TODO Auto-generated method stub
		return new Apply(xmlElement);
	}

	private static IExpression makeAttributeSelector(Metaset xmlElement)
			throws PolicySyntaxErrorException {
		// TODO Auto-generated method stub
		return new AttributeSelector(xmlElement);
	}

	private static IExpression makeAttributeValue(Metaset xmlElement)
			throws PolicySyntaxErrorException, DataTypeMappingException {
		// TODO Auto-generated method stub
		return AttributeValue.newInstance(xmlElement);
	}

	private static IExpression makeFunction(Metaset xmlElement)
			throws PolicySyntaxErrorException, IndeterminateException {
		// TODO Auto-generated method stub
		return new Function(xmlElement);
	}

	private static IExpression makeVariableRef(Metaset xmlElement)
			throws PolicySyntaxErrorException {
		// TODO Auto-generated method stub
		return new VariableReference(xmlElement);
	}

	private static IExpression makeActionAttrDesignator(Metaset xmlElement)
			throws PolicySyntaxErrorException {
		// TODO Auto-generated method stub
		return new ActionAttributeDesignator(xmlElement);
	}

	private static IExpression makeResourceAttrDesignator(Metaset xmlElement)
			throws PolicySyntaxErrorException {
		// TODO Auto-generated method stub
		return new ResourceAttributeDesignator(xmlElement);
	}

	private static IExpression makeSubjectAttrDesignator(Metaset xmlElement)
			throws PolicySyntaxErrorException {
		// TODO Auto-generated method stub
		return new SubjectAttributeDesignator(xmlElement);
	}

	private static IExpression makeEnvironmentAttrDesignator(Metaset xmlElement)
			throws PolicySyntaxErrorException {
		// TODO Auto-generated method stub
		return new EnvironmentAttributeDesignator(xmlElement);
	}
}
