/**
 * Function.java
 */
package org.act.xservices.rt.xsecurity.xacml.policy.rule;

import java.net.URI;

import org.act.xservices.rt.xom.Attribute;
import org.act.xservices.rt.xom.Metaset;
import org.act.xservices.rt.xsecurity.xacml.AXacmlElement;
import org.act.xservices.rt.xsecurity.xacml.IExpression;
import org.act.xservices.rt.xsecurity.xacml.context.IRequest;
import org.act.xservices.rt.xsecurity.xacml.exception.IndeterminateException;
import org.act.xservices.rt.xsecurity.xacml.exception.PolicySyntaxErrorException;
import org.act.xservices.rt.xsecurity.xacml.exception.XacmlFunctionLoadException;
import org.act.xservices.rt.xsecurity.xacml.pdp.function.FunctionRegistry;
import org.act.xservices.rt.xsecurity.xacml.pdp.function.XacmlFunction;
import org.act.xservices.rt.xsecurity.xacml.policy.UtilMetaset;
import org.act.xservices.rt.xsecurity.xacml.util.Constants;

/**
 * һXACML߽׺Ĳ
 * 
 * 
 */
public class Function extends AXacmlElement implements IExpression {

	public static final String FUNCTION_ID = "FunctionId";

	private URI functionId = null;

	private XacmlFunction function = null;

	public Function() {
	}

	public Function(URI functionId) throws IndeterminateException {
		this.functionId = functionId;
		try {
			this.function = FunctionRegistry.getInstance().lookup(
					this.functionId);
		} catch (XacmlFunctionLoadException e) {
			// TODO Auto-generated catch block
			// e.printStackTrace();
			IndeterminateException ie = new IndeterminateException(
					Constants.STATUS_PROCESSINGERROR,
					"XacmlFunction load error", e);
			throw ie;
		}
	}

	public Function(Metaset xmlElement) throws PolicySyntaxErrorException,
			IndeterminateException {
		super(xmlElement);
		Attribute[] funcIdAttrs = UtilMetaset.extractAttributesByName(
				xmlElement, FUNCTION_ID);
		UtilMetaset.checkEmpty(funcIdAttrs, FUNCTION_ID);
		UtilMetaset.checkNumber(funcIdAttrs, 1, FUNCTION_ID);
		String funcIdStr = funcIdAttrs[0].getValue().trim();
		this.functionId = URI.create(funcIdStr);

		try {
			this.function = FunctionRegistry.getInstance().lookup(
					this.functionId);
		} catch (XacmlFunctionLoadException e) {
			// TODO Auto-generated catch block
			// e.printStackTrace();
			IndeterminateException ie = new IndeterminateException(
					Constants.STATUS_PROCESSINGERROR,
					"XacmlFunction load error", e);
			throw ie;
		}
	}

	public Object evaluate(IRequest request)
			throws IndeterminateException {
		throw new IndeterminateException(Constants.STATUS_PROCESSINGERROR,
				"Function class does not support evaluate operation!", null);
	}

	public URI getFunctionId() {
		return functionId;
	}

	public void setFunctionId(URI functionId) throws IndeterminateException {
		this.functionId = functionId;
		try {
			this.function = FunctionRegistry.getInstance().lookup(
					this.functionId);
		} catch (XacmlFunctionLoadException e) {
			// TODO Auto-generated catch block
			// e.printStackTrace();
			IndeterminateException ie = new IndeterminateException(
					Constants.STATUS_PROCESSINGERROR,
					"XacmlFunction load error", e);
			throw ie;
		}
	}

	public XacmlFunction getFunction() {
		return function;
	}
}
