/**
 * VariableDefinition.java
 */
package org.act.xservices.rt.xsecurity.xacml.policy.rule;

import java.util.List;

import org.act.xservices.rt.xom.Attribute;
import org.act.xservices.rt.xom.Metaset;
import org.act.xservices.rt.xsecurity.xacml.AXacmlElement;
import org.act.xservices.rt.xsecurity.xacml.IExpression;
import org.act.xservices.rt.xsecurity.xacml.exception.DataTypeMappingException;
import org.act.xservices.rt.xsecurity.xacml.exception.IndeterminateException;
import org.act.xservices.rt.xsecurity.xacml.exception.PolicySyntaxErrorException;
import org.act.xservices.rt.xsecurity.xacml.policy.UtilMetaset;

/**
 * 
 */
public class VariableDefinition extends AXacmlElement {
	public static final String VARIABLE_ID = "VariableId";

	public static final String EXPRESSION = "Expression";

	private String variableId = null;

	private IExpression expression = null;

	public VariableDefinition() {
	}

	public VariableDefinition(String variableId, IExpression expression) {
		this.variableId = variableId;
		this.expression = expression;
		((AXacmlElement) this.expression).setParent(this);
	}

	public VariableDefinition(Metaset xmlElement)
			throws PolicySyntaxErrorException, DataTypeMappingException,
			IndeterminateException {
		super(xmlElement);
		Attribute[] defIdAttrs = UtilMetaset.extractAttributesByName(
				xmlElement, VARIABLE_ID);
		UtilMetaset.checkEmpty(defIdAttrs, VARIABLE_ID);
		UtilMetaset.checkNumber(defIdAttrs, 1, VARIABLE_ID);
		this.variableId = defIdAttrs[0].getValue().trim();

		//VariableDefinitionӦֻһԪأԪһExpression֪Expression
		List<Metaset> expList = xmlElement.elements();
		Metaset[] expChildren = expList.toArray(new Metaset[0]);
		UtilMetaset.checkEmpty(expChildren, EXPRESSION);
		UtilMetaset.checkNumber(expChildren, 1, EXPRESSION);
		this.expression = ExpressionFactory.newExpression(expChildren[0]);
		((AXacmlElement)this.expression).setParent(this);
	}

	public IExpression getExpression() {
		return expression;
	}

	public void setExpression(IExpression expression) {
		this.expression = expression;
		((AXacmlElement) this.expression).setParent(this);
	}

	public String getVariableId() {
		return variableId;
	}

	public void setVariableId(String variableId) {
		this.variableId = variableId;
	}

}
