/**
 * VariableReference.java
 */
package org.act.xservices.rt.xsecurity.xacml.policy.rule;

import org.act.xservices.rt.xom.Attribute;
import org.act.xservices.rt.xom.Metaset;
import org.act.xservices.rt.xsecurity.xacml.AXacmlElement;
import org.act.xservices.rt.xsecurity.xacml.IExpression;
import org.act.xservices.rt.xsecurity.xacml.IXacmlElement;
import org.act.xservices.rt.xsecurity.xacml.context.IRequest;
import org.act.xservices.rt.xsecurity.xacml.exception.DataTypeMappingException;
import org.act.xservices.rt.xsecurity.xacml.exception.IndeterminateException;
import org.act.xservices.rt.xsecurity.xacml.exception.PolicySyntaxErrorException;
import org.act.xservices.rt.xsecurity.xacml.policy.Policy;
import org.act.xservices.rt.xsecurity.xacml.policy.UtilMetaset;
import org.act.xservices.rt.xsecurity.xacml.util.Constants;

/**
 * 
 */
public class VariableReference extends AXacmlElement implements IExpression {
	public static final String VARIABLE_ID = "VariableId";

	private String variableId = null;

	private IExpression expReferenced = null;

	public VariableReference() {
	}

	public VariableReference(String variableId) {
		this.variableId = variableId;
	}

	public VariableReference(Metaset xmlElement)
			throws PolicySyntaxErrorException {
		super(xmlElement);
		Attribute[] vIdAttrs = UtilMetaset.extractAttributesByName(xmlElement,
				VARIABLE_ID);
		UtilMetaset.checkEmpty(vIdAttrs, VARIABLE_ID);
		UtilMetaset.checkNumber(vIdAttrs, 1, VARIABLE_ID);
		this.variableId = vIdAttrs[0].getValue().trim();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.act.xacml.IExpression#evaluate(org.act.xacml.context.IRequestContext)
	 */
	public Object evaluate(IRequest request)
			throws IndeterminateException, DataTypeMappingException{
		// TODO Auto-generated method stub

		if (this.expReferenced == null) {
			// ҵPolicy
			IXacmlElement result = parent;
			while (result != null && !(result instanceof Policy)) {
				result = result.getParent();
			}
			if (result == null) {
				IndeterminateException ie = new IndeterminateException(
						Constants.STATUS_SYNTAXERROR,
						"Can not find the containing Policy of the VariableReference!",
						null);
				throw ie;
			}
			// ӰPolicyҵ˱õı
			Policy policy = (Policy) result;
			VariableDefinition variableDef = policy
					.getVariableDef(this.variableId);
			if (variableDef == null) {
				IndeterminateException ie = new IndeterminateException(
						Constants.STATUS_SYNTAXERROR,
						"Can not find the VariableDefinition of the VariableReference in the containing Policy!",
						null);
				throw ie;
			}
			// õı
			this.expReferenced = variableDef.getExpression();

		}
		return this.expReferenced.evaluate(request);
	}

	public String getVariableId() {
		return variableId;
	}

	public void setVariableId(String variableId) {
		this.variableId = variableId;
	}
}
