/**
 * AttributeDesignator.java
 */
package org.act.xservices.rt.xsecurity.xacml.policy.target;

import java.net.URI;

import org.act.xservices.rt.xom.Attribute;
import org.act.xservices.rt.xom.Metaset;
import org.act.xservices.rt.xsecurity.xacml.context.IRequest;
import org.act.xservices.rt.xsecurity.xacml.exception.DataTypeMappingException;
import org.act.xservices.rt.xsecurity.xacml.exception.IndeterminateException;
import org.act.xservices.rt.xsecurity.xacml.exception.PolicySyntaxErrorException;
import org.act.xservices.rt.xsecurity.xacml.policy.UtilMetaset;
import org.act.xservices.rt.xsecurity.xacml.util.Constants;

/**
 * 
 */
public abstract class AAttributeDesignator extends AAttributeLocator {

	public static final String ATTRIBUTE_ID = "AttributeId";

	public static final String DATA_TYPE = "DataType";

	public static final String ISSUER = "Issuer";

	protected URI attributeId = null;

	protected URI dataType = null;

	protected String issuer = null;

	protected AAttributeDesignator(URI attributeId, URI dataType,
			String issuer, boolean mustBePresent) {
		super(mustBePresent);
		this.attributeId = attributeId;
		this.dataType = dataType;
		this.issuer = issuer;
	}

	protected AAttributeDesignator(Metaset xmlElement)
			throws PolicySyntaxErrorException {
		super(xmlElement);
		Attribute[] idAttrs = UtilMetaset.extractAttributesByName(xmlElement,
				ATTRIBUTE_ID);
		UtilMetaset.checkEmpty(idAttrs, ATTRIBUTE_ID);
		UtilMetaset.checkNumber(idAttrs, 1, ATTRIBUTE_ID);
		String idStr = idAttrs[0].getValue().trim();
		this.attributeId = URI.create(idStr);

		Attribute[] typeAttrs = UtilMetaset.extractAttributesByName(xmlElement,
				DATA_TYPE);
		UtilMetaset.checkEmpty(typeAttrs, DATA_TYPE);
		UtilMetaset.checkNumber(typeAttrs, 1, DATA_TYPE);
		String typeStr = typeAttrs[0].getValue().trim();
		this.dataType = URI.create(typeStr);

		Attribute[] issuerAttrs = UtilMetaset.extractAttributesByName(
				xmlElement, ISSUER);
		if (issuerAttrs != null && issuerAttrs.length != 0) {
			UtilMetaset.checkNumber(issuerAttrs, 1, ISSUER);
			this.issuer = issuerAttrs[0].getValue().trim();
		}
		
		
	}

	public URI getAttributeId() {
		return attributeId;
	}

	public void setAttributeId(URI attributeId) {
		this.attributeId = attributeId;
	}

	public URI getDataType() {
		return dataType;
	}

	public void setDataType(URI dataType) {
		this.dataType = dataType;
	}

	public String getIssuer() {
		return issuer;
	}

	public void setIssuer(String issuer) {
		this.issuer = issuer;
	}

	public boolean getMustBePresent() {
		return mustBePresent;
	}

	public void setMustBePresent(boolean mustBePresent) {
		this.mustBePresent = mustBePresent;
	}
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.act.xacml.IExpression#evaluate(org.act.xacml.context.IRequestContext)
	 */
	public Object evaluate(IRequest request)
			throws IndeterminateException, DataTypeMappingException{
		AttributeValue[] attributes = getAttributeValues(request);
		/*
		 * OASIS
		 * XACML淶2.0Ĺ涨һmustBePresent=trueȱʧ򷵻IndeterminateһmustBePresent=falseȱʧ
		 * 򷵻һհ
		 */
		if (attributes == null || attributes.length == 0) {
			if (this.mustBePresent) {
				throw new IndeterminateException(
						Constants.STATUS_MISSINGATTRIBUTE,
						"Missing must-be-present Attribute:" + attributeId
								+ " !", null);
			} else {
				attributes=getAttributeValuesFromRepository();
				if (attributes == null) {
					attributes = new AttributeValue[0];
				}
			}
		}
		return attributes;
	}

	protected abstract AttributeValue[] getAttributeValues(
			IRequest request);
	
	protected abstract AttributeValue[] getAttributeValuesFromRepository() throws DataTypeMappingException;
}
