/**
 * AEntityMatch.java
 */
package org.act.xservices.rt.xsecurity.xacml.policy.target;

import java.net.URI;

import org.act.xservices.rt.xom.Attribute;
import org.act.xservices.rt.xom.Metaset;
import org.act.xservices.rt.xsecurity.xacml.AXacmlElement;
import org.act.xservices.rt.xsecurity.xacml.IMatchable;
import org.act.xservices.rt.xsecurity.xacml.context.IRequest;
import org.act.xservices.rt.xsecurity.xacml.exception.DataTypeMappingException;
import org.act.xservices.rt.xsecurity.xacml.exception.IndeterminateException;
import org.act.xservices.rt.xsecurity.xacml.exception.PolicySyntaxErrorException;
import org.act.xservices.rt.xsecurity.xacml.exception.XacmlFunctionInvokeException;
import org.act.xservices.rt.xsecurity.xacml.exception.XacmlFunctionLoadException;
import org.act.xservices.rt.xsecurity.xacml.pdp.function.FunctionRegistry;
import org.act.xservices.rt.xsecurity.xacml.pdp.function.XacmlFunction;
import org.act.xservices.rt.xsecurity.xacml.policy.UtilMetaset;
import org.act.xservices.rt.xsecurity.xacml.util.Constants;

/**
 * 
 */
public abstract class AEntityMatch extends AXacmlElement implements IMatchable {

	public static final String MATCH_ID = "MatchId";

	public static final String ATTRIBUTE_VALUE = "AttributeValue";
	
	public static final String ATTRIBUTE_SELECTOR="AttributeSelector";
	

	protected URI matchId = null;

	protected AttributeValue attributeValue = null;

	protected IAttributeLocator attributeLocator = null;

	protected AEntityMatch() {
	}

	protected AEntityMatch(String elementName, URI matchId,
			AttributeValue attributeValue, IAttributeLocator attributeLocator) {
		super(elementName);
		this.matchId = matchId;
		this.attributeLocator = attributeLocator;
		((AXacmlElement)this.attributeLocator).setParent(this);
		this.attributeValue = attributeValue;
		((AXacmlElement)this.attributeValue).setParent(this);
	}

	protected AEntityMatch(Metaset xmlElement)
			throws PolicySyntaxErrorException, DataTypeMappingException {
		super(xmlElement);
		Attribute[] matchIdAttrs = UtilMetaset.extractAttributesByName(xmlElement, MATCH_ID);
		UtilMetaset.checkEmpty(matchIdAttrs, MATCH_ID);
		UtilMetaset.checkNumber(matchIdAttrs, 1, MATCH_ID);
		String matchIdStr = matchIdAttrs[0].getValue();
		this.matchId = URI.create(matchIdStr);

		Metaset[] attrValChildren = UtilMetaset.extractChildrenByName(xmlElement,
				ATTRIBUTE_VALUE);
		UtilMetaset.checkEmpty(attrValChildren, ATTRIBUTE_VALUE);
		UtilMetaset.checkNumber(attrValChildren, 1, ATTRIBUTE_VALUE);
		this.attributeValue = AttributeValue.newInstance(attrValChildren[0]);
		((AXacmlElement)this.attributeValue).setParent(this);
		
		Metaset[] selectorChildren=UtilMetaset.extractChildrenByName(xmlElement,ATTRIBUTE_SELECTOR);
		if(selectorChildren!=null && selectorChildren.length!=0){
			UtilMetaset.checkNumber(selectorChildren, 1, ATTRIBUTE_SELECTOR);
			this.attributeLocator=new AttributeSelector(selectorChildren[0]);
			((AXacmlElement)this.attributeLocator).setParent(this);
		}
	}

	public boolean match(IRequest request) throws IndeterminateException, DataTypeMappingException{
		try {
			FunctionRegistry funReg = FunctionRegistry.getInstance();
			XacmlFunction matchAlg = funReg.lookup(this.matchId);

			Object param1 = this.attributeValue.evaluate(request);
			AttributeValue[] params = (AttributeValue[]) this.attributeLocator
					.evaluate(request);

			for (AttributeValue param2 : params) {
				Object[] parameters = new Object[] { param1, param2 };
				AttributeValue result = (AttributeValue) matchAlg.invoke(
						request, parameters);
				if (result == AttributeValue.TRUE)
					return true;
			}

		} catch (XacmlFunctionInvokeException e) {
			// TODO Auto-generated catch block
			throw new IndeterminateException(Constants.STATUS_PROCESSINGERROR,
					null, e);
		} catch (XacmlFunctionLoadException e) {
			// TODO Auto-generated catch block
			throw new IndeterminateException(Constants.STATUS_PROCESSINGERROR,
					null, e);
		}
		return false;
	}

	public IAttributeLocator getAttributeLocator() {
		return attributeLocator;
	}

	public void setAttributeLocator(IAttributeLocator attributeLocator) {
		this.attributeLocator = attributeLocator;
		((AXacmlElement)this.attributeLocator).setParent(this);
	}

	public AttributeValue getAttributeValue() {
		return attributeValue;
	}

	public void setAttributeValue(AttributeValue attributeValue) {
		this.attributeValue = attributeValue;
		((AXacmlElement)this.attributeValue).setParent(this);
	}

	public URI getMatchId() {
		return matchId;
	}

	public void setMatchId(URI matchId) {
		this.matchId = matchId;
	}
}
