/**
 * AttributeSelector.java
 */
package org.act.xservices.rt.xsecurity.xacml.policy.target;

import java.net.URI;

import org.act.xservices.rt.xom.Metaset;
import org.act.xservices.rt.xsecurity.xacml.context.IRequest;
import org.act.xservices.rt.xsecurity.xacml.exception.IndeterminateException;
import org.act.xservices.rt.xsecurity.xacml.exception.PolicySyntaxErrorException;
import org.act.xservices.rt.xsecurity.xacml.policy.UtilMetaset;
import org.act.xservices.rt.xsecurity.xacml.util.Constants;

/**
 * 
 */
public class AttributeSelector extends AAttributeLocator {

	public static final String DATA_TYPE = "DataType";

	public static final String REQ_CTX_PATH = "RequestContextPath";

	private String requestContextPath = null;

	private URI dataType = null;

	public AttributeSelector(String requestContextPath, URI dataType,
			boolean mustBePresent) {
		super(mustBePresent);
		this.requestContextPath = requestContextPath;
		this.dataType = dataType;
	}

	public AttributeSelector(Metaset xmlElement)
			throws PolicySyntaxErrorException {
		super(xmlElement);

		Metaset[] pathChildren = UtilMetaset.extractChildrenByName(xmlElement, REQ_CTX_PATH);
		UtilMetaset.checkNumber(pathChildren, 1, REQ_CTX_PATH);
		this.requestContextPath = pathChildren[0].getText().trim();

		Metaset[] typeChildren = UtilMetaset.extractChildrenByName(xmlElement, DATA_TYPE);
		UtilMetaset.checkNumber(typeChildren, 1, DATA_TYPE);
		String type = typeChildren[0].getText().trim();
		this.dataType = URI.create(type);
	}

	public Object evaluate(IRequest request)
			throws IndeterminateException {
		// TODO Auto-generated method stub
		AttributeValue[] attributes = request.getAttributeValuesByXPath(
				requestContextPath, dataType);
		/*
		 * OASIS
		 * XACML淶2.0Ĺ涨һmustBePresent=trueȱʧ򷵻IndeterminateһmustBePresent=trueȱʧ
		 * 򷵻һհ
		 */
		if (attributes == null || attributes.length == 0) {
			if (this.mustBePresent) {
				throw new IndeterminateException(
						Constants.STATUS_MISSINGATTRIBUTE,
						"Missing must-be-present Attributes for XPath: "
								+ this.requestContextPath + " !", null);
			} else {
				if (attributes == null) {
					attributes = new AttributeValue[0];
				}
			}
		}
		return attributes;
	}

	public URI getDataType() {
		return dataType;
	}

	public void setDataType(URI dataType) {
		this.dataType = dataType;
	}

	public boolean isMustBePresent() {
		return mustBePresent;
	}

	public void setMustBePresent(boolean mustBePresent) {
		this.mustBePresent = mustBePresent;
	}

	public String getRequestContextPath() {
		return requestContextPath;
	}

	public void setRequestContextPath(String requestContextPath) {
		this.requestContextPath = requestContextPath;
	}

}
