/**
 * AttributeValue.java
 */
package org.act.xservices.rt.xsecurity.xacml.policy.target;

import java.net.URI;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.act.xservices.rt.xom.Attribute;
import org.act.xservices.rt.xom.Metaset;
import org.act.xservices.rt.xsecurity.xacml.AXacmlElement;
import org.act.xservices.rt.xsecurity.xacml.IExpression;
import org.act.xservices.rt.xsecurity.xacml.context.IRequest;
import org.act.xservices.rt.xsecurity.xacml.exception.DataTypeMappingException;
import org.act.xservices.rt.xsecurity.xacml.exception.IndeterminateException;
import org.act.xservices.rt.xsecurity.xacml.exception.PolicySyntaxErrorException;
import org.act.xservices.rt.xsecurity.xacml.policy.AXacmlPolicy;
import org.act.xservices.rt.xsecurity.xacml.policy.UtilMetaset;
import org.act.xservices.rt.xsecurity.xacml.util.Constants;
import org.act.xservices.rt.xsecurity.xacml.util.DataTypeRegistry;

/**
 * 
 */
public class AttributeValue extends AXacmlElement implements IExpression {

	public static final String DATA_TYPE = "DataType";

	public static final String ATTR_VALUE = "AttributeValue";

	private Object value = null;

	private Class<?> javaType = null;

	private URI dataType = null;

	private Map<String, String> nameSpaceMap = null;

	public static AttributeValue TRUE;

	public static AttributeValue FALSE;

	static {
		try {
			TRUE = new AttributeValue(Constants.TYPE_BOOLEAN, Boolean.TRUE);
			FALSE = new AttributeValue(Constants.TYPE_BOOLEAN, Boolean.FALSE);
		} catch (DataTypeMappingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	// AttributeValueΪ򵥱ʽԲҪֵ
	public Object evaluate(IRequest request)
			throws IndeterminateException {
		// TODO Auto-generated method stub
		return this;
	}

	protected AttributeValue(URI dataType, Object value)
			throws DataTypeMappingException {
		Class<?> typeExpected = DataTypeRegistry.getJavaType(dataType
				.toString());
		Class<?> typeActual = value.getClass();
		if (!typeExpected.isAssignableFrom(typeActual)) {
			throw new DataTypeMappingException(null, "Expected type: "
					+ typeExpected.getName() + ", but actual type is :"
					+ typeActual.getName(), null);
		}
		this.value = value;
		this.dataType = dataType;
		this.javaType = typeExpected;
		this.nameSpaceMap = new HashMap<String, String>();
	}

	protected AttributeValue(URI dataType, String value)
			throws DataTypeMappingException {
		Class<?> type = DataTypeRegistry.getJavaType(dataType.toString());
		this.value = DataTypeRegistry.getTypedValue(type, value);
		this.dataType = dataType;
		this.javaType = type;
		this.nameSpaceMap = new HashMap<String, String>();
	}

	protected AttributeValue(Metaset xmlElement)
			throws PolicySyntaxErrorException, DataTypeMappingException {
		super(xmlElement);
		Attribute[] attrs = UtilMetaset.extractAttributesByName(xmlElement,
				DATA_TYPE);
		
		UtilMetaset.checkEmpty(attrs, DATA_TYPE);
		UtilMetaset.checkNumber(attrs, 1, DATA_TYPE);
		URI dataType = URI.create(attrs[0].getValue());
		this.dataType = dataType;
		
		String dataValue =xmlElement.getText().trim();
		
		Class<?> type = DataTypeRegistry.getJavaType(dataType.toString());
		this.value = DataTypeRegistry.getTypedValue(type, dataValue);
		this.javaType = type;
		this.nameSpaceMap = new HashMap<String, String>();

		this.nameSpaceMap.put(xmlElement.getNamespacePrefix(), xmlElement
				.getNamespaceURI());
	}

	protected static boolean checkParams(URI dataType, Object value) {
		if (dataType != null && value != null) {
			return true;
		}
		return false;
	}

	public static AttributeValue newInstance(URI dataType, Object value)
			throws DataTypeMappingException {
		if (!checkParams(dataType, value)) {
			throw new DataTypeMappingException(null,
					"dataType or value is null!", null);
		}
		if (dataType.equals(Constants.TYPE_BOOLEAN)) {
			if (value instanceof Boolean) {
				if ((Boolean) value) {
					return TRUE;
				} else {
					return FALSE;
				}
			}
			if (value instanceof String) {
				String strValue = ((String) value).trim();
				if (strValue.equalsIgnoreCase("true")) {
					return TRUE;
				} else if (strValue.equalsIgnoreCase("false")) {
					return FALSE;
				}
			}
			throw new DataTypeMappingException(null, "Invalid boolean value! ",
					null);
		}
		return new AttributeValue(dataType, value);

	}

	public static AttributeValue newInstance(URI dataType, String value)
			throws DataTypeMappingException {
		if (!checkParams(dataType, value)) {
			throw new DataTypeMappingException(null,
					"dataType or value is null!", null);
		}
		if (dataType.equals(Constants.TYPE_BOOLEAN)) {
			value = value.trim();
			if (value.equalsIgnoreCase("true")) {
				return TRUE;
			} else if (value.equalsIgnoreCase("false")) {
				return FALSE;
			}
			throw new DataTypeMappingException(null, "Invalid boolean value! ",
					null);
		}
		return new AttributeValue(dataType, value);
	}

	public static AttributeValue newInstance(Metaset xmlElement)
			throws PolicySyntaxErrorException, DataTypeMappingException {
		Attribute[] attrs = UtilMetaset.extractAttributesByName(xmlElement,
				DATA_TYPE);
		UtilMetaset.checkNumber(attrs, 1, DATA_TYPE);
		URI dataType = URI.create(attrs[0].getValue());
		String dataValue = xmlElement.getText().trim();

		if (!checkParams(dataType, dataValue)) {
			throw new DataTypeMappingException(null,
					"dataType or value is null!", null);
		}
		if (dataType.equals(Constants.TYPE_BOOLEAN)) {
			dataValue = dataValue.trim();
			if (dataValue.equalsIgnoreCase("true")) {
				return TRUE;
			} else if (dataValue.equalsIgnoreCase("false")) {
				return FALSE;
			}
			throw new DataTypeMappingException(null, "Invalid boolean value! ",
					null);
		}
		return new AttributeValue(xmlElement);
	}

	public boolean equals(Object o) {
		if (this == o) {
			return true;
		}
		if (o.getClass() == this.getClass()) {
			String thisType = this.dataType.toString().trim();
			String otherType = ((AttributeValue) o).getDataType().toString()
					.trim();

			return thisType.equals(otherType)
					&& value.equals(((AttributeValue) o).getValue());
		}
		return false;
	}

	private void addNamespaceMapToRoot() {
		if (this.nameSpaceMap != null && this.nameSpaceMap.size() > 0) {
			if (root != null && root != this && root instanceof AXacmlPolicy) {
				Map<String, String> rootNSMap = ((AXacmlPolicy) root)
						.getNamespaceMap();
				Iterator<String> itr = this.nameSpaceMap.keySet().iterator();
				while (itr.hasNext()) {
					String key = itr.next();
					rootNSMap.put(key, this.nameSpaceMap.get(key));
				}
			}
		}
	}

	public int hashCode() {
		return value.hashCode();
	}

	public URI getDataType() {
		return dataType;
	}

	public void setDataType(URI dataType) {
		this.dataType = dataType;
	}

	public Class<?> getJavaType() {
		return javaType;
	}

	public void setJavaType(Class<?> javaType) {
		this.javaType = javaType;
	}

	public Map<String, String> getNameSpaceMap() {
		return nameSpaceMap;
	}

	public void setNameSpaceMap(Map<String, String> nameSpaceMap) {
		this.nameSpaceMap = nameSpaceMap;
	}

	public Object getValue() {
		this.addNamespaceMapToRoot();
		return value;
	}

	public void setValue(Object value) {
		this.value = value;
	}

}
