/**
 * Target.java
 */
package org.act.xservices.rt.xsecurity.xacml.policy.target;

import java.util.ArrayList;
import java.util.List;

import org.act.xservices.rt.xom.Metaset;
import org.act.xservices.rt.xsecurity.xacml.AXacmlElement;
import org.act.xservices.rt.xsecurity.xacml.IMatchable;
import org.act.xservices.rt.xsecurity.xacml.exception.DataTypeMappingException;
import org.act.xservices.rt.xsecurity.xacml.exception.PolicySyntaxErrorException;
import org.act.xservices.rt.xsecurity.xacml.policy.UtilMetaset;

/**
 * OASIS XACML淶2.0жpolicy:TargetԪء
 * 
 * 
 */
public class Target extends ConjunctiveMatch {

	public static final String SUBJECTS = "Subjects";

	public static final String RESOURCES = "Resources";

	public static final String ACTIONS = "Actions";

	public static final String ENVS = "Environments";

	private Subjects subjects = null;

	private Resources resources = null;

	private Actions actions = null;

	private Environments environments = null;

	public Target(Subjects subjects, Resources resources, Actions actions,
			Environments environments) {
		this.subjects = subjects;
		((AXacmlElement)this.subjects).setParent(this);
		this.resources = resources;
		((AXacmlElement)this.resources).setParent(this);
		this.actions = actions;
		((AXacmlElement)this.actions).setParent(this);
		this.environments = environments;
		((AXacmlElement)this.environments).setParent(this);
		
		List<IMatchable> list = new ArrayList<IMatchable>();
		if (subjects != null) {
			list.add(subjects);
		}
		if (resources != null) {
			list.add(resources);
		}
		if (actions != null) {
			list.add(actions);
		}
		if (environments != null) {
			list.add(environments);
		}
		this.matches = list.toArray(new IMatchable[0]);
	}

	public Target(Metaset xmlElement) throws PolicySyntaxErrorException,
			DataTypeMappingException {
		super(xmlElement);
		Metaset[] subs = UtilMetaset.extractChildrenByName(xmlElement, SUBJECTS);
		if (subs != null && subs.length != 0) {
			UtilMetaset.checkNumber(subs, 1, SUBJECTS);
			this.subjects = new Subjects(subs[0]);
			((AXacmlElement)this.subjects).setParent(this);
		}
		
		Metaset[] resz = UtilMetaset.extractChildrenByName(xmlElement, RESOURCES);
		if (resz != null && resz.length != 0) {
			UtilMetaset.checkNumber(resz, 1, RESOURCES);
			this.resources = new Resources(resz[0]);
			((AXacmlElement)this.resources).setParent(this);
		}
		
		Metaset[] acts = UtilMetaset.extractChildrenByName(xmlElement, ACTIONS);
		if (acts != null && acts.length != 0) {
			UtilMetaset.checkNumber(acts, 1, ACTIONS);
			this.actions = new Actions(acts[0]);
			((AXacmlElement)this.actions).setParent(this);
		}
		
		Metaset[] envs = UtilMetaset.extractChildrenByName(xmlElement, ENVS);
		if (envs != null && envs.length != 0) {
			UtilMetaset.checkNumber(envs, 1, ENVS);
			this.environments = new Environments(envs[0]);
			((AXacmlElement)this.environments).setParent(this);
		}
		
		List<IMatchable> list = new ArrayList<IMatchable>();
		if (this.subjects != null) {
			list.add(this.subjects);
		}
		if (this.resources != null) {
			list.add(this.resources);
		}
		if (this.actions != null) {
			list.add(this.actions);
		}
		if (this.environments != null) {
			list.add(this.environments);
		}
		this.matches = list.toArray(new IMatchable[0]);
	}

	public Actions getActions() {
		return actions;
	}

	public void setActions(Actions actions) {
		this.actions = actions;
		((AXacmlElement)this.actions).setParent(this);
	}

	public Environments getEnvironments() {
		return environments;
	}

	public void setEnvironments(Environments environments) {
		this.environments = environments;
		((AXacmlElement)this.environments).setParent(this);
	}

	public Resources getResources() {
		return resources;
	}

	public void setResources(Resources resources) {
		this.resources = resources;
		((AXacmlElement)this.resources).setParent(this);
	}

	public Subjects getSubjects() {
		return subjects;
	}

	public void setSubjects(Subjects subjects) {
		this.subjects = subjects;
		((AXacmlElement)this.subjects).setParent(this);
	}

}
