/**
 * ConformanceTest.java
 */
package org.act.xservices.rt.xsecurity.xacml.test.conformance;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.logging.Logger;

import org.act.xservices.rt.metaset.MetasetBuilder;
import org.act.xservices.rt.metaset.MetasetParser;
import org.act.xservices.rt.metaset.MetasetParserFactory;
import org.act.xservices.rt.metaset.exception.MetasetParsingException;
import org.act.xservices.rt.xom.Metaset;
import org.act.xservices.rt.xsecurity.xacml.context.IRequest;
import org.act.xservices.rt.xsecurity.xacml.context.Request;
import org.act.xservices.rt.xsecurity.xacml.context.Result;
import org.act.xservices.rt.xsecurity.xacml.exception.DataTypeMappingException;
import org.act.xservices.rt.xsecurity.xacml.exception.IndeterminateException;
import org.act.xservices.rt.xsecurity.xacml.exception.PolicySyntaxErrorException;
import org.act.xservices.rt.xsecurity.xacml.policy.Policy;
import org.act.xservices.rt.xsecurity.xacml.util.LoggerFactory;

/**
 * 
 */

// ע⣺ļ
public class ConformanceTest {

	private static final String TEST_CASE_DATA_PATH = "caseNew";



	private static MetasetParser builder = MetasetParserFactory.getMetasetParser();

	private static Logger logger = LoggerFactory.getLogger(ConformanceTest.class);

	public static void main(String[] args) throws MetasetParsingException,
			FileNotFoundException, PolicySyntaxErrorException,
			DataTypeMappingException, IndeterminateException {
		// TODO Auto-generated method stub

		List<String> policyList = new ArrayList<String>();
		List<String> requestList = new ArrayList<String>();
		List<String> responseList = new ArrayList<String>();

		// ӲзPolicyļRequestļResponseļ.һһӦģһPolicyӦһRequestļһResponseļ
		// ļǰ׺ͬΪ
		File dir = new File(TEST_CASE_DATA_PATH);
		if (dir != null && dir.isDirectory()) {
			File[] files = dir.listFiles();
			for (int i = 0; i < files.length; i++) {
				String fileName = files[i].getName();
				if (fileName.endsWith("Policy.xml")) {
					policyList.add(files[i].getAbsolutePath());
				} else if (fileName.endsWith("Request.xml")) {
					requestList.add(files[i].getAbsolutePath());
				} else if (fileName.endsWith("Response.xml")) {
					responseList.add(files[i].getAbsolutePath());
				}
			}
		} else {
			logger.info("Wrong test case data path.");
			return;
		}

		// ļԸеλýһһӦ
		String[] policyFiles = policyList.toArray(new String[0]);
		String[] requestFiles = requestList.toArray(new String[0]);
		String[] responseFiles = responseList.toArray(new String[0]);
		if (policyFiles.length == requestFiles.length
				&& requestFiles.length == responseFiles.length) {
			Arrays.sort(policyFiles);
			Arrays.sort(requestFiles);
			Arrays.sort(responseFiles);
		} else {
			logger
					.info("Test case data has been destroyed. Number error[Policy file :"
							+ policyFiles.length
							+ ", Request file :"
							+ requestFiles.length
							+ ", Response file :"
							+ responseFiles.length + "].");
			return;
		}

		// ִв
		String caseCode = null;
		int size = policyFiles.length;
		for (int i = 0; i < 28; i++) {

			try {

				File policyFile = new File(policyFiles[i]);
				File requestFile = new File(requestFiles[i]);
				File responseFile = new File(responseFiles[i]);
				caseCode = policyFile.getName().substring(0, 6);
				if (!caseCode.equals(requestFile.getName().substring(0, 6))
						|| !caseCode.equals(responseFile.getName().substring(0,
								6))) {
					logger
							.info("Test case data has been destroyed. Match error!");
					return;
				}

				System.out.println(caseCode);

				Metaset policyMetaset = builder.parse(new FileInputStream(
						policyFile));
				Metaset requestMetaset = builder.parse(new FileInputStream(
						requestFile));
				Metaset responseMetaset = builder.parse(new FileInputStream(
						responseFile));

				Policy policy = new Policy(policyMetaset);
				IRequest request = new Request(requestMetaset);
				Result result = policy.evaluate(request);
				String actualResult = null;
				switch (result.getDecision()) {
				case PERMIT:
					actualResult = "Permit";
					break;
				case DENY:
					actualResult = "Deny";
					break;
				case INDETERMINATE:
					actualResult = "Indeterminate";
					break;
				case NOTAPPLICABLE:
					actualResult = "NotApplicable";
					break;
				}

				String expectedResult = responseMetaset.filter(
						".\\Result\\Decision").elements().get(0).getText()
						.trim();

				System.out.println("\tActualResult: " + actualResult);
				System.out.println("\tExpectedResult: " + expectedResult);
			} catch (Exception e) {
				e.printStackTrace();
			}

		}

	}

}
