/**
 * MyClassFileLoader.java
 */
package org.act.xservices.rt.xsecurity.xacml.util;

import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.net.URLDecoder;
import java.util.Enumeration;
import java.util.HashSet;
import java.util.Set;

import org.act.xservices.rt.xsecurity.xacml.exception.XacmlFunctionLoadException;

/**
 * 
 */
public class ClassFileLoader {
	
	public static Set<Class<?>> loadClasses(String packageName) throws XacmlFunctionLoadException{
		Set<Class<?>> result=new HashSet<Class<?>>();
		ClassLoader loader=Thread.currentThread().getContextClassLoader();
		String path=packageName.replace('.', '/');
		try {
			Enumeration<URL> urls=loader.getResources(path);
			while(urls.hasMoreElements()){
				URL url=urls.nextElement();
				String protocol=url.getProtocol().toLowerCase();
				if(protocol.equals("file")){
					String dirPath=URLDecoder.decode(url.getFile(), "UTF-8");
					File dir=new File(dirPath);
					if(dir.isDirectory()){
						result.addAll(loadFromDirectory(dir,packageName));
					}
					else{
						throw new XacmlFunctionLoadException(null,"Invalid package name, failed to load classes!",null);
					}
				}
				else {
					throw new XacmlFunctionLoadException(null,"Only file protocol is supported currently!",null);
				}
			}
			
		} catch (IOException e) {
			// TODO Auto-generated catch block
			throw new XacmlFunctionLoadException(null,"Load xacml-function-class file error!",e);
		} catch (ClassNotFoundException e){
			throw new XacmlFunctionLoadException(null,"Load class error!",e);
		}
		return result;
	}
	
	private static Set<Class<?>> loadFromDirectory(File dir,String pkg) throws ClassNotFoundException{
		Set<Class<?>> result=new HashSet<Class<?>>();
		File[] files=dir.listFiles();
		for(int i=0;i<files.length;i++){
			if(files[i].isDirectory()){
				result.addAll(loadFromDirectory(files[i],pkg+"."+files[i].getName()));
			}
			else if(files[i].isFile()){
				String name=files[i].getName();
				if(name.endsWith(".class")){
					String stdName=standarizeClassName(pkg.trim(),name.trim());
					Class<?> clz=Class.forName(stdName);
					if(!clz.isAnnotation() && !clz.isInterface()){
						result.add(clz);
					}
				}
			}
		}
		return result;
	}
	
	//ļȥ".class"׺ϰ
	private static String standarizeClassName(String pkgName,String name){
		//".class"ĳΪ6
		String nameWithOutSuf=name.substring(0, name.length()-6);
		String stdName=pkgName+"."+nameWithOutSuf;
		return stdName;
	}
	public static void main(String[] args){
		String pkg="org.act.xacml.pdp";
		try {
			Set<Class<?>> list=loadClasses(pkg);
			System.out.println(list.size());
			for(Class<?> clz:list){
				System.out.println(clz.getName());
			}
			
		} catch (XacmlFunctionLoadException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
}
