/**
 * DataTypeRegistry.java
 */
package org.act.xservices.rt.xsecurity.xacml.util;


import java.lang.reflect.Constructor;
import java.lang.reflect.Method;
import java.math.BigDecimal;
import java.math.BigInteger;
import java.net.URI;
import java.util.Hashtable;
import java.util.Map;

import javax.naming.ldap.LdapName;

import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeConstants;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.Duration;
import javax.xml.datatype.XMLGregorianCalendar;
import javax.xml.namespace.QName;

import org.act.xservices.rt.xsecurity.xacml.exception.DataTypeMappingException;


/**
 * ṩXML SchematicJava֮ת
 * 
 */
public final class DataTypeRegistry {
   
    private static Map<Class<?>, Class<?>> primitiveMapper = new Hashtable<Class<?>, Class<?>>();
    private static Map<QName, Class<?>> javaByXml = new Hashtable<QName, Class<?>>();

    private static DatatypeFactory dataTypeFactory;

    public static final String XML_STRING = "string";
    public static final String W3C_XML_SCHEMA_NS_URI = "W3C_XML_SCHEMA_NS_URI";
    public static final String XML_BOOLEAN = "boolean";
    public static final String XML_INTEGER = "integer";
    public static final String XML_LONG = "long";
    public static final String XML_INT = "int";
    public static final String XML_DOUBLE = "double";
    public static final String XML_ANYURI = "anyURI";
    public static final String XPATH_DURATION_DAYTIME = "dayTimeDuration";
    public static final String XPATH_DURATION_YEARMONTH = "yearMonthDuration";

    public static final String XPATH_DRAFT_16AUG2002 = "http://www.w3.org/TR/2002/WD-xquery-operators-20020816";

    static {
    	//ԭʼӦİװ֮ת
        primitiveMapper.put(int.class, Integer.class);
        primitiveMapper.put(long.class, Long.class);
        primitiveMapper.put(boolean.class, Boolean.class);

        // XML SchemaԭʼJavaӳ
        javaByXml.put(new QName(W3C_XML_SCHEMA_NS_URI, XML_STRING), String.class);
        javaByXml.put(new QName(W3C_XML_SCHEMA_NS_URI, XML_BOOLEAN), Boolean.class);
        javaByXml.put(new QName(W3C_XML_SCHEMA_NS_URI, XML_INTEGER), BigInteger.class);
        javaByXml.put(new QName(W3C_XML_SCHEMA_NS_URI, XML_LONG), Long.class);
        javaByXml.put(new QName(W3C_XML_SCHEMA_NS_URI, XML_INT), Integer.class);
        javaByXml.put(new QName(W3C_XML_SCHEMA_NS_URI, XML_DOUBLE), BigDecimal.class);
        javaByXml.put(new QName(W3C_XML_SCHEMA_NS_URI, XML_ANYURI), URI.class);
        // ʱJavaӳ
        javaByXml.put(DatatypeConstants.TIME, XMLGregorianCalendar.class);
        javaByXml.put(DatatypeConstants.DATE, XMLGregorianCalendar.class);
        javaByXml.put(DatatypeConstants.DATETIME, XMLGregorianCalendar.class);

        //XACML淶2.0 Java͵ӳ
        javaByXml.put(new QName(XPATH_DRAFT_16AUG2002, XPATH_DURATION_DAYTIME), Duration.class);
        javaByXml.put(new QName(XPATH_DRAFT_16AUG2002, XPATH_DURATION_YEARMONTH), Duration.class);
        
    // javaByXml.put(new QName(Constants.TYPE_X500NAME.toString()), LdapName.class);
    }

    private DataTypeRegistry() {}

    public static Class<?> getJavaType(QName xmlType) throws DataTypeMappingException {
    	Class<?> result = javaByXml.get(xmlType);
    	if (result == null) {
    		throw new DataTypeMappingException(null,"Can't get the registered class for XML type: " + xmlType,null);
    	}
    	return result;
    }

    public static Class<?> getJavaType(String xmlType) throws DataTypeMappingException {
    	Class<?> result = javaByXml.get(XMLTypeAsQName(xmlType));
    	if (result == null) {
    		throw new DataTypeMappingException(null,"Can't get the registered Java class for XML type: " + xmlType,null);
    	}
    	return result;
    }

    public static void register(QName typeQName, Class<?> elemClass) {
        javaByXml.put(typeQName, elemClass);
    }

    public static void unregister(QName typeQName) {
        javaByXml.remove(typeQName);
    }

    /**
     * ַתΪQName
     * @param ַʾQName
     * @return QName
     */
    public static QName XMLTypeAsQName(String xmlType) {
        if (xmlType != null) {
            int pos = xmlType.indexOf("#");
            if (pos >= 0) {
            	String ns=xmlType.substring(0,pos);
            	String local=xmlType.substring(pos+1);
                return new QName(ns,local);
            }
            else {
                return new QName(null, xmlType);
            }
        }
        return null;
    }

    /**
     * QNameתΪַ
     * @param QName
     * @return ַʾQName
     */
    public static String XMLTypeAsString(QName qName) {
        if (qName != null) {
            String ns = qName.getNamespaceURI();
            String local = qName.getLocalPart();
            return ns == null ? local : (ns + "#" + local);
        }
        return null;
    }

    public static Class<?> getWrapperClassOfPrimitive(Class<?> primitive) throws DataTypeMappingException {
        if (primitive.isPrimitive()) {
            return primitiveMapper.get(primitive);
        }
        else {
            throw new DataTypeMappingException(null,"The given class is not a primitive type : " + primitive.getName(),null);
        }
    }

    public static Object getTypedValue(Class<?> type, String value) throws DataTypeMappingException {
        Class<?> classType = type;
        if (type.isArray()) {
            throw new DataTypeMappingException(null,"Currently we don't support get Array type value.",null);
        }
        else {
            if (type.isPrimitive()) {
                classType = getWrapperClassOfPrimitive(type);
            }

            try {
                String methodName = "valueOf";
                Method valueOf = classType.getMethod(methodName, new Class[] {String.class});
                return valueOf.invoke(null, new Object[] {value});
            }
            catch (Exception e) {}

            try {
                Constructor<?> cons = classType.getConstructor(new Class[] {String.class});
                return cons.newInstance(new Object[] {value});
            }
            catch (Exception e) {}
            
            try {
                String methodName = "new" + type.getSimpleName();
                Method newCalendarOrDuration = DatatypeFactory.class.getMethod(methodName, String.class);
                return newCalendarOrDuration.invoke(getDatatypeFactory(), value);
            }
            catch (Exception e) {}
        }

        throw new DataTypeMappingException(null,"The string value \"" + value + "\" could not be converted to class \"" + 
                type.getName() + "\"",null);
    }

    public static synchronized DatatypeFactory getDatatypeFactory() throws DatatypeConfigurationException {
        if (dataTypeFactory == null) {
            dataTypeFactory = DatatypeFactory.newInstance();
        }
        return dataTypeFactory;
    }
}
